/*
 * Copyright 1998-2007 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2007 S3 Graphics, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#ifndef __LINUX_VIACAP_H
#define __LINUX_VIACAP_H

#include "open/via_types.h"  /* for Bool */

#include "close/VidStruc.h"   /* VIDDATA, video top level structure */

#ifdef __KERNEL__
#include "via_v4l.h"    /* for via_v4l_devices */
#else

typedef struct _VIAAUDCTRL{
  unsigned long dwAudioMode ;  /* Audio Mode         */
  int   nVolume     ;  /* Volume             */
} VIAAUDCTRL ;
typedef VIAAUDCTRL * LPVIAAUDCTRL;

typedef struct _VIASETTUNERDATA    {
    int   divider;
    int   control;
} VIASETTUNERDATA ;
typedef VIASETTUNERDATA  * LPVIASETTUNERDATA;

#endif

/*
 * structure of Set Port Attribute
 */
typedef struct _VIASETPORTATTR    {
    int                   attribute;
    int                   value;
} VIASETPORTATTR ;
typedef VIASETPORTATTR  * LPVIASETPORTATTR;

/* for value definition */
/* Quality */
#define AUDIO_STEREO       0x00000001        /* ATOM("XV_AUDIOCTRL") */
#define AUDIO_SAP          0x00000002        /* ATOM("XV_AUDIOCTRL") */
#define AUDIO_DUAL         0x00000004        /* ATOM("XV_AUDIOCTRL") */

/* Mute */
#define AUDIO_ON           0x00000010        /* ATOM("XV_AUDIOCTRL") */
#define AUDIO_OFF          0x00000020        /* ATOM("XV_AUDIOCTRL") */

/* Source */  
#define AUDIO_TV0          0x00000100        /* ATOM("XV_AUDIOCTRL") */
#define AUDIO_TV1          0x00000200        /* ATOM("XV_AUDIOCTRL") */
#define AUDIO_SVIDEO       0x00000400        /* not available now    */
#define AUDIO_COMPOSE      0x00000800        /* 		               */	




/*Definition for  LPVIAGETPORTATTR->attribute*/
#define ATTR_ENCODING         0                  /* XV_ENCODING  */
#define ATTR_INIT_AUDIO       ATTR_ENCODING+50   /* XV_MUTE      */
#define ATTR_MUTE_ON          ATTR_ENCODING+51   /* XV_MUTE      */
#define ATTR_MUTE_OFF         ATTR_ENCODING+52   /* XV_MUTE      */
#define ATTR_VOLUME           ATTR_ENCODING+53   /* XV_VOLUME    */
#define ATTR_STEREO           ATTR_ENCODING+54   /* XV_VOLUME    */
#define ATTR_SAP              ATTR_ENCODING+55   /* XV_SAP       */
#define ATTR_TUNER_AUDIO_SWITCH   ATTR_ENCODING+56   /* XV_TUNER_AUDIO*/
#define ATTR_AUDIO_CONTROLByAP    ATTR_ENCODING+57   /* XV_AUDIOCTRL  */

#define DEV_TV0   0
#define DEV_TV1   1



int VIA_CAPSetFrameBuffer(PVIDDATA pVidData, LPCAPDEVICE lpDevice);
int VIA_ALPSetFrameBuffer(PVIDDATA pVidData, LPALPHADEVICE lpDevice);
unsigned long UpdateOverlayAlpha(PVIDDATA pVidData);
#ifdef __KERNEL__
void VIA_API_CALL UpdateVPort(via_device *via, unsigned long dwCapFlag);
void VIAStopVideo(via_device *via);
#endif

#endif  

