/*
 * Copyright 1998-2007 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2007 S3 Graphics, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#ifndef _LINUX_VIDEO_H
#define _LINUX_VIDEO_H

#ifdef XFree86Server
#include "xf86_ansic.h" /* for Bool */
#include "xf86.h"       /* for ScrnInfoPtr */
#include "xf86fbman.h"	/* Use XServer memory management */
#else
#include "open/via_types.h"  /* for Bool */
#endif

#ifdef __KERNEL__
#include "via_v4l.h"	/* for drm.h */
#endif

/* Use DRM memory management */

#ifdef HW_SLICE
/*typedef unsigned int size_t;*/
#include <stddef.h>
#endif

#include "drm.h"
#include "close/via_drm.h"
#include "close/via_mm.h"

#include "via_memmgr.h"	/* V4L memory management */
#include "open/viavid.h"
#include "VidStruc.h"   /* VIDDATA, video top level structure */

/*
 * Alignment macro functions
 */
#define ALIGN_TO_8(f)             (((f) + 7) & ~7)
#define ALIGN_TO_16(f)            (((f) + 15) & ~15)
#define ALIGN_TO_32(f)            (((f) + 31) & ~31)
#define ALIGN_TO_64(f)            (((f) + 63) & ~63)
#define ALIGN_TO_128(f)           (((f) + 127) & ~127)
#define ALIGN_TO_256(f)           (((f) + 255) & ~255)

#define ALIGN_TO_8_BYTE(f)        (((f) + 7) & ~7)
#define ALIGN_TO_16_BYTE(f)       (((f) + 15) & ~15)
#define ALIGN_TO_32_BYTE(f)       (((f) + 31) & ~31)
#define ALIGN_TO_64_BYTE(f)       (((f) + 63) & ~63)
#define ALIGN_TO_256_BYTE(f)      (((f) + 255) & ~255)

#define ALIGN_TO_64_BIT(f)        (((f) + 63) & ~63)
#define ALIGN_TO_256_BIT(f)       (((f) + 255) & ~255)

#define ENG_ALIGN_BYTE              ALIGN_TO_32_BYTE
#define ENG_ALIGN_64_BYTE           ALIGN_TO_64_BYTE
#define ENG_ALIGN_256_BYTE          ALIGN_TO_256_BYTE

#define ENG_ALIGN_BIT               ALIGN_TO_256_BIT

/*
 * Define for Utility functions using XvPutImage.
 */
/* Primary ID */
#define UT_XV_FUNC_BIOS                 0x11   /* Primary ID */
#define UT_XV_FUNC_DRIVER               0x12
#define UT_XV_FUNC_DEVICE               0x13
#define UT_XV_FUNC_PANEL                0x14
#define UT_XV_FUNC_TV                   0x15
#define UT_XV_FUNC_GAMMA                0x16
/* Secondary ID */
#define UT_XV_FUNC_BIOS_GetChipID                       0x01    /* Get Chip ID */
#define UT_XV_FUNC_BIOS_GetVersion                      0x02    /* Get the version of the BIOS */
#define UT_XV_FUNC_BIOS_GetDate                         0x03    /* Get the date (year, month, day) of the BIOS. */
#define UT_XV_FUNC_BIOS_GetVideoMemSizeMB       0x04    /* Get the video memory size, in MB */

#define UT_XV_FUNC_DRIVER_GetFileName                   0x01
#define UT_XV_FUNC_DRIVER_GetFileVersion                0x02
/* Return value */
#define UT_RESULT_OK                    1    /* Return value */
#define UT_RESULT_FAIL                  0
#define UT_RESULT_NO_FUNC               -1

enum SurfaceID{
    MPEG_REQ_SURFACE_ID = 1,
    SUBP_REQ_SURFACE_ID = 2,
    HQV_REQ_SURFACE_ID = 3,
    HQV2_REQ_SURFACE_ID = 4,
    TV0_REQ_SURFACE_ID = 5,
    TV1_REQ_SURFACE_ID = 6,
    ALPHA_REQ_SURFACE_ID = 7,
    SW_REQ_SURFACE_ID = 8,
    VQ_REQ_SURFACE_ID = 9,
    MPEG_DECODE_COUNT_REQ_SURFACE_ID = 10,
};


typedef struct _VIDEOMEMORY
{
    /* For V4L memory management */
    ViaMMReq MPEGMemRequest[2];
    ViaMMReq SUBPMemRequest[2];
    ViaMMReq HQVMemRequest;
    ViaMMReq HQV2MemRequest;
    ViaMMReq TV0MemRequest;
    ViaMMReq TV1MemRequest;
    ViaMMReq ALPHAMemRequest;
    ViaMMReq SWMemRequest[2];/*add by zepeng 20060110*/
    ViaMMReq VQMemRequest;

    ViaMMReq MPEGDecodeCountMemRequest;

    /* For DRM memory management */
    drm_via_mem_t MPEGfbRequest[2];
    drm_via_mem_t SUBPfbRequest[2];
    drm_via_mem_t HQVfbRequest;
    drm_via_mem_t HQV2fbRequest;
    drm_via_mem_t TV0fbRequest;
    drm_via_mem_t TV1fbRequest;
    drm_via_mem_t ALPHAfbRequest;
    drm_via_mem_t SWfbRequest;
    drm_via_mem_t VQfbRequest;

    drm_via_mem_t MPEGDecodeCountRequest;
    
    #ifdef XFree86Server
    FBLinearPtr   SWFBlinear;
    FBLinearPtr   HQVFBlinear;
    #endif
}VIDEOMEMORY, *LPVIDEOMEMORY;


typedef struct _ADJUSTFRAME
{
    int     x;
    int     y;
    unsigned long     dwFlags; /*Added for SAMM*/
} ADJUSTFRAME;
typedef ADJUSTFRAME * LPADJUSTFRAME;

/*
 * Indicates that the surface memory is composed of interleaved fields.
 */
#define DDOVER_INTERLEAVED                      0x00800000l

/*
  *  [2004,12,09] Joe Huang
  *  Record MPEG decode and HQV flip frame buffer status
  *  Add to solve the up-right and down-left tearing issue
  */
#define HQV_DST_NONE     0x00000000
#define HQV_DST_INDEX0  0x00000001
#define HQV_DST_INDEX1  0x00000002
#define HQV_DST_INDEX2  0x00000004
typedef struct _MPEGFBSTATUS
{
        unsigned long dwMPEGDecodeFrameCount; 

        /* Record MPEG decode really end. event tag in VQ to record decode end*/
        /* It is assigned in create surface */
        unsigned long dwDecodeEndCountPhysicalAddr; 

        unsigned long dwDecodeFrameNumberRecord[NUM_FRAME_BUFFERS];
        unsigned long dwFlipToHQVDstIndexRecord[NUM_FRAME_BUFFERS];        
        
} MPEGFBSTATUS, *LPMPEGFBSTATUS;


unsigned long LockSurface(PVIDDATA pVidData, LPDDLOCK lpLock);
#ifndef HW_SLICE
#ifdef WITHOUT_SOURCE
unsigned long UpdateOverlay(PVIDDATA pVidData, LPDDUPDATEOVERLAY lpUpdate);
#endif
#endif
unsigned long Flip(PVIDDATA pVidData, unsigned long DisplayBufferIndex);
unsigned long HQVControl(PVIDDATA pVidData, LPVIAVIDCTRL lpNewVidCtl );
void VIAAlphaWin(PVIDDATA pVidData, LPALPHACTRL lpParam);

void InitEngineManager(void);/*ENGINEMANAGER*/
unsigned long VIA_Engine_Allocate(PVIDDATA pVidData, unsigned long uPortId, unsigned long uEng_type, unsigned long uEng_id);/*ENGINEMANAGER*/
void VIA_Engine_Free_All(PVIDDATA pVidData, unsigned long uPortId);/*ENGINEMANAGER*/
unsigned long VIA_Engine_Swap(PVIDDATA pVidData, unsigned long uPortId, unsigned long uEng_type, unsigned long uEng_id_src, unsigned long uEng_id_dst);
unsigned long VIA_AdjustClientPriority(unsigned long uPortId, unsigned long uPriority);
void ScreenSwitchManager(PVIDDATA pVidData, LPDDUPDATEOVERLAY lpUpdate );
unsigned long UpdateOverlay_CheckVideoStatus(PVIDDATA pVidData, LPDDUPDATEOVERLAY lpUpdate);
#endif
