/*
 * Copyright 1998-2007 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2007 S3 Graphics, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#ifndef _VIAMPG
#define _VIAMPG

#include "via_types.h"  /* for Bool */

typedef struct _VIAMCSURFACE
{
    /*
     *  First part isn't used currently, will be removed later.
     */
    unsigned long dwSize;                   /* Size of this structure */
    unsigned long dwVersion;                /* Interface Version */
    unsigned long pDevice;                  /* Useless */
    void * BeginPicture;                        /* Pointer to VIA_BeginPicture */
    void * EndPicture;                          /* Pointer to VIA_Endicture */
    void * SetMacroBlock;                       /* Pointer to VIA_SetMacroBlock */
    void * SliceReceiveData;                    /* Pointer to VIA_SliceReceiveData */
    void * DriverProc;
    /* unsigned long dwCompose; */
    void * DisplayControl;
    void * SubPicture;
    void * SetDeInterlaceMode;                  /* Pointer to VIA_SetDeinterlaceMode */
    unsigned long dwState;                  /* Current state of MPG  */
    unsigned long dwBufferNumber;           /* Total no. of decoder buffer created. (Always 4 ) */
    Bool  bInitialized;                         /* Except dwState */
    unsigned long dwReserved[8];            /* For future expansion */

    /*
     * Client fills out the following values before calling BeginPicture
     */
    unsigned long dwTaskType;                /* Null, Decode or Display */

    /* Decode Picture */
    unsigned long dwPictureType;             /* I, P or B pictures */
    unsigned long dwDecodePictStruct;        /* Top Field, Bottom Field or Frame */
    unsigned long dwDecodeBuffIndex;         /* Decode buffer Index. 0 to ( dwBufferNumber -1) */
    unsigned long dwAlternateScan;           /* Zig-Zag Scan or Alternative Scan */

    /* Display picture */
    unsigned long dwDisplayPictStruct;       /* Top Field, Bottom Field or Frame */
    unsigned long dwDisplayBuffIndex;        /* Display buffer Index. 0 to ( dwBufferNumber -1) */
    unsigned long dwFrameRate;               /* Frame rate  multiplied by 1000 ( 29.97=29700 ) */
    unsigned long dwDeinterlaceMode;         /* BOB / Weave */
    unsigned long dwQMatrix[2][16];          /* Quantiser Matrix, 0:intra, 1:non_intra */
    unsigned long dwQMatrixChanged;          /* If The Quantiser Matrix is changed?        */
    unsigned long dwMBwidth;                 /* Macroblock Width                                           */
    /* unsigned long dwV_size_g2800; */      /* Decide if v_size is greater than 2800      */
    unsigned long dwMpeg2;                   /* Decide if it is a mpeg2 slice stream       */
    unsigned long dwTopFirst;
    unsigned long dwFramePredDct;
    unsigned long dwMBAmax;                  /* It is equal to mb_width * mb_height; */
    unsigned long dwIntravlc;
    unsigned long dwDcPrec;
    unsigned long dwQscaleType;
    unsigned long dwConcealMV;
    /* unsigned long dwSliceFirst; */
    unsigned long dwSecondField;
    unsigned long dwOldRefFrame;             /* Fordward reference */
    unsigned long dwRefFrame;                /* Backward reference */
    unsigned long BVMVRange;                 /* Backward Vertical Motion Vector Range   */
    unsigned long BHMVRange;                 /* Backward Horizontal Motion Vector Range */
    unsigned long FVMVRange;                 /* Fordward Vertical Motion Vector Range   */
    unsigned long FHMVRange;                 /* Fordward Horizontal Motion Vector Range */
    unsigned long dwMPEGProgressiveMode;     /* default value : VIA_PROGRESSIVE */
    int   framenum;
    unsigned long dwDecoderMode;             /* indicate H/W mpeg decoder mode: USE_MC, USE_IDCT, USE_SLICE */
    unsigned long dwMPEG4Decode;             /* Indicate to use the MPEG4 decoder */
    unsigned long dwLumaFilterMode;	         /* 0:bi-linear, 1:bi-cubic */
    unsigned long dwOverlapTSF;              /* Indicate to enable overlay transform filter */	
    unsigned long dwIntensityComp;           /* Indicate to enable intensity compensation */	
    unsigned long dwLumScale;                /* for intensity compensation */
    unsigned long dwLumShift;	             /* for intensity compensation */
    
    /* Picture parameter */
    /*unsigned long dwRefPicAddress[16]; */
    unsigned int wScalingLists4x4[6][16];
    unsigned int wScalingLists8x8[2][64];
    unsigned char bWeighted_pred_flag;
    unsigned char bWeighted_bipred_idc;
    unsigned char bDefaultScalingList;
    unsigned char bChromaFormat;
    /* Slice Information */
    /* unsigned char ReservedBits0[4]; */
    /* unsigned int ReservedBits1[4][64]; */
    unsigned int wRefIdxNum[2];              /* [0]:Fordward, [1]:Backward */
    unsigned long dwCurFieldOrderCnt[2];
    unsigned long dwRefFieldOrderCnt[16][2];
    unsigned char bSliceType;                /* 0:P, 1:B, 2:I */
    unsigned char bFieldPicFlag;
    unsigned char bBottomFieldFlag;
    unsigned char bDisableDeblock;
    unsigned char bRefList[2][32];
    unsigned char bRefFieldTerm[16][2];
    /* Weighted Prediction Information */
    unsigned char bWeightedLogDenom [2];    /* [] Range: 0~7 */
    /* unsigned int wReservedBits; */
    unsigned int wWeightedFactor[2][3][32]; /* Range: -128 ~ 127 */
    unsigned int wWeightedOffset[2][3][32]; /* Range: -128 ~ 127 */
    
    unsigned long dwRreserved[6];           /* for future expansion */
} VIAMPGSURFACE, * LPVIAMPGSURFACE;

/* definition for dwDecoderMode */
#define USE_SLICE                 0
#define USE_MC                    1
#define USE_MC_IDCT               2

/* dwTaskType of VIAMPGSURFACE */
#define VIA_TASK_NULL              0x00000000 /* Used only to check state and version */
#define VIA_TASK_DECODE            0x00000001 /* Decode new Picture also Display decoded picture */
#define VIA_TASK_DISPLAY           0x00000002 /* Display only */

/* dwPictureType of VIAMPGSURFACE */
#define VIA_PIC_TYPE_I              0x00000001
#define VIA_PIC_TYPE_P              0x00000002
#define VIA_PIC_TYPE_B              0x00000003

/* dwDecodePictStruct and dwDisplayPictStruct of VIAMPGSURFACE */
#define VIA_PICT_STRUCT_TOP                 0x00000001
#define VIA_PICT_STRUCT_BOTTOM              0x00000002
#define VIA_PICT_STRUCT_FRAME               0x00000003

/* dwAltScan */
#define VIA_ALTERNATE_SCAN                  0x1
#define VIA_ZIG_ZAG_SCAN                    0x0

/* dwDeinterlaceMode of VIAMPGSURFACE */
#define VIA_DEINTERLACE_WEAVE               0x00000000
#define VIA_DEINTERLACE_BOB                 0x00000001
#define VIA_DEINTERLACE_MEDIAN              0x00000002


/* dwMPEGProgressiveMode of VIAMPGSURFACE */
#define VIA_NON_PROGRESSIVE                 0x00000000
#define VIA_PROGRESSIVE                     0x00000010

/**************************************************************************
   VIA Macro Block Structure
***************************************************************************/

/*  
 *      store the IDCT coefficient values.
 */
typedef struct _IDCTCoef4Group {
    unsigned char  CoefIndex[4];
    short CoefValue[4];
} IDCTCoef4Group, *LPIDCTCoef4Group;


typedef struct  _VIAMACROBLOCK {
    unsigned long dwMBLKInfo;       /* Codedblocks, DCT type, Predition direction,    */
                                    /* Prediction mode and Motion Vectors information */
                                    /* of current Macro Block                         */
    unsigned long dwMBLKHOffset;    /* Macro Block horizontal offset                  */
    unsigned long dwMBLKVOffset;    /* Macro Block vertical offset                    */
    unsigned long dwMV[24];         /* Motion Vectors in half pixel resolution 	*/
                                    /* and based on the current Bacro Block's 	*/
                                    /* upper left corner.                                   */
                                    /* 1st forward motion vector or Motion Vector 0         */
                                    /* dwMV[0] is PMV[0][0][0] (Horizontal)                 */
                                    /* dwMV[1] is PMV[0][0][1] (Vertical  )                 */
                                    /* 2nd forward motion vector or Motion Vector 1         */
                                    /* dwMV[2] is PMV[0][1][0] (Horizontal)                 */
                                    /* dwMV[3] is PMV[0][1][1] (Vertical  )                 */
                                    /* 1st backward motion vector or Motion Vector 2        */
                                    /* dwMV[4] is PMV[1][0][0] (Horizontal)                 */
                                    /* dwMV[5] is PMV[1][0][1] (Vertical  )                 */
                                    /* 2nd backward motion vector or Motion Vector 3        */
                                    /* dwMV[6] is PMV[1][1][0] (Horizontal)                 */
                                    /* dwMV[7] is PMV[1][1][1] (Vertical  )                 */
                                    /* First forward UV motion vector or Motion Vector 4    */
                                    /* dwMV[8] is PMV[0][0][0] (Horizontal)                 */
                                    /* dwMV[9] is PMV[0][0][1] (Vertical  )                 */
                                    /* 3rd forward motion vector or Motion Vector 5         */
                                    /* dwMV[10] is PMV[0][0][0] (Horizontal)                */
                                    /* dwMV[11] is PMV[0][0][1] (Vertical  )                */
                                    /* 4th forward motion vector or Motion Vector 6         */
                                    /* dwMV[12] is PMV[0][1][0] (Horizontal)                */
                                    /* dwMV[13] is PMV[0][1][1] (Vertical  )                */
                                    /* 3rd backward motion vector or Motion Vector 7        */
                                    /* dwMV[14] is PMV[1][0][0] (Horizontal)                */
                                    /* dwMV[15] is PMV[1][0][1] (Vertical  )                */
                                    /* 4th backward motion vector or Motion Vector 8        */
                                    /* dwMV[16] is PMV[1][1][0] (Horizontal)                */
                                    /* dwMV[17] is PMV[1][1][1] (Vertical  )                */
                                    /* First backward UV motion vector or Motion Vector 9   */
                                    /* dwMV[18] is PMV[0][0][0] (Horizontal)                */
                                    /* dwMV[19] is PMV[0][0][1] (Vertical  )                */
                                    /* Second forward UV motion vector or Motion Vector 4   */
                                    /* dwMV[20] is PMV[0][0][0] (Horizontal)                */
                                    /* dwMV[21] is PMV[0][0][1] (Vertical  )                */
                                    /* Second backward UV motion vector or Motion Vector 9  */
                                    /* dwMV[22] is PMV[0][0][0] (Horizontal)                */
                                    /* dwMV[23] is PMV[0][0][1] (Vertical  )                */
    /* 09-01-1999 */
    /* Based on Scott's suggestion, I restore the original design as below. */
    void * lpIDCTBuff;           /* IDCT data for current macro block                      */
                                 /* Client needs to allocate and maintain 768 bytes        */
                                 /* For INTRA Macro Block                                  */
                                 /* ( 64 coeffiients per block * 1 bytes per coefficient * */
                                 /*   6 clocks ) buffer with the following layout.         */
                                 /*  Buf[  0..63 ] for Y0                                  */
                                 /*  Buf[ 64..127] for Y1                                  */
                                 /*  Buf[128..191] for Y2                                  */
                                 /*  Buf[192..255] for Y3                                  */
                                 /*  Buf[256..319] for Cb                                  */
                                 /*  Buf[320..383] for Cr                                  */
                                 /* For INTER Macro Block                                  */
                                 /* ( 64 coeffiients per block * 2 bytes per coefficient * */
                                 /*   6 clocks ) buffer with the following layout.         */
                                 /*  Buf[  0..127] for Y0                                  */
                                 /*  Buf[128..255] for Y1                                  */
                                 /*  Buf[256..383] for Y2                                  */
                                 /*  Buf[384..511] for Y3                                  */
                                 /*  Buf[512..639] for Cb                                  */
                                 /*  Buf[640..767] for Cr                                  */
    LPIDCTCoef4Group lpHWIDCTBuff;        /* We fill A.A.N. IDCT coefficient in this buffer which has 256                           */
                                 /* bytes per block,                                                                       */
                                 /* 128 bytes [0,2,4,.....126] are for Run of Y0  , where 0 <= Run <= 63                   */
                                 /* 128 bytes [1,3,5,.....127] are for Level of Y0, where -2048   <= Level  <= 2047        */
                                 /* 128 bytes [128,130,132,.....254] are for Run of Y1  , where 0 <= Run <= 63             */
                                 /* 128 bytes [129,131,133,.....255] are for Level of Y1, where -2048   <= Level  <= 2047  */
                                 /* 128 bytes [256,258,260,.....382] are for Run of Y2  , where 0 <= Run <= 63             */
                                 /* 128 bytes [257,259,261,.....383] are for Level of Y2, where -2048   <= Level  <= 2047  */
                                 /* 128 bytes [384,386,388,.....510] are for Run of Y3  , where 0 <= Run <= 63             */
                                 /* 128 bytes [385,387,389,.....511] are for Level of Y3, where -2048   <= Level  <= 2047  */
                                 /* 128 bytes [512,514,516,.....638] are for Run of Cb  , where 0 <= Run <= 63             */
                                 /* 128 bytes [513,515,517,.....639] are for Level of Cb, where -2048   <= Level  <= 2047  */
                                 /* 128 bytes [640,642,644,.....766] are for Run of Cr  , where 0 <= Run <= 63             */
                                 /* 128 bytes [641,643,645,.....767] are for Level of Cr, where -2048   <= Level  <= 2047  */
    unsigned char  byIDCTCount[6];        /* Number of (Run, Level) pair in each block                                              */
    /* Vincent Lai [2003.07.14],for VT3259 */
    unsigned char  bReserved[2];          /* Alignment */
    unsigned long dwVMBNum;               /* The macroblock number per column */
    unsigned long dwHMBNum;               /* The macroblock number per row */
    unsigned long dwBiDirectionRounding;  /* Bi-direction prediction rounding control */
    unsigned long dwRounding;             /* Interpolation rounding control for non-integer motion vector */
    unsigned long dwQblockSize;           /* Block size of quarter-pel MC in case of progressive MB 0:16x16 ,1:8x8 */
    unsigned long dwUseQMC;               /* Quarter-pel enable */
    unsigned long dwUseGMC;               /* GMC enable */
    unsigned long dwUse4MV;               /* 4MV enable */
    unsigned long dwScanType;             /* 0:Zig-Zag 1:Alternate Vertical 2:Alternate Horizontal */
                                                  /* [1:0] block u0,[3:2] block v0,[5:4] block y3,[7:6] block y2,[9:8] block y1,[11:10] block y0 */
    unsigned long dwDCScalValue[6];       /* DC scal value */
    unsigned long dwDCTMaxValue[6];       /* DCT coefficients clip value */
    unsigned long dwQP[6];                /* Inverse Quantization value */
    unsigned long dwIntra[6];             /* Intra block ,1:Intra block,0:Inter block */
			        
    unsigned long dwFastUVMC;       /** Fast U,V motion compensation flag */
    void *lpWmvHWIDCTBuff;     /*zhaolin[2006.11.22]: add two vars used to change the way to store wmv IDCT coeff*/
    unsigned long dwWmvIDCTCoeffCnt;
  	
} VIAMACROBLOCK, * LPVIAMACROBLOCK;


#define MBLK_LAST_MACROBLOCK   0x00800000   /* The last macroblock in this picture */

/* dwBLKInfo of VIAMACROBLOCK */
/* Prediction Type */
#define MBLK_PREDICY_BWD       0x00000100   /* Backward Prediction */
#define MBLK_PREDICY_FWD       0x00000080   /* Forward Prediction  */
#define MBLK_PREDICY_BD        (MBLK_PREDICY_BWD | MBLK_PREDICY_FWD) /* Bidirection Prediction */
#define MBLK_PREDICY_16X8      0x00000400   /* 16X8 Prediction       */
#define MBLK_PREDICY_DP        0x00000600   /* Dual Prime Prediction */
#define MBLK_PREDICY_FRAME     0x00000400   /* Frame Prediction      */
#define MBLK_PREDICY_FIELD     0x00000200   /* Field Prediction      */

/* DCT Type */
#define MBLK_DCT_FRAME         0x00000000   /* use Frame DCT for current macroblock */
#define MBLK_DCT_FIELD         0x00000040   /* use Field DCT for current macroblock */

/* Intra/Inter Flag */
#define MBLK_TYPE_INTRA        0x00000000   /* Intra BacroBlock */
#define MBLK_TYPE_INTER        0x80000000   /* Inter BacroBlock */

/* This 2 bits are not in "Macro Block Descrition register".
 * They are in "Motion Vector 0~4 register"
 * Please do "LPVIAMACROBLOCK->dwMV[?] = LPVIAMACROBLOCK->dwMV[?] | MBLK_MV?_REF_XXX | MBLK_REF_BUFF_INDEX_0"
 * before calling MPEGSetMacroBlock
 * Reference Field
 */
#define MBLK_MV_REF_BOTTOM     0x00008000   /* Reference Bottom field */
#define MBLK_REF_BUFF_INDEX_0  0x00000000
#define MBLK_REF_BUFF_INDEX_1  0x00002000
#define MBLK_REF_BUFF_INDEX_2  0x00004000
#define MBLK_REF_BUFF_INDEX_3  0x00006000

/* CBP */
#define MBLK_CBP_Y0            0x00000001   /* Indicate if Y0 block is coded */
#define MBLK_CBP_Y1            0x00000002   /* Indicate if Y1 block is coded */
#define MBLK_CBP_Y2            0x00000004   /* Indicate if Y2 block is coded */
#define MBLK_CBP_Y3            0x00000008   /* Indicate if Y3 block is coded */
#define MBLK_CBP_Cb            0x00000010   /* Indicate if Cb block is coded */
#define MBLK_CBP_Cr            0x00000020   /* Indicate if Cr block is coded */




/* 09-01-1999
 * Based on discuss between me and Jay.
 * The following structure and definition are for subpicture.
 */
/**************************************************************************
   VIA SubPicture Structure
***************************************************************************/
typedef struct  _VIASUBPICT {
    /*
     *  First part isn't used currently, will be removed later.
     */
    unsigned long dwSize;                   /* Size of this structure   */
    void * SubPicture;                      /* Pointer to VIA_SubPicture */
    unsigned char * lpSubpictureBuffer[2];  /* Subpicture frame buffer 0,1 */
    unsigned long dwPitch;                  /* Subpicture frame buffer pitch */
    unsigned long dwState;                  /* Current state of H/W SubPicture Engine */
    Bool  bInitialized;                     /* Except dwState */
    unsigned long dwReserved[8];            /* For future expansion */

    /*
     * Client fills out the following values before calling SubPicure
     */
    unsigned long dwTaskType;                /* Null, Write or Display */

    /* Write Buffer Index */
    unsigned long dwWriteBuffIndex;          /* Write buffer Index. 0 or 1 */

    /* Display Buffer Index */
    unsigned long dwDisplayBuffIndex;        /*Decode buffer Index. 0 or 1        */
    void * lpSPBuffer;                       /*Where SubPicture BMP located       */
    unsigned long  dwSPLeft;                 /*SubPicture position in Video frame */
    unsigned long  dwSPTop;
    unsigned long  dwSPWidth;                /* SubPicture width  */
    unsigned long  dwSPHeight;               /* SubPicture height */
    unsigned long  dwRamTab[16];
    unsigned long  dwRreserved[8];           /* For future expansion */
                                     /* dwRreserved[0] is used for SUBPICTURE surface Pitch */
} VIASUBPICT, * LPVIASUBPICT;


/* dwTaskType of VIASUBPICTURE */
#define VIA_TASK_SP_NULL    0x00000000  /* Used only to check state                           */
#define VIA_TASK_SP_DISPLAY 0x00000002  /* Display SubPciture which                           */
                                        /* frame Idx = dwDisplayBuffIndex                     */
                                        /* If callback function SubPicture receive            */
                                        /* flag = ( VIA_TASK_SP_WRITE | VIA_TASK_SP_DISPLAY ) */
                                        /* Write and Display can be done in one function call */
#define VIA_TASK_SP_RAMTAB  0x00000006  /* Update Ram_Table(16x24) */
#define VIA_TASK_SP_DISPLAY_DISABLE   0x00000008 /* Disable Display SubPciture */
#define VIA_TASK_SP_BLEND2  0x00000010  /* for XVMC, copies the source_surface(dwWriteBuffIndex)
                                         * to the target_surface(dwDisplayBuffIndex) and associates 
                                         * the subpicture with the target_surface. */
#define VIA_TASK_SP_CLEAR   0x00000020  /* for XVMC, Clear the area of the given subpicture 
                                         * to "color". */  
#endif

