/*
 * Copyright 1998-2007 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2007 S3 Graphics, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#include <stdarg.h> /* for variable argument */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/mman.h>

#include "via_h1hwreg.h"
#include "via_driver.h"
#include "via_dri.h"
#include "close/via_drm.h"

#include <errno.h>

unsigned long cur_pointer=0x440;
unsigned char*   MapBase; 

extern void VIASetCursorPosition(ScrnInfoPtr pScrn, int x, int y);

#define viaINREG(addr)         *((volatile unsigned long *)(MapBase + (addr)))
#define viaOUTREG(addr, val)   *((volatile unsigned long *)(MapBase + (addr))) = (val)
#define viaINREG16(addr)       *((volatile unsigned short *)(MapBase + (addr)))
#define viaOUTREG16(addr, val) *((volatile unsigned short *)(MapBase + (addr))) = (val)
typedef unsigned long DWORD;

static int
VIAWait3DEngineIdle(VIAPtr pVia)
{
    int loop = 0;

    mem_barrier();

    while ((VIAGETREG(VIA_REG_STATUS) &
          (VIA_3D_ENG_BUSY)) &&
          (loop++ < MAXLOOP))
        ;

    return loop >= MAXLOOP;
}
#if 0
void Add3DHdr(unsigned long type)
{
  cur_pointer = HC_REG_Hpara0;
  viaOUTREG(HC_REG_TRANS_SET,type);
}
void AddCmdData(unsigned long type)
{
  viaOUTREG(cur_pointer,type);
  cur_pointer += 4;
}
#else
#define Add3DHdr(type) do{ \
            cur_pointer = HC_REG_Hpara0; \
            viaOUTREG(HC_REG_TRANS_SET,type); \
        }while(0)
#define AddCmdData(type) do{ \
            viaOUTREG(cur_pointer,type); \
            cur_pointer += 4; \
            }while(0)
#endif
void hw3d_Scaling_PCI(VIAPtr pVia, unsigned long dstAddr, unsigned long srcAddr,unsigned long dstPitch, 
                    unsigned long srcPitch, unsigned long srcHeight,unsigned long srcWidth, RECTL rSource, 
                    RECTL rDst, int bitsPerPixel, int scalingmode)
{

    float x0, y0, x1, y1, s0, s1, t0, t1; 
    float xa, ya, xb, yb, xc, yc;   

    unsigned long   dwTxFM, dwExp; 
    unsigned long   dwWidthExp, dwHeightExp;
    
    unsigned long   dstX, dstY, srcX, srcY;
    unsigned long   dwSrcWidth, dwSrcHeight;
    unsigned long   dwDstWidth, dwDstHeight;

    unsigned long dwSrcFormat   = bitsPerPixel;
    unsigned long dwDstFormat   = bitsPerPixel;
    unsigned long dwSrcAddr     = srcAddr;
    unsigned long dwDstAddr     = dstAddr;
    unsigned long dwSrcPitch    = srcPitch;
    unsigned long dwDstPitch    = dstPitch;
    unsigned long dwOriSrcWidth     = srcWidth;
    unsigned long dwOriSrcHeight    = srcHeight;
    RECTL rSrc  = rSource;
    RECTL rDest = rDst;
    

    unsigned long dwRatio;

    srcX = rSrc.left;
    srcY = rSrc.top;
    dstX = rDest.left;
    dstY = rDest.top;

    dwSrcWidth = rSrc.right - rSrc.left -1;
    dwSrcHeight = rSrc.bottom - rSrc.top -1;
    dwDstWidth = rDest.right - dstX -1;
    dwDstHeight = rDest.bottom - dstY -1;
    

    switch (dwDstFormat)
    {            
    case 15:
        dwDstFormat = HC_HDBFM_RGB555 | HC_HDBLoc_Local;
        break;
        
    case 16:
        dwDstFormat = HC_HDBFM_RGB565 | HC_HDBLoc_Local;
        break;
        
    case 24:
        dwDstFormat = HC_HDBFM_ARGB0888 | HC_HDBLoc_Local;
        break;
        
    case 32:
        dwDstFormat = HC_HDBFM_ARGB8888 | HC_HDBLoc_Local;
        break;
        
    default:
        return;
    }
    dwDstFormat |= dwDstPitch;

    MapBase= pVia->MapBase;
    VIAWait3DEngineIdle(pVia);
    
    //1.Context setting
    /*For HW limitation, some times, when the former operation is throung AGP path, then go to PCI path,
      the opearation will fail, patch, rewrite the type*/
    Add3DHdr(HC_ParaType_Attr);/*Parameter type:0x01*/
    Add3DHdr(HC_ParaType_Attr);/*Parameter type:0x01*/

    AddCmdData(0x00000400|0x000100c0);   /*Enable: Color Write,Texture Mapping,Disable alpha blending*/
    AddCmdData( 0x43ffffff);                    /*[23:0] Lower 3 byte of Frame Buffer bit mask*/
    AddCmdData( 0x44000cff);                    /*[7:0] Highest byte of Frame Buffer bit mask;[11:8] ROP*/
    AddCmdData( 0x7800ffff);                    /*Line pattern*/
    AddCmdData( 0x79000001);                    /*Line pattern  repeat factor*/
    AddCmdData( 0x7affffff);                    /*Solid sharding color*/
    AddCmdData( 0x7b8000ff);                    /*[23] pixel Grid center(x.0, x.0)*/
    /*You should do modification from here*/
    /*Clipping window: Top[22:12] & Bottom[10:0]*/
    AddCmdData( 0x70000000 | (dstY<<12) | (rDest.bottom));
    /*Clipping window: Left[22:12] & Right[10:0] */                                         
    AddCmdData( 0x71000000 | (dstX<<12) | (rDest.right));
    /*[23:0] Lower 3 byte of DST Base Address*/
    AddCmdData( (0x40<<24) | (dwDstAddr & 0xffffff));
    /*[7:0] Highest byte of DST Base Address */                                         
    AddCmdData( (0x41<<24) | ((dwDstAddr & 0xff000000)>>24));                                              
    /*Destination buffer format[19:16]:RGB16 1,RGB32 9,pitch [13:0]*/
    AddCmdData( (0x42<<24) | dwDstFormat);
    /*End modification*/
    AddCmdData(HC_SubA_HABLCsat | HC_HABLCsat_MASK 
         | HC_HABLCa_Csrc 
         | HC_HABLFCa_OPC    
         | HC_HABLFCa_Asrc);
    AddCmdData(HC_SubA_HABLCop  | HC_HABLCb_Cdst 
         | HC_HABLFCb_InvOPC 
         | HC_HABLFCb_Asrc);
    AddCmdData(HC_SubA_HABLAsat | HC_HABLAsat_MASK 
         | HC_HABLAa_Asrc 
         | HC_HABLFAa_OPA    
         | HC_HABLFAa_Asrc);
    AddCmdData(HC_SubA_HABLAop  | HC_HABLAb_Adst 
         | HC_HABLFAb_InvOPA 
         | HC_HABLFAb_Asrc);
    AddCmdData(HC_SubA_HABLRCbias);
    AddCmdData(HC_SubA_HABLRFCa  );
    AddCmdData(HC_SubA_HABLRFCb  );
    AddCmdData(HC_SubA_HABLRAa   );
    AddCmdData(HC_SubA_HABLRAb   );
    /*End of Context setting*/
    AddCmdData(HC_CmdNULL1);
    AddCmdData(HC_CmdNULL2);    

                                   

    /*2.Texture setting*/
    if (dwSrcFormat == 16)
    {
        dwTxFM = HC_HTXnFM_RGB565;
    }
    else if((dwSrcFormat == 24)||(dwSrcFormat == 32))
    {
        dwTxFM = HC_HTXnFM_ARGB8888;
    }
    else {
	/* 'dwTxFM' may be used uninitialized in this function */
	dwTxFM = 0;
    }
    
    Add3DHdr(HC_ParaType_Tex); /*Parameter type:0x02*/
    Add3DHdr(HC_ParaType_Tex); /*Parameter type:0x02*/

    /*[23:0] Lower 3 byte of Src Base Address*/
    AddCmdData( (0x00<<24) | (dwSrcAddr & 0xffffff));
    /*[7:0] Highest byte of Src Base Address*/
    AddCmdData( (0x20<<24) | ((dwSrcAddr & 0xff000000)>>24));
    AddCmdData( (0x2b<<24) | (0x1<<19) | (dwSrcPitch & 0x3fff));
    /*[19] Enable non power of 2 pitch*/
    /*[13:0] Src pitch in bytes*/

    /*Width Exponential*/
    for(dwExp=0; (DWORD)(1<<dwExp) < dwOriSrcWidth; dwExp++);
    AddCmdData( (0x4b<<24) | dwExp);    
                                          
    /*Height Exponential */
    for(dwExp=0; (DWORD)(1<<dwExp) < dwOriSrcHeight ; dwExp++);
    AddCmdData( (0x51<<24) | dwExp); 



    /* Caculate the source texture coordinate value.*/
    for(dwWidthExp=0; (unsigned long)(1<<dwWidthExp)<(dwOriSrcWidth); dwWidthExp++);    
    for(dwHeightExp=0; (unsigned long)(1<<dwHeightExp)<(dwOriSrcHeight); dwHeightExp++);

    x0 = (float)dstX;
    y0 = (float)dstY;
    x1 = (float)(dwDstWidth*2 + dstX + 1);
    y1 = (float)(dwDstHeight*2 + dstY + 1);

    s0 = ((float)srcX + 0.5f)/((float)(1<<dwWidthExp));
    t0 = ((float)srcY + 0.5f)/((float)(1<<dwHeightExp));


    /* New S, T caculation of 3D hardware blt
       But not yet verified.
       dwRatioS = ((DWORD)(dstX+dwDstWidth+1) / (DWORD)(srcX+dwSrcWidth+1));*/
    dwRatio = ((DWORD)(dwDstWidth+1) / (DWORD)(dwSrcWidth+1));
    
    if (dwRatio >= 4) 
    {
        s1 = ((float)(srcX+dwSrcWidth*2+1)-1.0f)/((float)(1<<dwWidthExp));
    }
    else if (dwRatio >= 2) 
    {
        s1 = ((float)(srcX+dwSrcWidth*2+1)-0.5f)/((float)(1<<dwWidthExp));
    }
    else 
    {
        /* In D3D, the right, bottom edge will not draw, so we can set
           it to the value over the boundary of texture.*/
        s1 = ((float)(srcX+dwSrcWidth*2+1)+0.5f)/((float)(1<<dwWidthExp));
    }
    

    dwRatio = ((DWORD)(dwDstHeight+1) / (DWORD)(dwSrcHeight+1));
    if (dwRatio >= 4) 
    {
        t1 = ((float)(srcY+dwSrcHeight*2+1)-1.0f)/((float)(1<<dwHeightExp));
    }
    else if (dwRatio >= 2) 
    {
        t1 = ((float)(srcY+dwSrcHeight*2+1)-0.5f)/((float)(1<<dwHeightExp));
    }
    else 
    {
        /* In D3D, the right, bottom edge will not draw, so we can set
           it to the value over the boundary of texture.*/
        t1 = ((float)(srcY+dwSrcHeight*2+1)+0.5f)/((float)(1<<dwHeightExp));
    }

    /*Destination triangle vertex coordinates*/
    xa = s0;
    ya = t0;
    xb = s1;
    yb = t0;
    xc = s0;
    yc = t1;
    

    /*other texture setting "needn't be modified"*/
    /*[3:0] Src Format ARGB565/ARGB8888 in Local video mem.*/
    AddCmdData( (0x7b<<24) | dwTxFM);    
    /*Texture level*/
    AddCmdData( 0x77000000);
    if(scalingmode)/* linear: 1; no linear: 0*/
    {
    /*Texture filter : linear*/
        AddCmdData( 0x78002490); 
    }
    else
    {
        AddCmdData(0x78000000);
    }
    /*Texture mapping mode, single*/
    AddCmdData( 0x79000000);

    /*Begin setting the texture blending*/
    AddCmdData( 0x80800000);         
    AddCmdData( 0x8100d022);
    AddCmdData( 0x82a00000);
    AddCmdData( 0x8380c183);

    AddCmdData( 0x85ff0000);
    AddCmdData( 0x8600ff00);
    AddCmdData( 0x870000ff);
    AddCmdData( 0x8800ffff);
    AddCmdData( 0x89000000);
    AddCmdData( 0x8a000000);         
    /*End setting the texture blending*/
    AddCmdData(HC_CmdNULL1);
    AddCmdData(HC_CmdNULL2); 

    /*DD 3DCmdAlign(ppdev);*/
    

    /*3.General Texture attribute*/
    Add3DHdr( 0xfe020000);
    Add3DHdr( 0xfe020000);    
    /* Clear texture cache*/
    AddCmdData(0x00000001);
    /*[HW bug] let HW has enough time to receive bit[6]*/
    AddCmdData(HC_CmdNULL);
    AddCmdData(HC_CmdNULL);
    AddCmdData(HC_CmdNULL);    
    /*for texture number theroy:n+1 0:0+1=1;1:1+1=2;*/
    /*now is 0=>1:one texture*/
    AddCmdData( 0x00000002);
   /*End of the setting of  Texture attribute */    
    AddCmdData(HC_CmdNULL1);
    AddCmdData(HC_CmdNULL2);             
    /*Color Space Conversion setting*/
    /*Setting Coefficients for YUV to RGB conversion(SDTV)
    AddCmdData(0x0113000d);
    AddCmdData(0x02ed1316);
    AddCmdData(0x03071000);*/

    //3.Primitive Setting                       
    /*Command and Primitive Vertex Data*/
    Add3DHdr( 0x00000000);         /*Parameter type:0x00*/
    Add3DHdr( 0x00000000);         /*Parameter type:0x00*/
    
    AddCmdData( 0xec007f9c);        /*Cmd B*/
    AddCmdData( 0xee02105c);        /*Cmd A*/

    /*1st vertex coordinate*/
    AddCmdData(*(unsigned long*)&x0);       /*X,Dst position*/
    AddCmdData(*(unsigned long*)&y0);      /*Y,Dst position*/
    AddCmdData( 0x00000000);        /*Z=0.0*/
    AddCmdData( 0x3f800000);        /*W=1.0*/
    AddCmdData( 0xffff0000);        /*Cd,defuse Color*/
    AddCmdData( 0xffff0000);        /*Cs,specular color*/
    AddCmdData(*(unsigned long*)&xa);       /*S0,Src position*/
    AddCmdData(*(unsigned long*)&ya);      /*T0,Src position*/
        
    /*2nd vertex coordinate*/
    AddCmdData( *(unsigned long*)&x1);      /*X,Dst position*/
    AddCmdData( *(unsigned long*)&y0);       /*Y,Dst position*/
    AddCmdData( 0x00000000);        /*Z=0.0*/
    AddCmdData( 0x3f800000);        /*W=1.0*/
    AddCmdData( 0xffff0000);        /*Cd,defuse Color*/
    AddCmdData( 0xffff0000);        /*Cs,specular color*/
    AddCmdData( *(unsigned long*)&xb);      /*S1,Src position*/
    AddCmdData(*(unsigned long*)&yb);       /*T0,Src position*/

    /*3rd vertex coordinate*/
    AddCmdData( *(unsigned long*)&x0);      /*X,Dst position*/
    AddCmdData( *(unsigned long*)&y1);      /*Y,Dst position*/
    AddCmdData( 0x00000000);        /*Z=0.0*/
    AddCmdData( 0x3f800000);        /*W=1.0*/
    AddCmdData( 0xffff0000);        /*Cd,defuse Color*/
    AddCmdData( 0xffff0000);        /*Cs,specular color  */
    AddCmdData( *(unsigned long*)&xc);      /*S0,Src position*/
    AddCmdData( *(unsigned long*)&yc);      /*T1,Src position*/
    AddCmdData( 0xee121378);        /*Cmd A,fire  */
    VIAWait3DEngineIdle(pVia);    

}
/*JohnsonZheng[20070406]-tv scaling agp path*/
#define HC_HEADER2		0xF210F110
#define HC_DUMMY		0xCCCCCCCC

#if 0
FILE *fp;
#define AddCmdData(data) do {					\
    fp = fopen("/var/log/ring.txt","a+");			\
    if (fp){							\
	fprintf(fp,"0x%x=0x%x\n",cur_nr,(data));		\
	fflush(fp);						\
	fclose(fp);						\
    }								\
    _dwCurPointer[cur_nr++] = (data);				\
} while(0)
#endif

#ifdef AddCmdData
#undef AddCmdData
#define AddCmdData(data) _dwCurPointer[cur_nr++] = (data)
#endif
#ifdef Add3DHdr
#undef Add3DHdr
#define Add3DHdr(header) do{ \
	AddCmdData(0xF210F110); \
	AddCmdData(header); \
}while(0)
#endif

#define AlignCmdData() do {					\
    if(cur_nr % 2) {						\
	AddCmdData(HC_DUMMY);					\
    }								\
} while(0)

#define ResetCmdData() cur_nr = 0

void hw3d_Scaling_AGP(VIAPtr pVia, unsigned long dstAddr, unsigned long srcAddr,unsigned long dstPitch, 
                    unsigned long srcPitch, unsigned long srcHeight,unsigned long srcWidth, RECTL rSource, 
                    RECTL rDst, int bitsPerPixel, int scalingmode)
{

    float x0, y0, x1, y1, s0, s1, t0, t1; 
    float xa, ya, xb, yb, xc, yc;   
    unsigned long   dwTxFM, dwExp; 
    unsigned long   dwWidthExp, dwHeightExp;
    unsigned long   dstX, dstY, srcX, srcY;
    unsigned long   dwSrcWidth, dwSrcHeight;
    unsigned long   dwDstWidth, dwDstHeight;
    unsigned long dwSrcFormat   = bitsPerPixel;
    unsigned long dwDstFormat   = bitsPerPixel;
    unsigned long dwSrcAddr     = srcAddr;
    unsigned long dwDstAddr     = dstAddr;
    unsigned long dwSrcPitch    = srcPitch;
    unsigned long dwDstPitch    = dstPitch;
    unsigned long dwOriSrcWidth     = srcWidth;
    unsigned long dwOriSrcHeight    = srcHeight;
    RECTL rSrc  = rSource;
    RECTL rDest = rDst;
    unsigned long dwRatio;

    srcX = rSrc.left;
    srcY = rSrc.top;
    dstX = rDest.left;
    dstY = rDest.top;

    dwSrcWidth = rSrc.right - rSrc.left -1;
    dwSrcHeight = rSrc.bottom - rSrc.top -1;
    dwDstWidth = rDest.right - dstX -1;
    dwDstHeight = rDest.bottom - dstY -1;
    

    switch (dwDstFormat)
    {            
    case 15:
        dwDstFormat = HC_HDBFM_RGB555 | HC_HDBLoc_Local;
        break;
        
    case 16:
        dwDstFormat = HC_HDBFM_RGB565 | HC_HDBLoc_Local;
        break;
        
    case 24:
        dwDstFormat = HC_HDBFM_ARGB0888 | HC_HDBLoc_Local;
        break;
        
    case 32:
        dwDstFormat = HC_HDBFM_ARGB8888 | HC_HDBLoc_Local;
        break;
        
    default:
        return;
    }
    dwDstFormat |= dwDstPitch;
#if 0
#include "via_dma.h"
   static int count_nr=0;
   if(count_nr==0){
        count_nr = 1;

        drm_via_private_t* drm_dev_priv = (drm_via_private_t*)malloc(sizeof(drm_via_private_t));
        memset(drm_dev_priv,0,sizeof(drm_via_private_t));
        drm_dev_priv->mmio=(drm_map_t*)malloc(sizeof(drm_map_t));
        memset(drm_dev_priv->mmio,0,sizeof(drm_map_t));
        /*drm_dev_priv->agpAddr = gDRIPriv->agp.handle;
        drm_dev_priv->agpLinearAddr = viaScreen->agpLinearStart;
        drm_dev_priv->mmio->offset= ;
        drm_dev_priv->mmio->size = ;*/
        drm_dev_priv->mmio->handle = MapBase;
        pVia->drm_dev_priv = (void*)drm_dev_priv;
    }
#endif

    #define PCI_CMDBUF_SIZE 0x400
    unsigned int cur_nr=0;
    unsigned int _dwCurPointer[PCI_CMDBUF_SIZE]; 
    drm_via_cmdbuffer_t bufI;

    MapBase= pVia->MapBase;
    VIAWait3DEngineIdle(pVia);
    Add3DHdr(0x00fe0000);
    AddCmdData(0x80000000); //Wait 3D idle
    AddCmdData(0x80000000); //Wait 3D idle 
    AlignCmdData();
    //1.Context setting
    /*For HW limitation, some times, when the former operation is throung AGP path, then go to PCI path,
      the opearation will fail, patch, rewrite the type*/
    Add3DHdr(HC_ParaType_Attr);/*Parameter type:0x01*/
    //Add3DHdr(HC_ParaType_Attr);/*Parameter type:0x01*/

    AddCmdData(0x00000400|0x000100c0);   /*Enable: Color Write,Texture Mapping,Disable alpha blending*/
    AddCmdData( 0x43ffffff);                    /*[23:0] Lower 3 byte of Frame Buffer bit mask*/
    AddCmdData( 0x44000cff);                    /*[7:0] Highest byte of Frame Buffer bit mask;[11:8] ROP*/
    AddCmdData( 0x7800ffff);                    /*Line pattern*/
    AddCmdData( 0x79000001);                    /*Line pattern  repeat factor*/
    AddCmdData( 0x7affffff);                    /*Solid sharding color*/
    AddCmdData( 0x7b8000ff);                    /*[23] pixel Grid center(x.0, x.0)*/
    /*You should do modification from here*/
    /*Clipping window: Top[22:12] & Bottom[10:0]*/
    AddCmdData( 0x70000000 | (dstY<<12) | (rDest.bottom));
    /*Clipping window: Left[22:12] & Right[10:0] */                                         
    AddCmdData( 0x71000000 | (dstX<<12) | (rDest.right));
    /*[23:0] Lower 3 byte of DST Base Address*/
    AddCmdData( (0x40<<24) | (dwDstAddr & 0xffffff));
    /*[7:0] Highest byte of DST Base Address */                                         
    AddCmdData( (0x41<<24) | ((dwDstAddr & 0xff000000)>>24));                                              
    /*Destination buffer format[19:16]:RGB16 1,RGB32 9,pitch [13:0]*/
    AddCmdData( (0x42<<24) | dwDstFormat);
    /*End modification*/
#if 0
    AddCmdData(HC_SubA_HABLCsat | HC_HABLCsat_MASK 
         | HC_HABLCa_Csrc 
         | HC_HABLFCa_OPC    
         | HC_HABLFCa_Asrc);
    AddCmdData(HC_SubA_HABLCop  | HC_HABLCb_Cdst 
         | HC_HABLFCb_InvOPC 
         | HC_HABLFCb_Asrc);
    AddCmdData(HC_SubA_HABLAsat | HC_HABLAsat_MASK 
         | HC_HABLAa_Asrc 
         | HC_HABLFAa_OPA    
         | HC_HABLFAa_Asrc);
    AddCmdData(HC_SubA_HABLAop  | HC_HABLAb_Adst 
         | HC_HABLFAb_InvOPA 
         | HC_HABLFAb_Asrc);
    AddCmdData(HC_SubA_HABLRCbias);
    AddCmdData(HC_SubA_HABLRFCa  );
    AddCmdData(HC_SubA_HABLRFCb  );
    AddCmdData(HC_SubA_HABLRAa   );
    AddCmdData(HC_SubA_HABLRAb   );
    /*End of Context setting*/
    AddCmdData(HC_CmdNULL1);
    AddCmdData(HC_CmdNULL2);    
#endif
    AlignCmdData();
                                   

    /*2.Texture setting*/
    if (dwSrcFormat == 16)
    {
        dwTxFM = HC_HTXnFM_RGB565;
    }
    else if((dwSrcFormat == 24)||(dwSrcFormat == 32))
    {
        dwTxFM = HC_HTXnFM_ARGB8888;
    }
    else {
	/* 'dwTxFM' may be used uninitialized in this function */
	dwTxFM = 0;
    }
    
    Add3DHdr(0x20000); /*Parameter type:0x02*/
    //Add3DHdr(HC_ParaType_Tex); /*Parameter type:0x02*/

    /*[23:0] Lower 3 byte of Src Base Address*/
    AddCmdData( (0x00<<24) | (dwSrcAddr & 0xffffff));
    /*[7:0] Highest byte of Src Base Address*/
    AddCmdData( (0x20<<24) | ((dwSrcAddr & 0xff000000)>>24));
    AddCmdData( (0x2b<<24) | (0x1<<19) | (dwSrcPitch & 0x3fff));
    /*[19] Enable non power of 2 pitch*/
    /*[13:0] Src pitch in bytes*/

    /*Width Exponential*/
    for(dwExp=0; (DWORD)(1<<dwExp) < dwOriSrcWidth; dwExp++);
    AddCmdData( (0x4b<<24) | dwExp);    
                                          
    /*Height Exponential */
    for(dwExp=0; (DWORD)(1<<dwExp) < dwOriSrcHeight ; dwExp++);
    AddCmdData( (0x51<<24) | dwExp); 



    /* Caculate the source texture coordinate value.*/
    for(dwWidthExp=0; (unsigned long)(1<<dwWidthExp)<(dwOriSrcWidth); dwWidthExp++);    
    for(dwHeightExp=0; (unsigned long)(1<<dwHeightExp)<(dwOriSrcHeight); dwHeightExp++);

    x0 = (float)dstX;
    y0 = (float)dstY;
    x1 = (float)(dwDstWidth*2 + dstX + 1);
    y1 = (float)(dwDstHeight*2 + dstY + 1);

    s0 = ((float)srcX + 0.5f)/((float)(1<<dwWidthExp));
    t0 = ((float)srcY + 0.5f)/((float)(1<<dwHeightExp));


    /* New S, T caculation of 3D hardware blt
       But not yet verified.
       dwRatioS = ((DWORD)(dstX+dwDstWidth+1) / (DWORD)(srcX+dwSrcWidth+1));*/
    dwRatio = ((DWORD)(dwDstWidth+1) / (DWORD)(dwSrcWidth+1));
    
    if (dwRatio >= 4) 
    {
        s1 = ((float)(srcX+dwSrcWidth*2+1)-1.0f)/((float)(1<<dwWidthExp));
    }
    else if (dwRatio >= 2) 
    {
        s1 = ((float)(srcX+dwSrcWidth*2+1)-0.5f)/((float)(1<<dwWidthExp));
    }
    else 
    {
        /* In D3D, the right, bottom edge will not draw, so we can set
           it to the value over the boundary of texture.*/
        s1 = ((float)(srcX+dwSrcWidth*2+1)+0.5f)/((float)(1<<dwWidthExp));
    }
    

    dwRatio = ((DWORD)(dwDstHeight+1) / (DWORD)(dwSrcHeight+1));
    if (dwRatio >= 4) 
    {
        t1 = ((float)(srcY+dwSrcHeight*2+1)-1.0f)/((float)(1<<dwHeightExp));
    }
    else if (dwRatio >= 2) 
    {
        t1 = ((float)(srcY+dwSrcHeight*2+1)-0.5f)/((float)(1<<dwHeightExp));
    }
    else 
    {
        /* In D3D, the right, bottom edge will not draw, so we can set
           it to the value over the boundary of texture.*/
        t1 = ((float)(srcY+dwSrcHeight*2+1)+0.5f)/((float)(1<<dwHeightExp));
    }

    /*Destination triangle vertex coordinates*/
    xa = s0;
    ya = t0;
    xb = s1;
    yb = t0;
    xc = s0;
    yc = t1;
    

    /*other texture setting "needn't be modified"
     *[3:0] Src Format ARGB565/ARGB8888 in Local video mem.*/
    AddCmdData( (0x7b<<24) | dwTxFM);    
    /*Texture level*/
    AddCmdData( 0x77000000);
    if(scalingmode)/* linear: 1; no linear: 0*/
    {
    /*Texture filter : linear*/
        AddCmdData( 0x78002490); 
    }
    else
    {
        AddCmdData(0x78000000);
    }
    /*Texture mapping mode, single*/
    AddCmdData( 0x79000000);

    /*Begin setting the texture blending*/
    AddCmdData( 0x80800000);         
    AddCmdData( 0x8100d022);
    AddCmdData( 0x82a00000);
    AddCmdData( 0x8380c183);

    AddCmdData( 0x85ff0000);
    AddCmdData( 0x8600ff00);
    AddCmdData( 0x870000ff);
    AddCmdData( 0x8800ffff);
    AddCmdData( 0x89000000);
    AddCmdData( 0x8a000000);         
    /*End setting the texture blending*/
    /*AddCmdData(HC_CmdNULL1);
    AddCmdData(HC_CmdNULL2); */
    AlignCmdData();

    /*DD 3DCmdAlign(ppdev);*/
    

    /*3.General Texture attribute*/
    Add3DHdr( 0xfe020000);
    //Add3DHdr( 0xfe020000);    
    // Clear texture cache
    AddCmdData(0x00000001);
    //[HW bug] let HW has enough time to receive bit[6]
    AddCmdData(HC_CmdNULL);
    AddCmdData(HC_CmdNULL);
    AddCmdData(HC_CmdNULL);    
    /*for texture number theroy:n+1 0:0+1=1;1:1+1=2;*/
    /*now is 0=>1:one texture*/
    AddCmdData( 0x00000002);
   /*End of the setting of  Texture attribute */    
    /*AddCmdData(HC_CmdNULL1);
    AddCmdData(HC_CmdNULL2);*/
    AlignCmdData();             
    /*Color Space Conversion setting*/
    /*Setting Coefficients for YUV to RGB conversion(SDTV)
    AddCmdData(0x0113000d);
    AddCmdData(0x02ed1316);
    AddCmdData(0x03071000);*/

    //3.Primitive Setting                       
    /*Command and Primitive Vertex Data*/
    Add3DHdr( 0x00000000);         /*Parameter type:0x00*/
    //Add3DHdr( 0x00000000);         /*Parameter type:0x00*/
    
    AddCmdData( 0xec007f9c);        /*Cmd B*/
    AddCmdData( 0xee02105c);        /*Cmd A*/

    /*1st vertex coordinate*/
    AddCmdData(*(unsigned long*)&x0);       /*X,Dst position*/
    AddCmdData(*(unsigned long*)&y0);      /*Y,Dst position*/
    AddCmdData( 0x00000000);        /*Z=0.0*/
    AddCmdData( 0x3f800000);        /*W=1.0*/
    AddCmdData( 0xffff0000);        /*Cd,defuse Color*/
    AddCmdData( 0xffff0000);        /*Cs,specular color*/
    AddCmdData(*(unsigned long*)&xa);       /*S0,Src position*/
    AddCmdData(*(unsigned long*)&ya);      /*T0,Src position*/
        
    /*2nd vertex coordinate*/
    AddCmdData( *(unsigned long*)&x1);      /*X,Dst position*/
    AddCmdData( *(unsigned long*)&y0);       /*Y,Dst position*/
    AddCmdData( 0x00000000);        /*Z=0.0*/
    AddCmdData( 0x3f800000);        /*W=1.0*/
    AddCmdData( 0xffff0000);        /*Cd,defuse Color*/
    AddCmdData( 0xffff0000);        /*Cs,specular color*/
    AddCmdData( *(unsigned long*)&xb);      /*S1,Src position*/
    AddCmdData(*(unsigned long*)&yb);       /*T0,Src position*/

    /*3rd vertex coordinate*/
    AddCmdData( *(unsigned long*)&x0);      /*X,Dst position*/
    AddCmdData( *(unsigned long*)&y1);      /*Y,Dst position*/
    AddCmdData( 0x00000000);        /*Z=0.0*/
    AddCmdData( 0x3f800000);        /*W=1.0*/
    AddCmdData( 0xffff0000);        /*Cd,defuse Color*/
    AddCmdData( 0xffff0000);        /*Cs,specular color  */
    AddCmdData( *(unsigned long*)&xc);      /*S0,Src position*/
    AddCmdData( *(unsigned long*)&yc);      /*T1,Src position*/
    AddCmdData( 0xee121378);        /*Cmd A,fire  */
    AddCmdData( 0xee121378);        /*Cmd A,fire  */
    AlignCmdData();
    #if 0
    Add3DHdr(HC_ParaType_Attr);/*Parameter type:0x01*/
    AddCmdData (HC_DUMMY);
    AddCmdData (HC_DUMMY);
    AlignCmdData();
    #endif
//****************************************************************

#if 1
   int ret;
   bufI.size=4*cur_nr;
   bufI.buf=(char*)(_dwCurPointer);
   do {
                        /*ret = via_pci_cmdbuffer(pVia->drm_dev_priv,&bufI);
                        ret = via_cmdbuffer(pVia->drm_dev_priv,&bufI);*/
                        ret = drmCommandWrite(pVia->drmFD, DRM_VIA_CMDBUFFER,
                               &bufI, sizeof(bufI));
                        /*ret = drmCommandWrite(pVia->drmFD, DRM_VIA_PCICMD,
                               &bufI, sizeof(bufI));*/
    } while (ret == -EAGAIN);
#endif
   ResetCmdData();
  //****************************************************************
    //VIAWait3DEngineIdle(pVia);    

}
/*JohnsonZheng-End*/
void hw3d_CursorPositionUpdate(VIABIOSInfoPtr pBIOSInfo)
{
    int x,y;
    ScrnInfoPtr pScrn = xf86Screens[pBIOSInfo->scrnIndex];
    VIAPtr      pVia = VIAPTR(pScrn);
    x = pVia->DISP3DSCALCTRLInfo.CursorX;
    y = pVia->DISP3DSCALCTRLInfo.CursorY;
    VIASetCursorPosition(pScrn, x, y);
}

void hw3d_MonoCursorImageAdjust(VIAPtr pVia)
{
}

void hw3d_ARGBCursorImageAdjust(VIAPtr pVia)
{
    VIABIOSInfoPtr pBIOSInfo = pVia->pBIOSInfo;
    VIADISP3DSCALCTRLPtr p3DSCALCTRLInfo = &(pVia->DISP3DSCALCTRLInfo);
    RECTL rSrc, rDest;
    CARD32 dwSrcWidth, dwSrcHeight;
    CARD32 lSrcPitch, lDestPitch;
    DWORD     lSrcAddr=0,lDestAddr = 0;

    rSrc.left = rSrc.top = 0;
    rSrc.right = rSrc.bottom = MAX_CURS;
    dwSrcWidth = dwSrcHeight = MAX_CURS;
    /*Note the alpha cursor should be 32bpp */
    lSrcPitch = MAX_CURS * 4;
   
    rDest.left = rDest.top = 0;
    rDest.right = rSrc.right* (p3DSCALCTRLInfo->RealHActive) /(pBIOSInfo->CrtcHDisplay);
    rDest.bottom = rSrc.bottom* (p3DSCALCTRLInfo->RealVActive)/(pBIOSInfo->CrtcVDisplay);
    lDestPitch = MAX_CURS *4;
    
    lSrcAddr = pVia->Cursor3DScalingBufferStart;
    lDestAddr = pVia->CursorStart;

    if(!pVia->useAgp)
    {
        /*lock hw 3D in PCI path*/
#ifdef XF86DRI        
        viaLOCK_HARDWARE(p3DSCALCTRLInfo);
#endif        
        hw3d_Scaling_PCI(pVia,lDestAddr,lSrcAddr,lDestPitch, lSrcPitch,dwSrcHeight,dwSrcWidth, rSrc, rDest,32,1);
#ifdef XF86DRI        
        viaUNLOCK_HARDWARE(p3DSCALCTRLInfo);
#endif        
    }
    else
    {    
        hw3d_Scaling_AGP(pVia,lDestAddr,lSrcAddr,lDestPitch, lSrcPitch,dwSrcHeight,dwSrcWidth, rSrc, rDest,32,1);
    }    
}


