/*
 * Copyright 1998-2007 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2007 S3 Graphics, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
 
#include <sys/types.h>
#include <sys/mman.h>
#include "via_driver.h"
#include "via_tv.h"
#include "via_hw.h"
#include "via_tblIntegratedTV.h"
#include "via_IntegratedTV.h"

static void WriteIntegratedTVReg8(unsigned char* mmio_base, CARD8 offset, CARD8 data);
static void WriteIntegratedTVReg32(unsigned char* mmio_base, CARD8 offset, CARD32 data);
static CARD32 ReadIntegratedTVReg32(unsigned char* mmio_base, CARD8 offset);
static void WriteIntegratedTVGeneralTable(VIABIOSInfoPtr pBIOSInfo, CARD16 *tv_reg_offset_table, CARD32 *tv_reg_table);
static void SetIntegratedTVOutputSignal(VIABIOSInfoPtr pBIOSInfo, int DACSelect, TVSETTINGINFOPTR pTVSettingInfo);
static void WriteGfxCRTCTimingForTV(VIABIOSInfoPtr pBIOSInfo, CARD32  *tv_crtc_table, int IGAPath);
static void UpdateScalingFactor(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo);
static void SetHDTVPatchTable(VIABIOSInfoPtr pBIOSInfo, int mode_index, TVSETTINGINFOPTR pTVSettingInfo);
static void SetTVOutputPatchTable(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo);
static void PatchForMacrovision(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo);
static void VIABlankRectangle( VIAPtr pVia, CARD32 RectAddr, CARD32 RectDimension, CARD32 RectPitch);
static Bool BlackSclScrnBoundary(VIAPtr pVia, CARD32 RectAddr, RECTL rDest);
static Bool DISP3DScalWindowAdjust(RECTLPtr prSrc, RECTLPtr prDest, VIA3DSclSCNParasPtr pDISP3DScalScreenInfo);
static Bool VIASet3DScalScreenParas(VIABIOSInfoPtr  pBIOSInfo);
static void WriteIntegratedTVReg32Mask(unsigned char* mmio_base, CARD8 offset, CARD32 mask, CARD32 data);
static void VIASet3DScalCtrlParas(ScrnInfoPtr pScrn);
static void WriteIntegratedTVPatchTable(VIABIOSInfoPtr pBIOSInfo, CARD16 *tv_reg_table);


void EnableIntegratedTVShadowReg(VIABIOSInfoPtr pBIOSInfo)
{
    int i;
    VIABIOSInfoPtr  pVia = pBIOSInfo;

    /* Enable integrated TV shadow register. */

    /* Only for A0000 now, we may modify this if need other TV shadow memory. */

    /* RainLin [2005.10.11]: We should use VGAOUT8()/VGAIN8() instead of */
    /* write_reg()/read_reg() or it may fail when leave X-Window and back to */
    /* console in SAMM mode. */
    VGAOUT8(0x3c4, 0x52);
    VGAOUT8(0x3c5, 0x01);

    /* Wait data ready. */
    for (i=0; i<10; i++)
    {
        if (VGAIN8(0x3c5) & BIT0)
        {
            break;
        }
    }
}


void DisableIntegratedTVShadowReg(VIABIOSInfoPtr pBIOSInfo)
{
    VIABIOSInfoPtr  pVia = pBIOSInfo;

    /* RainLin [2005.10.11]: We should use VGAOUT8()/VGAIN8() instead of */
    /* write_reg()/read_reg() or it may fail when leave X-Window and back to */
    /* console in SAMM mode. */
    VGAOUT8(0x3c4, 0x52);
    VGAOUT8(0x3c5, 0x00);
}  


static void
WriteIntegratedTVReg8(unsigned char* mmio_base, CARD8 offset, CARD8 data)
{
    *(volatile CARD8 *)(mmio_base + offset) = data;
    DEBUG(ErrorF("Write Integrated TV Reg (byte): Offset 0x%2x, Value 0x%2x!\n", offset, data));
}


static void
WriteIntegratedTVReg32(unsigned char* mmio_base, CARD8 offset, CARD32 data)
{
    *(volatile CARD32 *)(mmio_base + offset) = data;
    DEBUG(ErrorF("Write Integrated TV Reg: Offset 0x%2x, Value 0x%8x!\n", offset, data));
}


static void
WriteIntegratedTVReg32Mask(unsigned char* mmio_base, CARD8 offset, CARD32 mask, CARD32 data)
{
    CARD32 old_data, new_data;

    old_data = *(volatile CARD32 *)(mmio_base + offset);
    new_data = (data & mask) | (old_data & (~mask));
    *(volatile CARD32 *)(mmio_base + offset) = new_data;

    DEBUG(ErrorF("Write Integrated TV Reg Mask: Offset 0x%2x, Value 0x%8x!\n", offset, new_data));
}


static CARD32
ReadIntegratedTVReg32(unsigned char* mmio_base, CARD8 offset)
{
    CARD32 data;

    data = *(volatile CARD32 *)(mmio_base + offset);

    return data;
}


static void
WriteIntegratedTVGeneralTable(VIABIOSInfoPtr pBIOSInfo, CARD16 *tv_reg_offset_table, CARD32 *tv_reg_table)
{
    int i = 0;
    
    while(tv_reg_offset_table[i] != 0xFFFF)
    {
        WriteIntegratedTVReg32(pBIOSInfo->IntegratedTVMapBase, GET_LOW_BYTE(tv_reg_offset_table[i]), tv_reg_table[i]);
        i++;
    }
}


static void
WriteIntegratedTVPatchTable(VIABIOSInfoPtr pBIOSInfo, CARD16 *tv_reg_table)
{
    int i = 0;

    while (tv_reg_table[i] != 0xFFFF) {
	WriteIntegratedTVReg8(pBIOSInfo->IntegratedTVMapBase,
			      GET_LOW_BYTE(tv_reg_table[i]),
			      GET_HIGH_BYTE(tv_reg_table[i]));
	i++;
    }
}


CARD32 AccessIntegratedTVRegs(VIABIOSInfoPtr pBIOSInfo, int reg_id, CARD32 value, int cmd)
{
    CARD8 offset;
    CARD32 mask, set_data, get_data, shift_bits, tmp1, tmp2;

    /* Note: It's better to call EnableIntegratedTVShadowReg() to enable integrated TV shadow register */
    /* before access the following registers, or it may fail to access them. */
    
    switch (reg_id)
    {
        case ITV_REG_ID_PK_ENABLE:
        {
            /* 00[0] */
            offset = 0x00;
            mask = 0x00000001;
            shift_bits = 0;

            break;
        }

        case ITV_REG_ID_SRC_SEL:
        {
            /* 00[18] */
            offset = 0x00;
            mask = 0x00040000;
            shift_bits = 18;
            
            break;
        }

        case ITV_REG_ID_SW_RESET:
        {
            /* 00[20] */
            offset = 0x00;
            mask = 0x00100000;
            shift_bits = 20;
            
            break;
        }

        case ITV_REG_ID_SYNC_BUF_MODE:
        {
            /* 00[30] */
            offset = 0x00;
            mask = 0x40000000;
            shift_bits = 30;
            
            break;
        }

        case ITV_REG_ID_SD_ENABLE:
        {
            /* 00[31] */
            offset = 0x00;
            mask = 0x80000000;
            shift_bits = 31;
            
            break;
        }

        case ITV_REG_ID_SRC_VSIZE:
        {
            /* 18[10:0] */
            offset = 0x18;
            mask = 0x000007FF;
            shift_bits = 0;
            
            break;
        }

        case ITV_REG_ID_SRC_HSIZE:
        {
            /* 18[21:11] */
            offset = 0x18;
            mask = 0x003FF800;
            shift_bits = 11;
            
            break;
        }

        case ITV_REG_ID_DE_FLK_THRESHOLD:
        {
            /* 20[31:27] + 20[15:11] */
            offset = 0x20;
                
            if (cmd == ITV_CMD_WRITE)
            {
                tmp1 = value & 0x1F;        /* bit4-bit0 */
                tmp2 = (value >> 5) & 0x1F; /* bit9-bit5 */
                set_data = (tmp2 << 27) | (tmp1 << 11);
                WriteIntegratedTVReg32Mask(pBIOSInfo->IntegratedTVMapBase, offset, 0xF800F800, set_data);
                
                return 0;  /* 0 means nothing, just for return. */
            }
            else
            {
                get_data = ReadIntegratedTVReg32(pBIOSInfo->IntegratedTVMapBase, offset);
                tmp1 = (get_data & 0x0000F800) >> 11;   /* bit4-bit0 */
                tmp2 = (get_data & 0xF8000000) >> 27;   /* bit9-bit5 */
                get_data = (tmp2 << 5) | tmp1;

                return get_data;
            }
        }
        
        case ITV_REG_ID_SCL_VFAC:
        {
            /* 24[12:0] */
            offset = 0x24;
            mask = 0x00001FFF;
            shift_bits = 0;
            
            break;
        }

        case ITV_REG_ID_DE_FLK_NUM:
        {
            /* 24[13] */
            offset = 0x24;
            mask = 0x00002000;
            shift_bits = 13;
            
            break;
        }
        
        case ITV_REG_ID_DF_WT_SEL:
        {
            /* 24[14] */
            offset = 0x24;
            mask = 0x00004000;
            shift_bits = 14;
            
            break;
        }

        case ITV_REG_ID_DF_TYPE:
        {
            /* 24[15] */
            offset = 0x24;
            mask = 0x00008000;
            shift_bits = 15;
            
            break;
        }
        
        case ITV_REG_ID_DE_FLICKER_ENABLE:
        {
            /* 24[29] */
            offset = 0x24;
            mask = 0x20000000;
            shift_bits = 29;
            
            break;
        }
        
        case ITV_REG_ID_SCL_HFAC:
        {
            /* 24[28:16] */
            offset = 0x24;
            mask = 0x1FFF0000;
            shift_bits = 16;
            
            break;
        }

        case ITV_REG_ID_DST_VSIZE:
        {
            /* 28[10:0] */
            offset = 0x28;
            mask = 0x000007FF;
            shift_bits = 0;
            
            break;
        }

        case ITV_REG_ID_DST_HSIZE:
        {
            /* 28[21:11] */
            offset = 0x28;
            mask = 0x003FF800;
            shift_bits = 11;
            
            break;
        }

        case ITV_REG_ID_TV_ENABLE:
        {
            /* 38[0] */
            offset = 0x38;
            mask = 0x00000001;
            shift_bits = 0;

            break;
        }

        case ITV_REG_ID_SEC_DAC_OUTPUT_SEL:
        {
            /* 38[27:26] */
            offset = 0x38;
            mask = 0x0C000000;
            shift_bits = 26;

            break;
        }

        case ITV_REG_ID_PRI_DAC_OUTPUT_SEL:
        {
            /* 38[29:28] */
            offset = 0x38;
            mask = 0x30000000;
            shift_bits = 28;

            break;
        }

        case ITV_REG_ID_TV_SENSE:
        {
            /* 3C[15:14] */
            offset = 0x3C;
            mask = 0x0000C000;
            shift_bits = 14;
            
            break;
        }
        
        case ITV_REG_ID_TV_SENSE_DATA:
        {
            /* 3C[31:24] */
            offset = 0x3C;
            mask = 0xFF000000;
            shift_bits = 24;
            
            break;
        }
        
        case ITV_REG_ID_DST_HSD:
        {
            /* 48[30:21] */
            offset = 0x48;
            mask = 0x7FE00000;
            shift_bits = 21;

            break;
        }

        case ITV_REG_ID_DST_VSD:
        {
            /* 4C[31:23] */
            offset = 0x4C;
            mask = 0xFF800000;
            shift_bits = 23;

            break;
        }

        case ITV_REG_ID_BLACK_LEVEL:
        {
            /* 54[30:23] */
            offset = 0x54;
            mask = 0x7F800000;
            shift_bits = 23;
            break;
        }

        case ITV_REG_ID_HUE:
        {
            /* 60[10:0] */
            offset = 0x60;
            mask = 0x000007FF;
            shift_bits = 0;
            break;
        }

        case ITV_REG_ID_SEC_DAC_SEL:
        {
            /* 6C[11:10] */
            offset = 0x6C;
            mask = 0x00000C00;
            shift_bits = 10;            
            break;
        }
        
        case ITV_REG_ID_PRI_DAC_SEL:
        {
            /* 6C[12] */
            offset = 0x6C;
            mask = 0x00001000;
            shift_bits = 12;
            
            break;
        }

        case ITV_REG_ID_TV_SENSE_STATUS:
        {
            /* E4[2] */
            offset = 0xE4;
            mask = 0x00000004;
            shift_bits = 2;
            
            break;
        }

        case ITV_REG_ID_PBPR_BLANK_LEVEL:
        {
            /* E8[8:0] */
            offset = 0xE8;
            mask = 0x000000FF;
            shift_bits = 0; 
            break;
        }
        
        case ITV_REG_ID_SEC_DAC_TV_SENSE_RGB_STATUS:
        {
            /* E8[28:25] */
            offset = 0xE8;
            mask = 0x0E000000;
            shift_bits = 25;            
            break;
        }

        case ITV_REG_ID_PRI_DAC_TV_SENSE_RGB_STATUS:
        {
            /* E8[30:28] */
            offset = 0xE8;
            mask = 0x70000000;
            shift_bits = 28;
            
            break;
        }
        
        default:
        {
            return 0;  /* 0 means nothing, just for return. */
        }
    }

    if (cmd == ITV_CMD_WRITE)
    {
        set_data = value << shift_bits;
        WriteIntegratedTVReg32Mask(pBIOSInfo->IntegratedTVMapBase, offset, mask, set_data);
        
        return 0;  /* 0 means nothing, just for return. */
    }
    else
    {
        get_data = ReadIntegratedTVReg32(pBIOSInfo->IntegratedTVMapBase, offset);
        get_data = (get_data & mask) >> shift_bits;

        return get_data;
    }
}


static void
SetIntegratedTVOutputSignal(VIABIOSInfoPtr pBIOSInfo, int DACSelect, TVSETTINGINFOPTR pTVSettingInfo)
{
    VIABIOSInfoPtr  pVia = pBIOSInfo;
    CARD8   sr5e;
    int DACOutputSelect;
    
    /* RainLin [2006.01.09]: HDTV only can use RGB or YPbPr output. */
    if ((pTVSettingInfo->TVType == TVTYPE_480P) ||
        (pTVSettingInfo->TVType == TVTYPE_576P) ||
        (pTVSettingInfo->TVType == TVTYPE_720P) ||
        (pTVSettingInfo->TVType == TVTYPE_1080I))
        
    {
        if ((pTVSettingInfo->TVOutput != TV_OUTPUT_RGB) &&
            (pTVSettingInfo->TVOutput != TV_OUTPUT_YPBPR))
        {
            pTVSettingInfo->TVOutput = TV_OUTPUT_RGB;
        }
    }

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "SetIntegratedTVOutputSignal: TVOutput = %d\n", pTVSettingInfo->TVOutput));

    /* RainLin [2006.01.16]: We could separate Composite and S-Video by */
    /* controlling DAC R/G/B. */
    VGAOUT8(0x3c4, SR5E);
    sr5e = VGAIN8(0x3c5) & 0xF1;    /* Turn on DAC R/G/B. */

    /* select primary DAC or secondary DAC */
    if (DACSelect == ITV_DAC_SECONDARY)
    {
        DACOutputSelect = ITV_REG_ID_SEC_DAC_OUTPUT_SEL;
    }
    else
    {
        DACOutputSelect = ITV_REG_ID_PRI_DAC_OUTPUT_SEL;
    }
    
    switch (pTVSettingInfo->TVOutput)
    {
        case TV_OUTPUT_COMPOSITE:
        {            
            AccessIntegratedTVRegs(pBIOSInfo, DACOutputSelect, ITV_VALUE_OUTPUT_SC, ITV_CMD_WRITE);
            
            /* Turn off DAC R and G, turn on DAC B. */
            VGAOUT8(0x3c5, sr5e | (BIT1+BIT2));
            break;
        }

        case TV_OUTPUT_SVIDEO:
        {
            
            AccessIntegratedTVRegs(pBIOSInfo, DACOutputSelect, ITV_VALUE_OUTPUT_SC, ITV_CMD_WRITE);
            
            /* Turn on DAC R and G, turn off DAC B. */
            VGAOUT8(0x3c5, sr5e | BIT3);
            break;
        }

        case TV_OUTPUT_COMPOSITE_SVIDEO:
        {
            AccessIntegratedTVRegs(pBIOSInfo, DACOutputSelect, ITV_VALUE_OUTPUT_SC, ITV_CMD_WRITE);            
            
            /* Turn on DAC R/G/B. */
            VGAOUT8(0x3c5, sr5e);
            break;
        }
        
        case TV_OUTPUT_RGB:
        {
            AccessIntegratedTVRegs(pBIOSInfo, DACOutputSelect, ITV_VALUE_OUTPUT_RGB, ITV_CMD_WRITE);

            /* Turn on DAC R/G/B. */
            VGAOUT8(0x3c5, sr5e);
            break;
        }
            
        case TV_OUTPUT_YPBPR:
        {            
            AccessIntegratedTVRegs(pBIOSInfo, DACOutputSelect, ITV_VALUE_OUTPUT_YPbPr, ITV_CMD_WRITE);

            /* Turn on DAC R/G/B. */
            VGAOUT8(0x3c5, sr5e);
            break;
        }
            
        default:
        {            
            AccessIntegratedTVRegs(pBIOSInfo, DACOutputSelect, ITV_VALUE_OUTPUT_SC, ITV_CMD_WRITE);

            /* Turn off DAC R and G, turn on DAC B. */
            VGAOUT8(0x3c5, sr5e | (BIT1+BIT2));
            break;
        }
    }
}


static void
WriteGfxCRTCTimingForTV(VIABIOSInfoPtr pBIOSInfo, CARD32  *tv_crtc_table, int IGAPath)
{
    int i;
    struct display_timing  tv_crtc_reg;
    
    for (i=0; i< NUM_CRTC_TIMING; i++)
    {
        switch(i)
        {
            case H_TOTAL_INDEX:
                tv_crtc_reg.hor_total = tv_crtc_table[i];
                DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "H_Total: %d!\n", tv_crtc_reg.hor_total));
                break;
                
            case H_ADDR_INDEX:
                tv_crtc_reg.hor_addr = tv_crtc_table[i];
                DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "H_Addr: %d!\n", tv_crtc_reg.hor_addr));
                break;
                
            case H_BLANK_START_INDEX:
                tv_crtc_reg.hor_blank_start = tv_crtc_table[i];
                DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "H_Blank_Start: %d!\n", tv_crtc_reg.hor_blank_start));
                break;
                
            case H_BLANK_END_INDEX:
                tv_crtc_reg.hor_blank_end = tv_crtc_table[i];
                DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "H_Blank_End: %d!\n", tv_crtc_reg.hor_blank_end));
                break;
                
            case H_SYNC_START_INDEX:
                tv_crtc_reg.hor_sync_start = tv_crtc_table[i];
                DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "H_Sync_Start: %d!\n", tv_crtc_reg.hor_sync_start));
                break;
                
            case H_SYNC_END_INDEX:
                tv_crtc_reg.hor_sync_end = tv_crtc_table[i];
                DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "H_Sync_End: %d!\n", tv_crtc_reg.hor_sync_end));
                break;
                
            case V_TOTAL_INDEX:
                tv_crtc_reg.ver_total = tv_crtc_table[i];
                break;
                
            case V_ADDR_INDEX:
                tv_crtc_reg.ver_addr = tv_crtc_table[i];
                break;
                
            case V_BLANK_START_INDEX:
                tv_crtc_reg.ver_blank_start = tv_crtc_table[i];
                break;
                
            case V_BLANK_END_INDEX:
                tv_crtc_reg.ver_blank_end = tv_crtc_table[i];
                break;
                
            case V_SYNC_START_INDEX:
                tv_crtc_reg.ver_sync_start = tv_crtc_table[i];
                break;
                
            case V_SYNC_END_INDEX:
                tv_crtc_reg.ver_sync_end = tv_crtc_table[i];
                break;
                
        }
    }

    load_crtc_timing(pBIOSInfo, tv_crtc_reg, IGAPath);
}


static void
UpdateScalingFactor(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo)
{
    CARD32 dst_hsize, dst_vsize, src_hsize, src_vsize, scal_factor;
    
    dst_hsize = AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_DST_HSIZE, 0, ITV_CMD_READ) + 1;
    dst_vsize = AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_DST_VSIZE, 0, ITV_CMD_READ) + 1;
    src_hsize = AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_SRC_HSIZE, 0, ITV_CMD_READ) + 1;
    src_vsize = AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_SRC_VSIZE, 0, ITV_CMD_READ) + 1;

    if ((pTVSettingInfo->TVType == TVTYPE_480P) ||
        (pTVSettingInfo->TVType == TVTYPE_576P) ||
        (pTVSettingInfo->TVType == TVTYPE_1080I))
    {
        scal_factor = (src_hsize * 2048) / dst_hsize;
    }
    else if (pTVSettingInfo->TVType == TVTYPE_720P)
    {
        /* RainLin [2006.01.10]: The two modes 720x480 and 720x576 have alignment issue */
        /* (alignment to 4), so the value of SRC_LINE_PITCH, DST_DQW_NUM and DST_LINE_PITCH */
        /* we set are greater than the desired ones, and this may cause the problem about */
        /* having garbage line on the right side, so we reduce the scaling factor to avoid */
        /* this problem. */
        /* Fix Me!! */
        if (((src_hsize == 720) && (src_vsize == 480)) ||
            ((src_hsize == 720) && (src_vsize == 576)))
        {
            scal_factor = ((src_hsize * 2048) / dst_hsize) - 3;
        }
        else
        {
            scal_factor = (src_hsize * 2048) / dst_hsize;
        }
    }
    else
    {
        scal_factor = (src_hsize * 2048) / (dst_hsize * 2);
    }
    
    AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_SCL_HFAC, scal_factor, ITV_CMD_WRITE);

    scal_factor = (src_vsize * 2048) / dst_vsize;
    AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_SCL_VFAC, scal_factor, ITV_CMD_WRITE);
}


static void
SetHDTVPatchTable(VIABIOSInfoPtr pBIOSInfo, int mode_index, TVSETTINGINFOPTR pTVSettingInfo)
{
    switch (pTVSettingInfo->TVType)
    {
        case TVTYPE_480P:
            WriteIntegratedTVPatchTable(pBIOSInfo, ITV_HDTV_PATCH_TABLE[mode_index].std_480p);
            break;
            
        case TVTYPE_576P:
            WriteIntegratedTVPatchTable(pBIOSInfo, ITV_HDTV_PATCH_TABLE[mode_index].std_576p);
            break;
            
        case TVTYPE_720P:
            WriteIntegratedTVPatchTable(pBIOSInfo, ITV_HDTV_PATCH_TABLE[mode_index].std_720p);
            break;
            
        case TVTYPE_1080I:
            WriteIntegratedTVPatchTable(pBIOSInfo, ITV_HDTV_PATCH_TABLE[mode_index].std_1080i);
            break;

        default:
            break;
    }
}


static void
SetTVOutputPatchTable(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo)
{
    if (pTVSettingInfo->TVOutput == TV_OUTPUT_RGB)
    {
        if (pTVSettingInfo->TVType == TVTYPE_NTSC)
        {
            WriteIntegratedTVPatchTable(pBIOSInfo, ITV_PATCH_TABLE_RGB_NTSC);
        }
        else if (pTVSettingInfo->TVType == TVTYPE_PAL)
        {
            WriteIntegratedTVPatchTable(pBIOSInfo, ITV_PATCH_TABLE_RGB_PAL);
        }
    }
    else if (pTVSettingInfo->TVOutput == TV_OUTPUT_YPBPR)
    {
        if (pTVSettingInfo->TVType == TVTYPE_NTSC)
        {
            WriteIntegratedTVPatchTable(pBIOSInfo, ITV_PATCH_TABLE_YPBPR_NTSC);
        }
        else if (pTVSettingInfo->TVType == TVTYPE_PAL)
        {
            WriteIntegratedTVPatchTable(pBIOSInfo, ITV_PATCH_TABLE_YPBPR_PAL);
        }
    }
}


static void
PatchForMacrovision(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo)
{
    CARD32 sr43;
    
    /* RainLin [2006.08.09]: We should do this patch for this issue: 
     * The H-sync cycle of VT3324A3 integrated TV disappeared when disable TV macrovision. */
    sr43 = read_reg(VIASR, SR43);
    
    if ((sr43 & BIT1) == 0) /* Macrovision disable. */
    {
        switch (pTVSettingInfo->TVType)
        {
            case TVTYPE_NTSC:
            case TVTYPE_480P:
                AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_PBPR_BLANK_LEVEL, 0x6D, ITV_CMD_WRITE);
                break;
                
            case TVTYPE_PAL:
            case TVTYPE_576P:
                AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_PBPR_BLANK_LEVEL, 0x71, ITV_CMD_WRITE);
                break;

            case TVTYPE_720P:
            case TVTYPE_1080I:
                AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_PBPR_BLANK_LEVEL, 0x72, ITV_CMD_WRITE);
                break;
        }        
    }
}


void
VIASetIntegratedTVMode(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo)
{
    int i, mode_index;
    CARD32  *tv_mode_table = NULL;
    CARD32  *tv_crtc_table = NULL;

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "SetIntegratedTVMode!\n"));
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "TV Mode = %d.\n", pTVSettingInfo->resTVMode));
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "TV Type = %d\n", pTVSettingInfo->TVType));
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "TV Output = %d\n", pTVSettingInfo->TVOutput));
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "TV IGA = %d\n", pTVSettingInfo->IGAPath));
    
    /* Try to get the suitable mode table. */
    for (i=0; i<NUM_TOTAL_INTEGRATEDTV_MODE_TABLE; i++)
    {
        mode_index = i;

        if (ITV_MODE_TABLE[i].mode_index == pTVSettingInfo->resTVMode)
        {
            break;
        }
    }
    
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO,"TV Mode Index to use is: %d.\n" , mode_index));

    switch (pTVSettingInfo->TVType)
    {
        case TVTYPE_NTSC:
        case TVTYPE_480P:
        {
            tv_mode_table = ITV_MODE_TABLE[mode_index].std_ntsc;
            tv_crtc_table = ITV_CRTC_TABLE[mode_index].std_ntsc;
            break;
        }
            
        case TVTYPE_PAL:
        case TVTYPE_576P:
        {
            tv_mode_table = ITV_MODE_TABLE[mode_index].std_pal;
            tv_crtc_table = ITV_CRTC_TABLE[mode_index].std_pal;
            break;
        }
            
        case TVTYPE_720P:
        {
            tv_mode_table = ITV_MODE_TABLE[mode_index].std_720p;
            tv_crtc_table = ITV_CRTC_TABLE[mode_index].std_720p;
            break;
        }
            
        case TVTYPE_1080I:
        {
            tv_mode_table = ITV_MODE_TABLE[mode_index].std_1080i;
            tv_crtc_table = ITV_CRTC_TABLE[mode_index].std_1080i;
            break;
        }
            
        default:
        {
            tv_mode_table = ITV_MODE_TABLE[mode_index].std_ntsc;
            tv_crtc_table = ITV_CRTC_TABLE[mode_index].std_ntsc;
            break;
        }
    }

    EnableIntegratedTVShadowReg(pBIOSInfo);

    /* Write TV Encoder Regsiters: */

    /* RainLin [2006.08.11]: Since we will use Bypass Mode for 1920x1080, so we'd better */
    /* set initial table for every mode to avoid some of these registers are modified */
    /* by 1920x1080's patch table. */
    WriteIntegratedTVGeneralTable(pBIOSInfo, ITV_INIT_TABLE_OFFSET, ITV_INIT_TABLE_VALUE);

    /* Common settings by mode: */
    WriteIntegratedTVGeneralTable(pBIOSInfo, ITV_MODE_TABLE_OFFSET, tv_mode_table);
    
    /* Write GFX CRTC timing by TV mode. */
    WriteGfxCRTCTimingForTV(pBIOSInfo, tv_crtc_table, pTVSettingInfo->IGAPath);

    SetTVClockSource(pBIOSInfo, pTVSettingInfo);
    
    /* Set TV output signal: */
    if (pBIOSInfo->Chipset == VIA_CX700)    
    {
        /* CX700 only has one DAC. CRT and InTV use the same DAC.*/
        SetIntegratedTVOutputSignal(pBIOSInfo, ITV_DAC_PRIMARY, pTVSettingInfo);
    }
    else if (pBIOSInfo->Chipset == VIA_CN750) 
    {
        /* CN750 has two DAC. CRT use primary DAC and InTV use secondary DAC currently. */
        SetIntegratedTVOutputSignal(pBIOSInfo, ITV_DAC_SECONDARY, pTVSettingInfo);
    }

    /* Patch for HDTV: */
    SetHDTVPatchTable(pBIOSInfo, mode_index, pTVSettingInfo);

    /* RainLin [2005.12.02]: We should modify some settings if use RGB/YPbPr output. */
    SetTVOutputPatchTable(pBIOSInfo, pTVSettingInfo);

    /* S.T.Chen[2006.5.12]: Move this part of the code to here to avoid 
       the patch table overwrites the correct value of source selection. */
    if (pTVSettingInfo->IGAPath == IGA2)
    {
        AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_SRC_SEL, ITV_VALUE_SRC_FROM_IGA2, ITV_CMD_WRITE);
        
        write_reg_mask(CR6A, VIACR, 0, BIT2);

        /* RainLin [2005.11.22]: Modify for sequence issue. */
        EnableSecondDisplayChannel();
    }
    else
    {
        AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_SRC_SEL, ITV_VALUE_SRC_FROM_IGA1, ITV_CMD_WRITE);
        
        /* For the case: TV IGA1 horizontal total can't be divided by 8. */
        PlusVCKToIGA1Timing (tv_crtc_table[H_TOTAL_INDEX]);
    }
    
    /* Update scaling factor: */
    UpdateScalingFactor(pBIOSInfo, pTVSettingInfo);

    /* Patch for macrovision: */
    PatchForMacrovision(pBIOSInfo, pTVSettingInfo);

    DisableIntegratedTVShadowReg(pBIOSInfo);
}


void SaveTVRegs_IntegratedTV(VIABIOSInfoPtr pBIOSInfo, VIARegPtr pSaveRegs)
{
    int i = 0;
    CARD8  offset = 0;
    
    EnableIntegratedTVShadowReg(pBIOSInfo);

    while (offset < ITV_OFFSET_MAX)
    {
        pSaveRegs->IntegratedTVRegs[i] = ReadIntegratedTVReg32(pBIOSInfo->IntegratedTVMapBase, offset);
        i++;
        offset = offset + 4;
    }

    DisableIntegratedTVShadowReg(pBIOSInfo);
}


void RestoreTVRegs_IntegratedTV(VIABIOSInfoPtr pBIOSInfo, VIARegPtr pRestoreRegs)
{
    int i = 0;
    CARD8 offset = 0;
    
    EnableIntegratedTVShadowReg(pBIOSInfo);

    while (offset < ITV_OFFSET_MAX)
    {
        WriteIntegratedTVReg32(pBIOSInfo->IntegratedTVMapBase, offset, 
                               pRestoreRegs->IntegratedTVRegs[i]);
        i++;
        offset = offset + 4;
    }

    DisableIntegratedTVShadowReg(pBIOSInfo);
}


void DoTVSWReset_IntegratedTV(VIABIOSInfoPtr pBIOSInfo)
{
    EnableIntegratedTVShadowReg(pBIOSInfo);

    AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_SD_ENABLE, ITV_VALUE_DISABLE, ITV_CMD_WRITE);
    AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_SW_RESET, ITV_VALUE_ENABLE, ITV_CMD_WRITE);
    AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_SW_RESET, ITV_VALUE_DISABLE, ITV_CMD_WRITE);

    /* RainLin [2005.12.15]: Use VIADelayIn_usec() instead of usleep(), because usleep() */
    /* seems not very stable. */
    VIADelayIn_usec(pBIOSInfo, 50000);  /* We should delay to wait IGA1's VSync. */
    
    AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_SD_ENABLE, ITV_VALUE_ENABLE, ITV_CMD_WRITE);

    DisableIntegratedTVShadowReg(pBIOSInfo);
}


void EnableTV_IntegratedTV(VIABIOSInfoPtr pBIOSInfo, int DACSelect)
{
    VIABIOSInfoPtr  pVia = pBIOSInfo;

    if (DACSelect == ITV_DAC_PRIMARY)
    {
        /* Turn on DPMS. */
        VGAOUT8(0x3d4, CR36);
        VGAOUT8(0x3d5, VGAIN8(0x3d5) & 0xCF);
    }
    
    EnableIntegratedTVShadowReg(pBIOSInfo);

    /* RainLin [2005.12.16]: For the problem TV may crash when we repeat to switch TV type */
    /* (NTSC->PAL->NTSC->PAL->...), to do the following sequence can help avoiding this problem. */
    /* 1. Disable Sync Buffer Mode before set mode and disable TV. */
    /* 2. Enable Sync Buffer Mode before enable TV. */    
    /* Suggested by HW team. */

    /* Enable Sync Buffer Mode: */
    AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_SYNC_BUF_MODE, ITV_VALUE_ENABLE, ITV_CMD_WRITE);
   
    /* Enable PK. */
    AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_PK_ENABLE, ITV_VALUE_ENABLE, ITV_CMD_WRITE);

    /* Enable SD. */
    AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_SD_ENABLE, ITV_VALUE_ENABLE, ITV_CMD_WRITE);

    /* Enable TV encoder. */
    AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_TV_ENABLE, ITV_VALUE_ENABLE, ITV_CMD_WRITE);

    /* Change DAC usage. */
    if (DACSelect == ITV_DAC_SECONDARY)
    {
        AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_SEC_DAC_SEL, ITV_VALUE_SEC_DAC_FOR_TV, ITV_CMD_WRITE);
    }
    else
    {
        AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_PRI_DAC_SEL, ITV_VALUE_DAC_FOR_TV, ITV_CMD_WRITE);
    }
    
    /* Disable SW Reset. */
    AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_SW_RESET, ITV_VALUE_DISABLE, ITV_CMD_WRITE);

    DisableIntegratedTVShadowReg(pBIOSInfo);
}


void DisableTV_IntegratedTV(VIABIOSInfoPtr pBIOSInfo, int DACSelect)
{
    EnableIntegratedTVShadowReg(pBIOSInfo);

    /* RainLin [2005.12.16]: For the problem TV may crash when we repeat to switch TV type */
    /* (NTSC->PAL->NTSC->PAL->...), to do the following sequence can help avoiding this problem. */
    /* 1. Disable Sync Buffer Mode before set mode and disable TV. */
    /* 2. Enable Sync Buffer Mode before enable TV. */    
    /* Suggested by HW team. */

    /* Enable SW Reset. */
    AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_SW_RESET, ITV_VALUE_ENABLE, ITV_CMD_WRITE);
    
    /* Disable Sync Buffer Mode: */
    AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_SYNC_BUF_MODE, ITV_VALUE_DISABLE, ITV_CMD_WRITE);

    /* Disable PK. */
    AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_PK_ENABLE, ITV_VALUE_DISABLE, ITV_CMD_WRITE);

    /* Disable SD. */
    AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_SD_ENABLE, ITV_VALUE_DISABLE, ITV_CMD_WRITE);

    /* Disable TV encoder. */
    AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_TV_ENABLE, ITV_VALUE_DISABLE, ITV_CMD_WRITE);

    /* Change DAC usage. */
    if (DACSelect == ITV_DAC_PRIMARY)
    {
        AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_PRI_DAC_SEL, ITV_VALUE_DAC_FOR_CRT, ITV_CMD_WRITE);
    }    

    DisableIntegratedTVShadowReg(pBIOSInfo);
}


void GetTVInfo_IntegratedTV(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo)
{
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "GetIntegratedTVInfo\n"));
    TVInfoPtr pTVInfo = &(pTVSettingInfo->TVInfo);

    EnableIntegratedTVShadowReg(pBIOSInfo);

    /* RainLin [2005.12.05]: Reduce the vertical level of size and position adjustment to avoid */
    /* incorrect display. */
    /* Size: */
    /* The scaling factor level and default value here are not absolute, they can change by request. */
    pTVInfo->ScalHLevel = 21;
    pTVInfo->ScalVLevel = 15;
    pTVInfo->DefaultScalH = 10;
    pTVInfo->DefaultScalV = 7;

    /* Positoin: */
    /* The position level and default value here are not absolute, they can change by request. */
    pTVInfo->PositionHLevel = 17;
    pTVInfo->PositionVLevel = 13;
    pTVInfo->DefaultPositionH = 9;
    pTVInfo->DefaultPositionV = 7;

    pTVInfo->DefaultFFilter = 0;
    pTVInfo->FFilterLevel = 1;

    pTVInfo->DefaultAFFilter = 0;
    pTVInfo->AFFilterLevel = 1023;
    
    /* Brightness: */
    pTVInfo->DefaultBrightness = AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_BLACK_LEVEL, 0, ITV_CMD_READ);
    pTVInfo->BrightnessLevel = 255;     /* 8 bits. */

    /* Hue: */
    pTVInfo->DefaultTINT = AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_HUE, 0, ITV_CMD_READ);
    pTVInfo->TINTLevel = 2047;       /* 11 bits. */
    
    DisableIntegratedTVShadowReg(pBIOSInfo);
}


void EnableTVFFilter_IntegratedTV(VIABIOSInfoPtr pBIOSInfo, int filter_type)
{
    EnableIntegratedTVShadowReg(pBIOSInfo);
    
    if (filter_type == ITV_FILTER_TYPE_NORMAL)
    {
        AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_DE_FLICKER_ENABLE, ITV_VALUE_ENABLE, ITV_CMD_WRITE);
        AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_DF_TYPE, ITV_FILTER_TYPE_NORMAL, ITV_CMD_WRITE);
        AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_DE_FLK_NUM, ITV_VALUE_FLK_NUM_WHOLE_IMAGE, ITV_CMD_WRITE);
    }
    else if (filter_type == ITV_FILTER_TYPE_ADAPTIVE)
    {
        AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_DE_FLICKER_ENABLE, ITV_VALUE_DISABLE, ITV_CMD_WRITE);
        AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_DE_FLK_NUM, ITV_VALUE_FLK_NUM_ONE_POINT, ITV_CMD_WRITE);
    }
    else
    {
        /* Turn off flicker filter. */
        AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_DE_FLICKER_ENABLE, ITV_VALUE_DISABLE, ITV_CMD_WRITE);

        /* Turn off adaptive flicker filter. */
        AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_DE_FLK_NUM, ITV_VALUE_FLK_NUM_WHOLE_IMAGE, ITV_CMD_WRITE);
    }

    DisableIntegratedTVShadowReg(pBIOSInfo);
}


void AdjustTVFFilter_IntegratedTV(VIABIOSInfoPtr pBIOSInfo, 
                                       TVSETTINGINFOPTR pTVSettingInfo,
                                       int filter_type,
                                       CARD32 desired_value)
{
    TVInfoPtr pTVInfo = &(pTVSettingInfo->TVInfo);

    EnableIntegratedTVShadowReg(pBIOSInfo);
    
    if (filter_type == ITV_FILTER_TYPE_NORMAL)
    {
        if (desired_value == 0xFFFF)
        {
            desired_value = pTVInfo->DefaultFFilter;
        }
        
        AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_DF_WT_SEL, desired_value, ITV_CMD_WRITE);
        
        pTVInfo->CurrentFFilter = desired_value;
    }
    else if(filter_type == ITV_FILTER_TYPE_ADAPTIVE)
    {
        if (desired_value == 0xFFFF)
        {
            desired_value = pTVInfo->DefaultAFFilter;
        }

        AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_DE_FLK_THRESHOLD, desired_value, ITV_CMD_WRITE);
        
        pTVInfo->CurrentAFFilter = desired_value;
    }

    DisableIntegratedTVShadowReg(pBIOSInfo);
}


void AdjustTVHue_IntegratedTV(VIABIOSInfoPtr pBIOSInfo,
                                    TVSETTINGINFOPTR pTVSettingInfo,
                                    CARD32 desired_value)
{
    TVInfoPtr pTVInfo = &(pTVSettingInfo->TVInfo);
    
    EnableIntegratedTVShadowReg(pBIOSInfo);
    AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_HUE, desired_value, ITV_CMD_WRITE);
    pTVInfo->CurrentTINT = desired_value;
    DisableIntegratedTVShadowReg(pBIOSInfo);
}


void AdjustTVBrightness_IntegratedTV(VIABIOSInfoPtr pBIOSInfo, CARD32 desired_value)
{
    EnableIntegratedTVShadowReg(pBIOSInfo);
    AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_BLACK_LEVEL, desired_value, ITV_CMD_WRITE);
    DisableIntegratedTVShadowReg(pBIOSInfo);
}


void AdjustTVPos_IntegratedTV(VIABIOSInfoPtr pBIOSInfo,
                                    TVSETTINGINFOPTR pTVSettingInfo,
                                    CARD32 desired_hpos,
                                    CARD32 desired_vpos)
{
    CARD32 dst_hpos, dst_vpos;
    TVInfoPtr pTVInfo = &(pTVSettingInfo->TVInfo);
    
    EnableIntegratedTVShadowReg(pBIOSInfo);

    dst_hpos = AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_DST_HSD, 0, ITV_CMD_READ);
    dst_vpos = AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_DST_VSD, 0, ITV_CMD_READ);

    dst_hpos = dst_hpos + (desired_hpos - pTVInfo->CurrentPositionH);
    dst_vpos = dst_vpos - (desired_vpos - pTVInfo->CurrentPositionV);

    AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_DST_HSD, dst_hpos, ITV_CMD_WRITE);
    AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_DST_VSD, dst_vpos, ITV_CMD_WRITE);

    pTVInfo->CurrentPositionH = desired_hpos;
    pTVInfo->CurrentPositionV = desired_vpos;

    DisableIntegratedTVShadowReg(pBIOSInfo);
}


void AdjustTVSize_IntegratedTV(VIABIOSInfoPtr pBIOSInfo, 
                                    TVSETTINGINFOPTR pTVSettingInfo,
                                    CARD32 desired_hsize,
                                    CARD32 desired_vsize)
{
    CARD32 dst_hsize, dst_vsize;
    TVInfoPtr pTVInfo = &(pTVSettingInfo->TVInfo);

    /* For the case when press the button "Center" of utility. */
    if (desired_hsize == 0xFFFF || desired_vsize == 0xFFFF)
    {
        desired_hsize = pTVInfo->DefaultScalH;
        desired_vsize = pTVInfo->DefaultScalV;
    }
    else
    {
        desired_hsize = desired_hsize - 1;
        desired_vsize = desired_vsize - 1;
    }
    
    EnableIntegratedTVShadowReg(pBIOSInfo);
    
    dst_hsize = AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_DST_HSIZE, 0, ITV_CMD_READ);
    dst_vsize = AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_DST_VSIZE, 0, ITV_CMD_READ);

    dst_hsize = dst_hsize + pTVSettingInfo->TVHScale * (desired_hsize - pTVInfo->CurrentScalH);
    dst_vsize = dst_vsize + pTVSettingInfo->TVVScale * (desired_vsize - pTVInfo->CurrentScalV);

    AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_DST_HSIZE, dst_hsize, ITV_CMD_WRITE);
    AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_DST_VSIZE, dst_vsize, ITV_CMD_WRITE);

    /* Update scaling factor: */
    UpdateScalingFactor(pBIOSInfo, pTVSettingInfo);
    
    pTVInfo->CurrentScalH = desired_hsize;
    pTVInfo->CurrentScalV = desired_vsize;

    pBIOSInfo->s3utility = TRUE;
    
    DisableIntegratedTVShadowReg(pBIOSInfo);
}


void RestoreUserTVSettings_IntegratedTV(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo)
{
    CARD32 dst_hpos, dst_vpos, dst_hsize, dst_vsize;
    TVInfoPtr pTVInfo = &(pTVSettingInfo->TVInfo);

    /* Restore brightness: */
    AdjustTVBrightness_IntegratedTV(pBIOSInfo, (CARD32)pTVInfo->CurrentBrightness);

    /* Restore hue: */
    AdjustTVHue_IntegratedTV(pBIOSInfo, pTVSettingInfo,(CARD32)pTVInfo->CurrentTINT);                

    /* Restore de-flicker: */
    if (pTVInfo->AdaptiveFFilterOn)
    {
        EnableTVFFilter_IntegratedTV(pBIOSInfo, ITV_FILTER_TYPE_ADAPTIVE);
    }
    else
    {
        EnableTVFFilter_IntegratedTV(pBIOSInfo, ITV_FILTER_TYPE_NORMAL);
    }

    AdjustTVFFilter_IntegratedTV(pBIOSInfo, pTVSettingInfo, ITV_FILTER_TYPE_NORMAL, (CARD32)pTVInfo->CurrentFFilter);
    AdjustTVFFilter_IntegratedTV(pBIOSInfo, pTVSettingInfo, ITV_FILTER_TYPE_ADAPTIVE, (CARD32)pTVInfo->CurrentAFFilter);

    EnableIntegratedTVShadowReg(pBIOSInfo);

    /* Restore position: */
    dst_hpos = AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_DST_HSD, 0, ITV_CMD_READ);
    dst_vpos = AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_DST_VSD, 0, ITV_CMD_READ);

    dst_hpos = dst_hpos + (pTVInfo->CurrentPositionH - pTVInfo->DefaultPositionH);
    dst_vpos = dst_vpos - (pTVInfo->CurrentPositionV - pTVInfo->DefaultPositionV);

    AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_DST_HSD, dst_hpos, ITV_CMD_WRITE);
    AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_DST_VSD, dst_vpos, ITV_CMD_WRITE);

    /* Restore size: */    
    dst_hsize = AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_DST_HSIZE, 0, ITV_CMD_READ);
    dst_vsize = AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_DST_VSIZE, 0, ITV_CMD_READ);

    dst_hsize = dst_hsize + pTVSettingInfo->TVHScale * (pTVInfo->CurrentScalH - pTVInfo->DefaultScalH);
    dst_vsize = dst_vsize + pTVSettingInfo->TVVScale * (pTVInfo->CurrentScalV - pTVInfo->DefaultScalV);

    AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_DST_HSIZE, dst_hsize, ITV_CMD_WRITE);
    AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_DST_VSIZE, dst_vsize, ITV_CMD_WRITE);
    UpdateScalingFactor(pBIOSInfo, pTVSettingInfo);
    
    DisableIntegratedTVShadowReg(pBIOSInfo);
}


int SenseIntegratedTV(VIABIOSInfoPtr pBIOSInfo, int DACSelect, TVSETTINGINFOPTR pTVSettingInfo)
{
    unsigned char cr30, cr37, cr43;
    CARD32 tv3f, sense_status, rgb_status, tv_encoder_status, dac_status, rgb_dac_status;
    CARD32 DACOption, DACRGBSense, DACForTV;

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "Sense Integraed TV Connect Status\n"));
    
    EnableIntegratedTVShadowReg(pBIOSInfo);

    if (pBIOSInfo->FirstTimeToSenseIntegratedTV)
    {
        /* Set TV mode (arbitrary) at least once before sense TV. */
        WriteIntegratedTVGeneralTable(pBIOSInfo, ITV_INIT_TABLE_OFFSET, ITV_INIT_TABLE_VALUE);
        WriteIntegratedTVGeneralTable(pBIOSInfo, ITV_MODE_TABLE_OFFSET, ITV_MODE_TABLE_NTSC_640x480);
        pBIOSInfo->FirstTimeToSenseIntegratedTV = FALSE;
    }

    /* Saved for restore later: */
    cr30 = read_reg(VIACR, CR30);
    cr37 = read_reg(VIACR, CR37);
    cr43 = read_reg(VIACR, CR43);
    rgb_dac_status = read_reg(VIASR, SR5E);
    tv3f = AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_TV_SENSE_DATA, 0, ITV_CMD_READ);
    tv_encoder_status = AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_TV_ENABLE, 0, ITV_CMD_READ);
    dac_status = AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_PRI_DAC_SEL, 0, ITV_CMD_READ);
    
    /* Select primary or secondary DAC */    
    if (DACSelect == ITV_DAC_SECONDARY)
    {
        DACOption = ITV_REG_ID_SEC_DAC_SEL;
        DACRGBSense = ITV_REG_ID_SEC_DAC_TV_SENSE_RGB_STATUS;
        DACForTV = ITV_VALUE_SEC_DAC_FOR_TV;
    }
    else
    {
        DACOption = ITV_REG_ID_PRI_DAC_SEL;
        DACRGBSense = ITV_REG_ID_PRI_DAC_TV_SENSE_RGB_STATUS;
        DACForTV = ITV_VALUE_DAC_FOR_TV;
    }
       
    /* To sense TV, we should: */
    /* 1. Set CR30[4:3]=01, CR37[2:0]=100, CR37[3]=0, CR43[5:4]=01 and TV.3F[7:0]=42h. */
    /* 2. Enable TV DAC R/G/B. */
    /* 3. Enable TV encoder. */
    /* 4. Change primary DAC to TV. */
    write_reg_mask(CR30, VIACR, BIT3, BIT4+BIT3);
    write_reg_mask(CR37, VIACR, BIT2, BIT3+BIT2+BIT1+BIT0);
    write_reg_mask(CR43, VIACR, BIT4, BIT5+BIT4);
    write_reg_mask(SR5E, VIASR, 0, BIT3+BIT2+BIT1);
    AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_TV_SENSE_DATA, 0x42, ITV_CMD_WRITE);
    AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_TV_ENABLE, ITV_VALUE_ENABLE, ITV_CMD_WRITE);    
    AccessIntegratedTVRegs(pBIOSInfo, DACOption, DACForTV, ITV_CMD_WRITE);
        
    VIADelayIn_usec(pBIOSInfo, 5);

    /* Start to sense: */
    AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_TV_SENSE, ITV_VALUE_MANUAL_SENSE, ITV_CMD_WRITE);
    AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_TV_SENSE, ITV_VALUE_NO_SENSE, ITV_CMD_WRITE);

    /* Restore: */    
    AccessIntegratedTVRegs(pBIOSInfo, DACOption, dac_status, ITV_CMD_WRITE);   
    AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_TV_ENABLE, tv_encoder_status, ITV_CMD_WRITE);
    AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_TV_SENSE_DATA, tv3f, ITV_CMD_WRITE);
    write_reg(CR30, VIACR, cr30);
    write_reg(CR37, VIACR, cr37);
    write_reg(CR43, VIACR, cr43);
    write_reg(SR5E, VIASR, rgb_dac_status);

    /* Get sense result: */    
    /*sense_status = AccessIntegratedTVRegs(pBIOSInfo, ITV_REG_ID_TV_SENSE_STATUS, 0, ITV_CMD_READ);*/    
    rgb_status = AccessIntegratedTVRegs(pBIOSInfo, DACRGBSense, 0, ITV_CMD_READ);
    
    /* S.T.Chen[2006.4.20]: In CN750, TVREG_E4[2] can't get correct sense status,
       so I use RGB sense status to determine the connect status.  */
    if (rgb_status)
    {
        sense_status = TRUE;
    }
    
    /* Check connected TV output: */
    if (!pTVSettingInfo->TVOutput)
    {
        switch(rgb_status)
        {
            case ITV_VALUE_DAC_R | ITV_VALUE_DAC_G:
            {
                pTVSettingInfo->TVOutput = TV_OUTPUT_SVIDEO;
                break;
            }

            case ITV_VALUE_DAC_B:
            {
                pTVSettingInfo->TVOutput = TV_OUTPUT_COMPOSITE;
                break;
            }

            case ITV_VALUE_DAC_R | ITV_VALUE_DAC_G | ITV_VALUE_DAC_B:
            {
                pTVSettingInfo->TVOutput = TV_OUTPUT_RGB;
                break;
            }

            default:
            {
                pTVSettingInfo->TVOutput = TV_OUTPUT_COMPOSITE;
                break;
            }
        }
    }

    DisableIntegratedTVShadowReg(pBIOSInfo);
    
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "Integrated TV RGB DAC status is %d.\n", rgb_status));
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "Integrated TV output signal is %d.\n", pTVSettingInfo->TVOutput));    
    
    return sense_status;
}

void InitIntegratedTVRegs(ScrnInfoPtr pScrn)
{
    VIAPtr pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr pBIOSInfo = pVia->pBIOSInfo;
    CARD8  offset = 0;
    
    EnableIntegratedTVShadowReg(pBIOSInfo);

    /* Clear all: */
    while (offset < ITV_OFFSET_MAX)
    {
        WriteIntegratedTVReg32(pBIOSInfo->IntegratedTVMapBase, offset, 0);
        offset = offset + 4;
    }

    /* Initialize: */
    WriteIntegratedTVGeneralTable(pBIOSInfo, ITV_INIT_TABLE_OFFSET, ITV_INIT_TABLE_VALUE);

    DisableIntegratedTVShadowReg(pBIOSInfo);
}

void SetIntegratedTVBufferAddress(ScrnInfoPtr pScrn)
{
    VIAPtr pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr pBIOSInfo = pVia->pBIOSInfo;
    CARD32 mask = 0x1FFFFFF0;  /* [28:4] */
    CARD32 start_address;
    CARD32 buffer_size = 0x80000;
    
    pVia->IntegratedTVBufferStart = pVia->FBFreeEnd - INTEGRATED_TV_BUFFER_SIZE;
    
    /* We should reserve the parts used for integrated TV buffer. */
    pVia->FBFreeEnd = pVia->FBFreeEnd - INTEGRATED_TV_BUFFER_SIZE;

    EnableIntegratedTVShadowReg(pBIOSInfo);
    
    /* Write Source Buffer 0~3 Base Address: */
    start_address = pVia->IntegratedTVBufferStart;
    WriteIntegratedTVReg32Mask(pBIOSInfo->IntegratedTVMapBase, 0x04, mask, start_address);

    start_address = start_address + buffer_size;
    WriteIntegratedTVReg32Mask(pBIOSInfo->IntegratedTVMapBase, 0x08, mask, start_address);

    start_address = start_address + buffer_size;
    WriteIntegratedTVReg32Mask(pBIOSInfo->IntegratedTVMapBase, 0x0C, mask, start_address);

    start_address = start_address + buffer_size;
    WriteIntegratedTVReg32Mask(pBIOSInfo->IntegratedTVMapBase, 0x10, mask, start_address);
    
    /* Write Destination Even and Odd Buffer Base Address: */
    start_address = start_address + buffer_size;
    WriteIntegratedTVReg32Mask(pBIOSInfo->IntegratedTVMapBase, 0x2C, mask, start_address);

    start_address = start_address + buffer_size;
    WriteIntegratedTVReg32Mask(pBIOSInfo->IntegratedTVMapBase, 0x30, mask, start_address);

    DisableIntegratedTVShadowReg(pBIOSInfo);
}

Bool TVEncoderIdentify_IntegratedTV(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo)
{
    /* VT3324 supports intergrated TV. */
    if ((pBIOSInfo->Chipset == VIA_CX700)
     || (pBIOSInfo->Chipset == VIA_CN750))
    {
        DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED, "Found intergrated TV encoder.\n"));
        pTVSettingInfo->TVEncoder = VIA_INTEGRATED_TV;
        
        return TRUE;
    }
    else
    {
        return FALSE;
    }
}

/*MarkTu [2006.08.15]:add to support 3324a3, CX700 don't support SDTV/HDTV*/
BOOL IsIntegratedTVMode(VIABIOSInfoPtr pBIOSInfo, int ModeIndex, TVSETTINGINFOPTR pTVSettingInfo)
{
    BOOL IsTVMode = FALSE;
    if(UNICHROME_VT3324 == pBIOSInfo->dwProjectIDRevision)
    {
         if (pTVSettingInfo->TVType == TVTYPE_480P ||
             pTVSettingInfo->TVType == TVTYPE_720P ||             
             pTVSettingInfo->TVType == TVTYPE_576P ||
             pTVSettingInfo->TVType == TVTYPE_1080I) 
         {
            IsTVMode = FALSE;
            return IsTVMode;
         }
    }
    if (ModeIndex == M640x480  || ModeIndex == M800x600  ||
                ModeIndex == M1024x768 || ModeIndex == M720x480  ||
                ModeIndex == M720x576  || ModeIndex == M1280x720 ||
                ModeIndex == M1920x1080)
    {
        IsTVMode = TRUE;
    }
    return IsTVMode;
}

/*3D DISPLAY SCaling supportting*/
void Reserve3DScalingMemory(ScrnInfoPtr pScrn)
{
    VIAPtr pVia = VIAPTR(pScrn);

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Reserve3DScalingMemory begin\n"));

    pVia->DISP3DScalingBufferStart = pVia->FBFreeEnd - (DISP_3DScaling_BUFFER_SIZE*(EXPAND_BUF_CNT));
    pVia->FBFreeEnd = pVia->FBFreeEnd - (DISP_3DScaling_BUFFER_SIZE*(EXPAND_BUF_CNT)); 

    pVia->Cursor3DScalingBufferStart = pVia->FBFreeEnd - (MAX_CURS * MAX_CURS  * 4);
    pVia->FBFreeEnd = pVia->FBFreeEnd - (MAX_CURS * MAX_CURS  * 4);
    
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Reserve3DScalingMemory exit\n"));

}


static void
VIABlankRectangle( VIAPtr pVia, CARD32 RectAddr, CARD32 RectDimension, CARD32 RectPitch)
{
    CARD32  GeMode;
    VIABIOSInfoPtr pBIOSInfo = pVia->pBIOSInfo;

    switch(pBIOSInfo->bitsPerPixel)
    {
        case 32:
            GeMode = 0x300;
            break;
        case 16:
            GeMode = 0x100;
            break;
        default:
	    /* XXX */
	    GeMode = 0;
            break;            
    }

    WaitIdle();
    VIASETREG(VIA_REG_GEMODE, GeMode);                                          /* 0x04*/
    VIASETREG(VIA_REG_DSTCOLORKEY, 0x00000000);                                 /* 0x18*/
    VIASETREG(VIA_REG_DSTBASE , (VIA_REG_GECMD + RectAddr) >> 3 );
    VIASETREG(VIA_REG_PITCH, ((RectPitch & 0xFFF8)<<13 )|VIA_PITCH_ENABLE);    /* 0x38*/
    VIASETREG(VIA_REG_DSTPOS, 0x0);                                             /* 0x0C*/
    VIASETREG(VIA_REG_DIMENSION, RectDimension);                                /* 0x10*/
    VIASETREG(VIA_REG_GECMD, VIA_GEC_FIXCOLOR_PAT|VIA_GEC_BLT | (0xF0 << 24));  /* 0x00*/
}  /* VIABlankRectangle()*/

static Bool
BlackSclScrnBoundary(VIAPtr pVia, CARD32 RectAddr, RECTL rDest)
{
    VIABIOSInfoPtr pBIOSInfo = pVia->pBIOSInfo;
    VIADISP3DSCALCTRLPtr p3DSCALCTRLInfo = &(pVia->DISP3DSCALCTRLInfo);
    CARD32 ulTopLeftOffset;
    long dwWidth, dwHeigght;
    CARD32 Demension;
    int BytePP;
    long dwShrinkMemWidth, dwShrinkMemHeight;
    CARD32 MemPitch;

    BytePP = pBIOSInfo->bitsPerPixel >> 3;
    dwShrinkMemWidth = p3DSCALCTRLInfo->RealHActive;
    dwShrinkMemHeight = p3DSCALCTRLInfo->RealHActive;

    MemPitch = dwShrinkMemWidth * BytePP;

    /*black left side rectangle  */
    /*we clear a little bit more to avoid align problem*/
    ulTopLeftOffset = 0; 
    dwWidth = rDest.left; 
    dwHeigght = dwShrinkMemHeight;
    Demension = (dwHeigght<<16) | dwWidth;
    if( (dwWidth > 0) && (dwHeigght > 0) )
        VIABlankRectangle( pVia, RectAddr, Demension, MemPitch);
        
    /*black right side rectangle*/
    ulTopLeftOffset = (rDest.right ) * BytePP ;
    dwWidth = dwShrinkMemWidth - rDest.right; 
    dwHeigght = dwShrinkMemHeight;
    Demension = (dwHeigght<<16) | dwWidth;
    if( (dwWidth > 0) && (dwHeigght > 0) )
            VIABlankRectangle( pVia, RectAddr+ulTopLeftOffset, Demension, MemPitch);


    /*black up rectangle*/
    ulTopLeftOffset = 0;
    dwWidth = dwShrinkMemWidth;
    dwHeigght = rDest.top;
    Demension = (dwHeigght<<16) | dwWidth;    
    if( (dwWidth > 0) && (dwHeigght > 0) )
        VIABlankRectangle( pVia, RectAddr+ulTopLeftOffset, Demension, MemPitch);

    /*draw bottom rectangle*/
    ulTopLeftOffset =  (rDest.bottom) * MemPitch;
    dwWidth = dwShrinkMemWidth;
    dwHeigght = dwShrinkMemHeight - rDest.bottom;
    Demension = (dwHeigght<<16) | dwWidth;
    if( (dwWidth > 0) && (dwHeigght > 0) )
        VIABlankRectangle( pVia, RectAddr+ulTopLeftOffset, Demension, MemPitch);

    return TRUE;


}

static Bool
DISP3DScalWindowAdjust(RECTLPtr prSrc, RECTLPtr prDest, VIA3DSclSCNParasPtr pDISP3DScalScreenInfo)
{
    CARD32 dwShrinkMemWidth;
    CARD32 dwShrinkMemHeight;
    CARD32 dwSrcMemWidth, dwSrcMemHeight;
    CARD32 dwDesMemWidth, dwDesMemHeight;

    dwShrinkMemWidth = pDISP3DScalScreenInfo->XSIZEMax;
    dwShrinkMemHeight = pDISP3DScalScreenInfo->YSIZEMax;

    dwSrcMemWidth = prSrc->right - prSrc->left;
    dwSrcMemHeight = prSrc->bottom - prSrc->top;
    
    /*software shrink adjust windows size*/
    prDest->left += pDISP3DScalScreenInfo->XSIZEOffset;
    prDest->top += pDISP3DScalScreenInfo->YSIZEOffset;
    prDest->right -= pDISP3DScalScreenInfo->XSIZEOffset; 
    prDest->bottom -= pDISP3DScalScreenInfo->YSIZEOffset;
  
    prDest->left += pDISP3DScalScreenInfo->XPOSITIONOffset;
    prDest->right += pDISP3DScalScreenInfo->XPOSITIONOffset;
    prDest->top -= pDISP3DScalScreenInfo->YPOSITIONOffset;
    prDest->bottom -= pDISP3DScalScreenInfo->YPOSITIONOffset;

    dwDesMemWidth = prDest->right - prDest->left;
    dwDesMemHeight = prDest->bottom - prDest->top;
  
    if(prDest->left < 0)
    {   
        prSrc->left += -prDest->left * dwSrcMemWidth /dwDesMemWidth;
        prDest->left = 0;
    }
  
    if(prDest->top < 0)
    {
        prSrc->top += -prDest->top * dwSrcMemHeight / dwDesMemHeight;
        prDest->top = 0;
    }
  
    if(prDest->right > dwShrinkMemWidth)
    {
        prSrc->right -= (prDest->right -dwShrinkMemWidth) * dwSrcMemWidth /dwDesMemWidth;
        prDest->right = dwShrinkMemWidth -1;
    }
  
    if(prDest->bottom > dwShrinkMemHeight)
    {
        prSrc->bottom -= (prDest->bottom - dwShrinkMemHeight) * dwSrcMemHeight / dwDesMemHeight;
        prDest->bottom = dwShrinkMemHeight -1;
    }
    return TRUE;
}

static Bool
VIASet3DScalScreenParas(VIABIOSInfoPtr  pBIOSInfo)
{
       VIA3DSclSCNParasPtr pTV3DScalInfo = &(pBIOSInfo->TVSettingInfo.TV3DScalInfo);
       /*Get the  physical size*/
       pTV3DScalInfo->XSIZEMax = pBIOSInfo->TVSettingInfo.HActive;
       pTV3DScalInfo->YSIZEMax = pBIOSInfo->TVSettingInfo.VActive;
       
       /*make adjust SIZE range 50% of physical mode, 25% up and down*/
       pTV3DScalInfo->XSIZEMaxRange= (pBIOSInfo->TVSettingInfo.HActive >> 1);
       pTV3DScalInfo->YSIZEMaxRange= (pBIOSInfo->TVSettingInfo.VActive >> 1);    


       pTV3DScalInfo->XScaleStep = 3;
       pTV3DScalInfo->YScaleStep = 3;

       pTV3DScalInfo->XScaleMaximum = (pTV3DScalInfo->XSIZEMaxRange / pTV3DScalInfo->XScaleStep);
       pTV3DScalInfo->YScaleMaximum = (pTV3DScalInfo->YSIZEMaxRange / pTV3DScalInfo->YScaleStep);
       
       pTV3DScalInfo->XScaleLevel = pTV3DScalInfo->XScaleDefault = pTV3DScalInfo->XScaleMaximum >>1;
       pTV3DScalInfo->YScaleLevel = pTV3DScalInfo->YScaleDefault = pTV3DScalInfo->YScaleMaximum >>1;
       
       pTV3DScalInfo->XSIZEOffset = (pTV3DScalInfo->XScaleDefault - pTV3DScalInfo->XScaleLevel)*pTV3DScalInfo->XScaleStep;
       pTV3DScalInfo->YSIZEOffset = (pTV3DScalInfo->YScaleDefault - pTV3DScalInfo->YScaleLevel)*pTV3DScalInfo->YScaleStep;

       pTV3DScalInfo->XSIZENow = pTV3DScalInfo->XSIZEMax - (pTV3DScalInfo->XScaleLevel*pTV3DScalInfo->XScaleStep);
       pTV3DScalInfo->YSIZENow = pTV3DScalInfo->YSIZEMax - (pTV3DScalInfo->YScaleLevel*pTV3DScalInfo->YScaleStep);

       /*make adjust POSITION range 50% of physical mode, 25% up and down       */
       pTV3DScalInfo->XPOSITIONMaxRange = (pBIOSInfo->TVSettingInfo.HActive >> 1);
       pTV3DScalInfo->YPOSITIONMaxRange = (pBIOSInfo->TVSettingInfo.VActive >> 1);
       
       pTV3DScalInfo->XPOSITIONStep = 3;
       pTV3DScalInfo->YPOSITIONStep = 3;

       pTV3DScalInfo->XPOSITIONMaximum = (pTV3DScalInfo->XPOSITIONMaxRange / pTV3DScalInfo->XPOSITIONStep);
       pTV3DScalInfo->YPOSITIONMaximum = (pTV3DScalInfo->YPOSITIONMaxRange / pTV3DScalInfo->YPOSITIONStep);
       
       pTV3DScalInfo->XPOSITIONLevel = pTV3DScalInfo->XPOSITIONDefault = pTV3DScalInfo->XPOSITIONMaximum >>1;
       pTV3DScalInfo->YPOSITIONLevel = pTV3DScalInfo->YPOSITIONDefault = pTV3DScalInfo->YPOSITIONMaximum >>1;

       pTV3DScalInfo->XPOSITIONOffset= (pTV3DScalInfo->XPOSITIONLevel - pTV3DScalInfo->XPOSITIONDefault)*pTV3DScalInfo->XPOSITIONStep;
       pTV3DScalInfo->YPOSITIONOffset= (pTV3DScalInfo->YPOSITIONLevel - pTV3DScalInfo->YPOSITIONDefault)*pTV3DScalInfo->YPOSITIONStep;

       pTV3DScalInfo->XPOSITIONNow =( pTV3DScalInfo->XPOSITIONLevel - pTV3DScalInfo->XPOSITIONDefault)*pTV3DScalInfo->XPOSITIONStep;
       pTV3DScalInfo->YPOSITIONNow =( pTV3DScalInfo->YPOSITIONLevel - pTV3DScalInfo->YPOSITIONDefault)*pTV3DScalInfo->YPOSITIONStep;

       pTV3DScalInfo->bNeedBlack3dScalingBorders = FALSE;

       if((pBIOSInfo->TVSettingInfo.IsPreSetting)&&(pBIOSInfo->TVSettingInfo.TVInfo.HasUserSetting))
       {
            pTV3DScalInfo->XScaleLevel = pBIOSInfo->TVSettingInfo.TVInfo.XScalingLevel;
            pTV3DScalInfo->YScaleLevel = pBIOSInfo->TVSettingInfo.TVInfo.YScalingLevel;
            pTV3DScalInfo->XSIZEOffset = (pTV3DScalInfo->XScaleDefault - pTV3DScalInfo->XScaleLevel)*pTV3DScalInfo->XScaleStep;
            pTV3DScalInfo->YSIZEOffset = (pTV3DScalInfo->YScaleDefault - pTV3DScalInfo->YScaleLevel)*pTV3DScalInfo->YScaleStep;
            
            pTV3DScalInfo->XPOSITIONLevel = pBIOSInfo->TVSettingInfo.TVInfo.XScalPosLevel;
            pTV3DScalInfo->YPOSITIONLevel = pBIOSInfo->TVSettingInfo.TVInfo.YScalPosLevel;
            pTV3DScalInfo->XPOSITIONOffset= (pTV3DScalInfo->XPOSITIONLevel - pTV3DScalInfo->XPOSITIONDefault)*pTV3DScalInfo->XPOSITIONStep;
            pTV3DScalInfo->YPOSITIONOffset= (pTV3DScalInfo->YPOSITIONLevel - pTV3DScalInfo->YPOSITIONDefault)*pTV3DScalInfo->YPOSITIONStep;
       }
       return TRUE;
       
}

static void
VIASet3DScalCtrlParas(ScrnInfoPtr pScrn)
{
    CARD32  Pitch;
    CARD32  ScalingAddress;
    VIAPtr pVia = VIAPTR(pScrn);    
    VIABIOSInfoPtr pBIOSInfo = pVia->pBIOSInfo;
    VIADISP3DSCALCTRLPtr p3DSCALCTRLInfo = &(pVia->DISP3DSCALCTRLInfo);

    /*Initialize the 3d scaling control for display */
    p3DSCALCTRLInfo->VirtualHActive = pBIOSInfo->displayWidth;
    p3DSCALCTRLInfo->BitsPerPixel = pBIOSInfo->bitsPerPixel;
    p3DSCALCTRLInfo->DISP3DScalIGAPath = pBIOSInfo->TVSettingInfo.IGAPath;

    Pitch = p3DSCALCTRLInfo->RealHActive*(p3DSCALCTRLInfo->BitsPerPixel >> 3);
    Pitch = (Pitch + 31) & ~ 31;
    p3DSCALCTRLInfo->DISP3DScalingBufPitch= Pitch;
    if (pBIOSInfo->IsSecondary) {
	p3DSCALCTRLInfo->OrigFBAddress = pScrn->fbOffset;
    }
    else {
	p3DSCALCTRLInfo->OrigFBAddress = 0;
    }

    p3DSCALCTRLInfo->PanningX = 0;
    p3DSCALCTRLInfo->PanningY = 0;

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO,
		     "pVia->videoRambytes=%d\n",pVia->videoRambytes));

    /* Re-allocate the scaling buffers for 3d scaling */
    ScalingAddress = pVia->DISP3DScalingBufferStart;
    if(ScalingAddress) {
	int i;
	CARD32  Dimension;

	Dimension = (p3DSCALCTRLInfo->RealVActive << 16) |
	    p3DSCALCTRLInfo->RealHActive;
	for (i = 0; i< (EXPAND_BUF_CNT); i++) {
	    p3DSCALCTRLInfo->DISP3DScalingBufAddr[i] =
		ScalingAddress + (Pitch* (p3DSCALCTRLInfo->RealVActive) * i);
	    VIABlankRectangle(pVia, p3DSCALCTRLInfo->DISP3DScalingBufAddr[i],
			      Dimension, Pitch);
	}
	p3DSCALCTRLInfo->DISP3DScalingBufIndex = 0;            
    }

    p3DSCALCTRLInfo->IsNeedAdjustScalScrn = TRUE;
}

void VIADISP3DScalingParasSetting(ScrnInfoPtr pScrn)
{
    CARD32 OffsetWidthByQWord, CountWidthByQWord;
    VIAPtr pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr pBIOSInfo = pVia->pBIOSInfo;
    TVSETTINGINFOPTR  pTVSettingInfo=&(pBIOSInfo->TVSettingInfo);
    VIADISP3DSCALCTRLPtr p3DSCALCTRLInfo = &(pVia->DISP3DSCALCTRLInfo);
    
    if (pBIOSInfo->ActiveDevice & VIA_DEVICE_TV)
    {

        /*1. first to get the physical size, and store the current mode size*/
        /*Replace the TV setting mode to native mode*/
        if ((TVTYPE_NTSC==pBIOSInfo->TVSettingInfo.TVType)|| (TVTYPE_480P==pBIOSInfo->TVSettingInfo.TVType) )
        {
            pBIOSInfo->TVSettingInfo.ModeIndex = M720x480;
            p3DSCALCTRLInfo->OrigHActive = pTVSettingInfo->HActive;
            p3DSCALCTRLInfo->OrigVActive = pTVSettingInfo->VActive;
            p3DSCALCTRLInfo->RealHActive = 720;
            p3DSCALCTRLInfo->RealVActive = 480;            
            pTVSettingInfo->HActive = 720;
            pTVSettingInfo->VActive = 480;
        }
        
        if((TVTYPE_PAL==pBIOSInfo->TVSettingInfo.TVType)|| (TVTYPE_576P==pBIOSInfo->TVSettingInfo.TVType))
        {
            pBIOSInfo->TVSettingInfo.ModeIndex = M720x576;
            p3DSCALCTRLInfo->OrigHActive = pTVSettingInfo->HActive;
            p3DSCALCTRLInfo->OrigVActive = pTVSettingInfo->VActive;
            p3DSCALCTRLInfo->RealHActive = 720;
            p3DSCALCTRLInfo->RealVActive = 576;            
            pTVSettingInfo->HActive = 720;
            pTVSettingInfo->VActive = 576;            
        }

        if (TVTYPE_720P==pBIOSInfo->TVSettingInfo.TVType )
        {
            pBIOSInfo->TVSettingInfo.ModeIndex = M1280x720;
            p3DSCALCTRLInfo->OrigHActive = pTVSettingInfo->HActive;
            p3DSCALCTRLInfo->OrigVActive = pTVSettingInfo->VActive;
            p3DSCALCTRLInfo->RealHActive = 1280;
            p3DSCALCTRLInfo->RealVActive = 720;            
            pTVSettingInfo->HActive = 1280;
            pTVSettingInfo->VActive = 720;            

        }   

        if (TVTYPE_1080I==pBIOSInfo->TVSettingInfo.TVType )
        {
            pBIOSInfo->TVSettingInfo.ModeIndex = M1920x1080;
            p3DSCALCTRLInfo->OrigHActive = pTVSettingInfo->HActive;
            p3DSCALCTRLInfo->OrigVActive = pTVSettingInfo->VActive;
            p3DSCALCTRLInfo->RealHActive = 1920;
            p3DSCALCTRLInfo->RealVActive = 1080;            
            pTVSettingInfo->HActive = 1920;
            pTVSettingInfo->VActive = 1080;            

        }   
       
        /*2.initial the scaling screen paras */
        VIASet3DScalScreenParas(pBIOSInfo);
       
        /*3. initial the scaling control paras*/
        VIASet3DScalCtrlParas(pScrn);
       
        /*4. Update the timing factor to IGA */
        OffsetWidthByQWord = (p3DSCALCTRLInfo->RealHActive * (p3DSCALCTRLInfo->BitsPerPixel >> 3)) >> 3;
        CountWidthByQWord = (p3DSCALCTRLInfo->RealHActive * (p3DSCALCTRLInfo->BitsPerPixel >> 3)) >> 3;
        
        if ((p3DSCALCTRLInfo->DISP3DScalIGAPath) == IGA1)
        {
            pBIOSInfo->IGA1SettingInfo.IsPanning = FALSE;
            pBIOSInfo->IGA1SettingInfo.HActive = p3DSCALCTRLInfo->RealHActive;
            pBIOSInfo->IGA1SettingInfo.VActive = p3DSCALCTRLInfo->RealVActive;
            pBIOSInfo->IGA1SettingInfo.OffsetWidthByQWord = OffsetWidthByQWord;
            pBIOSInfo->IGA1SettingInfo.CountWidthByQWord = CountWidthByQWord;
            pBIOSInfo->IGA1SettingInfo.SetStartingAddrFirstTime = TRUE;
            pBIOSInfo->IGA1SettingInfo.IsActive = TRUE;
        }
        else
        {
            pBIOSInfo->IGA2SettingInfo.IsPanning = FALSE;
            pBIOSInfo->IGA2SettingInfo.HActive = pVia->DISP3DSCALCTRLInfo.RealHActive;
            pBIOSInfo->IGA2SettingInfo.VActive = pVia->DISP3DSCALCTRLInfo.RealVActive;
            pBIOSInfo->IGA2SettingInfo.OffsetWidthByQWord = OffsetWidthByQWord;
            pBIOSInfo->IGA2SettingInfo.CountWidthByQWord = CountWidthByQWord;
            pBIOSInfo->IGA2SettingInfo.SetStartingAddrFirstTime = TRUE;
            pBIOSInfo->IGA2SettingInfo.IsActive = TRUE;
        }

        /*5. Enable the scaling enable*/
        pBIOSInfo->Is3DScalingEnable = TRUE;
    }   

}

void
VIADISP3DScalingImageProcessing(VIAPtr pVia)
{
    VIABIOSInfoPtr pBIOSInfo = pVia->pBIOSInfo;
    VIADISP3DSCALCTRLPtr p3DSCALCTRLInfo = &(pVia->DISP3DSCALCTRLInfo);
    VIA3DSclSCNParasPtr pDISP3DScalScreenInfo = &(pBIOSInfo->TVSettingInfo.TV3DScalInfo);
    int i, bufIndex;
    RECTL rSrc, rDest;
    CARD32 Panningoffset, dwSrcWidth, dwSrcHeight;
    CARD32 lSrcPitch;
    CARD32 SourcScalAddr, DestScalAddr;

    rSrc.left = rSrc.top = 0;
    rSrc.right = pBIOSInfo->CrtcHDisplay;
    rSrc.bottom = pBIOSInfo->CrtcVDisplay;
    dwSrcWidth = pBIOSInfo->CrtcHDisplay;
    dwSrcHeight = pBIOSInfo->CrtcVDisplay;
    /*Note here the pitch maybe is not right when panning, need to adjust them*/
    lSrcPitch = p3DSCALCTRLInfo->VirtualHActive*(p3DSCALCTRLInfo->BitsPerPixel>> 3);

    Panningoffset =( (p3DSCALCTRLInfo->VirtualHActive)*(p3DSCALCTRLInfo->PanningY)+ (p3DSCALCTRLInfo->PanningX))*(p3DSCALCTRLInfo->BitsPerPixel>> 3);
    SourcScalAddr = (p3DSCALCTRLInfo->OrigFBAddress) + Panningoffset;

    /*get the last startaddress*/
    bufIndex = p3DSCALCTRLInfo->DISP3DScalingBufIndex;
    if (bufIndex < 0 || bufIndex > (EXPAND_BUF_CNT-1))
    {
        bufIndex = 0;
    }
    DestScalAddr = p3DSCALCTRLInfo->DISP3DScalingBufAddr[bufIndex]; 

    /*get the current scaling buffer*/    
    p3DSCALCTRLInfo->DISP3DScalingBufIndex = (++p3DSCALCTRLInfo->DISP3DScalingBufIndex)%(EXPAND_BUF_CNT-1);
    bufIndex = p3DSCALCTRLInfo->DISP3DScalingBufIndex;
    
    rDest.left = rDest.top = 0;
    rDest.right = p3DSCALCTRLInfo->RealHActive;
    rDest.bottom = p3DSCALCTRLInfo->RealVActive;


    DISP3DScalWindowAdjust(&rSrc , &rDest, pDISP3DScalScreenInfo);
    if(!pVia->useAgp)
    {
        /*lock hw 3D in PCI path*/
#ifdef XF86DRI        
        viaLOCK_HARDWARE(p3DSCALCTRLInfo); 
#endif  

        hw3d_Scaling_PCI(pVia,p3DSCALCTRLInfo->DISP3DScalingBufAddr[bufIndex],SourcScalAddr,
			 p3DSCALCTRLInfo->DISP3DScalingBufPitch, lSrcPitch,dwSrcHeight,dwSrcWidth, rSrc, rDest,pBIOSInfo->bitsPerPixel,0);   
    
#ifdef XF86DRI
        viaUNLOCK_HARDWARE(p3DSCALCTRLInfo);
#endif        
    }
    else
        hw3d_Scaling_AGP(pVia,p3DSCALCTRLInfo->DISP3DScalingBufAddr[bufIndex],SourcScalAddr,
			 p3DSCALCTRLInfo->DISP3DScalingBufPitch, lSrcPitch,dwSrcHeight,dwSrcWidth, rSrc, rDest,pBIOSInfo->bitsPerPixel,0);   
 
    /*clean the shrink buffer if adjusted the size or position*/
    if(pDISP3DScalScreenInfo->bNeedBlack3dScalingBorders ==  TRUE)
    {
  
       for(i = 0; i < (EXPAND_BUF_CNT-1); i++)
       {
           BlackSclScrnBoundary(pVia, p3DSCALCTRLInfo->DISP3DScalingBufAddr[i], rDest);
       }
     
       pDISP3DScalScreenInfo->bNeedBlack3dScalingBorders = FALSE;
    }

    switch(p3DSCALCTRLInfo->DISP3DScalIGAPath)
    {
        case IGA1:
            VIASETREG(0x214, DestScalAddr>>1);
            break;
        case IGA2:
            VIASETREG(0x218, DestScalAddr);
            break;
    } 
}             
