 /*
 * Copyright 1998-2007 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2007 S3 Graphics, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

 
#include "via_driver.h"
#include "via_bandwidth.h"

void VIADisabledExtendedFIFO(VIABIOSInfoPtr pBIOSInfo)
{
    VIABIOSInfoPtr  pVia = pBIOSInfo;
    CARD32  dwGE230, dwGE298;

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIADisabledExtendedFIFO\n"));
    /* Cause of exit XWindow will dump back register value, others chipset no
     * need to set extended fifo value */
    if (pBIOSInfo->Chipset == VIA_CLE266 && pBIOSInfo->ChipRev < 15 &&
        (pBIOSInfo->HDisplay > 1024 || pBIOSInfo->HasSecondary)) {
        /* Turn off Extend FIFO */
        /* 0x298[29] */
        dwGE298 = VIAGETREG(0x298);
        VIASETREG(0x298, dwGE298 | 0x20000000);
        /* 0x230[21] */
        dwGE230 = VIAGETREG(0x230);
        VIASETREG(0x230, dwGE230 & ~0x00200000);
        /* 0x298[29] */
        dwGE298 = VIAGETREG(0x298);
        VIASETREG(0x298, dwGE298 & ~0x20000000);
    }
}

void VIAEnabledPrimaryExtendedFIFO(VIABIOSInfoPtr pBIOSInfo)
{
    VIABIOSInfoPtr  pVia = pBIOSInfo;
    CARD8   bRegTemp;
    /*CARD16  wRegTemp;*/
    CARD32  dwGE230, dwGE298;

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIAEnabledPrimaryExtendedFIFO\n"));
    switch (pBIOSInfo->Chipset) {
    case VIA_CLE266:
        if (pBIOSInfo->ChipRev > 14) {  /* For 3123Cx */
            if (pBIOSInfo->HasSecondary) {  /* SAMM or DuoView case */
                if (pBIOSInfo->HDisplay >= 1024)
    	        {
    	            /* 3c5.16[0:5] */
        	        VGAOUT8(0x3C4, 0x16);
            	    bRegTemp = VGAIN8(0x3C5);
    	            bRegTemp &= ~0x3F;
        	        bRegTemp |= 0x1C;
            	    VGAOUT8(0x3C5, bRegTemp);
        	        /* 3c5.17[0:6] */
            	    VGAOUT8(0x3C4, 0x17);
                	bRegTemp = VGAIN8(0x3C5);
    	            bRegTemp &= ~0x7F;
        	        bRegTemp |= 0x3F;
            	    VGAOUT8(0x3C5, bRegTemp);
            	    pBIOSInfo->EnableExtendedFIFO = TRUE;
    	        }
            }
            else   /* Single view or Simultaneoue case */
            {
                if (pBIOSInfo->HDisplay > 1024)
    	        {
    	            /* 3c5.16[0:5] */
        	        VGAOUT8(0x3C4, 0x16);
            	    bRegTemp = VGAIN8(0x3C5);
    	            bRegTemp &= ~0x3F;
        	        bRegTemp |= 0x17;
            	    VGAOUT8(0x3C5, bRegTemp);
        	        /* 3c5.17[0:6] */
            	    VGAOUT8(0x3C4, 0x17);
                	bRegTemp = VGAIN8(0x3C5);
    	            bRegTemp &= ~0x7F;
        	        bRegTemp |= 0x2F;
            	    VGAOUT8(0x3C5, bRegTemp);
            	    pBIOSInfo->EnableExtendedFIFO = TRUE;
    	        }
            }
            /* 3c5.18[0:5] */
            VGAOUT8(0x3C4, 0x18);
            bRegTemp = VGAIN8(0x3C5);
            bRegTemp &= ~0x3F;
            bRegTemp |= 0x17;
            bRegTemp |= 0x40;  /* force the preq always higher than treq */
            VGAOUT8(0x3C5, bRegTemp);
        }
        else {      /* for 3123Ax */
            if (pBIOSInfo->HDisplay > 1024 || pBIOSInfo->HasSecondary) {
                /* Turn on Extend FIFO */
                /* 0x298[29] */
                dwGE298 = VIAGETREG(0x298);
                VIASETREG(0x298, dwGE298 | 0x20000000);
                /* 0x230[21] */
                dwGE230 = VIAGETREG(0x230);
                VIASETREG(0x230, dwGE230 | 0x00200000);
                /* 0x298[29] */
                dwGE298 = VIAGETREG(0x298);
                VIASETREG(0x298, dwGE298 & ~0x20000000);

                /* 3c5.16[0:5] */
                VGAOUT8(0x3C4, 0x16);
                bRegTemp = VGAIN8(0x3C5);
                bRegTemp &= ~0x3F;
                bRegTemp |= 0x17;
                /* bRegTemp |= 0x10; */
                VGAOUT8(0x3C5, bRegTemp);
                /* 3c5.17[0:6] */
                VGAOUT8(0x3C4, 0x17);
                bRegTemp = VGAIN8(0x3C5);
                bRegTemp &= ~0x7F;
                bRegTemp |= 0x2F;
                /*bRegTemp |= 0x1F;*/
                VGAOUT8(0x3C5, bRegTemp);
                /* 3c5.18[0:5] */
                VGAOUT8(0x3C4, 0x18);
                bRegTemp = VGAIN8(0x3C5);
                bRegTemp &= ~0x3F;
                bRegTemp |= 0x17;
                bRegTemp |= 0x40;  /* force the preq always higher than treq */
                VGAOUT8(0x3C5, bRegTemp);
          	    pBIOSInfo->EnableExtendedFIFO = TRUE;
            }
        }
        break;
    case VIA_KM400:
        if (pBIOSInfo->HasSecondary) {  /* SAMM or DuoView case */
            if ((pBIOSInfo->HDisplay >= 1600) &&
                (pBIOSInfo->MemClk <= VIA_MEM_DDR200)) {
        	    /* enable CRT extendded FIFO */
            	VGAOUT8(0x3C4, 0x17);
                VGAOUT8(0x3C5, 0x1C);
    	        /* revise second display queue depth and read threshold */
        	    VGAOUT8(0x3C4, 0x16);
            	bRegTemp = VGAIN8(0x3C5);
    	        bRegTemp &= ~0x3F;
    	        bRegTemp = (bRegTemp) | (0x09);
                VGAOUT8(0x3C5, bRegTemp);
            }
            else {
                /* enable CRT extendded FIFO */
                VGAOUT8(0x3C4, 0x17);
                VGAOUT8(0x3C5,0x3F);
                /* revise second display queue depth and read threshold */
                VGAOUT8(0x3C4, 0x16);
                bRegTemp = VGAIN8(0x3C5);
                bRegTemp &= ~0x3F;
                bRegTemp = (bRegTemp) | (0x1C);
                VGAOUT8(0x3C5, bRegTemp);
            }
            /* 3c5.18[0:5] */
            VGAOUT8(0x3C4, 0x18);
            bRegTemp = VGAIN8(0x3C5);
            bRegTemp &= ~0x3F;
            bRegTemp |= 0x17;
            bRegTemp |= 0x40;  /* force the preq always higher than treq */
            VGAOUT8(0x3C5, bRegTemp);
       	    pBIOSInfo->EnableExtendedFIFO = TRUE;
        }
        else {
            if ( (pBIOSInfo->HDisplay > 1024) && (pBIOSInfo->HDisplay <= 1280) )
            {
                /* enable CRT extendded FIFO */
                VGAOUT8(0x3C4, 0x17);
                VGAOUT8(0x3C5, 0x3F);
                /* revise second display queue depth and read threshold */
                VGAOUT8(0x3C4, 0x16);
                bRegTemp = VGAIN8(0x3C5);
                bRegTemp &= ~0x3F;
                bRegTemp = (bRegTemp) | (0x17);
                VGAOUT8(0x3C5, bRegTemp);
           	    pBIOSInfo->EnableExtendedFIFO = TRUE;
            }
            else if ((pBIOSInfo->HDisplay > 1280))
            {
                /* enable CRT extendded FIFO */
                VGAOUT8(0x3C4, 0x17);
                VGAOUT8(0x3C5, 0x3F);
                /* revise second display queue depth and read threshold */
                VGAOUT8(0x3C4, 0x16);
                bRegTemp = VGAIN8(0x3C5);
                bRegTemp &= ~0x3F;
                bRegTemp = (bRegTemp) | (0x1C);
                VGAOUT8(0x3C5, bRegTemp);
           	    pBIOSInfo->EnableExtendedFIFO = TRUE;
            }
            else
            {
                /* enable CRT extendded FIFO */
                VGAOUT8(0x3C4, 0x17);
                VGAOUT8(0x3C5, 0x3F);
                /* revise second display queue depth and read threshold */
                VGAOUT8(0x3C4, 0x16);
                bRegTemp = VGAIN8(0x3C5);
                bRegTemp &= ~0x3F;
                bRegTemp = (bRegTemp) | (0x10);
                VGAOUT8(0x3C5, bRegTemp);
            }
            /* 3c5.18[0:5] */
            VGAOUT8(0x3C4, 0x18);
            bRegTemp = VGAIN8(0x3C5);
            bRegTemp &= ~0x3F;
            bRegTemp |= 0x17;
            bRegTemp |= 0x40;  /* force the preq always higher than treq */
            VGAOUT8(0x3C5, bRegTemp);
        }
        break;
    case VIA_K8M800:
        /*=* R1 Display FIFO depth (384 /8 -1 -> 0xbf) SR17[7:0] (8bits) *=*/
        VGAOUT8(0x3c4, 0x17);
        VGAOUT8(0x3c5, 0xbf);

        /*=* R2 Display fetch datum threshold value (328/4 -> 0x52)
             SR16[5:0], SR16[7] (7bits) *=*/
        VGAOUT8(0x3c4, 0x16);
        bRegTemp = VGAIN8(0x3c5) & ~0xBF;
        bRegTemp |= (0x52 & 0x3F);
        bRegTemp |= ((0x52 & 0x40) << 1);
        VGAOUT8(0x3c5, bRegTemp);

        /*=* R3 Switch to the highest agent threshold value (74 -> 0x4a)
             SR18[5:0], SR18[7] (7bits) *=*/
        VGAOUT8(0x3c4, 0x18);
        bRegTemp = VGAIN8(0x3c5) & ~0xBF;
        bRegTemp |= (0x4a & 0x3F);
        bRegTemp |= ((0x4a & 0x40) << 1);
        VGAOUT8(0x3c5, bRegTemp);
#if 0
        /*=* R4 Fetch Number for a scan line (unit: 8 bytes)
             SR1C[7:0], SR1D[1:0] (10bits) *=*/
        wRegTemp = (pBIOSInfo->offsetWidthByQWord >> 1) + 4;
        VGAOUT8(0x3c4, 0x1c);
        VGAOUT8(0x3c5, (CARD8)(wRegTemp & 0xFF));
        VGAOUT8(0x3c4, 0x1d);
        bRegTemp = VGAIN8(0x3c5) & ~0x03;
        VGAOUT8(0x3c5, bRegTemp | ((wRegTemp & 0x300) >> 8));
#endif
        if (pBIOSInfo->HDisplay >= 1400 && pBIOSInfo->bitsPerPixel == 32)
        {
            /*=* Max. length for a request SR22[4:0] (64/4 -> 0x10) *=*/
            VGAOUT8(0x3c4, 0x22);
            bRegTemp = VGAIN8(0x3c5) & ~0x1F;
            VGAOUT8(0x3c5, bRegTemp | 0x10);
        }
        else
        {
            /*=* Max. length for a request SR22[4:0]
                 (128/4 -> over flow 0x0) *=*/
            VGAOUT8(0x3c4, 0x22);
            bRegTemp = VGAIN8(0x3c5) & ~0x1F;
            VGAOUT8(0x3c5, bRegTemp);
        }
        break;
    case VIA_PM800:
        /*=* R1 Display FIFO depth (96-1 -> 0x5f) SR17[7:0] (8bits) *=*/
        VGAOUT8(0x3c4, 0x17);
        VGAOUT8(0x3c5, 0x5f);

        /*=* R2 Display fetch datum threshold value (32 -> 0x20)
             SR16[5:0], SR16[7] (7bits) *=*/
        VGAOUT8(0x3c4, 0x16);
        bRegTemp = VGAIN8(0x3c5) & ~0xBF;
        bRegTemp |= (0x20 & 0x3F);
        bRegTemp |= ((0x20 & 0x40) << 1);
        VGAOUT8(0x3c5, bRegTemp);

        /*=* R3 Switch to the highest agent threshold value (16 -> 0x10)
             SR18[5:0], SR18[7] (7bits) *=*/
        VGAOUT8(0x3c4, 0x18);
        bRegTemp = VGAIN8(0x3c5) & ~0xBF;
        bRegTemp |= (0x10 & 0x3F);
        bRegTemp |= ((0x10 & 0x40) << 1);
        VGAOUT8(0x3c5, bRegTemp);
#if 0
        /*=* R4 Fetch Number for a scan line (unit: 8 bytes)
             SR1C[7:0], SR1D[1:0] (10bits) *=*/
        wRegTemp = (pBIOSInfo->offsetWidthByQWord >> 1) + 4;
        VGAOUT8(0x3c4, 0x1c);
        VGAOUT8(0x3c5, (CARD8)(wRegTemp & 0xFF));
        VGAOUT8(0x3c4, 0x1d);
        bRegTemp = VGAIN8(0x3c5) & ~0x03;
        VGAOUT8(0x3c5, bRegTemp | ((wRegTemp & 0x300) >> 8));
#endif
        if (pBIOSInfo->HDisplay >= 1400 && pBIOSInfo->bitsPerPixel == 32)
        {
            /*=* Max. length for a request SR22[4:0] (64/4 -> 0x10) *=*/
            VGAOUT8(0x3c4, 0x22);
            bRegTemp = VGAIN8(0x3c5) & ~0x1F;
            VGAOUT8(0x3c5, bRegTemp | 0x10);
        }
        else
        {
            /*=* Max. length for a request SR22[4:0] (0x1F) *=*/
            VGAOUT8(0x3c4, 0x22);
            bRegTemp = VGAIN8(0x3c5) & ~0x1F;
            VGAOUT8(0x3c5, bRegTemp | 0x1F);
        }
        break;
    case VIA_K8M890:
        VGAOUT8(0x3c4, 0x16);
        VGAOUT8(0x3c5, 0x92);
        VGAOUT8(0x3c4, 0x17);
        VGAOUT8(0x3c5, 0xB3);
        VGAOUT8(0x3c4, 0x18);
        VGAOUT8(0x3c5, 0x8A);
        break;
    case VIA_P4M800PRO:
    case VIA_CX700:
        break;
    default:
        break;
    }
}

void VIAEnabledSecondaryExtendedFIFO(VIABIOSInfoPtr pBIOSInfo)
{
    VIABIOSInfoPtr  pVia = pBIOSInfo;
    /*CARD16  wRegTemp;*/
    CARD8   bRegTemp;

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIAEnabledSecondaryExtendedFIFO\n"));
    switch (pBIOSInfo->Chipset) {
    case VIA_CLE266:
        if (pBIOSInfo->ChipRev > 15) {  /* for 3123Cx */
            if (((pBIOSInfo->ActiveDevice & VIA_DEVICE_LCD) &&
                 (pBIOSInfo->LVDSSettingInfo.PanelSizeX >= 1024)) || (pBIOSInfo->HDisplay >= 1024)) {
                /* enable extendded FIFO */
                VGAOUT8(0x3D4, 0x6a);
                bRegTemp = VGAIN8(0x3D5);
                bRegTemp |= 0x20;
                VGAOUT8(0x3D5, bRegTemp);
                /* revise second display queue depth and read threshold */
                VGAOUT8(0x3D4, 0x68);
                VGAOUT8(0x3D5, 0xab);
            }
            else
            {
                /* disable extendded FIFO */
                VGAOUT8(0x3D4, 0x6a);
                bRegTemp = VGAIN8(0x3D5);
                bRegTemp &= ~0x20;
                VGAOUT8(0x3D5, bRegTemp);
                /* revise second display queue depth and read threshold */
                VGAOUT8(0x3D4, 0x68);
                VGAOUT8(0x3D5, 0x67);
            }
        }
        else {      /* for 3123Ax */
            /* TV highest X-Resolution is smaller than 1280,
             * pBIOSInfo->HDisplay >= 1280 don't care. */
            if ((pBIOSInfo->ActiveDevice & VIA_DEVICE_LCD) &&
                (((pBIOSInfo->LVDSSettingInfo.PanelSizeY > 768) && (pBIOSInfo->bitsPerPixel >= 24) &&
                  (pBIOSInfo->MemClk <= VIA_MEM_DDR200)) ||
                 ((pBIOSInfo->LVDSSettingInfo.PanelSizeX > 1280) && (pBIOSInfo->bitsPerPixel >= 24) &&
                  (pBIOSInfo->MemClk <= VIA_MEM_DDR266)))) {
                /* enable extendded FIFO */
                VGAOUT8(0x3D4, 0x6a);
                bRegTemp = VGAIN8(0x3D5);
                bRegTemp |= 0x20;
                VGAOUT8(0x3D5, bRegTemp);
                /* revise second display queue depth and read threshold */
                VGAOUT8(0x3D4, 0x68);
                VGAOUT8(0x3D5, 0xab);
            }
            else {
                /* disable extendded FIFO */
                VGAOUT8(0x3D4, 0x6a);
                bRegTemp = VGAIN8(0x3D5);
                bRegTemp &= ~0x20;
                VGAOUT8(0x3D5, bRegTemp);
                /* revise second display queue depth and read threshold */
                VGAOUT8(0x3D4, 0x68);
                VGAOUT8(0x3D5, 0x67);
            }
   	        pBIOSInfo->EnableExtendedFIFO = TRUE;
        }
        break;
    case VIA_KM400:
        if ((((pBIOSInfo->ActiveDevice & VIA_DEVICE_LCD) &&
              (pBIOSInfo->LVDSSettingInfo.PanelSizeX >= 1600)) || (pBIOSInfo->HDisplay >= 1600)) &&
            (pBIOSInfo->MemClk <= VIA_MEM_DDR200))
        {
    	    /* enable extendded FIFO */
            VGAOUT8(0x3D4, 0x6a);
            bRegTemp = VGAIN8(0x3D5);
            bRegTemp |= 0x20;
            VGAOUT8(0x3D5, bRegTemp);
            /* revise second display queue depth and read threshold */
            VGAOUT8(0x3D4, 0x68);
            VGAOUT8(0x3D5, 0xeb);
        }
        else if (((((pBIOSInfo->ActiveDevice & VIA_DEVICE_LCD) &&
                    (pBIOSInfo->LVDSSettingInfo.PanelSizeX > 1024)) || (pBIOSInfo->HDisplay > 1024)) &&
                  (pBIOSInfo->bitsPerPixel == 32) &&
                  (pBIOSInfo->MemClk <= VIA_MEM_DDR333)) ||
                 ((((pBIOSInfo->ActiveDevice & VIA_DEVICE_LCD) &&
                    (pBIOSInfo->LVDSSettingInfo.PanelSizeX == 1024)) || (pBIOSInfo->HDisplay == 1024)) &&
                  (pBIOSInfo->bitsPerPixel == 32) &&
                  (pBIOSInfo->MemClk <= VIA_MEM_DDR200)))
        {
    	    /* enable extendded FIFO */
            VGAOUT8(0x3D4, 0x6a);
            bRegTemp = VGAIN8(0x3D5);
            bRegTemp |= 0x20;
            VGAOUT8(0x3D5, bRegTemp);
            /* revise second display queue depth and read threshold */
            VGAOUT8(0x3D4, 0x68);
            VGAOUT8(0x3D5, 0xca);
        }
        else if (((((pBIOSInfo->ActiveDevice & VIA_DEVICE_LCD) &&
                    (pBIOSInfo->LVDSSettingInfo.PanelSizeX > 1280)) || (pBIOSInfo->HDisplay > 1280)) &&
                  (pBIOSInfo->bitsPerPixel == 16) &&
                  (pBIOSInfo->MemClk <= VIA_MEM_DDR333)) ||
                 ((((pBIOSInfo->ActiveDevice & VIA_DEVICE_LCD) &&
                    (pBIOSInfo->LVDSSettingInfo.PanelSizeX == 1280)) || (pBIOSInfo->HDisplay == 1280)) &&
                  (pBIOSInfo->bitsPerPixel == 16) &&
                  (pBIOSInfo->MemClk <= VIA_MEM_DDR200)))
        {
    	    /* enable extendded FIFO */
            VGAOUT8(0x3D4, 0x6a);
            bRegTemp = VGAIN8(0x3D5);
            bRegTemp |= 0x20;
            VGAOUT8(0x3D5, bRegTemp);
            /* revise second display queue depth and read threshold */
            VGAOUT8(0x3D4, 0x68);
            VGAOUT8(0x3D5, 0xab);
        }
        else
        {
    	    /* disable extendded FIFO */
        	VGAOUT8(0x3D4, 0x6a);
	        bRegTemp = VGAIN8(0x3D5);
    	    bRegTemp &= ~0x20;
        	VGAOUT8(0x3D5, bRegTemp);
	        /* revise second display queue depth and read threshold */
    	    VGAOUT8(0x3D4, 0x68);
        	VGAOUT8(0x3D5, 0x67);
        }
        break;
    case VIA_K8M800:
        /*=* R1 Display FIFO depth (384 /8 -1 -1)
             CR68[7:4], CR94[7], CR95[7] (6bits) *=*/
        VGAOUT8(0x3d4, 0x68);
        bRegTemp = VGAIN8(0x3d5) & ~0xF0;
        VGAOUT8(0x3d5, bRegTemp | ((0x2e & 0x0F) << 4));
        VGAOUT8(0x3d4, 0x94);
        bRegTemp = VGAIN8(0x3d5) & ~0x80;
        VGAOUT8(0x3d5, bRegTemp | ((0x2e & 0x10) << 3));
        VGAOUT8(0x3d4, 0x95);
        bRegTemp = VGAIN8(0x3d5) & ~0x80;
        VGAOUT8(0x3d5, bRegTemp | ((0x2e & 0x20) << 2));

        /*=* R2 Display fetch datum threshold value (328/4)
             CR68[3:0], CR95[6:4] (7bits) *=*/
        VGAOUT8(0x3d4, 0x68);
        bRegTemp = VGAIN8(0x3d5) & ~0x0F;
        VGAOUT8(0x3d5, bRegTemp | (0x52 & 0x0F));
        VGAOUT8(0x3d4, 0x95);
        bRegTemp = VGAIN8(0x3d5) & ~0x70;
        VGAOUT8(0x3d5, bRegTemp | (0x52 & 0x70));

        /*=* R3 Switch to the highest agent threshold value (74)
             CR92[3:0], CR95[2:0] (7bits) *=*/
        VGAOUT8(0x3d4, 0x92);
        bRegTemp = VGAIN8(0x3d5) & ~0x0F;
        VGAOUT8(0x3d5, bRegTemp | (0x4a & 0x0F));
        VGAOUT8(0x3d4, 0x95);
        bRegTemp = VGAIN8(0x3d5) & ~0x07;
        VGAOUT8(0x3d5, bRegTemp | ((0x4a & 0x70) >> 4));
#if 0
        /*=* R4 Fetch Number for a scan line (unit: 8 bytes)
             CR65[7:0], CR67[3:2] (10bits) *=*/
        wRegTemp = (pBIOSInfo->offsetWidthByQWord >> 1) + 4;
        VGAOUT8(0x3d4, 0x65);
        VGAOUT8(0x3d5, (CARD8)(wRegTemp & 0xFF));
        VGAOUT8(0x3d4, 0x67);
        bRegTemp = VGAIN8(0x3d5) & ~0x0C;
        VGAOUT8(0x3d5, bRegTemp | ((wRegTemp & 0x300) >> 6));
#endif
        if (pBIOSInfo->HDisplay >= 1400 && pBIOSInfo->bitsPerPixel == 32)
        {
            /*=* Max. length for a request CR94[6:0] (64/4) *=*/
            VGAOUT8(0x3d4, 0x94);
            bRegTemp = VGAIN8(0x3d5) & ~0x7F;
            VGAOUT8(0x3d5, bRegTemp | 0x10);
        }
        else
        {
            /*=* Max. length for a request CR94[6:0] (128/4) *=*/
            VGAOUT8(0x3d4, 0x94);
            bRegTemp = VGAIN8(0x3d5) & ~0x7F;
            VGAOUT8(0x3d5, bRegTemp | 0x20);
        }
        break;
    case VIA_PM800:
        /*=* R1 Display FIFO depth (12-1)
             CR68[7:4], CR94[7], CR95[7] (6bits) *=*/
        VGAOUT8(0x3d4, 0x68);
        bRegTemp = VGAIN8(0x3d5) & ~0xF0;
        VGAOUT8(0x3d5, bRegTemp | ((0x0b & 0x0F) << 4));
        VGAOUT8(0x3d4, 0x94);
        bRegTemp = VGAIN8(0x3d5) & ~0x80;
        VGAOUT8(0x3d5, bRegTemp | ((0x0b & 0x10) << 3));
        VGAOUT8(0x3d4, 0x95);
        bRegTemp = VGAIN8(0x3d5) & ~0x80;
        VGAOUT8(0x3d5, bRegTemp | ((0x0b & 0x20) << 2));

        /*=* R2 Display fetch datum threshold value (16)
             CR68[3:0], CR95[6:4] (7bits) *=*/
        VGAOUT8(0x3d4, 0x68);
        bRegTemp = VGAIN8(0x3d5) & ~0x0F;
        VGAOUT8(0x3d5, bRegTemp | (0x10 & 0x0F));
        VGAOUT8(0x3d4, 0x95);
        bRegTemp = VGAIN8(0x3d5) & ~0x70;
        VGAOUT8(0x3d5, bRegTemp | (0x10 & 0x70));

        /*=* R3 Switch to the highest agent threshold value (8)
             CR92[3:0], CR95[2:0] (7bits) *=*/
        VGAOUT8(0x3d4, 0x92);
        bRegTemp = VGAIN8(0x3d5) & ~0x0F;
        VGAOUT8(0x3d5, bRegTemp | (0x08 & 0x0F));
        VGAOUT8(0x3d4, 0x95);
        bRegTemp = VGAIN8(0x3d5) & ~0x07;
        VGAOUT8(0x3d5, bRegTemp | ((0x08 & 0x70) >> 4));
#if 0
        /*=* R4 Fetch Number for a scan line (unit: 8 bytes)
             CR65[7:0], CR67[3:2] (10bits) *=*/
        wRegTemp = (pBIOSInfo->offsetWidthByQWord >> 1) + 4;
        VGAOUT8(0x3d4, 0x65);
        VGAOUT8(0x3d5, (CARD8)(wRegTemp & 0xFF));
        VGAOUT8(0x3d4, 0x67);
        bRegTemp = VGAIN8(0x3d5) & ~0x0C;
        VGAOUT8(0x3d5, bRegTemp | ((wRegTemp & 0x300) >> 6));
#endif
        if (pBIOSInfo->HDisplay >= 1400 && pBIOSInfo->bitsPerPixel == 32)
        {
            /*=* Max. length for a request CR94[6:0] (64/4) *=*/
            VGAOUT8(0x3d4, 0x94);
            bRegTemp = VGAIN8(0x3d5) & ~0x7F;
            VGAOUT8(0x3d5, bRegTemp | 0x10);
        }
        else
        {
            /*=* Max. length for a request CR94[6:0] (128/4) *=*/
            VGAOUT8(0x3d4, 0x94);
            bRegTemp = VGAIN8(0x3d5) & ~0x7F;
            VGAOUT8(0x3d5, bRegTemp | 0x20);
        }
        break;
    case VIA_P4M800PRO:
    case VIA_K8M890:
    case VIA_P4M890:
    case VIA_CX700:
        break;
    default:
        break;
    }
}

void VIAFillExpireNumber(VIABIOSInfoPtr pBIOSInfo)
{
    VIABIOSInfoPtr  pVia = pBIOSInfo;
    CARD8   bRegTemp;
    pVIAPanel3C522Tue    TuneExpireNum;

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIAFillExpireNumber\n"));
    switch (pBIOSInfo->Chipset) {
    case VIA_CLE266:
        if (pBIOSInfo->ChipRev > 14) {
            TuneExpireNum = Panel_Tuning_LstC0;
        }
        else {
            TuneExpireNum = Panel_Tuning_Lst;
        }
        break;
    case VIA_KM400:
    /*case VIA_K8M800:
    case VIA_PM800:*/
        TuneExpireNum = Panel_Tuning_Lst3205;
        break;
    default:
        return;
        break;
    }
    while (TuneExpireNum->wPanelXres != 0) {
        if (TuneExpireNum->wPanelXres == pBIOSInfo->LVDSSettingInfo.PanelSizeX &&
            TuneExpireNum->wPanelYres == pBIOSInfo->LVDSSettingInfo.PanelSizeY &&
            TuneExpireNum->wPanelBpp == pBIOSInfo->bitsPerPixel &&
            TuneExpireNum->bRamClock == pBIOSInfo->MemClk) {
            VGAOUT8(0x3C4, 0x22);
            bRegTemp = VGAIN8(0x3C5) & ~0x1F;
            VGAOUT8(0x3C5, (bRegTemp | TuneExpireNum->bTuningValue));
        }
        TuneExpireNum ++;
    }
}
