/*
 * Copyright 1998-2007 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2007 S3 Graphics, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

/*************************************************************************
 *
 *  File:       via_bios.c
 *  Content:    Get the mode table from VGA BIOS and set mode by this table
 *
 ************************************************************************/

/*
#define CREATE_TV2_HEADERFILE
*/

#include "via_driver.h"
#include "via_hw.h"
#include "via_tv.h"
#include "via_mode.h"

#include "via_mode_204.h"

#include "via_refresh.h"
#include "via_rotate.h"
#include "via_vt1625.h"

/* Functions Prototype Declaration */
struct display_timing VIAGetTimingValue(VIABIOSInfoPtr pBIOSInfo, struct crt_mode_table *crt_table);

/* Extern Functions Prototypes Declaration */
extern void VIALCDSetMode(VIABIOSInfoPtr pBIOSInfo, LVDSSETTINGINFOPTR pLVDSSettingInfo);
extern int SearchModeSetting(int ModeInfoIndex);
extern void write_regx(struct io_reg RegTable[],int ItemNum);
extern void unlock_crt(void);
extern void fill_crtc_timing(VIABIOSInfoPtr pBIOSInfo, struct crt_mode_table *crt_table, int mode_array, int bpp_byte, int set_iga);
extern void VIAEnableLCD(VIABIOSInfoPtr pBIOSInfo, LVDSSETTINGINFOPTR pLVDSSettingInfo);
extern void VIADisableLCD(VIABIOSInfoPtr pBIOSInfo, LVDSSETTINGINFOPTR pLVDSSettingInfo);
extern void VIASetLCDOutputPath(VIABIOSInfoPtr pBIOSInfo, int set_iga, int output_interface);
extern unsigned char VIAGetPanelInfo(VIABIOSInfoPtr pBIOSInfo);
extern Bool VIASetDFPMode(VIABIOSInfoPtr pBIOSInfo);
extern void VIAEnableDFP(VIABIOSInfoPtr pBIOSInfo);
extern void VIADisableDFP(VIABIOSInfoPtr pBIOSInfo);
extern void VIADVISetMode(VIABIOSInfoPtr pBIOSInfo);
extern void VIASetDVIOutputPath(VIABIOSInfoPtr pBIOSInfo, int set_iga, int output_interface);
extern int VIAGetTVTabVer(VIABIOSInfoPtr pBIOSInfo, unsigned char *pBIOS);
extern void VIADisableTV(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo);
extern void VIATVSetMode(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo);
extern void VIASetTVOutputPath(VIABIOSInfoPtr pBIOSInfo, int set_iga, int output_interface);
extern unsigned char VIAGetDeviceDetectTV(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo);
extern unsigned char VIAGetDeviceDetectLCD(VIABIOSInfoPtr pBIOSInfo);
extern unsigned char VIAGetDeviceDetectDVI(VIABIOSInfoPtr pBIOSInfo);
extern void load_offset_reg(int h_addr, int bpp_byte, int set_iga);


static void VIAInitSettingInfo_IGA(VIABIOSInfoPtr pBIOSInfo, int DisplayPath, int IsPanning, int HActive, int VActive);
static void VIAAdjustActiveSize_LCD(VIABIOSInfoPtr pBIOSInfo, LVDSSETTINGINFOPTR pLVDSSettingInfo, int *pHActive, int *pVActive);
static void VIAAdjustActiveSize_TV(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo, int* pHActive, int* pVActive);
static void VIAAdjustActiveSize(VIABIOSInfoPtr pBIOSInfo);
static void VIAPrimaryDisplayOffsetPatch(VIABIOSInfoPtr pBIOSInfo);
static void VIAInitSettingInfo_CRT(VIABIOSInfoPtr pBIOSInfo);
static void VIAInitSettingInfo_TV(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo);
static void VIAInitSettingInfo_DVI(VIABIOSInfoPtr pBIOSInfo);
static void VIAInitSettingInfo_LCD(VIABIOSInfoPtr pBIOSInfo, LVDSSETTINGINFOPTR pLVDSSettingInfo);
static void VIASecondDisplayOffsetPatch(VIABIOSInfoPtr pBIOSInfo);
static void xwrite_regx(struct io_reg *ioreg);

/* Support Mode, the order must follow the rule "small to big". */
struct _VIASUPPORTMODE VIASupportMode[] = {
/*  Index,         HActive, VActive*/
    {"640x480",     M640x480,     640,     480},
    {"720x480",     M720x480,     720,     480},
    {"720x576",     M720x576,     720,     576},
    {"800x480",     M800x480,     800,     480},
    {"800x600",     M800x600,     800,     600},
    {"848x480",     M848x480,     848,     480},
    {"852x480",     M852x480,     856,     480},
    {"960x600",     M960x600,     960,     600}, 
    {"1000x600",    M1000x600,    1000,    600}, 
    {"1088x612",    M1088x612,    1088,    612}, 
    {"1024x512",    M1024x512,    1024,    512},  
    {"1024x600",    M1024x600,    1024,    600},     
    {"1024x576",    M1024x576,    1024,    576},        
    {"1024x768",    M1024x768,    1024,    768},
    {"1152x720",    M1152x720,    1152,    720},
    {"1152x864",    M1152x864,    1152,    864},
    {"1200x720",    M1200x720,    1200,    720},
    {"1280x600",    M1280x600,    1280,    600},
    {"1280x720",    M1280x720,    1280,    720},
    {"1280x768",    M1280x768,    1280,    768},
    {"1280x800",    M1280x800,    1280,    800},
    {"1280x960",    M1280x960,    1280,    960},
    {"1280x1024",   M1280x1024,   1280,    1024},
    {"1360x768",    M1360x768,    1360,    768},
    {"1366x768",    M1366x768,    1366,    768},
    {"1400x1050",   M1400x1050,   1400,    1050},
    {"1440x900",    M1440x900,    1440,    900},
    {"1440x1050",   M1440x1050,   1440,    1050},
    {"1600x900",    M1600x900,    1600,    900},
    {"1600x1024",   M1600x1024,   1600,    1024},
    {"1600x1200",   M1600x1200,   1600,    1200},
    {"1680x1050",   M1680x1050,   1680,    1050},
    {"1792x1344",   M1792x1344,   1792,    1344},
    {"1856x1392",   M1856x1392,   1856,    1392},
    {"1920x1080",   M1920x1080,   1920,    1080},
    {"1920x1200",   M1920x1200,   1920,    1200},
    {"1920x1440",   M1920x1440,   1920,    1440},   
    {"2048x1536",   M2048x1536,   2048,    1536},   
    {"",            -1,    0,       0}
};

/*=*
 *
 * char* VIAGetBIOSInfo(VIABIOSInfoPtr) - Get BIOS Release Date
 *
 * Return Type:    string pointer
 *
 * The Definition of Input Value:
 *
 *                 VIABIOSInfoPtr
 *
 * The Definition of Return Value:
 *
 *                 BIOS release date string pointer
 *=*/

char* VIAGetBIOSInfo(VIABIOSInfoPtr pBIOSInfo)
{
    return pBIOSInfo->pModeTable->BIOSDate;
}



/*=*
 *
 * unsigned char VIAGetActiveDisplay(VIABIOSInfoPtr, unsigned char)
 *
 *     - Get Active Display Device
 *
 * Return Type:    unsigned char
 *
 * The Definition of Input Value:
 *
 *                 VIABIOSInfoPtr
 *
 * The Definition of Return Value:
 *
 *                 Bit[7] 2nd Path
 *                 Bit[6] 1/0 MHS Enable/Disable
 *                 Bit[5] 0 = Bypass Callback, 1 = Enable Callback
 *                 Bit[4] 0 = Hot-Key Sequence Control (OEM Specific)
 *                 Bit[3] LCD
 *                 Bit[2] TV
 *                 Bit[1] CRT
 *                 Bit[0] DVI
 *=*/

unsigned char VIAGetActiveDisplay(VIABIOSInfoPtr pBIOSInfo)
{
    VIABIOSInfoPtr  pVia;
    unsigned char tmp;

    pVia = pBIOSInfo;

    tmp = (read_reg(VIACR, CR3E) & 0xF0) >> 4;

    tmp |= (read_reg(VIACR, CR3B) & 0x18) << 3;

    return tmp;
}

/*=*
 *
 * unsigned char VIAGetDeviceDetect(VIABIOSInfoPtr)
 *
 *     - Get Display Device Attched
 *
 * Return Type:    unsigned char
 *
 * The Definition of Input Value:
 *
 *                 VIABIOSInfoPtr
 *
 * The Definition of Return Value:
 *
 *                 Bit[7] Reserved ------------ 2nd TV Connector
 *                 Bit[6] Reserved ------------ 1st TV Connector
 *                 Bit[5] Reserved
 *                 Bit[4] CRT2
 *                 Bit[3] DFP
 *                 Bit[2] TV
 *                 Bit[1] LCD
 *                 Bit[0] CRT
 *=*/

unsigned char VIAGetDeviceDetect(VIABIOSInfoPtr pBIOSInfo)
{
    unsigned char   ret;

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIAGetDeviceDetect\n"));

    ret = VIA_DEVICE_CRT1; /* Default assume color CRT attached */

    ret |= VIAGetDeviceDetectTV(pBIOSInfo, &(pBIOSInfo->TVSettingInfo));
    ret |= VIAGetDeviceDetectLCD(pBIOSInfo);
    ret |= VIAGetDeviceDetectDVI(pBIOSInfo);

    if (pBIOSInfo->TVSettingInfo2.TVEncoder)
    {
        if(VIAGetDeviceDetectTV(pBIOSInfo, &(pBIOSInfo->TVSettingInfo2)))
        {
            ret |= VIA_DEVICE_TV2;
        }
    }
    
    return ret;
}

Bool VIAGetBIOSTable(VIABIOSInfoPtr pBIOSInfo)
{
    VIAModeTablePtr pViaModeTable = pBIOSInfo->pModeTable;
    unsigned char   *pBIOS = NULL, *pRom, *pTable;
    int             romSize;
    int             i = 0, sum;


    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIAGetBIOSTable\n"));

    if (!(pBIOS=xcalloc(1, 0x10000)))  {
        ErrorF("Allocate memory fail !!\n");
        return FALSE;
    }

    if (xf86ReadBIOS(0xC0000, 0, pBIOS, 0x10000) != 0x10000)  {
        pBIOSInfo->UseBIOS = FALSE;
        xfree(pBIOS);
        ErrorF("Read VGA BIOS image fail !!\n");
    }
    else {
        if (*((CARD16 *) pBIOS) != 0xAA55) {
            pBIOSInfo->UseBIOS = FALSE;
            xfree(pBIOS);
            ErrorF("VGA BIOS image is wrong!!\n");
        }
        else {
            romSize = *((CARD8 *) (pBIOS + VIA_BIOS_SIZE_POS)) * 512;
            pRom = pBIOS;
            sum = 0;

            for (i = 0; i < romSize; i++) {
                sum += *pRom++;
            }

            if (((CARD8) sum) != 0) {
                pBIOSInfo->UseBIOS = FALSE;
                xfree(pBIOS);
                ErrorF("VGA BIOS image is wrong!! CheckSum = %x\n", sum);
            }
        }
    }

    /* To check is TV Encoder match with BIOS TV mode table */
    if (pBIOSInfo->UseBIOS) {
        int TVEncoder = pBIOSInfo->TVSettingInfo.TVEncoder;
        if (TVEncoder && (VIAGetTVTabVer(pBIOSInfo, pBIOS) != (TVEncoder+1))) {
            pBIOSInfo->UseBIOS = FALSE;
            xfree(pBIOS);
        }
    }

    /* Get the start of Table */
    pRom = pBIOS + VIA_BIOS_OFFSET_POS;
    pTable = pBIOS + *((CARD16 *)pRom);


#ifdef CREATE_TV2_HEADERFILE
    VIAGetTV2Mask(pViaModeTable, pBIOS, pTable);
    VIAGetTV2NTSC(pViaModeTable, pBIOS, pTable);
    VIAGetTV2PAL(pViaModeTable, pBIOS, pTable);
    VIACreateTV2(pViaModeTable);
#else
    pViaModeTable->tv2MaskTable = tv2MaskTable;
    for (i = 0; i < VIA_BIOS_NUM_TV2; i++) {
        pViaModeTable->tv2Table[i] = tv2Table[i];
        pViaModeTable->tv2OverTable[i] = tv2OverTable[i];
    }
#endif

    if (!pBIOSInfo->UseBIOS) 
    {
        if (pBIOSInfo->Chipset >= VIA_K8M800)
        {
            /* For K8M800 or later chipsets: */
            pViaModeTable->BIOSVer = BIOSVer_204;
            pViaModeTable->BIOSDate = BIOSDate_204;
            pViaModeTable->NumModes = NumModes_204;
            pViaModeTable->NumPowerOn = NumPowerOn;
            pViaModeTable->NumPowerOff = NumPowerOff;
            pViaModeTable->Modes = Modes_204;
            pViaModeTable->commExtTable = commExtTable;
            pViaModeTable->stdModeExtTable = stdModeExtTable;
            
            for (i = 0; i < VIA_BIOS_NUM_LCD_POWER_SEQ; i++)
            {
                pViaModeTable->powerOn[i] = powerOn[i];
                pViaModeTable->powerOff[i] = powerOff[i];
            }
            
            pViaModeTable->modeFix = modeFix;
            DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "Get mode table from via_mode_204.h\n"));
        }
        else
        {            
            pViaModeTable->BIOSVer = BIOSVer;
            pViaModeTable->BIOSDate = BIOSDate;
            pViaModeTable->NumModes = NumModes;
            pViaModeTable->NumPowerOn = NumPowerOn;
            pViaModeTable->NumPowerOff = NumPowerOff;
            pViaModeTable->Modes = Modes;
            pViaModeTable->commExtTable = commExtTable;
            pViaModeTable->stdModeExtTable = stdModeExtTable;
            
            for (i = 0; i < VIA_BIOS_NUM_LCD_POWER_SEQ; i++)
            {
                pViaModeTable->powerOn[i] = powerOn[i];
                pViaModeTable->powerOff[i] = powerOff[i];
            }
            
            pViaModeTable->modeFix = modeFix;
            DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "Get mode table from via_mode.h\n"));
        }

        DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "BIOS Version: %x.%x\n",
                (pViaModeTable->BIOSVer >> 8), (pViaModeTable->BIOSVer & 0xFF)));
        DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "BIOS Release Date: %s\n", pViaModeTable->BIOSDate));
        DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "-- VIAGetBIOSTable Done!\n"));
        xfree(pBIOS);
        return TRUE;
    }

    return TRUE;	/* XXX control reaches end of non-void function */
}

Bool VIAFindRefreshRate(VIABIOSInfoPtr pBIOSInfo, int ModeIndex)
{
    struct VideoModeTable   *VideoMode;
    int     i;
    int     needRefresh = pBIOSInfo->Refresh;
    
    VideoMode = &CLE266Modes[SearchModeSetting(ModeIndex)];

    for (i = 0; i < VideoMode->mode_array; i++)
    {
        pBIOSInfo->FoundRefresh = VideoMode->crtc[i].refresh_rate;
        
        if (VideoMode->crtc[i].refresh_rate == needRefresh)
        {            
            return TRUE;
        }
    }

    xf86DrvMsg(pBIOSInfo->scrnIndex, X_ERROR, "Can't find refresh rate %d Hz.\n", needRefresh);

    return FALSE;
}

/* RainLin [2006.06.20]: Marked by RainLin, just use mode table we have to check supported */
/* refresh rate to simplify the flow. Above is the new function.  */
#if 0
int VIAFindSupportRefreshRate(VIABIOSInfoPtr pBIOSInfo, int resIndex)
{
    int             bppIndex, refIndex;
    int             needRefresh;
    int             *supRefTab;

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIAFindSupportRefreshRate\n"));
    bppIndex = 0;
    supRefTab = NULL;
    needRefresh = pBIOSInfo->Refresh;

    /* find min close up support refresh rate */
    if (needRefresh <= supportRef[0])
    {
        refIndex = 0;
    }
    else if (needRefresh >= supportRef[VIA_NUM_REFRESH_RATE - 1])
    {
        refIndex = VIA_NUM_REFRESH_RATE - 1;
    }
    else
    {
        for (refIndex = 0; refIndex < VIA_NUM_REFRESH_RATE; refIndex++)
        {
            if (needRefresh < supportRef[refIndex + 1])
            {
                break;
            }
        }
    }

    switch (pBIOSInfo->bitsPerPixel)
    {
        case 8:
            bppIndex = 0;
            break;
        case 16:
            bppIndex = 1;
            break;
        case 24:
        case 32:
            bppIndex = 2;
            break;
    }

    switch (pBIOSInfo->MemClk)
    {
        case VIA_MEM_SDR66:
        case VIA_MEM_SDR100:
            supRefTab = SDR100[bppIndex][resIndex];
            break;
        case VIA_MEM_SDR133:
            supRefTab = SDR133[bppIndex][resIndex];
            break;
        case VIA_MEM_DDR200:
            supRefTab = DDR200[bppIndex][resIndex];
            break;
        case VIA_MEM_DDR266:
        case VIA_MEM_DDR333:
        case VIA_MEM_DDR400:
            supRefTab = DDR266[bppIndex][resIndex];
            break;
    }

    for ( ; refIndex >= 0; refIndex--)
    {
        if (supRefTab[refIndex])
        {
            needRefresh = supportRef[refIndex];
            break;
        }
    }

    pBIOSInfo->FoundRefresh = needRefresh;
    return refIndex;
}
#endif


void VIAGetPanelSizeByIndex(int PanelSizeIndex, int *pSizeX, int *pSizeY)
{
    switch (PanelSizeIndex)
    {
        case VIA_PANEL6X4:
            *pSizeX = 640;
            *pSizeY = 480;
            break;
            
        case VIA_PANEL8X4:
            *pSizeX = 800;
            *pSizeY = 480;
            break;
            
        case VIA_PANEL8X6:
            *pSizeX = 800;
            *pSizeY = 600;
            break;
            
       case VIA_PANEL10X6:
            *pSizeX = 1024;
            *pSizeY = 600;
            break;
            
        case VIA_PANEL10X7:
            *pSizeX = 1024;
            *pSizeY = 768;
            break;

        case VIA_PANEL1280X720:
            *pSizeX = 1280;
            *pSizeY = 720;
            break;             
            
        case VIA_PANEL12X7:
            *pSizeX = 1280;
            *pSizeY = 768;
            break;
            
        case VIA_PANEL12X8:
            *pSizeX = 1280;
            *pSizeY = 800;
            break;
            
        case VIA_PANEL12X10:
            *pSizeX = 1280;
            *pSizeY = 1024;
            break;
            
        case VIA_PANEL14X10:
            *pSizeX = 1400;
            *pSizeY = 1050;
            break;

        case VIA_PANEL14X9:
            *pSizeX = 1440;
            *pSizeY = 900;
            break;
            
        case VIA_PANEL16X12:
            *pSizeX = 1600;
            *pSizeY = 1200;
            break;
            
        case VIA_PANEL1366X7:
            *pSizeX = 1366;
            *pSizeY = 768;
            break;
            
        case VIA_PANEL1360X7:
            *pSizeX = 1360;
            *pSizeY = 768;
            break;

        case VIA_PANEL1920x1080:
            *pSizeX = 1920;
            *pSizeY = 1080;
            break;
            
        case VIA_PANEL1920x1200:
            *pSizeX = 1920;
            *pSizeY = 1200;
            break;
                    
        default:
            *pSizeX = 0;
            *pSizeY = 0;
            ErrorF("Unknown Panel Size!!\n");
            break;
    }    
}


int
VIAGetModeIndex(int HorResolution, int VerResolution)
{
    int ModeIndex = M1024x768;
    int i = 0;    
    
    while (VIASupportMode[i].ModeIndex > -1)
    {        
        if ((HorResolution == VIASupportMode[i].HActive) &&
            (VerResolution == VIASupportMode[i].VActive))
        {
            ModeIndex = VIASupportMode[i].ModeIndex;
            break;
        }

        i++;
    }    
    
    return ModeIndex;
}


void VIAGetModeSizeByName(char *pName, int *pHSize, int *pVSize)
{
    int i = 0;

    if (!pName)
    {
        return;
    }

    while (VIASupportMode[i].ModeIndex > -1)
    {
        if (strcmp(pName, VIASupportMode[i].Name) == 0)
        {
            *pHSize = VIASupportMode[i].HActive;
            *pVSize = VIASupportMode[i].VActive;
            break;
        }

        i++;
    }
}

/*
*   STChen[2007.4.12]: Get mode size by mode index.
*/
void VIAGetModeSizeByIndex(int ModeIndex, int *pHSize, int *pVSize)
{
    int i = 0;
    
    while (VIASupportMode[i].ModeIndex > -1)
    {
        if (VIASupportMode[i].ModeIndex == ModeIndex)
        {
            *pHSize = VIASupportMode[i].HActive;
            *pVSize = VIASupportMode[i].VActive;
            break;
        }

        i++;
    }
}


void VIAGetPanelIndexByModeSize(CARD32 HRes, CARD32 VRes, CARD32* pPanelIndex)
{    
    /* Find Resolution index */
    switch (HRes)
    {
        case 640:
            if (VRes == 480)
            {
                *pPanelIndex = VIA_PANEL6X4;
            }
            break;
            
        case 800:
            if (VRes == 600)
            {
                *pPanelIndex = VIA_PANEL8X6;
            }
            else if (VRes == 480)
            {
                *pPanelIndex = VIA_PANEL8X4;
            }
            break;
            
        case 1024:
            if (VRes == 768)
            {
                *pPanelIndex = VIA_PANEL10X7;
            }
            else if (VRes == 600)
            {
                *pPanelIndex = VIA_PANEL10X6;
            }
            break;
            
        case 1280:
            if (VRes == 720)
            {
                *pPanelIndex = VIA_PANEL1280X720;
            }
            else if (VRes == 768)
            {
                *pPanelIndex = VIA_PANEL12X7;
            }
            else if (VRes == 800)
            {
                *pPanelIndex = VIA_PANEL12X8;
            }
            else if (VRes == 1024)
            {
                *pPanelIndex = VIA_PANEL12X10;
            }            
            break;
            
        case 1360:
            if (VRes == 768)
            {
                *pPanelIndex = VIA_PANEL1360X7;
            }
            break;  

        case 1366:
            if (VRes == 768)
            {
                *pPanelIndex = VIA_PANEL1366X7;
            }
            break;
            
        case 1400:
            if (VRes == 1050)
            {
                *pPanelIndex = VIA_PANEL14X10;
            }
            break;
            
        case 1440:
            if (VRes == 900)
            {
                *pPanelIndex = VIA_PANEL14X9;
            }
            break; 
            
        case 1600:
            if (VRes == 1200)
            {
                *pPanelIndex = VIA_PANEL16X12;
            }
            break;                    

        case 1920:
            if (VRes == 1080)
            {
                *pPanelIndex = VIA_PANEL1920x1080;
            }
            else if (VRes == 1200)
            {
                *pPanelIndex = VIA_PANEL1920x1200;
            }
            break;                                         

        default:
            *pPanelIndex = VIA_PANEL_INVALID;
            break;
    }
}


Bool VIAFindModeUseModeList(VIABIOSInfoPtr pBIOSInfo)
{
    VIABIOSInfoPtr  pVia = pBIOSInfo;
    VIAModeInfoPtr m = pVia->VIAModeList;
    int j = 0, refIndex = 0;

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIAFindModeUseModeList\n"));

    /* Find Wanted Mode no. */
    while (m)
    {
    	if (pBIOSInfo->bitsPerPixel == m->bpp &&
    	    pBIOSInfo->CrtcHDisplay == m->width &&
    	    pBIOSInfo->CrtcVDisplay == m->height)
    	{
    	    pBIOSInfo->modeNum = m->modeno;
    	    break;
    	}
    	m = m->next;
    }
    if (!pBIOSInfo->modeNum)
        return FALSE;

    if (pBIOSInfo->ActiveDevice & VIA_DEVICE_LCD) 
    {
        switch (pBIOSInfo->LVDSSettingInfo.PanelSizeID) 
        {
            case VIA_PANEL6X4:
                pBIOSInfo->LVDSSettingInfo.PanelSizeX = 640;
                pBIOSInfo->LVDSSettingInfo.PanelSizeY = 480;
                break;
            case VIA_PANEL8X4:
                pBIOSInfo->LVDSSettingInfo.PanelSizeX = 800;
                pBIOSInfo->LVDSSettingInfo.PanelSizeY = 480;
                break;
            case VIA_PANEL8X6:
                pBIOSInfo->LVDSSettingInfo.PanelSizeX = 800;
                pBIOSInfo->LVDSSettingInfo.PanelSizeY = 600;
                break;
            case VIA_PANEL10X7:
                pBIOSInfo->LVDSSettingInfo.PanelSizeX = 1024;
                pBIOSInfo->LVDSSettingInfo.PanelSizeY = 768;
                break;
            case VIA_PANEL1280X720:
                pBIOSInfo->LVDSSettingInfo.PanelSizeX = 1280;
                pBIOSInfo->LVDSSettingInfo.PanelSizeY = 720;
                break;                
            case VIA_PANEL12X7:
                pBIOSInfo->LVDSSettingInfo.PanelSizeX = 1280;
                pBIOSInfo->LVDSSettingInfo.PanelSizeY = 768;
                break;
            case VIA_PANEL12X8:
                pBIOSInfo->LVDSSettingInfo.PanelSizeX = 1280;
                pBIOSInfo->LVDSSettingInfo.PanelSizeY = 800;
                break;                
            case VIA_PANEL12X10:
                pBIOSInfo->LVDSSettingInfo.PanelSizeX = 1280;
                pBIOSInfo->LVDSSettingInfo.PanelSizeY = 1024;
                break;
            case VIA_PANEL14X10:
                pBIOSInfo->LVDSSettingInfo.PanelSizeX = 1400;
                pBIOSInfo->LVDSSettingInfo.PanelSizeY = 1050;
                break;
            case VIA_PANEL16X12:
                pBIOSInfo->LVDSSettingInfo.PanelSizeX = 1600;
                pBIOSInfo->LVDSSettingInfo.PanelSizeY = 1200;
                break;
            case VIA_PANEL1366X7:
                pBIOSInfo->LVDSSettingInfo.PanelSizeX = 1366;
                pBIOSInfo->LVDSSettingInfo.PanelSizeY = 768;
                break;
            case VIA_PANEL1360X7:
                pBIOSInfo->LVDSSettingInfo.PanelSizeX = 1360;
                pBIOSInfo->LVDSSettingInfo.PanelSizeY = 768;
                break;
            case VIA_PANEL10X6:
                pBIOSInfo->LVDSSettingInfo.PanelSizeX = 1024;
                pBIOSInfo->LVDSSettingInfo.PanelSizeY = 600;
                break;
            default:
                pBIOSInfo->LVDSSettingInfo.PanelSizeID = VIA_PANEL10X7;
                pBIOSInfo->LVDSSettingInfo.PanelSizeX = 1024;
                pBIOSInfo->LVDSSettingInfo.PanelSizeY = 768;
                break;
        }

        /* If resolution bigger than panel size: panning */
        if  ((pBIOSInfo->CrtcHDisplay == pBIOSInfo->LVDSSettingInfo.PanelSizeX) &&
             (pBIOSInfo->CrtcVDisplay == pBIOSInfo->LVDSSettingInfo.PanelSizeY))
        {
            pBIOSInfo->Panning = FALSE;
            pBIOSInfo->LVDSSettingInfo.Center = FALSE;
            pBIOSInfo->scaleY = FALSE;
        }
        else if ((pBIOSInfo->CrtcHDisplay < pBIOSInfo->LVDSSettingInfo.PanelSizeX) ||
                 (pBIOSInfo->CrtcVDisplay < pBIOSInfo->LVDSSettingInfo.PanelSizeY))
        {
            pBIOSInfo->Panning = FALSE;
            pBIOSInfo->LVDSSettingInfo.Center = FALSE;
            pBIOSInfo->scaleY = TRUE;
        }
        else if ((pBIOSInfo->CrtcHDisplay > pBIOSInfo->LVDSSettingInfo.PanelSizeX) ||
                 (pBIOSInfo->CrtcVDisplay > pBIOSInfo->LVDSSettingInfo.PanelSizeY)) 
        {
            pBIOSInfo->frameX1 = pBIOSInfo->LVDSSettingInfo.PanelSizeX - 1;
            pBIOSInfo->frameY1 = pBIOSInfo->LVDSSettingInfo.PanelSizeY - 1;
            pBIOSInfo->HDisplay = pBIOSInfo->LVDSSettingInfo.PanelSizeX;
            pBIOSInfo->VDisplay = pBIOSInfo->LVDSSettingInfo.PanelSizeY;
            pBIOSInfo->CrtcHDisplay = pBIOSInfo->LVDSSettingInfo.PanelSizeX;
            pBIOSInfo->CrtcVDisplay = pBIOSInfo->LVDSSettingInfo.PanelSizeY;
            pBIOSInfo->Panning = TRUE;
            pBIOSInfo->LVDSSettingInfo.Center = TRUE;
            pBIOSInfo->scaleY = FALSE;
        }

    }

    /* Find Resolution index */
    switch (pBIOSInfo->CrtcHDisplay)
    {
        case 640:
            if (pBIOSInfo->CrtcVDisplay == 480)
            {
                j = M640x480;
            }
            break;
        case 720:
            if (pBIOSInfo->CrtcVDisplay == 480)
            {
                j = M720x480;
            }
            else if (pBIOSInfo->CrtcVDisplay == 576)
            {
                j = M720x576;
            }
            break;
        case 800:
            if (pBIOSInfo->CrtcVDisplay == 600)
            {
                j = M800x600;
            }
            else if (pBIOSInfo->CrtcVDisplay == 480)
            {
                j = M800x480;
            }
            break;
        case 848:
            if (pBIOSInfo->CrtcVDisplay == 480)
            {
                j = M848x480;
            }
            break;
        case 856:
            if (pBIOSInfo->CrtcVDisplay == 480)
            {
                j = M852x480;
            }
            break;
        case 960:
            if (pBIOSInfo->CrtcVDisplay == 600)
            {
                j = M960x600;
            }
            break;
        case 1000:
            if (pBIOSInfo->CrtcVDisplay == 600)
            {
                j = M1000x600;
            }
            break;
        case 1088:
            if (pBIOSInfo->CrtcVDisplay == 612)
            {
                j = M1088x612;
            }
            break;
        case 1024:
            switch (pBIOSInfo->CrtcVDisplay)
            {
                case 512:
                    j = M1024x512;
                    break;
                case 576:
                    j = M1024x576;
                    break;
                case 600:
                    j = M1024x600;
                    break;
                case 768:
                    j = M1024x768;
                    break;
            }
            break;
        case 1152:
            if (pBIOSInfo->CrtcVDisplay == 864)
            {
                j = M1152x864;
            }
            else if (pBIOSInfo->CrtcVDisplay == 720)
            {
                j = M1152x720;
            }
            break;
        case 1200:
            if (pBIOSInfo->CrtcVDisplay == 720)
            {
                j = M1200x720;
            }
            break;
        case 1280:
            switch (pBIOSInfo->CrtcVDisplay)
            {
                case 600:
                    j = M1280x600;
                    break;
                case 720:
                    j = M1280x720;
                    break;
                case 768:
                    j = M1280x768;
                    break;
                case 800:
                    j = M1280x800;
                    break;
                case 960:
                    j = M1280x960;
                    break;
                case 1024:
                    j = M1280x1024;
                    break;
            }
            break;
        case 1360:
            if (pBIOSInfo->CrtcVDisplay == 768)
            {
                j = M1360x768;
            }
            break;
        case 1366:
            if (pBIOSInfo->CrtcVDisplay == 768)
            {
                j = M1366x768;
            }
            break;
        case 1400:
            if (pBIOSInfo->CrtcVDisplay == 1050)
            {
                j = M1400x1050;
            }
            break;
        case 1440:
            if (pBIOSInfo->CrtcVDisplay == 1050)
            {
                j = M1440x1050;
            }
            else if (pBIOSInfo->CrtcVDisplay == 900)
            {
                j = M1440x900;
            }
            break;
        case 1600:
            if (pBIOSInfo->CrtcVDisplay == 1200)
            {
                j = M1600x1200;
            }
            else if (pBIOSInfo->CrtcVDisplay == 1024)
            {
                j = M1600x1024;
            }
            else if (pBIOSInfo->CrtcVDisplay == 900)
            {
                j = M1600x900;
            }
            break;            
        case 1680:
            if (pBIOSInfo->CrtcVDisplay == 1050)
            {
                j = M1680x1050;
            }
            else if (pBIOSInfo->CrtcVDisplay == 600)
            {
                j = M1680x600;
            }
            break;  
        case 1792:
            if (pBIOSInfo->CrtcVDisplay == 1344)
            {
                j = M1792x1344;
            }
            break;
        case 1856:
            if (pBIOSInfo->CrtcVDisplay == 1392)
            {
                j = M1856x1392;
            }
            break;
        case 1920:
            if (pBIOSInfo->CrtcVDisplay == 1440)
            {
                j = M1920x1440;
            }
            break;
        case 2048:
            if (pBIOSInfo->CrtcVDisplay == 1536)
            {
                j = M2048x1536;
            }
            break;
    }

    /* Find Refresh Rate */
    if (j != M_INVALID &&
        !(pBIOSInfo->ActiveDevice &
          (VIA_DEVICE_LCD | VIA_DEVICE_TV | VIA_DEVICE_DFP)))
    {
        if (pBIOSInfo->OptRefresh)
        {
            pBIOSInfo->Refresh = pBIOSInfo->OptRefresh;
        }
        else
        {
            pBIOSInfo->Refresh = ((pBIOSInfo->Clock * 10000) / (pBIOSInfo->HTotal * pBIOSInfo->VTotal) + 5) / 10;
        }

        if (pBIOSInfo->Refresh > 120 && (m->SupRefMask & VIA_REF_120HZ))
        {
            pBIOSInfo->FoundRefresh = 120;
            refIndex = 10;
        }
        else if (pBIOSInfo->Refresh > 100 && (m->SupRefMask & VIA_REF_100HZ))
        {
            pBIOSInfo->FoundRefresh = 100;
            refIndex = 9;
        }
        else if (pBIOSInfo->Refresh > 90 && (m->SupRefMask & VIA_REF_90HZ))
        {
            pBIOSInfo->FoundRefresh = 90;
            refIndex = 8;
        }
        else if (pBIOSInfo->Refresh > 85 && (m->SupRefMask & VIA_REF_85HZ))
        {
            pBIOSInfo->FoundRefresh = 85;
            refIndex = 7;
        }
        else if (pBIOSInfo->Refresh > 80 && (m->SupRefMask & VIA_REF_80HZ))
        {
            pBIOSInfo->FoundRefresh = 80;
            refIndex = 6;
        }
        else if (pBIOSInfo->Refresh > 75 && (m->SupRefMask & VIA_REF_75HZ))
        {
            pBIOSInfo->FoundRefresh = 75;
            refIndex = 5;
        }
        else if (pBIOSInfo->Refresh > 72 && (m->SupRefMask & VIA_REF_72HZ))
        {
            pBIOSInfo->FoundRefresh = 72;
            refIndex = 4;
        }
        else if (pBIOSInfo->Refresh > 70 && (m->SupRefMask & VIA_REF_70HZ))
        {
            pBIOSInfo->FoundRefresh = 70;
            refIndex = 3;
        }
        else if (pBIOSInfo->Refresh > 65 && (m->SupRefMask & VIA_REF_65HZ))
        {
            pBIOSInfo->FoundRefresh = 65;
            refIndex = 2;
        }
        /*else if (pBIOSInfo->Refresh > 60 && (m->SupRefMask & VIA_REF_60HZ)*/
        else
        {
            pBIOSInfo->FoundRefresh = 60;
            refIndex = 0;
        }
    }

    pBIOSInfo->resMode = j;
    pBIOSInfo->offsetWidthByQWord = (pBIOSInfo->displayWidth * (pBIOSInfo->bitsPerPixel >> 3)) >> 3;
    pBIOSInfo->countWidthByQWord = (pBIOSInfo->CrtcHDisplay * (pBIOSInfo->bitsPerPixel >> 3)) >> 3;    

    return TRUE;
}


static void
VIAInitSettingInfo_IGA(VIABIOSInfoPtr pBIOSInfo, int DisplayPath, int IsPanning, int HActive, int VActive)
{
    int OffsetWidthByQWord, CountWidthByQWord;
    OffsetWidthByQWord = (pBIOSInfo->displayWidth * (pBIOSInfo->bitsPerPixel >> 3)) >> 3;
    CountWidthByQWord = (HActive * (pBIOSInfo->bitsPerPixel >> 3)) >> 3;
    
    if (DisplayPath == IGA1)
    {
        pBIOSInfo->IGA1SettingInfo.IsPanning = IsPanning;
        pBIOSInfo->IGA1SettingInfo.HActive = HActive;
        pBIOSInfo->IGA1SettingInfo.VActive = VActive;
        pBIOSInfo->IGA1SettingInfo.OffsetWidthByQWord = OffsetWidthByQWord;
        pBIOSInfo->IGA1SettingInfo.CountWidthByQWord = CountWidthByQWord;
        pBIOSInfo->IGA1SettingInfo.SetStartingAddrFirstTime = TRUE;
        pBIOSInfo->IGA1SettingInfo.IsActive = TRUE;
    }
    else
    {
        pBIOSInfo->IGA2SettingInfo.IsPanning = IsPanning;
        pBIOSInfo->IGA2SettingInfo.HActive = HActive;
        pBIOSInfo->IGA2SettingInfo.VActive = VActive;
        pBIOSInfo->IGA2SettingInfo.OffsetWidthByQWord = OffsetWidthByQWord;
        pBIOSInfo->IGA2SettingInfo.CountWidthByQWord = CountWidthByQWord;
        pBIOSInfo->IGA2SettingInfo.SetStartingAddrFirstTime = TRUE;
        pBIOSInfo->IGA2SettingInfo.IsActive = TRUE;
    }
}


static void
VIAAdjustActiveSize_LCD(VIABIOSInfoPtr pBIOSInfo, LVDSSETTINGINFOPTR pLVDSSettingInfo, int *pHActive, int *pVActive)
{
    int HResToSet = pBIOSInfo->CrtcHDisplay;
    int VResToSet = pBIOSInfo->CrtcVDisplay;

    if ((HResToSet > pLVDSSettingInfo->PanelSizeX) || 
        (VResToSet > pLVDSSettingInfo->PanelSizeY))
    {
        if ((*pHActive > pLVDSSettingInfo->PanelSizeX) || 
            (*pVActive > pLVDSSettingInfo->PanelSizeY))
        {
            /* Need to adjust active size: */
            *pHActive = pLVDSSettingInfo->PanelSizeX;
            *pVActive = pLVDSSettingInfo->PanelSizeY;
        }

        pLVDSSettingInfo->IsPanning = TRUE;
    }
    else if ((pBIOSInfo->VirtualX > HResToSet) || (pBIOSInfo->VirtualY > VResToSet))
    {
        /* If use Virtual Desktop, and the size is larger than the resolution, */
        /* we need to do panning, too. */
        pLVDSSettingInfo->IsPanning = TRUE;
    }
    else
    {
        pLVDSSettingInfo->IsPanning = FALSE;
    }
}



static void
VIAAdjustActiveSize_TV(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo, int* pHActive, int* pVActive)
{
    int HResToSet = pBIOSInfo->CrtcHDisplay;
    int VResToSet = pBIOSInfo->CrtcVDisplay;
    int MaxTVHRes, MaxTVVRes;
    
    /* Lobokong[2007.04.16]: When TV scaling function is enable, we neednt to take care of Panning. */
    if (pBIOSInfo->Is3DScaling)
    {
         pTVSettingInfo->IsPanning = FALSE; 
    }
    else
    {
    
        /* Try to find max size of TV: */
        pTVSettingInfo->ModeIndex = VIAGetModeIndex(HResToSet, VResToSet);

        if (!VIAIsTVMode(pBIOSInfo, pTVSettingInfo->ModeIndex, pTVSettingInfo))
        {
            /* The mode is invalid, set a TV mode as default: */
            if ((*pHActive < 800) || (*pVActive < 600))
            {
                pTVSettingInfo->ModeIndex = M640x480;
                MaxTVHRes = 640;
                MaxTVVRes = 480;
            }
            else if ((*pHActive < 1024) || (*pVActive < 768))
            {
                pTVSettingInfo->ModeIndex = M800x600;
                MaxTVHRes = 800;
                MaxTVVRes = 600;
            }
            else
            {
                pTVSettingInfo->ModeIndex = M1024x768;
                MaxTVHRes = 1024;
                MaxTVVRes = 768;
            }

            /* Change view size if needed: */
            if ((HResToSet > MaxTVHRes) || (VResToSet > MaxTVVRes))
            {
                if ((*pHActive > MaxTVHRes) || (*pVActive > MaxTVVRes))
                {
                    *pHActive = MaxTVHRes;
                    *pVActive = MaxTVVRes;
                }

                pTVSettingInfo->IsPanning = TRUE;
            }
            else
            {
                pTVSettingInfo->IsPanning = FALSE;
            }
        }        
        else
        {
            pTVSettingInfo->IsPanning = FALSE;
        }

        if ((pBIOSInfo->VirtualX > HResToSet) || (pBIOSInfo->VirtualY > VResToSet))
        {
            /* If use Virtual Desktop, and the size is larger than the resolution, */
            /* we need to do panning, too. */
            pTVSettingInfo->IsPanning = TRUE;
        }
    }        

}


static void
VIAAdjustActiveSize(VIABIOSInfoPtr pBIOSInfo)
{
    int     HResToSet = pBIOSInfo->CrtcHDisplay;
    int     VResToSet = pBIOSInfo->CrtcVDisplay;
    int     HActive = pBIOSInfo->CrtcHDisplay;
    int     VActive = pBIOSInfo->CrtcVDisplay;    

    /* Try to find a suitable view size, we should use the minimum one if more than one device */
    /* need to panning */
    
    if (pBIOSInfo->ActiveDevice & VIA_DEVICE_TV)
    {
        VIAAdjustActiveSize_TV(pBIOSInfo, &(pBIOSInfo->TVSettingInfo), &HActive, &VActive);
    }

    if (pBIOSInfo->ActiveDevice & VIA_DEVICE_TV2)
    {
        VIAAdjustActiveSize_TV(pBIOSInfo, &(pBIOSInfo->TVSettingInfo2), &HActive, &VActive);
    }

    if ((pBIOSInfo->ActiveDevice & VIA_DEVICE_CRT1) ||
        (pBIOSInfo->ActiveDevice & VIA_DEVICE_CRT2))
    {
        if ((HResToSet > pBIOSInfo->CRTSettingInfo.MonitorSizeH) ||
            (VResToSet > pBIOSInfo->CRTSettingInfo.MonitorSizeV))
        {
            if ((HActive > pBIOSInfo->CRTSettingInfo.MonitorSizeH) ||
                (VActive > pBIOSInfo->CRTSettingInfo.MonitorSizeV))
            {
                HActive = pBIOSInfo->CRTSettingInfo.MonitorSizeH;
                VActive = pBIOSInfo->CRTSettingInfo.MonitorSizeV;
            }

            pBIOSInfo->CRTSettingInfo.IsPanning = TRUE;
        }
        else if ((pBIOSInfo->VirtualX > HResToSet) || (pBIOSInfo->VirtualY > VResToSet))
        {
            /* If use Virtual Desktop, and the size is larger than the resolution, */
            /* we need to do panning, too. */
            pBIOSInfo->CRTSettingInfo.IsPanning = TRUE;
        }
        else
        {
            pBIOSInfo->CRTSettingInfo.IsPanning = FALSE;
        }
    }

    if (pBIOSInfo->ActiveDevice & VIA_DEVICE_DFP)
    {
        if ((HResToSet > pBIOSInfo->TMDSSettingInfo.PanelSizeH) ||
            (VResToSet > pBIOSInfo->TMDSSettingInfo.PanelSizeV))
        {
            if ((HActive > pBIOSInfo->TMDSSettingInfo.PanelSizeH) ||
                (VActive > pBIOSInfo->TMDSSettingInfo.PanelSizeV))
            {
                HActive = pBIOSInfo->TMDSSettingInfo.PanelSizeH;
                VActive = pBIOSInfo->TMDSSettingInfo.PanelSizeV;
            }

            pBIOSInfo->TMDSSettingInfo.IsPanning = TRUE;
        }
        else if ((pBIOSInfo->VirtualX > HResToSet) || (pBIOSInfo->VirtualY > VResToSet))
        {
            /* If use Virtual Desktop, and the size is larger than the resolution, */
            /* we need to do panning, too. */
            pBIOSInfo->TMDSSettingInfo.IsPanning = TRUE;
        }
        else
        {
            pBIOSInfo->TMDSSettingInfo.IsPanning = FALSE;
        }
    }

    if (pBIOSInfo->ActiveDevice & VIA_DEVICE_LCD)
    {
        VIAAdjustActiveSize_LCD(pBIOSInfo, &(pBIOSInfo->LVDSSettingInfo), &HActive, &VActive);
    }

    if (pBIOSInfo->ActiveDevice & VIA_DEVICE_LCD2)
    {
        VIAAdjustActiveSize_LCD(pBIOSInfo, &(pBIOSInfo->LVDSSettingInfo2), &HActive, &VActive);
    }

    /* STChen[2006.12.20]: If we use direct access window to rotate screen, 
       the screen could be panning. In order to avoid this situation, we must 
       switch the coordinate x,y of frame.
    */
    if ((pBIOSInfo->HWRotate == VIA_ROTATE_DEGREE_90) ||
        (pBIOSInfo->HWRotate == VIA_ROTATE_DEGREE_270))
    {
        pBIOSInfo->frameX1 = VActive - 1;
        pBIOSInfo->frameY1 = HActive - 1;
    }
    else /* non-rotate */
    {
        pBIOSInfo->frameX1 = HActive - 1;
        pBIOSInfo->frameY1 = VActive - 1;
    }
    
    pBIOSInfo->HDisplay = HActive;
    pBIOSInfo->VDisplay = VActive;
}

static void
VIAInitSettingInfo_CRT(VIABIOSInfoPtr pBIOSInfo)
{
    int     HActive, VActive;
    Bool    IsPanning = pBIOSInfo->CRTSettingInfo.IsPanning;

    if (IsPanning)
    {
        HActive = pBIOSInfo->HDisplay;
        VActive = pBIOSInfo->VDisplay;

        xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "CRT Max Resolution %dx%d, set virtual desktop!!\n", 
            pBIOSInfo->CRTSettingInfo.MonitorSizeH, pBIOSInfo->CRTSettingInfo.MonitorSizeV);
    }
    else
    {
        HActive = pBIOSInfo->CrtcHDisplay;
        VActive = pBIOSInfo->CrtcVDisplay;
    }

    VIAInitSettingInfo_IGA(pBIOSInfo, pBIOSInfo->CRTSettingInfo.IGAPath, IsPanning, HActive, VActive);

    pBIOSInfo->CRTSettingInfo.ModeIndex = VIAGetModeIndex(HActive, VActive);
    pBIOSInfo->CRTSettingInfo.HActive = HActive;
    pBIOSInfo->CRTSettingInfo.VActive = VActive;
}


static void
VIAInitSettingInfo_TV(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo)
{
    int     HActive, VActive;
    Bool    IsPanning = pTVSettingInfo->IsPanning;

    if (IsPanning)
    {
        HActive = pBIOSInfo->HDisplay;
        VActive = pBIOSInfo->VDisplay;
    }
    else
    {
        HActive = pBIOSInfo->CrtcHDisplay;
        VActive = pBIOSInfo->CrtcVDisplay;
    }

    VIAInitSettingInfo_IGA (pBIOSInfo, pTVSettingInfo->IGAPath, IsPanning, HActive, VActive);
    pTVSettingInfo->HActive = HActive;
    pTVSettingInfo->VActive = VActive;
}


static void
VIAInitSettingInfo_DVI(VIABIOSInfoPtr pBIOSInfo)
{
    int     HActive, VActive;
    Bool    IsPanning = pBIOSInfo->TMDSSettingInfo.IsPanning;

    if (IsPanning)
    {
        HActive = pBIOSInfo->HDisplay;
        VActive = pBIOSInfo->VDisplay;

        xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "DFP Max Resolution %dx%d, set virtual desktop!!\n", 
            pBIOSInfo->TMDSSettingInfo.PanelSizeH, pBIOSInfo->TMDSSettingInfo.PanelSizeV);
    }
    else
    {
        HActive = pBIOSInfo->CrtcHDisplay;
        VActive = pBIOSInfo->CrtcVDisplay;
    }
    VIAInitSettingInfo_IGA (pBIOSInfo, pBIOSInfo->TMDSSettingInfo.IGAPath, IsPanning, HActive, VActive);

    pBIOSInfo->TMDSSettingInfo.ModeIndex = VIAGetModeIndex(HActive, VActive);
    pBIOSInfo->TMDSSettingInfo.HActive = HActive;
    pBIOSInfo->TMDSSettingInfo.VActive = VActive;

    /* RainLin [2006.05.03]: To support for DELL 2405FPW Monitor. */
    /* For DVI, 1920x1080 and 1920x1200 should use CVT generated mode table instead. */
    if (pBIOSInfo->TMDSSettingInfo.ModeIndex == M1920x1080)
    {
        pBIOSInfo->TMDSSettingInfo.ModeIndex = M1920x1080_DVI;
    }
    else if (pBIOSInfo->TMDSSettingInfo.ModeIndex == M1920x1200)
    {
        pBIOSInfo->TMDSSettingInfo.ModeIndex = M1920x1200_DVI;
    }
}


static void
VIAInitSettingInfo_LCD(VIABIOSInfoPtr pBIOSInfo, LVDSSETTINGINFOPTR pLVDSSettingInfo)
{
    int     HActive, VActive;
    Bool    IsPanning = pLVDSSettingInfo->IsPanning;

    if (IsPanning)
    {
        HActive = pBIOSInfo->HDisplay;
        VActive = pBIOSInfo->VDisplay;

        xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "LCD Max Resolution %dx%d, set virtual desktop!!\n", 
            pLVDSSettingInfo->PanelSizeX, pLVDSSettingInfo->PanelSizeY);
    }
    else
    {
        HActive = pBIOSInfo->CrtcHDisplay;
        VActive = pBIOSInfo->CrtcVDisplay;
    }

    VIAInitSettingInfo_IGA(pBIOSInfo, pLVDSSettingInfo->IGAPath, IsPanning, HActive, VActive);

    pLVDSSettingInfo->ModeIndex = VIAGetModeIndex(HActive, VActive);
    pLVDSSettingInfo->HActive = HActive;
    pLVDSSettingInfo->VActive = VActive;
}


Bool VIAFindModeUseBIOSTable(VIABIOSInfoPtr pBIOSInfo)
{
    VIAModeTablePtr pViaModeTable;
    int             needRefresh;
    int             i = 0, ModeIndex;
    int             modeNum;
    Bool            IsFoundRefresh = FALSE;

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIAFindModeUseBIOSTable\n"));
    pViaModeTable = pBIOSInfo->pModeTable;

    /* detemine support highest resolution by Memory clk */
    pBIOSInfo->MemClk = (read_reg(VIACR, CR3D) & 0xF0) >> 4;
    
    if ((pBIOSInfo->bitsPerPixel > 16) && (pBIOSInfo->HDisplay == 1600) &&
        (pBIOSInfo->VDisplay == 1200) && (pBIOSInfo->MemClk < VIA_MEM_DDR266))
    {
        ErrorF("\n1600x1200 True Color only support in MEMCLK higher than DDR266 platform!!\n");
        ErrorF("Please use lower bpp or resolution.\n");
        return FALSE;
    }

    if ((pBIOSInfo->ActiveDevice & VIA_DEVICE_DFP) &&
        (pBIOSInfo->TMDSSettingInfo.DFPSize == M_INVALID))
    {
        VIAGetPanelInfo(pBIOSInfo);
    }

    if (!pBIOSInfo->ActiveDevice)
    {
        pBIOSInfo->ActiveDevice = VIAGetDeviceDetect(pBIOSInfo);
    }

    /* TV + LCD/DVI has no simultaneous, block it */
    if (!pBIOSInfo->DuoView&&((pBIOSInfo->ActiveDevice & VIA_DEVICE_TV) &&
        (pBIOSInfo->ActiveDevice & (VIA_DEVICE_LCD | VIA_DEVICE_DFP))))
    {
        if (pBIOSInfo->MobileFamily)
            pBIOSInfo->ActiveDevice = VIA_DEVICE_LCD;
        else
            pBIOSInfo->ActiveDevice = VIA_DEVICE_TV;
    }

    /* Find Resolution index: */
    ModeIndex = VIAGetModeIndex(pBIOSInfo->CrtcHDisplay, pBIOSInfo->CrtcVDisplay);

    if (pBIOSInfo->VirtualX > 0)
    {
        /* If set Virtual Desktop Size in xorg.conf: */
        pBIOSInfo->ActualDesktopSizeX = pBIOSInfo->VirtualX;
        pBIOSInfo->ActualDesktopSizeY = pBIOSInfo->VirtualY;
    }
    else
    {
        pBIOSInfo->ActualDesktopSizeX = pBIOSInfo->CrtcHDisplay;
        pBIOSInfo->ActualDesktopSizeY = pBIOSInfo->CrtcVDisplay;
    }

    /* Initial device's setting information: */
    VIAAdjustActiveSize(pBIOSInfo);
    
    if ((pBIOSInfo->ActiveDevice & VIA_DEVICE_CRT1) ||
        (pBIOSInfo->ActiveDevice & VIA_DEVICE_CRT2))
    {        
        VIAInitSettingInfo_CRT(pBIOSInfo);
    }
    
    if (pBIOSInfo->ActiveDevice & VIA_DEVICE_DFP)
    {
        VIAInitSettingInfo_DVI(pBIOSInfo);
    }
    
    if (pBIOSInfo->ActiveDevice & VIA_DEVICE_LCD)
    {
        VIAInitSettingInfo_LCD(pBIOSInfo, &(pBIOSInfo->LVDSSettingInfo));
    }

    if (pBIOSInfo->ActiveDevice & VIA_DEVICE_LCD2)
    {
        VIAInitSettingInfo_LCD(pBIOSInfo, &(pBIOSInfo->LVDSSettingInfo2));
    }

    if (pBIOSInfo->ActiveDevice & VIA_DEVICE_TV)
    {
        VIAInitSettingInfo_TV(pBIOSInfo, &(pBIOSInfo->TVSettingInfo));
    }

    if (pBIOSInfo->ActiveDevice & VIA_DEVICE_TV2)
    {
        VIAInitSettingInfo_TV(pBIOSInfo, &(pBIOSInfo->TVSettingInfo2));
    }


    for (i = 0; i < pViaModeTable->NumModes; i++)
    {
        if ((pViaModeTable->Modes[i].Bpp == pBIOSInfo->bitsPerPixel) &&
            (pViaModeTable->Modes[i].Width == pBIOSInfo->HDisplay) &&
            (pViaModeTable->Modes[i].Height == pBIOSInfo->VDisplay))
            break;
    }
    
    if (i == pViaModeTable->NumModes)
    {
        ErrorF("\nVIASetModeUseBIOSTable: Cannot find suitable mode!!\n");
        ErrorF("Mode setting in XF86Config(-4) is not supported!!\n");
        return FALSE;
    }
    
    pBIOSInfo->modeNum = modeNum = (int)pViaModeTable->Modes[i].Mode;
    pBIOSInfo->mode = i;

    /* Find suitable refresh rate: */
    
    /* RainLin [2005.09.06]: Set the refresh rate to 60MHz for DuoView. */
    /* Note: We should support other refresh rate later. */
    if (/*pBIOSInfo->DuoView ||*/ 
        (ModeIndex != M_INVALID &&
        !(pBIOSInfo->ActiveDevice & (VIA_DEVICE_LCD | VIA_DEVICE_LCD2 | VIA_DEVICE_TV | VIA_DEVICE_DFP))))
    {
        if (pBIOSInfo->OptRefresh)
        {
            pBIOSInfo->Refresh = pBIOSInfo->OptRefresh;           
            IsFoundRefresh = VIAFindRefreshRate(pBIOSInfo, ModeIndex);

            if (!IsFoundRefresh)
            {
                xf86DrvMsg(pBIOSInfo->scrnIndex, X_ERROR, "Refresh rate setting in XF86Config(-4) is not supported!!\n");
                xf86DrvMsg(pBIOSInfo->scrnIndex, X_ERROR, "Driver will try to find another refresh rate instead.\n");
            }
        }

        if (!IsFoundRefresh)
        {
            /* use the monitor information */
            /* needRefresh = (pBIOSInfo->Clock * 1000) / (pBIOSInfo->HTotal * pBIOSInfo->VTotal); */
            /* Do rounding */
            needRefresh = ((pBIOSInfo->Clock * 10000) / (pBIOSInfo->HTotal * pBIOSInfo->VTotal) + 5) / 10;
            pBIOSInfo->Refresh = needRefresh;            
            IsFoundRefresh = VIAFindRefreshRate(pBIOSInfo, ModeIndex);
        }

        if (!IsFoundRefresh)
        {
            pBIOSInfo->FoundRefresh = 60;
            xf86DrvMsg(pBIOSInfo->scrnIndex, X_ERROR, "Can't find suitable refresh rate, use 60Hz as default.\n");
        }
    }
    else
    {
        pBIOSInfo->FoundRefresh = 60;
    }

    pBIOSInfo->resMode = ModeIndex;
    pBIOSInfo->resMode2 = ModeIndex;

    /* pBIOSInfo->widthByQWord = (pBIOSInfo->displayWidth * (pBIOSInfo->bitsPerPixel >> 3)) >> 3; */
    pBIOSInfo->offsetWidthByQWord = (pBIOSInfo->displayWidth * (pBIOSInfo->bitsPerPixel >> 3)) >> 3;
    pBIOSInfo->countWidthByQWord = (pBIOSInfo->CrtcHDisplay * (pBIOSInfo->bitsPerPixel >> 3)) >> 3;

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "pBIOSInfo->FoundRefresh: %d\n", pBIOSInfo->FoundRefresh));
    return TRUE;
}

static void
VIAPrimaryDisplayOffsetPatch(VIABIOSInfoPtr pBIOSInfo)
{
    int     countWidthByQWord, offsetWidthByQWord;
    CARD8   cr13, cr35, sr1c, sr1d;

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIAPrimaryDisplayOffsetPatch\n"));    
    
    countWidthByQWord = pBIOSInfo->IGA1SettingInfo.CountWidthByQWord;
    offsetWidthByQWord = pBIOSInfo->IGA1SettingInfo.OffsetWidthByQWord;
    
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "countWidthByQWord = %d\n", countWidthByQWord));
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "offsetWidthByQWord = %d\n", offsetWidthByQWord));    

    /* IGA1's offset */
    cr13 = offsetWidthByQWord & 0xFF;
    cr35 = read_reg(VIACR, CR35) & 0x1F;    
    cr35 |= (offsetWidthByQWord & 0x700) >> 3;        /* bit 7:5: offset 10:8 */

    /* IGA1's quad-word count. */
    sr1c = ((countWidthByQWord >> 1) + 1) & 0xFF;
    sr1d = read_reg(VIASR, SR1D);
    sr1d = (sr1d & 0xFC) | (countWidthByQWord >> 9);

    write_reg(CR13, VIACR, cr13);
    write_reg(CR35, VIACR, cr35);
    write_reg(SR1C, VIASR, sr1c);
    write_reg(SR1D, VIASR, sr1d);
}

static void
VIASecondDisplayOffsetPatch(VIABIOSInfoPtr pBIOSInfo)
{    
    /* RainLin [2005.11.29]: If use virtual desktop, we should update */
    /* second display horizontal offset to the width of desktop. */ 
    if (pBIOSInfo->Is3DScaling)
    {
        load_offset_reg(pBIOSInfo->TVSettingInfo.HActive, pBIOSInfo->bitsPerPixel >> 3, IGA2);
    }
    else if (pBIOSInfo->displayWidth > pBIOSInfo->IGA2SettingInfo.HActive)
    {
        load_offset_reg(pBIOSInfo->displayWidth, pBIOSInfo->bitsPerPixel >> 3, IGA2);
    }
}


Bool VIAPrimarySetMode(ScrnInfoPtr pScrn)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;
    int mode = pBIOSInfo->modeNum;
    /*
    int countWidthByQWord, offsetWidthByQWord;
    unsigned char   cr13, cr35, sr1c, sr1d;
     */

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIAPrimarySetMode\n"));

    if (pBIOSInfo->LVDSSettingInfo.Center || pBIOSInfo->Panning)
    {
        BIOS_SetFlatPanelState(pScrn, FALSE);
    }
    else
    {
        BIOS_SetFlatPanelState(pScrn, TRUE);
    }

    /* Don't clear video memory */
    mode |= (1 << 15);
    /* enable linear addressing
    if (pVia->FrameBufferBase != 0xa0000)
	{
	    mode |= 1 << 14;
    }*/

    if(!VBESetVBEMode(pVia->pVbe, mode, NULL))
    {
        return FALSE;
    }

    if(!BIOS_SetActiveDevice(pScrn))
    {
        return FALSE;
    }

    /* Set Quadword offset and counter */
    /*=* Fix bandwidth problem when using virtual desktop *=*/
    VIAPrimaryDisplayOffsetPatch(pBIOSInfo);

    if (pBIOSInfo->FirstInit)
    {
        /* Clear Memory */
        memset(pBIOSInfo->FBBase, 0x00, pBIOSInfo->videoRambytes - VIA_FB_CURSOR_SIZE);
        /*pBIOSInfo->FirstInit = FALSE;*/
    }

    VIAEnabledPrimaryExtendedFIFO(pBIOSInfo);

    return TRUE;
}

Bool VIASetModeForMHS(VIABIOSInfoPtr pBIOSInfo)
{
    VIABIOSInfoPtr  pVia;

    pVia = pBIOSInfo;
    pBIOSInfo->SetTV = FALSE;
    int IGAToUse = -1;	/* XXX 'IGAToUse' may be used uninitialized in this function */
    
    if (pBIOSInfo->FirstInit)
    {
        /* Clear Memory. Sometimes it will appear some garbage under 1366x768 SAMM(LCD+CRT).  */      
        memset(pBIOSInfo->FBBase, 0x00, pBIOSInfo->videoRambytes - VIA_FB_CURSOR_SIZE); 
    }
    
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIASetModeForMHS\n"));
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "Active Device is 0x%x\n", pBIOSInfo->ActiveDevice));
    
    /* Turn off Screen */
    write_reg_mask(CR17, VIACR, 0x0, BIT7);

    if (pBIOSInfo->ActiveDevice & VIA_DEVICE_CRT1)
    {
        VIACRTSetMode(pBIOSInfo,pBIOSInfo->CRTSettingInfo.ModeIndex, pBIOSInfo->CRTSettingInfo.IGAPath);
        VIACRTEnable();
        IGAToUse = pBIOSInfo->CRTSettingInfo.IGAPath;
    }
    
    if (pBIOSInfo->ActiveDevice & VIA_DEVICE_CRT2)/*set mode for CRT2*/ 
    {
        VIACRTSetMode(pBIOSInfo,pBIOSInfo->CRTSettingInfo.ModeIndex, pBIOSInfo->CRT2_IGA);
        VIACRTEnable();
        pBIOSInfo->TVSettingInfo.IGAPath = pBIOSInfo->CRT2_IGA;	
        load_vt1625_regs(pBIOSInfo, &(pBIOSInfo->TVSettingInfo));/*fill vt1625 regs*/	
        write_reg(CR6B, VIACR, 0x01);
        IGAToUse = pBIOSInfo->CRT2_IGA;
    }    

    if (pBIOSInfo->ActiveDevice & VIA_DEVICE_TV)
    {
        VIATVSetMode(pBIOSInfo, &(pBIOSInfo->TVSettingInfo));
        IGAToUse = pBIOSInfo->TVSettingInfo.IGAPath;
    }

    if (pBIOSInfo->ActiveDevice & VIA_DEVICE_TV2)
    {
        VIATVSetMode(pBIOSInfo, &(pBIOSInfo->TVSettingInfo2));
        IGAToUse = pBIOSInfo->TVSettingInfo2.IGAPath;
    }
    
    if ((pBIOSInfo->Chipset == VIA_CLE266) && (pBIOSInfo->ChipRev < 15))
    {
        if (!pBIOSInfo->CLE266A2)
        {
            write_reg_mask(CR6C, VIACR, 0x80, BIT1+BIT2+BIT3+BIT4);
        }
    }

    if (pBIOSInfo->ActiveDevice & VIA_DEVICE_LCD)
    {
        VIALCDSetMode(pBIOSInfo, &(pBIOSInfo->LVDSSettingInfo));
        VIAEnableLCD(pBIOSInfo, &(pBIOSInfo->LVDSSettingInfo));
        IGAToUse = pBIOSInfo->LVDSSettingInfo.IGAPath;
    }

    if (pBIOSInfo->ActiveDevice & VIA_DEVICE_LCD2)
    {
        VIALCDSetMode(pBIOSInfo, &(pBIOSInfo->LVDSSettingInfo2));
        VIAEnableLCD(pBIOSInfo, &(pBIOSInfo->LVDSSettingInfo2));
        IGAToUse = pBIOSInfo->LVDSSettingInfo2.IGAPath;
    }


    /*move DFP setmode after LCD,else DVI will be turned off by VIADisableLCD on EPIA*/
    if (pBIOSInfo->ActiveDevice & VIA_DEVICE_DFP)
    {
        VIASetDFPMode(pBIOSInfo);
        VIAEnableDFP(pBIOSInfo);
        IGAToUse = pBIOSInfo->TMDSSettingInfo.IGAPath;
    }    

    /* Set Extended FIFO & Expire Number */
    VIAEnabledSecondaryExtendedFIFO(pBIOSInfo);
    VIAFillExpireNumber(pBIOSInfo); 

    VIAPitchAlignmentPatch(pBIOSInfo);
    
    /* Set Quadword offset and counter */
    /*
    These code will patch IGA2 regs,according secondary display size.
    When LCD2+LCD1,LCD is primary on IGA2,
    In this case,IGA2 regs should not patch.
    */

    if(!(pBIOSInfo->UseIGA2 && pBIOSInfo->TwoLCD))
    {
        if (IGAToUse == IGA1)
        {
            VIAPrimaryDisplayOffsetPatch(pBIOSInfo);
        }
        else
        {
            /* RainLin [2005.11.29]: Replaced by function call. */
            VIASecondDisplayOffsetPatch(pBIOSInfo);
        }
    }

    set_color_depth(pBIOSInfo, pBIOSInfo->bitsPerPixel>>3, IGA2);    

    /* Enable CRT Controller (3D5.17 Hardware Reset) */
    write_reg_mask(CR17, VIACR, 0x80, BIT7);

    /* Open Screen */
    (void) VGAIN8(0x3da);
    VGAOUT8(0x3c0, 0x20);

    /* Do TV Software Reset: */
    if (pBIOSInfo->ActualActiveDevice & VIA_DEVICE_TV)
    {
        VIADoTVSWReset(pBIOSInfo, &(pBIOSInfo->TVSettingInfo));                
    }        
    
    if (pBIOSInfo->ActualActiveDevice & VIA_DEVICE_TV2)
    {
        VIADoTVSWReset(pBIOSInfo, &(pBIOSInfo->TVSettingInfo2));
    }
    
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "-- VIASetModeForMHS Done!\n"));
    return TRUE;
}

Bool VIASaveGamma(ScrnInfoPtr pScrn, LOCO *colors) {
    VIAPtr  pVia = VIAPTR(pScrn);
    CARD8   sr1a;
    int     i;

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIASaveGamma\n"));
    /* 8 bpp mode can't adjust gamma */
    if (pScrn->bitsPerPixel == 8)
        return FALSE;

    /* Enable Gamma */
    switch (pVia->Chipset)
    {
        case VIA_CLE266:
        case VIA_KM400:
            write_reg_mask(SR16, VIASR, 0x80, BIT7);
            break;

        /* For K8M800 or later chipsets: */
        default:
            write_reg_mask(CR33, VIACR, 0x80, BIT7);
            break;
    }
    /* map to Primary Display's LUT */
    sr1a = read_reg(VIASR, SR1A);
    write_reg_mask(SR1A, VIASR, 0x0, BIT0);

    VGAOUT8(0x3c7, 0);
    for (i = 0; i < 256; i++)
    {
        colors[i].red = VGAIN8(0x3c9);
        colors[i].green = VGAIN8(0x3c9);
        colors[i].blue = VGAIN8(0x3c9);
    }

    write_reg(SR1A, VIASR, sr1a);

    return TRUE;
}

Bool VIARestoreGamma(ScrnInfoPtr pScrn, LOCO *colors) {
    VIAPtr pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr pBIOSInfo = pVia->pBIOSInfo;
    int i, sr1a, cr6a;

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIARestorePalette\n"));
    /* 8 bpp mode can't adjust gamma */
    if (pScrn->bitsPerPixel == 8)
        return FALSE;

    /* Enable Gamma */
    switch (pVia->Chipset)
    {
        case VIA_CLE266:
        case VIA_KM400:
            write_reg_mask(SR16, VIASR, 0x80, BIT7);
            break;

        /* For K8M800 or later chipsets: */
        default:
            write_reg_mask(CR33, VIACR, 0x80, BIT7);
            break;
    }
    sr1a = read_reg(VIASR, SR1A);
    write_reg_mask(SR1A, VIASR, 0x0, BIT0);                    
        
    VGAOUT8(0x3c8, 0);
    for (i = 0; i < 256; i++)
    {
        VGAOUT8(0x3c9, colors[i].red);
        VGAOUT8(0x3c9, colors[i].green);
        VGAOUT8(0x3c9, colors[i].blue);
    }

    /* Fill IGA1,IGA2 Gamma table simultanous.*/
    if (!((pBIOSInfo->Chipset == VIA_CLE266) && (pBIOSInfo->ChipRev < 15)))
    {
        write_reg_mask(SR1A, VIASR, 0x01, BIT0);
       
        write_reg_mask(CR6A, VIACR, 0x02, BIT1);

        /* We should set the format of IGA2's LUT to 8-BITs, because we use 8-BITs LUT except VT3123Ax. */
        /* This register added in VT3314 or later chipsets. */
        switch(pBIOSInfo->Chipset)
        {
            case VIA_CLE266:
            case VIA_KM400:
            case VIA_K8M800:
            case VIA_PM800:
                /* No need to set. */
                break;
        
            default:
                write_reg_mask(CR6A, VIACR, BIT5, BIT5);
                break;
        }              

        /* S.T.Chen[2006.6.1]:
         Situation:
           On VT3324, VT3293, when we switch to two devices, the second LUT maybe fault.
         Solution:
           Before we fill the second LUT, we have to enable second display channel.
        */
        /* Save */
        VGAOUT8(0x3D4, CR6A);
        cr6a = VGAIN8(0x3D5);
        
        VGAOUT8(0x3D4, CR6A);           
        if(!(VGAIN8(0x3D5) & 0x80))
        {                
            EnableSecondDisplayChannel();
        }

        /* Fill Second LUT */
        VGAOUT8(0x3c8, 0);
        for (i = 0; i < 256; i++)
        {
            VGAOUT8(0x3c9, colors[i].red);
            VGAOUT8(0x3c9, colors[i].green);
            VGAOUT8(0x3c9, colors[i].blue);
        }

        /* After filled second LUT, we should restore to previous value. */
        VGAOUT8(0x3D4, CR6A);
        VGAOUT8(0x3D5, cr6a);
    }

    write_reg(SR1A, VIASR, sr1a);
    return TRUE;
}

void VIAPitchAlignmentPatch(VIABIOSInfoPtr pBIOSInfo)
{
    CARD8      cr13, cr35, cr65, cr66, cr67;
    CARD32     dwScreenPitch= 0;
    CARD32     dwPitch;
    CARD8      sr1c, sr1d;

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED, "VIAPitchAlignmentPatch\n"));
    
    dwPitch = pBIOSInfo->HDisplay * (pBIOSInfo->bitsPerPixel >> 3);
    if (dwPitch & 0x1F) /* Is 32 Byte Alignment ? */
    {
        /* Pitch = byte per line. */
        dwPitch = ((pBIOSInfo->HDisplay + 31) & ~31) * (pBIOSInfo->bitsPerPixel >> 3);
        dwScreenPitch = ((dwPitch + 31) & ~31) >> 3;
        
        if (pBIOSInfo->UseIGA2 || pBIOSInfo->DuoView ||
           (pBIOSInfo->EPIA_DVI && (pBIOSInfo->DuoView == FALSE))
           || ((VIA_CLE266 == pBIOSInfo->Chipset) && (VIA_DEVICE_LCD == pBIOSInfo->ActiveDevice)))
        {            
            if (!(pBIOSInfo->Is3DScaling))
            {
                /* IGA2 offset */
                cr66 = (CARD8)(dwScreenPitch & 0xFF);
                write_reg(CR66, VIACR, cr66);
                cr67 = read_reg(VIACR, CR67) & 0xFC;
                cr67 |= (CARD8)((dwScreenPitch & 0x300) >> 8);
                write_reg(CR67, VIACR, cr67);

                /* IGA2 quad-word count */
                cr67 = read_reg(VIACR, CR67) & 0xF3;
                cr67 |= (CARD8)((dwScreenPitch & 0x600) >> 7);
                write_reg(CR67, VIACR, cr67);
                cr65 = (CARD8)((dwScreenPitch >> 1) & 0xFF);
                cr65 += 2;
                write_reg(CR65, VIACR, cr65);
            }
        }
        else
        {           
            /* IGA1 offset */
            cr13 = (CARD8)(dwScreenPitch & 0xFF);
            write_reg(CR13, VIACR, cr13);
            cr35 = read_reg(VIACR, CR35) & 0x1F;
            cr35 |= (CARD8)((dwScreenPitch & 0x700) >> 3);
            write_reg(CR35, VIACR, cr35);

            /* IGA1 quad-word count. */
            sr1c = ((dwScreenPitch >> 1) + 1) & 0xFF;
            write_reg(SR1C, VIASR, sr1c);
            sr1d = read_reg(VIASR, SR1D);
            sr1d = (sr1d & 0xFC) | (dwScreenPitch >> 9);            
            write_reg(SR1D, VIASR, sr1d);
        }
    }
}


Bool VIAGetBIOSInfoFromROM(VIAPtr pVia, unsigned char flags)
{
    VIABIOSInfoPtr pBIOSInfo = pVia->pBIOSInfo;
    unsigned char   *pBIOS = NULL, *pRom, *pTable;
    int             romSize, sum, i;


    DEBUG(xf86DrvMsg(pVia->pBIOSInfo->scrnIndex, X_INFO, "VIAGetBIOSVerFromROM\n"));
    /* Allocate shadow memory */
    if (!(pBIOS=xcalloc(1, 0x10000)))
    {
        ErrorF("Allocate memory fail !!\n");
        return FALSE;
    }
    /* Read BIOS ROM and check! */
    if (xf86ReadBIOS(0xC0000, 0, pBIOS, 0x10000) != 0x10000)
    {
        xfree(pBIOS);
        ErrorF("Read VGA BIOS image fail !!\n");
    }
    else
    {
        if (*((CARD16 *) pBIOS) != 0xAA55)
        {
            xfree(pBIOS);
            ErrorF("VGA BIOS image is wrong!!\n");
            return FALSE;
        }
        else
        {
            romSize = *((CARD8 *) (pBIOS + VIA_BIOS_SIZE_POS)) * 512;
            pRom = pBIOS;
            sum = 0;

            for (i = 0; i < romSize; i++)
            {
                sum += *pRom++;
            }

            if (((CARD8) sum) != 0)
            {
                xfree(pBIOS);
                ErrorF("VGA BIOS image is wrong!! CheckSum = %x\n", sum);
                return FALSE;
            }
        }
    }

    /* Get the start of Table */
    pRom = pBIOS + VIA_BIOS_OFFSET_POS;
    pTable = pBIOS + *((CARD16 *)pRom);

    if (flags & VIAGETBIOSDATE)
    {
        /* Get the start of bcpPost structure */
        pRom = pTable + VIA_BIOS_BCPPOST_POS;
        pRom = pBIOS + *((CARD16 *)pRom) + 10;

        pBIOSInfo->BIOSDateMonth = ((*pRom++) - 48)*10;
        pBIOSInfo->BIOSDateMonth += (*pRom++) - 48;
        pRom++;
        pBIOSInfo->BIOSDateDay = ((*pRom++) - 48)*10;
        pBIOSInfo->BIOSDateDay += (*pRom++) - 48;
        pRom++;
        pBIOSInfo->BIOSDateYear = ((*pRom++) - 48)*10;
        pBIOSInfo->BIOSDateYear += (*pRom) - 48;
        xf86DrvMsg(pVia->pBIOSInfo->scrnIndex, X_INFO, "VGA BIOS Date: %d/%d/%d\n" ,
                   pBIOSInfo->BIOSDateMonth, pBIOSInfo->BIOSDateDay, pBIOSInfo->BIOSDateYear);
    }

    if (flags & VIAGETBIOSVER)
    {
        /* Get the start of biosver structure */
        pRom = pTable + VIA_BIOS_BIOSVER_POS;
        pRom = pBIOS + *((CARD16 *)pRom);

        /* The offset should be 44, but the actual image is less three char. */
        /* pRom += 44; */
        pRom += 41;

        pVia->pBIOSVer->CoreBase = *pRom++;
        pVia->pBIOSVer->ProductID = *pRom++;
        pVia->pBIOSVer->MajorVer = *pRom++;
        pVia->pBIOSVer->MinorVer = *pRom++;
    }

    xfree(pBIOS);
    return TRUE;
}

unsigned char BIOS_GetActiveDevice(ScrnInfoPtr pScrn)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    int RealOff;
    pointer page = NULL;
    unsigned char   ret = 0x0;

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "BIOS_GetActiveDevice\n"));

    /* STChen[2007.3.8]: For MAMM. */
    if (pVia->IsMAMMEnable)
        return 0xFF;
    
    page = xf86Int10AllocPages(pVia->pInt10, 1, &RealOff);
    if (!page)
        return 0xFF;
    pVia->pInt10->ax = 0x4F14;
    pVia->pInt10->bx = 0x0103;
    pVia->pInt10->cx = 0;
    pVia->pInt10->num = 0x10;
    xf86ExecX86int10(pVia->pInt10);

    if ((pVia->pInt10->ax & 0xFF) != 0x4F)
    {
        DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_ERROR, "BIOS Get Active Device fail!\n"));
        if (page)
            xf86Int10FreePages(pVia->pInt10, page, 1);
        return 0xFF;
    }

    if (page)
        xf86Int10FreePages(pVia->pInt10, page, 1);

    if (pVia->pInt10->cx & 0x01)
        ret = VIA_DEVICE_CRT1;
    if (pVia->pInt10->cx & 0x02)
        ret |= VIA_DEVICE_LCD;
    if (pVia->pInt10->cx & 0x04)
        ret |= VIA_DEVICE_TV;
    if (pVia->pInt10->cx & 0x20)
        ret |= VIA_DEVICE_DFP;

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Return Value Is: %u\n", ret));
    return ret;
}



Bool BIOS_GetBIOSDate(ScrnInfoPtr pScrn)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;
    int RealOff;
    pointer page = NULL;

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "BIOS_GetBIOSDate\n"));

    /* STChen[2007.3.8]: For MAMM. */
    if (pVia->IsMAMMEnable)
        return FALSE;
    
    page = xf86Int10AllocPages(pVia->pInt10, 1, &RealOff);
    if (!page)
        return FALSE;
    pVia->pInt10->ax = 0x4F14;
    pVia->pInt10->bx = 0x0100;
    pVia->pInt10->cx = 0;
    pVia->pInt10->dx = 0;
    pVia->pInt10->si = 0;
    pVia->pInt10->num = 0x10;
    xf86ExecX86int10(pVia->pInt10);

    if ((pVia->pInt10->ax & 0xff) != 0x4f)
    {
        DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_ERROR, "Get BIOS Date fail!\n"));
        if (page)
            xf86Int10FreePages(pVia->pInt10, page, 1);
        return FALSE;
    }

    pBIOSInfo->BIOSDateYear = ((pVia->pInt10->bx >> 8) - 48) + ((pVia->pInt10->bx & 0xFF) - 48)*10;
    pBIOSInfo->BIOSDateMonth = ((pVia->pInt10->cx >> 8) - 48) + ((pVia->pInt10->cx & 0xFF) - 48)*10;
    pBIOSInfo->BIOSDateDay = ((pVia->pInt10->dx >> 8) - 48) + ((pVia->pInt10->dx & 0xFF) - 48)*10;

    if (page)
        xf86Int10FreePages(pVia->pInt10, page, 1);

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "BIOS Release Date Is: %d/%d/%d\n"
     , pBIOSInfo->BIOSDateYear + 2000, pBIOSInfo->BIOSDateMonth, pBIOSInfo->BIOSDateDay));
    return TRUE;
}

int BIOS_GetBIOSVersion(ScrnInfoPtr pScrn)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    int RealOff, ret;
    pointer page = NULL;

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "BIOS_GetBIOSVersion\n"));

    /* STChen[2007.3.8]: For MAMM. */
    if (pVia->IsMAMMEnable)
        return 0xFFFF;
    
    page = xf86Int10AllocPages(pVia->pInt10, 1, &RealOff);
    if (!page)
        return 0xFFFF;
    pVia->pInt10->ax = 0x4F14;
    pVia->pInt10->bx = 0x0000;
    pVia->pInt10->cx = 0;
    pVia->pInt10->num = 0x10;
    xf86ExecX86int10(pVia->pInt10);

    if ((pVia->pInt10->ax & 0xff) != 0x4f)
    {
        DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_ERROR, "Get BIOS Version fail!\n"));
        if (page)
            xf86Int10FreePages(pVia->pInt10, page, 1);
        return 0xFFFF;
    }

    if (page)
        xf86Int10FreePages(pVia->pInt10, page, 1);

    ret = pVia->pInt10->bx & 0xFFFF;
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Return Value Is: %u\n", ret));
    return ret;
}


Bool BIOS_GetVESAModeInfo(ScrnInfoPtr pScrn, int ModeNo, int* Xres, int* Yres, int* bpp)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    int RealOff;
    pointer page = NULL;

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "BIOS_GetVESAModeInfo\n"));

    /* STChen[2007.3.8]: For MAMM. */
    if (pVia->IsMAMMEnable)
        return FALSE;
    
    page = xf86Int10AllocPages(pVia->pInt10, 1, &RealOff);
    if (!page)
        return FALSE;
    pVia->pInt10->ax = 0x4F14;
    pVia->pInt10->bx = 0x0302;
    pVia->pInt10->cx = ModeNo;
    pVia->pInt10->dx = 0;
    pVia->pInt10->di = 0;
    pVia->pInt10->si = 0;
    pVia->pInt10->num = 0x10;

    /* Real execution */
    xf86ExecX86int10(pVia->pInt10);

    if (pVia->pInt10->ax != 0x4F)
    {
        DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_ERROR, "BIOS Get VESA Mode Info fail!\n"));
        if (page)
            xf86Int10FreePages(pVia->pInt10, page, 1);
        return FALSE;
    }

    *Xres = (int)pVia->pInt10->cx;
    *Yres = (int)pVia->pInt10->bx;
    *bpp = (int)(pVia->pInt10->dx & 0xFF);

    if (page)
        xf86Int10FreePages(pVia->pInt10, page, 1);

    return TRUE;
}


int BIOS_QueryModeList(ScrnInfoPtr pScrn, int serno, int* ModeNo)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    int RealOff;
    pointer page = NULL;

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "BIOS_QueryModeList\n"));

    /* STChen[2007.3.8]: For MAMM. */
    if (pVia->IsMAMMEnable)
        return 0;
    
    page = xf86Int10AllocPages(pVia->pInt10, 1, &RealOff);
    if (!page)
        return 0;
    pVia->pInt10->ax = 0x4F14;
    pVia->pInt10->bx = 0x0202;
    pVia->pInt10->cx = 0;
    pVia->pInt10->dx = serno;
    pVia->pInt10->num = 0x10;

    /* Real execution */
    xf86ExecX86int10(pVia->pInt10);

    if ((pVia->pInt10->ax & 0xFF) != 0x4F)
    {
        DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_ERROR, "BIOS Query Mode List fail!\n"));
        if (page)
            xf86Int10FreePages(pVia->pInt10, page, 1);
        return 0;
    }

    if (page)
        xf86Int10FreePages(pVia->pInt10, page, 1);

    *ModeNo = (int)pVia->pInt10->cx;
    return (int)pVia->pInt10->dx;
}

int BIOS_QuerySupportRefreshRate(ScrnInfoPtr pScrn, int ModeNo, int serno, int* refIndex)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    int RealOff;
    pointer page = NULL;

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "BIOS_QuerySupportRefreshRate\n"));

    /* STChen[2007.3.8]: For MAMM. */
    if (pVia->IsMAMMEnable)
        return FALSE;
    
    page = xf86Int10AllocPages(pVia->pInt10, 1, &RealOff);
    if (!page)
        return FALSE;
    pVia->pInt10->ax = 0x4F14;
    pVia->pInt10->bx = 0x0201;
    pVia->pInt10->cx = ModeNo;
    pVia->pInt10->dx = serno;
    pVia->pInt10->di = 0;
    pVia->pInt10->num = 0x10;

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "ModeNo:%x  SerialNo:%d\n", ModeNo, serno));
    /* Real execution */
    xf86ExecX86int10(pVia->pInt10);

    if (pVia->pInt10->ax != 0x4F)
    {
        DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_ERROR, "BIOS Query Support Refresh Rate fail!\n"));
        if (page)
            xf86Int10FreePages(pVia->pInt10, page, 1);
        return FALSE;
    }

    *refIndex = (int)pVia->pInt10->bx;

    if (page)
        xf86Int10FreePages(pVia->pInt10, page, 1);

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "ModeNo:%x  SerialNo:%d\n", ModeNo, pVia->pInt10->dx));

    return (int)pVia->pInt10->dx;
}

Bool BIOS_SetActiveDevice(ScrnInfoPtr pScrn)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;
    VIAModeTablePtr pViaModeTable = pBIOSInfo->pModeTable;
    int RealOff;
    pointer page = NULL;

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "BIOS_SetActiveDevice\n"));

    /* STChen[2007.3.8]: For MAMM. */
    if (pVia->IsMAMMEnable)
        return FALSE;
    
    page = xf86Int10AllocPages(pVia->pInt10, 1, &RealOff);
    if (!page)
        return FALSE;
    pVia->pInt10->ax = 0x4F14;
    pVia->pInt10->bx = 0x8003;
    pVia->pInt10->cx = 0;
    pVia->pInt10->dx = 0;
    pVia->pInt10->di = 0;
    pVia->pInt10->num = 0x10;

    /* Set Active Device and Translate BIOS byte definition */
    if (pBIOSInfo->ActiveDevice & VIA_DEVICE_CRT1)
        pVia->pInt10->cx = 0x01;
    if (pBIOSInfo->ActiveDevice & VIA_DEVICE_LCD)
        pVia->pInt10->cx |= 0x02;
    if (pBIOSInfo->ActiveDevice & VIA_DEVICE_TV)
        pVia->pInt10->cx |= 0x04;
    if (pBIOSInfo->ActiveDevice & VIA_DEVICE_DFP)
        pVia->pInt10->cx |= 0x20;
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Active Device: %d\n",
                     pVia->pInt10->cx));

    /* Set Current mode */
    pVia->pInt10->dx = pViaModeTable->Modes[pBIOSInfo->mode].Mode;
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Mode Nums: %d\n",
                     pVia->pInt10->dx));

    /* Set Current Refresh rate */
    switch(pBIOSInfo->Refresh)
    {
        case 60:
            pVia->pInt10->di = 0;
            break;
        case 75:
            pVia->pInt10->di = 5;
            break;
        case 85:
            pVia->pInt10->di = 7;
            break;
        case 100:
            pVia->pInt10->di = 9;
            break;
        case 120:
            pVia->pInt10->di = 10;
            break;
        default:
            pVia->pInt10->di = 0;
            break;
    }
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Refresh Rate Index: %d\n",
                     pVia->pInt10->di));

    /* Real execution */
    xf86ExecX86int10(pVia->pInt10);

    if ((pVia->pInt10->ax & 0xFF) != 0x4F)
    {
        DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
                         "BIOS Set Active Device fail!\n"));
        if (page)
            xf86Int10FreePages(pVia->pInt10, page, 1);
        return FALSE;
    }

    if (page)
        xf86Int10FreePages(pVia->pInt10, page, 1);

    return TRUE;
}


Bool BIOS_SetFlatPanelState(ScrnInfoPtr pScrn, Bool expansion)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    int RealOff;
    pointer page = NULL;

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "BIOS_SetFlatPanelState: %s.\n",
                     expansion ? "Expand" : "Center"));

    /* STChen[2007.3.8]: For MAMM. */
    if (pVia->IsMAMMEnable)
        return FALSE;
    
    page = xf86Int10AllocPages(pVia->pInt10, 1, &RealOff);
    if (!page)
        return FALSE;
    pVia->pInt10->ax = 0x4F14;
    pVia->pInt10->bx = 0x0306;
    pVia->pInt10->cx = (0x80 | expansion);
    pVia->pInt10->num = 0x10;

    /* Real execution */
    xf86ExecX86int10(pVia->pInt10);

    if (pVia->pInt10->ax != 0x4F)
    {
        DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
                         "BIOS Set Flat Panel State fail!\n"));
        if (page)
            xf86Int10FreePages(pVia->pInt10, page, 1);
        return FALSE;
    }

    if (page)
        xf86Int10FreePages(pVia->pInt10, page, 1);

    return TRUE;
}

void VIADelayIn_usec(VIABIOSInfoPtr pBIOSInfo, int usec)
{
    VIABIOSInfoPtr  pVia = pBIOSInfo;
    volatile long   i;
    unsigned char   tmp;

    if (usec > 0)
    {
        for (i = usec * 4; i > 0; i--)
        {
            tmp = VGAIN8(0x3c3);
        }
    }
}

static void
xwrite_regx(struct io_reg *ioreg)
{
    int count;
    struct io_reg *ptr;

    for (count = 0, ptr = ioreg; ptr->port != -1; ptr++, count++)
	;

    write_regx(ioreg, count);
}

Bool
VIASetMode(VIABIOSInfoPtr pBIOSInfo)
{
    VIABIOSInfoPtr  pVia = pBIOSInfo;

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO,
		     "VIASetMode!!\n"));
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO,
		     "Active Device is 0x%x\n", pBIOSInfo->ActiveDevice));

    /* Turn off Screen */
    write_reg_mask(CR17, VIACR, 0x00, BIT7);

    /* Clean Second Path Status */
    if (!pBIOSInfo->SAMM || pBIOSInfo->FirstInit) {
	/*=* VansonWu [2004.6.7] Reset IGA2 channel before disable IGA2 channel
	     or it may cause some line garbage. *=*/
	/* RainLin [2005.11.22]: Modify for sequence issue. */
	DisableSecondDisplayChannel();
	write_reg_mask(CR6A, VIACR, 0x00, 0x3D);

	write_reg(CR6B, VIACR, 0x00);
	write_reg(CR6C, VIACR, 0x00);
	write_reg(CR79,VIACR,0x0);

	/*=* VansonWu [2004.5.28] Clear CR47[3,6-7]
		      (IGA1 Timing Plus 2, added in VT3259 A3 or later) *=*/
	if (pBIOSInfo->Chipset >= VIA_PM800) {
	    write_reg_mask(CR47, VIACR, 0x00, BIT7 + BIT6 + BIT3);
	}
    }

    /* CRT set mode */
    if ((pBIOSInfo->s3utility == FALSE) || (!pBIOSInfo->SAMM)) {
	int i;

	VGAIN8(VIAStatus);
	VGAOUT8(VIAAR, 0x00);

	/* RainLin [2007.01.19]: Temporary solution for the problem: on VT3364, the device on IGA2 will */
	/* display incorrect when the system resumes from suspends. Because we don't know why the value of */
	/* starting address register CR62 will be changed from 0x00 to 0xff once we set Misc register, */
	/* so we are forced to change the flow to set Misc register before common setting for video mode */
	/* temporarily. */
	/* Write Misc Register: */
	VGAOUT8(VIAWMisc, VPIT.Misc);

	/* Write Common Setting for Video Mode */
	switch (pBIOSInfo->Chipset) {
	    case VIA_CLE266:
		xwrite_regx(CLE266_ModeXregs);
		break;
	    case VIA_KM400:
		xwrite_regx(KM400_ModeXregs);
		break;
	    case VIA_K8M800:
	    case VIA_PM800:
		xwrite_regx(CN400_ModeXregs);
		break;
	    case VIA_P4M800PRO:
	    case VIA_K8M890:
	    case VIA_P4M890:
	    case VIA_P4M900:
		xwrite_regx(CN900_ModeXregs);
		/* 
		 * STChen[2007.3.8]: Under MAMM, If VIA is secondary, vga BIOS will not initial registers. 
		 * So we have to load initial table by ourself.
		 */
		if (pBIOSInfo->IsMAMMEnable) {
		    xwrite_regx(CN900_ModeXregs_FOR_MAMM);
		}
		break;
	    case VIA_CX700:
	    case VIA_CN750:
		xwrite_regx(CX700_ModeXregs);
		break;
	}

	VIADisableLCD(pBIOSInfo, &(pBIOSInfo->LVDSSettingInfo));

	if (pBIOSInfo->LVDSSettingInfo2.ChipID) {
	    VIADisableLCD(pBIOSInfo, &(pBIOSInfo->LVDSSettingInfo2));
	}

	/* This must be done without any condtion.
	   On VT3324, we need to disable TV to switch primary DAC from TV to CRT
	   when we switch device from TV to CRT. */        
	VIADisableTV(pBIOSInfo, &(pBIOSInfo->TVSettingInfo));

	/* Even if TV2 is not active, TV2 still have to be disable. 
	   On dual TV case, if we change active state (TV+TV2 -> CRT+TV),       
	   the primary DAC need to be switched to CRT. */
	VIADisableTV(pBIOSInfo, &(pBIOSInfo->TVSettingInfo2));

	VIADisableDFP(pBIOSInfo);

	/* Fill VPIT Parameters */
	/* Write Sequencer */
	for ( i=1; i<=StdSR ; i++ ) {
	    write_reg((CARD8)i, VIASR, VPIT.SR[i - 1]);
	}

	/* Write Graphic Controller */
	for ( i=0; i<StdGR ; i++ ) {
	    write_reg((CARD8)i, VIAGR, VPIT.GR[i]);
	}

	/* Write Attribute Controller */
	for (i = 0; i < StdAR ; i++) {
	    VGAIN8(VIAStatus);
	    VGAOUT8(VIAAR, i);
	    VGAOUT8(VIAAR, VPIT.AR[i]);
	}

	VGAIN8(VIAStatus);
	VGAOUT8(VIAAR, 0x20);
    }

    /* STChen[2006.8.9]: On SAMM case(LCD+any device), if we press lcd expand checkbox,
       It will call this unnecessary CRT setmode and modify the IGA1's timing using default value.
       So we have to skip this setmode operation under the following condition. */       
    if (!((pBIOSInfo->SAMM) &&
	  (pBIOSInfo->PrimaryDevice == VIA_DEVICE_LCD) &&
	  (pBIOSInfo->s3utility == TRUE))) {
	VIACRTSetMode(pBIOSInfo, pBIOSInfo->CRTSettingInfo.ModeIndex, IGA1);
    }

    if (pBIOSInfo->TwoCRT) {
	VIACRTSetMode(pBIOSInfo, pBIOSInfo->CRTSettingInfo.ModeIndex, IGA1);
    }

    /* DVI set mode */
    if (pBIOSInfo->ActiveDevice & VIA_DEVICE_DFP) {
	VIADVISetMode(pBIOSInfo);
    }

    /* TV set mode */
    if (pBIOSInfo->ActiveDevice & VIA_DEVICE_TV) { 
	VIATVSetMode(pBIOSInfo, &(pBIOSInfo->TVSettingInfo));        
    }

    if (pBIOSInfo->ActiveDevice & VIA_DEVICE_TV2) {        
	VIATVSetMode(pBIOSInfo, &(pBIOSInfo->TVSettingInfo2));        
    }

    /* LCD set mode */
    if (pBIOSInfo->ActiveDevice & VIA_DEVICE_LCD) {
	VIALCDSetMode(pBIOSInfo, &(pBIOSInfo->LVDSSettingInfo));
    }

    if (pBIOSInfo->ActiveDevice & VIA_DEVICE_LCD2) {
	VIALCDSetMode(pBIOSInfo, &(pBIOSInfo->LVDSSettingInfo2));
    }

    /* Patch IGA1's quad-word count and offset for virtual desktop. */     
    if (!((pBIOSInfo->SAMM) &&
	  ((pBIOSInfo->PrimaryDevice == VIA_DEVICE_LCD) ||
	   (pBIOSInfo->PrimaryDevice == VIA_DEVICE_LCD2)) &&
	  (pBIOSInfo->s3utility == TRUE))) {
	VIAPrimaryDisplayOffsetPatch(pBIOSInfo);  
    }

    /*MarkTu[2006.09.25]:Need to patch for mode such as 1400x1050*/
    VIAPitchAlignmentPatch(pBIOSInfo);
    /* We need to do this patch when IGA2 is in use. And because LCD always uses IGA2, */
    /* so we should do this patch for LCD, too. */
    if ((pBIOSInfo->DuoView) ||
	((pBIOSInfo->ActiveDevice & VIA_DEVICE_LCD) &&
	 (pBIOSInfo->LVDSSettingInfo.IGAPath == IGA2)) ||
	((pBIOSInfo->ActiveDevice & VIA_DEVICE_LCD2) &&
	 (pBIOSInfo->LVDSSettingInfo2.IGAPath == IGA2))) {
	/* RainLin [2005.11.29]: Patch for using virtual desktop. */
	VIASecondDisplayOffsetPatch(pBIOSInfo);
    }

    /* Enable MMIO & PCI burst (1 wait state) */
    write_reg_mask(SR1A, VIASR, BIT2, BIT2);

    /* Enable modify CRTC starting address */
    write_reg_mask(CR11, VIACR, 0x00, BIT7);

    if (pBIOSInfo->FirstInit) {        
	/* Clear Memory */
	memset(pBIOSInfo->FBBase, 0x00,
	       pBIOSInfo->videoRambytes - VIA_FB_CURSOR_SIZE);
	/*pBIOSInfo->FirstInit = FALSE;*/
    }

    VIAEnabledPrimaryExtendedFIFO(pBIOSInfo);
    /*Single LCD use iga2, Cle266 need to set fifo*/
    if (pBIOSInfo->DuoView || ((VIA_CLE266 == pBIOSInfo->Chipset) &&
			       (VIA_DEVICE_LCD == pBIOSInfo->ActiveDevice))) {
	/* Set Extended FIFO & Expire Number */
	VIAEnabledSecondaryExtendedFIFO(pBIOSInfo);
    }

    /* Enable CRT Controller (3D5.17 Hardware Reset) */
    write_reg_mask(CR17, VIACR, BIT7, BIT7);

    /* CRT */
    if (pBIOSInfo->ActiveDevice & VIA_DEVICE_CRT1) {
	VIACRTEnable();

	/* CRT only */
	if (pBIOSInfo->ActiveDevice == VIA_DEVICE_CRT1) {
	    /* DVP0 and DVP1 port off */
	    write_reg_mask(SR1E, VIASR, 0x00, 0xF0);

	    /*2nd CRT use DFP port*/
            if (!pBIOSInfo->TwoCRT) {
	        /* DFPH and DFPL port off */
		write_reg_mask(SR2A, VIASR, 0x00, 0x0F);
	    }       
	}
    }
    else {
	if (pBIOSInfo->s3utility == FALSE)
	    VIACRTDisable();
    }

    /*=* For Advantech Project, support 2 LCD SAMM.
	 If mode is 640x480, disable IGA1 Horizontal and Vertical border. *=*/
    if ((pBIOSInfo->TwoLCD) &&
	(pBIOSInfo->LVDSSettingInfo.ModeIndex == M640x480)) {
	write_reg(CR03, VIACR, 0x83);
	write_reg(CR15, VIACR, 0xDF);
	write_reg(CR16, VIACR, 0x0C);
	/*=* Enable DVP1 *=*/
	write_reg_mask(SR1E, VIASR, 0xC0, 0xC0);
    }

    /* Open Screen */
    (void)VGAIN8(0x3da);
    VGAOUT8(0x3c0, 0x20);

    if (pBIOSInfo->ActiveDevice & VIA_DEVICE_LCD) {
	VIAEnableLCD(pBIOSInfo, &(pBIOSInfo->LVDSSettingInfo));
    }

    if (pBIOSInfo->ActiveDevice & VIA_DEVICE_LCD2) {
	VIAEnableLCD(pBIOSInfo, &(pBIOSInfo->LVDSSettingInfo2));
    }

    /* RainLin [2007.03.22]: When enable LCD only, the bit of Spread Spectrum (SR1E[3]) and */
    /* Spectrum IO selected bit (SR2C[0]) should be enabled, or the screen will scramble. */
    /*
    if (pBIOSInfo->ActiveDevice == VIA_DEVICE_LCD) {
	write_reg_mask(SR1E, VIASR, 0x08, BIT3);
	write_reg_mask(SR2C, VIASR, 0x01, BIT0);
    }
    else {
	write_reg_mask(SR1E, VIASR, 0x00, BIT3);
	write_reg_mask(SR2C, VIASR, 0x00, BIT0);
    }
    */

    /* Do TV Software Reset: */
    if (pBIOSInfo->ActiveDevice & VIA_DEVICE_TV) {
	VIADoTVSWReset(pBIOSInfo, &(pBIOSInfo->TVSettingInfo));                
    }

    if (pBIOSInfo->ActiveDevice & VIA_DEVICE_TV2) {
	VIADoTVSWReset(pBIOSInfo, &(pBIOSInfo->TVSettingInfo2));
    }

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO,
		     "-- VIASetMode Done!\n"));

    return (TRUE);
}

void
VIACRTSetMode(VIABIOSInfoPtr pBIOSInfo, int vmode_index, int set_iga)
{
    struct VideoModeTable *VideoMode;
    int i, port;
    CARD8 value, index, mask;
    struct crt_mode_table *crt_timing;
    struct patch_table *patch;

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIACRTSetMode!!\n"));

    VideoMode = &CLE266Modes[SearchModeSetting(vmode_index)];
    crt_timing = VideoMode->crtc;

    /* Write CRTC */
    fill_crtc_timing(pBIOSInfo, crt_timing, VideoMode->mode_array,
		     (pBIOSInfo->bitsPerPixel >> 3), set_iga);

    /* Update Patch Register */
    if ((pBIOSInfo->Chipset == VIA_CLE266) ||
	(pBIOSInfo->Chipset == VIA_KM400)) {
	for (patch = res_patch_table; patch->mode_index != 0xFFFFFFFF; patch++) {
	    if (patch->mode_index == vmode_index) {
		for (i = 0; i < patch->table_length; i++) {
		    index = patch->io_reg_table[i].index;
		    port = patch->io_reg_table[i].port;
		    value = patch->io_reg_table[i].value;
		    mask = patch->io_reg_table[i].mask ;
		    write_reg_mask(index, port, value, mask);
		}
	    }
	}
    }

    /* If K8M800, enable Prefetch Mode. */
    if ((pBIOSInfo->Chipset == VIA_K8M800) ||
	(pBIOSInfo->Chipset == VIA_K8M890)) {
	write_reg_mask(CR33, VIACR, BIT3, BIT3);
    }

    if (!((pBIOSInfo->s3utility == TRUE) &&
	  (pBIOSInfo->SAMM || pBIOSInfo->DuoView))) {
	VIASetOutputPath(pBIOSInfo, VIA_DEVICE_CRT1,set_iga, 0);
    }

    /*support Dual CRT with TV card vt1625*/        
    if (pBIOSInfo->TwoCRT && (!pBIOSInfo->MobileFamily)) {	
	if (!(pBIOSInfo->Chipset == VIA_CLE266)) {
	    write_reg_mask(SR2A, VIASR, 0x0C, BIT3+BIT2);
	    load_1625_CRT_regs(pBIOSInfo);  	
	}
    }
}

void VIACRTDisable(void)
{
    write_reg_mask(CR36, VIACR, BIT5+BIT4, BIT5+BIT4);
}

void VIACRTEnable(void)
{
    write_reg_mask(CR36, VIACR, 0x0, BIT5+BIT4);
}

struct display_timing VIAGetTimingValue(VIABIOSInfoPtr pBIOSInfo, struct crt_mode_table *crt_table)
{
    struct VideoModeTable   *video_mode;
    struct display_timing   crt_reg;
    int                     i;
    int                     index = 0;

    video_mode = &CLE266Modes[SearchModeSetting(pBIOSInfo->resMode2)];

    for(i=0;i<video_mode->mode_array;i++)
    {
        index = i;

        if (crt_table[i].refresh_rate == pBIOSInfo->FoundRefresh)
            break;
    }
    crt_reg = crt_table[index].crtc;

    return(crt_reg);
}

void VIASetOutputPath(VIABIOSInfoPtr pBIOSInfo, int device, int set_iga, int output_interface)
{
    switch(device)
    {
        case VIA_DEVICE_CRT1:
            VIASetCRTOutputPath(set_iga);
            break;
        case VIA_DEVICE_TV:
            VIASetTVOutputPath(pBIOSInfo, set_iga, output_interface);
            break;
        case VIA_DEVICE_DFP:
            VIASetDVIOutputPath(pBIOSInfo, set_iga, output_interface);
            break;
        case VIA_DEVICE_LCD:
            VIASetLCDOutputPath(pBIOSInfo, set_iga, output_interface);
            break;
    }
}

void VIASetCRTOutputPath(int set_iga)
{

    switch(set_iga)
    {
        case IGA1:
            write_reg_mask(SR16, VIASR, 0x00, BIT6);
            break;
        case IGA2:
        case IGA1_IGA2:
            /* RainLin [2005.11.22]: Modify for sequence issue. */
            EnableSecondDisplayChannel();
            
            write_reg_mask(SR16, VIASR, 0x40, BIT6);
            if (set_iga == IGA1_IGA2)
                write_reg_mask(CR6B, VIACR, 0x08, BIT3);
            break;
    }
}

void VIASetDisplayChannel(VIABIOSInfoPtr pBIOSInfo)
{
    if (((pBIOSInfo->ActiveDevice & VIA_DEVICE_LCD) && pBIOSInfo->LVDSSettingInfo.IsDualEdge) ||
        ((pBIOSInfo->ActiveDevice & VIA_DEVICE_LCD2) && pBIOSInfo->LVDSSettingInfo2.IsDualEdge))
    {
        /* For dual channel LCD: */
        /* Set to Dual LVDS channel. */
        write_reg_mask(CRD2, VIACR, 0x20, BIT4+BIT5);
    }
    /*We should  use ActualActiveDevice only when SAMM case, because ActualActiveDevice not always
      *updated, such as when utility change activedevice, load activedevice from usersetting, and it's not good 
      *to maintain two ActualActiveDevice(one in pVIA), this function's code just work, but it's obscure, need be
      *refined to clarify between SAMM and not SAMM case */
    else if(pBIOSInfo->SAMM && pBIOSInfo->ActualActiveDevice == (VIA_DEVICE_DFP | VIA_DEVICE_LCD))
    {
        /* For LCD+DFP: */
        /* Set to LVDS1 + TMDS channel. */
        write_reg_mask(CRD2, VIACR, 0x10, BIT4+BIT5);
    }
    else if(pBIOSInfo->ActiveDevice == (VIA_DEVICE_DFP | VIA_DEVICE_LCD))
    {
        /* For LCD+DFP: */
        /* Set to LVDS1 + TMDS channel. */
        write_reg_mask(CRD2, VIACR, 0x10, BIT4+BIT5);
    }
    else if ((pBIOSInfo->ActiveDevice & VIA_DEVICE_DFP) && (!(pBIOSInfo->ActiveDevice & VIA_DEVICE_LCD)))
    {
        /* Set to single TMDS channel. */
        write_reg_mask(CRD2, VIACR, 0x30, BIT4+BIT5);
    }
    else
    {
        /* Set to LVDS0 + LVDS1 channel. */
        write_reg_mask(CRD2, VIACR, 0x00, BIT4+BIT5);
    }
}

/*
    S.T.Chen[2006.6.26]: If monitor's max support resolution is a strange mode that
    we don't support, we have to correct it. Find a new mode that we support to replace it.

*/
Bool VIACorrectMonitorSize(int* MonitorH, int* MonitorV)
{   
    int i = 0;

    while (VIASupportMode[i].ModeIndex > -1)
    {
        if (*MonitorH < VIASupportMode[i].HActive)
        {
            i--;
            break;
        }
        else if ((*MonitorH == VIASupportMode[i].HActive)
              && (*MonitorV < VIASupportMode[i].VActive))
        {
            i--;
            break;
        }
        else if ((*MonitorH == VIASupportMode[i].HActive)
              && (*MonitorV == VIASupportMode[i].VActive))
        {
            break;
        }
        else
        {
            i++;
        }
    }
    
    if ((VIASupportMode[i].ModeIndex < 0) || (i < 0))
    {
        return FALSE;
    }
    else
    {
        *MonitorH = VIASupportMode[i].HActive;
        *MonitorV = VIASupportMode[i].VActive;        
        return TRUE;
    }
}


/*
*   STChen[2007.4.16]: Attach a new mode to the modeline.
*   So we don't need to add modeline in the config file.
*/
void ViaModesAttachHelper(ScrnInfoPtr pScrn, MonPtr monitorp, DisplayModePtr Modes)
{
    DisplayModePtr mode;
    DisplayModePtr last = monitorp->Last;
    int i;

    for (i = 0; Modes[i].name; i++)
    {        
        mode = xnfalloc(sizeof(DisplayModeRec));
        memcpy(mode, &Modes[i], sizeof(DisplayModeRec));
        mode->name = xnfstrdup(Modes[i].name);
        
        if (last)
        {
            mode->prev = last;
            last->next = mode;
        }
        else
        { 
            /* this is the first mode */
            monitorp->Modes = mode;
            mode->prev = NULL;
        }
        last = mode;
    }
    
    monitorp->Last = last;
}


Bool VIAEnumModes(VIABIOSInfoPtr pBIOSInfo, int IndexOfTable, VIASUPPORTMODE *pVIASupportMode, Bool *pIsSupported)
{
    int ItemNum;
    
    ItemNum = sizeof(VIASupportMode) / sizeof(VIASUPPORTMODE);

    *pIsSupported = FALSE;

    if (IndexOfTable < ItemNum)
    {
        memcpy(pVIASupportMode, &VIASupportMode[IndexOfTable], sizeof(VIASUPPORTMODE));

        /* Validate for TV: */
        if (pBIOSInfo->ActiveDevice & VIA_DEVICE_TV)
        {
            if (VIAIsTVMode(pBIOSInfo, pVIASupportMode->ModeIndex, &(pBIOSInfo->TVSettingInfo)))
            {
                if ((pVIASupportMode->ModeIndex == M720x480) &&
                    ((pBIOSInfo->TVSettingInfo.TVType == TVTYPE_PAL) || (pBIOSInfo->TVSettingInfo.TVType == TVTYPE_576P)))                    
                {
                    *pIsSupported = FALSE;
                }
                else if ((pVIASupportMode->ModeIndex == M720x576) &&
                         ((pBIOSInfo->TVSettingInfo.TVType == TVTYPE_NTSC) || (pBIOSInfo->TVSettingInfo.TVType == TVTYPE_480P)))
                {
                    *pIsSupported = FALSE;
                }
                else if ((pVIASupportMode->ModeIndex == M1280x720) && (pBIOSInfo->TVSettingInfo.TVType != TVTYPE_720P))
                {
                    *pIsSupported = FALSE;
                }
                else if ((pVIASupportMode->ModeIndex == M1920x1080) && (pBIOSInfo->TVSettingInfo.TVType != TVTYPE_1080I))
                {
                    *pIsSupported = FALSE;
                }
                else
                {
                    *pIsSupported = TRUE;
                }
            }
        }
        
        /* Validate for CRT: */
        if ((pBIOSInfo->ActiveDevice & VIA_DEVICE_CRT1) || (pBIOSInfo->ActiveDevice & VIA_DEVICE_CRT2))
        {
            if ((pBIOSInfo->CRTSettingInfo.MonitorSizeH >= pVIASupportMode->HActive) &&
                (pBIOSInfo->CRTSettingInfo.MonitorSizeV >= pVIASupportMode->VActive))
            {
                *pIsSupported = TRUE;
            }
        }

        /* Validate for DVI: */
        if (pBIOSInfo->ActiveDevice & VIA_DEVICE_DFP)
        {
            if ((pBIOSInfo->TMDSSettingInfo.PanelSizeH >= pVIASupportMode->HActive) &&
                (pBIOSInfo->TMDSSettingInfo.PanelSizeV >= pVIASupportMode->VActive))
            {
                *pIsSupported = TRUE;
            }
        }

        /* Validate for LCD: */
        if (pBIOSInfo->ActiveDevice & VIA_DEVICE_LCD)
        {
            if ((pBIOSInfo->LVDSSettingInfo.PanelSizeX >= pVIASupportMode->HActive) &&
                (pBIOSInfo->LVDSSettingInfo.PanelSizeY >= pVIASupportMode->VActive))
            {
                *pIsSupported = TRUE;
            }
        }

        /* Don't support if mode size is larger than virtual size. */
        if (pVIASupportMode->HActive >= pBIOSInfo->SaveVirtualX)
        {
            if (pVIASupportMode->VActive > pBIOSInfo->SaveVirtualY)
            {
                *pIsSupported = FALSE;
            }
        }
        
        return TRUE;
    }

    return FALSE;
}


void VIAEnumRefreshRate(int ModeIndex, int RefreshRateNum, int *pRefreshRate)
{
    struct VideoModeTable   *VideoMode;
    
    VideoMode = &CLE266Modes[SearchModeSetting(ModeIndex)];

    if (RefreshRateNum < VideoMode->mode_array)
    {
        *pRefreshRate = VideoMode->crtc[RefreshRateNum].refresh_rate;
    }
    else
    {
        *pRefreshRate = 0;
    }
}

/*
*   Purpose: Set DPA value to register.
*/
void VIASetDPA_Gfx(int DIPort, GFX_DPA_VALUE_PTR pGfxDPASetting)
{
    DEBUG(ErrorF("VIASetDPA_Gfx, Use Port %x.\n", DIPort));
    
    switch (DIPort)
    {
        case VIA_DI_DVP0:
        {
            /* DVP0 Clock Polarity and Adjust: */
            write_reg_mask(CR96, VIACR, pGfxDPASetting->DVP0, 0x0F);

            /* DVP0 Clock and Data Pads Driving: */
            write_reg_mask(SR1E, VIASR, pGfxDPASetting->DVP0ClockDri_S, BIT2);
            write_reg_mask(SR2A, VIASR, pGfxDPASetting->DVP0ClockDri_S1, BIT4);
            write_reg_mask(SR1B, VIASR, pGfxDPASetting->DVP0DataDri_S, BIT1);
            write_reg_mask(SR2A, VIASR, pGfxDPASetting->DVP0DataDri_S1, BIT5);

            DEBUG(ErrorF("DVP0 = %x.\n", pGfxDPASetting->DVP0));
            DEBUG(ErrorF("DVP0ClockDri_S = %x.\n", pGfxDPASetting->DVP0ClockDri_S));
            DEBUG(ErrorF("DVP0ClockDri_S1 = %x.\n", pGfxDPASetting->DVP0ClockDri_S1));
            DEBUG(ErrorF("DVP0DataDri_S = %x.\n", pGfxDPASetting->DVP0DataDri_S));
            DEBUG(ErrorF("DVP0DataDri_S1 = %x.\n", pGfxDPASetting->DVP0DataDri_S1));
            break;
        }

        case VIA_DI_DVP1:
        {
            /* DVP1 Clock Polarity and Adjust: */
            write_reg_mask(CR9B, VIACR, pGfxDPASetting->DVP1, 0x0F);

            /* DVP1 Clock and Data Pads Driving: */
            write_reg_mask(SR65, VIASR, pGfxDPASetting->DVP1Driving, 0x0F);
            
            DEBUG(ErrorF("DVP1 = %x.\n", pGfxDPASetting->DVP1));
            DEBUG(ErrorF("DVP1Driving = %x.\n", pGfxDPASetting->DVP1Driving));
            break;
        }

        case VIA_DI_DFPHIGH:
        {
            write_reg_mask(CR97, VIACR, pGfxDPASetting->DFPHigh, 0x0F);
            DEBUG(ErrorF("DFPHigh = %x.\n", pGfxDPASetting->DFPHigh));
            break;
        }

        case VIA_DI_DFPLOW:
        {
            write_reg_mask(CR99, VIACR, pGfxDPASetting->DFPLow, 0x0F);
            DEBUG(ErrorF("DFPLow = %x.\n", pGfxDPASetting->DFPLow));
            break;
        }

        case VIA_DI_DFPHIGHLOW:
        {
            write_reg_mask(CR97, VIACR, pGfxDPASetting->DFPHigh, 0x0F);
            write_reg_mask(CR99, VIACR, pGfxDPASetting->DFPLow, 0x0F);
            DEBUG(ErrorF("DFPHigh = %x.\n", pGfxDPASetting->DFPHigh));
            DEBUG(ErrorF("DFPLow = %x.\n", pGfxDPASetting->DFPLow));
            break;
        }
    }
}




