/*
 * Copyright 1998-2007 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2007 S3 Graphics, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

/*************************************************************************
 *
 *  File:       via_cursor.c
 *  Content:    Hardware cursor support for VIA/S3G UniChrome
 *
 ************************************************************************/

#include "via_driver.h"
#include "close/via.h"

static void VIALoadCursorImage(ScrnInfoPtr pScrn, unsigned char *src);
void VIASetCursorPosition(ScrnInfoPtr pScrn, int x, int y);
static void VIASetCursorColors(ScrnInfoPtr pScrn, int bg, int fg);
static Bool VIAUseHWCursor(ScreenPtr pScreen, CursorPtr pCurs);
/*cliff add forARGB cursor using hardware icon after starting X*/
static Bool VIAHiUseHWCursorARGB(ScreenPtr pScreen,CursorPtr pCursor);
static void VIAHiLoadCursorARGB (ScrnInfoPtr pScrn, CursorPtr pCurs);
/*
 * Read/write to the DAC via MMIO
 */

#define inCRReg(reg)        (VGAHWPTR(pScrn))->readCrtc(VGAHWPTR(pScrn), reg)
#define outCRReg(reg, val)  (VGAHWPTR(pScrn))->writeCrtc(VGAHWPTR(pScrn), reg, val)
#define inStatus1()         (VGAHWPTR(pScrn))->readST01(VGAHWPTR(pScrn))

/*cliff :change color cursor size from 32x32 to 64x64*/
Bool CallWhileARGB = FALSE;

void VIAARGBCursorInit(ScrnInfoPtr pScrn)
{
    VIAPtr pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;
    CARD32 dwHiControl;
    
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIAARGBCursorInit\n"));

#ifdef ARGB_CURSOR
    if( pBIOSInfo->ARGB_cursor)
    {
        switch(pBIOSInfo->Chipset )
        {
            case VIA_CX700:
            case VIA_CN750:
            case VIA_P4M890:
            case VIA_P4M900:
                if(!pVia->IsSecondary)
                {
                    VIASETREG(PRIM_HI_FBOFFSET, pVia->CursorStart);
                    
                    /* set 0 as transparent color key */
                    VIASETREG(PRIM_HI_TRANSCOLOR, 0);
                    VIASETREG(PRIM_HI_FIFO,0x0D000D0F);
                    VIASETREG(PRIM_HI_INVTCOLOR, 0X00FFFFFF);
                    VIASETREG(V327_HI_INVTCOLOR, 0X00FFFFFF);
                    dwHiControl = VIAGETREG(PRIM_HI_CTRL);
                    
                    /* Turn cursor off. */
                    VIASETREG(PRIM_HI_CTRL, dwHiControl & 0xFFFFFFFA);                    
                }
                else
                {
                    /* two hi engine point to same memory */
                    VIASETREG(PRIM_HI_FBOFFSET, pScrn->fbOffset + pVia->CursorStart);
                    VIASETREG(HI_FBOFFSET, pScrn->fbOffset + pVia->CursorStart);
                    
                    /* set 0 as transparent color key */
                    VIASETREG(HI_TRANSPARENT_COLOR, 0);
                    VIASETREG(HI_INVTCOLOR, 0X00FFFFFF);
                    VIASETREG(ALPHA_V3_PREFIFO_CONTROL, 0xE0000);
                    VIASETREG(ALPHA_V3_FIFO_CONTROL, 0xE0F0000);
                    dwHiControl = VIAGETREG(HI_CONTROL);
                    
                    /* Turn cursor off. */
                    VIASETREG(HI_CONTROL, dwHiControl & 0xFFFFFFFA);
                }

                break;
                
            default:               
                VIASETREG(HI_FBOFFSET, pScrn->fbOffset + pVia->CursorStart);
                VIASETREG(HI_TRANSPARENT_COLOR, 0);
                VIASETREG(HI_INVTCOLOR, 0X00FFFFFF);
                VIASETREG(ALPHA_V3_PREFIFO_CONTROL, 0xE0000);
                VIASETREG(ALPHA_V3_FIFO_CONTROL, 0xE0F0000);
                dwHiControl = VIAGETREG(HI_CONTROL);

                /* Turn cursor off. */
                VIASETREG(HI_CONTROL, dwHiControl & 0xFFFFFFFA);                         
        }
    }
#endif    
}

Bool
VIAHWCursorInit(ScreenPtr pScreen)
{
    ScrnInfoPtr pScrn = xf86Screens[pScreen->myNum];
    VIAPtr pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;
    xf86CursorInfoPtr infoPtr;

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIAHWCursorInit\n"));
    infoPtr = xf86CreateCursorInfoRec();
    if (!infoPtr)
        return FALSE;

    pVia->CursorInfoRec = infoPtr;

    infoPtr->MaxWidth = MAX_CURS;
    infoPtr->MaxHeight = MAX_CURS;
    infoPtr->Flags = HARDWARE_CURSOR_SOURCE_MASK_INTERLEAVE_64 |
                     HARDWARE_CURSOR_AND_SOURCE_WITH_MASK |
                     /*HARDWARE_CURSOR_SWAP_SOURCE_AND_MASK |*/
                     HARDWARE_CURSOR_TRUECOLOR_AT_8BPP |
                     HARDWARE_CURSOR_INVERT_MASK |
                     HARDWARE_CURSOR_BIT_ORDER_MSBFIRST|
                     0;
    
    infoPtr->SetCursorColors = VIASetCursorColors;
    infoPtr->SetCursorPosition = VIASetCursorPosition;
    infoPtr->LoadCursorImage = VIALoadCursorImage;
    infoPtr->HideCursor = VIAHideCursor;
    infoPtr->ShowCursor = VIAShowCursor;
    infoPtr->UseHWCursor = VIAUseHWCursor;

  
#ifdef ARGB_CURSOR
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "We will use HWCursor ARGB\n"));
    
    /*cliff add to enable ARGB cursor */
    infoPtr->UseHWCursorARGB = VIAHiUseHWCursorARGB;
    infoPtr->LoadCursorARGB = VIAHiLoadCursorARGB;
    pBIOSInfo->ARGB_cursor = TRUE;
#endif
   
    /*for NoAccel case*/
    if (!pVia->CursorStart)
    {
        if(!pVia->IsSecondary)
        {
            if(!pVia->HasSecondary)
            {  /*single case*/
                pVia->CursorStart = pVia->FBFreeEnd - VIA_CURSOR_SIZE;
                pVia->FBFreeEnd -= VIA_CURSOR_SIZE;
            }
        }
        else 
        { /*samm case:allocate  cursor memory in the end of secondary video memory  */
           pVia->CursorStart = pVia->FBFreeEnd - VIA_CURSOR_SIZE;
           pVia->FBFreeEnd -= VIA_CURSOR_SIZE;
         }
    }

    VIAARGBCursorInit(pScrn);

    return xf86InitCursor(pScreen, infoPtr);
}


/* for 64M address limitation ,never use mono hardware cursor!. */
static Bool VIAUseHWCursor(ScreenPtr pScreen, CursorPtr pCurs)
{
    return FALSE;
}


void VIAShowCursor(ScrnInfoPtr pScrn)
{
    VIAPtr pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;
    CARD32 dwCursorMode;

    if( pBIOSInfo->ARGB_cursor&&(!CallWhileARGB ))
    {      
        switch(pBIOSInfo->Chipset )
        {
            case VIA_CX700:
            case VIA_CN750:
            case VIA_P4M890:
            case VIA_P4M900:
                 /* Turn on Hardware icon Cursor */
                 if(!pVia->IsSecondary)
                 {
                     
                     VIASETREG(PRIM_HI_CTRL, 0x76000005);
                 }
                 else
                 {
                     
                     VIASETREG(HI_CONTROL, 0xf6000005);
                 }
                 break;
            
            default:
                /* Turn on Hardware icon Cursor */
                if(!pVia->IsSecondary)
                {
                    /*enableV4 Window Pre-fetch [bit 30]for solving VT3336  bug: when do some actions such as play video cursor will jump*/
                    VIASETREG(HI_CONTROL, 0x76000005);
                }
                else
                {
                    
                    VIASETREG(HI_CONTROL, 0xf6000005);
                }
        }
    }
    else
    {
        if ((pBIOSInfo->Chipset == VIA_CX700) || (pBIOSInfo->Chipset == VIA_CN750))
        {
            /* Set color to white and black. */
            /* Use 30 bits [29:0] to specify HW cursor background/foreground color. */
            VIASetCursorColors(pScrn, 0x3fffffff, 0);
        }

        dwCursorMode = VIAGETREG(VIA_REG_CURSOR_MODE);
        dwCursorMode &=~0x80000003;
        /* Turn on Hardware Cursor */
        if(!pVia->IsSecondary)
        VIASETREG(VIA_REG_CURSOR_MODE, dwCursorMode | 0x1);
        else
        VIASETREG(VIA_REG_CURSOR_MODE, dwCursorMode | 0x80000001);
    }
}


void VIAHideCursor(ScrnInfoPtr pScrn)
{
    VIAPtr pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;
    CARD32 dwCursorMode;
    CARD32 dwHiControl;
    if( pBIOSInfo->ARGB_cursor&&(!CallWhileARGB ))
    { 
        switch(pVia->pBIOSInfo->Chipset )
        {
            case VIA_CX700:
            case VIA_CN750:
            case VIA_P4M890:
            case VIA_P4M900:
                if(!pVia->IsSecondary)
                {
                    dwHiControl = VIAGETREG(PRIM_HI_CTRL);
                    /* Turn hardware icon cursor off. */
                    VIASETREG(PRIM_HI_CTRL, dwHiControl & 0xFFFFFFFA);
                    break;
                }
                else
                {
                    dwHiControl = VIAGETREG(HI_CONTROL);
                    /* Turn hardware icon cursor off. */
                    VIASETREG(HI_CONTROL, dwHiControl & 0xFFFFFFFA);
                break;
                }
            default:
                dwHiControl = VIAGETREG(HI_CONTROL);
                /* Turn hardware icon cursor off. */
                VIASETREG(HI_CONTROL, dwHiControl & 0xFFFFFFFA);
        }
    }
    else
    { 
        dwCursorMode = VIAGETREG(VIA_REG_CURSOR_MODE);
        /* Turn cursor off. */
        VIASETREG(VIA_REG_CURSOR_MODE, dwCursorMode & 0xFFFFFFFE);

        /* after showing X shape cursor switch to use hardware icon  engine*/
        if( pBIOSInfo->ARGB_cursor)
            CallWhileARGB = FALSE;
    }
}


static void VIALoadCursorImage(ScrnInfoPtr pScrn, unsigned char* src)
{
    VIAPtr pVia = VIAPTR(pScrn);
    CARD32 dwCursorMode;
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;

    WaitIdle();
   
    /*add   bool variable CallWhileARGB  because:  while use A_RGB(Hardware icon) cursor ,
     *when start X-window ,X will call this function first to show 'X'shape cursor,
     *so we should enable hardware cursor engine rather than hardware icon engine at this moment ,
     *after 'X' shape cursor had been shown ,then swith to  hardware icon 
     *engine to show other cursors,
    */
    if( pBIOSInfo->ARGB_cursor)
        CallWhileARGB = TRUE; 
       
    dwCursorMode = VIAGETREG(VIA_REG_CURSOR_MODE);
    /* Turn cursor off. */
    VIASETREG(VIA_REG_CURSOR_MODE, dwCursorMode & 0xFFFFFFFE);
    if(pBIOSInfo->Is3DScaling)
    {    
        /* Upload the cursor image to the frame buffer. */
        if(!pVia->IsSecondary)
        {        
            memcpy(pVia->FBBase + pVia->Cursor3DScalingBufferStart, src, MAX_CURS * MAX_CURS / 8 * 2);
            memset(pVia->FBBase + pVia->CursorStart,0x00 , MAX_CURS * MAX_CURS/8*2 );
            hw3d_MonoCursorImageAdjust(pVia);
            
        }
        else
        {
            // mono color cursor is simple, it use two bit one pixel. so it use HxV/8*2 Bytes
            memcpy(pVia->FBBase + pVia->Cursor3DScalingBufferStart, src, MAX_CURS * MAX_CURS / 8 * 2);
            memset(pVia->FBBase + pVia->CursorStart,0x00 , MAX_CURS * MAX_CURS/8*2 );
            hw3d_MonoCursorImageAdjust(pVia);
            
        }    
    }
    else
    {    
        /* Upload the cursor image to the frame buffer. */
        if(!pVia->IsSecondary)
        {        
            memcpy(pVia->FBBase + pVia->CursorStart, src, MAX_CURS * MAX_CURS / 8 * 2);
        }
        else
        {
            // mono color cursor is simple, it use two bit one pixel. so it use HxV/8*2 Bytes
            memcpy(pVia->FBBase + pVia->CursorStart, src, MAX_CURS * MAX_CURS / 8 * 2);
        }
    }
    /* Restore cursor status */
    VIASETREG(VIA_REG_CURSOR_MODE, dwCursorMode);
}

void VIASetCursorPosition(ScrnInfoPtr pScrn, int x, int y)
{
    VIAPtr          pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;
    unsigned char   xoff, yoff;
    CARD32          dwCursorMode;
    CARD32          dwHiControl;
    if(x > pBIOSInfo->HDisplay)
    {
        x = pBIOSInfo->HDisplay;
    }

    if (x < 0) 
    {
        xoff = ((-x) & 0xFE);
        x = 0;
    } 
    else 
    {
        xoff = 0;
    }

    /*It seems that if y - vdisplay<MAX_CURS, there will be garbage on scrren*/
    if(y > pBIOSInfo->VDisplay)
    {
        y = pBIOSInfo->VDisplay;
    }
    
    if (y < 0) 
    {
        yoff = ((-y) & 0xFE);
        y = 0;
    } 
    else 
    {
        yoff = 0;
        /* LCD Expand Mode Cursor Y Position Re-Calculated */
        if (pBIOSInfo->scaleY) 
        {
            y = (int)(((pBIOSInfo->LVDSSettingInfo.PanelSizeY * y) + (pBIOSInfo->resY >> 1)) / pBIOSInfo->resY);
        }
    }

    if(pBIOSInfo->Is3DScaling)
    {
        int DisplayX,DisplayY;
        int ScreenX,ScreenY; 
        
        pVia->DISP3DSCALCTRLInfo.CursorX = x;
        pVia->DISP3DSCALCTRLInfo.CursorY = y;
        
        DisplayX = pBIOSInfo->CrtcHDisplay;
        DisplayY = pBIOSInfo->CrtcVDisplay;

        ScreenX = pVia->DISP3DSCALCTRLInfo.RealHActive;
        ScreenY = pVia->DISP3DSCALCTRLInfo.RealVActive;

        x = (x * ScreenX) / DisplayX;
        y = (y * ScreenY) / DisplayY;

        x -= (x * 2 -ScreenX ) * (pBIOSInfo->TVSettingInfo.TV3DScalInfo.XSIZEOffset) /ScreenX;
        y -= (y * 2 -ScreenY ) * (pBIOSInfo->TVSettingInfo.TV3DScalInfo.YSIZEOffset) /ScreenY;
        
        x += pBIOSInfo->TVSettingInfo.TV3DScalInfo.XPOSITIONOffset;
        y -= pBIOSInfo->TVSettingInfo.TV3DScalInfo.YPOSITIONOffset;

        if(x > ScreenX)
        {
            x = ScreenX-1;
        }

        if (x < 0) 
        {
            xoff = ((-x) & 0xFE);
            x = 0;
        } 
        else 
        {
            xoff = 0;
        }

        /*It seems that if y - vdisplay<MAX_CURS, there will be garbage on scrren*/
        if(y > ScreenY)
        {
            y = ScreenY-1;
        }
        
        if (y < 0) 
        {
            yoff = ((-y) & 0xFE);
            y = 0;
        } 
        else 
        {
            yoff = 0;
        }        
  
    }
          

    if( pBIOSInfo->ARGB_cursor&&(!CallWhileARGB))
    { 
        /* DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIAHISetposition\n")); */

        switch(pBIOSInfo->Chipset )
        {
            case VIA_CX700:
            case VIA_CN750:
            case VIA_P4M890:
            case VIA_P4M900:
                if(!pVia->IsSecondary)
                {
                    dwHiControl = VIAGETREG(PRIM_HI_CTRL);
                    /* Turn cursor off. */
                    VIASETREG(PRIM_HI_CTRL, dwHiControl & 0xFFFFFFFA);
                    /*set hi position*/
                    VIASETREG(PRIM_HI_POSSTART, ((x << 16) | (y & 0x07ff)));
                    /*VIASETREG(PRIM_HI_POSEND, (((x+MAX_CURS) << 16) | ((y+MAX_CURS) & 0x07ff)));*/	  
                    VIASETREG(PRIM_HI_CENTEROFFSET, ((0<< 16) | (0 & 0x07ff)));
                    /* Restore cursor status */
                    VIASETREG(PRIM_HI_CTRL, dwHiControl);
                    break;
                }    
                else
                {
                    dwHiControl = VIAGETREG(HI_CONTROL);
                    /* Turn cursor off. */
                    VIASETREG(HI_CONTROL, dwHiControl & 0xFFFFFFFA);
                    /*set hi position*/
                    VIASETREG(HI_POSSTART, ((x << 16) | (y & 0x07ff)));
                    VIASETREG(HI_CENTEROFFSET, ((0<< 16) | (0 & 0x07ff)));
                    /* Restore cursor status */
                    VIASETREG(HI_CONTROL, dwHiControl);
                    break;
                }
            default:
                   DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "lobo\n", x, y, xoff, yoff));        
 
                dwHiControl = VIAGETREG(HI_CONTROL);
                /* Turn cursor off. */
                VIASETREG(HI_CONTROL, dwHiControl & 0xFFFFFFFA);
                /*set hi position*/
                VIASETREG(HI_POSSTART, ((x << 16) | (y & 0x07ff)));
                VIASETREG(HI_CENTEROFFSET, ((0<< 16) | (0 & 0x07ff)));
                /* Restore cursor status */
                VIASETREG(HI_CONTROL, dwHiControl);
        }
    }
    else
    {
        /* DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIASetposition\n")); */
        /* Hide cursor before set cursor position in order to avoid ghost cursor
         * image when directly set cursor position. It should be a HW bug but
         * we can use patch by SW. */
        dwCursorMode = VIAGETREG(VIA_REG_CURSOR_MODE);
        /* Turn cursor off. */
        VIASETREG(VIA_REG_CURSOR_MODE, dwCursorMode & 0xFFFFFFFE);
        VIASETREG(VIA_REG_CURSOR_ORG, ((xoff << 16) | (yoff & 0x003f)));
        VIASETREG(VIA_REG_CURSOR_POS, ((x << 16) | (y & 0x07ff)));

        /* Restore cursor status */
        VIASETREG(VIA_REG_CURSOR_MODE, dwCursorMode);
   }
}


static void VIASetCursorColors(ScrnInfoPtr pScrn, int bg, int fg)
{
    VIAPtr pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;

#ifdef ARGB_CURSOR
    /* Don't recolour cursors set with if use HI. */
    if (pBIOSInfo->ARGB_cursor&&(!CallWhileARGB))
        return;
#endif

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIASetCursorColors\n"));
    VIASETREG(VIA_REG_CURSOR_FG, fg);
    VIASETREG(VIA_REG_CURSOR_BG, bg);
}

#ifdef ARGB_CURSOR
#include "cursorstr.h"

/*cliff add to enable ALPHA cursor */
static Bool VIAHiUseHWCursorARGB(ScreenPtr pScreen, CursorPtr pCurs)
{
    ScrnInfoPtr pScrn = xf86Screens[pScreen->myNum];
    VIAPtr pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;

    /* DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIAUseHWCursorARGB\n")); */
    VIACheckCursorTypeToUse(pScrn);
    if(pBIOSInfo->DuoView&&(!pVia->HasSecondary))
    pVia->hwcursor = FALSE;
    /* RainLin [2006.03.22]: Switch SW/HW cursor dynamically. */
    if (pVia->hwcursor)
    {
        return TRUE;
    }
    else
    {
        return FALSE;
    }
}


static void
VIAHiLoadCursorARGB (ScrnInfoPtr pScrn, CursorPtr pCurs)
{
    VIAPtr pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;
    VIADISP3DSCALCTRLPtr p3DSCALCTRLInfo = &(pVia->DISP3DSCALCTRLInfo); 
    ScrnInfoPtr pScrn1 = xf86Screens[1];
    VIAPtr pVia1;
    CARD32        *dst         = NULL;
    CARD32  *image = pCurs->bits->argb;
    int            x, y, w, h;
    CARD32          dwHiControl,newHIwidth,newHIheight;
    CARD8 *tmp;
    CARD32 *cursortmpaddr;

    if(pBIOSInfo->SAMM)
    {
        pVia1 = VIAPTR(pScrn1);
    }
    else {
	pVia1 = NULL;
    }
    /* DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIAHiLoadCursorARGB\n")); */
       /*for single case */
    if((!pVia->IsSecondary)&&(!pVia->HasSecondary))
    {
        dst = (CARD32 *)(pVia->FBBase + pVia->CursorStart );
    }
    else
    {
        dst = (CARD32 *)(pVia1->FBBase + pVia1->CursorStart );
    }
    dwHiControl = VIAGETREG(HI_CONTROL);
    /* Turn cursor off. */
    VIASETREG(HI_CONTROL, dwHiControl & 0xFFFFFFFA);
    dwHiControl = VIAGETREG(PRIM_HI_CTRL);
    VIASETREG(PRIM_HI_CTRL, dwHiControl & 0xFFFFFFFA);
    if (!image)
        return;

    w = pCurs->bits->width;
    if (w > MAX_CURS)
        w = MAX_CURS;

    h = pCurs->bits->height;
    if (h > MAX_CURS)
        h = MAX_CURS;

    if((!pVia->IsSecondary)&&(!pVia->HasSecondary))
    {
        tmp =  (CARD8 *)(pVia->FBBase + pVia->CursorStart );

    }
    else
    {
        tmp = (CARD8 *)(pVia1->FBBase + pVia1->CursorStart );
    }
    
    if (pBIOSInfo->Is3DScaling)
    {   
        //Clear the cursor image
        memset(tmp,0,(w*h*4));
        
        cursortmpaddr =(CARD32 *)( pVia->FBBase + pVia->Cursor3DScalingBufferStart);
        // copy the image into the cursor  buffer, which is the source of the cursor adjusting
        for (y = 0; y < h; y++)
        {

            for (x = 0; x < w; x++)
            {
                *cursortmpaddr++ = *image++;
            }
            for (; x < MAX_CURS; x++)
            {   
                *cursortmpaddr++ = 0;
            }
            
        }
        
        // do cursor image adjust
        hw3d_ARGBCursorImageAdjust(pVia);

        newHIwidth = w* (p3DSCALCTRLInfo->RealHActive) /(pBIOSInfo->CrtcHDisplay);
        newHIheight = h* (p3DSCALCTRLInfo->RealVActive)/(pBIOSInfo->CrtcVDisplay);

        for (y = 0; y < newHIheight; y++)
        {
            for (x = 0; x < newHIwidth; x++)
            {
                dst++ ;
            }
            
            /* pad to the right with transparent */
            for (; x < MAX_CURS; x++)
            {   
                *dst++ = 0;
            }
        }
        
        /* pad below with transparent */
        for (; y < MAX_CURS; y++)
        {  
            
            for (x = 0; x < (MAX_CURS); x++)
            {
                *dst++ = 0;
            }
        }
    }
    else     
    {
        memset(tmp,0,(w*h*4));

        for (y = 0; y < h; y++)
        {
            for (x = 0; x < w; x++)
            {
                *dst++ = *image++;
            }
            
            /* pad to the right with transparent */
            for (; x < MAX_CURS; x++)
            {   
                *dst++ = 0;
            }
        }

        /* pad below with transparent */
        for (; y < MAX_CURS; y++)
        {  
            
            for (x = 0; x < (MAX_CURS); x++)
            {
                *dst++ = 0;
            }
        }
    }
}

#endif
