 /*
 * Copyright 1998-2007 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2007 S3 Graphics, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

 
#include "xf86.h"
#include "xf86_OSproc.h"
#include "xf86Priv.h"

#include "xf86PciInfo.h"
#include "xf86Pci.h"

#include "GL/glxtokens.h"

#include <errno.h>

#include "via_driver.h"
#include "via_dri.h"
#include "xf86drm.h"
#include "xf86drmVIA.h"
extern void GlxSetVisualConfigs(
    int nconfigs,
    __GLXvisualConfig *configs,
    void **configprivs
);
unsigned int agpaddr;
#define VIDEO	0 
#define AGP		1
#define AGP_PAGE_SIZE 4096
/*=* John Sheng [2003.5.31] agp tex *=*/
#define AGP_PAGES 4096
#define AGP_SIZE (AGP_PAGE_SIZE * AGP_PAGES)
#define AGP_CMDBUF_PAGES 4096
#define AGP_CMDBUF_SIZE (AGP_PAGE_SIZE * AGP_CMDBUF_PAGES)

static char VIAKernelDriverName[] = "via";
static char VIAClientDriverName[] = "via";
int test_alloc_FB(ScreenPtr pScreen, VIAPtr pVia, int Size);
int test_alloc_AGP(ScreenPtr pScreen, VIAPtr pVia, int Size);
static Bool VIAInitVisualConfigs(ScreenPtr pScreen);
static Bool VIADRIAgpInit(ScreenPtr pScreen, VIAPtr pVia);
static Bool VIADRIPciInit(ScreenPtr pScreen, VIAPtr pVia);
/*=* John Sheng [2003.7.04] *=*/
/*static Bool VIADRIFBInit(ScreenPtr pScreen, VIAPtr pVia);*/
Bool VIADRIFBInit(ScreenPtr pScreen, VIAPtr pVia);
static Bool VIADRIKernelInit(ScreenPtr pScreen, VIAPtr pVia);
static Bool VIADRIMapInit(ScreenPtr pScreen, VIAPtr pVia);

static Bool VIACreateContext(ScreenPtr pScreen, VisualPtr visual, 
                   drmContext hwContext, void *pVisualConfigPriv,
                   DRIContextType contextStore);
static void VIADestroyContext(ScreenPtr pScreen, drmContext hwContext,
                   DRIContextType contextStore);
static void VIADRISwapContext(ScreenPtr pScreen, DRISyncType syncType, 
                   DRIContextType readContextType, 
                   void *readContextStore,
                   DRIContextType writeContextType, 
                   void *writeContextStore);
static void VIADRIInitBuffers(WindowPtr pWin, RegionPtr prgn, CARD32 index);
static void VIADRIMoveBuffers(WindowPtr pParent, DDXPointRec ptOldOrg, 
                   RegionPtr prgnSrc, CARD32 index);
/*homer for vt3336*/
static int VIAInitS3gConf(ScreenPtr );
static int VIAReserveFBBuffer(ScreenPtr );
static int VIAReserveFBBuffer(ScreenPtr );
/*jeffrey jiang change for GCC 4.1 -add- 07.02.13*/
static int S3GInitVisualConfigs(ScreenPtr);

#if 0
static unsigned long LogTwo(unsigned long n);
static void VIADRIRingBufferCleanup(ScrnInfoPtr pScrn);
#endif

static Bool VIADRIRingBufferInit(ScrnInfoPtr pScrn);

static int VIAMapAGP(ScreenPtr pScreen, VIAPtr pVia);
static int VIAS3GFinishScreenInit(ScreenPtr pScreen);


#if 0
static unsigned long
LogTwo(unsigned long n)
{
    unsigned long log2 = 0;

    while (n > 1) {
	n >>= 1;
	log2++;
    }
    return log2;
}
#endif

Bool
VIASet3DSyncInfoParas(ScreenPtr pScreen)
{
    ScrnInfoPtr pScrn = xf86Screens[pScreen->myNum];
    VIAPtr pVia = VIAPTR(pScrn);
    VIADISP3DSCALCTRLPtr p3DSCALCTRLInfo = &(pVia->DISP3DSCALCTRLInfo);

    p3DSCALCTRLInfo->drmFD = pVia->drmFD;
    /*lobo, we found the index should be 2, always*/
    if ((DRIDrawablePrivPtr)(pScreen->devPrivates[2].ptr) != NULL) {
	p3DSCALCTRLInfo->xcontext = ((DRIScreenPrivPtr)
				     pScreen->devPrivates[2].ptr)->myContext;
	p3DSCALCTRLInfo->plock = &(((DRIScreenPrivPtr)
				    (pScreen->devPrivates[2].ptr))->pSAREA->lock);
	p3DSCALCTRLInfo->getShareArea = TRUE;
    }

    return (TRUE);
}

static Bool
VIADRIAgpInit(ScreenPtr pScreen, VIAPtr pVia)
{
    unsigned long  agp_phys;
    VIADRIPtr pVIADRI;
    DRIInfoPtr pDRIInfo;
    pDRIInfo = pVia->pDRIInfo;
    pVIADRI = pDRIInfo->devPrivate;
    pVia->agpSize = 0;

/*=* John Sheng [2004.6.3] For AMD64 *=*/
#ifdef __x86_64__
    return (FALSE);
#endif

    if (drmAgpAcquire(pVia->drmFD) < 0) {
	xf86DrvMsg(pScreen->myNum, X_ERROR, "[drm] drmAgpAcquire failed\n");
	return (FALSE);
    }

    if (drmAgpEnable(pVia->drmFD, drmAgpGetMode(pVia->drmFD)&~0x0) < 0) {
	xf86DrvMsg(pScreen->myNum, X_ERROR, "[drm] drmAgpEnable failed\n");
	return (FALSE);
    }

    xf86DrvMsg(pScreen->myNum, X_INFO, "[drm] drmAgpEnabled succeeded\n");

    if (drmAgpAlloc(pVia->drmFD, AGP_SIZE, 0, &agp_phys, &pVia->agpHandle) < 0) {
	xf86DrvMsg(pScreen->myNum, X_ERROR,
		   "[drm] drmAgpAlloc failed\n");
	drmAgpRelease(pVia->drmFD);
	return (FALSE);
    }

    if (drmAgpBind(pVia->drmFD, pVia->agpHandle, 0) < 0) {
	xf86DrvMsg(pScreen->myNum, X_ERROR,
		   "[drm] drmAgpBind failed\n");
	drmAgpFree(pVia->drmFD, pVia->agpHandle);
	drmAgpRelease(pVia->drmFD);

	return (FALSE);
    }

    pVia->agpSize = AGP_SIZE;
    pVia->agpAddr = (pointer)drmAgpBase(pVia->drmFD);
    xf86DrvMsg(pScreen->myNum, X_INFO,
	       "[drm] agpAddr = 0x%08lx\n", (unsigned long)pVia->agpAddr);

    /*Homer, h5s1 needn't share agp handle, and addmap drm buf is in VIAMapAGP()*/
    if (!H5S1_UMA_CHIPID) {
	pVIADRI->agp.size = pVia->agpSize;
	if (drmAddMap(pVia->drmFD, (drmHandle)0,
		      pVIADRI->agp.size, DRM_AGP, 0, 
		      &pVIADRI->agp.handle) < 0) {
	    xf86DrvMsg(pScreen->myNum, X_ERROR,
		       "[drm] Failed to map public agp area\n");
	    pVIADRI->agp.size = 0;
	    return (FALSE);
	}
	/* Map AGP from kernel to Xserver - Not really needed */
	drmMap(pVia->drmFD, pVIADRI->agp.handle,pVIADRI->agp.size,
	       (drmAddressPtr)&agpaddr);

	/*homer agp flush drm*/
	pVia->agp_handle=pVIADRI->agp.handle;
	/*homer end*/
	drmVIAAgpInit(pVia->drmFD, 0, AGP_SIZE);
    }
    xf86DrvMsg(pScreen->myNum, X_INFO, 
	       "[drm] agpBase = 0x%08lx\n", (unsigned long)pVia->agpBase);
    xf86DrvMsg(pScreen->myNum, X_INFO, 
	       "[drm] agpAddr = 0x%08lx\n", (unsigned long)pVia->agpAddr);
    xf86DrvMsg(pScreen->myNum, X_INFO, 
	       "[drm] agpSize = 0x%08lx\n", (unsigned long)pVia->agpSize);
    xf86DrvMsg(pScreen->myNum, X_INFO, 
	       "[drm] agp physical addr = 0x%08lx\n", agp_phys);

    return (TRUE);
}

/*=* John Sheng [2003.7.04] *=*/
/*static Bool VIADRIFBInit(ScreenPtr pScreen, VIAPtr pVia)*/
Bool VIADRIFBInit(ScreenPtr pScreen, VIAPtr pVia)
{   
    int FBSize = pVia->FBFreeEnd-pVia->FBFreeStart;
    int FBOffset = pVia->FBFreeStart; 
    VIADRIPtr pVIADRI = pVia->pDRIInfo->devPrivate;
    pVIADRI->fbOffset = FBOffset;
    pVIADRI->fbSize = pVia->videoRambytes;
    if(H5S1_UMA_CHIPID) /*Homer, for H5s1_UMA, go another path*/
    {
        if(!(VIAReserveFBBuffer(pScreen)))
    		return FALSE;
        return TRUE;
    }
    else
    {
        if (drmVIAFBInit(pVia->drmFD, FBOffset, FBSize) < 0) {
    	xf86DrvMsg(pScreen->myNum, X_ERROR,"[drm] failed to init frame buffer area\n");
    	return FALSE;		
        }
        else {
    	xf86DrvMsg(pScreen->myNum, X_INFO,"[drm] FBFreeStart= 0x%08x FBFreeEnd= 0x%08x FBSize= 0x%08x\n", pVia->FBFreeStart, pVia->FBFreeEnd, FBSize);
    	return TRUE;	
        }
    }
}
static Bool VIADRIPciInit(ScreenPtr pScreen, VIAPtr pVia)
{
    return TRUE;	
}

static Bool
VIAInitVisualConfigs(ScreenPtr pScreen)
{
    ScrnInfoPtr pScrn = xf86Screens[pScreen->myNum];
    VIAPtr pVia = VIAPTR(pScrn);
    int numConfigs = 0;
    __GLXvisualConfig *pConfigs = 0;
    VIAConfigPrivPtr pVIAConfigs = 0;
    VIAConfigPrivPtr *pVIAConfigPtrs = 0;
    int i, db, stencil, accum;

    switch (pScrn->bitsPerPixel) {
	case 8:
	case 24:
	    break;
	case 16:
	case 32:
	    /* John Sheng [2003.7.2] for visual config */
	    numConfigs = 16;
	    
	    if (!(pConfigs = (__GLXvisualConfig*)xcalloc(sizeof(__GLXvisualConfig),
						   numConfigs)))
		return FALSE;
	    if (!(pVIAConfigs = (VIAConfigPrivPtr)xcalloc(sizeof(VIAConfigPrivRec),
						    numConfigs))) {
    		xfree(pConfigs);
    		return FALSE;
	    }
	    if (!(pVIAConfigPtrs = (VIAConfigPrivPtr*)xcalloc(sizeof(VIAConfigPrivPtr),
							  numConfigs))) {
    		xfree(pConfigs);
    		xfree(pVIAConfigs);
    		return FALSE;
	    }
	    for (i=0; i<numConfigs; i++) 
    		pVIAConfigPtrs[i] = &pVIAConfigs[i];

	    i = 0;
	    for (accum = 0; accum <= 1; accum++) {
		/* John Sheng [2003.7.2] for visual config */
    		/*for (stencil=0; stencil<=1; stencil++) {*/
		for (stencil = 0; stencil <= 3; stencil++) {
    		    for (db = 0; db <= 1; db++) {
        		pConfigs[i].vid = -1;
        		pConfigs[i].class = -1;
        		pConfigs[i].rgba = TRUE;
        		pConfigs[i].redSize = -1;
        		pConfigs[i].greenSize = -1;
        		pConfigs[i].blueSize = -1;
        		pConfigs[i].redMask = -1;
        		pConfigs[i].greenMask = -1;
        		pConfigs[i].blueMask = -1;
        		pConfigs[i].alphaMask = 0;
			
			if (accum) {
        		    pConfigs[i].accumRedSize = 16;
        		    pConfigs[i].accumGreenSize = 16;
        		    pConfigs[i].accumBlueSize = 16;
        		    pConfigs[i].accumAlphaSize = 16;
        		}
			else {
        		    pConfigs[i].accumRedSize = 0;
        		    pConfigs[i].accumGreenSize = 0;
        		    pConfigs[i].accumBlueSize = 0;
        		    pConfigs[i].accumAlphaSize = 0;
        		}
        		if (db)
        		    pConfigs[i].doubleBuffer = TRUE;
        		else
        		    pConfigs[i].doubleBuffer = FALSE;
        		
			pConfigs[i].stereo = FALSE;
        		pConfigs[i].bufferSize = -1;
        		
			switch (stencil) {
        		    case 0:
            			pConfigs[i].depthSize = 0;
            			pConfigs[i].stencilSize = 0;
            			break;
        		    case 1:
            			pConfigs[i].depthSize = 16;
            			pConfigs[i].stencilSize = 0;
            			break;
        		    case 2:
            			pConfigs[i].depthSize = 32;
            			pConfigs[i].stencilSize = 0;
            			break;
        		    case 3:
            			pConfigs[i].depthSize = 24;
            			pConfigs[i].stencilSize = 8;
            			break;
        		}
        		
			pConfigs[i].auxBuffers = 0;
        		pConfigs[i].level = 0;
        		pConfigs[i].visualRating = GLX_NONE_EXT;
        		pConfigs[i].transparentPixel = 0;
        		pConfigs[i].transparentRed = 0;
        		pConfigs[i].transparentGreen = 0;
        		pConfigs[i].transparentBlue = 0;
	        	pConfigs[i].transparentAlpha = 0;
        		pConfigs[i].transparentIndex = 0;
        		i++;
		    }
		}
	    }
	
	if (i != numConfigs) {
	    xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
		"[dri] Incorrect initialization of visuals.  Disabling DRI.\n");
    		return FALSE;
	}
	    
	break;
    }
    
    pVia->numVisualConfigs = numConfigs;
    pVia->pVisualConfigs = pConfigs;
    pVia->pVisualConfigsPriv = pVIAConfigs;

    GlxSetVisualConfigs(numConfigs, pConfigs, (void**)pVIAConfigPtrs);

    return TRUE;
}

#if 0
/*JohnsonZheng[20070326]-initialize AGP ring buffer in 2D driver*/
static void
VIADRIRingBufferCleanup(ScrnInfoPtr pScrn)
{
    VIAPtr pVia = VIAPTR(pScrn);
    drm_via_dma_init_t ringBufInit;

    xf86DrvMsg(pScrn->scrnIndex, X_INFO, 
	       "[drm] Cleaning up DMA ring-buffer.\n");
    ringBufInit.func = VIA_CLEANUP_DMA;
    if (drmCommandWrite(pVia->drmFD, DRM_VIA_DMA_INIT, &ringBufInit,
			sizeof(ringBufInit))) {
	xf86DrvMsg(pScrn->scrnIndex, X_WARNING, 
		   "[drm] Failed to clean up DMA ring-buffer: %d\n", errno);
    }
}
#endif

static Bool VIADRIRingBufferInit(ScrnInfoPtr pScrn)
{
    VIAPtr pVia = VIAPTR(pScrn);
    if (!pVia->IsPCI) {
	drm_via_dma_init_t ringBufInit,init;
   	/*
    	* Check whether AGP DMA has been initialized.
    	*/
   	memset(&init, 0, sizeof(init));
   	memset(&ringBufInit, 0, sizeof(ringBufInit));
 	init.func = VIA_DMA_INITIALIZED;

     	if(0 == drmCommandWrite(pVia->drmFD, DRM_VIA_DMA_INIT,&init, sizeof(init)))
		return TRUE;
        ringBufInit.reg_pause_addr = 0x418;
	ringBufInit.offset = 0x0;
	ringBufInit.size = AGP_CMDBUF_SIZE;

	ringBufInit.func = VIA_INIT_DMA;
	if (drmCommandWrite(pVia->drmFD, DRM_VIA_DMA_INIT, &ringBufInit,
			    sizeof(ringBufInit))) {
	    xf86DrvMsg(pScrn->scrnIndex, X_ERROR, 
		       "[drm] Failed to initialize DMA ring-buffer: %d\n", errno);
	    return FALSE;
	}
	xf86DrvMsg(pScrn->scrnIndex, X_INFO, 
		   "[drm] Initialized AGP ring-buffer, size 0x%lx at AGP offset 0x%lx.\n",
		   ringBufInit.size, ringBufInit.offset);
    }
    return TRUE;
}
/*JohnsonZheng-End*/
Bool VIADRIScreenInit(ScreenPtr pScreen)
{
    ScrnInfoPtr pScrn = xf86Screens[pScreen->myNum];
    VIAPtr pVia = VIAPTR(pScrn);
    DRIInfoPtr pDRIInfo;
    VIADRIPtr pVIADRI;

	     if(H5S1_UMA_CHIPID)
		xf86DrvMsg(pScreen->myNum, X_INFO,
				"[drm] Detect H5 chipset: %04x  chipid: %04x\n",
				pVia->Chipset, pVia->ChipId);
		
    /* Check that the GLX, DRI, and DRM modules have been loaded by testing
    * for canonical symbols in each module. */
    if (!xf86LoaderCheckSymbol("GlxSetVisualConfigs")) return FALSE;
    if (!xf86LoaderCheckSymbol("DRIScreenInit"))       return FALSE;
    if (!xf86LoaderCheckSymbol("drmAvailable"))        return FALSE;
    if (!xf86LoaderCheckSymbol("DRIQueryVersion")) {
	xf86DrvMsg(pScreen->myNum, X_ERROR,
                 "[dri] VIADRIScreenInit failed (libdri.a too old)\n");
        return FALSE;
    }
    
    /* Check the DRI version */
/*jeffrey jiang change for get the libDRI version 5.0.0, so remove it first.*/
    if(!H5S1_UMA_CHIPID)
    {
        int major, minor, patch;
        DRIQueryVersion(&major, &minor, &patch);
        if (major < 4 || minor < 0) {
            xf86DrvMsg(pScreen->myNum, X_ERROR,
                    "[dri] VIADRIScreenInit failed because of a version mismatch.\n"
                    "[dri] libDRI version is %d.%d.%d but version 4.0.x is needed.\n"
                    "[dri] Disabling DRI.\n",
                    major, minor, patch);
            return FALSE;
        }
    }
    else /*Homer for support h5s1(vt3336/vt3364), now allocate additional record*/
    {
        ((VIARec *)(pScrn->driverPrivate))->pScrnConf = xnfcalloc(sizeof(ScrnConfRec), 1);
        ((VIARec *)(pScrn->driverPrivate))->pDriConf = xnfcalloc(sizeof(DriConfRec), 1);
        ((VIARec *)(pScrn->driverPrivate))->pServerInfo = xnfcalloc(sizeof(ServerInfoRec), 1);
        ((VIARec *)(pScrn->driverPrivate))->pAddrConf = xnfcalloc(sizeof(AddrConfRec), 1);
        ((VIARec *)(pScrn->driverPrivate))->pPciConf = xnfcalloc(sizeof(PciConfRec), 1);
        ((VIARec *)(pScrn->driverPrivate))->pChipConf = xnfcalloc(sizeof(ChipConfRec), 1);
    }
        /*homer end*/


    pDRIInfo = DRICreateInfoRec();
    
    if (!pDRIInfo) return FALSE;
    
    pVia->pDRIInfo = pDRIInfo;
    pDRIInfo->drmDriverName = VIAKernelDriverName;
    pDRIInfo->clientDriverName = VIAClientDriverName;
    if(H5S1_UMA_CHIPID) /*Homer, for H5s1_UMA, drm module use uma.ko instead of via.ko*/
    {
        strcpy(VIAKernelDriverName, "uma");
        strcpy(VIAClientDriverName, "uma");
    }
    pDRIInfo->busIdString = xalloc(64);
    sprintf(pDRIInfo->busIdString, "PCI:%d:%d:%d",
        ((pciConfigPtr)pVia->PciInfo->thisCard)->busnum,
        ((pciConfigPtr)pVia->PciInfo->thisCard)->devnum,
        ((pciConfigPtr)pVia->PciInfo->thisCard)->funcnum);
    pDRIInfo->ddxDriverMajorVersion = VIA_VERSION_MAJOR;
    pDRIInfo->ddxDriverMinorVersion = VIA_VERSION_MINOR;
    pDRIInfo->ddxDriverPatchVersion = PATCHLEVEL;
    pDRIInfo->frameBufferPhysicalAddress = (pointer)pVia->FrameBufferBase;
    pDRIInfo->frameBufferSize = pVia->videoRambytes;  
  
    pDRIInfo->frameBufferStride = (pScrn->displayWidth *
					    pScrn->bitsPerPixel / 8);
    pDRIInfo->ddxDrawableTableEntry = VIA_MAX_DRAWABLES;

    if (SAREA_MAX_DRAWABLES < VIA_MAX_DRAWABLES)
	pDRIInfo->maxDrawableTableEntry = SAREA_MAX_DRAWABLES;
    else
	pDRIInfo->maxDrawableTableEntry = VIA_MAX_DRAWABLES;

#ifdef NOT_DONE
    /* FIXME need to extend DRI protocol to pass this size back to client 
    * for SAREA mapping that includes a device private record
    */
    pDRIInfo->SAREASize = 
	((sizeof(XF86DRISAREARec) + 0xfff) & 0x1000); /* round to page */
    /* + shared memory device private rec */
#else
    /* For now the mapping works by using a fixed size defined
    * in the SAREA header
    */
    if (sizeof(XF86DRISAREARec)+sizeof(VIASAREAPriv) > SAREA_MAX) {
	xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
			"Data does not fit in SAREA\n");
	return FALSE;
    }
    pDRIInfo->SAREASize = SAREA_MAX;
#endif

    if (!(pVIADRI = (VIADRIPtr)xcalloc(sizeof(VIADRIRec),1))) {
	DRIDestroyInfoRec(pVia->pDRIInfo);
	pVia->pDRIInfo=0;
	return FALSE;
    }
    /*homer, for h5s1_UMA, copy the s3g rec ptr from VIARec*/
    if(H5S1_UMA_CHIPID)
    {
        pVIADRI->pServerInfo = pVia->pServerInfo;
        pDRIInfo->devPrivate= pVIADRI;
        pDRIInfo->devPrivateSize = sizeof(VIADRIRec);
        pDRIInfo->contextSize = sizeof(VIADRIContextRec);
    }
    else
    {
        pDRIInfo->devPrivate= pVIADRI;
        pDRIInfo->devPrivateSize = sizeof(VIADRIRec);
        pDRIInfo->contextSize = sizeof(VIADRIContextRec);
    }
    /*homer end*/
    pDRIInfo->CreateContext = VIACreateContext;
    pDRIInfo->DestroyContext = VIADestroyContext;
    pDRIInfo->SwapContext = VIADRISwapContext;
    pDRIInfo->InitBuffers = VIADRIInitBuffers;
    pDRIInfo->MoveBuffers = VIADRIMoveBuffers;
    pDRIInfo->bufferRequests = DRI_ALL_WINDOWS;

    if (!DRIScreenInit(pScreen, pDRIInfo, &pVia->drmFD)) {
	xf86DrvMsg(pScreen->myNum, X_ERROR,
	    "[dri] DRIScreenInit failed.  Disabling DRI.\n");
	xfree(pDRIInfo->devPrivate);
	pDRIInfo->devPrivate=0;
	DRIDestroyInfoRec(pVia->pDRIInfo);
	pVia->pDRIInfo=0;
	pVia->drmFD = -1;
	return FALSE;
    }

	   
    pVia->IsPCI = !VIADRIAgpInit(pScreen, pVia);
  
    if (pVia->IsPCI) {
	VIADRIPciInit(pScreen, pVia);
	xf86DrvMsg(pScrn->scrnIndex, X_INFO, "[dri] use pci.\n" );
    }
    else
	xf86DrvMsg(pScrn->scrnIndex, X_INFO, "[dri] use agp.\n" );
    /*=* John Sheng [2003.7.15] For VIDEO PIXEL_CACHE_MAP_SIZE 2M too small *=*/
    /*if (!(VIADRIFBInit(pScreen, pVia))) {
	VIADRICloseScreen(pScreen);
	xf86DrvMsg(pScrn->scrnIndex, X_ERROR, "[dri] frame buffer initialize fial .\n" );
	return FALSE;
    }*/
    
    xf86DrvMsg(pScrn->scrnIndex, X_INFO, "[dri] frame buffer initialized.\n" );
    if(!H5S1_UMA_CHIPID)
    {
        if (!(VIAInitVisualConfigs(pScreen))) 
        {
            VIADRICloseScreen(pScreen);
            return FALSE;
        }
    } /*homer for  h5s1 UMA*/
    else
    {
        if(!(S3GInitVisualConfigs(pScreen)))
        {
            VIADRICloseScreen(pScreen);
            return FALSE;
        }
    }
    xf86DrvMsg(pScrn->scrnIndex, X_INFO, "[dri] visual configs initialized.\n" );
  
    /* DRIScreenInit doesn't add all the common mappings.  Add additional mappings here. */
    if (!VIADRIMapInit(pScreen, pVia)) {
	VIADRICloseScreen(pScreen);
	return FALSE;
    }
    pVIADRI->regs.size = VIA_MMIO_REGSIZE;
    pVIADRI->regs.map = 0;
    pVIADRI->regs.handle = pVia->registerHandle;
    xf86DrvMsg(pScreen->myNum, X_INFO, "[drm] mmio Registers = 0x%08x\n",
	       pVIADRI->regs.handle);
    
    pVIADRI->drixinerama = pVia->drixinerama;
    /*=* John Sheng [2003.12.9] Tuxracer & VQ *=*/
    pVIADRI->VQEnable = pVia->VQEnable;

    xf86DrvMsg(pScrn->scrnIndex, X_INFO, "[dri] mmio maped.\n" );

    return TRUE;
}

void
VIADRICloseScreen(ScreenPtr pScreen)
{
    ScrnInfoPtr pScrn = xf86Screens[pScreen->myNum];
    VIAPtr pVia = VIAPTR(pScrn);
    if(H5S1_UMA_CHIPID)
    {
        drm_s3g_init_t init;
        /*Homer, call drm clean up ioctl before close screen*/
        DriConfPtr pDriConf = pVia->pDriConf;
        memset(&init, 0, sizeof(drm_s3g_init_t));
        init.func = S3G_CLEANUP;
        drmCommandWrite(pDriConf->drmFd, DRM_S3G_INIT, &init, sizeof(drm_s3g_init_t));
    }
    DRICloseScreen(pScreen);

    if (pVia->pDRIInfo) {
	if (pVia->pDRIInfo->devPrivate) {
    	    xfree(pVia->pDRIInfo->devPrivate);
    	    pVia->pDRIInfo->devPrivate=0;
	}
	DRIDestroyInfoRec(pVia->pDRIInfo);
	pVia->pDRIInfo=0;
    }
    
    if (pVia->pVisualConfigs) xfree(pVia->pVisualConfigs);
    if (pVia->pVisualConfigsPriv) xfree(pVia->pVisualConfigsPriv);
    if (pVia->agpSize) {
	xf86DrvMsg(pScreen->myNum, X_INFO, "[drm] Freeing agp memory\n");
        drmAgpFree(pVia->drmFD, pVia->agpHandle);
	xf86DrvMsg(pScreen->myNum, X_INFO, "[drm] Releasing agp module\n");
    	drmAgpRelease(pVia->drmFD);
    }
}

/* TODO: xserver receives driver's swapping event and do something
 *       according the data initialized in this function
 */
static Bool
VIACreateContext(ScreenPtr pScreen, VisualPtr visual, 
          drmContext hwContext, void *pVisualConfigPriv,
          DRIContextType contextStore)
{
    return TRUE;
}

static void
VIADestroyContext(ScreenPtr pScreen, drmContext hwContext, 
           DRIContextType contextStore)
{
}

Bool
VIADRIFinishScreenInit(ScreenPtr pScreen)
{
    ScrnInfoPtr pScrn = xf86Screens[pScreen->myNum];
    VIAPtr pVia = VIAPTR(pScrn);
    VIADRIPtr pVIADRI;
    pVIADRI=(VIADRIPtr)pVia->pDRIInfo->devPrivate;

    pVia->pDRIInfo->driverSwapMethod = DRI_HIDE_X_CONTEXT;
    
    DRIFinishScreenInit(pScreen);
    /*homer for vt3336*/
    if(H5S1_UMA_CHIPID)
    {
    VIAS3GFinishScreenInit(pScreen);
    }
    /*homer end*/
    if (!VIADRIKernelInit(pScreen, pVia)) {
	VIADRICloseScreen(pScreen);
	return FALSE;
    }
    xf86DrvMsg(pScreen->myNum, X_INFO, "[dri] kernel data initilized.\n");

    /* set SAREA value */
    {
	VIASAREAPriv *saPriv;

	saPriv=(VIASAREAPriv*)DRIGetSAREAPrivate(pScreen);
	assert(saPriv);
	memset(saPriv, 0, sizeof(*saPriv));
	saPriv->CtxOwner = -1;
    }
    pVIADRI=(VIADRIPtr)pVia->pDRIInfo->devPrivate;
    pVIADRI->deviceID=pVia->Chipset;  
    pVIADRI->width=pScrn->virtualX;
    pVIADRI->height=pScrn->virtualY;
    pVIADRI->mem=pScrn->videoRam*1024;
    pVIADRI->bytesPerPixel= (pScrn->bitsPerPixel+7) / 8; 
    pVIADRI->sarea_priv_offset = sizeof(XF86DRISAREARec);
    /* TODO */
    pVIADRI->scrnX=pVIADRI->width;
    pVIADRI->scrnY=pVIADRI->height;

    /*homer for vt3336*/
    pVia->useAgp = 0;
    if(H5S1_UMA_CHIPID)
    {
        pVia->pDRIInfo->devPrivate= pVia->pServerInfo;
        pVia->pDRIInfo->devPrivateSize = sizeof(ServerInfoRec);
        pVia->pDRIInfo->contextSize = 256;
    }else{
     /*homer end*/
    /*JohnsonZheng[20070411]-initia AGP ring buffer in 2D driver*/
    	pVia->useAgp=VIADRIRingBufferInit(pScrn);
    	
    /*JohnsonZheng-End*/
    }
    return TRUE;
}

static void
VIADRISwapContext(ScreenPtr pScreen, DRISyncType syncType, 
           DRIContextType oldContextType, void *oldContext,
           DRIContextType newContextType, void *newContext)
{
  /*ScrnInfoPtr pScrn = xf86Screens[pScreen->myNum];
  VIAPtr pVia = VIAPTR(pScrn);
  */   
  return;
}

static void
VIADRIInitBuffers(WindowPtr pWin, RegionPtr prgn, CARD32 index)
{
  /*ScreenPtr pScreen = pWin->drawable.pScreen;
  ScrnInfoPtr pScrn = xf86Screens[pScreen->myNum];
  VIAPtr pVia = VIAPTR(pScrn);
  */
  return;
}

static void
VIADRIMoveBuffers(WindowPtr pParent, DDXPointRec ptOldOrg, 
           RegionPtr prgnSrc, CARD32 index)
{
  /*ScreenPtr pScreen = pParent->drawable.pScreen;
  ScrnInfoPtr pScrn = xf86Screens[pScreen->myNum];
  VIAPtr pVia = VIAPTR(pScrn);
  */
  return;
}

/* Initialize the kernel data structures. */
static Bool
VIADRIKernelInit(ScreenPtr pScreen, VIAPtr pVia)
{
    drmVIAInit drmInfo;

    drmInfo.sarea_priv_offset = sizeof(XF86DRISAREARec);
    drmInfo.fb_offset = pVia->FrameBufferBase;
    drmInfo.mmio_offset         = pVia->registerHandle;

    if (pVia->IsPCI) {
	drmInfo.agpAddr = (CARD32)NULL;
    }
    else {
/*=* John Sheng [2004.6.3] For AMD64 *=*/
#ifndef __x86_64__
	drmInfo.agpAddr = (CARD32)pVia->agpAddr;
#else
	drmInfo.agpAddr = (CARD64)pVia->agpAddr;
#endif
    }

    /*homer agp flush drm*/
    drmInfo.agp_offset=(CARD32)pVia->agp_handle;
    /*homer end*/
    if (H5S1_UMA_CHIPID) {	/*drm init for H5s1_UMA*/
	DriConfPtr pDriConf = pVia->pDriConf;
	ChipConfPtr pChipConf = pVia->pChipConf;
	AddrConfPtr pAddrConf = pVia->pAddrConf;
	ServerInfoPtr pServerInfo = pVia->pServerInfo;
	drm_s3g_init_t init;
	int result;

	init.func = S3G_INIT;
	init.chip_index = S3G_INV;
	init.chip_sub_index = CHIP_H5S1;
	init.garttable_size = 0x0;
	/*because our h5s1 is AGP, but not PCIE, we don't need to create garttable*/
	init.garttable_offset = 0x0;
	/*for h5s1, dma buffer is in agp*/
	init.dma_handle = pDriConf->dmaHandle;

	/*homer, tell drm how to init dma-agp*/
	/*if agp init failed, then use pci*/
	if(pVia->IsPCI)
	    init.agp_type = AGP_DISABLED;
	/*For vt3336 and vt3364, the default agp type is ring buffer mode*/
	else
	    init.agp_type = AGP_RING_BUFFER;
	/*end*/

	init.sarea_priv_offset = sizeof(XF86DRISAREARec);
	init.fb_cpp = pVia->Bpp;
	init.front_offset = 0;
	init.depth_offset = pDriConf->depthOffset;

	init.mmio_handle = pDriConf->mmioHandle;

	init.fb_handle = pDriConf->fbHandle;
	init.front_handle = pDriConf->frontHandle;
	init.back_handle = pDriConf->frontHandle;
	init.depth_handle = pDriConf->frontHandle;

	/* Set total available video memory for 3D apps */
	/*because the meaning of fbFree in VIA and s3g is contrary to via*/
	init.available_fb_size = pAddrConf->fbFree - pVia->FBFreeStart;
	init.fb_base_address = pAddrConf->fbPhyAddr;    
	/*give the region follow on-screen to 3d*/
	init.back_offset = pVia->FBFreeStart;

	init.fb_tex_offset = pDriConf->fbTexOffset;
	init.fb_tex_size = pDriConf->fbTexSize;
	init.agp_tex_size = pDriConf->agpTexSize;
	init.agp_tex_handle = pDriConf->agpTexHandle;

	init.chip_agp = pChipConf->chipType;
	init.chip_sub_index = pServerInfo->chipSubIndex;
	init.chip_index = pServerInfo->chipIndex;

	/*???this value should consult to clb driver*/
	init.usec_timeout = 50000;
	/*Homer, now shadow is in video memory, so if agp init failed, shadow is still available*/
	init.shadow_size = pDriConf->shadowSize;
	init.shadow_handle = pDriConf->shadowHandle;

	if (pVia->IsPCI) {
	    init.DMA_size = 0x0;
	}
	else {
	    /*dma buffer is follow the shadow in AGP*/
            init.DMA_phys_address = pDriConf->agpBase;
	    /*the real size of ring buffer agp buffer size*/
	    init.DMA_size = AGP_CMDBUF_SIZE;
	}

	result = drmCommandWrite(pDriConf->drmFd, DRM_S3G_INIT, &init,
				 sizeof(drm_s3g_init_t));
	if (result < 0) {
	    xf86DrvMsg(pScreen->myNum, X_ERROR,
		       "[drm] ERROR! S3GDRIKernelInit:failed to initialize kernel module! (%d)\n", result);
	    return (FALSE);
	}

        return (TRUE);
    }
    else {
	if (drmVIAInitMAP(pVia->drmFD, &drmInfo) < 0) 
	    return (FALSE);
    }

    return (TRUE);
}

/* Add a map for the MMIO registers */
static Bool
VIADRIMapInit(ScreenPtr pScreen, VIAPtr pVia)
{
    int flags = 0;

    if (drmAddMap(pVia->drmFD, pVia->MmioBase, VIA_MMIO_REGSIZE,
		  DRM_REGISTERS, flags,
		  (drm_handle_t *)(&pVia->registerHandle)) < 0) {
	return FALSE;
    }
    if (H5S1_UMA_CHIPID) {	/*Homer for H5s1_UMA*/
	VIAInitS3gConf(pScreen);
	VIAMapAGP(pScreen, pVia);
    }
    xf86DrvMsg(pScreen->myNum, X_INFO,
	       "[drm] register handle = 0x%08lx\n", pVia->registerHandle);

    return (TRUE);
}

/*Homer, following functions are only used by H5s1_UMA(vt3336/vt3364)*/
static int
VIAMapAGP(ScreenPtr pScreen, VIAPtr pVia)
{
    ScrnInfoPtr pScrn = xf86Screens[pScreen->myNum];
    DriConfPtr pDriConf = pVia->pDriConf;
    ChipConfPtr pChipConf = pVia->pChipConf;
    AddrConfPtr pAddrConf = pVia->pAddrConf;
    ScrnConfPtr pScrnConf = pVia->pScrnConf;
    int dummy,*dummyptr;

    /*get pDriConf agp para*/
    pDriConf->agpHandle = (unsigned long)pVia->agpHandle;
    pDriConf->agpBase = (unsigned long)pVia->agpBase;
    pDriConf->agpSize = (unsigned long)pVia->agpSize;

    /*at first, get mmio handle from pVia*/
    pDriConf->mmioHandle = pVia->registerHandle;

    /* get linear frame buffer handle */
    DRIGetDeviceInfo(pScreen, &pDriConf->fbHandle,
		     &dummy, &dummy, &dummy, &dummy, (void **)&dummyptr);

    /* alloc shadow region */
    pDriConf->shadowSize = 16 * 1024;	/* 16K */
    pDriConf->shadowOffset = 0x0;

    /*Homer, now remove the shadow region to pci space*/
#define _DRM_CONSISTENT 5
    if (drmAddMap(pVia->drmFD,
		  0,
		  pDriConf->shadowSize,		/* size of shadow region */
		  _DRM_CONSISTENT, 0,
		  &pDriConf->shadowHandle) < 0) {
	xf86DrvMsg(pScreen->myNum, X_ERROR,
		   "[drm] Could not add shadow region mapping\n");
	return (FALSE);
    }

    if (!pVia->IsPCI) {			/*AGP has been enabled*/
	/* add dma buffer */
	int dmaBufSize = AGP_CMDBUF_SIZE;

	pDriConf->dmaBufAGP = TRUE;	/*tell user space DRI, agp ok*/
	if (drmAddMap(pVia->drmFD,
		      /* offset of AGP = 0x0, because shadow is in sys mem */
		      0x0,
                      dmaBufSize,	/* size of AGP */
		      DRM_AGP, 0,
		      &pDriConf->dmaHandle) < 0) {
	    xf86DrvMsg(pScreen->myNum, X_ERROR,
		       "[drm] Could not add DMA buffers mapping\n");
	    return (FALSE);
	}
	pDriConf->dmaBufOffs = 0x0;
    }
    else {		/*AGP is disable, use PCI*/
	/*tell user module, agp init fail and use PCI*/
	pDriConf->dmaBufAGP = FALSE;
    }

    pDriConf->agpMode   = drmAgpGetMode(pVia->drmFD);
    pDriConf->agpVendor = drmAgpVendorId(pVia->drmFD);
    pDriConf->agpDevice = drmAgpDeviceId(pVia->drmFD);

    pDriConf->frontPhyAddr = pVia->FrameBufferBase + pDriConf->frontOffset;
    pDriConf->backPhyAddr = pVia->FrameBufferBase + pDriConf->backOffset;
    pDriConf->depthPhyAddr = pVia->FrameBufferBase + pDriConf->depthOffset;

    /*homer, fill in Rec*/
    strcpy(pChipConf->chipName, pScrn->chipset);
    /*pChipConf->chipName = pScrn->chipset;*/
    pChipConf->maxIgas = 2;
    pChipConf->maxApts = 16;		/*like deltachrome*/
    if (H5S1_UMA_CHIPID) {
	pChipConf->pciID = pVia->ChipId;
	pChipConf->pciTag = pVia->PciTag;
	pChipConf->primary = 1;
	pChipConf->chipType = CHIP_AGP;
	pChipConf->samm = 0;
	pChipConf->mergefb = 0;
	pChipConf->bcipm = 1;		/*BCI power management on, like clb*/
	pChipConf->docking = 0;
	pChipConf->swizzleOff = 1;
    }
    /*homer, fill in AddrChonf*/
    pAddrConf->fbVirtAddr = (unsigned long)pVia->FBBase;
    pAddrConf->fbPhyAddr = (unsigned long)pVia->FrameBufferBase;
    pAddrConf->fbSize = pScrnConf->fbSize;
    /*    pAddrConf->fbFree = pVia->FBFreeStart;*/
    /*!!!this line is error, for s3g, fbFree means the highest side of videomem, which is contrary to via*/  
    pAddrConf->fbFree = pVia->FBFreeEnd; /*give the available end of fb*/
    pAddrConf->mmioPhyAddr = pVia->MmioBase;
    pAddrConf->mmioVirtAddr = pVia->MapBase;
    pAddrConf->mmioSize = VIA_MMIO_REGSIZE;
    pAddrConf->bciPhyAddr = pAddrConf->mmioPhyAddr + 0x10000;
    pAddrConf->bciVirtAddr = pAddrConf->mmioVirtAddr + 0x10000;
    pAddrConf->bciSize = 0x10000;
    pAddrConf->aptPhyAddr[0] = pScrnConf->fbPhyAddr;
    pAddrConf->aptPhyAddr[1] = pScrnConf->fbPhyAddr + pScrnConf->   fbOffset;
    pAddrConf->aptVirtAddr[0] = pAddrConf->fbVirtAddr;
    pAddrConf->aptVirtAddr[1] = pAddrConf->fbVirtAddr + pScrnConf->fbOffset;
    pAddrConf->aptPitch[0] = pScrnConf->scrnDelta;
    pAddrConf->aptUsing[0] = 1;/*what meaning? just set like Deltachrome*/

    return (TRUE);
}

int VIAReserveFBBuffer(ScreenPtr pScreen)
{
	/*get via rec*/
        ScrnInfoPtr pScrn = xf86Screens[pScreen->myNum];
	VIAPtr    pVia = VIAPTR(pScrn);
	/*get s3g rec*/
	ScrnConfPtr pScrnConf = VIAPTR(pScrn)->pScrnConf;
	DriConfPtr pDriConf = VIAPTR(pScrn)->pDriConf;
         /*although now 3d will dynamic alloc and free vidmem with drm, these setting seems to be unuseful*/
         /*but such steps will effect set modes in 3d dri*/
         /*when call s3gFillInModes() to setmodes, if (pDevInfo->backOffset != pDevInfo->depthOffset), DRI will think that there is no backbuf*/
    	ulong bufSize = pScrnConf->scrnScanline * pScrnConf->scrnDelta;
    	int totalBufs = pScrnConf->memScanline / pScrnConf->scrnScanline;
    	int texScanline,XBuffers,needBufs;
	Bool flag;

    	if (pScrnConf->Bpp == 2) /*homer, in s3g, &&pScrnConf->z24, no use?*/
	{
        	needBufs = 4;
        	flag = TRUE;
    	} 
	else 
	{
        	needBufs = 3;
        	flag = FALSE;
    	}
       	pScrnConf->xaaBufs = (totalBufs - needBufs) / 3;  /*reserve 1/3 of fb for off-screen*/

	XBuffers = pScrnConf->xaaBufs + 1;         
	pScrnConf->XScanline = XBuffers * pScrnConf->scrnScanline;
   	 /* reserver memory for back/depth */
    	pDriConf->frontOffset = pScrnConf->fbOffset;  
	/*frontOffset means on-screen start?*/
   	 pDriConf->backOffset = pVia->FBFreeStart; /*+ XBuffers * bufSize;*/
	/*because we do not use s3g XAA driver, so XBuffers is unuseful?*/
	/*backOffset begins after XAA(offscreen)*/
   	 pDriConf->depthOffset = pDriConf->backOffset + bufSize;
	/*depthOffset just after back buffer, backbuffer size = bufSize?so small?*/
  	  pDriConf->backY = pDriConf->backOffset / pScrnConf->scrnDelta;
   	 pDriConf->depthY = pDriConf->depthOffset / pScrnConf->scrnDelta;
	/*reserve tex buffer*/
        texScanline = pScrnConf->memScanline - ((needBufs-1) + XBuffers) * pScrnConf->scrnScanline;
        if (flag) 
            pDriConf->fbTexOffset = pDriConf->depthOffset + 2 * bufSize;
        else
            pDriConf->fbTexOffset = pDriConf->depthOffset + bufSize;
        pDriConf->fbTexY = pDriConf->fbTexOffset / pScrnConf->scrnDelta;
        pDriConf->fbTexSize = texScanline * pScrnConf->scrnDelta;
#if 0
        /*pVia->FBFreeStart = pDriConf->fbTexOffset+pDriConf->fbTexSize;*/
    	xf86DrvMsg(pScreen->myNum, X_INFO,
			"[drm] after allocate s3g offsets, FBFreestart is: %08x, and free size is: %d bytes\n",
			pVia->FBFreeStart, pVia->FBFreeEnd - pVia->FBFreeStart);
#endif
    return TRUE;
}
#define TILEDFBINUSE 0
/*collect VIA screen info to fill in that of s3g*/
int VIAInitS3gConf(ScreenPtr pScreen)
{
	/*via rec*/
        ScrnInfoPtr pScrn = xf86Screens[pScreen->myNum];
	VIAPtr    pVia = VIAPTR(pScrn);
	/*s3g rec*/
	ScrnConfPtr pScrnConf = VIAPTR(pScrn)->pScrnConf;
	DriConfPtr pDriConf = VIAPTR(pScrn)->pDriConf;

        /*get the physical address*/
        pScrnConf->fbPhyAddr=pVia->FrameBufferBase;
        pVia->MmioBase = pVia->PciInfo->memBase[1];

	
	/*copy samm flag*/
	pScrnConf->samm=pVia->pBIOSInfo->SAMM;   
	/*homer now set the default single-screen*/
	pScrnConf->master=1;
	/*set screen*/
    	pScrnConf->virtualX = pScrn->virtualX;
    	pScrnConf->virtualY = pScrn->virtualY;
    	pScrnConf->bpp = pScrn->bitsPerPixel;
    	pScrnConf->Bpp = pScrn->bitsPerPixel >> 3;
    	pScrnConf->scrnDelta = pScrn->virtualX * pScrnConf->Bpp;
#if TILEDFBINUSE
    if (pScrnConf->bpp == 8)
    {
#endif
        /* 32: Alignment for nontiled mode */
        pScrnConf->scrnDelta = (pScrnConf->scrnDelta + 31) & (~31);
        pScrnConf->scrnScanline = pScrn->virtualY;
        /* no use */
        pScrnConf->scrnWit = pScrnConf->scrnDelta  >> 7;
#if TILEDFBINUSE
    } 
    else 
    {
        /* align to a tile width */
        pScrnConf->scrnDelta = (pScrnConf->scrnDelta + 127) & (~127);
        /* align to a tile height */
        /* 4k per tile */
        pScrnConf->scrnScanline = (pScrn->virtualY + 31) & (~31);
        pScrnConf->scrnWit = pScrnConf->scrnDelta  >> 7;
    }
#endif
    pScrnConf->scrnWip = pScrnConf->scrnDelta / pScrnConf->Bpp;

    /* set framebuffer parameter:size/offset/linear address */
    if (!pScrnConf->samm) 
    {
        /* non-samm case */
        pScrnConf->fbOffset = 0; 
     /*seems that pScrnConf->fbOffset in s3g is different from  pVIADRI->fbOffset in VIA*/
    /*pVIADRI->fbOffset = pVia->FBFreeStart*/
    /*pScrnConf->linearVideoOffset = (pAddrConf->fbFree - 800*600*2) & (~(0x1000-1));*/
        pScrnConf->fbSize = (unsigned long)pVia->videoRambytes;
        pScrnConf->scrnLinear = (unsigned long)pVia->FBBase;
    } 
    /* allocate offscreen scanline by scanline */
    pScrnConf->memDelta = pScrnConf->scrnDelta;
    /* usable scanline(subtrace the scrnScanline) */
    pScrnConf->memScanline = pScrnConf->fbSize / pScrnConf->memDelta;
    pScrnConf->XScanline = pScrnConf->memScanline;
    /*fb start and fb pitch*/
    pScrnConf->scrnStart = (unsigned long)pVia->FBBase;
    pScrnConf->scrnPitch = pScrnConf->scrnDelta;
    pScrnConf->scrnPhyAddr = pVia->FrameBufferBase;
    pScrnConf->fbPhyAddr = pVia->FrameBufferBase + pScrnConf->fbOffset;
	/*pScrnConf->fbSize = pVia->FBFreeEnd - pVia->FBFreeStart;*/
#if TILEDFBINUSE
    pScrnConf->tiled = 1; /*fb is tiled, like clb*/
#else
    pScrnConf->tiled = 0;
#endif
    pScrnConf->driEnabled = 1;
    pScrnConf->accel = 1;
    pScrnConf->video = 1;
#if TILEDFBINUSE
    /*cracked, exactly should be pAddrConf->aptVirtAddr[0]*/
    pScrnConf->scrnTile = pScrnConf->scrnLinear;
#else
    pScrnConf->scrnTile = 0;
#endif
    /*
    pDriConf->dmaBufNum = S3G_NUM_BUFFERS;
    pDriConf->dmaBufSize = S3G_BUFFER_SIZE;
    */
    /*others*/
    pDriConf->drmFd = pVia->drmFD;
    pDriConf->pDRIInfo= pVia->pDRIInfo;

    return TRUE;
}

static int
VIAS3GFinishScreenInit(ScreenPtr pScreen)
{
    ScrnInfoPtr pScrn = xf86Screens[pScreen->myNum];
    /*DrvInfoPtr  pDrv = DRVPTR(pScrn);*/
    VIAPtr pVia = VIAPTR(pScrn);
    ScrnConfPtr pScrnConf=VIAPTR(pScrn)->pScrnConf;
    DriConfPtr  pDriConf=VIAPTR(pScrn)->pDriConf;
    ServerInfoPtr pServerInfo = VIAPTR(pScrn)->pServerInfo;

    /*get additional datas for vt3336*/
    memcpy(&pServerInfo->AddrConf, pVia->pAddrConf, sizeof(AddrConfRec));
    memcpy(&pServerInfo->ChipConf, pVia->pChipConf, sizeof(ChipConfRec));
    memcpy(&pServerInfo->ScrnConf, pVia->pScrnConf, sizeof(ScrnConfRec));
    memcpy(&pServerInfo->DriConf, pVia->pDriConf, sizeof(DriConfRec));
    memcpy(&pServerInfo->PciConf, pVia->pPciConf, sizeof(PciConfRec));

    if(H5S1_UMA_CHIPID)
    {
        pServerInfo->chipIndex = S3G_INV;
        pServerInfo->chipType = CHIP_AGP; /*CHIP_AGP*/
        pServerInfo->chipIndex = S3G_INV; /*S3G_CLB*/
        pServerInfo->chipSubIndex =CHIPID_H5S1; /*CHIP_CLB*/
        pServerInfo->isAGP=!pVia->IsPCI;
    }
    
    pServerInfo->sareaPrivOffset = sizeof(XF86DRISAREARec);
    /*
    pServerInfo->dmaBufNum = pDriConf->dmaBufNum;
    pServerInfo->dmaBufSize = pDriConf->dmaBufSize;
    */
    pServerInfo->dmaBufOffs = pDriConf->dmaBufOffs;
    
    pServerInfo->bufGBD = pScrnConf->gbd;
    pServerInfo->bufBpp = pScrnConf->Bpp;
    pServerInfo->bufSize = pScrnConf->scrnScanline * pScrnConf->scrnPitch;
    pServerInfo->bufPitch = pScrnConf->scrnPitch;
    pServerInfo->bufWit = pScrnConf->scrnWit;

    if ((pScrnConf->Bpp == 4)
        || (pScrnConf->Bpp == 2 && !pDriConf->z24)) {
        pServerInfo->zbufGBD = pScrnConf->gbd;
        pServerInfo->zbufBpp = pScrnConf->Bpp;
        pServerInfo->zbufSize = pScrnConf->scrnScanline * pScrnConf->scrnPitch;
        pServerInfo->zbufPitch = pScrnConf->scrnPitch;
        pServerInfo->zbufWit = pScrnConf->scrnWit;
    } else { /* 16bpp mode but 32bpp depth buffer */
        pServerInfo->zbufGBD = (2 * pScrnConf->scrnDelta) | (1<<16) | (1<<18);
        pServerInfo->zbufBpp = 2 * pScrnConf->Bpp;
        pServerInfo->zbufSize = pScrnConf->scrnScanline * (2 * pScrnConf->scrnPitch);
        pServerInfo->zbufPitch = 2 * pScrnConf->scrnPitch;
        pServerInfo->zbufWit = 2 * pScrnConf->scrnWit;
    }
    
    pServerInfo->frontOffset = pDriConf->frontOffset;
    pServerInfo->backOffset = pDriConf->backOffset;
    pServerInfo->depthOffset = pDriConf->depthOffset;
    pServerInfo->backY = pDriConf->backY;
    pServerInfo->depthY = pDriConf->depthY;
    
    pServerInfo->dmaHandle = pDriConf->dmaHandle;
    pServerInfo->frontHandle = pDriConf->frontHandle;
    pServerInfo->backHandle = pDriConf->backHandle;
    pServerInfo->depthHandle = pDriConf->depthHandle;
    pServerInfo->fbTexOffset = pDriConf->fbTexOffset;
    pServerInfo->fbTexSize = pDriConf->fbTexSize;

    pServerInfo->shadowSize = pDriConf->shadowSize;
    pServerInfo->shadowHandle = pDriConf->shadowHandle;
    
    pServerInfo->agpTexOffset = pDriConf->agpTexOffset;
    pServerInfo->agpTexSize = pDriConf->agpTexSize;
    pServerInfo->agpTexHandle = pDriConf->agpTexHandle;

    pServerInfo->mmioHandle = pVia->registerHandle;

    pDriConf->allowPageFlip = 0;
    /*sarea init*/
    drm_s3g_sarea_t  *pSAREA = DRIGetSAREAPrivate(pScreen);
    pSAREA->page_flip=0;
    return TRUE;
}


static __GLXvisualConfig * S3GInitVisConf16Bpp(ScreenPtr pScreen,int *numConfigs)
{
    __GLXvisualConfig *pConfigs=NULL ;
    ScrnInfoPtr pScrn = xf86Screens[pScreen->myNum];
    DriConfPtr  pDriConf=VIAPTR(pScrn)->pDriConf;
    int i, db, stencil, accum;

    *numConfigs = 8;

    pConfigs = (__GLXvisualConfig*)xcalloc(sizeof(__GLXvisualConfig),*numConfigs);
    if (!pConfigs) {
        return NULL;
    }
    
    i = 0;
    for (db = 0 ; db <= 1 ; db++) 
    { /* for accum buffer */
       for (accum = 0 ; accum <= 1 ; accum++) 
       { /* for stencil buffer */
          for (stencil = 0 ; stencil <= 1 ; stencil++) 
          { /* for depth buffer */
              pConfigs[i].vid         = -1;
              pConfigs[i].class       = -1;
              pConfigs[i].rgba        = TRUE;
              pConfigs[i].redSize     = 5;
              pConfigs[i].greenSize   = 6;
              pConfigs[i].blueSize    = 5;
              pConfigs[i].alphaSize   = 0;
              pConfigs[i].redMask     = 0x0000F800;
              pConfigs[i].greenMask   = 0x000007E0;
              pConfigs[i].blueMask    = 0x0000001F;
              pConfigs[i].alphaMask   = 0;
              if (accum) 
             {
                  pConfigs[i].accumRedSize    = 16;
                  pConfigs[i].accumGreenSize  = 16;
                  pConfigs[i].accumBlueSize   = 16;
                  pConfigs[i].accumAlphaSize  = 0;
              } 
              else 
              {
                  pConfigs[i].accumRedSize    = 0;
                  pConfigs[i].accumGreenSize  = 0;
                  pConfigs[i].accumBlueSize   = 0;
                  pConfigs[i].accumAlphaSize  = 0;
              }
              if (db) 
             {
                  pConfigs[i].doubleBuffer    = TRUE;
              } 
              else 
              {
                  pConfigs[i].doubleBuffer    = FALSE;
              }
              pConfigs[i].stereo        = FALSE;
              pConfigs[i].bufferSize    = 16;             
              if (pDriConf->z24) 
              { 
                  pConfigs[i].depthSize = 24;
              } else 
              {
                  pConfigs[i].depthSize = 16;
              }
              if (stencil) 
              {
                  pConfigs[i].stencilSize  = 8;
              } 
              else 
              {
                  pConfigs[i].stencilSize  = 0;
              }
              pConfigs[i].auxBuffers       = 0;
              pConfigs[i].level            = 0;
              if (accum) 
              {
                  pConfigs[i].visualRating    = GLX_SLOW_CONFIG;
              } else 
              {
                  pConfigs[i].visualRating    = GLX_NONE;
              }
              pConfigs[i].transparentPixel    = GLX_NONE;
              pConfigs[i].transparentRed      = 0;
              pConfigs[i].transparentGreen    = 0;
              pConfigs[i].transparentBlue     = 0;
              pConfigs[i].transparentAlpha    = 0;
              pConfigs[i].transparentIndex    = 0;
              i++;
           }
        }
    }

    return pConfigs;
}


static __GLXvisualConfig * S3GInitVisConf32Bpp(ScreenPtr pScreen,int *numConfigs)
{
    __GLXvisualConfig *pConfigs=NULL ;
    int i, db, accum,stencil;

    *numConfigs = 8;
    pConfigs = (__GLXvisualConfig*)xcalloc(sizeof(__GLXvisualConfig),*numConfigs);
    if (!pConfigs) 
    {
        return NULL;
    }
    
    i = 0;
    for (db = 0 ; db <= 1 ; db++) 
    {
        for (accum = 0 ; accum <= 1 ; accum++) 
        { /* and stencil */
            for (stencil = 0 ; stencil <= 1 ; stencil++) 
            {
                pConfigs[i].vid         = -1;
                pConfigs[i].class       = -1;
                pConfigs[i].rgba        = TRUE;
                pConfigs[i].redSize     = 8;
                pConfigs[i].greenSize   = 8;
                pConfigs[i].blueSize    = 8;
                pConfigs[i].alphaSize   = 8;
                pConfigs[i].redMask     = 0x00FF0000;
                pConfigs[i].greenMask   = 0x0000FF00;
                pConfigs[i].blueMask    = 0x000000FF;
                pConfigs[i].alphaMask   = 0xFF000000;
                if (accum) 
                {
                    pConfigs[i].accumRedSize    = 16;
                    pConfigs[i].accumGreenSize  = 16;
                    pConfigs[i].accumBlueSize   = 16;
                    pConfigs[i].accumAlphaSize  = 16;
                }
                else 
                {
                    pConfigs[i].accumRedSize    = 0;
                    pConfigs[i].accumGreenSize  = 0;
                    pConfigs[i].accumBlueSize   = 0;
                    pConfigs[i].accumAlphaSize  = 0;
                }
                if (db) 
                {
                    pConfigs[i].doubleBuffer    = TRUE;
                } else 
                {
                    pConfigs[i].doubleBuffer    = FALSE;
                }
                pConfigs[i].stereo            = FALSE;
                pConfigs[i].bufferSize        = 32;
                pConfigs[i].depthSize     = 24;
                if (stencil) {
                    pConfigs[i].stencilSize   = 8;
                }
                else 
                {
                    pConfigs[i].stencilSize   = 0;
                }
                pConfigs[i].auxBuffers   = 0;
                pConfigs[i].level        = 0;
                if (accum) 
                {
                    pConfigs[i].visualRating    = GLX_SLOW_CONFIG;
                } 
                else 
                {
                    pConfigs[i].visualRating    = GLX_NONE;
                }
                pConfigs[i].transparentPixel    = GLX_NONE;
                pConfigs[i].transparentRed      = 0;
                pConfigs[i].transparentGreen    = 0;
                pConfigs[i].transparentBlue     = 0;
                pConfigs[i].transparentAlpha    = 0;
                pConfigs[i].transparentIndex    = 0;
                i++;
            }
        }
    }
    return pConfigs;    
}

static int S3GInitVisualConfigs(ScreenPtr pScreen)
{
    ScrnInfoPtr pScrn = xf86Screens[pScreen->myNum];
    DriConfPtr  pDriConf=VIAPTR(pScrn)->pDriConf;
    __GLXvisualConfig *pConfigs=NULL ;
    int numConfigs;
    
    if (pScrn->bitsPerPixel == 16) 
    {
        pConfigs = S3GInitVisConf16Bpp(pScreen,&numConfigs);
    } 
    else if (pScrn->bitsPerPixel == 32) 
    {
        pConfigs = S3GInitVisConf32Bpp(pScreen,&numConfigs);        
    } 
    else 
    {
        return FALSE;
    }

    if (pConfigs) 
    {
        pDriConf->pVisualConfigs = pConfigs;
        GlxSetVisualConfigs(numConfigs, pConfigs, NULL);
        return TRUE;
    } 
    else 
    {
        return FALSE;
    }
}
/*homer end*/
