/*
 * Copyright 1998-2007 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2007 S3 Graphics, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHoOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
#ifdef HAVE_CONFIG_H            /*Al Zhang [2006.3.31]: add for driver compiling on X11R7(FC5)*/
#include "config.h"
#endif

#ifndef _VIA_DRIVER_H_
#define _VIA_DRIVER_H_ 1

/*#define DEBUG_PRINT*/
#ifdef DEBUG_PRINT
#define DEBUG(x) x
#else
#define DEBUG(x)
#endif

#include "vgaHW.h"
#include "xf86.h"
#include "xf86Resources.h"
#include "xf86Pci.h"
#include "xf86PciInfo.h"
#include "xf86_OSproc.h"
#include "compiler.h"
#include "xf86Cursor.h"
#include "mipointer.h"
#include "micmap.h"

#define USE_FB
#ifdef USE_FB
#include "fb.h"
#else
#include "cfb.h"
#include "cfb16.h"
#include "cfb32.h"
#endif

#include "xf86cmap.h"
#include "vbe.h"
#include "xaa.h"

#include "via_regs.h"
#include "via_bios.h"
#include "via_gpioi2c.h"

#ifdef XF86DRI
#define _XF86DRI_SERVER_
#include "sarea.h"
#include "dri.h"
#include "GL/glxint.h"
#include "via_dri.h"
#include "dristruct.h"
#endif

#include <X11/extensions/randr.h>

#include "close/capture.h"
#include "close/VidStruc.h"

#define DRIVER_NAME     "via"
#define DRIVER_VERSION  "4.1.0"
#define VERSION_MAJOR   4
#define VERSION_MINOR   1
#define VERSION_OS      0   /* 0: for 32 bits OS, 1: for 64 bits OS */
#define PATCHLEVEL      79
#define VIA_VERSION     ((VERSION_MAJOR<<24) | (VERSION_MINOR<<16) | PATCHLEVEL)

#ifdef XORG_VERSION_CURRENT 
#if XORG_VERSION_CURRENT >= XF86_VERSION_NUMERIC(6,9,0,0,0)
#define _XORG_VER_6_9_LATER_
#endif

#if XORG_VERSION_CURRENT >= XF86_VERSION_NUMERIC(6,8,0,0,0)
#define _XORG_VER_6_8_LATER_
#endif
#endif

#if XF86_VERSION_CURRENT < XF86_VERSION_NUMERIC(4,3,99,14,0)
#undef XFREE86_44
#else
#define XFREE86_44 1
#endif

#define VGAIN8(addr)        MMIO_IN8(pVia->MapBase+0x8000, addr)
#define VGAIN16(addr)       MMIO_IN16(pVia->MapBase+0x8000, addr)
#define VGAIN(addr)         MMIO_IN32(pVia->MapBase+0x8000, addr)

#define VGAOUT8(addr, val)  MMIO_OUT8(pVia->MapBase+0x8000, addr, val)
#define VGAOUT16(addr, val) MMIO_OUT16(pVia->MapBase+0x8000, addr, val)
#define VGAOUT(addr, val)   MMIO_OUT32(pVia->MapBase+0x8000, addr, val)

#define INREG(addr)         MMIO_IN32(pVia->MapBase, addr)
#define OUTREG(addr, val)   MMIO_OUT32(pVia->MapBase, addr, val)
#define INREG16(addr)       MMIO_IN16(pVia->MapBase, addr)
#define OUTREG16(addr, val) MMIO_OUT16(pVia->MapBase, addr, val)

#define VIA_PIXMAP_CACHE_SIZE   (2048 * 1024)
#define VIA_CURSOR_SIZE         (16 * 1024)
#define VIA_FB_CURSOR_SIZE      (8 * 1024)      /* reserve for frame buffer device driver */
#define VIA_VQ_SIZE             (256 * 1024)

/* S.T.Chen [2005.11.02]: used by get video memory size */
/* device ID */
#define CLE266              0x3123
#define KM400               0x3205
#define CN400_FUNCTION2     0x2259
#define CN400_FUNCTION3     0x3259
/* support VT3314 chipset */
#define CN900_FUNCTION2     0x2314
#define CN900_FUNCTION3     0x3208
/* VT3324 chipset */
#define CX700_FUNCTION2     0x2324
#define CX700_FUNCTION3     0x3324
/* VT3204 chipset*/
#define KM800_FUNCTION3      0x3204
/* VT3336 chipset*/
#define KM890_FUNCTION3      0x3336
/* VT3327 chipset*/
#define P4M890_FUNCTION3     0x3327
/* VT3293 chipset*/
#define CN750_FUNCTION3     0x3208
/* VT3364 chipset*/
#define P4M900_FUNCTION3    0x3364

/*homer, for vt3336/vt3364(h5s1) 3D support*/
#define H5S1_UMA_CHIPID (pVia->ChipId==PCI_CHIP_VT3336 || pVia->ChipId == PCI_CHIP_VT3364)
/*homer end*/

/*add to fix the "Undefined Symbol:vgaHWddc1SetSpeed" problem on FC5(32/64)
    used by function :VIAvgaHWddc1SetSpeed()*/
/* vertical timings */
#define DISPLAY_END 0x04
#define BLANK_START DISPLAY_END
#define SYNC_START BLANK_START
#define SYNC_END 0x09
#define BLANK_END SYNC_END
#define V_TOTAL BLANK_END

#define MAX_CURS 64

struct ViaModePriv {
    char   id[12]; /* "via" */
};

extern struct ViaModePriv ViaModePrivate;
extern DisplayModeRec ViaSupportModes[];

struct _vgaDdcSave {
    unsigned char cr03;
    unsigned char cr06;
    unsigned char cr07;
    unsigned char cr09;
    unsigned char cr10;
    unsigned char cr11;
    unsigned char cr12;
    unsigned char cr15;
    unsigned char cr16;
    unsigned char msr;
};


typedef struct {
    unsigned int    mode, refresh, resMode;
    int             countWidthByQWord;
    int             offsetWidthByQWord;

    unsigned char   Clock;

    /* Sequencer registers */
    unsigned char   SRRegs[256];
    
    /* CRTC registers */
    unsigned char   CRTCRegs[256];
    unsigned char   TVRegs[0xCF];
    unsigned char   TVRegs2[0xCF];
    unsigned long   IntegratedTVRegs[60];
    unsigned char   VT1636Regs[9];      /* We will save from Tx08 to Tx10. */
/*    unsigned char   LCDRegs[0x40];*/

    /* HW Cursor registers */
    CARD32          dwCursorMode, dwCursorPOS, dwCursorORG, dwCursorBG, dwCursorFG;

} VIARegRec, *VIARegPtr;
typedef struct hwLock {
    volatile unsigned int lock;
    char                      padding[60];
} hwLock, *hwLockPtr;


#define     EXPAND_BUF_CNT     2
typedef struct _VIADISP3DSCALCTRL 
{
    Bool    getShareArea;
    unsigned int     xcontext;
    int     drmFD;
    unsigned int hSArea;
    unsigned int pSArea;    
    drmLockPtr	plock;
    int     VirtualHActive;
    int     OrigHActive;
    int     OrigVActive;
    int     RealHActive;
    int     RealVActive;
    int     BitsPerPixel;
    int     DISP3DScalIGAPath;
    int     OrigFBAddress;
    int     PanningX;
    int     PanningY;
    CARD32  DISP3DScalingBufPitch;
    CARD32  DISP3DScalingBufAddr[EXPAND_BUF_CNT];
    int     DISP3DScalingBufIndex;
    CARD32  DISP3DScalingBufTotalSize;
    Bool    IsNeedAdjustScalScrn;
    int     CursorX;
    int     CursorY;
}VIADISP3DSCALCTRL, *VIADISP3DSCALCTRLPtr;


typedef struct _VIA {
    VIARegRec           SavedReg;
    VIARegRec           ModeReg;
    xf86CursorInfoPtr   CursorInfoRec;
    int                 stateMode;
    VIAModeInfoPtr      VIAModeList;
    Bool                ModeStructInit;
    int                 Bpp, Bpl, ScissB;
    unsigned            PlaneMask;

    unsigned long       videoRambytes;
    int                 videoRamKbytes;

    int                 FBFreeStart;
    int                 FBFreeEnd;
    int                 CursorStart;
    int                 VQStart;
    int                 VQEnd;
    int                 IntegratedTVBufferStart;

    /* These are physical addresses. */
    unsigned long       FrameBufferBase;
    unsigned long       MmioBase;

    /* These are linear addresses. */
    CARD8		*MapBase;
    CARD8		*VidMapBase;
    CARD8		*BltBase;
    CARD8		*IntegratedTVMapBase;
    CARD8		*MapBaseDense;
    CARD8		*FBBase;
    CARD8		*FBStart;

    Bool                PrimaryVidMapped;
    int                 dacSpeedBpp;
    int                 minClock, maxClock;
    int                 MCLK, REFCLK, LCDclk;
    double              refclk_fact;

    /* Here are all the Options */
    Bool                VQEnable;
    Bool                pci_burst;
    Bool                NoPCIRetry;
    Bool                hwcursor;
    Bool                ForceSWCursor;
    Bool                NoAccel;
    Bool                NoXVideo;
    Bool                shadowFB;       

    CloseScreenProcPtr  CloseScreen;
    pciVideoPtr         PciInfo;
    PCITAG              PciTag;
    int                 Chipset;
    int                 ChipId;
    int                 ChipRev;
    vbeInfoPtr          pVbe;
    int                 EntityIndex;

    /* Support for shadowFB and rotation */
    unsigned char*      ShadowPtr;
    int                 ShadowPitch;
    void                (*PointerMoved)(int index, int x, int y);

    /* Support for XAA acceleration */
    XAAInfoRecPtr       AccelInfoRec;
    xRectangle          Rect;
    CARD32              SavedCmd;
    CARD32              SavedFgColor;
    CARD32              SavedBgColor;
    CARD32              SavedPattern0;
    CARD32              SavedPattern1;
    CARD32              SavedPatternAddr;

    /* Support for Int10 processing */
    xf86Int10InfoPtr    pInt10;

    /* BIOS Info Ptr */
    VIABIOSInfoPtr      pBIOSInfo;
    VGABIOSVERPtr       pBIOSVer;

    Bool                IsSuspending;
    Bool                IsVIAModeInitRunOnceDone;
    
    /* Support for DGA */
    int                 numDGAModes;
    DGAModePtr          DGAModes;
    Bool                DGAactive;
    int                 DGAViewportStatus;

    /* The various wait handlers. */
    int                 (*myWaitIdle)(struct _VIA*);

    /* I2C & DDC */
    I2CBusPtr           I2C_Port1;
    I2CBusPtr           I2C_Port2;
    xf86MonPtr          DDC1;
    xf86MonPtr          DDC2;

    /* MHS */
    Bool                IsSecondary;
    Bool                HasSecondary;

    /* Capture information */
    VIACAPINFO     CapInfo[2];      /* 2 capture information */

/*
    CARD32              Cap0_Deinterlace;
    CARD32              Cap1_Deinterlace;

    Bool                Cap0_FieldSwap;
    Bool                NoCap0_HFilter;
    Bool                Capture_OverScanOff;
    Bool                NoMPEGHQV_VFilter;
*/
#ifdef XF86DRI
    Bool directRenderingEnabled;
/*JohnsonZheng[2007070413]*/
    Bool useAgp;
/*JohnsonZheng-End*/
    DRIInfoPtr pDRIInfo;
    int drmFD;
    int numVisualConfigs;
    __GLXvisualConfig* pVisualConfigs;
    VIAConfigPrivPtr pVisualConfigsPriv;
    drm_handle_t agpHandle;
    unsigned long registerHandle;
    pointer agpAddr;
    pointer agpBase;
    unsigned int agpSize;
    Bool IsPCI;
    Bool drixinerama;
#endif

    Bool    V4LEnabled;
    CARD16  ActiveDevice;   /* if SAMM, non-equal pBIOSInfo->ActiveDevice */
    unsigned char       *CursorImage;
    CARD32  CursorFG;
    CARD32  CursorBG;
    CARD32  CursorMC;
    PVIDDATA pVidData;
    int     shmid;      /* shared memory ID */

    char                video0Path[20];
    char                video1Path[20];
    char                video2Path[20];
    /*homer agp flush drm*/
    unsigned long agp_handle;/*map public agp  area*/
    /*homer end*/

    /* Rotation */
    Bool    RandRRotation;
    int     SWRotate;           /* For Software rotation */
    Bool    IsHWRotateEnabled;  /* For Hardware rotation */
    int     RotateFBStart;
    int     RotateFBEnd;

    /* Display 3D engine scaling*/
    Bool    DISP3DScaling;
    int     DISP3DScalingBufferStart;
    VIADISP3DSCALCTRL DISP3DSCALCTRLInfo;
    int     Cursor3DScalingBufferStart;

    /* For MAMM case, VIA as secondary display device. */
    Bool    IsMAMMEnable; 
    
    /*homer for vt3336(h5s1) 3d support*/
    AddrConfPtr pAddrConf;
    ChipConfPtr pChipConf;
    DriConfPtr pDriConf;
    ScrnConfPtr pScrnConf;
    ServerInfoPtr pServerInfo;
    PciConfPtr pPciConf;
    /*homer end*/
} VIARec, *VIAPtr;


typedef struct
{
    Bool IsDRIEnabled;

    Bool HasSecondary;
    Bool BypassSecondary;
    /*These two registers are used to make sure the CRTC2 is
      retored before CRTC_EXT, otherwise it could lead to blank screen.*/
    Bool IsSecondaryRestored;
    Bool RestorePrimary;

    ScrnInfoPtr pSecondaryScrn;
    ScrnInfoPtr pPrimaryScrn;
}VIAEntRec, *VIAEntPtr;

/*
 * structure for passing information to UpdateOverlay fn
 */
typedef struct _RECTL
{
    long     left;
    long     top;
    long     right;
    long     bottom;
} RECTL, *RECTLPtr;

/* Shortcuts.  These depend on a local symbol "pVia". */

#define WaitIdle()      pVia->myWaitIdle(pVia)
#define VIAPTR(p)       ((VIAPtr)((p)->driverPrivate))


/* Prototypes. */
void VIAAdjustFrame(int scrnIndex, int y, int x, int flags);
Bool VIASwitchMode(int scrnIndex, DisplayModePtr mode, int flags);
Bool VIACheckLCDExpand(ScrnInfoPtr pScrn);
void VIADebugBreak(void);
#ifdef XFree86LOADER
Bool VIALoadVideoOption(VIAPtr pVia,LPVIAGRAPHICINFO lpVIAGraphicInfo);
#endif
void VIACheckCursorTypeToUse(ScrnInfoPtr pScrn);

/* In via_cursor.c. */
Bool VIAHWCursorInit(ScreenPtr pScreen);
void VIAShowCursor(ScrnInfoPtr);
void VIAHideCursor(ScrnInfoPtr);
void VIAARGBCursorInit(ScrnInfoPtr pScrn);


/* In via_accel.c. */
Bool VIAInitAccel(ScreenPtr);
void VIAInitialize2DEngine(ScrnInfoPtr);
void VIAAccelSync(ScrnInfoPtr);


/* In via_shadow.c */
void VIAPointerMoved(int index, int x, int y);
void VIARefreshArea(ScrnInfoPtr pScrn, int num, BoxPtr pbox);
void VIARefreshArea8(ScrnInfoPtr pScrn, int num, BoxPtr pbox);
void VIARefreshArea16(ScrnInfoPtr pScrn, int num, BoxPtr pbox);
void VIARefreshArea24(ScrnInfoPtr pScrn, int num, BoxPtr pbox);
void VIARefreshArea32(ScrnInfoPtr pScrn, int num, BoxPtr pbox);


/* In via_bios.c */
Bool VIAPrimarySetMode(ScrnInfoPtr pScrn);
Bool VIAGetBIOSInfoFromROM(VIAPtr pVia, unsigned char flags);


/* In via_dga.c */
Bool VIADGAInit(ScreenPtr);

/* In via_i2c.c */
Bool VIAI2CInit(ScrnInfoPtr pScrn);

/* In via_gpioi2c.c */
Bool GPIOI2C_Write(VIABIOSInfoPtr pBIOSInfo, int SubAddress, CARD8 Data);
Bool GPIOI2C_Read(VIABIOSInfoPtr pBIOSInfo, int SubAddress, CARD8 *Buffer, int BufferLen);
Bool GPIOI2C_ReadByte(VIABIOSInfoPtr pBIOSInfo, int SubAddress, CARD8 *Buffer);
Bool GPIOI2C_Initial(VIABIOSInfoPtr pBIOSInfo, CARD8 SlaveDevice);

/*In via_video.c*/
void viaInitVideo(ScreenPtr pScreen);
void viaExitVideo(ScrnInfoPtr pScrn);
void viaResetVideo(ScrnInfoPtr pScrn);
void viaSaveVideo(ScrnInfoPtr pScrn);
void viaRestoreVideo(ScrnInfoPtr pScrn);

/* In Hw.c */
void VIADIPortPadOn(int DIPort);
void VIADIPortPadOff(int DIPort);

/* For utility interface */
void VIADisplayExtensionInit(ScrnInfoPtr pScrn);

/*For TV 3d Scaling*/
void hw3d_Scaling_PCI(VIAPtr pVia, unsigned long dstAddr, unsigned long srcAddr,unsigned long dstPitch, unsigned long srcPitch, unsigned long srcHeight,unsigned long srcWidth, RECTL rSource, RECTL rDst, int bitsPerPixel,int scalingmode);
void hw3d_Scaling_AGP(VIAPtr pVia, unsigned long dstAddr, unsigned long srcAddr,unsigned long dstPitch, unsigned long srcPitch, unsigned long srcHeight,unsigned long srcWidth, RECTL rSource, RECTL rDst, int bitsPerPixel,int scalingmode);
void hw3d_CursorPositionUpdate(VIABIOSInfoPtr pBIOSInfo);
void hw3d_MonoCursorImageAdjust(VIAPtr pVia);
void hw3d_ARGBCursorImageAdjust(VIAPtr pVia);


/* via_dri.h */
Bool VIADRIScreenInit(ScreenPtr pScreen);
void VIADRICloseScreen(ScreenPtr pScreen);
Bool VIADRIFinishScreenInit(ScreenPtr pScreen);
Bool VIASet3DSyncInfoParas(ScreenPtr pScreen);


/* Global Variables */
extern CARD8   *MMIOMapBase;

#ifdef XF86DRI
/* Lock the hardware and validate our state.  
 */
#define viaLOCK_HARDWARE(p3DSCALCTRLInfo)                                	\
	do {                                                    \
    	    char __ret = 0;                                     \
    	    DRM_CAS(((XF86DRISAREAPtr)p3DSCALCTRLInfo->plock), p3DSCALCTRLInfo->xcontext,        \
        	(DRM_LOCK_HELD|p3DSCALCTRLInfo->xcontext), __ret);      \
	} while (0)


/* Release the kernel lock.
 */
#define viaUNLOCK_HARDWARE(p3DSCALCTRLInfo)                                  	\
	DRM_UNLOCK(p3DSCALCTRLInfo->drmFD, ((XF86DRISAREAPtr)p3DSCALCTRLInfo->plock), p3DSCALCTRLInfo->xcontext);	
#endif

#endif  /* _VIA_DRIVER_H_ */
