/*
 * Copyright 1998-2007 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2007 S3 Graphics, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
#include "via_driver.h"
#include "via_dvi.h"
#include "via_hw.h"
#include "close/tvenc.h"
#include "via_gpioi2c.h"
#include "via_lcd.h"

/* Extern Functions Prototypes Declaration */
extern int SearchModeSetting(int ModeInfoIndex);
extern void fill_crtc_timing(VIABIOSInfoPtr pBIOSInfo, struct crt_mode_table *crt_table, int mode_array, int bpp_byte, int set_iga);
extern int g_DeviceI2CPort;

static void Write_REG_TMDS(VIABIOSInfoPtr pBIOSInfo, int index, CARD8 data);
static void VIADVIPatchSkew(VIABIOSInfoPtr pBIOSInfo, int DIPort);
static void VIADVIPatchSkew_DVP0(VIABIOSInfoPtr pBIOSInfo);
static void VIADVIPatchSkew_DVP1(VIABIOSInfoPtr pBIOSInfo);
static void VIADVIPatchSkew_DFPLOW(VIABIOSInfoPtr pBIOSInfo);
static Bool Read_REG_TMDS(VIABIOSInfoPtr pBIOSInfo, int index, CARD8 *pData);
static int VIATMDSIdentify_AD9389(VIABIOSInfoPtr pBIOSInfo);
static int VIATMDSIdentify_SiI9030(VIABIOSInfoPtr pBIOSInfo);
static int VIATMDSIdentify_Silicon164(VIABIOSInfoPtr pBIOSInfo);
static int VIATMDSIdentify_VT1632(VIABIOSInfoPtr pBIOSInfo);
static int VIATMDSIdentify_IntegratedTMDS(VIABIOSInfoPtr pBIOSInfo);
static void VIATMDSRegInit_AD9389(VIABIOSInfoPtr pBIOSInfo);
static void VIATMDSRegInit_SiI9030(VIABIOSInfoPtr pBIOSInfo);
static void VIATMDSRegInit_Silicon164(VIABIOSInfoPtr pBIOSInfo);
static void VIATMDSRegInit_VT1632(VIABIOSInfoPtr pBIOSInfo);
static void VIAInitDVIDIPort(VIABIOSInfoPtr pBIOSInfo, int IdentifyState);


void VIADVISetMode(VIABIOSInfoPtr pBIOSInfo)
{
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIADVISetMode!!\n"));

    VIASetDFPMode(pBIOSInfo);
    VIAEnableDFP(pBIOSInfo);
}

static
void VIADVIPatchSkew(VIABIOSInfoPtr pBIOSInfo, int DIPort)
{
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIADVIPatchSkew: DIPort=%d\n", DIPort));
    
    switch (DIPort)
    {
        case VIA_DI_DVP0:
            VIADVIPatchSkew_DVP0(pBIOSInfo);
            break;

        case VIA_DI_DVP1:
            VIADVIPatchSkew_DVP1(pBIOSInfo);
            break;
            
        case VIA_DI_DFPLOW:
            VIADVIPatchSkew_DFPLOW(pBIOSInfo);            
            break;

        case VIA_DI_DFPHIGH:
            break;

        default:
            break;
    }

    /* RainLin [2006.10.18]: Replace the default patch if get user DPA settings from xorg: */
    if (pBIOSInfo->UserDPASettingInfo.HasUserSetting_Gfx)
    {
        VIASetDPA_Gfx(DIPort, &(pBIOSInfo->UserDPASettingInfo.GfxDPA));
    }
}

static void
VIADVIPatchSkew_DVP0(VIABIOSInfoPtr pBIOSInfo)
{
    switch (pBIOSInfo->Chipset)
    {
        case VIA_P4M890:
        {
            if((pBIOSInfo->CrtcHDisplay==1600)&&(pBIOSInfo->CrtcVDisplay==1200))
                write_reg_mask(CR96, VIACR, 0x03, BIT0+BIT1+BIT2);
            else
                write_reg_mask(CR96, VIACR, 0x07, BIT0+BIT1+BIT2);
            break;
        }

        case VIA_P4M900:
        {
            /* RainLin [2006.11.06]: Validate by CN896(VT5943A-2), VN896(VT6363A1-2)*/
            /* and P4M900(VT8450C/VT8450D). */
            write_reg_mask(CR96, VIACR, 0x02, BIT0+BIT1+BIT2);
            break;
        }

        default:
        {
           break;
        }
    }
}

static void
VIADVIPatchSkew_DVP1(VIABIOSInfoPtr pBIOSInfo)
{
    switch (pBIOSInfo->Chipset)
    {
        case VIA_CX700:
        case VIA_CN750:
        {
            if(pBIOSInfo->TMDSSettingInfo.ChipID == VIA_AD9389); 
                write_reg_mask(CR9B, VIACR, 0x06, BIT0+BIT1+BIT2);
            break;
        }

        default:
        {
           break;
        }
    }
}

static void
VIADVIPatchSkew_DFPLOW(VIABIOSInfoPtr pBIOSInfo)
{
    switch (pBIOSInfo->Chipset)
    {
        case VIA_K8M890:
        {
            write_reg_mask(CR99, VIACR, 0x03, BIT0+BIT1);
            break;
        }
        
        case VIA_P4M900:        
        {
            /* RainLin [2006.11.06]: Validate by CN896(VT5943A-2), VN896(VT6363A1-2)*/
            /* and P4M900(VT8450C/VT8450D). */
            write_reg_mask(CR99, VIACR, 0x09, BIT0+BIT1+BIT2+BIT3);
            break;
        }

        case VIA_P4M890:        
        {
            write_reg_mask(CR99, VIACR, 0x0F, BIT0+BIT1+BIT2+BIT3);
            break;
        }

        default:
        {
           break;
        }
    }
}

void VIASetDVIOutputPath(VIABIOSInfoPtr pBIOSInfo, int set_iga, int output_interface)
{
    switch(output_interface)
    {
        case VIA_DI_DVP0:
            write_reg_mask(CR6B, VIACR, 0x01, BIT0);

            if (set_iga == IGA1)
            {
                write_reg_mask(CR96, VIACR, 0x00, BIT4);
                write_reg_mask(CR6C, VIACR, 0x21, BIT0+BIT5+BIT7);
            }
            else
            {
                write_reg_mask(CR96, VIACR, 0x10, BIT4);
                write_reg_mask(CR6C, VIACR, 0xA1, BIT0+BIT5+BIT7);
            }
                 
            write_reg_mask(SR1E, VIASR, 0xC0, BIT7+BIT6);
            break;

        case VIA_DI_DVP1:
            if (pBIOSInfo->Chipset == VIA_CLE266)
            {
                if (set_iga== IGA1)
                    write_reg_mask(CR93, VIACR, 0x21, BIT0+BIT5+BIT7);
                else
                    write_reg_mask(CR93, VIACR, 0xA1, BIT0+BIT5+BIT7);
            }
            else
            {
                if (set_iga== IGA1)
                    write_reg_mask(CR9B, VIACR, 0x00, BIT4);
                else
                    write_reg_mask(CR9B, VIACR, 0x10, BIT4);
            }

            write_reg_mask(SR1E, VIASR, 0x30, BIT4+BIT5);
            break;

        case VIA_DI_DFPHIGH:
            if (pBIOSInfo->Chipset != VIA_CLE266)
            {
                if (set_iga== IGA1)
                {
                    write_reg_mask(CR96, VIACR, 0x00, BIT4);
                    write_reg_mask(CR97, VIACR, 0x03, BIT0+BIT1+BIT4);
                }
                else
                {
                    write_reg_mask(CR96, VIACR, 0x10, BIT4);
                    write_reg_mask(CR97, VIACR, 0x13, BIT0+BIT1+BIT4);
                }
            }        
            write_reg_mask(SR2A, VIASR, 0x0C, BIT2+BIT3);
            break;
            
        case VIA_DI_DFPLOW:
            if (pBIOSInfo->Chipset == VIA_CLE266)
            {
                break;
            }

            if (set_iga == IGA1)
            {
                write_reg_mask(CR99, VIACR, 0x00, BIT4);
                write_reg_mask(CR9B, VIACR, 0x00, BIT4);
            }
            else
            {                
                write_reg_mask(CR99, VIACR, 0x10, BIT4);
                write_reg_mask(CR9B, VIACR, 0x10, BIT4);
            }

            write_reg_mask(SR2A, VIASR, 0x03, BIT0+BIT1);
            break;

        case VIA_DI_TMDS:
            if (set_iga == IGA1)
            {
                write_reg_mask(CR99, VIACR, 0x00, BIT4);
            }
            else
            {
                write_reg_mask(CR99, VIACR, 0x10, BIT4);
            }
            
            break;
    }
}

static void
Write_REG_TMDS(VIABIOSInfoPtr pBIOSInfo, int index, CARD8 data)
{       
    g_DeviceI2CPort = PORT_INDEX_I2C_31; 
    i2cWriteByte(pBIOSInfo, TMDS_I2C_ADDR_AD9389, index, data);
}

static Bool Read_REG_TMDS(VIABIOSInfoPtr pBIOSInfo, int index, CARD8 *pData)
{
    g_DeviceI2CPort = PORT_INDEX_I2C_31;
    if(i2cReadByte(pBIOSInfo, TMDS_I2C_ADDR_AD9389, index, pData))
        return TRUE;
    return FALSE;
}

static int
VIATMDSIdentify_AD9389(VIABIOSInfoPtr pBIOSInfo)
{
    unsigned char data, tmp;
    Bool  status;   
    
    if(Read_REG_TMDS(pBIOSInfo, 0x00, &data))
    {
        Write_REG_TMDS(pBIOSInfo, 0x00, 0x55);
        status = Read_REG_TMDS(pBIOSInfo, 0x00, &tmp);
        if((tmp == data) && (data <=0x1f))
        {
            pBIOSInfo->TMDSSettingInfo.ChipID = VIA_AD9389;
            pBIOSInfo->TMDSSettingInfo.I2CPort = PORT_INDEX_I2C_31; 
            return FOUND_EXTERNAL_DEV_BY_I2C;        
        }    
        Write_REG_TMDS(pBIOSInfo, 0x00, data);
    }       
    
    /* Sense AD9389 HDMI */    
    return NOT_FOUND_DEV;    
}


static int
VIATMDSIdentify_SiI9030(VIABIOSInfoPtr pBIOSInfo)
{
    unsigned char   W_Buffer[2], R_Buffer[4];
    I2CDevPtr       dev;
    
    /* Sense Silicon SiI9030 HDMI */
    if (xf86I2CProbeAddress(pBIOSInfo->I2C_Port2, 0x72))
    {
        dev = xf86CreateI2CDevRec();
        dev->DevName = "TMDS";
        dev->SlaveAddr = 0x72;
        dev->pI2CBus = pBIOSInfo->I2C_Port2;
        
        if (xf86I2CDevInit(dev))
        {
            W_Buffer[0] = 0x00;
            xf86I2CWriteRead(dev, W_Buffer,1, R_Buffer,4);

            if (R_Buffer[0] == 0x01 && R_Buffer[1] == 0x00)
            {  
                /* This is Sil9030 */
                if (R_Buffer[2] == 0x53 && R_Buffer[3] == 0x92)
                {
                    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED,
                                     "Found TMDS Transmiter Silicon SiI9030.\n"));
                    pBIOSInfo->TMDSSettingInfo.ChipID = VIA_SIL9030;
                    pBIOSInfo->TMDSSettingInfo.I2CPort = PORT_INDEX_I2C_31; 
                    
                    xf86DestroyI2CDevRec(dev,TRUE);

                    return FOUND_EXTERNAL_DEV_BY_I2C;
                }
                else
                {
                    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED, "Silicon: Unknow Chip!!\n"));
                    xf86DestroyI2CDevRec(dev,TRUE);
                }
            }
            else
            {
                xf86DestroyI2CDevRec(dev,TRUE);
            }
        }
        else
        {
            xf86DestroyI2CDevRec(dev,TRUE);
        }
    }

    /* Not found by I2C, use GPIO to try again: */

    /* CLE266 has no Low-Swing device */
    if (pBIOSInfo->Chipset != VIA_CLE266)
    {
        GPIOI2C_Initial(pBIOSInfo, 0x72);
        GPIOI2C_ReadByte(pBIOSInfo, 0, R_Buffer);      /* Get Vender ID */
        GPIOI2C_ReadByte(pBIOSInfo, 1, R_Buffer + 1);
        GPIOI2C_ReadByte(pBIOSInfo, 2, R_Buffer + 2);  /* Get Device ID */
        GPIOI2C_ReadByte(pBIOSInfo, 3, R_Buffer + 3);

        if (R_Buffer[0] == 0x01 && R_Buffer[1] == 0x0) /* Check Vender ID */
        {
            if (R_Buffer[2] == 0x53 && R_Buffer[3] == 0x92) /* Check Device ID */
            {
                DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED,
                                 "Found TMDS Transmiter Silicon SiI9030.\n"));
                pBIOSInfo->TMDSSettingInfo.ChipID = VIA_SIL9030;
                pBIOSInfo->TMDSSettingInfo.I2CPort = PORT_INDEX_GPIO_2C; 
                
                return FOUND_EXTERNAL_DEV_BY_GPIO;
            }
        }
    }

    return NOT_FOUND_DEV;
}

static int
VIATMDSIdentify_Silicon164(VIABIOSInfoPtr pBIOSInfo)
{
    unsigned char   W_Buffer[2], R_Buffer[4];
    I2CDevPtr       dev;
    
    /* Sense Silicon 164/168 */
    if (xf86I2CProbeAddress(pBIOSInfo->I2C_Port2, 0x70))
    {
        dev = xf86CreateI2CDevRec();
        dev->DevName = "TMDS";
        dev->SlaveAddr = 0x70;
        dev->pI2CBus = pBIOSInfo->I2C_Port2;

        if (xf86I2CDevInit(dev))
        {
            W_Buffer[0] = 0x00;
            xf86I2CWriteRead(dev, W_Buffer,1, R_Buffer,4);

            if (R_Buffer[0] == 0x01 && R_Buffer[1] == 0x00)
            {  
                /* This is Sil164 */
                if (R_Buffer[2] && R_Buffer[3])
                {
                    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED,
                                     "Found TMDS Transmiter Silicon164.\n"));
                    pBIOSInfo->TMDSSettingInfo.ChipID = VIA_SIL164;
                    pBIOSInfo->TMDSSettingInfo.I2CPort = PORT_INDEX_I2C_31; 
                    
                    xf86DestroyI2CDevRec(dev,TRUE);

                    return FOUND_EXTERNAL_DEV_BY_I2C;
                }
                else
                {
                    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED, "Silicon: Unknow Chip!!\n"));
                    xf86DestroyI2CDevRec(dev,TRUE);
                }
            }
            else
            {
                xf86DestroyI2CDevRec(dev,TRUE);
            }
        }
        else
        {
            xf86DestroyI2CDevRec(dev,TRUE);
        }
    }

    /* Not found by I2C, use GPIO to try again: */

    /* CLE266 has no Low-Swing device */
    if (pBIOSInfo->Chipset != VIA_CLE266)
    {
        GPIOI2C_Initial(pBIOSInfo, 0x70);
        GPIOI2C_ReadByte(pBIOSInfo, 0, R_Buffer);      /* Get Vender ID */
        GPIOI2C_ReadByte(pBIOSInfo, 1, R_Buffer + 1);
        GPIOI2C_ReadByte(pBIOSInfo, 2, R_Buffer + 2);  /* Get Device ID */
        GPIOI2C_ReadByte(pBIOSInfo, 3, R_Buffer + 3);

        if (R_Buffer[0] == 0x01 && R_Buffer[1] == 0x0) /* Check Vender ID */
        {
            if (R_Buffer[2] == 0x06 && R_Buffer[3] == 0x0) /* Check Device ID */
            {
                DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED,
                                 "Found TMDS Transmiter Silicon164.\n"));
                pBIOSInfo->TMDSSettingInfo.ChipID = VIA_SIL164;
                pBIOSInfo->TMDSSettingInfo.I2CPort = PORT_INDEX_GPIO_2C; 
                
                return FOUND_EXTERNAL_DEV_BY_GPIO;
            }
        }
    }

    return NOT_FOUND_DEV;
}

static int
VIATMDSIdentify_VT1632(VIABIOSInfoPtr pBIOSInfo)
{
    unsigned char   W_Buffer[2], R_Buffer[4];
    I2CDevPtr       dev;

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED, "VIATMDSIdentify_VT1632\n"));
    
    /* Sense VT1632 TMDS Transmiter */
    if (xf86I2CProbeAddress(pBIOSInfo->I2C_Port2, 0x10))
    {
        dev = xf86CreateI2CDevRec();
        dev->DevName = "TMDS";
        dev->SlaveAddr = 0x10;
        dev->pI2CBus = pBIOSInfo->I2C_Port2;

        if (xf86I2CDevInit(dev))
        {
            W_Buffer[0] = 0x00;
            xf86I2CWriteRead(dev, W_Buffer,1, R_Buffer,4);

            if (R_Buffer[0] == 0x06 && R_Buffer[1] == 0x11)
            {
                DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED,
                                 "Found VIA TMDS Transmiter!\n"));
                if (R_Buffer[2] == 0x92 && R_Buffer[3] == 0x31)
                {
                    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED, "VIA: VT1632!\n"));
                    pBIOSInfo->TMDSSettingInfo.ChipID = VIA_VT3192;
                    pBIOSInfo->TMDSSettingInfo.I2CPort = PORT_INDEX_I2C_31;                    
                    
                    xf86DestroyI2CDevRec(dev,TRUE);

                    return FOUND_EXTERNAL_DEV_BY_I2C;
                }
                else
                {
                    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED, "VIA: Unknow Chip!!\n"));
                    xf86DestroyI2CDevRec(dev,TRUE);
                }
            }
            else
            {
                xf86DestroyI2CDevRec(dev,TRUE);
            }
        }
        else
        {
            xf86DestroyI2CDevRec(dev,TRUE);
        }
    }

    /* Not found by I2C, use GPIO to try again: */

    /* CLE266 has no Low-Swing device */
    if (pBIOSInfo->Chipset != VIA_CLE266)
    {
        GPIOI2C_Initial(pBIOSInfo, 0x10);
        GPIOI2C_Read(pBIOSInfo, 0, R_Buffer, 4);
        if (R_Buffer[0] == 0x06 && R_Buffer[1] == 0x11)
        {
            if (R_Buffer[2] == 0x92 && R_Buffer[3] == 0x31)
            {
                DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED,
                                 "Found TMDS Transmiter VT1632.\n"));
                pBIOSInfo->TMDSSettingInfo.ChipID = VIA_VT3192;
                pBIOSInfo->TMDSSettingInfo.I2CPort = PORT_INDEX_GPIO_2C; 
                
                return FOUND_EXTERNAL_DEV_BY_GPIO;
            }
        }
    }

    return NOT_FOUND_DEV;
}

static int
VIATMDSIdentify_IntegratedTMDS(VIABIOSInfoPtr pBIOSInfo)
{
    /* VT3324 supports integrated TMDS. */
    /* RainLin [2006.03.08]: Not always support integrated TMDS now, */
    /* we should check support by hardware layout. */
    if (((pBIOSInfo->Chipset == VIA_CX700) || (pBIOSInfo->Chipset == VIA_CN750)) &&
        ((pBIOSInfo->DisplayHardwareLayout == HW_LAYOUT_DVI_ONLY) || 
         (pBIOSInfo->DisplayHardwareLayout == HW_LAYOUT_LCD_DVI)))
    {
        /* 1. Integrated TMDS can't be sensed, so we direct assign the TMDS chip ID
              according to gfx chip.
           2. Integrated TMDS use I2C port 31 to get edid. */
        DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED, "Found Integrated TMDS.\n"));
        pBIOSInfo->TMDSSettingInfo.ChipID = VIA_INTEGRATED_TMDS;
        pBIOSInfo->TMDSSettingInfo.I2CPort = PORT_INDEX_I2C_31;
        return FOUND_INTERNAL_DEV;
    }

    return NOT_FOUND_DEV;
}

static void
VIATMDSRegInit_AD9389(VIABIOSInfoPtr pBIOSInfo)
{
    unsigned char data;           
    Bool  status;
 
    
    /* Init AD9389 Register Here !!!    */
    /* i2c 2C 72 41 10     */
    Write_REG_TMDS(pBIOSInfo, 0x41, 0x10);  /*all circuit power on*/
    
    /* i2c 2C 72 AF 94 */
    Write_REG_TMDS(pBIOSInfo, 0xAF, 0x94);  /*Run hdcp encryption*/
    
    /* i2c 2C 72 15 0A */
    Write_REG_TMDS(pBIOSInfo, 0x15, 0x0A);  /*DDR RGB 444*/
    
    /* i2c 2C 72 16 2  */
    Write_REG_TMDS(pBIOSInfo, 0x16, 0x02);  /*RGB444 ,and runs falling edge*/
    
    /* i2c 2C 72 98 2 */      
    Write_REG_TMDS(pBIOSInfo, 0x98, 0x02);  /*Programming Tx to default value */
    
    /* i2c 2C 72 ba 80 */
    Write_REG_TMDS(pBIOSInfo, 0xBA, 0x80);  /* Going falling edge for DPA setting*/
    
    /* i2c 2C 72 43 A0 */        
    Write_REG_TMDS(pBIOSInfo, 0x43, 0xA0);  /*Programming Tx to Get monitor EDID*/
                          
    /* i2c 2C 72 af 14 */
    Write_REG_TMDS(pBIOSInfo, 0xaf, 0x14);  /*Run hdcp encryption*/

    status = Read_REG_TMDS(pBIOSInfo, 0xBE, &data);  /* BCAPS bit 1:HDMI Rx, 0:not HDMI Rx  */
    
    if(data & 0x80)
    {
        /* i2c 2C 72 02 18 */
        Write_REG_TMDS(pBIOSInfo, 0x02, 0x18); /*HDMI mode turn on(Audio output)*/ 
        
        /* i2c 2C 72 0A 11 */
        Write_REG_TMDS(pBIOSInfo, 0x0A, 0x11); /*SPDIF Audio,and runs at 256 bit Mclk_Pol*/
         
        /* i2c 2C 72 AF 16 */
        Write_REG_TMDS(pBIOSInfo, 0xAF, 0x16); /*HDMI mode turn on(Audio output)*/     

        /* i2c 2C 72 44 F8*/
        Write_REG_TMDS(pBIOSInfo, 0x44, 0xF8); /*Turn on Tx SPDIF audio*/
        /* Save HDMI and SPDIF info. to BIT0 and BIT1 of CR4F */
        /* CR4F.BIT0 1: HDMI on , 0: DVI mode */
        /* CR4F.BIT1 1: SPDIF audio on, 0: SPDIF audio off */
        write_reg_mask(CR4F, VIACR, 0x03, BIT0+BIT1);
        
     }
     else
     {
        /* i2c 2C 72 AF 14   */
        Write_REG_TMDS(pBIOSInfo, 0xaf, 0x14); /*DVI mode: for traditional DVI monitor use*/
        /* CR4F.BIT0 1: HDMI on , 0: DVI mode */
        /* CR4F.BIT1 1: SPDIF audio on, 0: SPDIF audio off */
        write_reg_mask(CR4F, VIACR, 0x00, BIT0+BIT1);
     }
    
}


static void
VIATMDSRegInit_SiI9030(VIABIOSInfoPtr pBIOSInfo)
{
    unsigned char   W_Buffer[2];
    I2CDevPtr       dev;
    Bool            IsInitialized = FALSE;

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED, "VIATMDSRegInit_SiI9030\n"));
    
    /* If hotkey, we don't fill any value. VGA BIOS will take care it. */        
    if (pBIOSInfo->Hotkey)
    {
        return;
    }

    if (!IsInitialized)
    {
        if (xf86I2CProbeAddress(pBIOSInfo->I2C_Port2, 0x72))
        {
            dev = xf86CreateI2CDevRec();
            dev->DevName = "TMDS";
            dev->SlaveAddr = 0x72;
            dev->pI2CBus = pBIOSInfo->I2C_Port2;
            
            if (xf86I2CDevInit(dev))
            {
                W_Buffer[0] = 0x08;

                if (pBIOSInfo->TMDSSettingInfo.BusWidth == VIA_DI_24BIT)
                {
                    /* DVI bus width is 24 Bits, we can select to use single or dual edge */
                    /* by option. */
                    if (pBIOSInfo->TMDSSettingInfo.IsDualEdge)
                    {
                        W_Buffer[1] = 0x3F;
                    }
                    else
                    {
                        W_Buffer[1] = 0x37;
                    }
                }
                else    /* 12Bit Only has Single Mode */
                {
                    /* DVI bus width is 12 Bits, we should use dual edge. */
                    W_Buffer[1] = 0x3B;
                }
                
                xf86I2CWriteRead(dev, W_Buffer,2, NULL,0);
                
                W_Buffer[0] = 0x0C;
                W_Buffer[1] = 0x09;
                xf86I2CWriteRead(dev, W_Buffer,2, NULL,0);

                IsInitialized = TRUE;
            }

            xf86DestroyI2CDevRec(dev,TRUE);
        }
    }

    /* If can't access by I2C, use GPIO to try again. */
    if (!IsInitialized && (pBIOSInfo->Chipset != VIA_CLE266))
    {
        GPIOI2C_Initial(pBIOSInfo, 0x72);
        
        if (pBIOSInfo->TMDSSettingInfo.BusWidth == VIA_DI_24BIT)
        {
            /* DVI bus width is 24 Bits, we can select to use single or dual edge */
            /* by option. */
            if (pBIOSInfo->TMDSSettingInfo.IsDualEdge)
            {
                GPIOI2C_Write(pBIOSInfo, 0x08, 0x3F);
            }
            else
            {
                GPIOI2C_Write(pBIOSInfo, 0x08, 0x37);
            }
        }
        else
        {
            /* DVI bus width is 12 Bits, we should use dual edge. */
            GPIOI2C_Write(pBIOSInfo, 0x08, 0x3B);
        }
        
        GPIOI2C_Write(pBIOSInfo, 0x0C, 0x09);
    }
}


static void
VIATMDSRegInit_Silicon164(VIABIOSInfoPtr pBIOSInfo)
{
    unsigned char   W_Buffer[2];
    I2CDevPtr       dev;
    Bool            IsInitialized = FALSE;

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED, "VIATMDSRegInit_Silicon164\n"));
    
    /* If hotkey, we don't fill any value. VGA BIOS will take care it. */        
    if (pBIOSInfo->Hotkey)
    {
        return;
    }

    if (!IsInitialized)
    {
        if (xf86I2CProbeAddress(pBIOSInfo->I2C_Port2, 0x70))
        {
            dev = xf86CreateI2CDevRec();
            dev->DevName = "TMDS";
            dev->SlaveAddr = 0x70;
            dev->pI2CBus = pBIOSInfo->I2C_Port2;

            if (xf86I2CDevInit(dev))
            {
                W_Buffer[0] = 0x08;

                if (pBIOSInfo->TMDSSettingInfo.BusWidth == VIA_DI_24BIT)
                {
                    /* DVI bus width is 24 Bits, we can select to use single or dual edge */
                    /* by option. */
                    if (pBIOSInfo->TMDSSettingInfo.IsDualEdge)
                    {
                        W_Buffer[1] = 0x3F;
                    }
                    else
                    {
                        W_Buffer[1] = 0x37;
                    }
                }
                else    /* 12Bit Only has Single Mode */
                {
                    /* DVI bus width is 12 Bits, we should use dual edge. */
                    W_Buffer[1] = 0x3B;
                }
                
                xf86I2CWriteRead(dev, W_Buffer,2, NULL,0);
                
                W_Buffer[0] = 0x0C;
                W_Buffer[1] = 0x09;
                xf86I2CWriteRead(dev, W_Buffer,2, NULL,0);

                IsInitialized = TRUE;
            }

            xf86DestroyI2CDevRec(dev,TRUE);
        }
    }

    /* If can't access by I2C, use GPIO to try again. */
    if (!IsInitialized && (pBIOSInfo->Chipset != VIA_CLE266))
    {
        GPIOI2C_Initial(pBIOSInfo, 0x70);
        
        if (pBIOSInfo->TMDSSettingInfo.BusWidth == VIA_DI_24BIT)
        {
            /* DVI bus width is 24 Bits, we can select to use single or dual edge */
            /* by option. */
            if (pBIOSInfo->TMDSSettingInfo.IsDualEdge)
            {
                GPIOI2C_Write(pBIOSInfo, 0x08, 0x3F);
            }
            else
            {
                GPIOI2C_Write(pBIOSInfo, 0x08, 0x37);
            }
        }
        else
        {
            /* DVI bus width is 12 Bits, we should use dual edge. */
            GPIOI2C_Write(pBIOSInfo, 0x08, 0x3B);
        }
        
        GPIOI2C_Write(pBIOSInfo, 0x0C, 0x09);
    }
}


static void
VIATMDSRegInit_VT1632(VIABIOSInfoPtr pBIOSInfo)
{
    unsigned char   W_Buffer[2];
    I2CDevPtr       dev;
    Bool            IsInitialized = FALSE;

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED, "VIATMDSRegInit_Silicon164\n"));
    
    /* If hotkey, we don't fill any value. VGA BIOS will take care it. */        
    if (pBIOSInfo->Hotkey)
    {
        return;
    }

    if (!IsInitialized)
    {
        if (xf86I2CProbeAddress(pBIOSInfo->I2C_Port2, 0x10))
        {
            dev = xf86CreateI2CDevRec();
            dev->DevName = "TMDS";
            dev->SlaveAddr = 0x10;
            dev->pI2CBus = pBIOSInfo->I2C_Port2;

            if (xf86I2CDevInit(dev))
            {
                W_Buffer[0] = 0x08;

                if (pBIOSInfo->TMDSSettingInfo.BusWidth == VIA_DI_24BIT)
                {
                    /* DVI bus width is 24 Bits, we can select to use single or dual edge */
                    /* by option. */
                    if (pBIOSInfo->TMDSSettingInfo.IsDualEdge)
                    {
                        W_Buffer[1] = 0x3F;
                    }
                    else
                    {
                        W_Buffer[1] = 0x37;
                    }
                }
                else
                {
                    /* DVI bus width is 12 Bits, we should use dual edge. */
                    W_Buffer[1] = 0x3B;
                }
                
                xf86I2CWriteRead(dev, W_Buffer,2, NULL,0);

                IsInitialized = TRUE;
            }

            xf86DestroyI2CDevRec(dev,TRUE);
        }
    }

    /* If can't access by I2C, use GPIO to try again. */
    if (!IsInitialized && (pBIOSInfo->Chipset != VIA_CLE266))
    {
        GPIOI2C_Initial(pBIOSInfo, 0x10);

        if (pBIOSInfo->TMDSSettingInfo.BusWidth == VIA_DI_24BIT)
        {
            /* DVI bus width is 24 Bits, we can select to use single or dual edge */
            /* by option. */
            if (pBIOSInfo->TMDSSettingInfo.IsDualEdge)
            {
                GPIOI2C_Write(pBIOSInfo, 0x08, 0x3F);
            }
            else
            {
                GPIOI2C_Write(pBIOSInfo, 0x08, 0x37);
            }
        }
        else
        {
            /* DVI bus width is 12 Bits, we should use dual edge. */
            GPIOI2C_Write(pBIOSInfo, 0x08, 0x3B);
        }
    }

}


void VIATMDSRegInit(VIABIOSInfoPtr pBIOSInfo)
{
    unsigned char   sr2a, sr1e, sr3e = 0, cr93 = 0;

    sr1e = read_reg(VIASR, SR1E);
    sr2a = read_reg(VIASR, SR2A);
    
    VIADIPortPadOn(pBIOSInfo->DFPDIPort);
    
    if (pBIOSInfo->Chipset == VIA_CLE266)
    {
        /*=* CR93[7] DI1 Data Source Selection: 1 = DSP2.
             CR93[5] DI1 Clock Source: 1 = internal.
             CR93[4] DI1 Clock Polarity.
             CR93[3:1] DI1 Clock Adjust. CR93[0] DI1 enable *=*/
        cr93 = read_reg(VIACR, CR93);
        write_reg(CR93, VIACR, 0x01);
    }
    else
    {
        /* SR3E[1]Multi-function selection:
           0 = Emulate I2C and DDC bus by GPIO2/3/4. */
        sr3e = read_reg(VIASR, SR3E);
        write_reg_mask(SR3E, VIASR, 0x0, BIT5);
    }
    
    switch (pBIOSInfo->TMDSSettingInfo.ChipID)
    {
        case VIA_AD9389:
            VIATMDSRegInit_AD9389(pBIOSInfo);
            break;
        
        case VIA_SIL9030:
            VIATMDSRegInit_SiI9030(pBIOSInfo);
            break;

        case VIA_SIL164:
            VIATMDSRegInit_Silicon164(pBIOSInfo);
            break;

        case VIA_VT3192:
            VIATMDSRegInit_VT1632(pBIOSInfo);
            break;

        default:
            break;
    }

    /* Restore status */
    write_reg(SR1E, VIASR, sr1e);
    write_reg(SR2A, VIASR, sr2a);
            
    if (pBIOSInfo->Chipset == VIA_CLE266)
    {
        write_reg(CR93, VIACR, cr93);
    }
    else
    {
        write_reg(SR3E, VIASR, sr3e);
    }
}


static void
VIAInitDVIDIPort(VIABIOSInfoPtr pBIOSInfo, int IdentifyState)
{
    if (pBIOSInfo->DFPDIPort != VIA_DI_NONE)
    {
        /* Do nothing, because DFP DIPort can be assigned by option. */
        return;
    }
        
    switch (pBIOSInfo->Chipset)
    {
        case VIA_CLE266:
             pBIOSInfo->DFPDIPort = VIA_DI_DVP1; 
             break;

       case VIA_CX700:
       case VIA_CN750:
            /*CarrieLiu[2006.07.27]: Add TMDS(VT1632A) on AMR with DVP1 support in VT3324*/
            if (IdentifyState == FOUND_INTERNAL_DEV)
            {
            	/*Integrated TMDS*/
                pBIOSInfo->DFPDIPort = VIA_DI_TMDS;
            }
            else 
            {
                /*External TMDS*/
                pBIOSInfo->DFPDIPort = VIA_DI_DVP1;            
            }
            break;
            
        case VIA_K8M890:
        case VIA_P4M900:
        case VIA_P4M890:
            /* TMDS on PCIE, we set DFPLOW as default. */
            pBIOSInfo->DFPDIPort = VIA_DI_DFPLOW;            
            break;

        default:
            if ((pBIOSInfo->TMDSSettingInfo.ChipID == VIA_SIL9030) || 
                     (IdentifyState == FOUND_EXTERNAL_DEV_BY_I2C))
            {
                /* TMDS on AMR, we set DVP0 as default. */
                pBIOSInfo->DFPDIPort = VIA_DI_DVP0;
            }
            else
            {
                /* TMDS on AGP, we set DVP1 as default. */
                pBIOSInfo->DFPDIPort = VIA_DI_DVP1;
            }
            
            break;
    }
}

Bool VIATMDSIdentify(VIABIOSInfoPtr pBIOSInfo)
{
    int             IdentifyState = NOT_FOUND_DEV;
    unsigned char   sr2a, sr1e, sr3e, cr93;

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIATMDSIdentify\n"));

    sr2a = sr1e = sr3e = cr93 = 0;
    switch (pBIOSInfo->Chipset)
    {
        case VIA_K8M890:
            /*=* DFP Low Pad on *=*/
            sr2a = read_reg(VIASR, SR2A);
            write_reg_mask(SR2A, VIASR, 0x03, BIT0+BIT1);
            break;

        case VIA_P4M900:
        case VIA_P4M890:
            /* DFP Low Pad on */
            sr2a = read_reg(VIASR, SR2A);
            write_reg_mask(SR2A, VIASR, 0x03, BIT0+BIT1);

            /* RainLin [2006.05.15]: VT3364 also supports DVI on AMR, so we need to sense DVP0, too. */
            /* DVP0 Pad on */
            sr1e = read_reg(VIASR, SR1E);
            write_reg_mask(SR1E, VIASR, 0xC0, BIT6+BIT7);
            break;
               
        default:
            /*=* DVP0/DVP1 Pad on *=*/
            sr1e = read_reg(VIASR, SR1E);
            write_reg_mask(SR1E, VIASR, 0xF0, BIT4+BIT5+BIT6+BIT7);
            /*=* SR3E[1]Multi-function selection:
                 0 = Emulate I2C and DDC bus by GPIO2/3/4. *=*/
            sr3e = read_reg(VIASR, SR3E);
            write_reg_mask(SR3E, VIASR, 0x0, BIT5);
            break;
    }
    
    if (pBIOSInfo->Chipset == VIA_CLE266)
    {
        /*=* CR93[7] DI1 Data Source Selection: 1 = DSP2.
             CR93[5] DI1 Clock Source: 1 = internal.
             CR93[4] DI1 Clock Polarity.
             CR93[3:1] DI1 Clock Adjust. CR93[0] DI1 enable *=*/
        cr93 = read_reg(VIACR, CR93);
        write_reg(CR93, VIACR, 0x01);
    }

    IdentifyState = VIATMDSIdentify_VT1632(pBIOSInfo);
    
    if (IdentifyState == NOT_FOUND_DEV)
    {
        IdentifyState = VIATMDSIdentify_AD9389(pBIOSInfo);
    }

    if (IdentifyState == NOT_FOUND_DEV)
    {
        IdentifyState = VIATMDSIdentify_SiI9030(pBIOSInfo);
    }

    if (IdentifyState == NOT_FOUND_DEV)
    {
        IdentifyState = VIATMDSIdentify_Silicon164(pBIOSInfo);
    }

    if (IdentifyState == NOT_FOUND_DEV)
    {
        IdentifyState = VIATMDSIdentify_IntegratedTMDS(pBIOSInfo);
    }
    
    /* Restore status */
    switch (pBIOSInfo->Chipset)
    {
        case VIA_K8M890:
            write_reg(SR2A, VIASR, sr2a);
            break;

        case VIA_P4M900:
        case VIA_P4M890:
            write_reg(SR2A, VIASR, sr2a);
            write_reg(SR1E, VIASR, sr1e);
            break;

        default: 
            write_reg(SR1E, VIASR, sr1e);
            write_reg(SR3E, VIASR, sr3e);
            break;
    }

    if (pBIOSInfo->Chipset == VIA_CLE266)
    {
        write_reg(CR93, VIACR, cr93);
    }

    
    if (IdentifyState == NOT_FOUND_DEV)
    {
        return FALSE;
    }
    else
    {
        VIAInitDVIDIPort(pBIOSInfo, IdentifyState);
        return TRUE;
    }
}

/*=*
 *
 * Bool VIASenseDVI(pBIOSInfo)
 *
 *     - Sense DVI Connector
 *
 * Return Type:    Bool
 *
 * The Definition of Input Value:
 *
 *                 VIABIOSInfoPtr
 *
 * The Definition of Return Value:
 *
 *                 DVI Attached  - TRUE
 *                 DVI Not Attached - FALSE
 *=*/
Bool VIASenseDVI(VIABIOSInfoPtr pBIOSInfo)
{
    unsigned char   sr1e, /*sr3e = 0,*/ cr6b = 0, cr91, cr93 = 0/*, cr9b = 0*/;
    unsigned char   SlaveAddr;
    Bool            ret = FALSE;
    I2CDevPtr       dev;
    unsigned char   W_Buffer[1];
    unsigned char   R_Buffer[1];

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIASenseDVI\n"));
    
    if (pBIOSInfo->NoDDCValue)
        return TRUE;

    /* RainLin [2005.08.30]: If we sense DVI connection status from TMDS, the step of */
    /* turning software power sequence on may effects LCD when we use utility to turn LCD off. */
    /* And since we can just get DVI connection status by querying DVI EDID, I remove */ 
    /* the codes about sense DVI connection status from TMDS. */
    /* Note: Not include CLE266. */
    
    if (pBIOSInfo->Chipset == VIA_CLE266)
    {
        /*=* DI1 Pad on *=*/
        sr1e = read_reg(VIASR, SR1E);
        write_reg_mask(SR1E, VIASR, 0x30, BIT4+BIT5);

        /*=* CR6B[0]VCK Input Selection: 1 = External clock. *=*/
        cr6b = read_reg(VIACR, CR6B);
        write_reg_mask(CR6B, VIACR, 0x08, BIT0+BIT1+BIT2+BIT3);

        /*=* CR91[4] VDD On [3] Data On [2] VEE On [1] Back Light Off
                 [0] Software Control Power Sequence *=*/
        cr91 = read_reg(VIACR, CR91);
        write_reg(CR91, VIACR, 0x1D);

        /*=* CR93[7] DI1 Data Source Selection: 1 = DSP2.
             CR93[5] DI1 Clock Source: 1 = internal.
             CR93[4] DI1 Clock Polarity.
             CR93[3:1] DI1 Clock Adjust. CR93[0] DI1 enable *=*/
        cr93 = read_reg(VIACR, CR93);
        write_reg(CR93, VIACR, 0x01);

        switch (pBIOSInfo->TMDSSettingInfo.ChipID)
        {
            case VIA_SIL164:
                SlaveAddr = 0x70;
                break;
            case VIA_VT3192:
                SlaveAddr = 0x10;
                break;
            default:
                return ret;
                break;
        }

        if (xf86I2CProbeAddress(pBIOSInfo->I2C_Port2, SlaveAddr))
        {
            dev = xf86CreateI2CDevRec();
            dev->DevName = "TMDS";
            dev->SlaveAddr = SlaveAddr;
            dev->pI2CBus = pBIOSInfo->I2C_Port2;

            if (xf86I2CDevInit(dev))
            {
                W_Buffer[0] = 0x09;
                xf86I2CWriteRead(dev, W_Buffer,1, R_Buffer,1);
                xf86DestroyI2CDevRec(dev,TRUE);
                if (R_Buffer[0] & 0x04)
                    ret = TRUE;
            }
            else
                xf86DestroyI2CDevRec(dev,TRUE);
        }

        /* Restore status */
        write_reg(SR1E, VIASR, sr1e);
        write_reg(CR6B, VIACR, cr6b);
        write_reg(CR91, VIACR, cr91);
        write_reg(CR93, VIACR, cr93);
    }

    if (!ret)
    {
        if(VIAQueryDVIEDID(pBIOSInfo))
        {
            ret = TRUE;
        }
    }

    return ret;
}

/*=*
 *
 * int VIAQueryDVIEDID(void)
 *
 *     - Query Flat Panel's EDID Table Version Through DVI Connector
 *
 * Return Type:    int
 *
 * The Definition of Input Value:
 *
 *                 VIABIOSInfoPtr
 *
 * The Definition of Return Value:
 *
 *                 0 - Found No EDID Table
 *                 1 - Found EDID1 Table
 *                 2 - Found EDID2 Table
 *=*/
int VIAQueryDVIEDID(VIABIOSInfoPtr pBIOSInfo)
{
    I2CDevPtr       dev;
    unsigned char   W_Buffer[1];
    unsigned char   R_Buffer[2];

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIAQueryDVIEDID\n"));
    if (xf86I2CProbeAddress(pBIOSInfo->I2C_Port2, 0xA0))
    {
        pBIOSInfo->dev = xf86CreateI2CDevRec();
        dev = pBIOSInfo->dev;
        dev->DevName = "EDID1";
        dev->SlaveAddr = 0xA0;
        dev->ByteTimeout = 2200; /* VESA DDC spec 3 p. 43 (+10 %) */
        dev->StartTimeout = 550;
        dev->BitTimeout = 40;
        dev->ByteTimeout = 40;
        dev->AcknTimeout = 40;
        dev->pI2CBus = pBIOSInfo->I2C_Port2;

        if (xf86I2CDevInit(dev))
        {
            W_Buffer[0] = 0;
            xf86I2CWriteRead(dev, W_Buffer,1, R_Buffer,2);
            if ((R_Buffer[0] == 0) && (R_Buffer[1] == 0xFF))
            {
                DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "Found EDID1 Table.\n"));
                xf86DestroyI2CDevRec(dev,TRUE);
                return 1; /* Found EDID1 Table */
            }
            else
            {
                xf86DestroyI2CDevRec(dev,TRUE);
            }
        }
        else
        {
            xf86DestroyI2CDevRec(dev,TRUE);
        }
    }

    if (xf86I2CProbeAddress(pBIOSInfo->I2C_Port2, 0xA2))
    {
        pBIOSInfo->dev = xf86CreateI2CDevRec();
        dev = pBIOSInfo->dev;
        dev->DevName = "EDID2";
        dev->SlaveAddr = 0xA2;
        dev->ByteTimeout = 2200; /* VESA DDC spec 3 p. 43 (+10 %) */
        dev->StartTimeout = 550;
        dev->BitTimeout = 40;
        dev->ByteTimeout = 40;
        dev->AcknTimeout = 40;
        dev->pI2CBus = pBIOSInfo->I2C_Port2;

        if (xf86I2CDevInit(dev))
        {
            W_Buffer[0] = 0;
            xf86I2CWriteRead(dev, W_Buffer,1, R_Buffer,1);
            if (R_Buffer[0] == 0x20)
            {
                DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "Found EDID2 Table.\n"));
                xf86DestroyI2CDevRec(dev,TRUE);
                return 2; /* Found EDID2 Table */
            }
            else
            {
                xf86DestroyI2CDevRec(dev,TRUE);
                return 0; /* Found No EDID Table */
            }
        }
        else
        {
            xf86DestroyI2CDevRec(dev,TRUE);
            return 0; /* Found No EDID Table */
        }
    }

    if (pBIOSInfo->Chipset != VIA_CLE266)
    {
        GPIOI2C_Initial(pBIOSInfo, 0xA0);
        GPIOI2C_Read(pBIOSInfo, 0, R_Buffer, 2);
        DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "R_Buffer[0]=%d\n", R_Buffer[0]));
        DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "R_Buffer[1]=%d\n", R_Buffer[1]));
        if ((R_Buffer[0] == 0) && (R_Buffer[1] == 0xFF))
            return 1; /* Found EDID1 Table */
        else
        {
            GPIOI2C_Initial(pBIOSInfo, 0xA2);
            GPIOI2C_Read(pBIOSInfo, 0, R_Buffer, 1);
            DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "R_Buffer[0]=%d\n", R_Buffer[0]));
            if (R_Buffer[0] == 0x20)
                return 2; /* Found EDID2 Table */
            else
                return 0; /* Found No EDID Table */
        }
    }

        return 0;
}


/*=*
 *
 * unsigned char VIAGetPanelSizeFromDDCv1(VIABIOSInfoPtr pBIOSInfo)
 *
 *     - Get Panel Size Using EDID1 Table
 *
 * Return Type:    unsigned char
 *
 * The Definition of Input Value:
 *
 *                 VIABIOSInfoPtr
 *
 * The Definition of Return Value:
 *
 *                 0 - 640x480
 *                 1 - 800x600
 *                 2 - 1024x768
 *                 3 - 1280x768
 *                 4 - 1280x1024
 *                 5 - 1400x1050
 *                 6 - 1600x1200
 *                 0xFF - Not Supported Panel Size
 *=*/
unsigned char VIAGetPanelSizeFromDDCv1(VIABIOSInfoPtr pBIOSInfo)
{
    ScrnInfoPtr     pScrn = xf86Screens[pBIOSInfo->scrnIndex];
    xf86MonPtr      pMon;
    int             i, max_h = 0, max_v = 0, tmp;
    unsigned char   W_Buffer[1];
    unsigned char   R_Buffer[1];
    I2CDevPtr       dev;
    Bool            bUseI2C = FALSE;
    
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIAGetPanelSizeFromDDCv1\n"));

    if (pBIOSInfo->TMDSSettingInfo.I2CPort == PORT_INDEX_I2C_31)
    {
        bUseI2C = TRUE;
    }
    
    if (bUseI2C)
    {
        if (xf86I2CProbeAddress(pBIOSInfo->I2C_Port2, 0xA0))
        {
            pBIOSInfo->dev = xf86CreateI2CDevRec();
            dev = pBIOSInfo->dev;
            dev->DevName = "EDID1";
            dev->SlaveAddr = 0xA0;
            dev->ByteTimeout = 2200; /* VESA DDC spec 3 p. 43 (+10 %) */
            dev->StartTimeout = 550;
            dev->BitTimeout = 40;
            dev->ByteTimeout = 40;
            dev->AcknTimeout = 40;
            dev->pI2CBus = pBIOSInfo->I2C_Port2;

            if (!xf86I2CDevInit(dev))
            {
                xf86DestroyI2CDevRec(pBIOSInfo->dev,TRUE);
                xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "Unknow panel size! Set default panel size.\n");
                pBIOSInfo->TMDSSettingInfo.DFPSize = M1024x768;
                return pBIOSInfo->TMDSSettingInfo.DFPSize;
            }
        }
    }
    else
    {
        GPIOI2C_Initial(pBIOSInfo, 0xA0);
    }
    
    for (i = 0x23; i < 0x35; i++)
    {
        switch (i)
        {
            case 0x23:
                if (bUseI2C)
                {
                    W_Buffer[0] = i;
                    xf86I2CWriteRead(pBIOSInfo->dev, W_Buffer,1, R_Buffer,1);
                }
                else
                {
                    GPIOI2C_Read(pBIOSInfo, i, R_Buffer, 1);
                }
                if (R_Buffer[0] & 0x3C)
                    max_h = 640;
                if (R_Buffer[0] & 0xC0)
                    max_h = 720;
                if (R_Buffer[0] & 0x03)
                    max_h = 800;
                break;
            case 0x24:
                if (bUseI2C)
                {
                    W_Buffer[0] = i;
                    xf86I2CWriteRead(pBIOSInfo->dev, W_Buffer,1, R_Buffer,1);
                }
                else
                {
                    GPIOI2C_Read(pBIOSInfo, i, R_Buffer, 1);
                }
                if (R_Buffer[0] & 0xC0)
                    max_h = 800;
                if (R_Buffer[0] & 0x1E)
                    max_h = 1024;
                if (R_Buffer[0] & 0x01)
                    max_h = 1280;
                break;
            case 0x26:
            case 0x28:
            case 0x2A:
            case 0x2C:
            case 0x2E:
            case 0x30:
            case 0x32:
            case 0x34:
                if (bUseI2C)
                {
                    W_Buffer[0] = i;
                    xf86I2CWriteRead(pBIOSInfo->dev, W_Buffer,1, R_Buffer,1);
                }
                else
                {
                    GPIOI2C_Read(pBIOSInfo, i, R_Buffer, 1);
                }
                if (R_Buffer[0] == 1)
                    break;
                tmp = (R_Buffer[0] + 31) << 3; /* data = (data + 31) * 8 */
                if (tmp > max_h)
                    max_h = tmp;
                break;
            default:
                break;
        }
    }

    if (bUseI2C)
    {
        xf86DestroyI2CDevRec(pBIOSInfo->dev,TRUE);

        pMon = xf86DoEDID_DDC2(pScrn->scrnIndex, pBIOSInfo->I2C_Port2);
        if (pMon)
        {
            pBIOSInfo->DDC2 =  pMon;
            xf86PrintEDID(pMon);
            xf86SetDDCproperties(pScrn, pMon);
            for (i = 0; i < 8; i++)
            {
                if (pMon->timings2[i].hsize > max_h)
                {
                    max_h = pMon->timings2[i].hsize;
                }
                
            }

            for (i = 0; i < 4; i++)
            {
                if (pMon->det_mon[i].type == DT)
                {
                    if (pMon->det_mon[i].section.d_timings.h_active > max_h)
                    {
                        max_h = pMon->det_mon[i].section.d_timings.h_active;
                        max_v = pMon->det_mon[i].section.d_timings.v_active;
                    }
                }
            }
            
            if (pBIOSInfo->DDC1)
            {
                xf86SetDDCproperties(pScrn, pBIOSInfo->DDC1);
            }
        }
    }
    

    switch (max_h)
    {
        case 640:
            pBIOSInfo->TMDSSettingInfo.DFPSize = M640x480;
            break;
        case 800:
            pBIOSInfo->TMDSSettingInfo.DFPSize = M800x600;
            break;
        case 1024:
            pBIOSInfo->TMDSSettingInfo.DFPSize = M1024x768;
            break;
        case 1280:
            pBIOSInfo->TMDSSettingInfo.DFPSize = M1280x1024;
            break;
        case 1400:
            pBIOSInfo->TMDSSettingInfo.DFPSize = M1400x1050;
            break;
        case 1440:
            pBIOSInfo->TMDSSettingInfo.DFPSize = M1440x1050;
            break;            
        case 1600:
            pBIOSInfo->TMDSSettingInfo.DFPSize = M1600x1200;
            break;
        case 1920:
            if (max_v == 1200)
            {
                pBIOSInfo->TMDSSettingInfo.DFPSize = M1920x1200;
            }
            else
            {
                pBIOSInfo->TMDSSettingInfo.DFPSize = M1920x1080;    /* Default */
            }
            
            break;
        default:
            pBIOSInfo->TMDSSettingInfo.DFPSize = M1024x768;
            xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "Unknow panel size max resolution = %d ! set default panel size.\n", max_h);
            break;
    }

    return pBIOSInfo->TMDSSettingInfo.DFPSize;
}

/*=*
 *
 * unsigned char VIAGetPanelSizeFromDDCv2(VIABIOSInfoPtr pBIOSInfo)
 *
 *     - Get Panel Size Using EDID2 Table
 *
 * Return Type:    unsigned char
 *
 * The Definition of Input Value:
 *
 *                 VIABIOSInfoPtr
 *
 * The Definition of Return Value:
 *
 *                 0 - 640x480
 *                 1 - 800x600
 *                 2 - 1024x768
 *                 3 - 1280x768
 *                 4 - 1280x1024
 *                 5 - 1400x1050
 *                 6 - 1600x1200
 *                 0xFF - Not Supported Panel Size
 *=*/
unsigned char VIAGetPanelSizeFromDDCv2(VIABIOSInfoPtr pBIOSInfo)
{
    int             HSize = 0;    
    unsigned char   W_Buffer[1];
    unsigned char   R_Buffer[2];
    I2CDevPtr       dev;

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO,
                     "VIAGetPanelSizeFromDDCv2\n"));
    if (pBIOSInfo->TMDSSettingInfo.I2CPort == PORT_INDEX_I2C_31)
    {
        if (xf86I2CProbeAddress(pBIOSInfo->I2C_Port2, 0xA0))
        {
            pBIOSInfo->dev = xf86CreateI2CDevRec();
            dev = pBIOSInfo->dev;
            dev->DevName = "EDID2";
            dev->SlaveAddr = 0xA2;
            dev->ByteTimeout = 2200; /* VESA DDC spec 3 p. 43 (+10 %) */
            dev->StartTimeout = 550;
            dev->BitTimeout = 40;
            dev->ByteTimeout = 40;
            dev->AcknTimeout = 40;
            dev->pI2CBus = pBIOSInfo->I2C_Port2;

            if (xf86I2CDevInit(dev))
            {
                /* Horizontal: 0x76, 0x77*/
                W_Buffer[0] = 0x77;
                xf86I2CWriteRead(pBIOSInfo->dev, W_Buffer,1, R_Buffer,1);
                HSize = R_Buffer[0];
                HSize = HSize << 8;
                W_Buffer[0] = 0x76;
                xf86I2CWriteRead(pBIOSInfo->dev, W_Buffer,1, R_Buffer,1);
                HSize |= R_Buffer[0];                

                xf86DestroyI2CDevRec(pBIOSInfo->dev,TRUE);
            }
            else
            {
                xf86DestroyI2CDevRec(pBIOSInfo->dev,TRUE);
                xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "Unknow panel size! Set default panel size.\n");
                pBIOSInfo->TMDSSettingInfo.DFPSize = M1024x768;
                return pBIOSInfo->TMDSSettingInfo.DFPSize;
            }
        }
    }
    else
    {
        GPIOI2C_Initial(pBIOSInfo, 0xA2);

        /* Horizontal: 0x76, 0x77*/
        GPIOI2C_Read(pBIOSInfo, 0x76, R_Buffer, 2);
        HSize = R_Buffer[0];
        HSize += R_Buffer[1] << 8;        
    }

    switch (HSize)
    {
        case 640:
            pBIOSInfo->TMDSSettingInfo.DFPSize = M640x480;
            break;
        case 800:
            pBIOSInfo->TMDSSettingInfo.DFPSize = M800x600;
            break;
        case 1024:
            pBIOSInfo->TMDSSettingInfo.DFPSize = M1024x768;
            break;
        case 1280:
            pBIOSInfo->TMDSSettingInfo.DFPSize = M1280x1024;
            break;
        case 1400:
            pBIOSInfo->TMDSSettingInfo.DFPSize = M1400x1050;
            break;
        case 1440:
            pBIOSInfo->TMDSSettingInfo.DFPSize = M1440x1050;
            break;            
        case 1600:
            pBIOSInfo->TMDSSettingInfo.DFPSize = M1600x1200;
            break;
        default:
            pBIOSInfo->TMDSSettingInfo.DFPSize = M1024x768;
            xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "Unknow panel size max resolution = %d ! set default panel size.\n", HSize);
            break;
    }

    return pBIOSInfo->TMDSSettingInfo.DFPSize;
}

/*=*
 *
 * unsigned char VIAGetPanelInfo(VIABIOSInfoPtr pBIOSInfo)
 *
 *     - Get Panel Size
 *
 * Return Type:    unsigned char
 *
 * The Definition of Input Value:
 *
 *                 VIABIOSInfoPtr
 *
 * The Definition of Return Value:
 *
 *                 0 - 640x480
 *                 1 - 800x600
 *                 2 - 1024x768
 *                 3 - 1280x768
 *                 4 - 1280x1024
 *                 5 - 1400x1050
 *                 6 - 1600x1200
 *                 0xFF - Not Supported Panel Size
 *=*/
unsigned char VIAGetPanelInfo(VIABIOSInfoPtr pBIOSInfo)
{
    unsigned char   sr1e, sr3e = 0, cr6b = 0, cr91, cr93 = 0, cr9b = 0;

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIAGetPanelInfo\n"));

    if (pBIOSInfo->TMDSSettingInfo.DFPSize == M_INVALID)
    {
        if (pBIOSInfo->Chipset == VIA_CLE266)
        {
            /*=* DI1 Pad on *=*/
            sr1e = read_reg(VIASR, SR1E);
            write_reg_mask(SR1E, VIASR, 0xF0, BIT4+BIT5+BIT6+BIT7);

            /*=* CR6B[0]VCK Input Selection: 1 = External clock. *=*/
            cr6b = read_reg(VIACR, CR6B);
            write_reg_mask(CR6B, VIACR, 0x08, BIT0+BIT1+BIT2+BIT3);

            /*=* CR91[4] VDD On [3] Data On [2] VEE On [1] Back Light Off
                     [0] Software Control Power Sequence *=*/
            cr91 = read_reg(VIACR, CR91);
            write_reg(CR91, VIACR, 0x1D);

            /*=* CR93[7] DI1 Data Source Selection: 1 = DSP2.
                 CR93[5] DI1 Clock Source: 1 = internal.
                 CR93[4] DI1 Clock Polarity.
                 CR93[3:1] DI1 Clock Adjust. CR93[0] DI1 enable *=*/
            cr93 = read_reg(VIACR, CR93);
            write_reg(CR93, VIACR, 0x01);
        }
        else
        {
            /*=* DVP0/DVP1 Pad on *=*/
            sr1e = read_reg(VIASR, SR1E);
            write_reg_mask(SR1E, VIASR, 0xF0, BIT4+BIT5+BIT6+BIT7);

            /*=* SR3E[1]Multi-function selection:
                        0 = Emulate I2C and DDC bus by GPIO2/3/4. *=*/
            sr3e = read_reg(VIASR, SR3E);
            write_reg_mask(SR3E, VIASR, 0x0, BIT5);

            /*=* CR91[4] VDD On [3] Data On [2] VEE On [1] Back Light Off
                     [0] Software Control Power Sequence *=*/
            cr91 = read_reg(VIACR, CR91);
            write_reg(CR91, VIACR, 0x1D);

            /*=* CR9B[4] DVP1 Data Source Selection: 1 = From secondary display.
                 CR9B[2:0] DVP1 Clock Adjust *=*/
            cr9b = read_reg(VIACR, CR9B);
            write_reg(CR9B, VIACR, 0x1F);
        }

        switch (VIAQueryDVIEDID(pBIOSInfo))
        {
            case 1:
                VIAGetPanelSizeFromDDCv1(pBIOSInfo);
                break;
            case 2:
                VIAGetPanelSizeFromDDCv2(pBIOSInfo);
                break;
            default:
                break;
        }

        /* Restore status */
        if (pBIOSInfo->Chipset == VIA_CLE266)
        {
            write_reg(SR1E, VIASR, sr1e);
            write_reg(CR6B, VIACR, cr6b);
            write_reg(CR91, VIACR, cr91);
            write_reg(CR93, VIACR, cr93);
        }
        else
        {
            write_reg(SR1E, VIASR, sr1e);
            write_reg(SR3E, VIASR, sr3e);
            write_reg(CR91, VIACR, cr91);
            write_reg(CR9B, VIACR, cr9b);
        }
    }
    
    VIAGetModeSizeByIndex(pBIOSInfo->TMDSSettingInfo.DFPSize, &pBIOSInfo->TMDSSettingInfo.PanelSizeH, &pBIOSInfo->TMDSSettingInfo.PanelSizeV);
    
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "DFPSize = %d\n", pBIOSInfo->TMDSSettingInfo.DFPSize));
    return (unsigned char)(pBIOSInfo->TMDSSettingInfo.DFPSize);
}

Bool VIASetDFPMode(VIABIOSInfoPtr pBIOSInfo)
{
    struct io_register      *reg = NULL;
    int                     load_reg_num=0;
    int                     reg_value=0;
    int                     offset;
	
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIASetDFPMode\n"));

    VideoSetMode(pBIOSInfo, pBIOSInfo->TMDSSettingInfo.IGAPath);

    if ((pBIOSInfo->IsSecondary)||(pBIOSInfo->DuoView))
    {
        /* Second Display Channel Enable */
        /* RainLin [2005.11.22]: Modify for sequence issue. */
        EnableSecondDisplayChannel();
        

        /*=* VansonWu [2004.5.10] Fixed Set DVI+TV/DVT+CRT/CRT+TV SAMM mode,
             the secondary monitor without display *=*/
        write_reg_mask(CR91, VIACR, 0x0, BIT5);
    }

    /* Disable LCD Scaling */
    if(pBIOSInfo->TMDSSettingInfo.IGAPath == IGA2)
    {
        write_reg(CR79, VIACR, 0x0);
    }
        
    if(pBIOSInfo->EPIA_DVI && pBIOSInfo->DuoView == FALSE && pBIOSInfo->SAMM== FALSE)
    {
        VideoSetMode(pBIOSInfo, pBIOSInfo->TMDSSettingInfo.IGAPath);
        switch(pBIOSInfo->TMDSSettingInfo.ModeIndex)
        {
            case M640x480:
                offset = 80;
                break;
            case M800x480:
            case M800x600:
                offset = 110;
                break;
            case M1024x768:
                offset = 150;
                break;
            case M1280x768:
            case M1280x800:
            case M1280x1024:
            case M1440x1050:
                offset = 190;
                break;
            case M1600x1200:
                offset = 250;
                break;
            case M1366x768:
            case M1360x768:
                offset = 212;                             
                break;
            default:
                offset = 140;
                break;
        }
        /* Offset for simultaneous */
        reg_value = offset;
        load_reg_num = offset_reg.iga2_offset_reg.reg_num;
        reg = offset_reg.iga2_offset_reg.reg;
        load_reg(reg_value, load_reg_num, reg, VIACR);
       
        write_reg_mask(CR6B, VIACR, 0x08, BIT3);
        write_reg_mask(CR6A, VIACR, 0x00, BIT6);
        write_reg_mask(CR6A, VIACR, 0x40, BIT6);
        write_reg_mask(SR16, VIASR, 0x40, BIT6);
    }

    switch (pBIOSInfo->Chipset)
    {
        case VIA_CLE266:
            if (pBIOSInfo->TMDSSettingInfo.IGAPath == IGA2)
            {
                write_reg_mask(CR93, VIACR, 0x89, BIT0+BIT1+BIT2+BIT3+BIT7);
            }
            else
            {
                write_reg_mask(CR93, VIACR, 0x09, BIT0+BIT1+BIT2+BIT3+BIT7);
            }
            break;
        case VIA_KM400:
            /* Set DVP1 clock skew 0x9B[2:0] and data source 0x9B[4] */
            if (pBIOSInfo->ChipRev < 5)   /* KM400 Ax */
            {
                if (pBIOSInfo->TMDSSettingInfo.IGAPath == IGA2)
                {
                    write_reg(CR9B, VIACR, 0x1F);
                }
                else
                {
                    write_reg(CR9B, VIACR, 0x0F);
                }
            }
            else
            {
                if (pBIOSInfo->TMDSSettingInfo.IGAPath == IGA2)
                {
                    write_reg(CR9B, VIACR, 0x17);
                }
                else
                {
                    write_reg(CR9B, VIACR, 0x07);
                }
            }
            break;
        case VIA_K8M800:
        case VIA_PM800:
        case VIA_P4M800PRO:
            if (pBIOSInfo->TMDSSettingInfo.IGAPath == IGA2)
            {
                write_reg(CR9B, VIACR, 0x11);
            }
            else
            {
                if(pBIOSInfo->IsDemoBoard)
                {
                    write_reg(CR9B, VIACR, 0x01);
                }
                else
            	{
                    write_reg_mask(CR97, VIACR, 0x01, BIT0);
            	}
            }
            break;
        default:
            break;
    }

    VIASetOutputPath(pBIOSInfo, VIA_DEVICE_DFP, pBIOSInfo->TMDSSettingInfo.IGAPath, pBIOSInfo->DFPDIPort);

    /* Adjust DPA Setting for clock skew problem: */
    VIADVIPatchSkew(pBIOSInfo, pBIOSInfo->DFPDIPort);

    /* Set display channel. */
    if ((pBIOSInfo->Chipset == VIA_CX700)||(pBIOSInfo->Chipset == VIA_CN750))
    {
        VIASetDisplayChannel(pBIOSInfo);
    }
    
    return TRUE;
}

/* If Enable DFP, turn on pad */
void VIAEnableDFP(VIABIOSInfoPtr pBIOSInfo)
{
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIAEnableDFP\n"));
    
    VIABIOSInfoPtr  pVia = pBIOSInfo;

    if (pBIOSInfo->DFPDIPort & VIA_DI_DVP0)
    {
        VGAOUT8(0x3c4, 0x1E);
        VGAOUT8(0x3c5, VGAIN8(0x3c5) | 0xC0);
    }
    if (pBIOSInfo->DFPDIPort & VIA_DI_DVP1)
    {
        VGAOUT8(0x3c4, 0x1E);
        VGAOUT8(0x3c5, VGAIN8(0x3c5) | 0x30);
    }
    if (pBIOSInfo->DFPDIPort & VIA_DI_DFPHIGH)
    {
        VGAOUT8(0x3c4, 0x2A);
        VGAOUT8(0x3c5, VGAIN8(0x3c5) | 0x0C);
    }
    if (pBIOSInfo->DFPDIPort & VIA_DI_DFPLOW)
    {
        VGAOUT8(0x3c4, 0x2A);
        VGAOUT8(0x3c5, VGAIN8(0x3c5) | 0x03);
    }
    if(pBIOSInfo->EPIA_DVI)      /*Support DVI on EPIA*/
    {
        write_reg(CR91, VIACR, 0x1f);
        write_reg(CR88, VIACR, 0x00);
    }
    if (pBIOSInfo->DFPDIPort & VIA_DI_TMDS)
    {
        VGAOUT8(0x3d4, 0x91);
        VGAOUT8(0x3d5, VGAIN8(0x3d5) & 0x7F);   /* Turn on Display period in the panel path. */
        
        VGAOUT8(0x3d4, 0xD2);
        VGAOUT8(0x3d5, VGAIN8(0x3d5) & 0xF7);   /* Turn on TMDS power. */
    }
}

/* If Disable DFP, turn off pad */
void VIADisableDFP(VIABIOSInfoPtr pBIOSInfo)
{
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIADisableDFP\n"));
    
    VIABIOSInfoPtr  pVia = pBIOSInfo;

    if (pBIOSInfo->DFPDIPort & VIA_DI_DVP0)
    {
        VGAOUT8(0x3c4, 0x1E);
        VGAOUT8(0x3c5, VGAIN8(0x3c5) & 0x3F);
    }

    if (pBIOSInfo->DFPDIPort & VIA_DI_DVP1)
    {
        VGAOUT8(0x3c4, 0x1E);
        VGAOUT8(0x3c5, VGAIN8(0x3c5) & 0xCF);
    }

    if (pBIOSInfo->DFPDIPort & VIA_DI_DFPHIGH)
    {
        VGAOUT8(0x3c4, 0x2A);
        VGAOUT8(0x3c5, VGAIN8(0x3c5) & 0xF3);
    }

    if (pBIOSInfo->DFPDIPort & VIA_DI_DFPLOW)
    {
        VGAOUT8(0x3c4, 0x2A);
        VGAOUT8(0x3c5, VGAIN8(0x3c5) & 0xFC);
    }

    if (pBIOSInfo->DFPDIPort & VIA_DI_TMDS)
    {
        VGAOUT8(0x3d4, 0xD2);
        VGAOUT8(0x3d5, VGAIN8(0x3d5) | 0x08);   /* Turn off TMDS power. */
    }
}

void
VideoSetMode(VIABIOSInfoPtr pBIOSInfo, int set_iga)
{
    struct  VideoModeTable *VideoMode;
    struct display_timing   mode_crt_reg;
    struct crt_mode_table   *mode_crt_table;

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO,
		     "VideoSetMode: Set IGA%d Mode!\n", set_iga));

    VideoMode = &CLE266Modes
        [SearchModeSetting(pBIOSInfo->TMDSSettingInfo.ModeIndex)];
    mode_crt_table = VideoMode->crtc;
    mode_crt_reg = mode_crt_table->crtc;

    fill_crtc_timing(pBIOSInfo, mode_crt_table, VideoMode->mode_array,
		     (pBIOSInfo->bitsPerPixel >> 3), set_iga);

    if (pBIOSInfo->EPIA_DVI && (pBIOSInfo->DuoView == FALSE)) {
	fill_crtc_timing(pBIOSInfo, mode_crt_table, VideoMode->mode_array,
			 (pBIOSInfo->bitsPerPixel >> 3), IGA2);

	if (pBIOSInfo->TMDSSettingInfo.ModeIndex == M640x480 &&
	    pBIOSInfo->FoundRefresh == 60) {
	    /* The border is 8 pixels. */
	    mode_crt_reg.hor_blank_start = mode_crt_reg.hor_blank_start - 8;

	    /* Blanking time should add left and right borders. */
	    mode_crt_reg.hor_blank_end = mode_crt_reg.hor_blank_end + 16;
	}

	/* Fill shadow registers */
	load_crtc_shadow_timing(pBIOSInfo, mode_crt_reg, mode_crt_reg);
	/* Fetch count for simultaneous*/        
	load_fetch_count_reg(mode_crt_reg.hor_addr, 4, IGA2);
    }

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VideoSetMode --Done--\n"));
}

unsigned char VIAGetDeviceDetectDVI(VIABIOSInfoPtr pBIOSInfo)
{
    unsigned char   ret = 0x0;

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIAGetDeviceDetectDVI\n"));

    if(pBIOSInfo->TMDSSettingInfo.ChipID)
    {
        if(VIASenseDVI(pBIOSInfo))
        {
            ret = VIA_DEVICE_DFP;
            pBIOSInfo->DVIAttach = TRUE;
            DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED, "DVI has Attachment.\n"));
            if(pBIOSInfo->TMDSSettingInfo.DFPSize == M_INVALID)
                VIAGetPanelInfo(pBIOSInfo);
        }
        else
        {
            pBIOSInfo->DVIAttach = FALSE;
            DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED, "DVI hasn't Attachment.\n"));
        }
    }

    return ret;
}

