/*
 * Copyright 1998-2007 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2007 S3 Graphics, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

/*************************************************************************
 *
 *  File:       hw.h
 *  Content:    migration from frame buffer device driver's function.
 *
 ************************************************************************/

#ifndef __HW_H__
#define __HW_H__ 1

/* MISC */
#define ARRAY_SIZE(x)   (sizeof(x)/sizeof(*(x)))

/* Sequencer Registers */
#define SR10    0x10
#define SR12    0x12
#define SR15    0x15
#define SR16    0x16
#define SR17    0x17
#define SR18    0x18
#define SR1A    0x1A
#define SR1B    0x1B
#define SR1C    0x1C
#define SR1D    0x1D
#define SR1E    0x1E
#define SR1F    0x1F
#define SR19    0x19
#define SR20    0x20
#define SR21    0x21
#define SR22    0x22
#define SR26    0x26
#define SR2A    0x2A
#define SR2C    0x2C
#define SR2D    0x2D
#define SR2E    0x2E

#define SR39    0x39
#define SR3E    0x3E
#define SR3F    0x3F
#define SR40    0x40
#define SR42    0x42
#define SR43    0x43
#define SR44    0x44
#define SR45    0x45
#define SR46    0x46
#define SR47    0x47
#define SR48    0x48
#define SR49    0x49
#define SR4A    0x4A
#define SR4B    0x4B
#define SR4C    0x4C
#define SR4F    0x4F
#define SR50    0x50
#define SR52    0x52
#define SR57    0x57
#define SR5D    0x5D
#define SR5E    0x5E
#define SR65    0x65


/* CRT Controller Registers */
#define CR00    0x00
#define CR01    0x01
#define CR02    0x02
#define CR03    0x03
#define CR04    0x04
#define CR05    0x05
#define CR06    0x06
#define CR07    0x07
#define CR08    0x08
#define CR09    0x09
#define CR0A    0x0A
#define CR0B    0x0B
#define CR0C    0x0C
#define CR0D    0x0D
#define CR0E    0x0E
#define CR0F    0x0F
#define CR10    0x10
#define CR11    0x11
#define CR12    0x12
#define CR13    0x13
#define CR14    0x14
#define CR15    0x15
#define CR16    0x16
#define CR17    0x17
#define CR18    0x18


/* Extend CRT Controller Registers */
#define CR30    0x30
#define CR31    0x31
#define CR32    0x32
#define CR33    0x33
#define CR34    0x34
#define CR35    0x35
#define CR36    0x36
#define CR37    0x37
#define CR38    0x38
#define CR39    0x39
#define CR3A    0x3A
#define CR3B    0x3B
#define CR3C    0x3C
#define CR3D    0x3D
#define CR3E    0x3E
#define CR3F    0x3F
#define CR40    0x40
#define CR41    0x41
#define CR42    0x42
#define CR43    0x43
#define CR44    0x44
#define CR45    0x45
#define CR46    0x46
#define CR47    0x47
#define CR48    0x48
#define CR49    0x49
#define CR4A    0x4A
#define CR4B    0x4B
#define CR4C    0x4C
#define CR4D    0x4D
#define CR4E    0x4E
#define CR4F    0x4F
#define CR50    0x50
#define CR51    0x51
#define CR52    0x52
#define CR53    0x53
#define CR54    0x54
#define CR55    0x55
#define CR56    0x56
#define CR57    0x57
#define CR58    0x58
#define CR59    0x59
#define CR5A    0x5A
#define CR5B    0x5B
#define CR5C    0x5C
#define CR5D    0x5D
#define CR5E    0x5E
#define CR5F    0x5F
#define CR60    0x60
#define CR61    0x61
#define CR62    0x62
#define CR63    0x63
#define CR64    0x64
#define CR65    0x65
#define CR66    0x66
#define CR67    0x67
#define CR68    0x68
#define CR69    0x69
#define CR6A    0x6A
#define CR6B    0x6B
#define CR6C    0x6C
#define CR6D    0x6D
#define CR6E    0x6E
#define CR6F    0x6F
#define CR70    0x70
#define CR71    0x71
#define CR72    0x72
#define CR73    0x73
#define CR74    0x74
#define CR75    0x75
#define CR76    0x76
#define CR77    0x77
#define CR78    0x78
#define CR79    0x79
#define CR7A    0x7A
#define CR7B    0x7B
#define CR7C    0x7C
#define CR7D    0x7D
#define CR7E    0x7E
#define CR7F    0x7F
#define CR80    0x80
#define CR81    0x81
#define CR82    0x82
#define CR83    0x83
#define CR84    0x84
#define CR85    0x85
#define CR86    0x86
#define CR87    0x87
#define CR88    0x88
#define CR89    0x89
#define CR8A    0x8A
#define CR8B    0x8B
#define CR8C    0x8C
#define CR8D    0x8D
#define CR8E    0x8E
#define CR8F    0x8F
#define CR90    0x90
#define CR91    0x91
#define CR92    0x92
#define CR93    0x93
#define CR94    0x94
#define CR95    0x95
#define CR96    0x96
#define CR97    0x97
#define CR98    0x98
#define CR99    0x99
#define CR9A    0x9A
#define CR9B    0x9B
#define CR9C    0x9C
#define CR9D    0x9D
#define CR9E    0x9E
#define CR9F    0x9F
#define CRA0    0xA0
#define CRA1    0xA1
#define CRA2    0xA2
#define CRA3    0xA3
#define CRA4    0xA4
#define CRA5    0xA5
#define CRA6    0xA6
#define CRA7    0xA7
#define CRA8    0xA8
#define CRA9    0xA9
#define CRAA    0xAA
#define CRAB    0xAB
#define CRAC    0xAC
#define CRD0    0xD0
#define CRD1    0xD1
#define CRD2    0xD2
#define CRD3    0xD3
#define CRD4    0xD4
#define CRD5    0xD5
#define CRD6    0xD6
#define CRD7    0xD7

/* Definition Refresh Rate */
#define REFRESH_50      50
#define REFRESH_60      60
#define REFRESH_75      75
#define REFRESH_85      85
#define REFRESH_100     100
#define REFRESH_120     120

/* Definition Sync Polarity*/
#define NEGATIVE        1
#define POSITIVE        0

/* 640x480@60 Sync Polarity (DMT Mode) */
#define M640X480_R60_HSP        NEGATIVE
#define M640X480_R60_VSP        NEGATIVE

/* 640x480@75 Sync Polarity (DMT Mode) */
#define M640X480_R75_HSP        NEGATIVE
#define M640X480_R75_VSP        NEGATIVE

/* 640x480@85 Sync Polarity (DMT Mode) */
#define M640X480_R85_HSP        NEGATIVE
#define M640X480_R85_VSP        NEGATIVE

/* 640x480@100 Sync Polarity (GTF Mode) */
#define M640X480_R100_HSP       NEGATIVE
#define M640X480_R100_VSP       POSITIVE

/* 640x480@120 Sync Polarity (GTF Mode) */
#define M640X480_R120_HSP       NEGATIVE
#define M640X480_R120_VSP       POSITIVE

/* 720x480@60 Sync Polarity  (GTF Mode) */
#define M720X480_R60_HSP        NEGATIVE
#define M720X480_R60_VSP        POSITIVE

/* 720x576@60 Sync Polarity  (GTF Mode) */
#define M720X576_R60_HSP        NEGATIVE
#define M720X576_R60_VSP        POSITIVE

/* 800x480@60 Sync Polarity  (GTF Mode) */
#define M800X480_R60_HSP        NEGATIVE
#define M800X480_R60_VSP        POSITIVE

/* 800x600@60 Sync Polarity (DMT Mode) */
#define M800X600_R60_HSP        POSITIVE
#define M800X600_R60_VSP        POSITIVE

/* 800x600@75 Sync Polarity (DMT Mode) */
#define M800X600_R75_HSP        POSITIVE
#define M800X600_R75_VSP        POSITIVE

/* 800x600@85 Sync Polarity (DMT Mode) */
#define M800X600_R85_HSP        POSITIVE
#define M800X600_R85_VSP        POSITIVE

/* 800x600@100 Sync Polarity (GTF Mode) */
#define M800X600_R100_HSP       NEGATIVE
#define M800X600_R100_VSP       POSITIVE

/* 800x600@120 Sync Polarity (GTF Mode) */
#define M800X600_R120_HSP       NEGATIVE
#define M800X600_R120_VSP       POSITIVE

/* 848x480@60 Sync Polarity  (GTF Mode) */
#define M848X480_R60_HSP        NEGATIVE
#define M848X480_R60_VSP        POSITIVE

/* 852x480@60 Sync Polarity  (GTF Mode) */
#define M852X480_R60_HSP        NEGATIVE
#define M852X480_R60_VSP        POSITIVE

/* 960x600@60 Sync Polarity (GTF Mode) */
#define M960X600_R60_HSP        NEGATIVE
#define M960X600_R60_VSP        POSITIVE

/* 1000x600@60 Sync Polarity (GTF Mode) */
#define M1000X600_R60_HSP       NEGATIVE
#define M1000X600_R60_VSP       POSITIVE

/* 1024x512@60 Sync Polarity (GTF Mode) */
#define M1024X512_R60_HSP       NEGATIVE
#define M1024X512_R60_VSP       POSITIVE

/* 1024x576@60 Sync Polarity (GTF Mode) */
#define M1024X576_R60_HSP       NEGATIVE
#define M1024X576_R60_VSP       POSITIVE

/*1024x600@60 Sync Polarity (GTF Mode)*/
#define M1024X600_R60_HSP       NEGATIVE            
#define M1024X600_R60_VSP       POSITIVE

/* 1024x768@60 Sync Polarity (DMT Mode) */
#define M1024X768_R60_HSP       NEGATIVE
#define M1024X768_R60_VSP       NEGATIVE

/* 1024x768@75 Sync Polarity (DMT Mode) */
#define M1024X768_R75_HSP       POSITIVE
#define M1024X768_R75_VSP       POSITIVE

/* 1024x768@85 Sync Polarity (DMT Mode) */
#define M1024X768_R85_HSP       POSITIVE
#define M1024X768_R85_VSP       POSITIVE

/* 1024x768@100 Sync Polarity (GTF Mode) */
#define M1024X768_R100_HSP      NEGATIVE
#define M1024X768_R100_VSP      POSITIVE

/* 1088x612@60 Sync Polarity (GTF Mode) */
#define M1088X612_R60_HSP       NEGATIVE
#define M1088X612_R60_VSP       POSITIVE

/* 1152x720@60 Sync Polarity (GTF Mode) */
#define M1152X720_R60_HSP       NEGATIVE
#define M1152X720_R60_VSP       POSITIVE

/* 1152x864@75 Sync Polarity (DMT Mode) */
#define M1152X864_R75_HSP       POSITIVE
#define M1152X864_R75_VSP       POSITIVE

/* 1200x720@60 Sync Polarity (GTF Mode) */
#define M1200X720_R60_HSP       NEGATIVE
#define M1200X720_R60_VSP       POSITIVE

/* 1280x600@60 Sync Polarity (GTF Mode) */
#define M1280x600_R60_HSP       NEGATIVE
#define M1280x600_R60_VSP       POSITIVE

/* 1280x720@50 Sync Polarity  (GTF Mode) */
#define M1280X720_R50_HSP       NEGATIVE
#define M1280X720_R50_VSP       POSITIVE

/* 1280x720@60 Sync Polarity  (GTF Mode) */
#define M1280X720_R60_HSP       NEGATIVE
#define M1280X720_R60_VSP       POSITIVE

/* 1280x768@50 Sync Polarity  (GTF Mode) */
#define M1280X768_R50_HSP       NEGATIVE
#define M1280X768_R50_VSP       POSITIVE

/* 1280x768@60 Sync Polarity  (GTF Mode) */
#define M1280X768_R60_HSP       NEGATIVE
#define M1280X768_R60_VSP       POSITIVE

/* 1280x800@60 Sync Polarity  (GTF Mode) */
#define M1280X800_R60_HSP       NEGATIVE
#define M1280X800_R60_VSP       POSITIVE

/* 1280x960@60 Sync Polarity (DMT Mode) */
#define M1280X960_R60_HSP       POSITIVE
#define M1280X960_R60_VSP       POSITIVE

/* 1280x1024@60 Sync Polarity (DMT Mode) */
#define M1280X1024_R60_HSP      POSITIVE
#define M1280X1024_R60_VSP      POSITIVE

/* 1280x1024@75 Sync Polarity (DMT Mode) */
#define M1280X1024_R75_HSP      POSITIVE
#define M1280X1024_R75_VSP      POSITIVE

/* 1280x1024@85 Sync Polarity (DMT Mode) */
#define M1280X1024_R85_HSP      POSITIVE
#define M1280X1024_R85_VSP      POSITIVE

/* 1360x768@60 Sync Polarity (GTF Mode) */
#define M1360X768_R60_HSP       POSITIVE
#define M1360X768_R60_VSP       POSITIVE

/* 1368x768@50 Sync Polarity (GTF Mode) */
#define M1368X768_R50_HSP       NEGATIVE
#define M1368X768_R50_VSP       POSITIVE

/* 1368x768@60 Sync Polarity (GTF Mode) */
#define M1368X768_R60_HSP       NEGATIVE
#define M1368X768_R60_VSP       POSITIVE

/* 1400x1050@60 Sync Polarity (GTF Mode) */
#define M1400X1050_R60_HSP      NEGATIVE
#define M1400X1050_R60_VSP      POSITIVE

/* 1440x900@60 Sync Polarity (GTF Mode) */
#define M1440X900_R60_HSP       NEGATIVE
#define M1440X900_R60_VSP       POSITIVE

/* 1440x1050@60 Sync Polarity (GTF Mode) */
#define M1440X1050_R60_HSP      NEGATIVE
#define M1440X1050_R60_VSP      POSITIVE

/* 1600x900@60 Sync Polarity (GTF Mode) */
#define M1600X900_R60_HSP       NEGATIVE
#define M1600X900_R60_VSP       POSITIVE

/* 1600x1024@60 Sync Polarity (GTF Mode) */
#define M1600X1024_R60_HSP      NEGATIVE
#define M1600X1024_R60_VSP      POSITIVE

/* 1600x1200@60 Sync Polarity (DMT Mode) */
#define M1600X1200_R60_HSP      POSITIVE
#define M1600X1200_R60_VSP      POSITIVE

/* 1600x1200@75 Sync Polarity (DMT Mode) */
#define M1600X1200_R75_HSP      POSITIVE
#define M1600X1200_R75_VSP      POSITIVE

/* 1680x600@60 Sync Polarity (GTF Mode) */
#define M1680x600_R60_HSP       NEGATIVE
#define M1680x600_R60_VSP       POSITIVE

/* 1680x1050@60 Sync Polarity (GTF Mode) */
#define M1680x1050_R60_HSP      NEGATIVE
#define M1680x1050_R60_VSP      NEGATIVE

/* 1792x1344@60 Sync Polarity (GTF Mode) */
#define M1792x1344_R60_HSP      NEGATIVE
#define M1792x1344_R60_VSP      POSITIVE

/* 1856x1392@60 Sync Polarity (GTF Mode) */
#define M1856x1392_R60_HSP      NEGATIVE
#define M1856x1392_R60_VSP      POSITIVE

/* 1920x1080@60 Sync Polarity (GTF & CVT Mode) */
#define M1920X1080_R60_HSP      NEGATIVE
#define M1920X1080_R60_VSP      POSITIVE

/* 1920x1080@60 Sync Polarity (CVT Reduce Blanking Mode) */
#define M1920X1080_DVI_R60_HSP  POSITIVE            
#define M1920X1080_DVI_R60_VSP  NEGATIVE

/* 1920x1200@60 Sync Polarity (GTF Mode) */
#define M1920X1200_R60_HSP      NEGATIVE
#define M1920X1200_R60_VSP      POSITIVE

/* 1920x1200@60 Sync Polarity (CVT Mode) */
#define M1920X1200_DVI_R60_HSP  POSITIVE            
#define M1920X1200_DVI_R60_VSP  NEGATIVE

/* 1920x1440@60 Sync Polarity (DMT Mode) */
#define M1920X1440_R60_HSP      NEGATIVE
#define M1920X1440_R60_VSP      POSITIVE

/* 1920x1440@75 Sync Polarity (DMT Mode) */
#define M1920X1440_R75_HSP      NEGATIVE
#define M1920X1440_R75_VSP      POSITIVE

/* 2048x1536@60 Sync Polarity (GTF Mode) */
#define M2048x1536_R60_HSP      NEGATIVE
#define M2048x1536_R60_VSP      POSITIVE

/* define PLL index: */
#define CLK_25_175M     25175000
#define CLK_26_719M     26719000
#define CLK_26_880M     26880000
#define CLK_29_581M     29581000
#define CLK_31_490M     31490000
#define CLK_31_500M     31500000
#define CLK_31_728M     31728000
#define CLK_32_668M     32688000
#define CLK_36_000M     36000000
#define CLK_40_000M     40000000
#define CLK_41_291M     41291000
#define CLK_43_163M     43163000
#define CLK_46_000M     46000000
#define CLK_46_996M     46996000
#define CLK_48_000M     48000000
#define CLK_48_875M     48875000
#define CLK_49_500M     49500000
#define CLK_52_406M     52406000
#define CLK_52_977M     52977000
#define CLK_56_250M     56250000
#define CLK_60_466M     60466000
#define CLK_61_500M     61500000
#define CLK_65_000M     65000000
#define CLK_65_178M     65178000
#define CLK_67_295M     67295000
#define CLK_68_179M     68179000
#define CLK_69_924M     69924000
#define CLK_70_159M     70159000
#define CLK_74_481M     74481000
#define CLK_78_750M     78750000
#define CLK_80_136M     80136000
#define CLK_83_375M     83375000
#define CLK_83_950M     83950000
#define CLK_85_500M     85500000
#define CLK_85_860M     85860000
#define CLK_94_500M     94500000
#define CLK_106_500M    106500000
#define CLK_108_000M    108000000
#define CLK_113_309M    113309000
#define CLK_118_840M    118840000
#define CLK_122_614M    122614000
#define CLK_125_104M    125104000
#define CLK_135_000M    135000000
#define CLK_136_700M    136700000
#define CLK_138_400M    138400000
#define CLK_144_300M    144300000
#define CLK_146_760M    146760000
/*#define CLK_148_500M    148500000*/
#define CLK_153_920M    153920000
#define CLK_157_500M    157500000
#define CLK_162_000M    162000000
#define CLK_172_798M    172798000
#define CLK_193_295M    193295000
#define CLK_202_500M    202500000
#define CLK_204_000M    204000000
#define CLK_218_500M    218500000
#define CLK_234_000M    234000000
#define CLK_267_250M    267250000
#define CLK_297_500M    297500000


/*CLE266 PLL value*/
#define CLE266_PLL_25_175M     0x0000C763
#define CLE266_PLL_26_719M     0x0000471A
#define CLE266_PLL_26_880M     0x0000440F
/*#define CLE266_PLL_29_581M     0x0000C877*/
#define CLE266_PLL_29_581M     0x00004F3E
#define CLE266_PLL_31_490M     0x00004721
#define CLE266_PLL_31_500M     0x0000C3B5
#define CLE266_PLL_31_728M     0x0000471F
#define CLE266_PLL_32_668M     0x0000C449
#define CLE266_PLL_36_000M     0x0000C5E5
#define CLE266_PLL_40_000M     0x0000C459
#define CLE266_PLL_41_291M     0x00004417
#define CLE266_PLL_43_163M     0x0000C579
#define CLE266_PLL_46_000M     0x0000875A
#define CLE266_PLL_46_996M     0x0000C4E9
#define CLE266_PLL_48_000M     0x00001443
#define CLE266_PLL_48_875M     0x00001D63
#define CLE266_PLL_49_500M     0x00008653
#define CLE266_PLL_52_406M     0x0000C475
#define CLE266_PLL_52_977M     0x00004525
#define CLE266_PLL_56_250M     0x000047B7
#define CLE266_PLL_60_466M     0x0000494C
#define CLE266_PLL_61_500M     0x00001456
#define CLE266_PLL_65_000M     0x000086ED
#define CLE266_PLL_65_178M     0x0000855B
#define CLE266_PLL_67_295M     0x0000855E
#define CLE266_PLL_68_179M     0x00000413
#define CLE266_PLL_69_924M     0x00001153
#define CLE266_PLL_70_159M     0x00001462
#define CLE266_PLL_74_481M     0x0000051A
#define CLE266_PLL_78_750M     0x00004321
#define CLE266_PLL_80_136M     0x0000051C
#define CLE266_PLL_83_375M     0x0000C25D
#define CLE266_PLL_83_950M     0x00000729
#define CLE266_PLL_85_500M     0x00008230           /* 85.909080 MHz*/
#define CLE266_PLL_85_860M     0x00004754
#define CLE266_PLL_94_500M     0x00000521
#define CLE266_PLL_106_500M    0x00008477          /* 106.491463 MHz*/
#define CLE266_PLL_108_000M    0x00008479
#define CLE266_PLL_113_309M    0x00000C5F
#define CLE266_PLL_118_840M    0x00004553
#define CLE266_PLL_122_614M    0x0000073C
#define CLE266_PLL_125_104M    0x000006B5
#define CLE266_PLL_135_000M    0x0000455E
#define CLE266_PLL_136_700M    0x00000C73
#define CLE266_PLL_138_400M    0x00000957
#define CLE266_PLL_144_300M    0x00008379
#define CLE266_PLL_146_760M    0x00004567
/*#define CLE266_PLL_148_500M    0x0000*/
#define CLE266_PLL_153_920M    0x00000856
#define CLE266_PLL_157_500M    0x000005B7
#define CLE266_PLL_162_000M    0x00004571
#define CLE266_PLL_172_798M    0x00004579    
#define CLE266_PLL_193_295M    0x0000086C
#define CLE266_PLL_202_500M    0x00000763
#define CLE266_PLL_204_000M    0x00000764
#define CLE266_PLL_218_500M    0x0000065C
#define CLE266_PLL_234_000M    0x00000662
#define CLE266_PLL_267_250M    0x00000670
#define CLE266_PLL_297_500M    0x000005E6


/*K800 PLL value*/
#define K800_PLL_25_175M     0x00539001
#define K800_PLL_26_719M     0x00759002
#define K800_PLL_26_880M     0x001C8C80
/*#define K800_PLL_29_581M     0x00000000*/
/*#define K800_PLL_29_581M     0x00619081*/ /* 29.710 Mhz  */
#define K800_PLL_29_581M     0x00A49003 /* 29.531 Mhz  */
#define K800_PLL_31_490M     0x006F9001
#define K800_PLL_31_500M     0x008B9002
#define K800_PLL_31_728M     0x00AF9003
#define K800_PLL_32_668M     0x00909002
#define K800_PLL_36_000M     0x009F9002
#define K800_PLL_40_000M     0x00578C02
#define K800_PLL_41_291M     0x00438C01
#define K800_PLL_43_163M     0x00778C03
#define K800_PLL_46_000M     0x00658C02
#define K800_PLL_46_996M     0x00818C83
#define K800_PLL_48_000M     0x00848C83
#define K800_PLL_48_875M     0x00508C81
#define K800_PLL_49_500M     0x00518C01
#define K800_PLL_52_406M     0x00738C02
#define K800_PLL_52_977M     0x00928C83
#define K800_PLL_56_250M     0x007C8C02
#define K800_PLL_60_466M     0x00A78C83
#define K800_PLL_61_500M     0x00AA8C83
#define K800_PLL_65_000M     0x006B8C01
#define K800_PLL_65_178M     0x00B48C83
#define K800_PLL_67_295M     0x00BA8C83
#define K800_PLL_68_179M     0x00708C01
#define K800_PLL_69_924M     0x00C18C83
#define K800_PLL_70_159M     0x00C28C83
#define K800_PLL_74_481M     0x007B8C81
#define K800_PLL_78_750M     0x00408801
#define K800_PLL_80_136M     0x00428801
#define K800_PLL_83_375M     0x005B0882
#define K800_PLL_83_950M     0x00738803
#define K800_PLL_85_500M     0x00768883
#define K800_PLL_85_860M     0x00768883
#define K800_PLL_94_500M     0x00828803
#define K800_PLL_106_500M    0x00758882        /* 106.491463 MHz */
#define K800_PLL_108_000M    0x00778882
#define K800_PLL_113_309M    0x005D8881
#define K800_PLL_118_840M    0x00A48883
#define K800_PLL_122_614M    0x00878882
#define K800_PLL_125_104M    0x00688801
#define K800_PLL_135_000M    0x001A4081
#define K800_PLL_136_700M    0x00BD8883
#define K800_PLL_138_400M    0x00728881
#define K800_PLL_144_300M    0x00778881
#define K800_PLL_146_760M    0x00CC8883
/*#define K800_PLL_148_500M    0x0000*/
#define K800_PLL_153_920M    0x00548482
#define K800_PLL_157_500M    0x00142080
#define K800_PLL_162_000M    0x006F8483
#define K800_PLL_172_798M    0x00778483   
#define K800_PLL_193_295M    0x004F8481
#define K800_PLL_202_500M    0x00538481
#define K800_PLL_204_000M    0x008D8483
#define K800_PLL_218_500M    0x00978483
#define K800_PLL_234_000M    0x00608401
#define K800_PLL_267_250M    0x006E8481
#define K800_PLL_297_500M    0x00A48402


/* PLL for PM800 */
#define PM800_PLL_25_175M     0x00539001
#define PM800_PLL_26_719M     0x00759002
#define PM800_PLL_26_880M     0x001C8C80
/*#define PM800_PLL_29_581M     0x00619081*/ /* 29.710 Mhz  */
#define PM800_PLL_29_581M     0x00A49003 /* 29.531 Mhz  */
#define PM800_PLL_31_490M     0x006F9001
#define PM800_PLL_31_500M     0x008B9002
#define PM800_PLL_31_728M     0x00AF9003
#define PM800_PLL_32_668M     0x00909002
#define PM800_PLL_36_000M     0x009F9002
#define PM800_PLL_40_000M     0x00578C02
#define PM800_PLL_41_291M     0x00438C01
#define PM800_PLL_43_163M     0x00778C03
#define PM800_PLL_46_000M     0x00658C02
#define PM800_PLL_46_996M     0x00818C83
#define PM800_PLL_48_000M     0x00848C83
#define PM800_PLL_48_875M     0x00508C81
#define PM800_PLL_49_500M     0x00518C01
#define PM800_PLL_52_406M     0x00738C02
#define PM800_PLL_52_977M     0x00928C83
#define PM800_PLL_56_250M     0x007C8C02
#define PM800_PLL_60_466M     0x00A78C83
#define PM800_PLL_61_500M     0x00AA8C83
#define PM800_PLL_65_000M     0x006B8C01
#define PM800_PLL_65_178M     0x00B48C83
#define PM800_PLL_67_295M     0x00BA8C83
#define PM800_PLL_68_179M     0x00708C01
#define PM800_PLL_69_924M     0x00C18C83
#define PM800_PLL_70_159M     0x00C28C83
#define PM800_PLL_74_481M     0x007B8C81
#define PM800_PLL_78_750M     0x00408801
#define PM800_PLL_80_136M     0x00428801
#define PM800_PLL_83_375M     0x005B0882
#define PM800_PLL_83_950M     0x00738803
#define PM800_PLL_85_500M     0x00768883         /* 85.909080 MHz*/
#define PM800_PLL_85_860M     0x00768883
#define PM800_PLL_94_500M     0x00828803
#define PM800_PLL_106_500M    0x00758882        /* 106.491463 MHz */
#define PM800_PLL_108_000M    0x00778882
#define PM800_PLL_113_309M    0x005D8881
#define PM800_PLL_118_840M    0x00A48883
#define PM800_PLL_122_614M    0x00878882
#define PM800_PLL_125_104M    0x00688801
#define PM800_PLL_135_000M    0x001A4081
#define PM800_PLL_136_700M    0x00BD8883
#define PM800_PLL_138_400M    0x00728881
#define PM800_PLL_144_300M    0x00778881
#define PM800_PLL_146_760M    0x00CC8883
#define PM800_PLL_153_920M    0x00548482
#define PM800_PLL_157_500M    0x00142080
#define PM800_PLL_162_000M    0x006F8483
#define PM800_PLL_172_798M    0x00778483
#define PM800_PLL_193_295M    0x004F8481
#define PM800_PLL_202_500M    0x00538481
#define PM800_PLL_204_000M    0x008D8483
#define PM800_PLL_218_500M    0x00978483
#define PM800_PLL_234_000M    0x00608401
#define PM800_PLL_267_250M    0x006E8481
#define PM800_PLL_297_500M    0x00A48402


/* PLL for VT3324 */
#define CX700_25_175M     0x008B1003
#define CX700_26_719M     0x00931003
#define CX700_26_880M     0x00941003
/*#define CX700_29_581M     0x00619081*/ /* 29.710 Mhz  */
#define CX700_29_581M     0x00A49003 /* 29.531 Mhz  */
#define CX700_31_490M     0x00AE1003
#define CX700_31_500M     0x00AE1003
#define CX700_31_728M     0x00AF1003
#define CX700_32_668M     0x00B51003
#define CX700_36_000M     0x00C81003
#define CX700_40_000M     0x006E0C03
#define CX700_41_291M     0x00710C03
#define CX700_43_163M     0x00770C03
#define CX700_46_000M     0x007F0C03
#define CX700_46_996M     0x00818C83
#define CX700_48_000M     0x00840C03
#define CX700_48_875M     0x00508C81
#define CX700_49_500M     0x00880C03
#define CX700_52_406M     0x00730C02
#define CX700_52_977M     0x00920C03
#define CX700_56_250M     0x009B0C03
#define CX700_60_466M     0x00460C00
#define CX700_61_500M     0x00AA0C03
#define CX700_65_000M     0x006B0C01
#define CX700_65_178M     0x006B0C01
#define CX700_67_295M     0x00BA0C03
#define CX700_68_179M     0x00BC0C03
#define CX700_69_924M     0x00C10C03
#define CX700_70_159M     0x00C20C03
#define CX700_74_481M     0x00CE0C03
#define CX700_78_750M     0x006C0803
#define CX700_80_136M     0x006E0803
#define CX700_83_375M     0x005B0882
#define CX700_83_950M     0x00730803
#define CX700_85_500M     0x00760803         /* 85.909080 MHz*/
#define CX700_85_860M     0x00760803
#define CX700_94_500M     0x00820803
#define CX700_106_500M    0x00750802
#define CX700_108_000M    0x00950803
#define CX700_122_614M    0x00870802
#define CX700_113_309M    0x005D0801
#define CX700_118_840M    0x00A40803
#define CX700_125_104M    0x00AD0803
#define CX700_135_000M    0x00950802
#define CX700_136_700M    0x00BD0803
#define CX700_138_400M    0x00720801
#define CX700_144_300M    0x00778881
#define CX700_146_760M    0x00CC0803
#define CX700_153_920M    0x00540402
#define CX700_157_500M    0x006C0403
#define CX700_162_000M    0x006F0403
#define CX700_172_798M    0x00770403
#define CX700_193_295M    0x00850403
#define CX700_202_500M    0x008C0403
#define CX700_204_000M    0x008D0403
#define CX700_218_500M    0x00970403
#define CX700_234_000M    0x00600401
#define CX700_267_250M    0x00B90403
#define CX700_297_500M    0x00CE0403

/* Define Bit Field */
#define BIT0    0x01
#define BIT1    0x02
#define BIT2    0x04
#define BIT3    0x08
#define BIT4    0x10
#define BIT5    0x20
#define BIT6    0x40
#define BIT7    0x80

/* Definition CRTC Timing Index */
#define H_TOTAL_INDEX               0
#define H_ADDR_INDEX                1
#define H_BLANK_START_INDEX         2
#define H_BLANK_END_INDEX           3
#define H_SYNC_START_INDEX          4
#define H_SYNC_END_INDEX            5
#define V_TOTAL_INDEX               6
#define V_ADDR_INDEX                7
#define V_BLANK_START_INDEX         8
#define V_BLANK_END_INDEX           9
#define V_SYNC_START_INDEX          10
#define V_SYNC_END_INDEX            11
#define H_TOTAL_SHADOW_INDEX        12
#define H_BLANK_END_SHADOW_INDEX    13
#define V_TOTAL_SHADOW_INDEX        14
#define V_ADDR_SHADOW_INDEX         15
#define V_BLANK_SATRT_SHADOW_INDEX  16
#define V_BLANK_END_SHADOW_INDEX    17
#define V_SYNC_SATRT_SHADOW_INDEX   18
#define V_SYNC_END_SHADOW_INDEX     19

/* Define TV Function Index */
#define TV_BASE_FUNC                0
#define TV_RGB_FUNC                 1
#define TV_YPBPR_FUNC               2
#define TV_SDTV_RGB_FUNC            3
#define TV_SDTV_YPBPR_FUNC          4
#define TV_DeDotCrawl_FUNC          5
#define TV_HDTV_720P_RGB_FUNC       6
#define TV_HDTV_720P_YPBPR_FUNC     7
#define TV_HDTV_1080I_RGB_FUNC      8
#define TV_HDTV_1080I_YPBPR_FUNC    9
#define TV_HDTV_1080P_RGB_FUNC      10
#define TV_HDTV_1080P_YPBPR_FUNC    11

/* define TV DeDotCrawl Register */
#define     VT1622_DEDOTCRAWL_REG       0x11
#define     VT1623_DEDOTCRAWL_REG       0x11
#define     VT1625_DEDOTCRAWL_REG       0x11

/***************************************************/
/* Definition IGA1 Design Method of CRTC Registers */
/***************************************************/
#define IGA1_HOR_TOTAL_FORMULA(x)           ((x)/8)-5
#define IGA1_HOR_ADDR_FORMULA(x)            ((x)/8)-1
#define IGA1_HOR_BLANK_START_FORMULA(x)     ((x)/8)-1
#define IGA1_HOR_BLANK_END_FORMULA(x,y)     ((x+y)/8)-1
#define IGA1_HOR_SYNC_START_FORMULA(x)      ((x)/8)
#define IGA1_HOR_SYNC_END_FORMULA(x,y)      ((x+y)/8)

#define IGA1_VER_TOTAL_FORMULA(x)           (x)-2
#define IGA1_VER_ADDR_FORMULA(x)            (x)-1
#define IGA1_VER_BLANK_START_FORMULA(x)     (x)-1
#define IGA1_VER_BLANK_END_FORMULA(x,y)     (x+y)-1
#define IGA1_VER_SYNC_START_FORMULA(x)      (x)-1
#define IGA1_VER_SYNC_END_FORMULA(x,y)      (x+y)-1

/***************************************************/
/* Definition IGA2 Design Method of CRTC Registers */
/***************************************************/
#define IGA2_HOR_TOTAL_FORMULA(x)           (x)-1
#define IGA2_HOR_ADDR_FORMULA(x)            (x)-1
#define IGA2_HOR_BLANK_START_FORMULA(x)     (x)-1
#define IGA2_HOR_BLANK_END_FORMULA(x,y)     (x+y)-1
#define IGA2_HOR_SYNC_START_FORMULA(x)      (x)-1
#define IGA2_HOR_SYNC_END_FORMULA(x,y)      (x+y)-1

#define IGA2_VER_TOTAL_FORMULA(x)           (x)-1
#define IGA2_VER_ADDR_FORMULA(x)            (x)-1
#define IGA2_VER_BLANK_START_FORMULA(x)     (x)-1
#define IGA2_VER_BLANK_END_FORMULA(x,y)     (x+y)-1
#define IGA2_VER_SYNC_START_FORMULA(x)      (x)-1
#define IGA2_VER_SYNC_END_FORMULA(x,y)      (x+y)-1

/**********************************************************/
/* Definition IGA2 Design Method of CRTC Shadow Registers */
/**********************************************************/
#define IGA2_HOR_TOTAL_SHADOW_FORMULA(x)           (x/8)-5
#define IGA2_HOR_BLANK_END_SHADOW_FORMULA(x,y)     ((x+y)/8)-1
#define IGA2_VER_TOTAL_SHADOW_FORMULA(x)           (x)-2
#define IGA2_VER_ADDR_SHADOW_FORMULA(x)            (x)-1
#define IGA2_VER_BLANK_START_SHADOW_FORMULA(x)     (x)-1
#define IGA2_VER_BLANK_END_SHADOW_FORMULA(x,y)     (x+y)-1
#define IGA2_VER_SYNC_START_SHADOW_FORMULA(x)      (x)
#define IGA2_VER_SYNC_END_SHADOW_FORMULA(x,y)      (x+y)

/* Define Register Number for IGA1 CRTC Timing */
#define IGA1_HOR_TOTAL_REG_NUM          2           /* location: {CR00,0,7},{CR36,3,3} */
#define IGA1_HOR_ADDR_REG_NUM           1           /* location: {CR01,0,7} */
#define IGA1_HOR_BLANK_START_REG_NUM    1           /* location: {CR02,0,7} */
#define IGA1_HOR_BLANK_END_REG_NUM      3           /* location: {CR03,0,4},{CR05,7,7},{CR33,5,5} */
#define IGA1_HOR_SYNC_START_REG_NUM     2           /* location: {CR04,0,7},{CR33,4,4} */
#define IGA1_HOR_SYNC_END_REG_NUM       1           /* location: {CR05,0,4} */
#define IGA1_VER_TOTAL_REG_NUM          4           /* location: {CR06,0,7},{CR07,0,0},{CR07,5,5},{CR35,0,0} */
#define IGA1_VER_ADDR_REG_NUM           4           /* location: {CR12,0,7},{CR07,1,1},{CR07,6,6},{CR35,2,2} */
#define IGA1_VER_BLANK_START_REG_NUM    4           /* location: {CR15,0,7},{CR07,3,3},{CR09,5,5},{CR35,3,3} */
#define IGA1_VER_BLANK_END_REG_NUM      1           /* location: {CR16,0,7} */
#define IGA1_VER_SYNC_START_REG_NUM     4           /* location: {CR10,0,7},{CR07,2,2},{CR07,7,7},{CR35,1,1} */
#define IGA1_VER_SYNC_END_REG_NUM       1           /* location: {CR11,0,3} */

/* Define Register Number for IGA1 Shadow CRTC Timing */
#define IGA2_SHADOW_HOR_TOTAL_REG_NUM       2       /* location: {CR6D,0,7},{CR71,3,3} */
#define IGA2_SHADOW_HOR_BLANK_END_REG_NUM   1       /* location: {CR6E,0,7}            */
#define IGA2_SHADOW_VER_TOTAL_REG_NUM       2       /* location: {CR6F,0,7},{CR71,0,2} */
#define IGA2_SHADOW_VER_ADDR_REG_NUM        2       /* location: {CR70,0,7},{CR71,4,6} */
#define IGA2_SHADOW_VER_BLANK_START_REG_NUM 2       /* location: {CR72,0,7},{CR74,4,6} */
#define IGA2_SHADOW_VER_BLANK_END_REG_NUM   2       /* location: {CR73,0,7},{CR74,0,2} */
#define IGA2_SHADOW_VER_SYNC_START_REG_NUM  2       /* location: {CR75,0,7},{CR76,4,6} */
#define IGA2_SHADOW_VER_SYNC_END_REG_NUM    1       /* location: {CR76,0,3}            */

/* Define Register Number for IGA2 CRTC Timing */
#define IGA2_HOR_TOTAL_REG_NUM          2           /* location: {CR50,0,7},{CR55,0,3} */
#define IGA2_HOR_ADDR_REG_NUM           2           /* location: {CR51,0,7},{CR55,4,6} */
#define IGA2_HOR_BLANK_START_REG_NUM    2           /* location: {CR52,0,7},{CR54,0,2} */
#define IGA2_HOR_BLANK_END_REG_NUM      3           /* location: CLE266: {CR53,0,7},{CR54,3,5} => CLE266's CR5D[6] is reserved, so it may have problem to set 1600x1200 on IGA2. */
                                                    /*           Others: {CR53,0,7},{CR54,3,5},{CR5D,6,6} */
#define IGA2_HOR_SYNC_START_REG_NUM     4           /* location: {CR56,0,7},{CR54,6,7},{CR5C,7,7} */
                                                    /*           VT3314 and Later: {CR56,0,7},{CR54,6,7},{CR5C,7,7}, {CR5D,7,7} */
#define IGA2_HOR_SYNC_END_REG_NUM       2           /* location: {CR57,0,7},{CR5C,6,6} */
#define IGA2_VER_TOTAL_REG_NUM          2           /* location: {CR58,0,7},{CR5D,0,2} */
#define IGA2_VER_ADDR_REG_NUM           2           /* location: {CR59,0,7},{CR5D,3,5} */
#define IGA2_VER_BLANK_START_REG_NUM    2           /* location: {CR5A,0,7},{CR5C,0,2} */
#define IGA2_VER_BLANK_END_REG_NUM      2           /* location: {CR5E,0,7},{CR5C,3,5} */
#define IGA2_VER_SYNC_START_REG_NUM     2           /* location: {CR5E,0,7},{CR5F,5,7} */
#define IGA2_VER_SYNC_END_REG_NUM       1           /* location: {CR5F,0,4} */

/* Define Offset and Fetch Count Register*/
#define IGA1_OFFSET_REG_NUM             2                                 /* location: {CR13,0,7},{CR35,5,7} */
#define IGA1_OFFSER_ALIGN_BYTE          8                                 /* 8 bytes alignment. */
#define IGA1_OFFSET_FORMULA(x,y)        (x*y)/IGA1_OFFSER_ALIGN_BYTE      /* x: H resolution, y: color depth */

#define IGA1_FETCH_COUNT_REG_NUM        2                                 /* location: {SR1C,0,7},{SR1D,0,1} */
#define IGA1_FETCH_COUNT_ALIGN_BYTE     16                                /* 16 bytes alignment. */
#define IGA1_FETCH_COUNT_PATCH_VALUE    4
#define IGA1_FETCH_COUNT_FORMULA(x,y)   ((x*y)/IGA1_FETCH_COUNT_ALIGN_BYTE)+ IGA1_FETCH_COUNT_PATCH_VALUE




#define IGA2_OFFSET_REG_NUM             2           /* location: {CR66,0,07},{CR67,0,1} */
#define IGA2_OFFSET_ALIGN_BYTE		    8
#define IGA2_OFFSET_FORMULA(x,y)	    (x*y)/IGA2_OFFSET_ALIGN_BYTE	 /* x: H resolution, y: color depth */

#define IGA2_FETCH_COUNT_REG_NUM        2           /* location: {CR65,0,07},{CR67,2,3} */
#define IGA2_FETCH_COUNT_ALIGN_BYTE	    16
#define IGA2_FETCH_COUNT_PATCH_VALUE	0
#define IGA2_FETCH_COUNT_FORMULA(x,y)	((x*y)/IGA1_FETCH_COUNT_ALIGN_BYTE)+ IGA2_FETCH_COUNT_PATCH_VALUE

/*Define Display OFFSET*/
/*These value are by HW suggested value*/
#define K800_IGA1_FIFO_MAX_DEPTH                384     /* location: {SR17,0,7} */
#define K800_IGA1_FIFO_THRESHOLD                328     /* location: {SR16,0,5},{SR16,7,7} */
#define K800_IGA1_FIFO_HIGH_THRESHOLD           296     /* location: {SR18,0,5},{SR18,7,7} */
#define K800_IGA1_DISPLAY_QUEUE_EXPIRE_NUM      0       /* location: {SR22,0,4}. (128/4) =64, K800 must be set zero, */
                                                        /* because HW only 5 bits */

#define K800_IGA2_FIFO_MAX_DEPTH                384     /* location: {CR68,4,7},{CR94,7,7},{CR95,7,7} */
#define K800_IGA2_FIFO_THRESHOLD                328     /* location: {CR68,0,3},{CR95,4,6} */
#define K800_IGA2_FIFO_HIGH_THRESHOLD           296     /* location: {CR92,0,3},{CR95,0,2} */
#define K800_IGA2_DISPLAY_QUEUE_EXPIRE_NUM      128     /* location: {CR94,0,6} */

#define P880_IGA1_FIFO_MAX_DEPTH                192     /* location: {SR17,0,7} */
#define P880_IGA1_FIFO_THRESHOLD                128     /* location: {SR16,0,5},{SR16,7,7} */
#define P880_IGA1_FIFO_HIGH_THRESHOLD           64      /* location: {SR18,0,5},{SR18,7,7} */
#define P880_IGA1_DISPLAY_QUEUE_EXPIRE_NUM      0       /* location: {SR22,0,4}. (128/4) =64, K800 must be set zero, */
                                                        /* because HW only 5 bits */

#define P880_IGA2_FIFO_MAX_DEPTH                96      /* location: {CR68,4,7},{CR94,7,7},{CR95,7,7} */
#define P880_IGA2_FIFO_THRESHOLD                64      /* location: {CR68,0,3},{CR95,4,6} */
#define P880_IGA2_FIFO_HIGH_THRESHOLD           32      /* location: {CR92,0,3},{CR95,0,2} */
#define P880_IGA2_DISPLAY_QUEUE_EXPIRE_NUM      128     /* location: {CR94,0,6} */

/* VT3314 chipset*/
#define CN900_IGA1_FIFO_MAX_DEPTH               96 /* location: {SR17,0,7}*/
#define CN900_IGA1_FIFO_THRESHOLD               80 /* location: {SR16,0,5},{SR16,7,7}*/
#define CN900_IGA1_FIFO_HIGH_THRESHOLD          64  /* location: {SR18,0,5},{SR18,7,7}*/
#define CN900_IGA1_DISPLAY_QUEUE_EXPIRE_NUM     0   /* location: {SR22,0,4}. (128/4) =64, P800 must be set zero, because HW only 5 bits*/

#define CN900_IGA2_FIFO_MAX_DEPTH               96  /* location: {CR68,4,7},{CR94,7,7},{CR95,7,7}*/
#define CN900_IGA2_FIFO_THRESHOLD               80  /* location: {CR68,0,3},{CR95,4,6}*/
#define CN900_IGA2_FIFO_HIGH_THRESHOLD          32   /* location: {CR92,0,3},{CR95,0,2}*/
#define CN900_IGA2_DISPLAY_QUEUE_EXPIRE_NUM     128 /* location: {CR94,0,6}*/

/* For VT3324, these values are suggested by HW */
#define CX700_IGA1_FIFO_MAX_DEPTH               192     /* location: {SR17,0,7}*/
#define CX700_IGA1_FIFO_THRESHOLD               128     /* location: {SR16,0,5},{SR16,7,7}*/
#define CX700_IGA1_FIFO_HIGH_THRESHOLD          128     /* location: {SR18,0,5},{SR18,7,7} */
#define CX700_IGA1_DISPLAY_QUEUE_EXPIRE_NUM     124     /* location: {SR22,0,4} */

#define CX700_IGA2_FIFO_MAX_DEPTH               96      /* location: {CR68,4,7},{CR94,7,7},{CR95,7,7}*/
#define CX700_IGA2_FIFO_THRESHOLD               64      /* location: {CR68,0,3},{CR95,4,6}*/
#define CX700_IGA2_FIFO_HIGH_THRESHOLD          32      /* location: {CR92,0,3},{CR95,0,2} */
#define CX700_IGA2_DISPLAY_QUEUE_EXPIRE_NUM     128     /* location: {CR94,0,6}*/

/* VT3336 chipset*/
#define K8M890_IGA1_FIFO_MAX_DEPTH               360 /* location: {SR17,0,7}*/
#define K8M890_IGA1_FIFO_THRESHOLD               328 /* location: {SR16,0,5},{SR16,7,7}*/
#define K8M890_IGA1_FIFO_HIGH_THRESHOLD          296 /* location: {SR18,0,5},{SR18,7,7}*/
#define K8M890_IGA1_DISPLAY_QUEUE_EXPIRE_NUM     124 /* location: {SR22,0,4}.*/

#define K8M890_IGA2_FIFO_MAX_DEPTH               360 /* location: {CR68,4,7},{CR94,7,7},{CR95,7,7}*/
#define K8M890_IGA2_FIFO_THRESHOLD               328 /* location: {CR68,0,3},{CR95,4,6}*/
#define K8M890_IGA2_FIFO_HIGH_THRESHOLD          296 /* location: {CR92,0,3},{CR95,0,2}*/
#define K8M890_IGA2_DISPLAY_QUEUE_EXPIRE_NUM     124 /* location: {CR94,0,6}*/


/* VT3327 chipset*/
#define P4M890_IGA1_FIFO_MAX_DEPTH               96  /* location: {SR17,0,7}*/
#define P4M890_IGA1_FIFO_THRESHOLD               76  /* location: {SR16,0,5},{SR16,7,7}*/
#define P4M890_IGA1_FIFO_HIGH_THRESHOLD          64  /* location: {SR18,0,5},{SR18,7,7}*/
#define P4M890_IGA1_DISPLAY_QUEUE_EXPIRE_NUM     32  /* location: {SR22,0,4}. (32/4) =8*/

#define P4M890_IGA2_FIFO_MAX_DEPTH               96  /* location: {CR68,4,7},{CR94,7,7},{CR95,7,7}*/
#define P4M890_IGA2_FIFO_THRESHOLD               76  /* location: {CR68,0,3},{CR95,4,6}*/
#define P4M890_IGA2_FIFO_HIGH_THRESHOLD          64  /* location: {CR92,0,3},{CR95,0,2}*/
#define P4M890_IGA2_DISPLAY_QUEUE_EXPIRE_NUM     32  /* location: {CR94,0,6}*/


/* For VT3293  */
#define CN750_IGA1_FIFO_MAX_DEPTH               96     /* location: {SR17,0,7}*/
#define CN750_IGA1_FIFO_THRESHOLD               76     /* location: {SR16,0,5},{SR16,7,7}*/
#define CN750_IGA1_FIFO_HIGH_THRESHOLD          76     /* location: {SR18,0,5},{SR18,7,7} */
#define CN750_IGA1_DISPLAY_QUEUE_EXPIRE_NUM     32     /* location: {SR22,0,4} */

#define CN750_IGA2_FIFO_MAX_DEPTH               96      /* location: {CR68,4,7},{CR94,7,7},{CR95,7,7}*/
#define CN750_IGA2_FIFO_THRESHOLD               76      /* location: {CR68,0,3},{CR95,4,6}*/
#define CN750_IGA2_FIFO_HIGH_THRESHOLD          76      /* location: {CR92,0,3},{CR95,0,2} */
#define CN750_IGA2_DISPLAY_QUEUE_EXPIRE_NUM     32     /* location: {CR94,0,6}*/

/* VT3364 chipset*/
#define P4M900_IGA1_FIFO_MAX_DEPTH               96     /* location: {SR17,0,7}*/
#define P4M900_IGA1_FIFO_THRESHOLD               76     /* location: {SR16,0,5},{SR16,7,7}*/
#define P4M900_IGA1_FIFO_HIGH_THRESHOLD          76     /* location: {SR18,0,5},{SR18,7,7}*/
#define P4M900_IGA1_DISPLAY_QUEUE_EXPIRE_NUM     32     /* location: {SR22,0,4}.*/

#define P4M900_IGA2_FIFO_MAX_DEPTH               96     /* location: {CR68,4,7},{CR94,7,7},{CR95,7,7}*/
#define P4M900_IGA2_FIFO_THRESHOLD               76     /* location: {CR68,0,3},{CR95,4,6}*/
#define P4M900_IGA2_FIFO_HIGH_THRESHOLD          76     /* location: {CR92,0,3},{CR95,0,2}*/
#define P4M900_IGA2_DISPLAY_QUEUE_EXPIRE_NUM     32     /* location: {CR94,0,6}*/

#define IGA1_FIFO_DEPTH_SELECT_REG_NUM          1
#define IGA1_FIFO_THRESHOLD_REG_NUM             2
#define IGA1_FIFO_HIGH_THRESHOLD_REG_NUM        2
#define IGA1_DISPLAY_QUEUE_EXPIRE_NUM_REG_NUM   1

#define IGA2_FIFO_DEPTH_SELECT_REG_NUM          3
#define IGA2_FIFO_THRESHOLD_REG_NUM             2
#define IGA2_FIFO_HIGH_THRESHOLD_REG_NUM        2
#define IGA2_DISPLAY_QUEUE_EXPIRE_NUM_REG_NUM   1


#define IGA1_FIFO_DEPTH_SELECT_FORMULA(x)                   (x/2)-1
#define IGA1_FIFO_THRESHOLD_FORMULA(x)                      (x/4)
#define IGA1_DISPLAY_QUEUE_EXPIRE_NUM_FORMULA(x)            (x/4)
#define IGA1_FIFO_HIGH_THRESHOLD_FORMULA(x)                 (x/4)
#define IGA2_FIFO_DEPTH_SELECT_FORMULA(x)                   ((x/2)/4)-1
#define IGA2_FIFO_THRESHOLD_FORMULA(x)                      (x/4)
#define IGA2_DISPLAY_QUEUE_EXPIRE_NUM_FORMULA(x)            (x/4)
#define IGA2_FIFO_HIGH_THRESHOLD_FORMULA(x)                 (x/4)


/* LCD Scaling factor                  */
/* x: indicate setting horizontal size */
/* y: indicate panel horizontal size   */
#define CLE266_LCD_HOR_SCF_FORMULA(x,y)     (((x-1)*1024)/(y-1))    /* Horizontal scaling factor 10 bits (2^10) */
#define CLE266_LCD_VER_SCF_FORMULA(x,y)     (((x-1)*1024)/(y-1))    /* Vertical scaling factor 10 bits (2^10)   */
#define K800_LCD_HOR_SCF_FORMULA(x,y)       (((x-1)*4096)/(y-1))    /* Horizontal scaling factor 10 bits (2^12) */
#define K800_LCD_VER_SCF_FORMULA(x,y)       (((x-1)*2048)/(y-1))    /* Vertical scaling factor 10 bits (2^11)   */

#define LCD_HOR_SCALING_FACTOR_REG_NUM      3               /* location: {CR9F,0,1},{CR77,0,7},{CR79,4,5} */
#define LCD_VER_SCALING_FACTOR_REG_NUM      3               /* location: {CR79,3,3},{CR78,0,7},{CR79,6,7} */
#define LCD_HOR_SCALING_FACTOR_REG_NUM_CLE  2               /* location: {CR77,0,7},{CR79,4,5} */
#define LCD_VER_SCALING_FACTOR_REG_NUM_CLE  2               /* location: {CR78,0,7},{CR79,6,7} */


/*Define display timing*/

struct display_timing {
    CARD16  hor_total;
    CARD16  hor_addr;
    CARD16  hor_blank_start;
    CARD16  hor_blank_end;
    CARD16  hor_sync_start;
    CARD16  hor_sync_end;
    CARD16  ver_total;
    CARD16  ver_addr;
    CARD16  ver_blank_start;
    CARD16  ver_blank_end;
    CARD16  ver_sync_start;
    CARD16  ver_sync_end;
};

/************************************************/
/*      Define IGA1 Display Timing              */
/************************************************/
struct io_register {
    CARD8   io_addr;
    CARD8   start_bit;
    CARD8   end_bit;
};


/* IGA1 Horizontal Total */
struct iga1_hor_total
{
    int     reg_num;
    struct  io_register reg[IGA1_HOR_TOTAL_REG_NUM];
};

/* IGA1 Horizontal Addressable Video */
struct iga1_hor_addr {
    int     reg_num;
    struct  io_register reg[IGA1_HOR_ADDR_REG_NUM];
};

/* IGA1 Horizontal Blank Start */
struct iga1_hor_blank_start {
    int     reg_num;
    struct  io_register reg[IGA1_HOR_BLANK_START_REG_NUM];
};

/* IGA1 Horizontal Blank End */
struct iga1_hor_blank_end {
    int     reg_num;
    struct  io_register reg[IGA1_HOR_BLANK_END_REG_NUM];
};

/* IGA1 Horizontal Sync Start */
struct iga1_hor_sync_start {
    int     reg_num;
    struct  io_register reg[IGA1_HOR_SYNC_START_REG_NUM];
};

/* IGA1 Horizontal Sync End */
struct iga1_hor_sync_end {
    int     reg_num;
    struct  io_register reg[IGA1_HOR_SYNC_END_REG_NUM];
};

/* IGA1 Vertical Total */
struct iga1_ver_total {
    int     reg_num;
    struct  io_register reg[IGA1_VER_TOTAL_REG_NUM];
};

/* IGA1 Vertical Addressable Video */
struct iga1_ver_addr {
    int     reg_num;
    struct  io_register reg[IGA1_VER_ADDR_REG_NUM];
};

/* IGA1 Vertical Blank Start */
struct iga1_ver_blank_start {
    int     reg_num;
    struct  io_register reg[IGA1_VER_BLANK_START_REG_NUM];
};

/* IGA1 Vertical Blank End */
struct iga1_ver_blank_end {
    int     reg_num;
    struct  io_register reg[IGA1_VER_BLANK_END_REG_NUM];
};

/* IGA1 Vertical Sync Start */
struct iga1_ver_sync_start {
    int     reg_num;
    struct  io_register reg[IGA1_VER_SYNC_START_REG_NUM];
};

/* IGA1 Vertical Sync End */
struct iga1_ver_sync_end {
    int     reg_num;
    struct  io_register reg[IGA1_VER_SYNC_END_REG_NUM];
};

/************************************************/
/*      Define IGA2 Shadow Display Timing       */
/************************************************/

/* IGA2 Shadow Horizontal Total */
struct iga2_shadow_hor_total {
    int     reg_num;
    struct  io_register reg[IGA2_SHADOW_HOR_TOTAL_REG_NUM];
};

/* IGA2 Shadow Horizontal Blank End */
struct iga2_shadow_hor_blank_end {
    int     reg_num;
    struct  io_register reg[IGA2_SHADOW_HOR_BLANK_END_REG_NUM];
};

/* IGA2 Shadow Vertical Total */
struct iga2_shadow_ver_total {
    int     reg_num;
    struct  io_register reg[IGA2_SHADOW_VER_TOTAL_REG_NUM];
};

/* IGA2 Shadow Vertical Addressable Video */
struct iga2_shadow_ver_addr {
    int     reg_num;
    struct  io_register reg[IGA2_SHADOW_VER_ADDR_REG_NUM];
};

/* IGA2 Shadow Vertical Blank Start */
struct iga2_shadow_ver_blank_start {
    int     reg_num;
    struct  io_register reg[IGA2_SHADOW_VER_BLANK_START_REG_NUM];
};

/* IGA2 Shadow Vertical Blank End */
struct iga2_shadow_ver_blank_end {
    int     reg_num;
    struct  io_register reg[IGA2_SHADOW_VER_BLANK_END_REG_NUM];
};

/* IGA2 Shadow Vertical Sync Start */
struct iga2_shadow_ver_sync_start {
    int     reg_num;
    struct  io_register reg[IGA2_SHADOW_VER_SYNC_START_REG_NUM];
};

/* IGA2 Shadow Vertical Sync End */
struct iga2_shadow_ver_sync_end {
    int     reg_num;
    struct  io_register reg[IGA2_SHADOW_VER_SYNC_END_REG_NUM];
};

/************************************************/
/*      Define IGA2 Display Timing              */
/************************************************/

/* IGA2 Horizontal Total */
struct iga2_hor_total {
    int     reg_num;
    struct  io_register reg[IGA2_HOR_TOTAL_REG_NUM];
};

/* IGA2 Horizontal Addressable Video */
struct iga2_hor_addr {
    int     reg_num;
    struct  io_register reg[IGA2_HOR_ADDR_REG_NUM];
};

/* IGA2 Horizontal Blank Start */
struct iga2_hor_blank_start {
    int     reg_num;
    struct  io_register reg[IGA2_HOR_BLANK_START_REG_NUM];
};

/* IGA2 Horizontal Blank End */
struct iga2_hor_blank_end {
    int     reg_num;
    struct  io_register reg[IGA2_HOR_BLANK_END_REG_NUM];
};

/* IGA2 Horizontal Sync Start */
struct iga2_hor_sync_start {
    int     reg_num;
    struct  io_register reg[IGA2_HOR_SYNC_START_REG_NUM];
};

/* IGA2 Horizontal Sync End */
struct iga2_hor_sync_end {
    int     reg_num;
    struct  io_register reg[IGA2_HOR_SYNC_END_REG_NUM];
};

/* IGA2 Vertical Total */
struct iga2_ver_total {
    int     reg_num;
    struct  io_register reg[IGA2_VER_TOTAL_REG_NUM];
};

/* IGA2 Vertical Addressable Video */
struct iga2_ver_addr {
    int     reg_num;
    struct  io_register reg[IGA2_VER_ADDR_REG_NUM];
};

/* IGA2 Vertical Blank Start */
struct iga2_ver_blank_start {
    int     reg_num;
    struct  io_register reg[IGA2_VER_BLANK_START_REG_NUM];
};

/* IGA2 Vertical Blank End */
struct iga2_ver_blank_end {
    int     reg_num;
    struct  io_register reg[IGA2_VER_BLANK_END_REG_NUM];
};

/* IGA2 Vertical Sync Start */
struct iga2_ver_sync_start {
    int     reg_num;
    struct  io_register reg[IGA2_VER_SYNC_START_REG_NUM];
};

/* IGA2 Vertical Sync End */
struct iga2_ver_sync_end {
    int     reg_num;
    struct  io_register reg[IGA2_VER_SYNC_END_REG_NUM];
};

/* IGA1 Offset Register */
struct iga1_offset {
    int     reg_num;
    struct  io_register reg[IGA1_OFFSET_REG_NUM];
};

/* IGA2 Offset Register */
struct iga2_offset {
    int     reg_num;
    struct  io_register reg[IGA2_OFFSET_REG_NUM];
};

struct offset{
    struct iga1_offset            iga1_offset_reg;
    struct iga2_offset            iga2_offset_reg;
};

/* IGA1 Fetch Count Register */
struct iga1_fetch_count {
    int     reg_num;
    struct  io_register reg[IGA1_FETCH_COUNT_REG_NUM];
};

/* IGA2 Fetch Count Register */
struct iga2_fetch_count {
    int     reg_num;
    struct  io_register reg[IGA2_FETCH_COUNT_REG_NUM];
};

struct fetch_count{
    struct iga1_fetch_count       iga1_fetch_count_reg;
    struct iga2_fetch_count       iga2_fetch_count_reg;
};

struct pll_map {
    CARD32  clk;
    CARD32  cle266_pll;
    CARD32  k800_pll;
    CARD32  pm800_pll;
    CARD32  cx700_pll;
};

/* LCD Scaling Factor */
struct _lcd_hor_scaling_factor{
    int     reg_num;
    struct  io_register reg[LCD_HOR_SCALING_FACTOR_REG_NUM];
};

struct _lcd_ver_scaling_factor{
    int     reg_num;
    struct  io_register reg[LCD_VER_SCALING_FACTOR_REG_NUM];
};

struct _lcd_scaling_factor{
    struct _lcd_hor_scaling_factor  lcd_hor_scaling_factor;
    struct _lcd_ver_scaling_factor  lcd_ver_scaling_factor;
};

/*Display FIFO Relation Registers*/
struct iga1_fifo_depth_select {
    int     reg_num;
    struct  io_register reg[IGA1_FIFO_DEPTH_SELECT_REG_NUM];
};

struct iga1_fifo_threshold_select {
    int     reg_num;
    struct  io_register reg[IGA1_FIFO_THRESHOLD_REG_NUM];
};

struct iga1_fifo_high_threshold_select {
    int     reg_num;
    struct  io_register reg[IGA1_FIFO_HIGH_THRESHOLD_REG_NUM];
};

struct iga1_display_queue_expire_num {
    int     reg_num;
    struct  io_register reg[IGA1_DISPLAY_QUEUE_EXPIRE_NUM_REG_NUM];
};

struct iga2_fifo_depth_select {
    int     reg_num;
    struct  io_register reg[IGA2_FIFO_DEPTH_SELECT_REG_NUM];
};

struct iga2_fifo_threshold_select {
    int     reg_num;
    struct  io_register reg[IGA2_FIFO_THRESHOLD_REG_NUM];
};

struct iga2_fifo_high_threshold_select {
    int     reg_num;
    struct  io_register reg[IGA2_FIFO_HIGH_THRESHOLD_REG_NUM];
};

struct iga2_display_queue_expire_num {
    int     reg_num;
    struct  io_register reg[IGA2_DISPLAY_QUEUE_EXPIRE_NUM_REG_NUM];
};

struct fifo_depth_select {
    struct  iga1_fifo_depth_select iga1_fifo_depth_select_reg;
    struct  iga2_fifo_depth_select iga2_fifo_depth_select_reg;
};

struct fifo_threshold_select {
    struct  iga1_fifo_threshold_select iga1_fifo_threshold_select_reg;
    struct  iga2_fifo_threshold_select iga2_fifo_threshold_select_reg;
};

struct fifo_high_threshold_select {
    struct  iga1_fifo_high_threshold_select iga1_fifo_high_threshold_select_reg;
    struct  iga2_fifo_high_threshold_select iga2_fifo_high_threshold_select_reg;
};

struct display_queue_expire_num {
    struct  iga1_display_queue_expire_num iga1_display_queue_expire_num_reg;
    struct  iga2_display_queue_expire_num iga2_display_queue_expire_num_reg;
};



struct iga1_crtc_timing {
    struct iga1_hor_total         hor_total;
    struct iga1_hor_addr          hor_addr;
    struct iga1_hor_blank_start   hor_blank_start;
    struct iga1_hor_blank_end     hor_blank_end;
    struct iga1_hor_sync_start    hor_sync_start;
    struct iga1_hor_sync_end      hor_sync_end;
    struct iga1_ver_total         ver_total;
    struct iga1_ver_addr          ver_addr;
    struct iga1_ver_blank_start   ver_blank_start;
    struct iga1_ver_blank_end     ver_blank_end;
    struct iga1_ver_sync_start    ver_sync_start;
    struct iga1_ver_sync_end      ver_sync_end;
};

struct iga2_shadow_crtc_timing {
    struct iga2_shadow_hor_total        hor_total_shadow;
    struct iga2_shadow_hor_blank_end    hor_blank_end_shadow;
    struct iga2_shadow_ver_total        ver_total_shadow;
    struct iga2_shadow_ver_addr         ver_addr_shadow;
    struct iga2_shadow_ver_blank_start  ver_blank_start_shadow;
    struct iga2_shadow_ver_blank_end    ver_blank_end_shadow;
    struct iga2_shadow_ver_sync_start   ver_sync_start_shadow;
    struct iga2_shadow_ver_sync_end     ver_sync_end_shadow;
};

struct iga2_crtc_timing {
    struct iga2_hor_total         hor_total;
    struct iga2_hor_addr          hor_addr;
    struct iga2_hor_blank_start   hor_blank_start;
    struct iga2_hor_blank_end     hor_blank_end;
    struct iga2_hor_sync_start    hor_sync_start;
    struct iga2_hor_sync_end      hor_sync_end;
    struct iga2_ver_total         ver_total;
    struct iga2_ver_addr          ver_addr;
    struct iga2_ver_blank_start   ver_blank_start;
    struct iga2_ver_blank_end     ver_blank_end;
    struct iga2_ver_sync_start    ver_sync_start;
    struct iga2_ver_sync_end      ver_sync_end;
};

typedef struct IODATA {
    CARD8   Index;
    CARD8   Mask;
    CARD8   Data;
}IODATA, *IODATAPTR;


extern CARD8 *MMIOMapBase;

static __inline__ CARD8 read_reg(int io_port, CARD8 index);
static __inline__ void write_reg(CARD8 index, CARD16 io_port, CARD8 data);
static __inline__ void write_reg_mask(CARD8 index, int io_port, CARD8 data, CARD8 mask);

static __inline__ CARD8
read_reg(int io_port, CARD8 index)
{
    MMIO_OUT8(MMIOMapBase, io_port, index);
    return(MMIO_IN8(MMIOMapBase, io_port+1));
}

static __inline__ void
write_reg(CARD8 index, CARD16 io_port, CARD8 data)
{
    MMIO_OUT8(MMIOMapBase, io_port, index);
    MMIO_OUT8(MMIOMapBase, io_port+1, data);
    DEBUG(ErrorF("Index=0x%2X, Value=0x%2X\n", index, data));
}

static __inline__ void
write_reg_mask(CARD8 index, int io_port, CARD8 data, CARD8 mask)
{
    CARD8 tmp;

    MMIO_OUT8(MMIOMapBase, io_port, index);
    tmp = MMIO_IN8(MMIOMapBase, io_port+1);
    MMIO_OUT8(MMIOMapBase, io_port+1, (data & mask) | (tmp & (~mask)));
    DEBUG(ErrorF("Index=0x%2X, Value=0x%2X\n", index, ((data & mask) | (tmp & (~mask)))));
}


/******************************************************************************
 * Global data								      *
 ******************************************************************************/
extern struct pll_map pll_value[];
extern struct fifo_depth_select display_fifo_depth_reg;
extern struct fifo_threshold_select fifo_threshold_select_reg;
extern struct fifo_high_threshold_select fifo_high_threshold_select_reg;
extern struct display_queue_expire_num display_queue_expire_num_reg;
extern struct offset offset_reg;
extern struct fetch_count fetch_count_reg;
extern struct iga1_crtc_timing iga1_crtc_reg;
extern struct iga2_shadow_crtc_timing iga2_shadow_crtc_reg;
extern struct iga2_crtc_timing iga2_crtc_reg;

#include "via_mode.h"

/******************************************************************************
 * Prototypes								      *
 ******************************************************************************/
void load_reg(int timing_value, int load_reg_num, struct io_register *reg, int io_type);
void load_offset_reg(int h_addr, int bpp_byte, int set_iga);
void load_fetch_count_reg(int h_addr, int bpp_byte, int set_iga);
void EnableSecondDisplayChannel(void);
void DisableSecondDisplayChannel(void);

#endif /* __HW_H__ */
