/*
 * Copyright 1998-2007 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2007 S3 Graphics, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
#include "via_driver.h"
#include "via_lcd.h"
#include "via_hw.h"
#include "via_vt1636.h"

/* Functions Prototype Declaration */
struct display_timing lcd_centering_timging(struct display_timing mode_crt_reg, struct display_timing panel_crt_reg);

/* Extern Functions Prototypes Declaration */
extern struct display_timing VIAGetTimingValue(VIABIOSInfoPtr pBIOSInfo, struct crt_mode_table *crt_table);
extern void fill_crtc_timing(VIABIOSInfoPtr pBIOSInfo, struct crt_mode_table *crt_table, int mode_array, int bpp_byte, int set_iga);
extern void load_offset_reg(int h_addr, int bpp_byte, int set_iga);
extern void load_fetch_count_reg(int h_addr, int bpp_byte, int set_iga);
extern void load_reg(int timing_value, int load_reg_num, struct io_register *reg, int io_type);
extern void write_regx(struct io_reg RegTable[],int ItemNum);
extern int SearchModeSetting(int ModeInfoIndex);
extern void load_crtc_timing(VIABIOSInfoPtr pBIOSInfo, struct display_timing device_timing, int set_iga);
extern void load_FIFO_reg(VIABIOSInfoPtr pBIOSInfo, int hor_active, int ver_active, int set_iga);
extern void SetVCLK(VIABIOSInfoPtr pBIOSInfo, CARD32 CLK, int set_iga);
extern void unlock_crt(void);
extern void lock_crt(void);

static Bool VIALVDSIdentify_VT1631(VIABIOSInfoPtr pBIOSInfo, LVDSSETTINGINFOPTR pLVDSSettingInfo, CARD32 PortNum);
static Bool VIALVDSIdentify_IntegratedLVDS(VIABIOSInfoPtr pBIOSInfo);
static void VIAPowerOnLVDSChannel(VIABIOSInfoPtr pBIOSInfo, LVDSSETTINGINFOPTR pLVDSSettingInfo);
static void VIAPowerOffLVDSChannel(VIABIOSInfoPtr pBIOSInfo, LVDSSETTINGINFOPTR pLVDSSettingInfo);
static void EnableIntegratedLVDS(VIABIOSInfoPtr pBIOSInfo, LVDSSETTINGINFOPTR pLVDSSettingInfo);
static void DisableIntegratedLVDS(VIABIOSInfoPtr pBIOSInfo, LVDSSETTINGINFOPTR pLVDSSettingInfo);


/* ++++++ P880 ++++++ */
/*   Panel 1600x1200  */
static struct io_reg P880_LCD_RES_6X4_16X12[]  = {
  {VIACR,CR50,0xFF,0x73},{VIACR,CR55,0x0F,0x08},                        /* IGA2 Horizontal Total            */
  {VIACR,CR53,0xFF,0x73},{VIACR,CR54,0x38,0x00},{VIACR,CR5D,0x40,0x40}, /* IGA2 Horizontal Blank End        */
  {VIACR,CR6D,0xFF,0x5A},{VIACR,CR71,0x08,0x00},                        /* IGA2 Horizontal Total Shadow     */
  {VIACR,CR6E,0xFF,0x5E},                                               /* IGA2 Horizontal Blank End Shadow */
  {VIACR,CR66,0xFF,0xD6},{VIACR,CR67,0x03,0x00},                        /* IGA2 Offset                      */
  {VIASR,SR44,0xFF,0x7D},{VIASR,SR45,0xFF,0x8C},{VIASR,SR46,0xFF,0x02}  /* VCLK                             */
};
#define NUM_TOTAL_P880_LCD_RES_6X4_16X12 ARRAY_SIZE(P880_LCD_RES_6X4_16X12)

static struct io_reg P880_LCD_RES_7X4_16X12[]  = {
  {VIACR,CR50,0xFF,0x67},{VIACR,CR55,0x0F,0x08},                        /* IGA2 Horizontal Total            */
  {VIACR,CR53,0xFF,0x67},{VIACR,CR54,0x38,0x00},{VIACR,CR5D,0x40,0x40}, /* IGA2 Horizontal Blank End        */
  {VIACR,CR6D,0xFF,0x74},{VIACR,CR71,0x08,0x00},                        /* IGA2 Horizontal Total Shadow     */
  {VIACR,CR6E,0xFF,0x78},                                               /* IGA2 Horizontal Blank End Shadow */
  {VIACR,CR66,0xFF,0xF5},{VIACR,CR67,0x03,0x00},                        /* IGA2 Offset                      */
  {VIASR,SR44,0xFF,0x78},{VIASR,SR45,0xFF,0x8C},{VIASR,SR46,0xFF,0x01}  /* VCLK                             */
};
#define NUM_TOTAL_P880_LCD_RES_7X4_16X12 ARRAY_SIZE(P880_LCD_RES_7X4_16X12)

static struct io_reg P880_LCD_RES_8X6_16X12[]  = {
  {VIACR,CR50,0xFF,0x65},{VIACR,CR55,0x0F,0x08},                        /* IGA2 Horizontal Total            */
  {VIACR,CR53,0xFF,0x65},{VIACR,CR54,0x38,0x00},{VIACR,CR5D,0x40,0x40}, /* IGA2 Horizontal Blank End        */
  {VIACR,CR6D,0xFF,0x7F},{VIACR,CR71,0x08,0x00},                        /* IGA2 Horizontal Total Shadow     */
  {VIACR,CR6E,0xFF,0x83},                                               /* IGA2 Horizontal Blank End Shadow */
  {VIACR,CR66,0xFF,0xE1},{VIACR,CR67,0x03,0x00},                        /* IGA2 Offset                      */
  {VIASR,SR44,0xFF,0x6D},{VIASR,SR45,0xFF,0x88},{VIASR,SR46,0xFF,0x03}  /* VCLK                             */
};
#define NUM_TOTAL_P880_LCD_RES_8X6_16X12 ARRAY_SIZE(P880_LCD_RES_8X6_16X12)

static struct io_reg P880_LCD_RES_10X7_16X12[]  = {
  {VIACR,CR50,0xFF,0x65},{VIACR,CR55,0x0F,0x08},                        /* IGA2 Horizontal Total            */
  {VIACR,CR53,0xFF,0x65},{VIACR,CR54,0x38,0x00},{VIACR,CR5D,0x40,0x40}, /* IGA2 Horizontal Blank End        */
  {VIACR,CR6D,0xFF,0xAB},{VIACR,CR71,0x08,0x00},                        /* IGA2 Horizontal Total Shadow     */
  {VIACR,CR6E,0xFF,0xAF},                                               /* IGA2 Horizontal Blank End Shadow */
  {VIACR,CR66,0xFF,0xF0},{VIACR,CR67,0x03,0x00},                        /* IGA2 Offset                      */
  {VIASR,SR44,0xFF,0x92},{VIASR,SR45,0xFF,0x88},{VIASR,SR46,0xFF,0x03}  /* VCLK                             */
};
#define NUM_TOTAL_P880_LCD_RES_10X7_16X12 ARRAY_SIZE(P880_LCD_RES_10X7_16X12)

static struct io_reg P880_LCD_RES_12X10_16X12[]  = {
  {VIACR,CR50,0xFF,0x7D},{VIACR,CR55,0x0F,0x08},                        /* IGA2 Horizontal Total            */
  {VIACR,CR53,0xFF,0x7D},{VIACR,CR54,0x38,0x00},{VIACR,CR5D,0x40,0x40}, /* IGA2 Horizontal Blank End        */
  {VIACR,CR6D,0xFF,0xD0},{VIACR,CR71,0x08,0x00},                        /* IGA2 Horizontal Total Shadow     */
  {VIACR,CR6E,0xFF,0xD4},                                               /* IGA2 Horizontal Blank End Shadow */
  {VIACR,CR66,0xFF,0xFA},{VIACR,CR67,0x03,0x00},                        /* IGA2 Offset                      */
  {VIASR,SR44,0xFF,0xF6},{VIASR,SR45,0xFF,0x88},{VIASR,SR46,0xFF,0x05}  /* VCLK                             */
};
#define NUM_TOTAL_P880_LCD_RES_12X10_16X12 ARRAY_SIZE(P880_LCD_RES_12X10_16X12)

/*   Panel 1400x1050   */
static struct io_reg P880_LCD_RES_6X4_14X10[]    = {                    /* 640x480                          */
  {VIACR,CR50,0xFF,0x9D},{VIACR,CR55,0x0F,0x56},                        /* IGA2 Horizontal Total            */
  {VIACR,CR53,0xFF,0x9D},{VIACR,CR54,0x38,0x75},{VIACR,CR5D,0x40,0x24}, /* IGA2 Horizontal Blank End        */
  {VIACR,CR6D,0xFF,0x5F},{VIACR,CR71,0x08,0x44},                        /* IGA2 Horizontal Total Shadow     */
  {VIACR,CR6E,0xFF,0x63},                                               /* IGA2 Horizontal Blank End Shadow */
  {VIACR,CR66,0xFF,0xB4},{VIACR,CR67,0x03,0x00},                        /* IGA2 Offset                      */
  {VIASR,SR44,0xFF,0xC6},{VIASR,SR45,0xFF,0x8C},{VIASR,SR46,0xFF,0x05}  /* VCLK                             */
};
#define NUM_TOTAL_P880_LCD_RES_6X4_14X10 ARRAY_SIZE(P880_LCD_RES_6X4_14X10)

static struct io_reg P880_LCD_RES_8X6_14X10[]    = {                    /* 800x600                          */
  {VIACR,CR50,0xFF,0x9D},{VIACR,CR55,0x0F,0x56},                        /* IGA2 Horizontal Total            */
  {VIACR,CR53,0xFF,0x9D},{VIACR,CR54,0x38,0x75},{VIACR,CR5D,0x40,0x24}, /* IGA2 Horizontal Blank End        */
  {VIACR,CR6D,0xFF,0x7F},{VIACR,CR71,0x08,0x44},                        /* IGA2 Horizontal Total Shadow     */
  {VIACR,CR6E,0xFF,0x83},                                               /* IGA2 Horizontal Blank End Shadow */
  {VIACR,CR66,0xFF,0xBE},{VIACR,CR67,0x03,0x00},                        /* IGA2 Offset                      */
  {VIASR,SR44,0xFF,0x06},{VIASR,SR45,0xFF,0x8D},{VIASR,SR46,0xFF,0x05}  /* VCLK                             */
};
#define NUM_TOTAL_P880_LCD_RES_8X6_14X10 ARRAY_SIZE(P880_LCD_RES_8X6_14X10)


/* ++++++ K400 ++++++ */
#if 0
/*   Panel 1366x768   */
static struct io_reg K400_LCD_RES_6X4_1366X7[]  = {                      /* 640x400                          */
  {VIACR,CR50,0xFF,0x47},{VIACR,CR55,0x0F,0x35},                        /* IGA2 Horizontal Total            */
  {VIACR,CR53,0xFF,0x47},{VIACR,CR54,0x38,0x2B},{VIACR,CR5D,0x40,0x13}, /* IGA2 Horizontal Blank End        */
  {VIACR,CR6D,0xFF,0x60},{VIACR,CR71,0x08,0x23},                        /* IGA2 Horizontal Total Shadow     */
  {VIACR,CR6E,0xFF,0x64},                                               /* IGA2 Horizontal Blank End Shadow */
  {VIACR,CR66,0xFF,0x8C},{VIACR,CR67,0x03,0x00},                        /* IGA2 Offset                      */
  {VIASR,SR46,0xFF,0x87},{VIASR,SR47,0xFF,0x4C}                         /* VCLK                             */
};
#define NUM_TOTAL_K400_LCD_RES_6X4_1366X7 ARRAY_SIZE(K400_LCD_RES_6X4_1366X7)

static struct io_reg K400_LCD_RES_7X4_1366X7[]  = {
  {VIACR,CR50,0xFF,0x3B},{VIACR,CR55,0x0F,0x35},                        /* IGA2 Horizontal Total            */
  {VIACR,CR53,0xFF,0x3B},{VIACR,CR54,0x38,0x2B},{VIACR,CR5D,0x40,0x13}, /* IGA2 Horizontal Blank End        */
  {VIACR,CR6D,0xFF,0x71},{VIACR,CR71,0x08,0x23},                        /* IGA2 Horizontal Total Shadow     */
  {VIACR,CR6E,0xFF,0x75},                                               /* IGA2 Horizontal Blank End Shadow */
  {VIACR,CR66,0xFF,0x96},{VIACR,CR67,0x03,0x00},                        /* IGA2 Offset                      */
  {VIASR,SR46,0xFF,0x05},{VIASR,SR47,0xFF,0x10}                         /* VCLK                             */
};
#define NUM_TOTAL_K400_LCD_RES_7X4_1366X7 ARRAY_SIZE(K400_LCD_RES_7X4_1366X7)

static struct io_reg K400_LCD_RES_8X6_1366X7[]  = {                      /* 800x600                          */
  {VIACR,CR50,0xFF,0x37},{VIACR,CR55,0x0F,0x35},                        /* IGA2 Horizontal Total            */
  {VIACR,CR53,0xFF,0x37},{VIACR,CR54,0x38,0x2B},{VIACR,CR5D,0x40,0x13}, /* IGA2 Horizontal Blank End        */
  {VIACR,CR6D,0xFF,0x7E},{VIACR,CR71,0x08,0x23},                        /* IGA2 Horizontal Total Shadow     */
  {VIACR,CR6E,0xFF,0x82},                                               /* IGA2 Horizontal Blank End Shadow */
  {VIACR,CR66,0xFF,0x8C},{VIACR,CR67,0x03,0x00},                        /* IGA2 Offset                      */
  {VIASR,SR46,0xFF,0x84},{VIASR,SR47,0xFF,0xB9}                         /* VCLK                             */
};
#define NUM_TOTAL_K400_LCD_RES_8X6_1366X7 ARRAY_SIZE(K400_LCD_RES_8X6_1366X7)

static struct io_reg K400_LCD_RES_10X7_1366X7[]  = {                     /* 1024x768                         */
  {VIACR,CR50,0xFF,0x9D},{VIACR,CR55,0x0F,0x56},                        /* IGA2 Horizontal Total            */
  {VIACR,CR53,0xFF,0x9D},{VIACR,CR54,0x38,0x75},{VIACR,CR5D,0x40,0x24}, /* IGA2 Horizontal Blank End        */
  {VIACR,CR6D,0xFF,0xA3},{VIACR,CR71,0x08,0x44},                        /* IGA2 Horizontal Total Shadow     */
  {VIACR,CR6E,0xFF,0xA7},                                               /* IGA2 Horizontal Blank End Shadow */
  {VIACR,CR66,0xFF,0xC3},{VIACR,CR67,0x03,0x04},                        /* IGA2 Offset                      */
  {VIASR,SR46,0xFF,0x05},{VIASR,SR47,0xFF,0x1E}                         /* VCLK                             */
};
#define NUM_TOTAL_K400_LCD_RES_10X7_1366X7 ARRAY_SIZE(K400_LCD_RES_10X7_1366X7)

static struct io_reg K400_LCD_RES_12X10_1366X7[]  = {                    /* 1280x768, 1280x960, 1280x1024    */
  {VIACR,CR50,0xFF,0x97},{VIACR,CR55,0x0F,0x56},                        /* IGA2 Horizontal Total            */
  {VIACR,CR53,0xFF,0x97},{VIACR,CR54,0x38,0x75},{VIACR,CR5D,0x40,0x24}, /* IGA2 Horizontal Blank End        */
  {VIACR,CR6D,0xFF,0xCE},{VIACR,CR71,0x08,0x44},                        /* IGA2 Horizontal Total Shadow     */
  {VIACR,CR6E,0xFF,0xD2},                                               /* IGA2 Horizontal Blank End Shadow */
  {VIACR,CR66,0xFF,0xC9},{VIACR,CR67,0x03,0x04},                        /* IGA2 Offset                      */
  {VIASR,SR46,0xFF,0x84},{VIASR,SR47,0xFF,0x79}                         /* VCLK                             */
};
#define NUM_TOTAL_K400_LCD_RES_12X10_1366X7 ARRAY_SIZE(K400_LCD_RES_12X10_1366X7)
#endif

/* ++++++ K400 ++++++ */
/*   Panel 1600x1200  */
static struct io_reg K400_LCD_RES_6X4_16X12[]  = {
  {VIACR,CR50,0xFF,0x73},{VIACR,CR55,0x0F,0x08},                        /* IGA2 Horizontal Total            */
  {VIACR,CR53,0xFF,0x73},{VIACR,CR54,0x38,0x00},{VIACR,CR5D,0x40,0x40}, /* IGA2 Horizontal Blank End        */
  {VIACR,CR6D,0xFF,0x5A},{VIACR,CR71,0x08,0x00},                        /* IGA2 Horizontal Total Shadow     */
  {VIACR,CR6E,0xFF,0x5E},                                               /* IGA2 Horizontal Blank End Shadow */
  {VIACR,CR66,0xFF,0xDA},{VIACR,CR67,0x03,0x00},                        /* IGA2 Offset                      */
  {VIASR,SR46,0xFF,0xC4},{VIASR,SR47,0xFF,0x7F}                         /* VCLK                             */
};
#define NUM_TOTAL_K400_LCD_RES_6X4_16X12 ARRAY_SIZE(K400_LCD_RES_6X4_16X12)

static struct io_reg K400_LCD_RES_7X4_16X12[]  = {
  {VIACR,CR50,0xFF,0x67},{VIACR,CR55,0x0F,0x08},                        /* IGA2 Horizontal Total            */
  {VIACR,CR53,0xFF,0x67},{VIACR,CR54,0x38,0x00},{VIACR,CR5D,0x40,0x40}, /* IGA2 Horizontal Blank End        */
  {VIACR,CR6D,0xFF,0x74},{VIACR,CR71,0x08,0x00},                        /* IGA2 Horizontal Total Shadow     */
  {VIACR,CR6E,0xFF,0x78},                                               /* IGA2 Horizontal Blank End Shadow */
  {VIACR,CR66,0xFF,0xF5},{VIACR,CR67,0x03,0x00},                        /* IGA2 Offset                      */
  {VIASR,SR46,0xFF,0x46},{VIASR,SR47,0xFF,0x3D}                         /* VCLK                             */
};
#define NUM_TOTAL_K400_LCD_RES_7X4_16X12 ARRAY_SIZE(K400_LCD_RES_7X4_16X12)

static struct io_reg K400_LCD_RES_8X6_16X12[]  = {
  {VIACR,CR50,0xFF,0x65},{VIACR,CR55,0x0F,0x08},                        /* IGA2 Horizontal Total            */
  {VIACR,CR53,0xFF,0x65},{VIACR,CR54,0x38,0x00},{VIACR,CR5D,0x40,0x40}, /* IGA2 Horizontal Blank End        */
  {VIACR,CR6D,0xFF,0x7F},{VIACR,CR71,0x08,0x00},                        /* IGA2 Horizontal Total Shadow     */
  {VIACR,CR6E,0xFF,0x83},                                               /* IGA2 Horizontal Blank End Shadow */
  {VIACR,CR66,0xFF,0xE1},{VIACR,CR67,0x03,0x00},                        /* IGA2 Offset                      */
  {VIASR,SR46,0xFF,0x85},{VIASR,SR47,0xFF,0x6F}                         /* VCLK                             */
};
#define NUM_TOTAL_K400_LCD_RES_8X6_16X12 ARRAY_SIZE(K400_LCD_RES_8X6_16X12)

static struct io_reg K400_LCD_RES_10X7_16X12[]  = {
  {VIACR,CR50,0xFF,0x65},{VIACR,CR55,0x0F,0x08},                        /* IGA2 Horizontal Total            */
  {VIACR,CR53,0xFF,0x65},{VIACR,CR54,0x38,0x00},{VIACR,CR5D,0x40,0x40}, /* IGA2 Horizontal Blank End        */
  {VIACR,CR6D,0xFF,0xAB},{VIACR,CR71,0x08,0x00},                        /* IGA2 Horizontal Total Shadow     */
  {VIACR,CR6E,0xFF,0xAF},                                               /* IGA2 Horizontal Blank End Shadow */
  {VIACR,CR66,0xFF,0xF0},{VIACR,CR67,0x03,0x00},                        /* IGA2 Offset                      */
  {VIASR,SR46,0xFF,0x45},{VIASR,SR47,0xFF,0x4A}                         /* VCLK                             */
};
#define NUM_TOTAL_K400_LCD_RES_10X7_16X12 ARRAY_SIZE(K400_LCD_RES_10X7_16X12)

static struct io_reg K400_LCD_RES_12X10_16X12[]  = {
  {VIACR,CR50,0xFF,0x7D},{VIACR,CR55,0x0F,0x08},                        /* IGA2 Horizontal Total            */
  {VIACR,CR53,0xFF,0x7D},{VIACR,CR54,0x38,0x00},{VIACR,CR5D,0x40,0x40}, /* IGA2 Horizontal Blank End        */
  {VIACR,CR6D,0xFF,0xD0},{VIACR,CR71,0x08,0x00},                        /* IGA2 Horizontal Total Shadow     */
  {VIACR,CR6E,0xFF,0xD4},                                               /* IGA2 Horizontal Blank End Shadow */
  {VIACR,CR66,0xFF,0xFA},{VIACR,CR67,0x03,0x00},                        /* IGA2 Offset                      */
  {VIASR,SR46,0xFF,0x47},{VIASR,SR47,0xFF,0x7C}                         /* VCLK                             */
};
#define NUM_TOTAL_K400_LCD_RES_12X10_16X12 ARRAY_SIZE(K400_LCD_RES_12X10_16X12)

/* ++++++ KM400 ++++++ */
/*   Panel 1400x1050   */
static struct io_reg K400_LCD_RES_6X4_14X10[]  = {                      /* 640x400                          */
  {VIACR,CR50,0xFF,0x9D},{VIACR,CR55,0x0F,0x56},                        /* IGA2 Horizontal Total            */
  {VIACR,CR53,0xFF,0x9D},{VIACR,CR54,0x38,0x75},{VIACR,CR5D,0x40,0x24}, /* IGA2 Horizontal Blank End        */
  {VIACR,CR6D,0xFF,0x5F},{VIACR,CR71,0x08,0x44},                        /* IGA2 Horizontal Total Shadow     */
  {VIACR,CR6E,0xFF,0x63},                                               /* IGA2 Horizontal Blank End Shadow */
  {VIACR,CR66,0xFF,0xB4},{VIACR,CR67,0x03,0x00},                        /* IGA2 Offset                      */
  {VIASR,SR46,0xFF,0x07},{VIASR,SR47,0xFF,0x19} 						/* VCLK                             */
};
#define NUM_TOTAL_K400_LCD_RES_6X4_14X10 ARRAY_SIZE(K400_LCD_RES_6X4_14X10)

static struct io_reg K400_LCD_RES_8X6_14X10[]  = {                      /* 800x600                          */
  {VIACR,CR50,0xFF,0x9D},{VIACR,CR55,0x0F,0x56},                        /* IGA2 Horizontal Total            */
  {VIACR,CR53,0xFF,0x9D},{VIACR,CR54,0x38,0x75},{VIACR,CR5D,0x40,0x24}, /* IGA2 Horizontal Blank End        */
  {VIACR,CR6D,0xFF,0x7F},{VIACR,CR71,0x08,0x44},                        /* IGA2 Horizontal Total Shadow     */
  {VIACR,CR6E,0xFF,0x83},                                               /* IGA2 Horizontal Blank End Shadow */
  {VIACR,CR66,0xFF,0xBE},{VIACR,CR67,0x03,0x00},                        /* IGA2 Offset                      */
  {VIASR,SR46,0xFF,0x07},{VIASR,SR47,0xFF,0x21}                         /* VCLK                             */
};
#define NUM_TOTAL_K400_LCD_RES_8X6_14X10 ARRAY_SIZE(K400_LCD_RES_8X6_14X10)

static struct io_reg K400_LCD_RES_10X7_14X10[]  = {                     /* 1024x768                         */
  {VIACR,CR50,0xFF,0x9D},{VIACR,CR55,0x0F,0x56},                        /* IGA2 Horizontal Total            */
  {VIACR,CR53,0xFF,0x9D},{VIACR,CR54,0x38,0x75},{VIACR,CR5D,0x40,0x24}, /* IGA2 Horizontal Blank End        */
  {VIACR,CR6D,0xFF,0xA3},{VIACR,CR71,0x08,0x44},                        /* IGA2 Horizontal Total Shadow     */
  {VIACR,CR6E,0xFF,0xA7},                                               /* IGA2 Horizontal Blank End Shadow */
  {VIACR,CR66,0xFF,0xC3},{VIACR,CR67,0x03,0x04},                        /* IGA2 Offset                      */
  {VIASR,SR46,0xFF,0x05},{VIASR,SR47,0xFF,0x1E}                         /* VCLK                             */
};
#define NUM_TOTAL_K400_LCD_RES_10X7_14X10 ARRAY_SIZE(K400_LCD_RES_10X7_14X10)

static struct io_reg K400_LCD_RES_12X10_14X10[]  = {                    /* 1280x768, 1280x960, 1280x1024    */
  {VIACR,CR50,0xFF,0x97},{VIACR,CR55,0x0F,0x56},                        /* IGA2 Horizontal Total            */
  {VIACR,CR53,0xFF,0x97},{VIACR,CR54,0x38,0x75},{VIACR,CR5D,0x40,0x24}, /* IGA2 Horizontal Blank End        */
  {VIACR,CR6D,0xFF,0xCE},{VIACR,CR71,0x08,0x44},                        /* IGA2 Horizontal Total Shadow     */
  {VIACR,CR6E,0xFF,0xD2},                                               /* IGA2 Horizontal Blank End Shadow */
  {VIACR,CR66,0xFF,0xC9},{VIACR,CR67,0x03,0x04},                        /* IGA2 Offset                      */
  {VIASR,SR46,0xFF,0x84},{VIASR,SR47,0xFF,0x79}                         /* VCLK                             */
};
#define NUM_TOTAL_K400_LCD_RES_12X10_14X10 ARRAY_SIZE(K400_LCD_RES_12X10_14X10)

/* ++++++ K400 ++++++ */
/*   Panel 1280x1024  */
static struct io_reg K400_LCD_RES_6X4_12X10[]  = {
  {VIACR,CR50,0xFF,0x9D},{VIACR,CR55,0x0F,0x46},                        /* IGA2 Horizontal Total            */
  {VIACR,CR53,0xFF,0x9D},{VIACR,CR54,0x38,0x74},{VIACR,CR5D,0x40,0x1C}, /* IGA2 Horizontal Blank End        */
  {VIACR,CR6D,0xFF,0x5F},{VIACR,CR71,0x08,0x34},                        /* IGA2 Horizontal Total Shadow     */
  {VIACR,CR6E,0xFF,0x63},                                               /* IGA2 Horizontal Blank End Shadow */
  {VIACR,CR66,0xFF,0xAA},{VIACR,CR67,0x03,0x00},                        /* IGA2 Offset                      */
  {VIASR,SR46,0xFF,0x07},{VIASR,SR47,0xFF,0x19}                         /* VCLK                             */
};
#define NUM_TOTAL_K400_LCD_RES_6X4_12X10 ARRAY_SIZE(K400_LCD_RES_6X4_12X10)

static struct io_reg K400_LCD_RES_7X4_12X10[]  = {
  {VIACR,CR50,0xFF,0x9D},{VIACR,CR55,0x0F,0x46},                        /* IGA2 Horizontal Total            */
  {VIACR,CR53,0xFF,0x9D},{VIACR,CR54,0x38,0x74},{VIACR,CR5D,0x40,0x1C}, /* IGA2 Horizontal Blank End        */
  {VIACR,CR6D,0xFF,0x68},{VIACR,CR71,0x08,0x34},                        /* IGA2 Horizontal Total Shadow     */
  {VIACR,CR6E,0xFF,0x6C},                                               /* IGA2 Horizontal Blank End Shadow */
  {VIACR,CR66,0xFF,0xA8},{VIACR,CR67,0x03,0x00},                        /* IGA2 Offset                      */
  {VIASR,SR46,0xFF,0x87},{VIASR,SR47,0xFF,0xED}                         /* VCLK                             */
};
#define NUM_TOTAL_K400_LCD_RES_7X4_12X10 ARRAY_SIZE(K400_LCD_RES_7X4_12X10)

static struct io_reg K400_LCD_RES_8X6_12X10[]  = {
  {VIACR,CR50,0xFF,0x9D},{VIACR,CR55,0x0F,0x46},                        /* IGA2 Horizontal Total            */
  {VIACR,CR53,0xFF,0x9D},{VIACR,CR54,0x38,0x74},{VIACR,CR5D,0x40,0x1C}, /* IGA2 Horizontal Blank End        */
  {VIACR,CR6D,0xFF,0x7F},{VIACR,CR71,0x08,0x34},                        /* IGA2 Horizontal Total Shadow     */
  {VIACR,CR6E,0xFF,0x83},                                               /* IGA2 Horizontal Blank End Shadow */
  {VIACR,CR66,0xFF,0xBE},{VIACR,CR67,0x03,0x00},                        /* IGA2 Offset                      */
  {VIASR,SR46,0xFF,0x07},{VIASR,SR47,0xFF,0x21}                         /* VCLK                             */
};
#define NUM_TOTAL_K400_LCD_RES_8X6_12X10 ARRAY_SIZE(K400_LCD_RES_8X6_12X10)

static struct io_reg K400_LCD_RES_10X7_12X10[]  = {
  {VIACR,CR50,0xFF,0x9D},{VIACR,CR55,0x0F,0x46},                        /* IGA2 Horizontal Total            */
  {VIACR,CR53,0xFF,0x9D},{VIACR,CR54,0x38,0x74},{VIACR,CR5D,0x40,0x1C}, /* IGA2 Horizontal Blank End        */
  {VIACR,CR6D,0xFF,0xA3},{VIACR,CR71,0x08,0x34},                        /* IGA2 Horizontal Total Shadow     */
  {VIACR,CR6E,0xFF,0xA7},                                               /* IGA2 Horizontal Blank End Shadow */
  {VIACR,CR66,0xFF,0xBE},{VIACR,CR67,0x03,0x04},                        /* IGA2 Offset                      */
  {VIASR,SR46,0xFF,0x05},{VIASR,SR47,0xFF,0x1E}                         /* VCLK                             */
};
#define NUM_TOTAL_K400_LCD_RES_10X7_12X10 ARRAY_SIZE(K400_LCD_RES_10X7_12X10)


/* ++++++ K400 ++++++ */
/*   Panel 1024x768   */
static struct io_reg K400_LCD_RES_6X4_10X7[]  = {
  {VIACR,CR50,0xFF,0x47},{VIACR,CR55,0x0F,0x35},                        /* IGA2 Horizontal Total            */
  {VIACR,CR53,0xFF,0x47},{VIACR,CR54,0x38,0x2B},{VIACR,CR5D,0x40,0x13}, /* IGA2 Horizontal Blank End        */
  {VIACR,CR6D,0xFF,0x60},{VIACR,CR71,0x08,0x23},                        /* IGA2 Horizontal Total Shadow     */
  {VIACR,CR6E,0xFF,0x64},                                               /* IGA2 Horizontal Blank End Shadow */
  {VIACR,CR66,0xFF,0x8C},{VIACR,CR67,0x03,0x00},                        /* IGA2 Offset                      */
  {VIASR,SR46,0xFF,0x87},{VIASR,SR47,0xFF,0x4C}                         /* VCLK                             */
};

#define NUM_TOTAL_K400_LCD_RES_6X4_10X7 ARRAY_SIZE(K400_LCD_RES_6X4_10X7)

static struct io_reg K400_LCD_RES_7X4_10X7[]  = {
  {VIACR,CR50,0xFF,0x3B},{VIACR,CR55,0x0F,0x35},                        /* IGA2 Horizontal Total            */
  {VIACR,CR53,0xFF,0x3B},{VIACR,CR54,0x38,0x2B},{VIACR,CR5D,0x40,0x13}, /* IGA2 Horizontal Blank End        */
  {VIACR,CR6D,0xFF,0x71},{VIACR,CR71,0x08,0x23},                        /* IGA2 Horizontal Total Shadow     */
  {VIACR,CR6E,0xFF,0x75},                                               /* IGA2 Horizontal Blank End Shadow */
  {VIACR,CR66,0xFF,0x96},{VIACR,CR67,0x03,0x00},                        /* IGA2 Offset                      */
  {VIASR,SR46,0xFF,0x05},{VIASR,SR47,0xFF,0x10}                         /* VCLK                             */
};

#define NUM_TOTAL_K400_LCD_RES_7X4_10X7 ARRAY_SIZE(K400_LCD_RES_7X4_10X7)

static struct io_reg K400_LCD_RES_8X6_10X7[]  = {
  {VIACR,CR50,0xFF,0x37},{VIACR,CR55,0x0F,0x35},                        /* IGA2 Horizontal Total            */
  {VIACR,CR53,0xFF,0x37},{VIACR,CR54,0x38,0x2B},{VIACR,CR5D,0x40,0x13}, /* IGA2 Horizontal Blank End        */
  {VIACR,CR6D,0xFF,0x7E},{VIACR,CR71,0x08,0x23},                        /* IGA2 Horizontal Total Shadow     */
  {VIACR,CR6E,0xFF,0x82},                                               /* IGA2 Horizontal Blank End Shadow */
  {VIACR,CR66,0xFF,0x8C},{VIACR,CR67,0x03,0x00},                        /* IGA2 Offset                      */
  {VIASR,SR46,0xFF,0x84},{VIASR,SR47,0xFF,0xB9}                         /* VCLK                             */
};
#define NUM_TOTAL_K400_LCD_RES_8X6_10X7 ARRAY_SIZE(K400_LCD_RES_8X6_10X7)

/* ++++++ K400 ++++++ */
/*   Panel 800x600    */
static struct io_reg K400_LCD_RES_6X4_8X6[]  = {
  {VIACR,CR50,0xFF,0x1A},{VIACR,CR55,0x0F,0x34},                        /* IGA2 Horizontal Total            */
  {VIACR,CR53,0xFF,0x1A},{VIACR,CR54,0x38,0xE3},{VIACR,CR5D,0x40,0x12}, /* IGA2 Horizontal Blank End        */
  {VIACR,CR6D,0xFF,0x5F},{VIACR,CR71,0x08,0x22},                        /* IGA2 Horizontal Total Shadow     */
  {VIACR,CR6E,0xFF,0x63},                                               /* IGA2 Horizontal Blank End Shadow */
  {VIACR,CR66,0xFF,0x6E},{VIACR,CR67,0x03,0x00},                        /* IGA2 Offset                      */
  {VIASR,SR46,0xFF,0x86},{VIASR,SR47,0xFF,0xB3}                         /* VCLK                             */
};
#define NUM_TOTAL_K400_LCD_RES_6X4_8X6 ARRAY_SIZE(K400_LCD_RES_6X4_8X6)

static struct io_reg K400_LCD_RES_7X4_8X6[]  = {
  {VIACR,CR50,0xFF,0x1F},{VIACR,CR55,0x0F,0x34},                        /* IGA2 Horizontal Total            */
  {VIACR,CR53,0xFF,0x1F},{VIACR,CR54,0x38,0xE3},{VIACR,CR5D,0x40,0x12}, /* IGA2 Horizontal Blank End        */
  {VIACR,CR6D,0xFF,0x7F},{VIACR,CR71,0x08,0x22},                        /* IGA2 Horizontal Total Shadow     */
  {VIACR,CR6E,0xFF,0x83},                                               /* IGA2 Horizontal Blank End Shadow */
  {VIACR,CR66,0xFF,0x78},{VIACR,CR67,0x03,0x00},                        /* IGA2 Offset                      */
  {VIASR,SR46,0xFF,0xC4},{VIASR,SR47,0xFF,0x59}                         /* VCLK                             */
};
#define NUM_TOTAL_K400_LCD_RES_7X4_8X6 ARRAY_SIZE(K400_LCD_RES_7X4_8X6)

static struct _lcd_scaling_factor lcd_scaling_factor = {
    /* LCD Horizontal Scaling Factor Register */
    {LCD_HOR_SCALING_FACTOR_REG_NUM, {{CR9F,0,1}, {CR77,0,7}, {CR79,4,5}}},
    /* LCD Vertical Scaling Factor Register */
    {LCD_VER_SCALING_FACTOR_REG_NUM, {{CR79,3,3}, {CR78,0,7}, {CR79,6,7}}}
};
static struct _lcd_scaling_factor lcd_scaling_factor_CLE = {
    /* LCD Horizontal Scaling Factor Register */
    {LCD_HOR_SCALING_FACTOR_REG_NUM_CLE, {{CR77,0,7}, {CR79,4,5}}},
    /* LCD Vertical Scaling Factor Register */
    {LCD_VER_SCALING_FACTOR_REG_NUM_CLE, {{CR78,0,7}, {CR79,6,7}}}
};


void VIALCDSetMode(VIABIOSInfoPtr pBIOSInfo, LVDSSETTINGINFOPTR pLVDSSettingInfo)
{
    struct io_register      *reg = NULL;
    struct display_timing   mode_crt_reg, panel_crt_reg;
    int                     load_reg_num=0;
    int                     reg_value=0;
    int                     set_hres, set_vres, h_active, v_active, h_size, v_size;
    int                     panel_hres, panel_vres;
    struct crt_mode_table   *mode_crt_table, *panel_crt_table;
    struct VideoModeTable   *VideoMode;
    CARD32                  pll_D_N;
    int                     offset;
    int                     set_iga = pLVDSSettingInfo->IGAPath;

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIALCDSetMode!! ActiveDeviceID=0x%x, IGA=%d!!\n", pBIOSInfo->ActiveDevice, pLVDSSettingInfo->IGAPath));

    VideoMode = &CLE266Modes[SearchModeSetting(pLVDSSettingInfo->ModeIndex)];           /* Get mode table */
    mode_crt_table = VideoMode->crtc;
    mode_crt_reg = mode_crt_table->crtc;

    VideoMode = &CLE266Modes[SearchModeSetting(VIAGetModeIndexByPanelSize(pLVDSSettingInfo->PanelSizeID))];    /* Get panel table */
    panel_crt_table = VideoMode->crtc;
    panel_crt_reg = panel_crt_table->crtc;

    /* RainLin [2006.06.08]: The meaning of resolution to set here should not be view size but */
    /* real desktop size. */
    set_hres = pBIOSInfo->ActualDesktopSizeX;
    set_vres = pBIOSInfo->ActualDesktopSizeY;
    h_active = pLVDSSettingInfo->HActive;
    v_active = pLVDSSettingInfo->VActive;
    panel_hres = pLVDSSettingInfo->PanelSizeX;
    panel_vres = pLVDSSettingInfo->PanelSizeY;

    if(pLVDSSettingInfo->IGAPath == IGA1)
    {
        pLVDSSettingInfo->Center = TRUE;
    }

    if (pLVDSSettingInfo->ChipID == VIA_VT1636)
    {
        VIAInitLVDS_VT1636(pBIOSInfo, pLVDSSettingInfo);
    }
    
    /*cliff add while LCD in Expansion,for mergedfb*/
    if(pBIOSInfo->MergedFB)
    {
        fill_crtc_timing(pBIOSInfo,mode_crt_table, VideoMode->mode_array, (pBIOSInfo->bitsPerPixel >> 3), pLVDSSettingInfo->IGAPath);
        pLVDSSettingInfo->VClk = mode_crt_table->clk;
    }
    else
    {
        fill_crtc_timing(pBIOSInfo,panel_crt_table, VideoMode->mode_array, (pBIOSInfo->bitsPerPixel >> 3), pLVDSSettingInfo->IGAPath);
        pLVDSSettingInfo->VClk = panel_crt_table->clk;
    }

    if(set_iga == IGA1)
    { 
        unlock_crt();
        write_reg(CR09, VIACR, 0x00); /*initial CR09=0*/
        write_reg_mask(CR11, VIACR, 0x00, BIT4+BIT5+BIT6);
        write_reg_mask(CR17, VIACR, 0x00, BIT7);

        load_crtc_timing(pBIOSInfo, lcd_centering_timging(mode_crt_reg, panel_crt_reg), IGA1);
        write_reg_mask(CR79, VIACR, 0x00, BIT0+BIT1+BIT2);        /* LCD scaling disabled */
        pBIOSInfo->scaleY = FALSE;
        lock_crt();  
    }
    else
    {
        /* Expansion */
        if (pLVDSSettingInfo->Center == FALSE)
        {
            pLVDSSettingInfo->PanelSizeY = panel_crt_reg.ver_addr;
            pBIOSInfo->resY = mode_crt_reg.ver_addr;

            /* Mode 640x480 has border, but LCD didn't have border. */
            /* So we would delete border. */    
            if((pBIOSInfo->EPIA_DVI && (pBIOSInfo->resMode2 == M640x480) && (pBIOSInfo->FoundRefresh == 60)) ||
               ((pBIOSInfo->ActiveDevice & (VIA_DEVICE_LCD | VIA_DEVICE_DFP)) && 
               (pBIOSInfo->resMode2 == M640x480) && (pBIOSInfo->FoundRefresh == 60)))
            {
                /* The border is 8 pixels. */
                panel_crt_reg.hor_blank_start = panel_crt_reg.hor_blank_start - 8;

                /* Blanking time should add left and right borders. */
                panel_crt_reg.hor_blank_end = panel_crt_reg.hor_blank_end + 16;
            }

            /* expansion timing IGA2 always loaded panel set timing */
            load_crtc_timing(pBIOSInfo, panel_crt_reg, IGA2);

            if ((h_active < panel_hres ) || (v_active < panel_vres) ||
                (set_hres < panel_hres ) || (set_vres < panel_vres))
            {
                if (h_active < set_hres)
                {
                    h_size = h_active;
                }
                else
                {
                    h_size = set_hres;
                }

                if (v_active < set_vres)
                {
                    v_size = v_active;
                }
                else
                {
                    v_size = set_vres;
                }
                
                VIALoadLCDScaling(pBIOSInfo, h_size, v_size, pLVDSSettingInfo->PanelSizeX, pLVDSSettingInfo->PanelSizeY);
                pBIOSInfo->scaleY = TRUE;
            }
                
        }
        else /* Centering */
        {
            /* Mode 640x480 has border, but LCD didn't have border. */
            /* So we would delete border. */    
            if((pBIOSInfo->EPIA_DVI && (pBIOSInfo->resMode2 == M640x480) && (pBIOSInfo->FoundRefresh == 60)) ||
               ((pBIOSInfo->ActiveDevice & (VIA_DEVICE_LCD | VIA_DEVICE_DFP)) && 
               (pBIOSInfo->resMode2 == M640x480) && (pBIOSInfo->FoundRefresh == 60)))
            {
                /* The border is 8 pixels. */
                panel_crt_reg.hor_blank_start = panel_crt_reg.hor_blank_start - 8;

                /* Blanking time should add left and right borders. */
                panel_crt_reg.hor_blank_end = panel_crt_reg.hor_blank_end + 16;
            }

            /* centering timing IGA2 always loaded panel and mode releative timing */
            load_crtc_timing(pBIOSInfo, lcd_centering_timging(mode_crt_reg, panel_crt_reg), IGA2);
            write_reg_mask(CR79, VIACR, 0x00, BIT0+BIT1+BIT2);        /* LCD scaling disabled */
            pBIOSInfo->scaleY = FALSE;
        }
    }

    if (set_iga== IGA1_IGA2)
    {
        load_crtc_shadow_timing(pBIOSInfo, mode_crt_reg, panel_crt_reg);       /* Fill shadow registers */

        switch(pLVDSSettingInfo->PanelSizeID)
        {
            case VIA_PANEL6X4:
                offset = 80;
                break;
            case VIA_PANEL8X4:
            case VIA_PANEL8X6:
                offset = 110;
                break;
            case VIA_PANEL10X7:
                offset = 150;
                break;
            case VIA_PANEL12X7:
            case VIA_PANEL12X8:
            case VIA_PANEL12X10:
            case VIA_PANEL14X10:
                offset = 190;
                break;
            case VIA_PANEL16X12:
                offset = 250;
                break;
            case VIA_PANEL1366X7:
            case VIA_PANEL1360X7:
                offset = 212;                             
                break;
            default:
                offset = 140;
                break;
        }

        /* Offset for simultaneous */
        reg_value = offset;
        load_reg_num = offset_reg.iga2_offset_reg.reg_num;
        reg = offset_reg.iga2_offset_reg.reg;
        load_reg(reg_value, load_reg_num, reg, VIACR);

        load_fetch_count_reg(mode_crt_reg.hor_addr, 4, IGA2);           /* Fetch count for simultaneous*/
    }
    else  /* IGA2 for SAMM */
    {
        /*add by cliff for mergedfb */ 
        if(pBIOSInfo->MergedFB)
        {
            if((pBIOSInfo->Scrn2Position == viaRightOf)||(pBIOSInfo->Scrn2Position == viaLeftOf))
            {
                load_offset_reg((mode_crt_reg.hor_addr*2),(pBIOSInfo->bitsPerPixel>>3),set_iga);
            }
            
            if((pBIOSInfo->Scrn2Position == viaBelow)||(pBIOSInfo->Scrn2Position == viaAbove))
            {
                load_offset_reg(mode_crt_reg.hor_addr,(pBIOSInfo->bitsPerPixel>>3),set_iga);
            }
        }
        else
        {
            load_offset_reg(mode_crt_reg.hor_addr, (pBIOSInfo->bitsPerPixel>>3), set_iga);
        }
        
        load_fetch_count_reg(mode_crt_reg.hor_addr, (pBIOSInfo->bitsPerPixel>>3),set_iga);    /* Fetch count for IGA2 only */
        load_FIFO_reg(pBIOSInfo, mode_crt_reg.hor_addr, mode_crt_reg.ver_addr, set_iga);
        set_color_depth(pBIOSInfo, pBIOSInfo->bitsPerPixel>>3, set_iga);
    }

    VIA_fill_lcd_format(pLVDSSettingInfo->IsDualEdge, pLVDSSettingInfo->IsDithering);

    pll_D_N = get_clk_value(pBIOSInfo, panel_crt_table[0].clk);
    DEBUG(ErrorF("PLL=0x%x\n", pll_D_N));
    SetVCLK(pBIOSInfo, pll_D_N, set_iga);

    VIASetOutputPath(pBIOSInfo, VIA_DEVICE_LCD, set_iga, pLVDSSettingInfo->DIPort);

    /* Adjust DPA Setting for clock skew problem: */
    VIALCDPatchSkew(pBIOSInfo, pLVDSSettingInfo);

    /* Set display channel. */
    if (pLVDSSettingInfo->ChipID == VIA_INTEGRATED_LVDS)
    {
        VIASetDisplayChannel(pBIOSInfo);
    }

    /* set CRT to IGA2 path */
    if ((set_iga == IGA1_IGA2)||(set_iga == IGA1))
    {
        write_reg_mask(SR16, VIASR, 0x40, BIT6);
    }

    /* If K8M800, enable LCD Prefetch Mode. */
    if ((pBIOSInfo->Chipset == VIA_K8M800)||(pBIOSInfo->Chipset == VIA_K8M890) )
    {
        write_reg_mask(CR6A, VIACR, 0x01, BIT0);
    }

    VIALoadLCDPatchRegs(pBIOSInfo, set_iga); 
}

void VIALoadLCDScaling(VIABIOSInfoPtr pBIOSInfo, int set_hres, int set_vres, int panel_hres, int panel_vres)
{
    struct io_register  *reg = NULL;
    int                 reg_value;
    int                 load_reg_num;

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, 
        "VIALoadLCDScaling: set_res=%dx%d, panel size=%dx%d\n", set_hres, set_vres, panel_hres, panel_vres));

    /* LCD Scaling Enable */
    write_reg_mask(CR79, VIACR, 0x07, BIT0+BIT1+BIT2);    

    /* STChen[2006.6.29]: P4M900_A0's scaling register has a little problem on reading it.
     * So we use another method to access it.
     */
    if (pBIOSInfo->Chipset == VIA_P4M900)
    {
        VIALoadScalingFactorForP4M900(pBIOSInfo, set_hres, set_vres, panel_hres, panel_vres);
        return;
    }

    /* Check if expansion for horizontal */
    if(set_hres < panel_hres)
    {
        /* Load Horizontal Scaling Factor */
        if (pBIOSInfo->Chipset >= VIA_K8M800)
        {
            /* For VIA_K8M800 or later chipsets. */
            reg_value = K800_LCD_HOR_SCF_FORMULA(set_hres, panel_hres);
            write_reg_mask(CRA2, VIACR, 0xC0, BIT7+BIT6);       /* Horizontal scaling enabled */
            load_reg_num = lcd_scaling_factor.lcd_hor_scaling_factor.reg_num;
            reg = lcd_scaling_factor.lcd_hor_scaling_factor.reg;
            load_reg(reg_value, load_reg_num, reg, VIACR);
        }
        else
        {
            reg_value = CLE266_LCD_HOR_SCF_FORMULA(set_hres, panel_hres);
            load_reg_num = lcd_scaling_factor_CLE.lcd_hor_scaling_factor.reg_num;
            reg = lcd_scaling_factor_CLE.lcd_hor_scaling_factor.reg;
            load_reg(reg_value, load_reg_num, reg, VIACR);                 
        }

        DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "Horizontal Scaling value = %d\n", reg_value));
    }
    else
    {
        write_reg_mask(CRA2, VIACR, 0x00, BIT7);      /* Horizontal scaling disabled */
    }

    
    /* Check if expansion for vertical */
    if(set_vres < panel_vres)
    {
        /* Load Vertical Scaling Factor */
        if (pBIOSInfo->Chipset >= VIA_K8M800)
        {
            /* For VIA_K8M800 or later chipsets. */
            reg_value = K800_LCD_VER_SCF_FORMULA(set_vres, panel_vres);
            write_reg_mask(CRA2, VIACR, 0x08, BIT3);       /* Vertical scaling enabled */
            load_reg_num = lcd_scaling_factor.lcd_ver_scaling_factor.reg_num;
            reg = lcd_scaling_factor.lcd_ver_scaling_factor.reg;
            load_reg(reg_value, load_reg_num, reg, VIACR);                
        }
        else
        {
            reg_value = CLE266_LCD_VER_SCF_FORMULA(set_vres, panel_vres);
            load_reg_num = lcd_scaling_factor_CLE.lcd_ver_scaling_factor.reg_num;
            reg = lcd_scaling_factor_CLE.lcd_ver_scaling_factor.reg;
            load_reg(reg_value, load_reg_num, reg, VIACR);                  
        }

        DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "Vertical Scaling value = %d\n", reg_value));
    }
    else
    {
        write_reg_mask(CRA2, VIACR, 0x00, BIT3);      /* Vertical scaling disabled */
    }
}

void VIALoadLCDPatchRegs(VIABIOSInfoPtr pBIOSInfo, int set_iga)
{
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIALoadLCDPatchRegs!!\n"));

    unlock_crt();

    /* Patch for simultaneous & Expansion */
    if((set_iga == IGA1_IGA2)&&(!pBIOSInfo->LVDSSettingInfo.Center))
    {
        switch (pBIOSInfo->Chipset)
        {
            case VIA_CLE266:
            case VIA_KM400:
                VIA_load_lcd_k400_patch_tbl(pBIOSInfo);
                break;
            case VIA_K8M800:
                break;
            default:
                VIA_load_lcd_p880_patch_tbl(pBIOSInfo);
                break;
        }
    }
    lock_crt();
}

void VIA_load_lcd_k400_patch_tbl(VIABIOSInfoPtr pBIOSInfo)
{
    struct io_reg   *lcd_patch_reg = NULL;
    int             reg_num = 0;
    VIABIOSInfoPtr  pVia = pBIOSInfo;

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIA_load_lcd_k400_patch_tbl!!\n"));

    switch(pBIOSInfo->LVDSSettingInfo.PanelSizeID)
    {
        case VIA_PANEL8X6:      /* LCD 800x600 */
            switch(pBIOSInfo->LVDSSettingInfo.ModeIndex)
            {
                case M640x480:
                    reg_num = NUM_TOTAL_K400_LCD_RES_6X4_8X6;
                    lcd_patch_reg = K400_LCD_RES_6X4_8X6;
                    break;
                case M720x480:
                case M720x576:
                    reg_num = NUM_TOTAL_K400_LCD_RES_7X4_8X6;
                    lcd_patch_reg = K400_LCD_RES_7X4_8X6;
                    break;
            }
            break;
        case VIA_PANEL10X7:     /* LCD 1024x768 */
            switch(pBIOSInfo->LVDSSettingInfo.ModeIndex)
            {
                case M640x480:
                    reg_num = NUM_TOTAL_K400_LCD_RES_6X4_10X7;
                    lcd_patch_reg = K400_LCD_RES_6X4_10X7;
                    break;
                case M720x480:
                case M720x576:
                    reg_num = NUM_TOTAL_K400_LCD_RES_7X4_10X7;
                    lcd_patch_reg = K400_LCD_RES_7X4_10X7;
                    break;
                case M800x600:
                    reg_num = NUM_TOTAL_K400_LCD_RES_8X6_10X7;
                    lcd_patch_reg = K400_LCD_RES_8X6_10X7;
                    break;
            }
            break;
        case VIA_PANEL12X10:    /* LCD 1280x1024 */
            switch(pBIOSInfo->LVDSSettingInfo.ModeIndex)
            {
                case M640x480:
                    reg_num = NUM_TOTAL_K400_LCD_RES_6X4_12X10;
                    lcd_patch_reg = K400_LCD_RES_6X4_12X10;
                    break;
                case M720x480:
                case M720x576:
                    reg_num = NUM_TOTAL_K400_LCD_RES_7X4_12X10;
                    lcd_patch_reg = K400_LCD_RES_7X4_12X10;
                    break;
                case M800x600:
                    reg_num = NUM_TOTAL_K400_LCD_RES_8X6_12X10;
                    lcd_patch_reg = K400_LCD_RES_8X6_12X10;
                    break;
                case M1024x768:
                    reg_num = NUM_TOTAL_K400_LCD_RES_10X7_12X10;
                    lcd_patch_reg = K400_LCD_RES_10X7_12X10;
                    break;

            }
            break;
        case VIA_PANEL14X10:    /* LCD 1400x1050 */
            switch(pBIOSInfo->LVDSSettingInfo.ModeIndex)
            {
                case M640x480:
                    reg_num = NUM_TOTAL_K400_LCD_RES_6X4_14X10;
                    lcd_patch_reg = K400_LCD_RES_6X4_14X10;
                    break;
                case M800x600:
                    reg_num = NUM_TOTAL_K400_LCD_RES_8X6_14X10;
                    lcd_patch_reg = K400_LCD_RES_8X6_14X10;
                    break;
                case M1024x768:
                    reg_num = NUM_TOTAL_K400_LCD_RES_10X7_14X10;
                    lcd_patch_reg = K400_LCD_RES_10X7_14X10;
                    break;
                case M1280x768:
                case M1280x800:                
                case M1280x960:
                case M1280x1024:
                    reg_num = NUM_TOTAL_K400_LCD_RES_12X10_14X10;
                    lcd_patch_reg = K400_LCD_RES_12X10_14X10;
                    break;
            }
            break;
        case VIA_PANEL16X12:    /* LCD 1600x1200 */
            switch(pBIOSInfo->LVDSSettingInfo.ModeIndex)
            {
                case M640x480:
                    reg_num = NUM_TOTAL_K400_LCD_RES_6X4_16X12;
                    lcd_patch_reg = K400_LCD_RES_6X4_16X12;
                    break;
                case M720x480:
                case M720x576:
                    reg_num = NUM_TOTAL_K400_LCD_RES_7X4_16X12;
                    lcd_patch_reg = K400_LCD_RES_7X4_16X12;
                    break;
                case M800x600:
                    reg_num = NUM_TOTAL_K400_LCD_RES_8X6_16X12;
                    lcd_patch_reg = K400_LCD_RES_8X6_16X12;
                    break;
                case M1024x768:
                    reg_num = NUM_TOTAL_K400_LCD_RES_10X7_16X12;
                    lcd_patch_reg = K400_LCD_RES_10X7_16X12;
                    break;
                case M1280x768:
                case M1280x800:                
                case M1280x960:
                case M1280x1024:
                    reg_num = NUM_TOTAL_K400_LCD_RES_12X10_16X12;
                    lcd_patch_reg = K400_LCD_RES_12X10_16X12;
                    break;
            }
            break;
        case VIA_PANEL1366X7:    /* LCD 1366x768 */
            /*switch(pBIOSInfo->LVDSSettingInfo.ModeIndex)
            {
                case M640x480:
                    reg_num = NUM_TOTAL_K400_LCD_RES_6X4_1366X7;
                    lcd_patch_reg = K400_LCD_RES_6X4_1366X7;
                    break;
                case M720x480:
                case M720x576:
                    reg_num = NUM_TOTAL_K400_LCD_RES_7X4_1366X7;
                    lcd_patch_reg = K400_LCD_RES_7X4_1366X7;
                    break;
                case M800x600:
                    reg_num = NUM_TOTAL_K400_LCD_RES_8X6_1366X7;
                    lcd_patch_reg = K400_LCD_RES_8X6_1366X7;
                    break;
                case M1024x768:
                    reg_num = NUM_TOTAL_K400_LCD_RES_10X7_1366X7;
                    lcd_patch_reg = K400_LCD_RES_10X7_1366X7;
                    break;
                case M1280x768:
                case M1280x800:                
                case M1280x960:
                case M1280x1024:
                    reg_num = NUM_TOTAL_K400_LCD_RES_12X10_1366X7;
                    lcd_patch_reg = K400_LCD_RES_12X10_1366X7;
                    break;
            }*/
            break;
        case VIA_PANEL1360X7:    /* LCD 1360x768 */
            break;            
    }

    if (reg_num != 0)
    {
        /* H.W. Reset : ON */
        write_reg_mask(CR17, VIACR, 0x00, BIT7);

        write_regx(lcd_patch_reg, reg_num);

        /* H.W. Reset : OFF */
        write_reg_mask(CR17, VIACR, 0x80, BIT7);

        /* Reset PLL */
        write_reg_mask(SR40, VIASR, 0x02, BIT1);
        write_reg_mask(SR40, VIASR, 0x00, BIT1);

        /* Fire! */
        VGAOUT8(VIAWMisc, VGAIN8(VIARMisc) | (BIT2+BIT3));
    }
}

void VIA_load_lcd_p880_patch_tbl(VIABIOSInfoPtr pBIOSInfo)
{
    struct io_reg   *lcd_patch_reg = NULL;
    int             reg_num = 0;
    VIABIOSInfoPtr  pVia = pBIOSInfo;

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIA_load_lcd_p880_patch_tbl!!\n"));

    switch(pBIOSInfo->LVDSSettingInfo.PanelSizeID)
    {
        case VIA_PANEL14X10:
            switch(pBIOSInfo->LVDSSettingInfo.ModeIndex)
            {
                case M640x480:
                    reg_num = NUM_TOTAL_P880_LCD_RES_6X4_14X10;
                    lcd_patch_reg = P880_LCD_RES_6X4_14X10;
                    break;
                case M800x600:
                    reg_num = NUM_TOTAL_P880_LCD_RES_8X6_14X10;
                    lcd_patch_reg = P880_LCD_RES_8X6_14X10;
                    break;
            }
            break;
        case VIA_PANEL16X12:
            switch(pBIOSInfo->LVDSSettingInfo.ModeIndex)
            {
                case M640x480:
                    reg_num = NUM_TOTAL_P880_LCD_RES_6X4_16X12;
                    lcd_patch_reg = P880_LCD_RES_6X4_16X12;
                    break;
                case M720x480:
                case M720x576:
                    reg_num = NUM_TOTAL_P880_LCD_RES_7X4_16X12;
                    lcd_patch_reg = P880_LCD_RES_7X4_16X12;
                    break;
                case M800x600:
                    reg_num = NUM_TOTAL_P880_LCD_RES_8X6_16X12;
                    lcd_patch_reg = P880_LCD_RES_8X6_16X12;
                    break;
                case M1024x768:
                    reg_num = NUM_TOTAL_P880_LCD_RES_10X7_16X12;
                    lcd_patch_reg = P880_LCD_RES_10X7_16X12;
                    break;
                case M1280x768:
                case M1280x960:
                case M1280x1024:
                    reg_num = NUM_TOTAL_P880_LCD_RES_12X10_16X12;
                    lcd_patch_reg = P880_LCD_RES_12X10_16X12;
                    break;
            }
            break;
    }

    if (reg_num != 0)
    {
        /* H.W. Reset : ON */
        write_reg_mask(CR17, VIACR, 0x00, BIT7);

        write_regx(lcd_patch_reg, reg_num);

        /* H.W. Reset : OFF */
        write_reg_mask(CR17, VIACR, 0x80, BIT7);

        /* Reset PLL */
        write_reg_mask(SR40, VIASR, 0x02, BIT1);
        write_reg_mask(SR40, VIASR, 0x00, BIT1);

        /* Fire! */
        VGAOUT8(VIAWMisc, VGAIN8(VIARMisc) | (BIT2+BIT3));
    }
}

void VIASetLCDOutputPath(VIABIOSInfoPtr pBIOSInfo, int set_iga, int output_interface)
{
    switch(set_iga)
    {
        case IGA1:
            write_reg_mask(CR6B, VIACR, 0x00, BIT3);
            /* RainLin [2005.11.22]: Modify for sequence issue. */
            DisableSecondDisplayChannel();
            write_reg_mask(CR6A, VIACR, 0x08, BIT3);
            break;
        case IGA2:
            write_reg_mask(CR6B, VIACR, 0x00, BIT3);
            /*------IGA2 Enable sequence------*/
            /* RainLin [2005.11.22]: Replaced by function call. */
            EnableSecondDisplayChannel();
            write_reg_mask(CR6A, VIACR, 0x08, BIT3);
            break;
        case IGA1_IGA2:
            write_reg_mask(CR6B, VIACR, 0x08, BIT3);
            /* RainLin [2005.11.22]: Modify for sequence issue. */
            DisableSecondDisplayChannel();
            write_reg_mask(CR6A, VIACR, 0x08, BIT3);
            break;
    }

    switch(output_interface)
    {
        case VIA_DI_DVP0:
            if (set_iga == IGA1)
            {
                write_reg_mask(CR96, VIACR, 0x00, BIT4);
            }
            else
            {
                write_reg(CR91, VIACR, 0x00);
                write_reg_mask(CR96, VIACR, 0x10, BIT4);
            }

            break;
            
        case VIA_DI_DVP1:
            if (set_iga== IGA1)
            {
                write_reg_mask(CR9B, VIACR, 0x00, BIT4);
            }
            else
            {
                write_reg(CR91, VIACR, 0x00);
                write_reg_mask(CR9B, VIACR, 0x10, BIT4);
            }
            
            break;
            
        case VIA_DI_DFPHIGH:
            if (set_iga== IGA1)
            {
                write_reg_mask(CR97, VIACR, 0x00, BIT4);
            }
            else
            {
                write_reg(CR91, VIACR, 0x00);
                write_reg_mask(CR97, VIACR, 0x10, BIT4);
                write_reg_mask(CR96, VIACR, 0x10, BIT4);
            }
            break;
        case VIA_DI_DFPLOW:
            if (set_iga== IGA1)
            {
                write_reg_mask(CR99, VIACR, 0x00, BIT4);
                write_reg_mask(CR9B, VIACR, 0x00, BIT4);
            }
            else
            {
                write_reg(CR91, VIACR, 0x00);
                write_reg_mask(CR99, VIACR, 0x10, BIT4);
                write_reg_mask(CR9B, VIACR, 0x10, BIT4);
            }

            break;

        case VIA_DI_DFPHIGHLOW:
            if(( pBIOSInfo->Chipset==VIA_K8M890 ) ||( pBIOSInfo->Chipset==VIA_P4M890)) 
               write_reg_mask(CR97, VIACR, 0x84, BIT7+BIT2+BIT1+BIT0);
            if (set_iga== IGA1)
            {
                write_reg_mask(CR97, VIACR, 0x00, BIT4);
                write_reg_mask(CR99, VIACR, 0x00, BIT4);                
            }
            else
            {
                write_reg(CR91, VIACR, 0x00);
                write_reg_mask(CR97, VIACR, 0x10, BIT4);
                write_reg_mask(CR99, VIACR, 0x10, BIT4);
            }
            break;

        case VIA_DI_LVDS0:
            if (set_iga == IGA1)
            {
                write_reg_mask(CR99, VIACR, 0x00, BIT4);
            }
            else
            {
                write_reg_mask(CR99, VIACR, 0x10, BIT4);
            }
            
            break;
            
        case VIA_DI_LVDS1:
            if (set_iga == IGA1)
            {
                write_reg_mask(CR97, VIACR, 0x00, BIT4);
            }
            else
            {
                write_reg_mask(CR97, VIACR, 0x10, BIT4);
            }
            
            break;

        case VIA_DI_LVDS0LVDS1:
            if (set_iga == IGA1)
            {
                if ((pBIOSInfo->DisplayHardwareLayout == HW_LAYOUT_LCD_ONLY) ||
                    (pBIOSInfo->DisplayHardwareLayout == HW_LAYOUT_LCD_EXTERNAL_LCD2))
                {
                    /* For dual channel LVDS: */
                    write_reg_mask(CR99, VIACR, 0x00, BIT4);
                }
                else
                {
                    /* For 1 channel LVDS0: */
                    if (pBIOSInfo->ActiveDevice & VIA_DEVICE_LCD)
                    {
                        write_reg_mask(CR99, VIACR, 0x00, BIT4);
                    }

                    /* For 1 channel LVDS1: */
                    if (pBIOSInfo->ActiveDevice & VIA_DEVICE_LCD2)
                    {
                        write_reg_mask(CR97, VIACR, 0x00, BIT4);
                    }
                }
            }
            else
            {
                if ((pBIOSInfo->DisplayHardwareLayout == HW_LAYOUT_LCD_ONLY) ||
                    (pBIOSInfo->DisplayHardwareLayout == HW_LAYOUT_LCD_EXTERNAL_LCD2))
                {
                    /* For dual channel LVDS: */
                    write_reg_mask(CR99, VIACR, 0x10, BIT4);
                }
                else
                {
                    /* For 1 channel LVDS0: */
                    if (pBIOSInfo->ActiveDevice & VIA_DEVICE_LCD)
                    {
                        write_reg_mask(CR99, VIACR, 0x10, BIT4);
                    }

                    /* For 1 channel LVDS1: */
                    if (pBIOSInfo->ActiveDevice & VIA_DEVICE_LCD2)
                    {
                        write_reg_mask(CR97, VIACR, 0x10, BIT4);
                    }
                }
            }
            
            break;
    }
}

int VIAGetModeIndexByPanelSize(int PanelSizeID)
{
    switch(PanelSizeID)
    {
        case VIA_PANEL6X4:
            return M640x480;
         case VIA_PANEL8X6:
            return M800x600;
	    case VIA_PANEL10X6:
			 return M1024x600;
        case VIA_PANEL10X7:
            return M1024x768;
        case VIA_PANEL12X7:
            return M1280x768;
        case VIA_PANEL12X8:
            return M1280x800;            
        case VIA_PANEL12X10:
            return M1280x1024;
        case VIA_PANEL14X10:
            return M1400x1050;
        case VIA_PANEL16X12:
            return M1600x1200;
        case VIA_PANEL1366X7:
            return M1366x768;
        case VIA_PANEL1360X7:
            return M1360x768;            
        default:
            return M1024x768;
    }
}

void VIA_fill_lcd_format(Bool bIsDualEdge, Bool bIsDithering)
{
    CARD8   bdithering = 0, bdual = 0;

    if(bIsDualEdge)
    {
       bdual = BIT4;
    }
    if(bIsDithering)
    {
       bdithering = BIT0;
    }

    write_reg_mask(CR88, VIACR, (bdithering | bdual), BIT4+BIT0);   /* Dual & Dithering */
}



static Bool
VIALVDSIdentify_VT1631(VIABIOSInfoPtr pBIOSInfo, LVDSSETTINGINFOPTR pLVDSSettingInfo, CARD32 PortNum)
{
    CARD8 Buffer[2];

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED, "VIALVDSIdentify_VT1631: Try Port No. 0x%x\n", PortNum));

    pLVDSSettingInfo->I2CPort = PortNum;
    pLVDSSettingInfo->I2CAddress = LVDS_I2C_ADDR_VT1631;
    
    /* Check vendor ID first: */
    if (!GPIOI2CRead_LVDS(pBIOSInfo, pLVDSSettingInfo, 0x00, &(Buffer[0])))
    {
        return FALSE;
    }
    
    if (!GPIOI2CRead_LVDS(pBIOSInfo, pLVDSSettingInfo, 0x01, &(Buffer[1])))
    {
        return FALSE;
    }

    if (!((Buffer[0] == 0x06) && (Buffer[1] == 0x11)))
    {        
        return FALSE;        
    }

    /* Vendor ID is correct, so continue to check Chip ID: */
    if (!GPIOI2CRead_LVDS(pBIOSInfo, pLVDSSettingInfo, 0x02, &(Buffer[0])))
    {
        return FALSE;
    }
    
    if (!GPIOI2CRead_LVDS(pBIOSInfo, pLVDSSettingInfo, 0x03, &(Buffer[1])))
    {
        return FALSE;
    }

    if ((Buffer[0] == 0x91) && (Buffer[1] == 0x31))
    {
        /* if hotkey, we don't fill any value. VGA BIOS will take care it. */
        if (!pBIOSInfo->Hotkey)
        {
            if (pBIOSInfo->LVDSSettingInfo.IsDualEdge)
            {
                GPIOI2CWrite_LVDS(pBIOSInfo, pLVDSSettingInfo, 0x08, 0x0D);
            }
            else
            {
                GPIOI2CWrite_LVDS(pBIOSInfo, pLVDSSettingInfo, 0x08, 0x09);
            }
        }

        pLVDSSettingInfo->ChipID = VIA_VT3191;
        pBIOSInfo->LVDSSettingInfo.DIPort = VIA_DI_DFPHIGHLOW;
        DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED, "Found VIA VT1631 LVDS on port %x!\n", PortNum));        

        return TRUE;
    }

    return FALSE;
}


Bool VIAPostLCDNotDVO(VIABIOSInfoPtr pBIOSInfo)
{
    unsigned char   cr6a, cr6b, cr91, cr93;
    Bool            ret = FALSE;

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIAPostLCDNotDVO\n"));

    /*=* CR6A[6]Second Display Channel Reset: 0: reset
             [3]LCD Enable: 1: enable                   *=*/
    cr6a = read_reg(VIACR, CR6A);
    write_reg_mask(CR6A, VIACR, 0x48, BIT3+BIT6);

    /*=* CR6B[0]VCK Input Selection: 1 = External clock. *=*/
    cr6b = read_reg(VIACR, CR6B);
    write_reg_mask(CR6B, VIACR, 0x08, BIT0+BIT1+BIT2+BIT3);

    /*=* CR91[4] VDD On [3] Data On [2] VEE On [1] Back Light Off
             [0] Software Control Power Sequence *=*/
    cr91 = read_reg(VIACR, CR91);
    write_reg(CR91, VIACR, 0x1D);

    /*=* CR93[7] DI1 Data Source Selection: 1 = DSP2.
         CR93[5] DI1 Clock Source: 1 = internal.
         CR93[4] DI1 Clock Polarity.
         CR93[3:1] DI1 Clock Adjust. CR93[0] DI1 enable *=*/
    cr93 = read_reg(VIACR, CR93);
    write_reg(CR93, VIACR, 0x01);

    /* Sense VT1631 */
    ret = VIALVDSIdentify_VT1631(pBIOSInfo, &(pBIOSInfo->LVDSSettingInfo), PORT_INDEX_I2C_31);



    if((pBIOSInfo->MobileFamily)&&(!ret))
    {
        DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "LCD Hardwired\n"));
        pBIOSInfo->LVDSSettingInfo.ChipID = VIA_Hardwired;
        ret = TRUE;
    }

    /* Restore status */
    write_reg(CR6A, VIACR, cr6a);
    write_reg(CR6B, VIACR, cr6b);
    write_reg(CR91, VIACR, cr91);
    write_reg(CR93, VIACR, cr93);

    return ret;
}

Bool VIAPostLCDDVO(VIABIOSInfoPtr pBIOSInfo)
{        
    unsigned char cr91, cr9b;    

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIAPostLCDDVO\n"));
    
    /*=* CR91[4] VDD On [3] Data On [2] VEE On [1] Back Light Off
             [0] Software Control Power Sequence *=*/
    cr91 = read_reg(VIACR, CR91);
    write_reg(CR91, VIACR, 0x1D);
    
    /*=* CR9B[4] DVP1 Data Source Selection: 1 = From secondary display.
         CR9B[2:0] DVP1 Clock Adjust *=*/     
    cr9b = read_reg(VIACR, CR9B);
    write_reg(CR9B, VIACR, 0x01);

    if (pBIOSInfo->MobileFamily)
    {
        VIALVDSIdentify(pBIOSInfo);
        
        /* If LVDS type is assigned, we have to determine LCD port. */
        if (pBIOSInfo->LVDSSettingInfo.ChipID)
        {
            VIAInitLCDDIPort(pBIOSInfo, &(pBIOSInfo->LVDSSettingInfo));
        }

        if (pBIOSInfo->LVDSSettingInfo2.ChipID)
        {
            VIAInitLCDDIPort(pBIOSInfo, &(pBIOSInfo->LVDSSettingInfo2));
        }
    }

    /* Restore status */    
    write_reg(CR91, VIACR, cr91);
    write_reg(CR9B, VIACR, cr9b);

    return TRUE;
}


/*
    S.T.Chen[2006.6.12]: To determine LCD port.
*/
void VIAInitLCDDIPort(VIABIOSInfoPtr pBIOSInfo, LVDSSETTINGINFOPTR pLVDSSettingInfo)
{
    if (pLVDSSettingInfo->DIPort != VIA_DI_NONE)
    {
        /* Do nothing, because LCD DIPort can be assigned by option. */
        return;
    }
    
    if (pLVDSSettingInfo->ChipID == VIA_VT1636)
    {
        pLVDSSettingInfo->DIPort = VIA_DI_DVP1;
    }
    else if(pLVDSSettingInfo->ChipID == VIA_INTEGRATED_LVDS)
    {
        /* RainLin [2005/11/15]: Only support integrated LVDS for mobile family. */
        /* VT3324 supports integrated LVDS. */   
        VIACheckDIPortOfIntegratedLVDS(pBIOSInfo, pLVDSSettingInfo);
    }
    else
    {        
        /* RainLin [2006.04.13]: For the case VT1637 (PCIE LVDS Transmitter) on VT3336/VT3327, */
        /* we should use DFP Low to enable LCD, otherwise we use DFP High-Low as default */
        /* FixMe!! We should find a better method to do this. */
        if(((pBIOSInfo->Chipset == VIA_K8M890)||
            (pBIOSInfo->Chipset == VIA_P4M890)||
            (pBIOSInfo->Chipset == VIA_P4M900)) && 
           (pLVDSSettingInfo->BusWidth == VIA_DI_12BIT))
        {
            pLVDSSettingInfo->DIPort = VIA_DI_DFPLOW;
        }
        else
        {
            pLVDSSettingInfo->DIPort = VIA_DI_DFPHIGHLOW;
        }
           
        pLVDSSettingInfo->BusWidth = VIA_DI_24BIT;     /* Low-Swing LCD always use 24Bit */        
    }

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED, "LCD DIPort = %d\n", pLVDSSettingInfo->DIPort));
}


static Bool
VIALVDSIdentify_IntegratedLVDS(VIABIOSInfoPtr pBIOSInfo)
{
    if (pBIOSInfo->DisplayHardwareLayout == HW_LAYOUT_LCD_EXTERNAL_LCD2)
    {
        /* Two dual channel LCD (Internal LVDS + External LVDS): */
        /* If we have an external LVDS, such as VT1636, we should have its chip ID already. */
        if (pBIOSInfo->LVDSSettingInfo.ChipID)
        {
            pBIOSInfo->LVDSSettingInfo2.ChipID = VIA_INTEGRATED_LVDS;
            DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED, "Support two dual channel LVDS! (Internal LVDS + External LVDS)\n"));
        }
        else
        {
            pBIOSInfo->LVDSSettingInfo.ChipID = VIA_INTEGRATED_LVDS;
            DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED, "Not found external LVDS, so can't support two dual channel LVDS!\n"));
        }
    }
    else if (pBIOSInfo->DisplayHardwareLayout == HW_LAYOUT_LCD_LCD2)
    {
        /* Two single channel LCD (Internal LVDS + Internal LVDS): */
        pBIOSInfo->LVDSSettingInfo.ChipID = VIA_INTEGRATED_LVDS;
        pBIOSInfo->LVDSSettingInfo2.ChipID = VIA_INTEGRATED_LVDS;
        DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED, "Support two single channel LVDS! (Internal LVDS + Internal LVDS)\n"));
    }
    else if (pBIOSInfo->DisplayHardwareLayout != HW_LAYOUT_DVI_ONLY)
    {
        /* If we have found external LVDS, just use it, otherwise, we will use internal LVDS as default. */
        if (!pBIOSInfo->LVDSSettingInfo.ChipID)
        {
            pBIOSInfo->LVDSSettingInfo.ChipID = VIA_INTEGRATED_LVDS;
            DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED, "Found Integrated LVDS!\n"));
        }
    }
    else
    {
        pBIOSInfo->LVDSSettingInfo.ChipID = VIA_NONETRANSMITTER;
        DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED, "Do not support LVDS!\n"));
        return FALSE;
    }

    return TRUE;
}


/*
    S.T.Chen[2006.6.12]: To identify lvds.
*/
Bool VIALVDSIdentify(VIABIOSInfoPtr pBIOSInfo)
{
    /* Sense VT1636 */
    if (!VIALVDSIdentify_VT1636(pBIOSInfo, &(pBIOSInfo->LVDSSettingInfo), PORT_INDEX_I2C_31))
    {
        VIALVDSIdentify_VT1636(pBIOSInfo, &(pBIOSInfo->LVDSSettingInfo), PORT_INDEX_GPIO_2C);
    }

    /* Sense Integrated LVDS: */
    if((pBIOSInfo->Chipset == VIA_CX700) || (pBIOSInfo->Chipset == VIA_CN750))
    {
        VIALVDSIdentify_IntegratedLVDS(pBIOSInfo);
    }

    if(pBIOSInfo->LVDSSettingInfo.ChipID)
    {
        return TRUE;
    }
    
    /* Sense VT1631 */
    if (VIALVDSIdentify_VT1631(pBIOSInfo, &(pBIOSInfo->LVDSSettingInfo), PORT_INDEX_I2C_31))
    {
        /* Found VT1631 on port 0x31! */
        return TRUE;
    }

    if (VIALVDSIdentify_VT1631(pBIOSInfo, &(pBIOSInfo->LVDSSettingInfo), PORT_INDEX_GPIO_2C))
    {
        /* Found VT1631 on port 0x2C! */
        return TRUE;
    }


    if(!pBIOSInfo->LVDSSettingInfo.ChipID)
    {
        /* Hardwire LVDS */
        pBIOSInfo->LVDSSettingInfo.ChipID = read_reg(VIACR, CR3E) >> 4;
        DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED,
            "Using Hardwire LVDS! LVDS id=0x%x\n", pBIOSInfo->LVDSSettingInfo.ChipID));
    }

    if(pBIOSInfo->LVDSSettingInfo.ChipID)
    {
        return TRUE;
    }
    else
    {
        return FALSE;
    }
}


void VIACheckDIPortOfIntegratedLVDS(VIABIOSInfoPtr pBIOSInfo, LVDSSETTINGINFOPTR pLVDSSettingInfo)
{
    /* Determine LCD DI Port by hardware layout. */
    switch (pBIOSInfo->DisplayHardwareLayout)
    {
        case HW_LAYOUT_LCD_ONLY:
        {
            if (pLVDSSettingInfo->IsDualEdge)
            {
                pLVDSSettingInfo->DIPort = VIA_DI_LVDS0LVDS1;
            }
            else
            {
                pLVDSSettingInfo->DIPort = VIA_DI_LVDS0;
            }
            
            break;
        }

        case HW_LAYOUT_DVI_ONLY:
        {
            pLVDSSettingInfo->DIPort = VIA_DI_NONE;
            break;
        }

        case HW_LAYOUT_LCD_LCD2:
        case HW_LAYOUT_LCD_EXTERNAL_LCD2:
        {
            pLVDSSettingInfo->DIPort = VIA_DI_LVDS0LVDS1;
            break;
        }

        case HW_LAYOUT_LCD_DVI:
        {
            pLVDSSettingInfo->DIPort = VIA_DI_LVDS1;
            break;
        }
        
        default:
        {
            pLVDSSettingInfo->DIPort = VIA_DI_LVDS1;
            break;
        }
    }

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED, "Display Hardware Layout: 0x%x, LCD DI Port: 0x%x\n", 
        pBIOSInfo->DisplayHardwareLayout, pLVDSSettingInfo->DIPort));
}

static void
VIAPowerOnLVDSChannel(VIABIOSInfoPtr pBIOSInfo, LVDSSETTINGINFOPTR pLVDSSettingInfo)
{
    VIABIOSInfoPtr  pVia = pBIOSInfo;
    
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED, "VIAPowerOnLVDSChannel\n"));

    switch (pLVDSSettingInfo->DIPort)
    {
        case VIA_DI_LVDS0:
        {
            VGAOUT8(0x3d4, CRD2);
            VGAOUT8(0x3d5, VGAIN8(0x3d5) & 0x7F);
            break;
        }

        case VIA_DI_LVDS1:
        {
            VGAOUT8(0x3d4, CRD2);
            VGAOUT8(0x3d5, VGAIN8(0x3d5) & 0xBF);
            break;
        }

        case VIA_DI_LVDS0LVDS1:
        {
            VGAOUT8(0x3d4, CRD2);
            VGAOUT8(0x3d5, VGAIN8(0x3d5) & 0x3F);
            break;
        }
    }
}

static void
VIAPowerOffLVDSChannel(VIABIOSInfoPtr pBIOSInfo, LVDSSETTINGINFOPTR pLVDSSettingInfo)
{
    VIABIOSInfoPtr  pVia = pBIOSInfo;
    
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED, "VIAPowerOffLVDSChannel\n"));

    switch (pLVDSSettingInfo->DIPort)
    {
        case VIA_DI_LVDS0:
        {
            VGAOUT8(0x3d4, CRD2);
            VGAOUT8(0x3d5, VGAIN8(0x3d5) | 0x80);
            break;
        }

        case VIA_DI_LVDS1:
        {
            VGAOUT8(0x3d4, CRD2);
            VGAOUT8(0x3d5, VGAIN8(0x3d5) | 0x40);
            break;
        }

        case VIA_DI_LVDS0LVDS1:
        {
            VGAOUT8(0x3d4, CRD2);
            VGAOUT8(0x3d5, VGAIN8(0x3d5) | 0xC0);
            break;
        }
    }
}

static void
EnableIntegratedLVDS(VIABIOSInfoPtr pBIOSInfo, LVDSSETTINGINFOPTR pLVDSSettingInfo)
{
    VIABIOSInfoPtr  pVia = pBIOSInfo;
    Bool bTurnOnFirstPowerSequence = FALSE;
    Bool bTurnOnSecondPowerSequence = FALSE;

    if (pLVDSSettingInfo->DIPort == VIA_DI_LVDS0LVDS1)
    {
        if ((pLVDSSettingInfo->IsDualEdge) ||
            (pBIOSInfo->DisplayHardwareLayout == HW_LAYOUT_LCD_EXTERNAL_LCD2))
        {
            bTurnOnFirstPowerSequence = TRUE;
        }
        else
        {                
            /* For two single channel LCD: */
            if (pBIOSInfo->ActiveDevice & VIA_DEVICE_LCD)
            {
                bTurnOnFirstPowerSequence = TRUE;
            }

            if (pBIOSInfo->ActiveDevice & VIA_DEVICE_LCD2)
            {
                bTurnOnSecondPowerSequence = TRUE;
            }
        }
    }
    else if (pLVDSSettingInfo->DIPort == VIA_DI_LVDS1)
    {
        /* For single channel LCD (use LVDS1) */
        bTurnOnSecondPowerSequence = TRUE;
    }
    else
    {
        /* For single channel LCD (use LVDS0) */
        bTurnOnFirstPowerSequence = TRUE;
    }
 
    if (bTurnOnFirstPowerSequence)
    {
        /* Use first power sequence control: */
        
        /* Use hardware control power sequence. */
        VGAOUT8(0x3d4, CR91);
        VGAOUT8(0x3d5, VGAIN8(0x3d5) & 0xFE);

        /* Turn on back light. */
        VGAOUT8(0x3d4, CR91);
        VGAOUT8(0x3d5, VGAIN8(0x3d5) & 0x3F);

        /* Turn on hardware power sequence. */
        VGAOUT8(0x3d4, CR6A);
        VGAOUT8(0x3d5, VGAIN8(0x3d5) | 0x08);
    }
        
    if (bTurnOnSecondPowerSequence)
    {
        /* Use second power sequence control: */
        
        /* Use hardware control power sequence. */
        VGAOUT8(0x3d4, CRD3);
        VGAOUT8(0x3d5, VGAIN8(0x3d5) & 0xFE);

        /* Turn on back light. */
        VGAOUT8(0x3d4, CRD3);
        VGAOUT8(0x3d5, VGAIN8(0x3d5) & 0x3F);

        /* Turn on hardware power sequence. */
        VGAOUT8(0x3d4, CRD4);
        VGAOUT8(0x3d5, VGAIN8(0x3d5) | 0x02);
    }
    
    /* Turn DFP High/Low pad on. */
    VGAOUT8(0x3c4, SR2A);
    VGAOUT8(0x3c5, VGAIN8(0x3c5) | 0x0F);

    /* Power on LVDS channel. */
    VIAPowerOnLVDSChannel(pBIOSInfo, pLVDSSettingInfo);
}


static void
DisableIntegratedLVDS(VIABIOSInfoPtr pBIOSInfo, LVDSSETTINGINFOPTR pLVDSSettingInfo)
{
    VIABIOSInfoPtr  pVia = pBIOSInfo;
    Bool bTurnOffFirstPowerSequence = FALSE;
    Bool bTurnOffSecondPowerSequence = FALSE;

    if (pLVDSSettingInfo->DIPort == VIA_DI_LVDS0LVDS1)
    {
        if ((pLVDSSettingInfo->IsDualEdge) ||
            (pBIOSInfo->DisplayHardwareLayout == HW_LAYOUT_LCD_EXTERNAL_LCD2))
        {
            bTurnOffFirstPowerSequence = TRUE;
        }
        else
        {                
            /* For two single channel LCD: */
            if (pBIOSInfo->ActiveDevice & VIA_DEVICE_LCD)
            {
                bTurnOffFirstPowerSequence = TRUE;
            }

            if (pBIOSInfo->ActiveDevice & VIA_DEVICE_LCD2)
            {
                bTurnOffSecondPowerSequence = TRUE;
            }
        }
    }
    else if (pLVDSSettingInfo->DIPort == VIA_DI_LVDS1)
    {
        /* For single channel LCD (use LVDS1) */
        bTurnOffSecondPowerSequence = TRUE;
    }
    else
    {
        /* For single channel LCD (use LVDS0) */
        bTurnOffFirstPowerSequence = TRUE;
    }

    if (bTurnOffFirstPowerSequence)
    {
        /* Use first power sequence control: */

        /* Turn off power sequence. */
        VGAOUT8(0x3d4, CR6A);
        VGAOUT8(0x3d5, VGAIN8(0x3d5) & 0xF7);

        usleep(1);

        /* Turn off back light. */
        VGAOUT8(0x3d4, CR91);
        VGAOUT8(0x3d5, 0xC0);
    }

    if (bTurnOffSecondPowerSequence)
    {
        /* Use second power sequence control: */

        /* Turn off power sequence. */
        VGAOUT8(0x3d4, CRD4);
        VGAOUT8(0x3d5, VGAIN8(0x3d5) & 0xFD);

        usleep(1);

        /* Turn off back light. */
        VGAOUT8(0x3d4, CRD3);
        VGAOUT8(0x3d5, 0xC0);
    }

    /* Turn DFP High/Low Pad off. */
    VGAOUT8(0x3c4, SR2A);
    VGAOUT8(0x3c5, VGAIN8(0x3c5) & 0xF0);

    /* Power off LVDS channel. */
    VIAPowerOffLVDSChannel(pBIOSInfo, pLVDSSettingInfo);
}

/*=*
 *
 * Bool VIASetPanelState(VIABIOSInfoPtr, unsigned char)
 *
 *     - Set Flat Panel Expaension/Centering State
 *
 * Return Type:    Bool
 *
 * The Definition of Input Value:
 *
 *                 VIABIOSInfoPtr
 *
 *                 Bit[7:1] Reserved
 *                 Bit[0] 0/1 = Centering/Expansion
 *
 * The Definition of Return Value:
 *
 *                 Success - TRUE
 *                 Not Success - FALSE
 *=*/
Bool VIASetPanelState(VIABIOSInfoPtr pBIOSInfo, unsigned char state)
{
    if(state & 0x01)
    {
        write_reg_mask(CR3B, VIACR, 0x01, BIT0);
    }
    else
    {
        write_reg_mask(CR3B, VIACR, 0x0, BIT0);
    }

    return TRUE;
}

/*=*
 *
 * unsigned char VIAGetPanelState(VIABIOSInfoPtr)
 *
 *     - Get Flat Panel Expaension/Centering State
 *
 * Return Type:    unsigend char
 *
 * The Definition of Input Value:
 *
 *                 VIABIOSInfoPtr
 *
 *                 Bit[7:1] Reserved
 *                 Bit[0] 0/1 = Centering/Expansion
 *
 * The Definition of Return Value:
 *
 *                 Bit[7:1] Reserved
 *                 Bit[0] 0/1 = Centering/Expansion
 *=*/
unsigned char VIAGetPanelState(VIABIOSInfoPtr pBIOSInfo)
{
    unsigned char tmp;

    tmp = read_reg(VIACR, CR3B) & 0x01;

    return tmp;
}

void VIAEnableLCD(VIABIOSInfoPtr pBIOSInfo, LVDSSETTINGINFOPTR pLVDSSettingInfo)
{
    VIABIOSInfoPtr  pVia = pBIOSInfo;
    VIAModeTablePtr pViaModeTable = pBIOSInfo->pModeTable;
    int             j, k;
    int             port, offset, mask, data;

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED, "VIAEnableLCD\n"));

    /* RainLin [2005.11.18]: Because VT3324 has 2 power sequence controllers, */
    /* so it's better to divide the codes from other LVDS chips to be more clearly.*/
    if (pLVDSSettingInfo->ChipID == VIA_INTEGRATED_LVDS)
    {
        DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED, "VIAEnableLCD VIA_INTEGRATED_LVDS\n"));
        EnableIntegratedLVDS(pBIOSInfo, pLVDSSettingInfo);
        return;
    }

    if (pLVDSSettingInfo->ChipID == VIA_VT1636)
    {
        DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED, "VIAEnableLCD VIA_VT1636\n"));
        VIAEnableLVDS_VT1636(pBIOSInfo, pLVDSSettingInfo);
        return;
    }

    /* Only CLE266 use software power sequencing */
    /* Find Panel Size Index for PowerSeq Table */
    if (pBIOSInfo->Chipset == VIA_CLE266)
    {
        /* Software control power sequence */
        VGAOUT8(0x3d4, 0x91);
        VGAOUT8(0x3d5, VGAIN8(0x3d5) | 0x01);

        if (pLVDSSettingInfo->PanelSizeID != VIA_PANEL_INVALID)
        {
            /*we are always using powerOn[1]*/
            j = 1;
        }
        else
        {
            j = 0;
        }

        usleep(1);
        for (k = 0; k < pViaModeTable->powerOn[j].numEntry; k++)
        {
            port = pViaModeTable->powerOn[j].port[k];
            offset = pViaModeTable->powerOn[j].offset[k];
            mask = pViaModeTable->powerOn[j].mask[k];
            data = pViaModeTable->powerOn[j].data[k] & mask;
            VGAOUT8(0x300+port, offset);
            VGAOUT8(0x301+port, (VGAIN8(0x301+port) & ~mask) | data);
            usleep(pViaModeTable->powerOn[j].delay[k]);
        }
        usleep(10);
    }
    else
    {
        /* Hardware control power sequence */
        VGAOUT8(0x3d4, 0x91);
        VGAOUT8(0x3d5, VGAIN8(0x3d5) & 0xFE);
    }

    /* CLE266Cx: Enable LCD display path & back light */
    VGAOUT8(0x3d4, 0x91);
    VGAOUT8(0x3d5, VGAIN8(0x3d5) & 0x3F);

    /* Enable LCD */
    VGAOUT8(0x3d4, 0x6A);
    VGAOUT8(0x3d5, VGAIN8(0x3d5) | 0x08);

    if (pBIOSInfo->Chipset == VIA_CLE266)
    {
        /* DI1 Pad on */
        VGAOUT8(0x3c4, 0x1E);
        VGAOUT8(0x3c5, VGAIN8(0x3c5) | 0x30);
        /* DI1 Enable */
        VGAOUT8(0x3d4, 0x93);
        VGAOUT8(0x3d5, VGAIN8(0x3d5) | 0x01);

        /*=* For Advantech Project, support two LCD MHS *=*/
        if((pBIOSInfo->TwoLCD)&&(pBIOSInfo->IsSecondary))
        {
            VGAOUT8(0x3d4, 0x6B);
            VGAOUT8(0x3d5, VGAIN8(0x3d5) | 0x01);
            VGAOUT8(0x3d4, 0x6C);
            VGAOUT8(0x3d5, VGAIN8(0x3d5) | 0x21);
            /*for Toshiba case support Dual LVDS on CLE266*/
            VGAOUT8(0x3c4, 0x1E);
            VGAOUT8(0x3c5, VGAIN8(0x3c5) | 0xF1);
        }
    }
    else
    {
        /* if active LCD, turn DFP High/Low pad on */
        VGAOUT8(0x3c4, 0x2A);
        VGAOUT8(0x3c5, VGAIN8(0x3c5) | 0x0F);
    }

    /* Patch for Chrontel LVDS PLL Lock */
    if (!pBIOSInfo->Hotkey)
    {
    }   /* non-using device switch hotkey */
}

void VIADisableLCD(VIABIOSInfoPtr pBIOSInfo, LVDSSETTINGINFOPTR pLVDSSettingInfo)
{
    VIABIOSInfoPtr  pVia = pBIOSInfo;
    VIAModeTablePtr pViaModeTable = pBIOSInfo->pModeTable;
    int             j, k;
    int             port, offset, mask, data;

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED, "VIADisableLCD\n"));

    /* RainLin [2005.11.18]: Because VT3324 has 2 power sequence controllers, */
    /* so it's better to divide the codes from other LVDS chips to be more clearly.*/
    if (pLVDSSettingInfo->ChipID == VIA_INTEGRATED_LVDS)
    {
        DisableIntegratedLVDS(pBIOSInfo, pLVDSSettingInfo);
        return;
    }

    if (pLVDSSettingInfo->ChipID == VIA_VT1636)
    {
        VIADisableLVDS_VT1636(pBIOSInfo, pLVDSSettingInfo);
        return;
    }
    
    /* Patch for Chrontel LVDS PLL Lock */
    if (!pBIOSInfo->Hotkey)
    {
    }

    /* Disable LCD */
    VGAOUT8(0x3d4, 0x6A);
    VGAOUT8(0x3d5, VGAIN8(0x3d5) & 0xF7);
    usleep(1);

    /* Find Panel Size Index */
    if (pBIOSInfo->Chipset == VIA_CLE266)
    {
        if (pLVDSSettingInfo->PanelSizeID != VIA_PANEL_INVALID)
        {
            /*we are always using powerOn[1]*/
            j = 1;
        }
        else
        {
            j = 0;
        }

        for (k = 0; k < pViaModeTable->powerOff[j].numEntry; k++)
        {
            port = pViaModeTable->powerOff[j].port[k];
            offset = pViaModeTable->powerOff[j].offset[k];
            mask = pViaModeTable->powerOff[j].mask[k];
            data = pViaModeTable->powerOff[j].data[k] & mask;
            VGAOUT8(0x300+port, offset);
            VGAOUT8(0x301+port, (VGAIN8(0x301+port) & ~mask) | data);
            usleep(pViaModeTable->powerOff[j].delay[k]);
         }
    }

    if (pBIOSInfo->Chipset == VIA_CLE266)
    {
        /* DI1 Pad off
        VGAOUT8(0x3c4, 0x1E);
        VGAOUT8(0x3c5, VGAIN8(0x3c5) & 0xCF); */
        /* Software off Back Light */
        VGAOUT8(0x3d4, 0x91);
        VGAOUT8(0x3d5, 0xC1);
        /* DI1 Disable
        VGAOUT8(0x3d4, 0x93);
        VGAOUT8(0x3d5, VGAIN8(0x3d5) & 0xFE); */
    }
    else
    {
        /* if not active LCD, turn off VDD, turn off DFP High-Low */
        if (pBIOSInfo->MobileFamily)
        {
            /* Software off Back Light */
            VGAOUT8(0x3d4, 0x91);
            VGAOUT8(0x3d5, 0xC0);
            /* DFP High/Low Pad off */
            VGAOUT8(0x3c4, 0x2A);
            VGAOUT8(0x3c5, VGAIN8(0x3c5) & 0xF0);
        }
    }
}



/*
    S.T.Chen[2006.6.19]: Set LCD Panel Info according to Panel ID.
*/
Bool VIASetLCDPanelInfo(VIABIOSInfoPtr pBIOSInfo, int panel_id)
{
    switch (panel_id)
    {
        case 0x0:
            pBIOSInfo->LVDSSettingInfo.PanelSizeID = VIA_PANEL6X4;
            pBIOSInfo->LVDSSettingInfo.IsDualEdge = FALSE;
            pBIOSInfo->LVDSSettingInfo.IsDithering = TRUE;
            break;
        case 0x1:
            pBIOSInfo->LVDSSettingInfo.PanelSizeID = VIA_PANEL8X6;
            pBIOSInfo->LVDSSettingInfo.IsDualEdge = FALSE;
            pBIOSInfo->LVDSSettingInfo.IsDithering = TRUE;
            break;
        case 0x2:
            pBIOSInfo->LVDSSettingInfo.PanelSizeID = VIA_PANEL10X7;
            pBIOSInfo->LVDSSettingInfo.IsDualEdge = FALSE;
            pBIOSInfo->LVDSSettingInfo.IsDithering = TRUE;
            break;
        case 0x3:
            pBIOSInfo->LVDSSettingInfo.PanelSizeID = VIA_PANEL12X7;
            pBIOSInfo->LVDSSettingInfo.IsDualEdge = FALSE;
            pBIOSInfo->LVDSSettingInfo.IsDithering = TRUE;
            break;
        case 0x4:
            pBIOSInfo->LVDSSettingInfo.PanelSizeID = VIA_PANEL12X10;
            pBIOSInfo->LVDSSettingInfo.IsDualEdge = TRUE;
            pBIOSInfo->LVDSSettingInfo.IsDithering = TRUE;
            break;
        case 0x5:
            pBIOSInfo->LVDSSettingInfo.PanelSizeID = VIA_PANEL14X10;
            pBIOSInfo->LVDSSettingInfo.IsDualEdge = TRUE;
            pBIOSInfo->LVDSSettingInfo.IsDithering = TRUE;
            break;
        case 0x6:
            pBIOSInfo->LVDSSettingInfo.PanelSizeID = VIA_PANEL16X12;
            pBIOSInfo->LVDSSettingInfo.IsDualEdge = TRUE;
            pBIOSInfo->LVDSSettingInfo.IsDithering = TRUE;
            break;
        case 0x7:
            pBIOSInfo->LVDSSettingInfo.PanelSizeID = VIA_PANEL12X8;
            pBIOSInfo->LVDSSettingInfo.IsDualEdge = FALSE;
            pBIOSInfo->LVDSSettingInfo.IsDithering = TRUE;
            break;
        case 0x8:
            pBIOSInfo->LVDSSettingInfo.PanelSizeID = VIA_PANEL8X4;
            pBIOSInfo->LVDSSettingInfo.IsDualEdge = FALSE;
            pBIOSInfo->LVDSSettingInfo.IsDithering = TRUE;
            break;
        case 0x9:
            pBIOSInfo->LVDSSettingInfo.PanelSizeID = VIA_PANEL1366X7;
            pBIOSInfo->LVDSSettingInfo.IsDualEdge = FALSE;
            pBIOSInfo->LVDSSettingInfo.IsDithering = FALSE;
            break;
        case 0xA:
            pBIOSInfo->LVDSSettingInfo.PanelSizeID = VIA_PANEL1360X7;
            pBIOSInfo->LVDSSettingInfo.IsDualEdge = FALSE;
            pBIOSInfo->LVDSSettingInfo.IsDithering = FALSE;
            break;
        case 0xB:
            pBIOSInfo->LVDSSettingInfo.PanelSizeID = VIA_PANEL10X7;
            pBIOSInfo->LVDSSettingInfo.IsDualEdge = TRUE;
            pBIOSInfo->LVDSSettingInfo.IsDithering = FALSE;
            break;
        case 0xC:
            pBIOSInfo->LVDSSettingInfo.PanelSizeID = VIA_PANEL12X7;
            pBIOSInfo->LVDSSettingInfo.IsDualEdge = FALSE;
            pBIOSInfo->LVDSSettingInfo.IsDithering = FALSE;
            break;
        case 0xD:
            pBIOSInfo->LVDSSettingInfo.PanelSizeID = VIA_PANEL12X10;
            pBIOSInfo->LVDSSettingInfo.IsDualEdge = TRUE;
            pBIOSInfo->LVDSSettingInfo.IsDithering = FALSE;
            break;
        case 0xE:
            pBIOSInfo->LVDSSettingInfo.PanelSizeID = VIA_PANEL14X10;
            pBIOSInfo->LVDSSettingInfo.IsDualEdge = TRUE;
            pBIOSInfo->LVDSSettingInfo.IsDithering = FALSE;
            break;
        case 0xF:
            pBIOSInfo->LVDSSettingInfo.PanelSizeID = VIA_PANEL16X12;
            pBIOSInfo->LVDSSettingInfo.IsDualEdge = TRUE;
            pBIOSInfo->LVDSSettingInfo.IsDithering = FALSE;
            break;
        case 0x10:  /*For Panasonic 1280x768 18bit Dual-Channel Panel */
            pBIOSInfo->LVDSSettingInfo.PanelSizeID = VIA_PANEL12X7;
            pBIOSInfo->LVDSSettingInfo.IsDualEdge = TRUE;
            pBIOSInfo->LVDSSettingInfo.IsDithering = FALSE;
            break;            
         case 0x11:  
            pBIOSInfo->LVDSSettingInfo.PanelSizeID = VIA_PANEL10X6;
            pBIOSInfo->LVDSSettingInfo.IsDualEdge = FALSE;
            pBIOSInfo->LVDSSettingInfo.IsDithering = TRUE;
            break;  
        default:
            return FALSE;
            break;
    }
    return TRUE;

}

void VIAPreInit_LVDS(VIABIOSInfoPtr pBIOSInfo)
{
    int tmp;
            
    if (pBIOSInfo->LVDSSettingInfo.PanelSizeID == VIA_PANEL_INVALID)
    {
        tmp = read_reg(VIACR, CR3F) & 0x0F;
        VIASetLCDPanelInfo(pBIOSInfo, tmp);
        
        DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "Get PanelID From Scratch Pad is %d\n", 
            pBIOSInfo->LVDSSettingInfo.PanelSizeID));
    }

    VIAGetPanelSizeByIndex(pBIOSInfo->LVDSSettingInfo.PanelSizeID, &(pBIOSInfo->LVDSSettingInfo.PanelSizeX), &(pBIOSInfo->LVDSSettingInfo.PanelSizeY));

    /* About LCD2, we just set the same panel info as LCD1 now: */
    pBIOSInfo->LVDSSettingInfo2.PanelSizeID = pBIOSInfo->LVDSSettingInfo.PanelSizeID;
    pBIOSInfo->LVDSSettingInfo2.IsDualEdge = pBIOSInfo->LVDSSettingInfo.IsDualEdge;
    pBIOSInfo->LVDSSettingInfo2.IsDithering = pBIOSInfo->LVDSSettingInfo.IsDithering;
    pBIOSInfo->LVDSSettingInfo2.PanelSizeX = pBIOSInfo->LVDSSettingInfo.PanelSizeX;
    pBIOSInfo->LVDSSettingInfo2.PanelSizeY = pBIOSInfo->LVDSSettingInfo.PanelSizeY;
    pBIOSInfo->LVDSSettingInfo2.ChipID = VIA_NONETRANSMITTER;
}

unsigned char VIAGetDeviceDetectLCD(VIABIOSInfoPtr pBIOSInfo)
{
    unsigned char   ret = 0x0;

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIAGetDeviceDetectLCD\n"));
    
    if (pBIOSInfo->LVDSSettingInfo.ChipID)        
    {
        pBIOSInfo->LCDAttach = TRUE;
        ret = VIA_DEVICE_LCD;
    }

    if (pBIOSInfo->LVDSSettingInfo2.ChipID)
    {
        pBIOSInfo->LCDAttach = TRUE;
        ret |= VIA_DEVICE_LCD2;
    }

    return ret;
}

struct display_timing lcd_centering_timging(struct display_timing mode_crt_reg, struct display_timing panel_crt_reg)
{
    struct display_timing crt_reg;

    crt_reg.hor_total = panel_crt_reg.hor_total;
    crt_reg.hor_addr = mode_crt_reg.hor_addr;
    crt_reg.hor_blank_start = (panel_crt_reg.hor_addr - mode_crt_reg.hor_addr)/2 + crt_reg.hor_addr;
    crt_reg.hor_blank_end = panel_crt_reg.hor_blank_end;
    crt_reg.hor_sync_start = (panel_crt_reg.hor_sync_start - panel_crt_reg.hor_blank_start) + crt_reg.hor_blank_start;
    crt_reg.hor_sync_end = panel_crt_reg.hor_sync_end;

    crt_reg.ver_total = panel_crt_reg.ver_total;
    crt_reg.ver_addr = mode_crt_reg.ver_addr;
    crt_reg.ver_blank_start = (panel_crt_reg.ver_addr - mode_crt_reg.ver_addr)/2 + crt_reg.ver_addr;
    crt_reg.ver_blank_end = panel_crt_reg.ver_blank_end;
    crt_reg.ver_sync_start = (panel_crt_reg.ver_sync_start - panel_crt_reg.ver_blank_start) + crt_reg.ver_blank_start;
    crt_reg.ver_sync_end = panel_crt_reg.ver_sync_end;

    return crt_reg;
}

void
load_crtc_shadow_timing(VIABIOSInfoPtr pBIOSInfo, struct display_timing mode_timing, struct display_timing panel_timing)
{
    struct io_register  *reg = NULL;
    int                 i;
    int                 load_reg_Num = 0;
    int                 reg_value = 0;

    if(!(pBIOSInfo->LVDSSettingInfo.Center))    /* Expansion */
    {
       for(i=12;i<20;i++)
       {
          switch(i)
          {
           case H_TOTAL_SHADOW_INDEX:
                reg_value = IGA2_HOR_TOTAL_SHADOW_FORMULA(panel_timing.hor_total);
                load_reg_Num = iga2_shadow_crtc_reg.hor_total_shadow.reg_num;
                reg = iga2_shadow_crtc_reg.hor_total_shadow.reg;
                break;
           case H_BLANK_END_SHADOW_INDEX:
                reg_value = IGA2_HOR_BLANK_END_SHADOW_FORMULA(panel_timing.hor_blank_start, panel_timing.hor_blank_end);
                load_reg_Num = iga2_shadow_crtc_reg.hor_blank_end_shadow.reg_num;
                reg = iga2_shadow_crtc_reg.hor_blank_end_shadow.reg;
                break;
           case V_TOTAL_SHADOW_INDEX:
                reg_value = IGA2_VER_TOTAL_SHADOW_FORMULA(panel_timing.ver_total);
                load_reg_Num = iga2_shadow_crtc_reg.ver_total_shadow.reg_num;
                reg = iga2_shadow_crtc_reg.ver_total_shadow.reg;
                break;
           case V_ADDR_SHADOW_INDEX:
                reg_value = IGA2_VER_ADDR_SHADOW_FORMULA(panel_timing.ver_addr);
                load_reg_Num = iga2_shadow_crtc_reg.ver_addr_shadow.reg_num;
                reg = iga2_shadow_crtc_reg.ver_addr_shadow.reg;
                break;
           case V_BLANK_SATRT_SHADOW_INDEX:
                reg_value = IGA2_VER_BLANK_START_SHADOW_FORMULA(panel_timing.ver_blank_start);
                load_reg_Num = iga2_shadow_crtc_reg.ver_blank_start_shadow.reg_num;
                reg = iga2_shadow_crtc_reg.ver_blank_start_shadow.reg;
                break;
           case V_BLANK_END_SHADOW_INDEX:
                reg_value = IGA2_VER_BLANK_END_SHADOW_FORMULA(panel_timing.ver_blank_start, panel_timing.ver_blank_end);
                load_reg_Num = iga2_shadow_crtc_reg.ver_blank_end_shadow.reg_num;
                reg = iga2_shadow_crtc_reg.ver_blank_end_shadow.reg;
                break;
           case V_SYNC_SATRT_SHADOW_INDEX:
                reg_value = IGA2_VER_SYNC_START_SHADOW_FORMULA(panel_timing.ver_sync_start);
                load_reg_Num = iga2_shadow_crtc_reg.ver_sync_start_shadow.reg_num;
                reg = iga2_shadow_crtc_reg.ver_sync_start_shadow.reg;
                break;
           case V_SYNC_END_SHADOW_INDEX:
                reg_value = IGA2_VER_SYNC_END_SHADOW_FORMULA(panel_timing.ver_sync_start, panel_timing.ver_sync_end);
                load_reg_Num = iga2_shadow_crtc_reg.ver_sync_end_shadow.reg_num;
                reg = iga2_shadow_crtc_reg.ver_sync_end_shadow.reg;
                break;
          }
          load_reg(reg_value, load_reg_Num, reg, VIACR);
       }
    }
    else    /* Centering */
    {
       for(i=12;i<20;i++)
       {
          switch(i)
          {
           case H_TOTAL_SHADOW_INDEX:
                reg_value = IGA2_HOR_TOTAL_SHADOW_FORMULA(panel_timing.hor_total);
                load_reg_Num = iga2_shadow_crtc_reg.hor_total_shadow.reg_num;
                reg = iga2_shadow_crtc_reg.hor_total_shadow.reg;
                break;
           case H_BLANK_END_SHADOW_INDEX:
                reg_value = IGA2_HOR_BLANK_END_SHADOW_FORMULA(panel_timing.hor_blank_start, panel_timing.hor_blank_end);
                load_reg_Num = iga2_shadow_crtc_reg.hor_blank_end_shadow.reg_num;
                reg = iga2_shadow_crtc_reg.hor_blank_end_shadow.reg;
                break;
           case V_TOTAL_SHADOW_INDEX:
                reg_value = IGA2_VER_TOTAL_SHADOW_FORMULA(panel_timing.ver_total);
                load_reg_Num = iga2_shadow_crtc_reg.ver_total_shadow.reg_num;
                reg = iga2_shadow_crtc_reg.ver_total_shadow.reg;
                break;
           case V_ADDR_SHADOW_INDEX:
                reg_value = IGA2_VER_ADDR_SHADOW_FORMULA(mode_timing.ver_addr);
                load_reg_Num = iga2_shadow_crtc_reg.ver_addr_shadow.reg_num;
                reg = iga2_shadow_crtc_reg.ver_addr_shadow.reg;
                break;
           case V_BLANK_SATRT_SHADOW_INDEX:
                reg_value = IGA2_VER_BLANK_START_SHADOW_FORMULA(mode_timing.ver_blank_start);
                load_reg_Num = iga2_shadow_crtc_reg.ver_blank_start_shadow.reg_num;
                reg = iga2_shadow_crtc_reg.ver_blank_start_shadow.reg;
                break;
           case V_BLANK_END_SHADOW_INDEX:
                reg_value = IGA2_VER_BLANK_END_SHADOW_FORMULA(panel_timing.ver_blank_start, panel_timing.ver_blank_end);
                load_reg_Num = iga2_shadow_crtc_reg.ver_blank_end_shadow.reg_num;
                reg = iga2_shadow_crtc_reg.ver_blank_end_shadow.reg;
                break;
           case V_SYNC_SATRT_SHADOW_INDEX:
                reg_value = IGA2_VER_SYNC_START_SHADOW_FORMULA((panel_timing.ver_sync_start - panel_timing.ver_blank_start) + (panel_timing.ver_addr - mode_timing.ver_addr)/2 + mode_timing.ver_addr);
                load_reg_Num = iga2_shadow_crtc_reg.ver_sync_start_shadow.reg_num;
                reg = iga2_shadow_crtc_reg.ver_sync_start_shadow.reg;
                break;
           case V_SYNC_END_SHADOW_INDEX:
                reg_value = IGA2_VER_SYNC_END_SHADOW_FORMULA((panel_timing.ver_sync_start - panel_timing.ver_blank_start) + (panel_timing.ver_addr - mode_timing.ver_addr)/2 + mode_timing.ver_addr, panel_timing.ver_sync_end);
                load_reg_Num = iga2_shadow_crtc_reg.ver_sync_end_shadow.reg_num;
                reg = iga2_shadow_crtc_reg.ver_sync_end_shadow.reg;
                break;
          }
          load_reg(reg_value, load_reg_Num, reg, VIACR);
       }
    }
}



void VIALCDPatchSkew(VIABIOSInfoPtr pBIOSInfo, LVDSSETTINGINFOPTR pLVDSSettingInfo)
{
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIALCDPatchSkew: DIPort=%d\n", pLVDSSettingInfo->DIPort));

    /* STChen [2006.11.28]: Load graphic chip and lvds's DPA setting. */     
    VIALoadDPASetting_Gfx(pBIOSInfo, pLVDSSettingInfo->ChipID,pLVDSSettingInfo->DIPort, VIAGetClkRangeIndex(pLVDSSettingInfo->VClk));    
    VIALoadDPASetting_LVDS(pBIOSInfo, pLVDSSettingInfo);

    /* RainLin [2006.10.18]: Replace the default patch if get user DPA settings from xorg: */
    if (pBIOSInfo->UserDPASettingInfo.HasUserSetting_Gfx)
    {        
        VIASetDPA_Gfx(pLVDSSettingInfo->DIPort, &(pBIOSInfo->UserDPASettingInfo.GfxDPA));        
    }

    if ((pLVDSSettingInfo->ChipID == VIA_VT1636) && pBIOSInfo->UserDPASettingInfo.HasUserSetting_VT1636)
    {
        VIASetDPA_VT1636(pBIOSInfo, pLVDSSettingInfo, &(pBIOSInfo->UserDPASettingInfo.VT1636DPA));
    }
}



void VIALoadScalingFactorForP4M900(VIABIOSInfoPtr pBIOSInfo, int set_hres, int set_vres, int panel_hres, int panel_vres)
{    
    int     h_scaling_factor;
    int     v_scaling_factor;
    CARD8   cra2 = 0;
    CARD8   cr77 = 0;
    CARD8   cr78 = 0;
    CARD8   cr79 = 0;
    CARD8   cr9f = 0;
    
    /* Check if expansion for horizontal */
    if(set_hres < panel_hres)
    {
        /* Load Horizontal Scaling Factor */
    
        /* For VIA_K8M800 or later chipsets. */
        h_scaling_factor = K800_LCD_HOR_SCF_FORMULA(set_hres, panel_hres);
        cr9f = h_scaling_factor & 0x0003; /* HSCaleFactor[1:0] at CR9F[1:0] */        
        cr77 = (h_scaling_factor & 0x03FC)>>2; /* HSCaleFactor[9:2] at CR77[7:0] */        
        cr79 = (h_scaling_factor & 0x0C00)>>10; /* HSCaleFactor[11:10] at CR79[5:4] */
        cr79 <<= 4;
        
        /* Horizontal scaling enabled */
        cra2 = 0xC0;
        
        DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "Horizontal Scaling value = %d\n", h_scaling_factor));
    }
    else
    {
        /* Horizontal scaling disabled */
        cra2 = 0x00;        
    }

    
    /* Check if expansion for vertical */
    if(set_vres < panel_vres)
    {
        /* Load Vertical Scaling Factor */
        
        /* For VIA_K8M800 or later chipsets. */
        v_scaling_factor = K800_LCD_VER_SCF_FORMULA(set_vres, panel_vres);
        
        /* Vertical scaling enabled */
        cra2 |= 0x08;
        cr79 |= ((v_scaling_factor & 0x0001)<<3); /* VSCaleFactor[0] at CR79[3] */                
        cr78 |= (v_scaling_factor & 0x01FE) >> 1; /* VSCaleFactor[8:1] at CR78[7:0] */        
        cr79 |= ((v_scaling_factor & 0x0600) >> 9) << 6; /* VSCaleFactor[10:9] at CR79[7:6] */
        
        DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "Vertical Scaling value = %d\n", v_scaling_factor));
    }
    else
    {
        /* Vertical scaling disabled */
        cra2 |= 0x00;      
    }

    write_reg_mask(CRA2, VIACR, cra2, BIT3+BIT6+BIT7);
    write_reg_mask(CR77, VIACR, cr77, 0xFF);
    write_reg_mask(CR78, VIACR, cr78, 0xFF);
    write_reg_mask(CR79, VIACR, cr79, 0xF8);
    write_reg_mask(CR9F, VIACR, cr9f, BIT0+BIT1);
    
}



