/*
 * Copyright 1998-2007 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2007 S3 Graphics, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#ifndef _VIA_MODE_H_
#define _VIA_MODE_H_ 1

/* standard VGA IO port */
#define VIARMisc    0x3CC
#define VIAWMisc    0x3C2
#define VIAStatus   0x3DA
#define VIACR       0x3D4
#define VIASR       0x3C4
#define VIAGR       0x3CE
#define VIAAR       0x3C0

#define StdCR       0x19
#define StdSR       0x04
#define StdGR       0x09
#define StdAR       0x14

#define PatchCR     11

/* Define Return Value */
#define FAIL        -1
#define OK          1

/* Define Color Depth  */
#define MODE_8BPP   1
#define MODE_16BPP  2
#define MODE_32BPP  4

/* PLL Setting value for CLK */
#define CLK_31M     0xC3B5
#define CLK_36M     0xC5E5
#define CLK_49M     0x8653
#define CLK_56M     0x47B7
#define CLK_78M     0x4321

/* Video Mode Index
#define M640x480	0x00
#define M720x480	0x01
#define M720x576	0x02
#define M800x600	0x03
#define M800x480	0x04
#define M848x480	0x05
#define M852x480	0x06
#define M1024x512	0x07
#define M1024x576	0x08
#define M1024x768	0x09
#define M1152x864	0x0A
#define M1280x768	0x0B
#define M1280x800	0x0C
#define M1280x960	0x0D
#define M1280x1024	0x0E
#define M1368x768	0x0F
#define M1440x1050	0x10
#define M1600x1200	0x11
#define M1920x1440	0x12
#define M1280x720	0x13
#define M1920x1080	0x14
#define M1400x1050	0x15
*/
struct io_reg {
    int     port;
    CARD8   index;
    CARD8   mask;
    CARD8   value;
};

struct patch_table {
    int     mode_index;
    int     table_length;
    struct  io_reg *io_reg_table;
};

struct crt_mode_table {
    int     refresh_rate;
    CARD32  clk;
    int     h_sync_polarity;
    int     v_sync_polarity;
    struct  display_timing crtc;
};

struct VPITTable {
    CARD8   Misc;
    CARD8   SR[StdSR];
    CARD8   GR[StdGR];
    CARD8   AR[StdAR];
};

struct VideoModeTable {
    int     ModeIndex;
    struct  crt_mode_table *crtc;
    int     mode_array;
};

extern unsigned short BIOSVer;
extern char BIOSDate[];
extern unsigned short NumModes;
extern int NumPowerOn;
extern int NumPowerOff;
extern VIAModeEntry Modes[];

extern VIABIOSRegTableRec commExtTable;
extern VIABIOSRegTableRec stdModeExtTable;
extern VIALCDPowerSeqRec powerOn[];
extern VIALCDPowerSeqRec powerOff[];
extern VIALCDModeFixRec modeFix;

extern struct io_reg CN400_ModeXregs[];
extern struct io_reg CN900_ModeXregs[];
extern struct io_reg CN900_ModeXregs_FOR_MAMM[];
extern struct io_reg KM400_ModeXregs[];
extern struct io_reg CX700_ModeXregs[];
extern struct io_reg CLE266_ModeXregs[];
extern struct patch_table res_patch_table[];
extern struct VPITTable VPIT;
extern struct VideoModeTable CLE266Modes[];

#endif  /* _VIA_MODE_H_ */
