/*
 * Copyright 1998-2007 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2007 S3 Graphics, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#include "via_driver.h"
#include "via_regs.h"
#include "via_rotate.h"

#if 0
static unsigned int GetReciprocal(int dwSourcePitch);
#endif

/*
*   STChen[2006.12.20]: If we compile a file with flag "-O2", it could cause strict-aliasing.
*   Strict-aliasing: Dereferencing pointer to object of different types will
*   never refer to the same memory location. ex: (unsigned int*)(&frecptial)
*   Note: In order to avoid above warning, we should use union to solve this
*   issue. 
*/
union NewFloat
{
    float   OriginalType;
    CARD32  IntegerType;
};


#if 0
/*
 * Get reciprocal of source pitch. ex: 1/sp = 0.x, if sp=256d, x=004000h
 */ 
static unsigned int
GetReciprocal(int dwSourcePitch)
{    
    CARD32 dwexp = 0;
    CARD32 dwtep = 0;
    CARD32 dwRecptial = 0;
    float frecptial = 0000.0f;               
    union NewFloat TmpFloat;

    frecptial = (float)dwSourcePitch;
    frecptial = 1.0f / frecptial;
    
    TmpFloat.OriginalType = frecptial;
    dwRecptial = TmpFloat.IntegerType;

    /* Get 23 bits Mantissa */     
    dwtep = dwRecptial & 0x7FFFFF;               

    /*  Default bit 23 is one 0-23, total 24 bits   */
    dwtep |= 0x800000;

    /*  shift bits 23 of Mantissa to bit 31 (23 -0) to (31 - 8) */
    dwtep <<= 8;        

    /* Get 8 bits exp */    
    dwexp = (dwRecptial & 0x7F800000)>>23;

    /* Sub Bias 127 */
    dwexp -= 127;

    /* dwexp always be negative becausse the range of result is 1 - 0  */
    dwexp *= -1;

    /* the virtual value "1" had be shift to bit 31 (the first bit of Mantissa)  */
    dwexp--;

    /* Shift Mantissa according dwexp   */    
    dwtep >>= dwexp;    

    /* Shift bit 31 of Mantissa to bit 21 */
    /* get 0-21, total 22 bits Mantissa*/
    dwtep >>= 10;        

    return ((CARD32)dwtep);
}
#endif

/*
*   STChen[2006.12.28]:
*   Enable 2D Engine rotate function to acceleration 2D function.
*/
void Enable2DEngineRotate(ScrnInfoPtr pScrn, CARD32* pGEMode, CARD32* pCommand)
{
    CARD32  dwWidth = pScrn->virtualX;  /* This value is before rotate. */
    CARD32  dwHeight = pScrn->virtualY; /* This value is before rotate. */    

    VIAPtr  pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo;
    pBIOSInfo = pVia->pBIOSInfo;    

    dwWidth = (dwWidth + 7) & ~7;

    /*  STChen[2007.1.12]: Need to be fixed.
    *   when we rotate 1400x1050 8bpp mode into 90 or 270 degree. 
    *   The screen is fault because 1050 is not a correct value for rotate pitch.
    *   After my experiment, I found that 1052 is a suitable value.
    *   I can't explain it but I still apply this value to the rotate pitch.
    *   So this issue need to be solved in the future.
    */
    if ((pScrn->virtualX == 1400) && (pScrn->virtualY == 1050) &&
        (pScrn->bitsPerPixel == 8) &&
        ((pBIOSInfo->HWRotate == VIA_ROTATE_DEGREE_90) ||
         (pBIOSInfo->HWRotate == VIA_ROTATE_DEGREE_270)))
    {
        dwHeight += 2;
    }
    
    /*switch (pBIOSInfo->HWRotate)
    {
        case VIA_ROTATE_DEGREE_90:
            RotateX = dwHeight;
            RotateY = dwWidth;
            RotateDegree = VIA_GEM_ROTATE_90;            
            break;
            
        case VIA_ROTATE_DEGREE_180:
            RotateX = dwWidth;
            RotateY = dwHeight;
            RotateDegree = VIA_GEM_ROTATE_180;
            break;
            
        case VIA_ROTATE_DEGREE_270:
            RotateX = dwHeight;
            RotateY = dwWidth;
            RotateDegree = VIA_GEM_ROTATE_270;            
            break;                    
    }*/
    /*
    if (pVia->IsHWRotateEnabled)
    {*/
        /* Modify GEMode and Command according to rotate degree. */
       /* *pGEMode |= RotateDegree;
        *pCommand |= VIA_GEC_ROT;
*/
        /* Set resolution of rotation surface. */
  /*      VIASETREG(VIA_REG_ROTSRC, ((RotateY-1<<16))|(RotateX-1) );
        VIASETREG(VIA_REG_ROTDST, ((RotateY-1<<16))|(RotateX-1) );        
    }*/
}



/*
*   Only some chipsets have direct access window.
*/
Bool IsSupportDirectAccessWindow(int ChipID)
{
    int IsSupport = FALSE;
    
    switch (ChipID)
    {
        case VIA_PM800:
        case VIA_CX700:
        case VIA_P4M890:
        case VIA_P4M900:
            IsSupport = TRUE;
            break;
    }

    return IsSupport;
}


/*
*   Only some chipsets have 2D rotation engine.
*/
Bool IsSupport2DRotateEngine(int ChipID)
{
    int IsSupport = FALSE;
    
    switch (ChipID)
    {
        case VIA_PM800:
        case VIA_CX700:        
            IsSupport = TRUE;
            break;
    }

    return IsSupport;
}


/*
*   If the chipset doesn't have the capabilities to rotate, we have to modify the flag.
*   Note: This function must be called before 2D acceleration initialization.
*/
void ProbeRotateCaps(ScrnInfoPtr pScrn)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo;
    pBIOSInfo = pVia->pBIOSInfo;    
    
    if (pVia->IsHWRotateEnabled)
    {
        /* Hardware rotation need direct access window, otherwise we can't rotate the screen. */
        if (!IsSupportDirectAccessWindow(pBIOSInfo->Chipset))
        {
            pVia->IsHWRotateEnabled = FALSE;
        }

        /* If the chipset doesn't support rotate engine, we can't turn on 2D acceleration. */
        if (!IsSupport2DRotateEngine(pBIOSInfo->Chipset))
        {
            pVia->NoAccel = TRUE;
        }
    }

}
 
