/*
 * Copyright 1998-2007 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2007 S3 Graphics, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#ifndef _VIA_TBLINTEGRATEDTV_H_
#define _VIA_TBLINTEGRATEDTV_H_ 1

#define INTEGRATED_TVCLK_27MHZ     0x01865884
#define INTEGRATED_TVCLK_54MHZ     0x00C65884
#define INTEGRATED_TVCLK_74_25MHZ  0x01043084
#define INTEGRATED_TVCLK_148_5MHZ  0x00843084

#define ITV_OFFSET_MAX  0xF0

/***************************/
/*  Integrated TV Table    */
/***************************/

struct integrated_tv_mode_table {
    int              mode_index;
    unsigned long    *std_ntsc;
    unsigned long    *std_pal;
    unsigned long    *std_720p;
    unsigned long    *std_1080i;
};

struct integrated_tv_hdtv_patch_table {
    int             mode_index;
    unsigned short  *std_480p;
    unsigned short  *std_576p;
    unsigned short  *std_720p;
    unsigned short  *std_1080i;
};

unsigned short ITV_INIT_TABLE_OFFSET[] = {
    0x0000,
    0x001C,
    0x00DC,
    0xFFFF  /*End*/
};
 
unsigned long ITV_INIT_TABLE_VALUE[] = {
    0x40412000,     /* 00-03 */
    0x40000000,     /* 1C-1F */

    /* TVCLK PLL. */
    0xA0000000,     /* DC-DF */
};

unsigned short ITV_MODE_TABLE_OFFSET[] = {
    0x0014,
    0x0018,
    0x0020,
    0x0024,
    0x0028,
    0x0034,
    0x0038,
    0x003C,
    0x0040,
    0x0044,
    0x0048,
    0x004C,
    0x0050,
    0x0054,
    0x0058,
    0x005C,
    0x0060,
    0x00E0,
    0xFFFF  /*End*/
};

/**************************************************************************************/
/************************************* NTSC *******************************************/
/**************************************************************************************/
unsigned long ITV_MODE_TABLE_NTSC_640x480[] = {
    /* Src Buffer Line Pitch. */    
    0x00770500,                 /* 14-17 */

    /* Src Image Size and Position. */
    0x09D3F9DF,                 /* 18-1B */
    0x027F01DF,                 /* 20-23 */

    /* Scaling Factor and De-Flicker. */
    0x63EB48FA,                 /* 24-27 */        

    /* Dst Image Size. */
    0x141461AA,                 /* 28-2B */

    /* Dst Buffer Line Pitch & DQWord. */
    0x008E0500,                 /* 34-37 */

    /* Encoder Control. */
    0xC000A214,                 /* 38-3B */
    
    /* Encoder CRTC. */
    0x427F06B3,                 /* 3C-3F */
    0x00F400B1,                 /* 40-43 */
    0x06910100,                 /* 44-47 */
    0x01000000,                 /* 48-4B */
    0x088680C0,                 /* 4C-4F */
    0x488F08C8,                 /* 50-53 */
    
    /* Luma Data */
    0x0A9AA886,                 /* 54-57 */

    /* FSCI */
    0x21F07C16,                 /* 58-5B */

    /* Burst and Sync Shape */
    0x00F01010,                 /* 5C-5F */

    /* Hue and Sub-Carrier */
    0x00000000,                 /* 60-63 */

    /* TVCLK PLL */
    INTEGRATED_TVCLK_27MHZ      /* E0-E3 */
};

unsigned long ITV_CRTC_TABLE_NTSC_640x480[] = {
    858,    /* H_Total */
    640,    /* H_Display_End */
    640,    /* H_Blanking_Start */
    218,    /* H_Blanking_End */
    688,    /* H_Sync_Start */
    56,     /* H_Sync_End */
    525,    /* V_Total */
    480,    /* V_Display_End */
    480,    /* V_Blanking_Start */
    45,     /* V_Blanking_End */
    488,    /* V_Sync_Start */
    7       /* V_Sync_End */
};


unsigned long ITV_MODE_TABLE_NTSC_720x480[] = {
    /* Src Buffer Line Pitch. */    
    0x007705C0,                 /* 14-17 */

    /* Src Image Size and Position. */
    0x09D679DF,                 /* 18-1B */
    0x02CF01DF,                 /* 20-23 */

    /* Scaling Factor and De-Flicker. */
    0x646C4903,                 /* 24-27 */        

    /* Dst Image Size. */
    0x169451A9,                 /* 28-2B */

    /* Dst Buffer Line Pitch & DQWord. */
    0x008E05C0,                 /* 34-37 */

    /* Encoder Control. */
    0xC000A214,                 /* 38-3B */
    
    /* Encoder CRTC. */
    0x427F06B3,                 /* 3C-3F */
    0x00F400B1,                 /* 40-43 */
    0x06910100,                 /* 44-47 */
    0x01A00000,                 /* 48-4B */
    0x088680C0,                 /* 4C-4F */
    0x488F08C8,                 /* 50-53 */
    
    /* Luma Data */
    0x0A9AA886,                 /* 54-57 */

    /* FSCI */
    0x21F07C16,                 /* 58-5B */

    /* Burst and Sync Shape */
    0x00F01010,                 /* 5C-5F */

    /* Hue and Sub-Carrier */
    0x00000000,                 /* 60-63 */

    /* TVCLK PLL */
    INTEGRATED_TVCLK_27MHZ      /* E0-E3 */
};

unsigned long ITV_CRTC_TABLE_NTSC_720x480[] = {
    858,    /* H_Total */
    720,    /* H_Display_End */
    720,    /* H_Blanking_Start */
    138,    /* H_Blanking_End */
    736,    /* H_Sync_Start */
    72,     /* H_Sync_End */
    525,    /* V_Total */
    480,    /* V_Display_End */
    480,    /* V_Blanking_Start */
    45,     /* V_Blanking_End */
    481,    /* V_Sync_Start */
    3       /* V_Sync_End */
};


unsigned long ITV_MODE_TABLE_NTSC_800x600[] = {
    /* Src Buffer Line Pitch. */    
    0x00770640,                 /* 14-17 */

    /* Src Image Size and Position. */
    0x09D8FA57,                 /* 18-1B */
    0x031F0257,                 /* 20-23 */

    /* Scaling Factor and De-Flicker. */
    0x64E84B3D,                 /* 24-27 */        

    /* Dst Image Size. */
    0x191459AA,                 /* 28-2B */

    /* Dst Buffer Line Pitch & DQWord. */
    0x008E0640,                 /* 34-37 */

    /* Encoder Control. */
    0xC000A214,                 /* 38-3B */
    
    /* Encoder CRTC. */
    0x427F06B3,                 /* 3C-3F */
    0x00F400B1,                 /* 40-43 */
    0x06910100,                 /* 44-47 */
    0x01000000,                 /* 48-4B */
    0x088680C0,                 /* 4C-4F */
    0x488F08C8,                 /* 50-53 */

    /* Luma Data */
    0x0A9AA886,                 /* 54-57 */
    
    /* FSCI */
    0x21F07C16,                 /* 58-5B */

    /* Burst and Sync Shape */
    0x00F01010,                 /* 5C-5F */

    /* Hue and Sub-Carrier */
    0x00000000,                 /* 60-63 */

    /* TVCLK PLL */
    INTEGRATED_TVCLK_54MHZ      /* E0-E3 */
};

unsigned long ITV_CRTC_TABLE_NTSC_800x600[] = {
    1092,   /* H_Total */
    800,    /* H_Display_End */
    800,    /* H_Blanking_Start */
    292,    /* H_Blanking_End */
    840,    /* H_Sync_Start */
    136,    /* H_Sync_End */
    825,    /* V_Total */
    600,    /* V_Display_End */
    600,    /* V_Blanking_Start */
    225,    /* V_Blanking_End */
    606,    /* V_Sync_Start */
    15      /* V_Sync_End */
};


unsigned long ITV_MODE_TABLE_NTSC_1024x768[] = {
    /* Src Buffer Line Pitch. */    
    0x00770800,                 /* 14-17 */

    /* Src Image Size and Position. */
    0x09DFFAFF,                 /* 18-1B */
    0x03FF02FF,                 /* 20-23 */

    /* Scaling Factor and De-Flicker. */
    0x66454E58,                 /* 24-27 */        

    /* Dst Image Size. */
    0x201461AA,                 /* 28-2B */

    /* Dst Buffer Line Pitch & DQWord. */
    0x008E0800,                 /* 34-37 */

    /* Encoder Control. */
    0xC000A214,                 /* 38-3B */
    
    /* Encoder CRTC. */
    0x427F06B3,                 /* 3C-3F */
    0x00F400B1,                 /* 40-43 */
    0x06910100,                 /* 44-47 */
    0x01000000,                 /* 48-4B */
    0x088680C0,                 /* 4C-4F */
    0x488F08C8,                 /* 50-53 */

    /* Luma Data */
    0x0A9AA886,                 /* 54-57 */
    
    /* FSCI */
    0x21F07C16,                 /* 58-5B */

    /* Burst and Sync Shape */
    0x00F01010,                 /* 5C-5F */

    /* Hue and Sub-Carrier */
    0x00000000,                 /* 60-63 */

    /* TVCLK PLL */
    INTEGRATED_TVCLK_54MHZ      /* E0-E3 */
};

unsigned long ITV_CRTC_TABLE_NTSC_1024x768[] = {
    1092,   /* H_Total */
    1024,   /* H_Display_End */
    1024,   /* H_Blanking_Start */
    68,     /* H_Blanking_End */
    1032,   /* H_Sync_Start */
    32,     /* H_Sync_End */
    825,    /* V_Total */
    768,    /* V_Display_End */
    768,    /* V_Blanking_Start */
    57,     /* V_Blanking_End */
    800,    /* V_Sync_Start */
    6       /* V_Sync_End */
};
        
/**************************************************************************************/
/************************************* PAL ********************************************/
/**************************************************************************************/

unsigned long ITV_MODE_TABLE_PAL_640x480[] = {
    /* Src Buffer Line Pitch. */    
    0x00770500,                 /* 14-17 */

    /* Src Image Size and Position. */
    0x09D3F9DF,                 /* 18-1B */
    0x227F01DF,                 /* 20-23 */

    /* Scaling Factor and De-Flicker. */
    0x6405478E,                 /* 24-27 */        

    /* Dst Image Size. */
    0x1413D9FA,                 /* 28-2B */

    /* Dst Buffer Line Pitch & DQWord. */
    0x008E0500,                 /* 34-37 */

    /* Encoder Control. */
    0xC048A214,                 /* 38-3B */
    
    /* Encoder CRTC. */
    0x427F06BF,                 /* 3C-3F */
    0x00EF00B2,                 /* 40-43 */
    0x06990120,                 /* 44-47 */
    0x01200010,                 /* 48-4B */
    0x0A0680C0,                 /* 4C-4F */
    0x489608D3,                 /* 50-53 */
    
    /* Luma Data */
    0x001C0090,                 /* 54-57 */

    /* FSCI */
    0x2A08D01C,                 /* 58-5B */

    /* Burst and Sync Shape */
    0x01081011,                 /* 5C-5F */

    /* Hue and Sub-Carrier */
    0x00120000,                 /* 60-63 */

    /* TVCLK PLL */
    INTEGRATED_TVCLK_27MHZ      /* E0-E3 */
};

unsigned long ITV_CRTC_TABLE_PAL_640x480[] = {
    864,    /* H_Total */
    640,    /* H_Display_End */
    640,    /* H_Blanking_Start */
    224,    /* H_Blanking_End */
    704,    /* H_Sync_Start */
    56,     /* H_Sync_End */
    625,    /* V_Total */
    480,    /* V_Display_End */
    480,    /* V_Blanking_Start */
    145,    /* V_Blanking_End */
    500,    /* V_Sync_Start */
    6       /* V_Sync_End */
};


unsigned long ITV_MODE_TABLE_PAL_720x576[] = {
    /* Src Buffer Line Pitch. */    
    0x007705C0,                 /* 14-17 */

    /* Src Image Size and Position. */
    0x09D67A3F,                 /* 18-1B */
    0x22CF023F,                 /* 20-23 */

    /* Scaling Factor and De-Flicker. */
    0x64834916,                 /* 24-27 */        

    /* Dst Image Size. */
    0x1693E9FA,                 /* 28-2B */

    /* Dst Buffer Line Pitch & DQWord. */
    0x008E05C0,                 /* 34-37 */

    /* Encoder Control. */
    0xC048A214,                 /* 38-3B */
    
    /* Encoder CRTC. */
    0x427F06BF,                 /* 3C-3F */
    0x00EF00B2,                 /* 40-43 */
    0x06990120,                 /* 44-47 */
    0x01200010,                 /* 48-4B */
    0x0A0680C0,                 /* 4C-4F */
    0x489608D3,                 /* 50-53 */
    
    /* Luma Data */
    0x001C0090,                 /* 54-57 */

    /* FSCI */
    0x2A08D01C,                 /* 58-5B */

    /* Burst and Sync Shape */
    0x01081011,                 /* 5C-5F */

    /* Hue and Sub-Carrier */
    0x00120000,                 /* 60-63 */

    /* TVCLK PLL */
    INTEGRATED_TVCLK_27MHZ      /* E0-E3 */
};

unsigned long ITV_CRTC_TABLE_PAL_720x576[] = {
    864,    /* H_Total */
    720,    /* H_Display_End */
    720,    /* H_Blanking_Start */
    144,    /* H_Blanking_End */
    760,    /* H_Sync_Start */
    40,     /* H_Sync_End */
    625,    /* V_Total */
    576,    /* V_Display_End */
    576,    /* V_Blanking_Start */
    49,    /* V_Blanking_End */
    577,    /* V_Sync_Start */
    3       /* V_Sync_End */
};


unsigned long ITV_MODE_TABLE_PAL_800x600[] = {
    /* Src Buffer Line Pitch. */    
    0x00770640,                 /* 14-17 */

    /* Src Image Size and Position. */
    0x09D8FA57,                 /* 18-1B */
    0x231F0257,                 /* 20-23 */

    /* Scaling Factor and De-Flicker. */
    0x6503497A,                 /* 24-27 */        

    /* Dst Image Size. */
    0x1913E9F9,                 /* 28-2B */

    /* Dst Buffer Line Pitch & DQWord. */
    0x008E0640,                 /* 34-37 */

    /* Encoder Control. */
    0xC048A214,                 /* 38-3B */
    
    /* Encoder CRTC. */
    0x427F06BF,                 /* 3C-3F */
    0x00EF00B2,                 /* 40-43 */
    0x06990120,                 /* 44-47 */
    0x01200010,                 /* 48-4B */
    0x0A0680C0,                 /* 4C-4F */
    0x489608D3,                 /* 50-53 */
    
    /* Luma Data */
    0x001C0090,                 /* 54-57 */

    /* FSCI */
    0x2A08D01C,                 /* 58-5B */

    /* Burst and Sync Shape */
    0x01081011,                 /* 5C-5F */

    /* Hue and Sub-Carrier */
    0x00120000,                 /* 60-63 */

    /* TVCLK PLL */
    INTEGRATED_TVCLK_54MHZ      /* E0-E3 */
};

unsigned long ITV_CRTC_TABLE_PAL_800x600[] = {
    1350,   /* H_Total */
    800,    /* H_Display_End */
    800,    /* H_Blanking_Start */
    550,    /* H_Blanking_End */
    816,    /* H_Sync_Start */
    64,     /* H_Sync_End */
    800,    /* V_Total */
    600,    /* V_Display_End */
    600,    /* V_Blanking_Start */
    200,    /* V_Blanking_End */
    606,    /* V_Sync_Start */
    15      /* V_Sync_End */
};


unsigned long ITV_MODE_TABLE_PAL_1024x768[] = {
    /* Src Buffer Line Pitch. */    
    0x00770800,                 /* 14-17 */

    /* Src Image Size and Position. */
    0x09DFFAFF,                 /* 18-1B */
    0x13FF02FF,                 /* 20-23 */

    /* Scaling Factor and De-Flicker. */
    0x66704C2A,                 /* 24-27 */        

    /* Dst Image Size. */
    0x2013D9F8,                 /* 28-2B */

    /* Dst Buffer Line Pitch & DQWord. */
    0x008E0800,                 /* 34-37 */

    /* Encoder Control. */
    0xC048A214,                 /* 38-3B */
    
    /* Encoder CRTC. */
    0x427F06BF,                 /* 3C-3F */
    0x00EF00B2,                 /* 40-43 */
    0x06990120,                 /* 44-47 */
    0x01200010,                 /* 48-4B */
    0x0A0680C0,                 /* 4C-4F */
    0x489608D3,                 /* 50-53 */
    
    /* Luma Data */
    0x001C0090,                 /* 54-57 */

    /* FSCI */
    0x2A08D01C,                 /* 58-5B */

    /* Burst and Sync Shape */
    0x01081011,                 /* 5C-5F */

    /* Hue and Sub-Carrier */
    0x00120000,                 /* 60-63 */

    /* TVCLK PLL */
    INTEGRATED_TVCLK_54MHZ      /* E0-E3 */
};

unsigned long ITV_CRTC_TABLE_PAL_1024x768[] = {
    1200,   /* H_Total */
    1024,   /* H_Display_End */
    1024,   /* H_Blanking_Start */
    176,    /* H_Blanking_End */
    1048,   /* H_Sync_Start */
    16,     /* H_Sync_End */
    900,    /* V_Total */
    768,    /* V_Display_End */
    768,    /* V_Blanking_Start */
    132,    /* V_Blanking_End */
    800,    /* V_Sync_Start */
    6       /* V_Sync_End */
};

/**************************************************************************************/
/************************************* HDTV: 720P *************************************/
/**************************************************************************************/
unsigned long ITV_MODE_TABLE_720P_1280x720[] = {
    /* Src Buffer Line Pitch. */    
    0x00770A00,                 /* 14-17 */

    /* Src Image Size and Position. */
    0x09E7FACF,                 /* 18-1B */
    0x24FF02CF,                 /* 20-23 */

    /* Scaling Factor and De-Flicker. */
    0x045A27FF,                 /* 24-27 */        

    /* Dst Image Size. */
    0x2827FACF,                 /* 28-2B */

    /* Dst Buffer Line Pitch & DQWord. */
    0x00770A00,                 /* 34-37 */

    /* Encoder Control. */
    0xC0E0A214,                 /* 38-3B */

    /* Encoder CRTC. */
    0x42280671,                 /* 3C-3F */
    0x00000000,                 /* 40-43 */
    0x062C0117,                 /* 44-47 */
    0x000C592C,                 /* 48-4B */
    0x0608527D,                 /* 4C-4F */
    0x48CD18CD,                 /* 50-53 */

    /* Luma Data */
    0x009CA891,                 /* 54-57 */

    /* FSCI */
    0x21F02016,                 /* 58-5B */

    /* Burst and Sync Shape */
    0x00F0102F,                 /* 5C-5F */

    /* Hue and Sub-Carrier */
    0x00000000,                 /* 60-63 */

    /* TVCLK PLL */
    INTEGRATED_TVCLK_74_25MHZ   /* E0-E3 */
};

unsigned long ITV_CRTC_TABLE_720P_1280x720[] = {
    1650,   /* H_Total          */
    1280,   /* H_Display_End    */
    1280,   /* H_Blanking_Start */
    370,    /* H_Blanking_End   */
    1296,   /* H_Sync_Start     */
    48,     /* H_Sync_End       */
    750,    /* V_Total          */
    720,    /* V_Display_End    */
    720,    /* V_Blanking_Start */
    30,     /* V_Blanking_End   */
    721,    /* V_Sync_Start     */
    5       /* V_Sync_End       */
};

unsigned long ITV_CRTC_TABLE_720P_640x480[] = {
    900,    /* H_Total          */
    640,    /* H_Display_End    */
    640,    /* H_Blanking_Start */
    260,    /* H_Blanking_End   */
    656,    /* H_Sync_Start     */
    72,     /* H_Sync_End       */
    500,    /* V_Total          */
    480,    /* V_Display_End    */
    480,    /* V_Blanking_Start */
    20,     /* V_Blanking_End   */
    481,    /* V_Sync_Start     */
    2       /* V_Sync_End       */
};

unsigned long ITV_CRTC_TABLE_720P_720x480[] = {
    900,    /* H_Total          */
    720,    /* H_Display_End    */
    720,    /* H_Blanking_Start */
    180,    /* H_Blanking_End   */
    736,    /* H_Sync_Start     */
    72,     /* H_Sync_End       */
    500,    /* V_Total          */
    480,    /* V_Display_End    */
    480,    /* V_Blanking_Start */
    20,     /* V_Blanking_End   */
    481,    /* V_Sync_Start     */
    3       /* V_Sync_End       */
};

unsigned long ITV_CRTC_TABLE_720P_720x576[] = {
    1440,   /* H_Total          */
    720,    /* H_Display_End    */
    720,    /* H_Blanking_Start */
    720,    /* H_Blanking_End   */
    736,    /* H_Sync_Start     */
    80,     /* H_Sync_End       */
    625,    /* V_Total          */
    576,    /* V_Display_End    */
    576,    /* V_Blanking_Start */
    49,     /* V_Blanking_End   */
    577,    /* V_Sync_Start     */
    3       /* V_Sync_End       */
};

unsigned long ITV_CRTC_TABLE_720P_800x600[] = {
    1440,   /* H_Total          */
    800,    /* H_Display_End    */
    800,    /* H_Blanking_Start */
    640,    /* H_Blanking_End   */
    840,    /* H_Sync_Start     */
    136,    /* H_Sync_End       */
    625,    /* V_Total          */
    600,    /* V_Display_End    */
    600,    /* V_Blanking_Start */
    25,     /* V_Blanking_End   */
    606,    /* V_Sync_Start     */
    15      /* V_Sync_End       */
};

unsigned long ITV_CRTC_TABLE_720P_1024x768[] = {
    1250,   /* H_Total          */
    1024,   /* H_Display_End    */
    1024,   /* H_Blanking_Start */
     226,   /* H_Blanking_End   */
    1064,   /* H_Sync_Start     */
     120,   /* H_Sync_End       */
     880,   /* V_Total          */
     768,   /* V_Display_End    */
     768,   /* V_Blanking_Start */
     112,   /* V_Blanking_End   */
     774,   /* V_Sync_Start     */
       3    /* V_Sync_End       */
};

/**************************************************************************************/
/************************************* HDTV: 1080I *************************************/
/**************************************************************************************/
unsigned long ITV_MODE_TABLE_1080I_1920x1080[] = {
    /* Src Buffer Line Pitch. */    
    0x00770F00,                 /* 14-17 */

    /* Src Image Size and Position. */
    0x09FBFC37,                 /* 18-1B */
    0x277F0437,                 /* 20-23 */

    /* Scaling Factor and De-Flicker. */
    0x68002803,                 /* 24-27 */        

    /* Dst Image Size. */
    0x3C3BFC37,                 /* 28-2B */

    /* Dst Buffer Line Pitch & DQWord. */
    0x00770F00,                 /* 34-37 */

    /* Encoder Control. */
    0xC0D0A214,                 /* 38-3B */

    /* Encoder CRTC. */
    0x422C0897,                 /* 3C-3F */
    0x00000000,                 /* 40-43 */
    0x087100EC,                 /* 44-47 */
    0x000840B0,                 /* 48-4B */
    0x0506B1E2,                 /* 4C-4F */
    0x48B208F7,                 /* 50-53 */

    /* Luma Data */
    0x00A0A0A5,                 /* 54-57 */

    /* FSCI */
    0x21F02016,                 /* 58-5B */

    /* Burst and Sync Shape */
    0x01281010,                 /* 5C-5F */

    /* Hue and Sub-Carrier */
    0x00000000,                 /* 60-63 */
    
    /* TVCLK PLL */
    INTEGRATED_TVCLK_148_5MHZ   /* E0-E3 */
};

unsigned long ITV_CRTC_TABLE_1080I_1920x1080[] = {
    2200,   /* H_Total          */ 
    1920,   /* H_Display_End    */
    1920,   /* H_Blanking_Start */
    280,    /* H_Blanking_End   */
    1960,   /* H_Sync_Start     */
    104,    /* H_Sync_End       */
    1125,   /* V_Total          */
    1080,   /* V_Display_End    */
    1080,   /* V_Blanking_Start */
    45,     /* V_Blanking_End   */
    1085,   /* V_Sync_Start     */
    3       /* V_Sync_End       */
};

unsigned long ITV_CRTC_TABLE_1080I_640x480[] = {
    900,    /* H_Total          */
    640,    /* H_Display_End    */
    640,    /* H_Blanking_Start */
    260,    /* H_Blanking_End   */
    656,    /* H_Sync_Start     */
    72,     /* H_Sync_End       */
    500,    /* V_Total          */
    480,    /* V_Display_End    */
    480,    /* V_Blanking_Start */
    20,     /* V_Blanking_End   */
    481,    /* V_Sync_Start     */
    2       /* V_Sync_End       */
};

unsigned long ITV_CRTC_TABLE_1080I_720x480[] = {
    900,    /* H_Total          */
    720,    /* H_Display_End    */
    720,    /* H_Blanking_Start */
    180,    /* H_Blanking_End   */
    736,    /* H_Sync_Start     */
    96,     /* H_Sync_End       */
    500,    /* V_Total          */
    480,    /* V_Display_End    */
    480,    /* V_Blanking_Start */
    20,     /* V_Blanking_End   */
    485,    /* V_Sync_Start     */
    6       /* V_Sync_End       */
};

unsigned long ITV_CRTC_TABLE_1080I_720x576[] = {
    1440,   /* H_Total          */
    720,    /* H_Display_End    */
    720,    /* H_Blanking_Start */
    720,    /* H_Blanking_End   */
    736,    /* H_Sync_Start     */
    96,     /* H_Sync_End       */
    625,    /* V_Total          */
    576,    /* V_Display_End    */
    576,    /* V_Blanking_Start */
    49,     /* V_Blanking_End   */
    577,    /* V_Sync_Start     */
    2       /* V_Sync_End       */
};

unsigned long ITV_CRTC_TABLE_1080I_800x600[] = {
    1440,   /* H_Total          */ 
    800,    /* H_Display_End    */
    800,    /* H_Blanking_Start */
    640,    /* H_Blanking_End   */
    824,    /* H_Sync_Start     */
    56,     /* H_Sync_End       */
    625,    /* V_Total          */
    600,    /* V_Display_End    */
    600,    /* V_Blanking_Start */
    25,     /* V_Blanking_End   */
    605,    /* V_Sync_Start     */
    6       /* V_Sync_End       */
};

unsigned long ITV_CRTC_TABLE_1080I_1024x768[] = {
    1250,   /* H_Total          */ 
    1024,   /* H_Display_End    */
    1024,   /* H_Blanking_Start */
    226,    /* H_Blanking_End   */
    1032,   /* H_Sync_Start     */
    64,     /* H_Sync_End       */
    792,    /* V_Total          */
    768,    /* V_Display_End    */
    768,    /* V_Blanking_Start */
    24,     /* V_Blanking_End   */
    770,    /* V_Sync_Start     */
    3       /* V_Sync_End       */
};

unsigned long ITV_NULL_TABLE[] = {
    0xFFFF
};


struct integrated_tv_mode_table ITV_MODE_TABLE[] = {
    {VIA_TVRES_640X480,   ITV_MODE_TABLE_NTSC_640x480,  ITV_MODE_TABLE_PAL_640x480,  ITV_MODE_TABLE_720P_1280x720, ITV_MODE_TABLE_1080I_1920x1080},
    {VIA_TVRES_720X480,   ITV_MODE_TABLE_NTSC_720x480,  ITV_MODE_TABLE_NTSC_720x480, ITV_MODE_TABLE_720P_1280x720, ITV_MODE_TABLE_1080I_1920x1080},
    {VIA_TVRES_720X576,   ITV_MODE_TABLE_PAL_720x576,   ITV_MODE_TABLE_PAL_720x576,  ITV_MODE_TABLE_720P_1280x720, ITV_MODE_TABLE_1080I_1920x1080},
    {VIA_TVRES_800X600,   ITV_MODE_TABLE_NTSC_800x600,  ITV_MODE_TABLE_PAL_800x600,  ITV_MODE_TABLE_720P_1280x720, ITV_MODE_TABLE_1080I_1920x1080},
    {VIA_TVRES_1024X768,  ITV_MODE_TABLE_NTSC_1024x768, ITV_MODE_TABLE_PAL_1024x768, ITV_MODE_TABLE_720P_1280x720, ITV_MODE_TABLE_1080I_1920x1080},
    {VIA_TVRES_1280X720,  ITV_NULL_TABLE,               ITV_NULL_TABLE,              ITV_MODE_TABLE_720P_1280x720, ITV_NULL_TABLE},
    {VIA_TVRES_1920X1080, ITV_NULL_TABLE,               ITV_NULL_TABLE,              ITV_NULL_TABLE,               ITV_MODE_TABLE_1080I_1920x1080}
};


struct integrated_tv_mode_table ITV_CRTC_TABLE[] = {
    {VIA_TVRES_640X480,   ITV_CRTC_TABLE_NTSC_640x480,  ITV_CRTC_TABLE_PAL_640x480,  ITV_CRTC_TABLE_720P_640x480,  ITV_CRTC_TABLE_1080I_640x480},
    {VIA_TVRES_720X480,   ITV_CRTC_TABLE_NTSC_720x480,  ITV_CRTC_TABLE_NTSC_720x480, ITV_CRTC_TABLE_720P_720x480,  ITV_CRTC_TABLE_1080I_720x480},
    {VIA_TVRES_720X576,   ITV_CRTC_TABLE_PAL_720x576,   ITV_CRTC_TABLE_PAL_720x576,  ITV_CRTC_TABLE_720P_720x576,  ITV_CRTC_TABLE_1080I_720x576},
    {VIA_TVRES_800X600,   ITV_CRTC_TABLE_NTSC_800x600,  ITV_CRTC_TABLE_PAL_800x600,  ITV_CRTC_TABLE_720P_800x600,  ITV_CRTC_TABLE_1080I_800x600},
    {VIA_TVRES_1024X768,  ITV_CRTC_TABLE_NTSC_1024x768, ITV_CRTC_TABLE_PAL_1024x768, ITV_CRTC_TABLE_720P_1024x768, ITV_CRTC_TABLE_1080I_1024x768},
    {VIA_TVRES_1280X720,  ITV_NULL_TABLE,               ITV_NULL_TABLE,              ITV_CRTC_TABLE_720P_1280x720, ITV_NULL_TABLE},
    {VIA_TVRES_1920X1080, ITV_NULL_TABLE,               ITV_NULL_TABLE,              ITV_NULL_TABLE,               ITV_CRTC_TABLE_1080I_1920x1080}
};


#define NUM_TOTAL_INTEGRATEDTV_MODE_TABLE ARRAY_SIZE(ITV_MODE_TABLE)

/**************************************************************************************/
/********************************** Patch Table for RGB *******************************/
/**************************************************************************************/
unsigned short ITV_PATCH_TABLE_RGB_NTSC[] = {
    0xE342,     /* BST_ED   */
    0x8D46,     /* VID_ED   */
    0x404A,     /* HSD      */
    0x7F50,     /* VR_GAIN  */
    0x1851,     /* SC_ON_VR */
    0x7C52,     /* UB_GAIN  */
    0x5853,     /* SC_ON_UB */
    0x7254,     /* YG_GAIN  */
    0x165C,     /* SYNC_SP  */
    0x285E,     /* BRST_AMP */
    0x015F,     /* BRST_AMP */
    0xffff
};

unsigned short ITV_PATCH_TABLE_RGB_PAL[] = {
    0xF442,     /* BST_ED       */
    0x1E44,     /* VID_ST       */
    0x004A,     /* HSD          */
    0x9C50,     /* VR_GAIN      */
    0x9852,     /* UB_GAIN      */
    0xA554,     /* YG_GAIN      */
    0xA055,     /* Blank Level  */
    0x285E,     /* BRST_AMP     */
    0xffff
};


/**************************************************************************************/
/******************************** Patch Table for YPbPr *******************************/
/**************************************************************************************/
unsigned short ITV_PATCH_TABLE_YPBPR_NTSC[] = {
    0xE342,     /* BST_ED       */
    0x8D46,     /* VID_ED       */
    0x404A,     /* HSD          */
    0x9750,     /* VR_GAIN      */
    0x9952,     /* UB_GAIN      */
    0x8B54,     /* YG_GAIN      */
    0x0057,     /* Blank Level  */
    0x285E,     /* BRST_AMP     */
    0x015F,     /* BRST_AMP     */
    0x76E8,
    0xffff
};

unsigned short ITV_PATCH_TABLE_YPBPR_PAL[] = {
    0xF442,     /* BST_ED       */
    0x1E44,     /* VID_ST       */
    0x9C46,     /* VID_ED       */
    0x004A,     /* HSD          */
    0x9C50,     /* VR_GAIN      */
    0x9852,     /* UB_GAIN      */
    0x285E,     /* BRST_AMP     */
    0xffff
};



/**************************************************************************************/
/******************************** Patch Table for 480P ********************************/
/**************************************************************************************/
unsigned short ITV_PATCH_TABLE_480P_640x480[] = {
    0xC424, 0x0825, 0xE526, 0x0327,     /* Scaling Factor            */
    0xB528, 0x0129, 0x292A,             /* DST H/V size              */
    0x803A,                             /* TV's Standard             */
    0x0040, 0x0042,                     /* BST_ST, BST_ED            */
    0xFE44, 0x0045, 0x8F46,             /* VID_ST, VID_ED            */
    0x404A,                             /* HSD                       */
    0xCC4C, 0x814D, 0x0F4F,             /* HP, VP, VSD               */
    0x6B50, 0x6B52,                     /* Chroma Ctrl               */
    0x9054, 0xA055, 0x1C56, 0x0057,     /* Luma Ctrl                 */
    0x175C, 0x005D, 0x005E, 0x005F,     /* Burst and Sync Shape Ctrl */
    0x83E2,
    0xFFFF
};

unsigned short ITV_PATCH_TABLE_480P_720x480[] = {
    0xC424, 0x0825, 0x6226, 0x0427,     /* Scaling Factor            */
    0xB528, 0x0129, 0xA92A,             /* DST H/V size              */
    0x803A,                             /* TV's Standard             */
    0x0040, 0x0042,                     /* BST_ST, BST_ED            */
    0xFE44, 0x0045, 0x8F46,             /* VID_ST, VID_ED            */
    0x404A,                             /* HSD                       */
    0xA74C, 0x814D, 0x0F4F,             /* HP, VP, VSD               */
    0x6B50, 0x6B52,                     /* Chroma Ctrl               */
    0x9054, 0xA055, 0x1C56, 0x0057,     /* Luma Ctrl                 */
    0x175C, 0x005D, 0x005E, 0x005F,     /* Burst and Sync Shape Ctrl */
    0x83E2,
    0xFFFF
};

unsigned short ITV_PATCH_TABLE_480P_800x600[] = {
    0xF624, 0x0A25, 0xDF26, 0x0427,     /* Scaling Factor            */
    0xB528, 0x0129, 0x292A,             /* DST H/V size              */
    0x8436,                             /* DST_BUF_LINE              */
    0x803A,                             /* TV's Standard             */
    0x0040, 0x0042,                     /* BST_ST, BST_ED            */
    0xFE44, 0x0045, 0x8F46,             /* VID_ST, VID_ED            */
    0x404A,                             /* HSD                       */
    0xA04C, 0x814D, 0x0F4F,             /* HP, VP, VSD               */
    0x6B50, 0x6B52,                     /* Chroma Ctrl               */
    0x9054, 0xA055, 0x1C56, 0x0057,     /* Luma Ctrl                 */
    0x175C, 0x005D, 0x005E, 0x005F,     /* Burst and Sync Shape Ctrl */
    0xC3E2,
    0xFFFF
};

unsigned short ITV_PATCH_TABLE_480P_1024x768[] = {
    0x0A24, 0x4126, 0x0627,             /* Scaling Factor            */
    0xB528, 0xE129, 0x282A,             /* DST H/V size              */
    0x803A,                             /* TV's Standard             */
    0x0040, 0x0042,                     /* BST_ST, BST_ED            */
    0xFE44, 0x0045, 0x8F46,             /* VID_ST, VID_ED            */
    0x404A,                             /* HSD                       */
    0xD44C, 0x814D, 0x0F4F,             /* HP, VP, VSD               */
    0x6B50, 0x6B52,                     /* Chroma Ctrl               */
    0x9054, 0xA055, 0x1C56, 0x0057,     /* Luma Ctrl                 */
    0x175C, 0x005D, 0x005E, 0x005F,     /* Burst and Sync Shape Ctrl */
    0xC3E2,
    0xFFFF
};


/**************************************************************************************/
/******************************** Patch Table for 576P ********************************/
/**************************************************************************************/
unsigned short ITV_PATCH_TABLE_576P_640x480[] = {
    0x6924, 0xF726, 0x0327,             /* Scaling Factor            */
    0x0528, 0x3A29, 0x282A,             /* DST H/V size              */
    0x883A,                             /* TV's Standard             */
    0xF442,                             /* BST_ST, BST_ED            */
    0x2144, 0x9C46,                     /* VID_ST, VID_ED            */
    0x0048,                             /* BRD_ST                    */
    0x104C, 0x824D, 0x124F,             /* HP, VP, VSD               */
    0x9050, 0x9052,                     /* Chroma Ctrl               */
    0x8F54, 0xA055,                     /* Luma Ctrl                 */
    0x105C, 0x005E, 0x005F,             /* Burst and Sync Shape Ctrl */
    0x83E2,
    0xFFFF
};

unsigned short ITV_PATCH_TABLE_576P_720x576[] = {
    0xE524, 0x0825, 0x7826, 0x0427,     /* Scaling Factor            */
    0x0528, 0x3A29, 0xA82A,             /* DST H/V size              */
    0x883A,                             /* TV's Standard             */
    0xF442,                             /* BST_ST, BST_ED            */
    0x2144, 0x9C46,                     /* VID_ST, VID_ED            */
    0x0048,                             /* BRD_ST                    */
    0x104C, 0x824D, 0x124F,             /* HP, VP, VSD               */
    0x9050, 0x9052,                     /* Chroma Ctrl               */
    0x8F54, 0xA055,                     /* Luma Ctrl                 */
    0x105C, 0x005E, 0x005F,             /* Burst and Sync Shape Ctrl */
    0x83E2,
    0xFFFF
};

unsigned short ITV_PATCH_TABLE_576P_800x600[] = {
    0x4424, 0xF726, 0x0427,             /* Scaling Factor            */
    0x0528, 0x3A29, 0x282A,             /* DST H/V size              */
    0x0F36, 0x0137,                     /* DST_BUF_LINE              */
    0x883A,                             /* TV's Standard             */
    0xF442,                             /* BST_ST, BST_ED            */
    0x2144, 0x9C46,                     /* VID_ST, VID_ED            */
    0x0048,                             /* BRD_ST                    */
    0xA04C, 0x814D, 0x124F,             /* HP, VP, VSD               */
    0x9050, 0x9052,                     /* Chroma Ctrl               */
    0x8F54, 0xA055,                     /* Luma Ctrl                 */
    0x105C, 0x005E, 0x005F,             /* Burst and Sync Shape Ctrl */
    0xC3E2,
    0xFFFF
};

unsigned short ITV_PATCH_TABLE_576P_1024x768[] = {
    0xEA24, 0x0B25, 0x5D26, 0x0627,     /* Scaling Factor            */
    0x0328, 0x3A29, 0x282A,             /* DST H/V size              */
    0x883A,                             /* TV's Standard             */
    0x0040, 0x0042,                     /* BST_ST, BST_ED            */
    0x2144, 0x9C46,                     /* VID_ST, VID_ED            */
    0x0048,                             /* BRD_ST                    */
    0x204C, 0x824D, 0x124F,             /* HP, VP, VSD               */
    0x9050, 0x9052,                     /* Chroma Ctrl               */
    0x9054, 0xA055,                     /* Luma Ctrl                 */
    0x105C, 0x005E, 0x005F,             /* Burst and Sync Shape Ctrl */
    0xC3E2,
    0xFFFF
};

/**************************************************************************************/
/******************************** Patch Table for 720P ********************************/
/**************************************************************************************/
unsigned short ITV_PATCH_TABLE_720P_640x480[] = {
    0x0515, 0xDF18, 0xF919,
    0xD31A, 0xDF20, 0x0121,
    0x7F22, 0x2223,
    0x5424, 0x0525,
    0x5826,
    0xCA29, 0x142B,
    0x0535,
    0x4C4A, 0x8A4C,
    0x00DC,
    0xC4E2,
    0x02E3,
    0xFFFF
};

unsigned short ITV_PATCH_TABLE_720P_720x480[] = {
    0xC014, 0x0515, 0xDF18,
    0x7919, 0xD61A, 0xDF20,
    0x0121, 0xCF22, 0x2223,
    0x5524, 0x0525, 0x8726,
    0x0427, 0x272A, 0x172B,
    0xC034, 0x0535, 0x7C4C,
    0xC4E2, 0x02E3,
    0xFFFF
};

unsigned short ITV_PATCH_TABLE_720P_720x576[] = {
    0xC014, 0x0515, 0x3F18,
    0x7A19, 0xD61A, 0x3F20,
    0xCF22, 0x2223,
    0x6524, 0x0625, 0x8826,
    0x0427, 0x272A, 0x172B,
    0xC034, 0x0535,        
    0x64E2,
    0xFFFF
};

unsigned short ITV_PATCH_TABLE_720P_800x600[] = {
    0x4014, 0x0615, 0x8F16,
    0x5718, 0xD81A, 0x0A1B,
    0x5720, 0x1F22, 0x2323,
    0xAE24, 0x0625, 0x0726,
    0x0527, 0x192B,
    0x4034, 0x0635, 0xF036, 0x074C,
    0x64E2,
    0xFFFF
};

unsigned short ITV_PATCH_TABLE_720P_1024x768[] = {
    0x0815, 0x8F16,
    0xFF18, 0xDF1A, 0x0A1B,
    0xFF20, 0x2323,
    0x8724, 0x2825, 0x0926,
    0x0727, 0x202B,
    0x0835, 0xF036, 0x074C,
    0x00DC, 
    0x21E0, 0x0CE1, 0x24E2, 0x01E3, /* 66MHZ */
    0xFFFF
};

/**************************************************************************************/
/******************************** Patch Table for 1080I ********************************/
/**************************************************************************************/
unsigned short ITV_PATCH_TABLE_1080I_640x480[] = {
    0x0515, 0xDF18, 0xF919,
    0xD31A, 0xDF20, 0x0121,
    0x2223, 0x8E24, 0x2325,
    0xAB26, 0x6227, 0x142B,
    0x0535, 0x0936, 0x0137,
    0xEB4C,
    0xC4E2, 0x02E3,
    0xFFFF
};

unsigned short ITV_PATCH_TABLE_1080I_720x480[] = {
    0xC014, 0x0515, 0xDF18,
    0x7919, 0xD61A, 0xDF20,
    0x0121, 0xCF22, 0x2223,
    0x8E24, 0x2325, 0x0226,
    0x6327, 0x172B, 
    0xC034, 0x0535, 
    0xC4E2, 0x02E3,
    0xFFFF
};

unsigned short ITV_PATCH_TABLE_1080I_720x576[] = {
    0xC014, 0x0515, 0x3F18,
    0x7A19, 0xD61A, 0x3F20,
    0x0221, 0xCF22, 0x2223,
    0x4524, 0x2425, 0x0226,
    0x6327, 0x172B,
    0xC034, 0x0535,         
    0x64E2, 0x01E3,
    0xFFFF
};

unsigned short ITV_PATCH_TABLE_1080I_800x600[] = {
    0x4014, 0x0615, 0xFA19,
    0x5718, 0xD81A, 0x0221,
    0x5720, 0x1F22, 0x2323,
    0x7324, 0x2425, 0x5626,
    0x6327, 0x192B, 
    0x4034, 0x0635,
    0x0936, 0x0137, 0xEB4C,
    0x64E2, 0x01E3,
    0xFFFF
};

unsigned short ITV_PATCH_TABLE_1080I_1024x768[] = {
    0x0815, 0xFF18, 0xFA19,
    0xDF1A, 0x0221,
    0xFF20, 0xFF22, 0x2323,
    0xB224, 0x2525, 0x4626,
    0x6427, 0x202B,        
    0x0835,                        
    0x44E2, 0x01E3,
    0xFFFF
};

unsigned short ITV_PATCH_TABLE_1080I_1920x1080[] = {
    0x8002, 0x8003,
    0x3828, 0x782B,
    0xE84A, 0x7F4B, 0x004C, 0x084D, 0x804E, 0x044F,
    0xFFFF
};


unsigned short ITV_PATCH_TABLE_NULL[] = {
    0xFFFF
};

struct integrated_tv_hdtv_patch_table ITV_HDTV_PATCH_TABLE[] = {
    {VIA_TVRES_640X480,   ITV_PATCH_TABLE_480P_640x480,  ITV_PATCH_TABLE_576P_640x480,  ITV_PATCH_TABLE_720P_640x480, ITV_PATCH_TABLE_1080I_640x480},
    {VIA_TVRES_720X480,   ITV_PATCH_TABLE_480P_720x480,  ITV_PATCH_TABLE_480P_720x480,  ITV_PATCH_TABLE_720P_720x480, ITV_PATCH_TABLE_1080I_720x480},
    {VIA_TVRES_720X576,   ITV_PATCH_TABLE_576P_720x576,  ITV_PATCH_TABLE_576P_720x576,  ITV_PATCH_TABLE_720P_720x576, ITV_PATCH_TABLE_1080I_720x576},
    {VIA_TVRES_800X600,   ITV_PATCH_TABLE_480P_800x600,  ITV_PATCH_TABLE_576P_800x600,  ITV_PATCH_TABLE_720P_800x600, ITV_PATCH_TABLE_1080I_800x600},
    {VIA_TVRES_1024X768,  ITV_PATCH_TABLE_480P_1024x768, ITV_PATCH_TABLE_576P_1024x768, ITV_PATCH_TABLE_720P_1024x768,ITV_PATCH_TABLE_1080I_1024x768},
    {VIA_TVRES_1280X720,  ITV_PATCH_TABLE_NULL,          ITV_PATCH_TABLE_NULL,          ITV_PATCH_TABLE_NULL,         ITV_PATCH_TABLE_NULL},
    {VIA_TVRES_1920X1080, ITV_PATCH_TABLE_NULL,          ITV_PATCH_TABLE_NULL,          ITV_PATCH_TABLE_NULL,         ITV_PATCH_TABLE_1080I_1920x1080}

};


#endif
