/*
 * Copyright 1998-2007 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2007 S3 Graphics, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
#include "via_driver.h"
#include "via_hw.h"
#include "via_tv.h"
#include "via_IntegratedTV.h"
#include "via_vt1625.h"
#include "via_tblDPASetting.h"
#include "via_utility.h"
#include "via_gpioi2c.h"

/* Extern Functions Prototypes Declaration */
extern void write_regx(struct io_reg RegTable[],int ItemNum);
extern void SetVCLK(VIABIOSInfoPtr pBIOSInfo, CARD32 CLK, int set_iga);
extern __inline__ void unlock_crt(void);
extern void load_FIFO_reg(VIABIOSInfoPtr pBIOSInfo, int hor_active, int ver_active, int set_iga);
extern void VIASetDPA_Gfx(int DIPort, GFX_DPA_VALUE_PTR pGfxDPASetting);
extern int g_DeviceI2CPort;

static void VIATVSetGfxDPA(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo, TV_DPA_TABLE_PTR pTVDPATable);
static void VIATVSetTVEncoderDPA(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo, TV_DPA_TABLE_PTR pTVDPATable);
static void VIATVPatchSkew_VT1625(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo);
static void VIATVPatchSkew_VT1625_Old(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo);


/*****************************************************************************
 * NOTE: These were initially in via_tv2.h, and it seens they were created   *
 * by code in via_vt1621.c when CREATE_TV2_HEADERFILE, i.e. dumping register *
 * values?, but, looking at the code that should have generated these tables,*
 * there should not exist CARD8 fields with 16 bit values; maybe the tables  *
 * were edited by hand...						     *
 */
VIABIOSTVMASKTableRec tv2MaskTable = {
    {
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF,	0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF,	0xFF, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00,	0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF,	0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00,	0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00,	0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
    },
    {
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0x00, 0xFF, 0x00, 0x00,	0x00, 0x00, 0x00, 0x00,
	0xFF, 0x00, 0xFF, 0x00, 0x00, 0xFF, 0xFF, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
    },
    {
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF,	0xFF, 0xFF, 0xFF, 0xFF,
	0x00, 0x00, 0x00, 0x00, 0x00, 0xFF, 0xFF, 0xFF,
	0x00, 0x00, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00
    },
    0x3F, 0x38, 61, 13, 22
};

VIABIOSTV2TableRec tv2Table[] = {
    {
        {
	    0x64, 0x03, 0x22, 0x33, 0x43, 0x00, 0x10, 0x7D,
	    0xAC, 0x05, 0x99, 0x17, 0x93, 0xA5, 0x03, 0xBA,
	    0x00, 0x00, 0x0A, 0xCD, 0x80, 0x28, 0xBE, 0xFF,
	    0x7F, 0x20, 0x00, 0x01, 0x02, 0x80, 0x00, 0x00,
	    0x75, 0x0C, 0x04, 0x6D, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x48, 0x01, 0x02, 0x00, 0xFC, 0xF9,
	    0xFF, 0x10, 0x23, 0x2C, 0x09, 0x08, 0x0A, 0x0C,
	    0x0D, 0x0D, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
	{
	    0x64, 0x03, 0x22, 0x33, 0x43, 0x00, 0x10, 0x7D,
	    0xAC, 0x05, 0x99, 0x17, 0x93, 0xA5, 0x03, 0xBA,
	    0x00, 0x00, 0x0A, 0xCD, 0x80, 0x28, 0xBE, 0xFF,
	    0x7F, 0x20, 0x00, 0x01, 0x02, 0x80, 0x00, 0x00,
	    0x75, 0x0C, 0x04, 0x6D, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x48, 0x02, 0x02, 0xFD, 0x06, 0xF8,
	    0x0B, 0xF3, 0x0F, 0x70, 0x05, 0xF9, 0x0B, 0xF1,
	    0x11, 0x6E, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
	{
	    0x5D, 0x4F, 0x4F, 0x81, 0x52, 0x9E, 0x56, 0xBA,
	    0x00, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x08, 0x00, 0xDF, 0x00, 0x00, 0xDF, 0x57, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
	{
	    0x20, 0x40, 0x80, 0x00, 0x47, 0x1C, 0x00, 0x00
	},
	{
	    0x00, 0x00, 0x00, 0x47, 0x1C, 0x00, 0x00, 0x00
	},
        {
	    0x0F, 0x7F, 0x7F, 0x0F, 0x9A, 0x23, 0x8F, 0xEF,
	    0x57, 0xDF, 0xDF, 0x57, 0x11, 0x0A, 0x08, 0x50,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x28, 0x50, 0x00,
	    0x00, 0x00, 0x80, 0x20, 0x80, 0x00, 0x00, 0x00
	},
        {
	    0x0F, 0x7F, 0x7F, 0x0F, 0x9A, 0x23, 0x8F, 0xEF,
	    0x57, 0xDF, 0xDF, 0x57, 0x11, 0x0A, 0x08, 0x50,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x50, 0xA0, 0x40,
	    0x00, 0x00, 0x80, 0x20, 0x80, 0x00, 0x00, 0x00
	},
	{
	    0x0F, 0x7F, 0x7F, 0x0F, 0x9A, 0x23, 0x8F, 0xEF,
	    0x57, 0xDF, 0xDF, 0x57, 0x11, 0x0A, 0x08, 0x50,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0xA0, 0x40, 0x81,
	    0x00, 0x00, 0x80, 0x20, 0x80, 0x00, 0x00, 0x00
	},
        {
	    0x7E07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
	},
        {
	    0x02, 0x811, 0xF617, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
	},
        {
	    0x64, 0x01, 0x02, 0x33, 0x40, 0x00, 0x10, 0xAD,
	    0xD3, 0x37, 0xA3, 0x00, 0x94, 0xFF, 0x03, 0xBA,
	    0x00, 0x00, 0x07, 0x26, 0x2C, 0x20, 0x50, 0x63,
	    0xD5, 0x25, 0x00, 0x01, 0x02, 0x80, 0x00, 0x00,
	    0x75, 0x0C, 0x04, 0x76, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x48, 0x00, 0x00, 0xFE, 0xFC, 0xFD,
	    0x05, 0x12, 0x1F, 0x25, 0x0B, 0x08, 0x0A, 0x0C,
	    0x0D, 0x0D, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
	},
	{
	    0x64, 0x01, 0x02, 0x33, 0x40, 0x00, 0x10, 0xAD,
	    0xD3, 0x37, 0xA3, 0x00, 0x94, 0xFF, 0x03, 0xBA,
	    0x00, 0x00, 0x07, 0x26, 0x2C, 0x20, 0x50, 0x63,
	    0xD5, 0x25, 0x00, 0x01, 0x02, 0x80, 0x00, 0x00,
	    0x75, 0x0C, 0x04, 0x76, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x48, 0xFE, 0x03, 0xFB, 0x06, 0xF8,
	    0x0A, 0xF5, 0x0C, 0x73, 0x06, 0xF8, 0x0B, 0xF2,
	    0x10, 0x6F, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
	},
        {
	    0x64, 0x4F, 0x4F, 0x88, 0x53, 0x83, 0x6F, 0xBA,
	    0x00, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x11, 0x00, 0xDF, 0x00, 0x00, 0xDF, 0x70, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
	},
	{
	    0x20, 0x40, 0x80, 0x04, 0x47, 0x1C, 0x00, 0x00
	},
        {
	    0x00, 0x00, 0x00, 0x47, 0x1C, 0x00, 0x00, 0x00
	},
        {
	    0x47, 0x7F, 0x7F, 0x47, 0x9A, 0x23, 0x95, 0x1A,
	    0x70, 0xDF, 0xDF, 0x70, 0x51, 0x0A, 0x11, 0x5B,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x28, 0x50, 0x00,
	    0x00, 0x00, 0x80, 0x20, 0x80, 0x00, 0x00, 0x00
	},
        {
	    0x47, 0x7F, 0x7F, 0x47, 0x9A, 0x23, 0x95, 0x1A,
	    0x70, 0xDF, 0xDF, 0x70, 0x51, 0x0A, 0x11, 0x5B,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x50, 0xA0, 0x40,
	    0x00, 0x00, 0x80, 0x20, 0x80, 0x00, 0x00, 0x00
	},
        {
	    0x47, 0x7F, 0x7F, 0x47, 0x9A, 0x23, 0x95, 0x1A,
	    0x70, 0xDF, 0xDF, 0x70, 0x51, 0x0A, 0x11, 0x5B,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0xA0, 0x40, 0x81,
	    0x00, 0x00, 0x80, 0x20, 0x80, 0x00, 0x00, 0x00
	},
        {
	    0xB607, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
	}
    },
    {
        {
	    0x84, 0x03, 0x2A, 0x33, 0x43, 0x00, 0x10, 0xDD,
	    0xB9, 0x15, 0x99, 0x17, 0x93, 0xA5, 0x03, 0xBA,
	    0x00, 0x00, 0x0A, 0xED, 0x98, 0x1C, 0x96, 0x50,
	    0x5E, 0x1B, 0x00, 0x01, 0x02, 0x80, 0x00, 0x00,
	    0x75, 0x0C, 0x04, 0x6D, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x48, 0x00, 0xFF, 0xFD, 0xFC, 0xFF,
	    0x07, 0x13, 0x1E, 0x22, 0x0D, 0x08, 0x09, 0x0A,
	    0x0B, 0x0C, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
	},
	{
	    0x84, 0x03, 0x2A, 0x33, 0x43, 0x00, 0x10, 0xDD,
	    0xB9, 0x15, 0x99, 0x17, 0x93, 0xA5, 0x03, 0xBA,
	    0x00, 0x00, 0x0A, 0xED, 0x98, 0x1C, 0x96, 0x50,
	    0x5E, 0x1B, 0x00, 0x01, 0x02, 0x80, 0x00, 0x00,
	    0x75, 0x0C, 0x04, 0x6D, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x48, 0x02, 0x02, 0xFD, 0x06, 0xF8,
	    0x0B, 0xF3, 0x0F, 0x70, 0x05, 0xF9, 0x0B, 0xF1,
	    0x11, 0x6E, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
	},
        {
	    0x80, 0x63, 0x63, 0x84, 0x69, 0x1A, 0xEC, 0xF0,
	    0x00, 0x60, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x5C, 0x00, 0x57, 0x00, 0x00, 0x57, 0xED, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
	},
        {
	    0x00, 0x40, 0x80, 0x1E, 0x47, 0x1C, 0x00, 0x00
	},
        {
	    0x00, 0x00, 0x00, 0x47, 0x1C, 0x00, 0x00, 0x00
	},
        {
	    0x27, 0x1F, 0x1F, 0x27, 0xE3, 0x34, 0x48, 0x83,
	    0xED, 0x57, 0x57, 0xED, 0x52, 0x12, 0x5C, 0x51,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x32, 0x64, 0x00,
	    0x00, 0x00, 0x80, 0x20, 0x80, 0x00, 0x00, 0x00
	},
        {
	    0x27, 0x1F, 0x1F, 0x27, 0xE3, 0x34, 0x48, 0x83,
	    0xED, 0x57, 0x57, 0xED, 0x52, 0x12, 0x5C, 0x51,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x64, 0xC8, 0x40,
	    0x00, 0x00, 0x80, 0x20, 0x80, 0x00, 0x00, 0x00
	},
        {
	    0x27, 0x1F, 0x1F, 0x27, 0xE3, 0x34, 0x48, 0x83,
	    0xED, 0x57, 0x57, 0xED, 0x52, 0x12, 0x5C, 0x51,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0xC8, 0x90, 0x81,
	    0x00, 0x00, 0x80, 0x20, 0x80, 0x00, 0x00, 0x00
	},
        {
	    0xDE07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
	},
        {
	    0x02, 0x811, 0x5717, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
	},
        {
	    0x84, 0x03, 0x1A, 0x33, 0x40, 0x00, 0x10, 0x85,
	    0xF1, 0x4B, 0xA3, 0x00, 0x94, 0xFF, 0x03, 0xBA,
	    0x00, 0x00, 0x07, 0x25, 0x2C, 0x1C, 0x18, 0x28,
	    0x87, 0x1F, 0x00, 0x01, 0x02, 0x80, 0x00, 0x00,
	    0x75, 0x0C, 0x04, 0x76, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x48, 0xFF, 0xFE, 0xFD, 0xFE, 0x02,
	    0x0A, 0x13, 0x1A, 0x1D, 0x0F, 0x08, 0x09, 0x0A,
	    0x0B, 0x0B, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
	},
        {
	    0x84, 0x03, 0x1A, 0x33, 0x40, 0x00, 0x10, 0x85,
	    0xF1, 0x4B, 0xA3, 0x00, 0x94, 0xFF, 0x03, 0xBA,
	    0x00, 0x00, 0x07, 0x25, 0x2C, 0x1C, 0x18, 0x28,
	    0x87, 0x1F, 0x00, 0x01, 0x02, 0x80, 0x00, 0x00,
	    0x75, 0x0C, 0x04, 0x76, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x48, 0xFB, 0x04, 0xFB, 0x07, 0xF8,
	    0x09, 0xF6, 0x0A, 0x74, 0x06, 0xF8, 0x0B, 0xF2,
	    0x10, 0x6F, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
	},
        {
	    0x73, 0x63, 0x63, 0x97, 0x67, 0x91, 0xEC, 0xF0,
	    0x00, 0x60, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x5C, 0x00, 0x57, 0x00, 0x00, 0x57, 0xED, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
	},
        {
	    0x20, 0x40, 0x80, 0x04, 0x47, 0x1C, 0x00, 0x00
	},
        {
	    0x00, 0x00, 0x00, 0x47, 0x1C, 0x00, 0x00, 0x00
	},
        {
	    0xBF, 0x1F, 0x1F, 0xBF, 0xDB, 0x33, 0x38, 0x88,
	    0xED, 0x57, 0x57, 0xED, 0x52, 0x12, 0x5C, 0x51,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x32, 0x64, 0x00,
	    0x00, 0x00, 0x80, 0x20, 0x80, 0x00, 0x00, 0x00
	},
        {
	    0xBF, 0x1F, 0x1F, 0xBF, 0xDB, 0x33, 0x38, 0x88,
	    0xED, 0x57, 0x57, 0xED, 0x52, 0x12, 0x5C, 0x51,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x64, 0xC8, 0x40,
	    0x00, 0x00, 0x80, 0x20, 0x80, 0x00, 0x00, 0x00
	},
        {
	    0xBF, 0x1F, 0x1F, 0xBF, 0xDB, 0x33, 0x38, 0x88,
	    0xED, 0x57, 0x57, 0xED, 0x52, 0x12, 0x5C, 0x51,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0xC8, 0x90, 0x81,
	    0x00, 0x00, 0x80, 0x20, 0x80, 0x00, 0x00, 0x00
	},
        {
	    0x8607, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
	}
    }
};

VIABIOSTV2TableRec tv2OverTable[] = {
    {
        {
	    0x64, 0x03, 0x02, 0x33, 0x43, 0x00, 0x10, 0x7D,
	    0x72, 0x05, 0x99, 0x17, 0x93, 0xA5, 0x03, 0xBA,
	    0x00, 0x00, 0x0D, 0x45, 0x38, 0x34, 0xF1, 0x91,
	    0x24, 0x25, 0x00, 0x01, 0x02, 0x80, 0x00, 0x00,
	    0x75, 0x0C, 0x04, 0x6D, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x48, 0x00, 0x01, 0x02, 0xFF, 0xF9,
	    0xFA, 0x0C, 0x26, 0x32, 0x07, 0x08, 0x0A, 0x0D,
	    0x0E, 0x0F, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
	},
        {
	    0x64, 0x03, 0x02, 0x33, 0x43, 0x00, 0x10, 0x7D,
	    0x72, 0x05, 0x99, 0x17, 0x93, 0xA5, 0x03, 0xBA,
	    0x00, 0x00, 0x0D, 0x45, 0x38, 0x34, 0xF1, 0x91,
	    0x24, 0x25, 0x00, 0x01, 0x02, 0x80, 0x00, 0x00,
	    0x75, 0x0C, 0x04, 0x6D, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x48, 0x02, 0x02, 0xFD, 0x06, 0xF8,
	    0x0B, 0xF3, 0x0F, 0x70, 0x05, 0xF9, 0x0B, 0xF1,
	    0x11, 0x6E, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
	},
        {
	    0x5D, 0x4F, 0x4F, 0x81, 0x52, 0x9E, 0x0B, 0x3E,
	    0x00, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0xEE, 0x00, 0xDF, 0x00, 0x00, 0xDF, 0x0C, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
	},
        {
	    0x20, 0x40, 0x80, 0x01, 0x47, 0x1C, 0x00, 0x00
	},
        {
	    0x00, 0x00, 0x00, 0x47, 0x1C, 0x00, 0x00, 0x00
	},
        {
	    0x0F, 0x7F, 0x7F, 0x0F, 0x9A, 0x23, 0x8F, 0xEF,
	    0x0C, 0xDF, 0xDF, 0x0C, 0x11, 0x0A, 0xEE, 0x31,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x28, 0x50, 0x00,
	    0x00, 0x00, 0x80, 0x20, 0x80, 0x00, 0x00, 0x00
	},
        {
	    0x0F, 0x7F, 0x7F, 0x0F, 0x9A, 0x23, 0x8F, 0xEF,
	    0x0C, 0xDF, 0xDF, 0x0C, 0x11, 0x0A, 0xEE, 0x31,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x50, 0xA0, 0x40,
	    0x00, 0x00, 0x80, 0x20, 0x80, 0x00, 0x00, 0x00
	},
        {
	    0x0F, 0x7F, 0x7F, 0x0F, 0x9A, 0x23, 0x8F, 0xEF,
	    0x0C, 0xDF, 0xDF, 0x0C, 0x11, 0x0A, 0xEE, 0x31,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0xA0, 0x40, 0x81,
	    0x00, 0x00, 0x80, 0x20, 0x80, 0x00, 0x00, 0x00
	},
        {
	    0x7E07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
	},
        {
	    0x02, 0x811, 0x9917, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
	},
        {
	    0x64, 0x01, 0x12, 0x33, 0x40, 0x00, 0x10, 0x1D,
	    0x68, 0x26, 0xA3, 0x00, 0x94, 0xFF, 0x03, 0xBA,
	    0x00, 0x00, 0x0F, 0x67, 0x58, 0x3C, 0x24, 0xBC,
	    0x4A, 0x2F, 0x00, 0x01, 0x0A, 0x80, 0x00, 0x00,
	    0x75, 0x0C, 0x04, 0x76, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x48, 0x00, 0x01, 0x01, 0xFE, 0xFA,
	    0xFD, 0x0E, 0x24, 0x2E, 0x07, 0x07, 0x0A, 0x0D,
	    0x0F, 0x0F, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
	},
        {
	    0x64, 0x01, 0x12, 0x33, 0x40, 0x00, 0x10, 0x1D,
	    0x68, 0x26, 0xA3, 0x00, 0x94, 0xFF, 0x03, 0xBA,
	    0x00, 0x00, 0x0F, 0x67, 0x58, 0x3C, 0x24, 0xBC,
	    0x4A, 0x2F, 0x00, 0x01, 0x0A, 0x80, 0x00, 0x00,
	    0x75, 0x0C, 0x04, 0x76, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x48, 0xFD, 0x03, 0xFB, 0x07, 0xF8,
	    0x0A, 0xF5, 0x0B, 0x74, 0x06, 0xF8, 0x0B, 0xF2,
	    0x10, 0x6F, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
	},
        {
	    0x73, 0x4F, 0x4F, 0x97, 0x54, 0x8F, 0xF2, 0x1F,
	    0x00, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0xE5, 0x00, 0xDF, 0x00, 0x00, 0xDF, 0xF3, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
	},
        {
	    0x20, 0x40, 0x80, 0x01, 0x47, 0x1C, 0x00, 0x00
	},
        {
	    0x00, 0x00, 0x00, 0x47, 0x1C, 0x00, 0x00, 0x00
	},
        {
	    0xBF, 0x7F, 0x7F, 0xBF, 0x9A, 0x23, 0x90, 0x65,
	    0xF3, 0xDF, 0xDF, 0xF3, 0x49, 0x09, 0xE5, 0x26,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x28, 0x50, 0x00,
	    0x00, 0x00, 0x80, 0x20, 0x80, 0x00, 0x00, 0x00
	},
        {
	    0xBF, 0x7F, 0x7F, 0xBF, 0x9A, 0x23, 0x90, 0x65,
	    0xF3, 0xDF, 0xDF, 0xF3, 0x49, 0x09, 0xE5, 0x26,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x50, 0xA0, 0x40,
	    0x00, 0x00, 0x80, 0x20, 0x80, 0x00, 0x00, 0x00
	},
        {
	    0xBF, 0x7F, 0x7F, 0xBF, 0x9A, 0x23, 0x90, 0x65,
	    0xF3, 0xDF, 0xDF, 0xF3, 0x49, 0x09, 0xE5, 0x26,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0xA0, 0x40, 0x81,
	    0x00, 0x00, 0x80, 0x20, 0x80, 0x00, 0x00, 0x00
	},
        {
	    0x1E07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
	}
    },
    {
        {
	    0x84, 0x03, 0x0A, 0x33, 0x43, 0x00, 0x10, 0xC5,
	    0xAD, 0x10, 0x99, 0x17, 0x93, 0xA5, 0x03, 0xBA,
	    0x00, 0x00, 0x04, 0x07, 0x20, 0x0C, 0x08, 0x00,
	    0x00, 0x1C, 0x00, 0x01, 0x02, 0x80, 0x00, 0x00,
	    0x75, 0x0C, 0x04, 0x6D, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x48, 0x00, 0xFF, 0xFD, 0xFC, 0xFE,
	    0x06, 0x13, 0x1E, 0x23, 0x0D, 0x08, 0x0A, 0x0B,
	    0x0C, 0x0C, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
	},
        {
	    0x84, 0x03, 0x0A, 0x33, 0x43, 0x00, 0x10, 0xC5,
	    0xAD, 0x10, 0x99, 0x17, 0x93, 0xA5, 0x03, 0xBA,
	    0x00, 0x00, 0x04, 0x07, 0x20, 0x0C, 0x08, 0x00,
	    0x00, 0x1C, 0x00, 0x01, 0x02, 0x80, 0x00, 0x00,
	    0x75, 0x0C, 0x04, 0x6D, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x48, 0x02, 0x02, 0xFD, 0x06, 0xF8,
	    0x0B, 0xF3, 0x0F, 0x70, 0x05, 0xF9, 0x0B, 0xF1,
	    0x11, 0x6E, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
	},
        {
	    0x7D, 0x63, 0x63, 0x81, 0x69, 0x18, 0xBA, 0xF0,
	    0x00, 0x60, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x5A, 0x00, 0x57, 0x00, 0x00, 0x57, 0xBB, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
	},
        {
	    0x00, 0x40, 0x80, 0x01, 0x47, 0x1C, 0x00, 0x00
	},
        {
	    0x00, 0x00, 0x00, 0x47, 0x1C, 0x00, 0x00, 0x00
	},
        { 
	    0x0F, 0x1F, 0x1F, 0x0F, 0xE3, 0x34, 0x44, 0xBC,
	    0xBB, 0x57, 0x57, 0xBB, 0x52, 0x12, 0x5A, 0x48,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x32, 0x64, 0x00,
	    0x00, 0x00, 0x80, 0x20, 0x80, 0x00, 0x00, 0x00
	},
        {
	    0x0F, 0x1F, 0x1F, 0x0F, 0xE3, 0x34, 0x44, 0xBC,
	    0xBB, 0x57, 0x57, 0xBB, 0x52, 0x12, 0x5A, 0x48,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x64, 0xC8, 0x40,
	    0x00, 0x00, 0x80, 0x20, 0x80, 0x00, 0x00, 0x00
	},
        {
	    0x0F, 0x1F, 0x1F, 0x0F, 0xE3, 0x34, 0x44, 0xBC,
	    0xBB, 0x57, 0x57, 0xBB, 0x52, 0x12, 0x5A, 0x48,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0xC8, 0x90, 0x81,
	    0x00, 0x00, 0x80, 0x20, 0x80, 0x00, 0x00, 0x00
	},
        {
	    0xCE07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
	},
        {
	    0x02, 0x811, 0x817, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
	},
        {
	    0x84, 0x03, 0x02, 0x33, 0x40, 0x00, 0x10, 0x75,
	    0x7B, 0x34, 0xA3, 0x00, 0x94, 0xFF, 0x03, 0xBA,
	    0x00, 0x00, 0x08, 0xFD, 0xEF, 0x20, 0x0C, 0x8C,
	    0x79, 0x26, 0x00, 0x01, 0x02, 0x80, 0x00, 0x00,
	    0x75, 0x0C, 0x04, 0x76, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x48, 0xFF, 0xFE, 0xFD, 0xFE, 0x02,
	    0x0A, 0x13, 0x1A, 0x1D, 0x0F, 0x08, 0x09, 0x0A,
	    0x0B, 0x0B, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
	},
        {
	    0x84, 0x03, 0x02, 0x33, 0x40, 0x00, 0x10, 0x75,
	    0x7B, 0x34, 0xA3, 0x00, 0x94, 0xFF, 0x03, 0xBA,
	    0x00, 0x00, 0x08, 0xFD, 0xEF, 0x20, 0x0C, 0x8C,
	    0x79, 0x26, 0x00, 0x01, 0x02, 0x80, 0x00, 0x00,
	    0x75, 0x0C, 0x04, 0x76, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x48, 0xFB, 0x04, 0xFB, 0x07, 0xF8,
	    0x09, 0xF6, 0x0A, 0x74, 0x06, 0xF8, 0x0B, 0xF2,
	    0x10, 0x6F, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
	},
        {
	    0x71, 0x63, 0x63, 0x95, 0x67, 0x90, 0x6F, 0xF0,
	    0x00, 0x60, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	    0x57, 0x00, 0x57, 0x00, 0x00, 0x57, 0x70, 0x00,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
	},
        {
	    0x20, 0x40, 0x80, 0x01, 0x47, 0x1C, 0x00, 0x00
	},
        {
	    0x00, 0x00, 0x00, 0x47, 0x1C, 0x00, 0x00, 0x00
	},
        {
	    0xAF, 0x1F, 0x1F, 0xAF, 0xDB, 0x33, 0x35, 0x7D,
	    0x70, 0x57, 0x57, 0x70, 0x52, 0x12, 0x58, 0x5B,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x32, 0x64, 0x00,
	    0x00, 0x00, 0x80, 0x20, 0x80, 0x00, 0x00, 0x00
	},
        {
	    0xAF, 0x1F, 0x1F, 0xAF, 0xDB, 0x33, 0x35, 0x7D,
	    0x70, 0x57, 0x57, 0x70, 0x52, 0x12, 0x58, 0x5B,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0x64, 0xC8, 0x40,
	    0x00, 0x00, 0x80, 0x20, 0x80, 0x00, 0x00, 0x00
	},
        {
	    0xAF, 0x1F, 0x1F, 0xAF, 0xDB, 0x33, 0x35, 0x7D,
	    0x70, 0x57, 0x57, 0x70, 0x52, 0x12, 0x58, 0x5B,
	    0x00, 0x00, 0x00, 0x00, 0x00, 0xC8, 0x90, 0x81,
	    0x00, 0x00, 0x80, 0x20, 0x80, 0x00, 0x00, 0x00
	},
        {
	    0x7E07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
	}
    }
};


void GPIOI2CWrite_TV(VIABIOSInfoPtr pBIOSInfo, int index, CARD8 data)
{
    g_DeviceI2CPort = pBIOSInfo->TVI2CPort;
    i2cWriteByte(pBIOSInfo, pBIOSInfo->TVI2CAdd, index, data);
}

CARD8
GPIOI2CRead_TV(VIABIOSInfoPtr pBIOSInfo, int index)
{
    CARD8 data;
    
    g_DeviceI2CPort = pBIOSInfo->TVI2CPort;
    i2cReadByte(pBIOSInfo, pBIOSInfo->TVI2CAdd, index, &data);
    return data;
}

Bool
VIAIsTVMode(VIABIOSInfoPtr pBIOSInfo, int ModeIndex, TVSETTINGINFOPTR pTVSettingInfo)
{
    BOOL IsTVMode = FALSE;
    
    switch (pTVSettingInfo->TVEncoder)
    {
        case VIA_VT1621:
            if (ModeIndex == M640x480 || ModeIndex == M800x600)
            {
                IsTVMode = TRUE;
            }
            
            break;
            
        case VIA_VT1622:
        case VIA_VT1622A:
        case VIA_VT1623:
            if (ModeIndex == M640x480  || ModeIndex == M800x600 ||
                ModeIndex == M1024x768 || ModeIndex == M720x480 ||
                ModeIndex == M720x576  || ModeIndex == M848x480)
            {
                IsTVMode = TRUE;
            }
            
            break;
            
        case VIA_VT1625:
            if (ModeIndex == M640x480  || ModeIndex == M800x600  ||
                ModeIndex == M1024x768 || ModeIndex == M720x480  ||
                ModeIndex == M720x576  || ModeIndex == M1280x720 ||
                ModeIndex == M1920x1080)
            {
                IsTVMode = TRUE;
            }            
            break;
            
       case VIA_INTEGRATED_TV:
            IsTVMode = IsIntegratedTVMode(pBIOSInfo, ModeIndex, pTVSettingInfo);
            break;

    }

    return IsTVMode;
}

void VIATVSetMode(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo)
{
    BOOL    setTV = FALSE;
    int     resMode;

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIATVSetMode!!\n"));

    resMode = pTVSettingInfo->ModeIndex;
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIATVSetMode: modeindex= %d !!\n",resMode));

    unlock_crt();

    /* Found TV resmod index */
    switch (resMode)
    {
        case M640x480:
            pTVSettingInfo->resTVMode = VIA_TVRES_640X480;
            break;
        case M800x600:
            pTVSettingInfo->resTVMode = VIA_TVRES_800X600;
            break;
        case M1024x768:
            pTVSettingInfo->resTVMode = VIA_TVRES_1024X768;
            break;
        case M848x480:
            pTVSettingInfo->resTVMode = VIA_TVRES_848X480;
            break;
        case M720x480:
            pTVSettingInfo->resTVMode = VIA_TVRES_720X480;
            if (pTVSettingInfo->TVType == TVTYPE_480P ||
                pTVSettingInfo->TVType == TVTYPE_720P ||
                pTVSettingInfo->TVType == TVTYPE_1080I)
            {
                /* SDTV only support RGB & YPBPR output signal type */
                if ((pTVSettingInfo->TVOutput != TV_OUTPUT_RGB) &&
                    (pTVSettingInfo->TVOutput != TV_OUTPUT_YPBPR))
                {
                    pTVSettingInfo->TVOutput = TV_OUTPUT_RGB;
                }
            }
            else
            {
                pTVSettingInfo->TVType = TVTYPE_NTSC;
            }
            break;
        case M720x576:
            pTVSettingInfo->resTVMode = VIA_TVRES_720X576;

            /* RainLin [2005.07.05]: */
            /* Bug fix: SV-VT3259-Linux-071(CR63643): CRT+HDTV Simultaneous: Set TV output format as 576P */
            /* then change resolution to 720x576, caused CRT, TV, HDTV icons all be enabled. (CN400 Mandrake9.2) */
            /* Reason: Use wrong TV type to check the active state of TV and HDTV when utility query UT_XV_FUNC_DEVICE_GetActiveState */
            /* to get active state. */
            /* Solution: Don't change TV type from 576P to PAL in mode 720x576 for HDTV case. */
            if (pTVSettingInfo->TVType == TVTYPE_576P ||
                pTVSettingInfo->TVType == TVTYPE_720P || 
                pTVSettingInfo->TVType == TVTYPE_1080I)
            {
                /* SDTV only support RGB & YPBPR output signal type */
                if ((pTVSettingInfo->TVOutput != TV_OUTPUT_RGB) &&
                    (pTVSettingInfo->TVOutput != TV_OUTPUT_YPBPR))
                {
                    pTVSettingInfo->TVOutput = TV_OUTPUT_RGB;
                }
            }
            else
            {
                pTVSettingInfo->TVType = TVTYPE_PAL;
            }
            break;
        case M1280x720:
            pTVSettingInfo->resTVMode = VIA_TVRES_1280X720;
            pTVSettingInfo->TVType = TVTYPE_720P;
            /* HDTV only support RGB & YPBPR output signal type */
            if ((pTVSettingInfo->TVOutput != TV_OUTPUT_RGB) &&
                (pTVSettingInfo->TVOutput != TV_OUTPUT_YPBPR))
            {
                pTVSettingInfo->TVOutput = TV_OUTPUT_RGB;
            }
            break;
        case M1920x1080:
            pTVSettingInfo->resTVMode = VIA_TVRES_1920X1080;
            pTVSettingInfo->TVType = TVTYPE_1080I;
            /* HDTV only support RGB & YPBPR output signal type */
            if ((pTVSettingInfo->TVOutput != TV_OUTPUT_RGB) &&
                (pTVSettingInfo->TVOutput != TV_OUTPUT_YPBPR))
            {
                pTVSettingInfo->TVOutput = TV_OUTPUT_RGB;
            }
            break;
        default :
            pTVSettingInfo->resTVMode = resMode;
            ErrorF("No TV suitable mode found!! mode:%d\n", resMode);
            break;
    }
    
    switch (pTVSettingInfo->TVEncoder)
    {
        case VIA_VT1621:
            if (VIAIsTVMode(pBIOSInfo, resMode, pTVSettingInfo))
            {
                setTV = TRUE;
                VIAPreSetTV2Mode(pBIOSInfo);
                VIAPostSetTV2Mode(pBIOSInfo);
            }
            break;
        case VIA_VT1622:
            if (VIAIsTVMode(pBIOSInfo, resMode, pTVSettingInfo))
            {
                setTV = TRUE;
                load_vt1622_regs(pBIOSInfo);
            }
            break;
        case VIA_VT1622A:
        case VIA_VT1623:
            if (VIAIsTVMode(pBIOSInfo, resMode, pTVSettingInfo))
            {
                setTV = TRUE;
                load_vt1623_regs(pBIOSInfo);
            }
            break;
        case VIA_VT1625:
            if (VIAIsTVMode(pBIOSInfo, resMode, pTVSettingInfo))
            {
                setTV = TRUE;
                load_vt1625_regs(pBIOSInfo, pTVSettingInfo);
            }
            break;

        case VIA_INTEGRATED_TV:
            if (VIAIsTVMode(pBIOSInfo, resMode, pTVSettingInfo))
            {
                setTV = TRUE;
                VIASetIntegratedTVMode(pBIOSInfo, pTVSettingInfo);
            }
            
            break;
            
    }

    if (setTV)
    {
        /* Adjust clock skew. */
        VIATVPatch(pBIOSInfo, pTVSettingInfo);
        
        VIAEnableTV(pBIOSInfo, pTVSettingInfo);

        /* get driver default TV setting */
        VIAUTGetTVInfo(pBIOSInfo, pTVSettingInfo);
        
        /* restore utility user TV setting */
        VIARestoreUserTVSetting(pBIOSInfo, pTVSettingInfo);        

        if (pTVSettingInfo->IGAPath == IGA2)
        {
            /*load IGA2 offset and fetch count*/
            if(pBIOSInfo->MergedFB)
            { 
                if((pBIOSInfo->Scrn2Position == viaRightOf)||(pBIOSInfo->Scrn2Position == viaLeftOf))
                {
                     load_offset_reg((pTVSettingInfo->HActive*2),(pBIOSInfo->bitsPerPixel>>3),IGA2);
                }
                
                if((pBIOSInfo->Scrn2Position == viaBelow)||(pBIOSInfo->Scrn2Position == viaAbove))
                {
                    load_offset_reg(pTVSettingInfo->HActive,(pBIOSInfo->bitsPerPixel>>3),IGA2);
                }
                
                load_fetch_count_reg(pTVSettingInfo->HActive,(pBIOSInfo->bitsPerPixel>>3),IGA2);
            }
            else
            {
                load_fetch_count_reg(pTVSettingInfo->HActive, (pBIOSInfo->bitsPerPixel>>3),IGA2);
                load_offset_reg(pTVSettingInfo->HActive,(pBIOSInfo->bitsPerPixel>>3),IGA2);
            }
            
            load_FIFO_reg(pBIOSInfo, pBIOSInfo->CrtcHDisplay, pBIOSInfo->CrtcVDisplay, IGA2);
            set_color_depth(pBIOSInfo, pBIOSInfo->bitsPerPixel>>3, IGA2);
        }

        if (pBIOSInfo->TVSettingInfo.TVTypeIsNTSCJ == TRUE)
            GPIOI2CWrite_TV(pBIOSInfo, 0x0B, 0);
    }
}

void VIASetTVOutputPath(VIABIOSInfoPtr pBIOSInfo, int set_iga, int output_interface)
{
    switch(set_iga)
    {
        case IGA1:
            /* Set IGA1 display channel to normal case beside CLE266.AX chip */
            /* and disable simultaneous display. */
            if (pBIOSInfo->dwProjectIDRevision == UNICHROME_VT3123AX)
            {
                SetVCLK(pBIOSInfo, 0x471c, IGA1);
                write_reg_mask(CR6B, VIACR, 0xC0, BIT3+BIT6+BIT7);
            }
            else
            {
                write_reg_mask(CR6B, VIACR, 0x00, BIT3+BIT6+BIT7);
            }
            break;
        case IGA2:
            /* Set IGA2 display channel to normal case beside CLE266.AX chip */
            /* and disable simultaneous display. */
            /* RainLin [2005.11.22]: Modify for sequence issue. */
            EnableSecondDisplayChannel();

            if (pBIOSInfo->dwProjectIDRevision == UNICHROME_VT3123AX)
            {
                write_reg_mask(CR6B, VIACR, 0x30, BIT3+BIT4+BIT5);
            }
            else
            {
                write_reg_mask(CR6B, VIACR, 0x00, BIT3+BIT6+BIT7);
            }

            /* Disable LCD Scaling */
            write_reg_mask(CR79, VIACR, 0x00, BIT0);
            break;
        case IGA1_IGA2:
            /* Set IGA1_IGA2 display channel to normal case beside CLE266.AX chip */
            /* and enable simultaneous display. */
            /* RainLin [2005.11.22]: Modify for sequence issue. */
            EnableSecondDisplayChannel();

            if (pBIOSInfo->dwProjectIDRevision == UNICHROME_VT3123AX)
            {
                write_reg_mask(CR6B, VIACR, 0xC0, BIT3+BIT6+BIT7);
            }
            else
            {
                write_reg_mask(CR6B, VIACR, 0x00, BIT3+BIT6+BIT7);
            }
            break;
    }

    switch(output_interface)
    {
        case VIA_DI_DVP0:
            if (set_iga== IGA1)
            {
                write_reg_mask(CR6C, VIACR, 0x00, BIT7);
            }
            else
            {
                write_reg_mask(CR6C, VIACR, 0x80, BIT7);
            }

            if (pBIOSInfo->dwProjectIDRevision == UNICHROME_VT3123AX)
            {
                write_reg_mask(CR6B, VIACR, 0x01, BIT0);
                write_reg_mask(CR6C, VIACR, 0x01, BIT0);
            }
            else
            {
                write_reg_mask(CR6B, VIACR, 0x00, BIT0);
                write_reg_mask(CR6C, VIACR, 0x21, BIT0+BIT5);
            }

            write_reg_mask(SR1E, VIASR, 0xC0, BIT6+BIT7);
            break;
        case VIA_DI_DVP1:
            if (set_iga== IGA1)
            {
                write_reg_mask(CR6C, VIACR, 0x00, BIT7);
                write_reg_mask(CR9B, VIACR, 0x00, BIT4);
            }
            else
            {
                write_reg_mask(CR6C, VIACR, 0x80, BIT7);
                write_reg_mask(CR9B, VIACR, 0x10, BIT4);
            }

            write_reg_mask(CR6C, VIACR, 0x21, BIT0+BIT5);
            write_reg_mask(SR1E, VIASR, 0x30, BIT4+BIT5);
            break;
        case VIA_DI_DFPHIGH:
            if (set_iga== IGA1)
            {
                write_reg_mask(CR6C, VIACR, 0x00, BIT7);
                write_reg_mask(CR96, VIACR, 0x00, BIT4);
                write_reg_mask(CR97, VIACR, 0x00, BIT4);
            }
            else
            {
                write_reg_mask(CR6C, VIACR, 0x80, BIT7);
                write_reg_mask(CR96, VIACR, 0x10, BIT4);
                write_reg_mask(CR97, VIACR, 0x10, BIT4);
            }

            write_reg_mask(CR6C, VIACR, 0x21, BIT0+BIT5);
            write_reg_mask(SR2A, VIASR, 0x0C, BIT2+BIT3);
            break;
        case VIA_DI_DFPLOW:
            if (set_iga== IGA1)
            {
                write_reg_mask(CR6C, VIACR, 0x00, BIT7);
                write_reg_mask(CR9B, VIACR, 0x00, BIT4);
                write_reg_mask(CR99, VIACR, 0x00, BIT4);
            }
            else
            {
                write_reg_mask(CR6C, VIACR, 0x80, BIT7);
                write_reg_mask(CR9B, VIACR, 0x10, BIT4);
                write_reg_mask(CR99, VIACR, 0x10, BIT4);
            }

            write_reg_mask(CR6C, VIACR, 0x21, BIT0+BIT5);
            write_reg_mask(SR2A, VIASR, 0x03, BIT0+BIT1);
            break;
    }
}


void VIATVPatchSkew_K8(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo)
{
    switch(pBIOSInfo->TVDIPort)
    {
        case VIA_DI_DVP0:
            write_reg_mask(CR96, VIACR, 0x07, BIT0+BIT1+BIT2);
            break;
        case VIA_DI_DVP1:
            write_reg_mask(CR9B, VIACR, 0x07, BIT0+BIT1+BIT2);
            break;
        case VIA_DI_DFPHIGH:
            write_reg_mask(CR97, VIACR, 0x07, BIT0+BIT1+BIT2);
            break;
        case VIA_DI_DFPLOW:
            write_reg_mask(CR99, VIACR, 0x07, BIT0+BIT1+BIT2);
            break;
    }
}


static void
VIATVSetGfxDPA(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo, TV_DPA_TABLE_PTR pTVDPATable)
{
    GFX_DPA_VALUE_PTR pGfxDPASettingValue = NULL;
    GFX_DPA_TVTYPE_MAP_TABLE_PTR pGfxDPATVTypeMapTable = NULL;
    GFX_DPA_TVSIZE_MAP_TABLE_PTR pGfxDPATVSizeMapTable = NULL;
    
    if (pTVDPATable)
    {
        pGfxDPATVTypeMapTable = pTVDPATable->pGfxDPATVTypeMapTable;
    }

    if (pGfxDPATVTypeMapTable)
    {
        switch (pTVSettingInfo->TVType)
        {
            case TVTYPE_NTSC:
                pGfxDPATVSizeMapTable = &(pGfxDPATVTypeMapTable->StdNTSC);
                break;

            case TVTYPE_PAL:
                pGfxDPATVSizeMapTable = &(pGfxDPATVTypeMapTable->StdPAL);
                break;

            case TVTYPE_480P:
                pGfxDPATVSizeMapTable = &(pGfxDPATVTypeMapTable->Std480p);
                break;            
                
            case TVTYPE_576P:
                pGfxDPATVSizeMapTable = &(pGfxDPATVTypeMapTable->Std576P);
                break;

            case TVTYPE_720P:
                pGfxDPATVSizeMapTable = &(pGfxDPATVTypeMapTable->Std720p);
                break;
                
            case TVTYPE_1080I:
                pGfxDPATVSizeMapTable = &(pGfxDPATVTypeMapTable->Std1080i);
                break;

            default:
                break;
        }
    }

    if (pGfxDPATVSizeMapTable)
    {
        switch (pTVSettingInfo->TVVScan)
        {
            case VIA_TVNORMAL:
                pGfxDPASettingValue = pGfxDPATVSizeMapTable->pNormalscan;
                break;

            case VIA_TVFIT:
                pGfxDPASettingValue = pGfxDPATVSizeMapTable->pFitscan;
                break;

            case VIA_TVOVER:
                pGfxDPASettingValue = pGfxDPATVSizeMapTable->pOverscan;
                break;

            default:
                break;
        }
    }

    if (pGfxDPASettingValue)
    {
        VIASetDPA_Gfx(pBIOSInfo->TVDIPort, pGfxDPASettingValue);
    }
}


static void
VIATVSetTVEncoderDPA(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo, TV_DPA_TABLE_PTR pTVDPATable)
{
    TV_DPA_TVTYPE_MAP_TABLE_PTR pTVDPATVTypeMapTable = NULL;
    TV_DPA_TVSIZE_MAP_TABLE_PTR pTVDPATVSizeMapTable = NULL;
    CARD32 *pTVDPASettingValue = NULL;
    int index;
    CARD8 data, mask, tmp_data;
    
    if (pTVDPATable)
    {
        pTVDPATVTypeMapTable = pTVDPATable->pTVDPATVTypeMapTable;
    }

    if (pTVDPATVTypeMapTable)
    {
        switch (pTVSettingInfo->TVType)
        {
            case TVTYPE_NTSC:
                pTVDPATVSizeMapTable = &(pTVDPATVTypeMapTable->StdNTSC);
                break;

            case TVTYPE_PAL:
                pTVDPATVSizeMapTable = &(pTVDPATVTypeMapTable->StdPAL);
                break;

            case TVTYPE_480P:
                pTVDPATVSizeMapTable = &(pTVDPATVTypeMapTable->Std480p);
                break;            
                
            case TVTYPE_576P:
                pTVDPATVSizeMapTable = &(pTVDPATVTypeMapTable->Std576P);
                break;

            case TVTYPE_720P:
                pTVDPATVSizeMapTable = &(pTVDPATVTypeMapTable->Std720p);
                break;
                
            case TVTYPE_1080I:
                pTVDPATVSizeMapTable = &(pTVDPATVTypeMapTable->Std1080i);
                break;

            default:
                break;
        }
    }

    if (pTVDPATVSizeMapTable)
    {
        switch (pTVSettingInfo->TVVScan)
        {
            case VIA_TVNORMAL:
                pTVDPASettingValue = pTVDPATVSizeMapTable->pNormalscan;
                break;

            case VIA_TVFIT:
                pTVDPASettingValue = pTVDPATVSizeMapTable->pFitscan;
                break;

            case VIA_TVOVER:
                pTVDPASettingValue = pTVDPATVSizeMapTable->pOVerscan;
                break;

            default:
                break;
        }
    }

    if (pTVDPASettingValue)
    {
        while (*pTVDPASettingValue != 0xFFFFFFFF)
        {
            index = *pTVDPASettingValue & 0xFF;
            mask = (*pTVDPASettingValue >> 8) & 0xFF;
            data = (*pTVDPASettingValue >> 16) & 0xFF;
            
            tmp_data = GPIOI2CRead_TV(pBIOSInfo, index);
            tmp_data = (tmp_data & (~mask));
            GPIOI2CWrite_TV(pBIOSInfo, index, tmp_data | data);

            pTVDPASettingValue++;
        }
    }
}


static void
VIATVPatchSkew_VT1625(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo)
{
    GFX_DPA_INDEX_TABLE *gfxptr;
    TV_DPA_TABLE_PTR pTVDPATable = NULL;

    /* Find TV DPA table by chip */
    for (gfxptr = GFX_DPA_INDEX_TBL; gfxptr->ChipSet != 0xFFFFFFFF; gfxptr++) {
	if (gfxptr->ChipSet == pBIOSInfo->Chipset) {
	    pTVDPATable = gfxptr->pVT1625DPATbl;
	    break;
	}
    }

    if (pTVDPATable == NULL) {
	return;
    }

    /* Try to find Gfx DPA Setting table by TV Mode, TV Type and TV Size */
    while (pTVDPATable->ModeIndex != VIA_TVRES_INVALID) {
	if (pTVDPATable->ModeIndex == pTVSettingInfo->resTVMode) {
	    break;
	}
	pTVDPATable++;
    }

    /* Set Gfx and TV Encoder DPA */
    if (pTVDPATable) {
	VIATVSetGfxDPA(pBIOSInfo, pTVSettingInfo, pTVDPATable);
	VIATVSetTVEncoderDPA(pBIOSInfo, pTVSettingInfo, pTVDPATable);
    }
}


static void
VIATVPatchSkew_VT1625_Old(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo)
{
    /* RainLin [2006.12.06]: Move codes from load_vt1625_regs() to here.*/
    /* For older chip such as VT3205. */
    
    /* Patch for HDTV type 720P at 1024x768 and HDTV discolor issue */
    if((pTVSettingInfo->TVType == TVTYPE_720P)&&(pTVSettingInfo->resTVMode == VIA_TVRES_1024X768))
    {
        if (pBIOSInfo->TVDIPort & VIA_DI_DVP0)
        {
            write_reg_mask(CR96, VIACR, 0x04, BIT2+BIT1+BIT0);
        }
        
        if (pBIOSInfo->TVDIPort & VIA_DI_DVP1)
        {
            if (pBIOSInfo->Chipset == VIA_KM400 &&
                pBIOSInfo->ChipRev < 5)
            {
                /*=* VT3205 Ax *=*/
                write_reg_mask(CR9B, VIACR, 0x07, BIT2+BIT1+BIT0);
            }
            else
            {
                write_reg_mask(CR9B, VIACR, 0x05, BIT2+BIT1+BIT0);
            }
        }
        
        if (pBIOSInfo->TVDIPort & VIA_DI_DFPHIGH)
        {
            write_reg_mask(CR96, VIACR, 0x04, BIT2+BIT1+BIT0);
            write_reg_mask(CR97, VIACR, 0x04, BIT2+BIT1+BIT0);
        }
        
        if (pBIOSInfo->TVDIPort & VIA_DI_DFPLOW)
        {
            write_reg_mask(CR9B, VIACR, 0x04, BIT2+BIT1+BIT0);
            write_reg_mask(CR99, VIACR, 0x04, BIT2+BIT1+BIT0);
        }
    }
    else
    {
        if (pBIOSInfo->TVDIPort & VIA_DI_DVP0)
        {
            write_reg_mask(CR96, VIACR, 0x01, BIT2+BIT1+BIT0);
        }
        
        if (pBIOSInfo->TVDIPort & VIA_DI_DVP1)
        {
            if (pBIOSInfo->Chipset == VIA_KM400 &&
                pBIOSInfo->ChipRev < 5)
            {
                /*=* VT3205 Ax *=*/
                write_reg_mask(CR9B, VIACR, 0x07, BIT2+BIT1+BIT0);
            }
            else
            {
                write_reg_mask(CR9B, VIACR, 0x05, BIT2+BIT1+BIT0);
            }
        }
        
        if (pBIOSInfo->TVDIPort & VIA_DI_DFPHIGH)
        {
            write_reg_mask(CR96, VIACR, 0x01, BIT2+BIT1+BIT0);
            write_reg_mask(CR97, VIACR, 0x01, BIT2+BIT1+BIT0);
        }
        
        if (pBIOSInfo->TVDIPort & VIA_DI_DFPLOW)
        {
            write_reg_mask(CR9B, VIACR, 0x01, BIT2+BIT1+BIT0);
            write_reg_mask(CR99, VIACR, 0x01, BIT2+BIT1+BIT0);
        }
    }
}


void VIATVPatch(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo)
{
    switch(pTVSettingInfo->TVEncoder)
    {
        case VIA_VT1622A:
        case VIA_VT1623:
            switch(pBIOSInfo->Chipset)
            {
                case VIA_K8M800:
                    VIATVPatchSkew_K8(pBIOSInfo, pTVSettingInfo);
                    break;
                default:
                    break;
            }
            
            break;
            
        case VIA_VT1625:
            if (pBIOSInfo->Chipset >= VIA_PM800)
            {
                VIATVPatchSkew_VT1625(pBIOSInfo, pTVSettingInfo);
            }
            else
            {
                /* For older chip such as VT3205. */
                VIATVPatchSkew_VT1625_Old(pBIOSInfo, pTVSettingInfo);
            }
            
            break;
            
        default:
            break;
    }

    /* RainLin [2006.10.18]: Replace the default patch if get user DPA settings from xorg: */
    if (pBIOSInfo->UserDPASettingInfo.HasUserSetting_Gfx)
    {
        VIASetDPA_Gfx(pBIOSInfo->TVDIPort, &(pBIOSInfo->UserDPASettingInfo.GfxDPA));
    }

    if (pBIOSInfo->UserDPASettingInfo.HasUserSetting_VT1625 && (pTVSettingInfo->TVEncoder == VIA_VT1625))
    {
        VIASetDPA_VT1625(pBIOSInfo, pBIOSInfo->UserDPASettingInfo.VT1625DPA);
    }

    if (pBIOSInfo->UserDPASettingInfo.HasUserSetting_VT1623 && 
        ((pTVSettingInfo->TVEncoder == VIA_VT1622A) || (pTVSettingInfo->TVEncoder == VIA_VT1623)))
    {
        VIASetDPA_VT1623(pBIOSInfo, pBIOSInfo->UserDPASettingInfo.VT1623DPA);
    }
}

/*=*
 *
 * int VIACheckTVExist(VIABIOSInfoPtr) - Check TV Endcoder
 *
 * Return Type:    int
 *
 * The Definition of Input Value:
 *
 *                 VIABIOSInfoPtr
 *
 * The Definition of Return Value:
 *
 *                 0 - None
 *                 1 - VIA VT1621
 *                 2 - VIA VT1622
 *                 3 - Chrontel 7009
 *                 4 - Chrontel 7019
 *                 5 - Philips SAA7108
 *=*/
int VIACheckTVExist(VIABIOSInfoPtr pBIOSInfo)
{
    I2CDevPtr       dev;
    unsigned char   W_Buffer[1], R_Buffer[1];
    unsigned char   sr1e = 0, sr2a = 0, sr3e = 0, cr6b = 0, cr6c = 0;

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIACheckTVExist\n"));
    
    if (pBIOSInfo->dwProjectIDRevision == UNICHROME_VT3259Lite)
    {
        return VIA_NONETV;
    }


    if (TVEncoderIdentify_VT1625(pBIOSInfo, &(pBIOSInfo->TVSettingInfo)))
    {        
        InitTV_VT1625(pBIOSInfo);

        if (!(pBIOSInfo->DisplayHardwareLayout == HW_LAYOUT_TV_EXTERNAL_TV2))
        {
            return pBIOSInfo->TVSettingInfo.TVEncoder;
        }
    }

    if (pBIOSInfo->Chipset == VIA_CLE266)
    {
        /*=* DI0 Pad on *=*/
        sr1e = read_reg(VIASR, SR1E);
        write_reg_mask(SR1E, VIASR, 0xC0, BIT6+BIT7);
        /*=* CR6B[0]VCK Input Selection: 1 = External clock. *=*/
        cr6b = read_reg(VIACR, CR6B);
        /*=* CR6C[0]TV Clock Input Enable *=*/
        cr6c = read_reg(VIACR, CR6C);
        if (pBIOSInfo->ChipRev < 15)
        {
            write_reg_mask(CR6B, VIACR, 0x1, BIT0);
            write_reg_mask(CR6C, VIACR, 0x1, BIT0);
        }
        else
        {
            write_reg_mask(CR6C, VIACR, 0x21, BIT0+BIT5);
        }
    }
    else
    {
        /*=* DVP0/DVP1 Pad on *=*/
        sr1e = read_reg(VIASR, SR1E);
        write_reg_mask(SR1E, VIASR, 0xF0, BIT4+BIT5+BIT6+BIT7);
        /*=* DFP High-half Pad on *=*/
        sr2a = read_reg(VIASR, SR2A);
        write_reg_mask(SR2A, VIASR, 0x0C, BIT3+BIT2);
        /*=* SR3E[1]Multi-function selection:
                    0 = Emulate I2C and DDC bus by GPIO2/3/4. *=*/
        sr3e = read_reg(VIASR, SR3E);
        write_reg_mask(SR3E, VIASR, 0x0, BIT5);
        /*=* CR6C[0]TV Clock Input Enable *=*/
        cr6c = read_reg(VIACR, CR6C);
        write_reg_mask(CR6C, VIACR, 0x21, BIT0+BIT5);
    }

    /* Check For TV2/TV3 */
    if (!pBIOSInfo->TVSettingInfo.TVEncoder)
    {
        if (xf86I2CProbeAddress(pBIOSInfo->I2C_Port2, 0x40)) {
            dev = xf86CreateI2CDevRec();
            dev->DevName = "TV";
            dev->SlaveAddr = 0x40;
            dev->pI2CBus = pBIOSInfo->I2C_Port2;

            if (xf86I2CDevInit(dev))
            {
                W_Buffer[0] = 0x1B;
                xf86I2CWriteRead(dev, W_Buffer,1, R_Buffer,1);
                xf86DestroyI2CDevRec(dev,TRUE);
                switch (R_Buffer[0])
                {
                    case 0x2:
                        DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED,
                                "Found TVEncoder VT1621!\n"));
                        pBIOSInfo->TVSettingInfo.TVEncoder = VIA_VT1621;
                        pBIOSInfo->TVI2CAdd = 0x40;
                        pBIOSInfo->TVDIPort = VIA_DI_DVP0;
                        break;
                    case 0x3:
                        DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED,
                                "Found TVEncoder VT1622!\n"));
                        pBIOSInfo->TVSettingInfo.TVEncoder = VIA_VT1622;
                        pBIOSInfo->TVI2CAdd = 0x40;
                        pBIOSInfo->TVDIPort = VIA_DI_DVP0;
                        break;
                    case 0x10:
                        DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED,
                                "Found TVEncoder VT1622A!\n"));
                        pBIOSInfo->TVSettingInfo.TVEncoder = VIA_VT1622A;
                        pBIOSInfo->TVI2CAdd = 0x40;
                        pBIOSInfo->TVDIPort = VIA_DI_DVP0;
                        break;
                    default:
                        pBIOSInfo->TVSettingInfo.TVEncoder = VIA_NONETV;
                        break;
                }
            }
            else
                xf86DestroyI2CDevRec(dev,TRUE);
        }
        else
            pBIOSInfo->TVSettingInfo.TVEncoder = VIA_NONETV;
    }


    if (pBIOSInfo->TVSettingInfo.TVEncoder == VIA_NONETV)
    {
        if (GPIOI2C_Initial(pBIOSInfo, 0x40))
        {
            GPIOI2C_Read(pBIOSInfo, 0x1B, R_Buffer, 1);
            switch (R_Buffer[0])
            {
                case 0x10:
                    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED,
                            "Found TVEncoder VT1623!\n"));
                    pBIOSInfo->TVSettingInfo.TVEncoder = VIA_VT1623;
                    pBIOSInfo->TVI2CAdd = 0x40;                    
                    break;
                    
                default:
                    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED,
                            "Unknow TVEncoder Type:%d!\n", R_Buffer[0]));
                    break;
            }
        }
    }

    /*=* VansonWu [2003.10.20] Due to TV3 has 3 level H-Scal, others only has
     *   2 level. If not TV3, we let overscan equal Fit for coding convenience.
     *=*/
    if (pBIOSInfo->TVSettingInfo.TVVScan == VIA_TVOVER &&
        pBIOSInfo->TVSettingInfo.TVEncoder != VIA_VT1622 &&
        pBIOSInfo->TVSettingInfo.TVEncoder != VIA_VT1622A &&
        pBIOSInfo->TVSettingInfo.TVEncoder != VIA_VT1623 &&
        pBIOSInfo->TVSettingInfo.TVEncoder != VIA_VT1625)
    {
    xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED,"VIACheckTVExist:TVVScan=%d!\n", pBIOSInfo->TVSettingInfo.TVVScan);    
        pBIOSInfo->TVSettingInfo.TVVScan = VIA_TVFIT;
    }

    if (pBIOSInfo->Chipset == VIA_CLE266)
    {
        write_reg(SR1E, VIASR, sr1e);
        write_reg(CR6B, VIACR, cr6b);
        write_reg(CR6C, VIACR, cr6c);
    }
    else
    {
        write_reg(SR1E, VIASR, sr1e);
        write_reg(SR2A, VIASR, sr2a);
        write_reg(SR3E, VIASR, sr3e);
        write_reg(CR6C, VIACR, cr6c);
    }

    if (pBIOSInfo->TVSettingInfo.TVEncoder == VIA_NONETV)
    {
        if (TVEncoderIdentify_IntegratedTV(pBIOSInfo, &(pBIOSInfo->TVSettingInfo)))
        {
            return pBIOSInfo->TVSettingInfo.TVEncoder;
        }
    }

    if (pBIOSInfo->DisplayHardwareLayout == HW_LAYOUT_TV_EXTERNAL_TV2)
    {        
        TVEncoderIdentify_IntegratedTV(pBIOSInfo, &(pBIOSInfo->TVSettingInfo2));
        DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED,
                "TV2 Encoder:%d!\n", pBIOSInfo->TVSettingInfo2.TVEncoder));
    }    
    
    return pBIOSInfo->TVSettingInfo.TVEncoder;
}

void load_tv_output_signal(VIABIOSInfoPtr pBIOSInfo, CARD16 *tv_func_reg, TVSETTINGINFOPTR pTVSettingInfo)
{
    I2CDevPtr       dev;
    unsigned char   W_Buffer[2];
    int             i = 0;

    while(tv_func_reg[i]!= 0xFFFF)
    {
        if((pTVSettingInfo->TVEncoder == VIA_VT1622A) ||
           (pTVSettingInfo->TVEncoder == VIA_VT1622))
        {
            dev = xf86CreateI2CDevRec();
            if(pTVSettingInfo->TVEncoder == VIA_VT1622) {
                dev->DevName = "VT1622";
            }
            else {
                dev->DevName = "VT1622A";
            }
            dev->SlaveAddr = 0x40;
            dev->pI2CBus = pBIOSInfo->I2C_Port2;

            xf86I2CDevInit(dev);

            W_Buffer[0] = GET_LOW_BYTE(tv_func_reg[i]);
            W_Buffer[1] = GET_HIGH_BYTE(tv_func_reg[i]);
            xf86I2CWriteRead(dev, W_Buffer, 2, NULL, 0);

            xf86DestroyI2CDevRec(dev, TRUE);
        }
        else if(pTVSettingInfo->TVEncoder == VIA_VT1625)
        {
            GPIOI2CWrite_TV(pBIOSInfo, GET_LOW_BYTE(tv_func_reg[i]), GET_HIGH_BYTE(tv_func_reg[i]));
        }
        else
        {
            GPIOI2C_Write(pBIOSInfo, GET_LOW_BYTE(tv_func_reg[i]), GET_HIGH_BYTE(tv_func_reg[i]));
        }
        DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "Write TV Reg Offset %#x value %#x!\n" , GET_LOW_BYTE(tv_func_reg[i]), GET_HIGH_BYTE(tv_func_reg[i])));
        i++;
    }
}

/* Check TV mode table is TV2+ or TV3 */
int VIAGetTVTabVer(VIABIOSInfoPtr pBIOSInfo, unsigned char *pBIOS)
{
    unsigned char       *pRom, *pTable;

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIAGetTVTabVer\n"));
    /* Get the start of Table */
    pRom = pBIOS + VIA_BIOS_OFFSET_POS;
    pTable = pBIOS + *((CARD16 *)pRom);

    /* Get start of TV Mask Table */
    pRom = pTable + VIA_BIOS_TVMASKTAB_POS;
    pRom = pBIOS + *((CARD16 *)pRom);

    pRom += 12;
    if (*((CARD16 *)pRom) == 0x03) {
        pBIOSInfo->BIOSTVTabVer = 2;
        return 2;
    }
    else {
        pBIOSInfo->BIOSTVTabVer = 3;
        return 3;
    }
}

void VIAEnableTV(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo)
{
    CARD8           W_Buffer[3];
    I2CDevPtr       dev;
    VIABIOSInfoPtr  pVia = pBIOSInfo;

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED, "VIAEnableTV\n"));

    if (pTVSettingInfo->TVEncoder == VIA_INTEGRATED_TV)
    {
        if (pBIOSInfo->Chipset == VIA_CX700)
        {
            EnableTV_IntegratedTV(pBIOSInfo, ITV_DAC_PRIMARY);
        }
        else if (pBIOSInfo->Chipset == VIA_CN750)
        {
            EnableTV_IntegratedTV(pBIOSInfo, ITV_DAC_SECONDARY);
        }
    }
    else
    {
        /*=* Fix CLE266 CD, when CRT+TV, TV will abnormal. Ian *=*/
        if (pBIOSInfo->dwProjectIDRevision == UNICHROME_VT3123AX)
        {
            switch (pTVSettingInfo->IGAPath)
            {
                case IGA1:
                    VIASetOutputPath(pBIOSInfo,VIA_DEVICE_TV,IGA1,pBIOSInfo->TVDIPort);
                    break;
                case IGA2:
                    VIASetOutputPath(pBIOSInfo,VIA_DEVICE_TV,IGA2,pBIOSInfo->TVDIPort);
                    break;
                case IGA1_IGA2:
                    break;
            }
        }

        {
            dev = xf86CreateI2CDevRec();
            dev->DevName = "TV";
            dev->SlaveAddr = pBIOSInfo->TVI2CAdd;
            dev->pI2CBus = pBIOSInfo->I2C_Port2;
            xf86I2CDevInit(dev);

            switch (pTVSettingInfo->TVEncoder)
            {
                case VIA_VT1621:
                case VIA_VT1622:
                case VIA_VT1622A:
                    W_Buffer[0] = 0x0E;
                    switch (pTVSettingInfo->TVOutput)
                    {
                        case TV_OUTPUT_COMPOSITE:
                            W_Buffer[1] = 0x07;
                            break;
                        case TV_OUTPUT_SVIDEO:
                            W_Buffer[1] = 0x09;
                            break;
                        case TV_OUTPUT_COMPOSITE_SVIDEO:
                            W_Buffer[1] = 0x01;
                            break;
                        case TV_OUTPUT_RGB:
                            W_Buffer[1] = 0x0;
                            break;
                        case TV_OUTPUT_YPBPR:
                            W_Buffer[1] = 0x08;
                            break;
                    }
                    xf86I2CWriteRead(dev, W_Buffer,2, NULL,0);
                    break;
                case VIA_VT1623:
                    W_Buffer[0] = 0x0E;
                    switch (pTVSettingInfo->TVOutput)
                    {
                        case TV_OUTPUT_COMPOSITE:
                            W_Buffer[1] = 0x07;
                            break;
                        case TV_OUTPUT_SVIDEO:
                            W_Buffer[1] = 0x09;
                            break;
                        case TV_OUTPUT_COMPOSITE_SVIDEO:
                            W_Buffer[1] = 0x01;
                            break;
                        case TV_OUTPUT_RGB:
                            W_Buffer[1] = 0x0;
                            break;
                        case TV_OUTPUT_YPBPR:
                            W_Buffer[1] = 0x08;
                            break;
                    }
                    GPIOI2C_Initial(pBIOSInfo, 0x40);
                    GPIOI2C_Write(pBIOSInfo, W_Buffer[0], W_Buffer[1]);
                    break;
                case VIA_VT1625:
                    EnableTV_VT1625(pBIOSInfo, pTVSettingInfo);
                    break;
                    
                default:
                    break;
            }
            xf86DestroyI2CDevRec(dev,TRUE);
        }   /*=* not Twin FS454 TV Encoder *=*/

        /* VansonWu [2003.8.27] Pad on/off according PMS */
        if (pBIOSInfo->TVDIPort & VIA_DI_DVP0)
        {
            VGAOUT8(0x3c4, 0x1E);
            VGAOUT8(0x3c5, VGAIN8(0x3c5) | 0xC0);
        }
        
        if (pBIOSInfo->TVDIPort & VIA_DI_DVP1)
        {
            VGAOUT8(0x3c4, 0x1E);
            VGAOUT8(0x3c5, VGAIN8(0x3c5) | 0x30);
            if (pBIOSInfo->Chipset == VIA_CLE266)
            {
                VGAOUT8(0x3d4, 0x93);
                VGAOUT8(0x3d5, VGAIN8(0x3d5) | 0x01);
            }
        }
        
        if (pBIOSInfo->TVDIPort & VIA_DI_DFPHIGH)
        {
            VGAOUT8(0x3c4, 0x2A);
            VGAOUT8(0x3c5, VGAIN8(0x3c5) | 0x0C);
        }
        
        if (pBIOSInfo->TVDIPort & VIA_DI_DFPLOW)
        {
            VGAOUT8(0x3c4, 0x2A);
            VGAOUT8(0x3c5, VGAIN8(0x3c5) | 0x0C);
        }
    }
}

void VIADisableTV(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo)
{
    CARD8           W_Buffer[3];
    I2CDevPtr       dev;
    VIABIOSInfoPtr  pVia = pBIOSInfo;

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED, "VIADisableTV\n"));

    if (pTVSettingInfo->TVEncoder == VIA_INTEGRATED_TV)
    {
        if (pBIOSInfo->Chipset == VIA_CX700)
        {
            DisableTV_IntegratedTV(pBIOSInfo, ITV_DAC_PRIMARY);
        }
        else if (pBIOSInfo->Chipset == VIA_CN750)
        {
            DisableTV_IntegratedTV(pBIOSInfo, ITV_DAC_SECONDARY);
        }
    }
    else
    {
        /* VansonWu [2003.8.27] Digital Interface Pad off */
        if (pBIOSInfo->TVDIPort & VIA_DI_DVP0)
        {
            VGAOUT8(0x3c4, 0x1E);
            VGAOUT8(0x3c5, VGAIN8(0x3c5) & 0x3F);
        }
        if (pBIOSInfo->TVDIPort & VIA_DI_DVP1)
        {
            VGAOUT8(0x3c4, 0x1E);
            VGAOUT8(0x3c5, VGAIN8(0x3c5) & 0xCF);
            if (pBIOSInfo->Chipset == VIA_CLE266)
            {
                VGAOUT8(0x3d4, 0x93);
                VGAOUT8(0x3d5, VGAIN8(0x3d5) & 0xFE);
            }
        }
        if (pBIOSInfo->TVDIPort & VIA_DI_DFPHIGH)
        {
            VGAOUT8(0x3c4, 0x2A);
            VGAOUT8(0x3c5, VGAIN8(0x3c5) & 0xF3);
        }
        if (pBIOSInfo->TVDIPort & VIA_DI_DFPLOW)
        {
            VGAOUT8(0x3c4, 0x2A);
            VGAOUT8(0x3c5, VGAIN8(0x3c5) & 0xFC);
        }

        {
            dev = xf86CreateI2CDevRec();
            dev->DevName = "TV";
            dev->SlaveAddr = pBIOSInfo->TVI2CAdd;
            dev->pI2CBus = pBIOSInfo->I2C_Port2;
            xf86I2CDevInit(dev);

            switch (pTVSettingInfo->TVEncoder)
            {
                case VIA_VT1621:
                    W_Buffer[0] = 0x0E;
                    W_Buffer[1] = 0x03;
                    xf86I2CWriteRead(dev, W_Buffer,2, NULL,0);
                    break;
                case VIA_VT1622:
                case VIA_VT1622A:
                    W_Buffer[0] = 0x0E;
                    W_Buffer[1] = 0x0F;
                    xf86I2CWriteRead(dev, W_Buffer,2, NULL,0);
                    break;
                case VIA_VT1623:
                    GPIOI2C_Initial(pBIOSInfo, 0x40);
                    GPIOI2C_Write(pBIOSInfo, 0x0E, 0x0F);
                    break;
                case VIA_VT1625:
                    DisableTV_VT1625(pBIOSInfo);
                    break;
                default:
                    break;
            }
            xf86DestroyI2CDevRec(dev,TRUE);
        }   /*=* not Twin FS454 TV Encoder *=*/
    }
}


/* Sense TV to determine whether TV is connect and choice the output signal. */
unsigned char VIAGetDeviceDetectTV(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo)
{
    unsigned char   ret = 0x0, sense;
    int DACSelect  = 0; /* for integrated tv */
        
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIAGetDeviceDetectTV\n"));


    switch(pTVSettingInfo->TVEncoder)
    {
        case VIA_NONETV:
            pBIOSInfo->TVAttach = FALSE;
            break;
        case VIA_VT1621:
            sense = VIASensorTV2(pBIOSInfo);
            if (!pTVSettingInfo->TVOutput && sense == 0x05)
            {
                pBIOSInfo->TVAttach = FALSE; /* No TV Output Attached */
            }
            else
            {
                ret = VIA_DEVICE_TV;
                pBIOSInfo->TVAttach = TRUE;
                if (!pTVSettingInfo->TVOutput)
                {
                    if (sense == 0)
                    {
                        /*tmp |= 0xC0;  Connect S_Video + Composite */
                        pTVSettingInfo->TVOutput = TV_OUTPUT_COMPOSITE_SVIDEO;
                    }
                    if (sense == 0x01)
                    {
                        /*tmp |= 0x80;  Connect S_Video */
                        pTVSettingInfo->TVOutput = TV_OUTPUT_SVIDEO;
                    }
                    else
                    {
                        /*tmp |= 0x40;  Connect Composite */
                        pTVSettingInfo->TVOutput = TV_OUTPUT_COMPOSITE;
                    }
                }
                DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED, "TV2 sense = %d ,TVOutput = %d\n", sense, pTVSettingInfo->TVOutput));
            }
            break;
        case VIA_VT1622:
        case VIA_VT1622A:
        case VIA_VT1623:
            sense = VIASensorTV3(pBIOSInfo);
            if (!pTVSettingInfo->TVOutput && sense == 0x0F)
            {
                pBIOSInfo->TVAttach = FALSE; /* No TV Output Attached */
            }
            else
            {
                ret = VIA_DEVICE_TV;
                pBIOSInfo->TVAttach = TRUE;
                if (!pTVSettingInfo->TVOutput)
                {
                    switch (sense)
                    {
                        case 0x0:   /* DAC A, B, C, D */
                            pTVSettingInfo->TVOutput = TV_OUTPUT_RGB;
                            break;
                        case 0x01:  /* DAC A, B, C */
                            pTVSettingInfo->TVOutput = TV_OUTPUT_COMPOSITE_SVIDEO;
                            break;
                        case 0x07:  /* DAC A */
                            pTVSettingInfo->TVOutput = TV_OUTPUT_COMPOSITE;
                            break;
                        case 0x08:  /* DAC B, C, D */
                            pTVSettingInfo->TVOutput = TV_OUTPUT_YPBPR;
                            break;
                        case 0x09:  /* DAC B, C */
                            pTVSettingInfo->TVOutput = TV_OUTPUT_SVIDEO;
                            break;
                        default:
                            pTVSettingInfo->TVOutput = TV_OUTPUT_COMPOSITE_SVIDEO;
                            break;
                    }
                }
                DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED, "TV3 sense = %d ,TVOutput = %d\n", sense, pTVSettingInfo->TVOutput));
            }
            break;
        case VIA_VT1625:
            sense = VIASensorVT1625(pBIOSInfo);
            if (!pTVSettingInfo->TVOutput && sense == 0x3F)
            {
                pBIOSInfo->TVAttach = FALSE; /* No TV Output Attached */
            }
            else
            {
                ret = VIA_DEVICE_TV;
                pBIOSInfo->TVAttach = TRUE;
                if (!pTVSettingInfo->TVOutput)
                {
                    switch (sense)
                    {
                        case 0x07:  /* DAC A, B, C */
                            pTVSettingInfo->TVOutput = TV_OUTPUT_COMPOSITE_SVIDEO;
                            break;
                        case 0x0F:  /* DAC A, B */
                            pTVSettingInfo->TVOutput = TV_OUTPUT_SVIDEO;
                            break;
                        case 0x38:  /* DAC D, E, F */
                            pTVSettingInfo->TVOutput = TV_OUTPUT_YPBPR;
                            break;
                        case 0x37:  /* DAC C */
                            pTVSettingInfo->TVOutput = TV_OUTPUT_COMPOSITE;
                            break;
#if 0
                        case 0x38:  /* DAC D, E, F */
                            pTVSettingInfo->TVOutput = TV_OUTPUT_RGB;
                            break;
#endif
                        case 0x0:  /* DAC A, B, C, D, E, F */
                        case 0x10:  /* S-Video C may not be detected. */
                            pTVSettingInfo->TVOutput = TV_OUTPUT_YPBPR_COMPOSITE_SVIDEO;
                            break;
                        default:
                            ErrorF("Unknown signal type!\n");
                            pTVSettingInfo->TVOutput = TV_OUTPUT_COMPOSITE;
                            break;
                    }
                }
                DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED, "VT1625 sense = %#x ,TVOutput = %d\n", sense, pTVSettingInfo->TVOutput));
            }
            ret |= VIA_DEVICE_CRT2; /*ian*/            
            break;

        case VIA_INTEGRATED_TV:
            /* Determine which DAC we should use according to chipset. */
            if (pBIOSInfo->Chipset == VIA_CX700)
            {
                DACSelect = ITV_DAC_PRIMARY;
            }
            else if (pBIOSInfo->Chipset == VIA_CN750)
            {
                DACSelect = ITV_DAC_SECONDARY;
            }

            /* start to sense */                        
            if (SenseIntegratedTV(pBIOSInfo, DACSelect, pTVSettingInfo) == ITV_VALUE_TV_CONNECTED)
            {
                ret = VIA_DEVICE_TV;
                pBIOSInfo->TVAttach = TRUE;
            }
            else
            {
                pBIOSInfo->TVAttach = FALSE;
            }
            
            break;
            
        default:
            break;
    }

    switch(pTVSettingInfo->TVOutput) 
    {
        case TV_OUTPUT_COMPOSITE:
            break;
        case TV_OUTPUT_SVIDEO:
            break;
        case TV_OUTPUT_RGB:
            break;
        case TV_OUTPUT_YPBPR:
            break;
        case TV_OUTPUT_COMPOSITE_SVIDEO:
            pTVSettingInfo->TVOutput=TV_OUTPUT_SVIDEO;
            break;
        case TV_OUTPUT_YPBPR_COMPOSITE_SVIDEO:
            pTVSettingInfo->TVOutput=TV_OUTPUT_YPBPR;
            break;
        case TV_OUTPUT_RGB_COMPOSITE_SVIDEO:
            pTVSettingInfo->TVOutput=TV_OUTPUT_RGB;
            break;
        default:
            pTVSettingInfo->TVOutput=TV_OUTPUT_SVIDEO;
    }
    
    return ret;
}

void VIASaveTV(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo, VIARegPtr save)
{
    VIABIOSInfoPtr  pVia = pBIOSInfo;
    I2CDevPtr       dev;
    unsigned char   W_Buffer[1], R_Buffer[1];
    int             i;

    switch (pTVSettingInfo->TVEncoder) {
        case VIA_NONETV:
            break;

        case VIA_INTEGRATED_TV:
            /* RainLin [2006.01.23]: It's better to save the original values before initialize, so */
            /* we move the call SaveTVRegs_IntegratedTV() before InitIntegratedTVRegs(). */
            break;

        case VIA_VT1625:
            SaveTVRegs_VT1625(pBIOSInfo, save);
            break;
            
        case VIA_VT1623:
            GPIOI2C_Initial(pBIOSInfo, 0x40);
            for (i = 0; i < 0x6C; i++)
            {
                GPIOI2C_ReadByte(pBIOSInfo, i, (save->TVRegs + i));
                /*save->TVRegs[i] = TVRegs[i];*/
                /*DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Save TV Register[%d]0x%X\n", i, save->TVRegs[i]));*/
            }
            break;
        case VIA_VT1621:
        case VIA_VT1622:
        case VIA_VT1622A:
            if (xf86I2CProbeAddress(pVia->I2C_Port2, pBIOSInfo->TVI2CAdd))
            {
                dev = xf86CreateI2CDevRec();
                dev->DevName = "TV";
                dev->SlaveAddr = pBIOSInfo->TVI2CAdd;
                dev->pI2CBus = pVia->I2C_Port2;

                if (xf86I2CDevInit(dev))
                {
                    for (i = 0; i < 0x8F; i++)
                    {
                        W_Buffer[0] = i;
                        xf86I2CWriteRead(dev, W_Buffer,1, R_Buffer, 1);
                        save->TVRegs[i] = R_Buffer[0];
                    }
                    xf86DestroyI2CDevRec(dev,TRUE);
                }
                else
                    xf86DestroyI2CDevRec(dev,TRUE);
            }
            break;
    }

}

void VIARestoreTV(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo, VIARegPtr restore)
{
    VIABIOSInfoPtr  pVia = pBIOSInfo;
    I2CDevPtr       dev;
    unsigned char   W_Buffer[3];
    int             i;

    switch (pTVSettingInfo->TVEncoder)
    {
        case VIA_VT1621:
        case VIA_VT1622:
        case VIA_VT1622A:
            dev = xf86CreateI2CDevRec();
            dev->DevName = "TV";
            dev->SlaveAddr = pBIOSInfo->TVI2CAdd;
            dev->pI2CBus = pVia->I2C_Port2;
            if (xf86I2CDevInit(dev))
            {
               
                for (i = 0; i < 0x8F; i++) {
                    W_Buffer[0] = (unsigned char)(i);
                    W_Buffer[1] = restore->TVRegs[i];
                    xf86I2CWriteRead(dev, W_Buffer, 2, NULL,0);
                }
                xf86DestroyI2CDevRec(dev,TRUE);
            }
            else
            {
                xf86DestroyI2CDevRec(dev,TRUE);
            }
            break;
        case VIA_VT1623:
            GPIOI2C_Initial(pBIOSInfo, 0x40);
            for (i = 0; i < 0x6C; i++)
            {
                GPIOI2C_Write(pBIOSInfo, i, restore->TVRegs[i]);
                /*DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Write Back Register[%d]:0x%X\n", i, restore->TVRegs[i]));*/
            }
            break;
        case VIA_VT1625:
            RestoreTVRegs_VT1625(pBIOSInfo, restore);
            break;
            
        case VIA_INTEGRATED_TV:
            RestoreTVRegs_IntegratedTV(pBIOSInfo, restore);
            break;
            
        default:
            break;
    }

}


void PlusVCKToIGA1Timing(CARD16 h_total)
{
    /* RainLin [2005.12.15]: Clear first. */
    write_reg_mask (CR47, VIACR, 0x00, BIT7+BIT6+BIT3);
    
    switch (h_total % 8)
    {
        case 2:
        {
            write_reg_mask (CR47, VIACR, BIT7, BIT7);
            break;
        }
        
        case 4:
        {
            write_reg_mask (CR47, VIACR, BIT6, BIT6);
            break;
        }
        
        case 6:
        {
            write_reg_mask (CR47, VIACR, BIT3, BIT3);
            break;
        }
        
        default:
        {
            break;
        }
    }
}

/* RainLin [2006.03.09]: Add this function to control TV clock source by case. */
void SetTVClockSource(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo)
{   
    if (pTVSettingInfo->TVEncoder == VIA_INTEGRATED_TV)
    {
        /* From TVPLL. */
        if (pTVSettingInfo->IGAPath == IGA1)
        {
            write_reg_mask(CR6C, VIACR, 0x50, 0xF0);
        }
        else
        {
            write_reg_mask(CR6C, VIACR, 0x05, 0x0F);
        }
    }
    else
    {
        /* External TV: */
        /* CarrieLiu [2006.07.26]: Correct CX700 External TV output port from DVP0 to DVP1 */
        switch(pBIOSInfo->Chipset)
        {
            case VIA_CX700:
                /* From DVP1 TV clock. */
                if (pTVSettingInfo->IGAPath == IGA1)
                {
                    write_reg_mask(CR6C, VIACR, 0xB0, 0xF0);
                }
                else
                {
                    write_reg_mask(CR6C, VIACR, 0x0B, 0x0F);
                }               
                break;
                
            case VIA_CN750:
                /* From DVP0 TV clock. */
                if (pTVSettingInfo->IGAPath == IGA1)
                {
                    write_reg_mask(CR6C, VIACR, 0x90, 0xF0);
                }
                else
                {
                    write_reg_mask(CR6C, VIACR, 0x09, 0x0F);
                }
                break;
                
            default:
                if (pTVSettingInfo->IGAPath == IGA1)
                {
                    write_reg_mask(CR6C, VIACR, BIT5+BIT0, BIT5+BIT0);
                }
                else
                {
                    write_reg_mask(CR6C, VIACR, BIT7+BIT5+BIT0, BIT7+BIT5+BIT0);
                }    
        }    
    }
}

void
VIADoTVSWReset(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo)
{
    /* RainLin [2006.05.23]: Do TV Software Reset can help TV image sync well with IGA1. */
    if (pTVSettingInfo->TVEncoder == VIA_INTEGRATED_TV)
    {
        DoTVSWReset_IntegratedTV(pBIOSInfo);
    }
    else if (pTVSettingInfo->TVEncoder == VIA_VT1625)
    {
        DoTVSWReset_VT1625(pBIOSInfo);
    }
}

