 /*
 * Copyright 1998-2007 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2007 S3 Graphics, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#include "via_driver.h"
#include "via_hw.h"
#include "via_tv.h"


/*=*
 *
 * unsigned char VIASensorTV2(VIABIOSInfoPtr pBIOSInfo)
 *
 *     - Sense TV2+ Encoder
 *
 * Return Type:    unsigned char
 *
 * The Definition of Input Value:
 *
 *                 VIABIOSInfoPtr
 *
 * The Definition of Return Value:
 *
 *                 Bit[2] Bit[0]
 *                 0      0      - Composite + S-Video
 *                 0      1      - S-Video
 *                 1      0      - Composite
 *                 1      1      - None
 *=*/
unsigned char VIASensorTV2(VIABIOSInfoPtr pBIOSInfo)
{
    I2CDevPtr       dev;
    unsigned char   save, tmp = 0x05;
    unsigned char   W_Buffer[2];
    unsigned char   R_Buffer[1];

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIASensorTV2\n"));
    dev = xf86CreateI2CDevRec();
    dev->DevName = "VT1621";
    dev->SlaveAddr = 0x40;
    dev->pI2CBus = pBIOSInfo->I2C_Port2;

    if (xf86I2CDevInit(dev))
    {
        W_Buffer[0] = 0x0E;
        xf86I2CWriteRead(dev, W_Buffer,1, R_Buffer,1);
        save = R_Buffer[0];
        W_Buffer[1] = 0x08;
        xf86I2CWriteRead(dev, W_Buffer,2, NULL,0);
        W_Buffer[1] = 0;
        xf86I2CWriteRead(dev, W_Buffer,2, NULL,0);
        W_Buffer[0] = 0x0F;
        xf86I2CWriteRead(dev, W_Buffer,1, R_Buffer,1);
        tmp = R_Buffer[0] & 0x0F;
        W_Buffer[0] = 0x0E;
        W_Buffer[1] = save;
        xf86I2CWriteRead(dev, W_Buffer,2, NULL,0);
        xf86DestroyI2CDevRec(dev,TRUE);
    }
    else
    {
        xf86DestroyI2CDevRec(dev,TRUE);
    }

    return tmp;
}

void VIAPreSetTV2Mode(VIABIOSInfoPtr pBIOSInfo)
{
    VIAModeTablePtr pViaModeTable = pBIOSInfo->pModeTable;
    VIABIOSTVMASKTablePtr   TVMaskTbl;
    CARD8           *TV;
    CARD16          *DotCrawl, *Patch2;
    unsigned int    tvIndx = pBIOSInfo->TVSettingInfo.resTVMode;
    int             i, j;
    unsigned char   W_Buffer[2];
    unsigned char   W_Other[2];
    unsigned char   R_Buffer[1];
    I2CDevPtr       dev;

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIAPreSetTV2Mode\n"));
    TV = NULL;
    DotCrawl = NULL;
    Patch2 = NULL;

    TVMaskTbl = &pViaModeTable->tv2MaskTable;
    if (pBIOSInfo->TVSettingInfo.TVType == TVTYPE_PAL)
    {
        switch (pBIOSInfo->TVSettingInfo.TVVScan)
        {
            case VIA_TVNORMAL:
                Patch2 = pViaModeTable->tv2Table[tvIndx].PatchPAL2;
                if (pBIOSInfo->TVSettingInfo.TVOutput == TV_OUTPUT_COMPOSITE)
                    TV = pViaModeTable->tv2Table[tvIndx].TVPALC;
                else
                    TV = pViaModeTable->tv2Table[tvIndx].TVPALS;
                break;
            case VIA_TVFIT:
                Patch2 = pViaModeTable->tv2OverTable[tvIndx].PatchPAL2;
                if (pBIOSInfo->TVSettingInfo.TVOutput == TV_OUTPUT_COMPOSITE)
                    TV = pViaModeTable->tv2OverTable[tvIndx].TVPALC;
                else
                    TV = pViaModeTable->tv2OverTable[tvIndx].TVPALS;
                break;
        }
    }
    else
    {
        switch (pBIOSInfo->TVSettingInfo.TVVScan)
        {
            case VIA_TVNORMAL:
                Patch2 = pViaModeTable->tv2Table[tvIndx].PatchNTSC2;
                DotCrawl = pViaModeTable->tv2Table[tvIndx].DotCrawlNTSC;
                if (pBIOSInfo->TVSettingInfo.TVOutput == TV_OUTPUT_COMPOSITE)
                    TV = pViaModeTable->tv2Table[tvIndx].TVNTSCC;
                else
                    TV = pViaModeTable->tv2Table[tvIndx].TVNTSCS;
                break;
            case VIA_TVFIT:
                Patch2 = pViaModeTable->tv2OverTable[tvIndx].PatchNTSC2;
                DotCrawl = pViaModeTable->tv2OverTable[tvIndx].DotCrawlNTSC;
                if (pBIOSInfo->TVSettingInfo.TVOutput == TV_OUTPUT_COMPOSITE)
                    TV = pViaModeTable->tv2OverTable[tvIndx].TVNTSCC;
                else
                    TV = pViaModeTable->tv2OverTable[tvIndx].TVNTSCS;
                break;
        }
    }

    dev = xf86CreateI2CDevRec();
    dev->DevName = "VT1621";
    dev->SlaveAddr = 0x40;
    dev->pI2CBus = pBIOSInfo->I2C_Port2;

    xf86I2CDevInit(dev);

    for (i = 0, j = 0; i < TVMaskTbl->numTV
         && j < VIA_BIOS_MAX_NUM_TV_REG; j++)
    {
        if (TVMaskTbl->TV[j] == 0xFF)
        {
            W_Buffer[0] = (unsigned char)j;
            W_Buffer[1] = TV[i];
            xf86I2CWriteRead(dev, W_Buffer,2, NULL,0);
            i++;
        }
    }

    /* Turn on all Composite and S-Video output */
    W_Other[0] = 0x0E;
    W_Other[1] = 0;
    xf86I2CWriteRead(dev, W_Other,2, NULL,0);

    if ((pBIOSInfo->TVSettingInfo.TVDotCrawl) &&
        (pBIOSInfo->TVSettingInfo.TVType == TVTYPE_NTSC))
    {
        int numReg = (int)(DotCrawl[0]);

        for (i = 1; i < (numReg + 1); i++)
        {
            W_Other[0] = (unsigned char)(DotCrawl[i] & 0xFF);
            if (W_Other[0] == 0x11)
            {
                xf86I2CWriteRead(dev, W_Other,1, R_Buffer,1);
                W_Other[1] = R_Buffer[0] | (unsigned char)(DotCrawl[i] >> 8);
            }
            else
            {
                W_Other[1] = (unsigned char)(DotCrawl[i] >> 8);
            }
            xf86I2CWriteRead(dev, W_Other,2, NULL,0);
        }
    }

    if ((pBIOSInfo->IsSecondary)||(pBIOSInfo->TVSettingInfo.IGAPath == IGA2))
    {
        int numPatch;

        /* Patch as setting 2nd path */
        numPatch = (int)(pViaModeTable->tv2MaskTable.misc2 >> 5);
        for (i = 0; i < numPatch; i++)
        {
            W_Other[0] = (unsigned char)(Patch2[i] & 0xFF);
            W_Other[1] = (unsigned char)(Patch2[i] >> 8);
            xf86I2CWriteRead(dev, W_Other,2, NULL,0);
        }
    }

    xf86DestroyI2CDevRec(dev,TRUE);
}

void VIAPostSetTV2Mode(VIABIOSInfoPtr pBIOSInfo)
{
    VIABIOSInfoPtr  pVia;
    VIAModeTablePtr pViaModeTable = pBIOSInfo->pModeTable;
    CARD8           *CRTC1, *CRTC2, *Misc1, *Misc2;
    unsigned int    tvIndx = pBIOSInfo->TVSettingInfo.resTVMode;
    int             i, j, data;


    pVia = pBIOSInfo;
    CRTC1 = NULL;
    CRTC2 = NULL;
    Misc1 = NULL;
    Misc2 = NULL;

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIAPostSetTV2Mode\n"));
    if (pBIOSInfo->TVSettingInfo.TVType == TVTYPE_PAL)
    {
        switch (pBIOSInfo->TVSettingInfo.TVVScan)
        {
            case VIA_TVNORMAL:
                CRTC1 = pViaModeTable->tv2Table[tvIndx].CRTCPAL1;
                switch (pBIOSInfo->bitsPerPixel)
                {
                    case 8:
                        CRTC2 = pViaModeTable->tv2Table[tvIndx].CRTCPAL2_8BPP;
                        break;
                    case 16:
                        CRTC2 = pViaModeTable->tv2Table[tvIndx].CRTCPAL2_16BPP;
                        break;
                    case 24:
                    case 32:
                        CRTC2 = pViaModeTable->tv2Table[tvIndx].CRTCPAL2_32BPP;
                        break;
                }
                Misc1 = pViaModeTable->tv2Table[tvIndx].MiscPAL1;
                Misc2 = pViaModeTable->tv2Table[tvIndx].MiscPAL2;
                break;
            case VIA_TVFIT:
                CRTC1 = pViaModeTable->tv2OverTable[tvIndx].CRTCPAL1;
                switch (pBIOSInfo->bitsPerPixel)
                {
                    case 8:
                        CRTC2 = pViaModeTable->tv2OverTable[tvIndx].CRTCPAL2_8BPP;
                        break;
                    case 16:
                        CRTC2 = pViaModeTable->tv2OverTable[tvIndx].CRTCPAL2_16BPP;
                        break;
                    case 24:
                    case 32:
                        CRTC2 = pViaModeTable->tv2OverTable[tvIndx].CRTCPAL2_32BPP;
                        break;
                }
                Misc1 = pViaModeTable->tv2OverTable[tvIndx].MiscPAL1;
                Misc2 = pViaModeTable->tv2OverTable[tvIndx].MiscPAL2;
                break;
            default:
                ErrorF("VIAPostSetTV2Mode: unknow TVVScan %d!\n", pBIOSInfo->TVSettingInfo.TVVScan);
                break;
        }
    }
    else
    {
        switch (pBIOSInfo->TVSettingInfo.TVVScan)
        {
            case VIA_TVNORMAL:
                CRTC1 = pViaModeTable->tv2Table[tvIndx].CRTCNTSC1;
                switch (pBIOSInfo->bitsPerPixel)
                {
                    case 8:
                        CRTC2 = pViaModeTable->tv2Table[tvIndx].CRTCNTSC2_8BPP;
                        break;
                    case 16:
                        CRTC2 = pViaModeTable->tv2Table[tvIndx].CRTCNTSC2_16BPP;
                        break;
                    case 24:
                    case 32:
                        CRTC2 = pViaModeTable->tv2Table[tvIndx].CRTCNTSC2_32BPP;
                        break;
                }
                Misc1 = pViaModeTable->tv2Table[tvIndx].MiscNTSC1;
                Misc2 = pViaModeTable->tv2Table[tvIndx].MiscNTSC2;
                break;
            case VIA_TVFIT:
                CRTC1 = pViaModeTable->tv2OverTable[tvIndx].CRTCNTSC1;
                switch (pBIOSInfo->bitsPerPixel)
                {
                    case 8:
                        CRTC2 = pViaModeTable->tv2OverTable[tvIndx].CRTCNTSC2_8BPP;
                        break;
                    case 16:
                        CRTC2 = pViaModeTable->tv2OverTable[tvIndx].CRTCNTSC2_16BPP;
                        break;
                    case 24:
                    case 32:
                        CRTC2 = pViaModeTable->tv2OverTable[tvIndx].CRTCNTSC2_32BPP;
                        break;
                }
                Misc1 = pViaModeTable->tv2OverTable[tvIndx].MiscNTSC1;
                Misc2 = pViaModeTable->tv2OverTable[tvIndx].MiscNTSC2;
                break;
            default:
                ErrorF("VIAPostSetTV2Mode: unknow TVVScan %d!\n", pBIOSInfo->TVSettingInfo.TVVScan);
                break;
        }
    }

    if ((pBIOSInfo->IsSecondary)||(pBIOSInfo->TVSettingInfo.IGAPath == IGA2))
    {
        for (i = 0, j = 0; i < pViaModeTable->tv2MaskTable.numCRTC2; j++)
        {
            if (pViaModeTable->tv2MaskTable.CRTC2[j] == 0xFF)
            {
                write_reg((CARD8)(j + 0x50), VIACR, CRTC2[j]);
                i++;
            }
        }

        j = 3;

        if (pViaModeTable->tv2MaskTable.misc2 & 0x18)
        {
            write_reg_mask(CR17, VIACR, 0x0, BIT7);

            /* CLE266Ax use 2x XCLK */
            if ((pBIOSInfo->Chipset == VIA_CLE266) &&
                (pBIOSInfo->ChipRev < 15))
            {
                write_reg(SR44, VIASR, 0x47);
                write_reg(SR45, VIASR, 0x1C);
                j+=2;
            }
            else
            {
#if 0
                write_reg(SR44, VIASR, Misc2[j++]);
                write_reg(SR45, VIASR, Misc2[j++]);
#else
                j+=2;
#endif
            }

            write_reg_mask(CR17, VIACR, 0x80, BIT7);

            /* Reset PLL */
            write_reg_mask(SR40, VIASR, 0x07, BIT0+BIT1+BIT2);
            write_reg_mask(SR40, VIASR, 0x0, BIT0+BIT1+BIT2);

            /* Use external clock */
            data = VGAIN8(0x3cc) | 0x0C;
            VGAOUT8(0x3c2, data);
        }

        /* RainLin [2005.11.22]: Modify for sequence issue. */
        EnableSecondDisplayChannel();

        if ((pBIOSInfo->Chipset == VIA_CLE266) &&
            (pBIOSInfo->ChipRev < 15))
        {
            /* CLE266Ax use 2x XCLK */
            write_reg_mask(CR6B, VIACR, 0x21, BIT0+BIT5);
            if (pBIOSInfo->CLE266A2)
            {
                write_reg_mask(CR6C, VIACR, 0x1D, BIT0+BIT2+BIT3+BIT4);
            }
            else
            {
                write_reg_mask(CR6C, VIACR, 0x01, BIT0);
            }
        }
        else
        {
#if 0
            write_reg_mask(CR6B, VIACR, 0x01, BIT0);
            write_reg_mask(CR6C, VIACR, 0x01, BIT0);
#else
            write_reg_mask(CR6C, VIACR, 0x21, BIT0+BIT5);
#endif
        }

		/* VansonWu [2003.8.27] add select data source according using digital interface */
        if (pBIOSInfo->Chipset != VIA_CLE266)
        {
            if (pBIOSInfo->TVDIPort & VIA_DI_DVP0)
            {
                /* Set DVP0 clock skew CR96[2:0] and data source CR96[4] */
                write_reg_mask(CR96, VIACR, 0x10, BIT4);
            }
            if (pBIOSInfo->TVDIPort & VIA_DI_DVP1)
            {
                /* Set DVP1 clock skew CR9B[2:0] and data source CR9B[4] */
                if (pBIOSInfo->Chipset == VIA_KM400 &&
                    pBIOSInfo->ChipRev < 5)
                {
                    /*=* VT3205 Ax *=*/
                    write_reg(CR9B, VIACR, 0x1F);
                }
                else
                {
                    write_reg(CR9B, VIACR, 0x15);
                }
            }
            if (pBIOSInfo->TVDIPort & VIA_DI_DFPHIGH)
            {
                write_reg_mask(CR96, VIACR, 0x10, BIT4);
                write_reg_mask(CR97, VIACR, 0x10, BIT4);
            }
            if (pBIOSInfo->TVDIPort & VIA_DI_DFPLOW)
            {
                write_reg_mask(CR9B, VIACR, 0x10, BIT4);
                write_reg_mask(CR99, VIACR, 0x10, BIT4);
            }
            /*=* VansonWu [2004.5.10] Fixed Set DVI+TV/DVT+CRT/CRT+TV SAMM mode,
                 the secondary monitor without display *=*/
            write_reg_mask(CR91, VIACR, 0x0, BIT5);
        }

        /* Disable LCD Scaling */
        write_reg(CR79, VIACR, 0x0);
    }
    else
    {
        for (i = 0, j = 0; i < pViaModeTable->tv2MaskTable.numCRTC1; j++)
        {
            if (pViaModeTable->tv2MaskTable.CRTC1[j] == 0xFF)
            {
                write_reg((CARD8)j, VIACR, CRTC1[j]);
                i++;
            }
        }

        j = 0;

        if (Misc1[j] & 0x20)
        {
            write_reg_mask(CR33, VIACR, 0x20, BIT5);
            j++;
        }
        else
        {
            write_reg_mask(CR33, VIACR, 0x0, BIT5);
            j++;
        }

        if ((pBIOSInfo->Chipset == VIA_CLE266) &&
            (pBIOSInfo->ChipRev < 15))
        {
            write_reg(CR6A, VIACR, Misc1[j++]);
            write_reg(CR6B, VIACR, Misc1[j++]);
            write_reg(CR6C, VIACR, Misc1[j++]);
        }
        else
        {
            j+=3;
        }

        if (pViaModeTable->tv2MaskTable.misc1 & 0x30)
        {
            /* reset engine */
            write_reg_mask(CR17, VIACR, 0x0, BIT7);

            /* CLE266Ax use 2x XCLK */
            if ((pBIOSInfo->Chipset == VIA_CLE266) &&
                (pBIOSInfo->ChipRev < 15))
            {
                write_reg(SR46, VIASR, 0x47);
                write_reg(SR47, VIASR, 0x1C);
                j+=2;
            }
            else
            {
#if 0
                write_reg(SR46, VIASR, Misc1[j++]);
                write_reg(SR47, VIASR, Misc1[j++]);
#else
                j+=2;
#endif
            }

            write_reg_mask(CR17, VIACR, 0x80, BIT7);
            /* reset engine :end */

            /* Reset PLL */
            write_reg_mask(SR40, VIASR, 0x07, BIT0+BIT1+BIT2);
            write_reg_mask(SR40, VIASR, 0x0, BIT0+BIT1+BIT2);

            /* Use external clock */
            data = VGAIN8(0x3cc) | 0x0C;
            VGAOUT8(0x3c2, data);
        }

        write_reg_mask(CR6A, VIACR, 0x40, BIT6);
        if ((pBIOSInfo->Chipset == VIA_CLE266) &&
            (pBIOSInfo->ChipRev < 15))
        {
            /* CLE266Ax use 2x XCLK */
            write_reg_mask(CR6B, VIACR, 0x81, BIT0+BIT7);
            if (pBIOSInfo->CLE266A2)
            {
                write_reg_mask(CR6C, VIACR, 0x11, BIT0+BIT4);
            }
            else
            {
                write_reg_mask(CR6C, VIACR, 0x01, BIT0);
            }
        }
        else
        {
#if 0
            write_reg(CR6B, VIACR, 0x01);
            write_reg_mask(CR6C, VIACR, 0x01, BIT0);
#else
            write_reg_mask(CR6C, VIACR, 0x21, BIT0+BIT5);
#endif
        }
    }
}

#ifdef CREATE_TV2_HEADERFILE
Bool
VIACreateTV2(VIAModeTablePtr pViaModeTable)
{
    int i, j;
    FILE *pFile;

    if ((pFile = fopen("via_tv2.h", "w+")) == NULL) {
	ErrorF("Can't open \"via_tv2.h\" file!!\n");
	return (FALSE);
    }

    fprintf(pFile, "#ifndef _VIA_TV2MODETABLE_H\n");
    fprintf(pFile, "#define _VIA_TV2MODETABLE_H\n");
    fprintf(pFile, "\n");

    /* tv2MaskTable */
    fprintf(pFile, "VIABIOSTVMASKTableRec tv2MaskTable = {\n");
    fprintf(pFile, "    { ");	/* tv2MaskTable.TV[] */
    for (i = 0; i < VIA_BIOS_MAX_NUM_TV_REG; i++) {
	if (i == (VIA_BIOS_MAX_NUM_TV_REG - 1))
	    fprintf(pFile, "%#X },\n",	/* tv2MaskTable.TV[] */
		    pViaModeTable->tv2MaskTable.TV[i]);
        else
	    fprintf(pFile, "%#X, ",
		    pViaModeTable->tv2MaskTable.TV[i]);
    }

    fprintf(pFile, "    { ");		/* tv2MaskTable.CRTC1[] */
    for (i = 0; i < VIA_BIOS_MAX_NUM_TV_CRTC; i++) {
	if (i == (VIA_BIOS_MAX_NUM_TV_CRTC - 1))
	    fprintf(pFile, "%#X },\n",	/* tv2MaskTable.CRTC1[] */
		    pViaModeTable->tv2MaskTable.CRTC1[i]);
	else
	    fprintf(pFile, "%#X, ",
		    pViaModeTable->tv2MaskTable.CRTC1[i]);
    }

    fprintf(pFile, "    { ");		/* tv2MaskTable.CRTC2[] */
    for (i = 0; i < VIA_BIOS_MAX_NUM_TV_CRTC; i++) {
	if (i == (VIA_BIOS_MAX_NUM_TV_CRTC - 1))
	    fprintf(pFile, "%#X },\n",	/* tv2MaskTable.CRTC2[] */
		    pViaModeTable->tv2MaskTable.CRTC2[i]);
	else
	    fprintf(pFile, "%#X, ",
		    pViaModeTable->tv2MaskTable.CRTC2[i]);
    }

    fprintf(pFile, "    %#X, %#X, %d, %d, %d \n};\n\n",
	    pViaModeTable->tv2MaskTable.misc1,
	    pViaModeTable->tv2MaskTable.misc2,
	    pViaModeTable->tv2MaskTable.numTV,
	    pViaModeTable->tv2MaskTable.numCRTC1,
	    pViaModeTable->tv2MaskTable.numCRTC2);

    fprintf(pFile, "VIABIOSTV2TableRec tv2Table[] = {\n");	/* tv2Table */
    for (i = 0; i < VIA_BIOS_NUM_TV2; i++) {
	fprintf(pFile, "    {\n");
	fprintf(pFile, "        { ");		/* tv2Table.TVNTSCC[] */
	for (j = 0; j < VIA_BIOS_MAX_NUM_TV_REG; j++) {
	    if (j == (VIA_BIOS_MAX_NUM_TV_REG - 1))
		fprintf(pFile, "%#X },\n",	/* tv2Table.TVNTSCC[] */
			pViaModeTable->tv2Table[i].TVNTSCC[j]);
	    else
		fprintf(pFile, "%#X, ",
			pViaModeTable->tv2Table[i].TVNTSCC[j]);
	}

	fprintf(pFile, "        { ");		/* tv2Table.TVNTSCS[] */
	for (j = 0; j < VIA_BIOS_MAX_NUM_TV_REG; j++) {
	    if (j == (VIA_BIOS_MAX_NUM_TV_REG - 1))
		fprintf(pFile, "%#X },\n",	/* tv2Table.TVNTSCS[] */
			pViaModeTable->tv2Table[i].TVNTSCS[j]);
	    else
		fprintf(pFile, "%#X, ",
			pViaModeTable->tv2Table[i].TVNTSCS[j]);
	}

	fprintf(pFile, "        { ");		/* tv2Table.CRTCNTSC1[] */
	for (j = 0; j < VIA_BIOS_MAX_NUM_TV_CRTC; j++) {
	    if (j == (VIA_BIOS_MAX_NUM_TV_CRTC - 1))
		fprintf(pFile, "%#X },\n",	/* tv2Table.CRTCNTSC1[] */
			pViaModeTable->tv2Table[i].CRTCNTSC1[j]);
	    else
		fprintf(pFile, "%#X, ",
			pViaModeTable->tv2Table[i].CRTCNTSC1[j]);
	}

	fprintf(pFile, "        { ");		/* tv2Table.MiscNTSC1[] */
	for (j = 0; j < VIA_BIOS_NUM_TV_SPECIAL_REG; j++) {
	    if (j == (VIA_BIOS_NUM_TV_SPECIAL_REG - 1))
		fprintf(pFile, "%#X },\n",	/* tv2Table.MiscNTSC1[] */
			pViaModeTable->tv2Table[i].MiscNTSC1[j]);
	    else
		fprintf(pFile, "%#X, ",
			pViaModeTable->tv2Table[i].MiscNTSC1[j]);
	}

	fprintf(pFile, "        { ");		/* tv2Table.MiscNTSC2[] */
        for (j = 0; j < VIA_BIOS_NUM_TV_SPECIAL_REG; j++) {
            if (j == (VIA_BIOS_NUM_TV_SPECIAL_REG - 1))
                fprintf(pFile, "%#X },\n",	/* tv2Table.MiscNTSC2[] */
                        pViaModeTable->tv2Table[i].MiscNTSC2[j]);
            else
                fprintf(pFile, "%#X, ",
			pViaModeTable->tv2Table[i].MiscNTSC2[j]);
        }

        fprintf(pFile, "        { ");		/* tv2Table.CRTCNTSC2_8BPP[] */
	for (j = 0; j < VIA_BIOS_MAX_NUM_TV_CRTC; j++) {
	    if (j == (VIA_BIOS_MAX_NUM_TV_CRTC - 1))
		fprintf(pFile, "%#X },\n",	/* tv2Table.CRTCNTSC2_8BPP[] */
			pViaModeTable->tv2Table[i].CRTCNTSC2_8BPP[j]);
	    else
		fprintf(pFile, "%#X, ",
			pViaModeTable->tv2Table[i].CRTCNTSC2_8BPP[j]);
	}

	fprintf(pFile, "        { ");		/* tv2Table.CRTCNTSC2_16BPP[] */
	for (j = 0; j < VIA_BIOS_MAX_NUM_TV_CRTC; j++) {
	    if (j == (VIA_BIOS_MAX_NUM_TV_CRTC - 1))
		fprintf(pFile, "%#X },\n",	/* tv2Table.CRTCNTSC2_16BPP[] */
			pViaModeTable->tv2Table[i].CRTCNTSC2_16BPP[j]);
	    else
		fprintf(pFile, "%#X, ",
			pViaModeTable->tv2Table[i].CRTCNTSC2_16BPP[j]);
	}

	fprintf(pFile, "        { ");		/* tv2Table.CRTCNTSC2_32BPP[] */
	for (j = 0; j < VIA_BIOS_MAX_NUM_TV_CRTC; j++) {
	    if (j == (VIA_BIOS_MAX_NUM_TV_CRTC - 1))
		fprintf(pFile, "%#X },\n",	/* tv2Table.CRTCNTSC2_32BPP[] */
			pViaModeTable->tv2Table[i].CRTCNTSC2_32BPP[j]);
	    else
		fprintf(pFile, "%#X, ",
			pViaModeTable->tv2Table[i].CRTCNTSC2_32BPP[j]);
	}

	fprintf(pFile, "        { ");		/* tv2Table.PatchNTSC2[] */
	for (j = 0; j < VIA_BIOS_MAX_NUM_TV_PATCH; j++) {
	    if (j == (VIA_BIOS_MAX_NUM_TV_PATCH - 1))
		fprintf(pFile, "%#X },\n",	/* tv2Table.PatchNTSC[] */
			pViaModeTable->tv2Table[i].PatchNTSC2[j]);
	    else
		fprintf(pFile, "%#X, ",
			pViaModeTable->tv2Table[i].PatchNTSC2[j]);
	}

	fprintf(pFile, "        { ");		/* tv2Table.DotCrawlNTSC[] */
	for (j = 0; j < VIA_BIOS_NUM_TV_OTHER; j++) {
	    if (j == (VIA_BIOS_NUM_TV_OTHER - 1))
		fprintf(pFile, "%#X },\n",	/* tv2Table.DotCrawlNTSC[] */
			pViaModeTable->tv2Table[i].DotCrawlNTSC[j]);
	    else
		fprintf(pFile, "%#X, ",
			pViaModeTable->tv2Table[i].DotCrawlNTSC[j]);
	}

	fprintf(pFile, "        { ");		/* tv2Table.TVPALC[] */
	for (j = 0; j < VIA_BIOS_MAX_NUM_TV_REG; j++) {
	    if (j == (VIA_BIOS_MAX_NUM_TV_REG - 1))
		fprintf(pFile, "%#X },\n",	/* tv2Table.TVPALC[] */
			pViaModeTable->tv2Table[i].TVPALC[j]);
	    else
		fprintf(pFile, "%#X, ",
			pViaModeTable->tv2Table[i].TVPALC[j]);
	}

	fprintf(pFile, "        { ");		/* tv2Table.TVPALS[] */
	for (j = 0; j < VIA_BIOS_MAX_NUM_TV_REG; j++) {
	    if (j == (VIA_BIOS_MAX_NUM_TV_REG - 1))
		fprintf(pFile, "%#X },\n",	/* tv2Table.TVPALS[] */
			pViaModeTable->tv2Table[i].TVPALS[j]);
	    else
		fprintf(pFile, "%#X, ",
			pViaModeTable->tv2Table[i].TVPALS[j]);
	}

	fprintf(pFile, "        { ");		/* tv2Table.CRTCPAL1[] */
	for (j = 0; j < VIA_BIOS_MAX_NUM_TV_CRTC; j++) {
	    if (j == (VIA_BIOS_MAX_NUM_TV_CRTC - 1))
		fprintf(pFile, "%#X },\n",	/* tv2Table.CRTCPAL1[] */
			pViaModeTable->tv2Table[i].CRTCPAL1[j]);
	    else
		fprintf(pFile, "%#X, ",
			pViaModeTable->tv2Table[i].CRTCPAL1[j]);
	}

	fprintf(pFile, "        { ");		/* tv2Table.MiscPAL1[] */
	for (j = 0; j < VIA_BIOS_NUM_TV_SPECIAL_REG; j++) {
	    if (j == (VIA_BIOS_NUM_TV_SPECIAL_REG - 1))
		fprintf(pFile, "%#X },\n",	/* tv2Table.MiscPAL1[] */
			pViaModeTable->tv2Table[i].MiscPAL1[j]);
	    else
		fprintf(pFile, "%#X, ",
			pViaModeTable->tv2Table[i].MiscPAL1[j]);
        }

	fprintf(pFile, "        { ");		/* tv2Table.MiscPAL2[] */
	for (j = 0; j < VIA_BIOS_NUM_TV_SPECIAL_REG; j++) {
	    if (j == (VIA_BIOS_NUM_TV_SPECIAL_REG - 1))
		fprintf(pFile, "%#X },\n",	/* tv2Table.MiscPAL2[] */
			pViaModeTable->tv2Table[i].MiscPAL2[j]);
	    else
		fprintf(pFile, "%#X, ",
			pViaModeTable->tv2Table[i].MiscPAL2[j]);
	}

	fprintf(pFile, "        { ");		/* tv2Table.CRTCPAL2_8BPP[] */
	for (j = 0; j < VIA_BIOS_MAX_NUM_TV_CRTC; j++) {
	    if (j == (VIA_BIOS_MAX_NUM_TV_CRTC - 1))
		fprintf(pFile, "%#X },\n",	/* tv2Table.CRTCPAL2_8BPP[] */
			pViaModeTable->tv2Table[i].CRTCPAL2_8BPP[j]);
	    else
		fprintf(pFile, "%#X, ",
			pViaModeTable->tv2Table[i].CRTCPAL2_8BPP[j]);
        }

	fprintf(pFile, "        { ");		/* tv2Table.CRTCPAL2_16BPP[] */
	for (j = 0; j < VIA_BIOS_MAX_NUM_TV_CRTC; j++) {
	    if (j == (VIA_BIOS_MAX_NUM_TV_CRTC - 1))
		fprintf(pFile, "%#X },\n",	/* tv2Table.CRTCPAL2_16BPP[] */
			pViaModeTable->tv2Table[i].CRTCPAL2_16BPP[j]);
	    else
		fprintf(pFile, "%#X, ",
			pViaModeTable->tv2Table[i].CRTCPAL2_16BPP[j]);
	}

	fprintf(pFile, "        { ");		/* tv2Table.CRTCPAL2_32BPP[] */
	for (j = 0; j < VIA_BIOS_MAX_NUM_TV_CRTC; j++) {
	    if (j == (VIA_BIOS_MAX_NUM_TV_CRTC - 1))
		fprintf(pFile, "%#X },\n",	/* tv2Table.CRTCPAL2_32BPP[] */
			pViaModeTable->tv2Table[i].CRTCPAL2_32BPP[j]);
	    else
		fprintf(pFile, "%#X, ",
			pViaModeTable->tv2Table[i].CRTCPAL2_32BPP[j]);
	}

	fprintf(pFile, "        { ");		/* tv2Table.PatchPAL2[] */
	for (j = 0; j < VIA_BIOS_MAX_NUM_TV_PATCH; j++) {
	    if (j == (VIA_BIOS_MAX_NUM_TV_PATCH - 1))
		fprintf(pFile, "%#X }\n",	/* tv2Table.PatchPAL2[] */
			pViaModeTable->tv2Table[i].PatchPAL2[j]);
	    else
		fprintf(pFile, "%#X, ",
			pViaModeTable->tv2Table[i].PatchPAL2[j]);
	}

	if (i == (VIA_BIOS_NUM_TV2 - 1))
	    fprintf(pFile, "    }\n};\n\n");
	else
	    fprintf(pFile, "    }, \n");
    }

    /* tv2OverTable */
    fprintf(pFile, "VIABIOSTV2TableRec tv2OverTable[] = {\n");
    for (i = 0; i < VIA_BIOS_NUM_TV2; i++) {
	fprintf(pFile, "    {\n");
	fprintf(pFile, "        { ");		/* tv2OverTable.TVNTSCC[] */
	for (j = 0; j < VIA_BIOS_MAX_NUM_TV_REG; j++) {
	    if (j == (VIA_BIOS_MAX_NUM_TV_REG - 1))
		fprintf(pFile, "%#X },\n",	/* tv2OverTable.TVNTSCC[] */
			pViaModeTable->tv2OverTable[i].TVNTSCC[j]);
	    else
		fprintf(pFile, "%#X, ",
			pViaModeTable->tv2OverTable[i].TVNTSCC[j]);
	}

	fprintf(pFile, "        { ");		/* tv2OverTable.TVNTSCS[] */
	for (j = 0; j < VIA_BIOS_MAX_NUM_TV_REG; j++) {
	    if (j == (VIA_BIOS_MAX_NUM_TV_REG - 1))
		fprintf(pFile, "%#X },\n",	/* tv2OverTable.TVNTSCS[] */
			pViaModeTable->tv2OverTable[i].TVNTSCS[j]);
	    else
		fprintf(pFile, "%#X, ",
			pViaModeTable->tv2OverTable[i].TVNTSCS[j]);
	}

	fprintf(pFile, "        { ");		/* tv2OverTable.CRTCNTSC1[] */
	for (j = 0; j < VIA_BIOS_MAX_NUM_TV_CRTC; j++) {
	    if (j == (VIA_BIOS_MAX_NUM_TV_CRTC - 1))
		fprintf(pFile, "%#X },\n",	/* tv2OverTable.CRTCNTSC1[] */
			pViaModeTable->tv2OverTable[i].CRTCNTSC1[j]);
	    else
		fprintf(pFile, "%#X, ",
			pViaModeTable->tv2OverTable[i].CRTCNTSC1[j]);
	}

	fprintf(pFile, "        { ");		/* tv2OverTable.MiscNTSC1[] */
	for (j = 0; j < VIA_BIOS_NUM_TV_SPECIAL_REG; j++) {
	    if (j == (VIA_BIOS_NUM_TV_SPECIAL_REG - 1))
		fprintf(pFile, "%#X },\n", /* tv2OverTable.MiscNTSC1[] */
			pViaModeTable->tv2OverTable[i].MiscNTSC1[j]);
	    else
		fprintf(pFile, "%#X, ",
			pViaModeTable->tv2OverTable[i].MiscNTSC1[j]);
	}

	fprintf(pFile, "        { ");		/* tv2OverTable.MiscNTSC2[] */
	for (j = 0; j < VIA_BIOS_NUM_TV_SPECIAL_REG; j++) {
	    if (j == (VIA_BIOS_NUM_TV_SPECIAL_REG - 1))
		fprintf(pFile, "%#X },\n", /* tv2OverTable.MiscNTSC2[] */
			pViaModeTable->tv2OverTable[i].MiscNTSC2[j]);
	    else
		fprintf(pFile, "%#X, ",
			pViaModeTable->tv2OverTable[i].MiscNTSC2[j]);
	}

	fprintf(pFile, "        { ");		/* tv2OverTable.CRTCNTSC2_8BPP[] */
	for (j = 0; j < VIA_BIOS_MAX_NUM_TV_CRTC; j++) {
	    if (j == (VIA_BIOS_MAX_NUM_TV_CRTC - 1))
		fprintf(pFile, "%#X },\n",	/* tv2OverTable.CRTCNTSC2_8BPP[] */
			pViaModeTable->tv2OverTable[i].CRTCNTSC2_8BPP[j]);
	    else
		fprintf(pFile, "%#X, ",
			pViaModeTable->tv2OverTable[i].CRTCNTSC2_8BPP[j]);
	}

	fprintf(pFile, "        { ");		/* tv2OverTable.CRTCNTSC2_16BPP[] */
	for (j = 0; j < VIA_BIOS_MAX_NUM_TV_CRTC; j++) {
	    if (j == (VIA_BIOS_MAX_NUM_TV_CRTC - 1))
		fprintf(pFile, "%#X },\n",	/* tv2OverTable.CRTCNTSC2_16BPP[] */
			pViaModeTable->tv2OverTable[i].CRTCNTSC2_16BPP[j]);
	    else
		fprintf(pFile, "%#X, ",
			pViaModeTable->tv2OverTable[i].CRTCNTSC2_16BPP[j]);
	}

	fprintf(pFile, "        { ");		/* tv2OverTable.CRTCNTSC2_32BPP[] */
	for (j = 0; j < VIA_BIOS_MAX_NUM_TV_CRTC; j++) {
	    if (j == (VIA_BIOS_MAX_NUM_TV_CRTC - 1))
		fprintf(pFile, "%#X },\n",	/* tv2OverTable.CRTCNTSC2_32BPP[] */
			pViaModeTable->tv2OverTable[i].CRTCNTSC2_32BPP[j]);
	    else
		fprintf(pFile, "%#X, ",
			pViaModeTable->tv2OverTable[i].CRTCNTSC2_32BPP[j]);
	}

	fprintf(pFile, "        { ");		/* tv2OverTable.PatchNTSC2[] */
	for (j = 0; j < VIA_BIOS_MAX_NUM_TV_PATCH; j++) {
	    if (j == (VIA_BIOS_MAX_NUM_TV_PATCH - 1))
		fprintf(pFile, "%#X },\n",	/* tv2OverTable.PatchNTSC[] */
			pViaModeTable->tv2OverTable[i].PatchNTSC2[j]);
	    else
		fprintf(pFile, "%#X, ",
			pViaModeTable->tv2OverTable[i].PatchNTSC2[j]);
	}

	fprintf(pFile, "        { ");		/* tv2OverTable.DotCrawlNTSC[] */
	for (j = 0; j < VIA_BIOS_NUM_TV_OTHER; j++) {
	    if (j == (VIA_BIOS_NUM_TV_OTHER - 1))
		fprintf(pFile, "%#X },\n",	/* tv2OverTable.DotCrawlNTSC[] */
			pViaModeTable->tv2OverTable[i].DotCrawlNTSC[j]);
	    else
		fprintf(pFile, "%#X, ",
			pViaModeTable->tv2OverTable[i].DotCrawlNTSC[j]);
	}

	fprintf(pFile, "        { ");		/* tv2OverTable.TVPALC[] */
	for (j = 0; j < VIA_BIOS_MAX_NUM_TV_REG; j++) {
	    if (j == (VIA_BIOS_MAX_NUM_TV_REG - 1))
		fprintf(pFile, "%#X },\n",	/* tv2OverTable.TVPALC[] */
			pViaModeTable->tv2OverTable[i].TVPALC[j]);
	    else
		fprintf(pFile, "%#X, ",
			pViaModeTable->tv2OverTable[i].TVPALC[j]);
	}

	fprintf(pFile, "        { ");		/* tv2OverTable.TVPALS[] */
	for (j = 0; j < VIA_BIOS_MAX_NUM_TV_REG; j++) {
	    if (j == (VIA_BIOS_MAX_NUM_TV_REG - 1))
		fprintf(pFile, "%#X },\n",	/* tv2OverTable.TVPALS[] */
			pViaModeTable->tv2OverTable[i].TVPALS[j]);
	    else
		fprintf(pFile, "%#X, ",
			pViaModeTable->tv2OverTable[i].TVPALS[j]);
	}

	fprintf(pFile, "        { ");		/* tv2OverTable.CRTCPAL1[] */
	for (j = 0; j < VIA_BIOS_MAX_NUM_TV_CRTC; j++) {
	    if (j == (VIA_BIOS_MAX_NUM_TV_CRTC - 1))
		fprintf(pFile, "%#X },\n",	/* tv2OverTable.CRTCPAL1[] */
			pViaModeTable->tv2OverTable[i].CRTCPAL1[j]);
	    else
		fprintf(pFile, "%#X, ",
			pViaModeTable->tv2OverTable[i].CRTCPAL1[j]);
	}

	fprintf(pFile, "        { ");		/* tv2OverTable.MiscPAL1[] */
	for (j = 0; j < VIA_BIOS_NUM_TV_SPECIAL_REG; j++) {
	    if (j == (VIA_BIOS_NUM_TV_SPECIAL_REG - 1))
		fprintf(pFile, "%#X },\n",	/* tv2OverTable.MiscPAL1[] */
			pViaModeTable->tv2OverTable[i].MiscPAL1[j]);
	    else
		fprintf(pFile, "%#X, ",
			pViaModeTable->tv2OverTable[i].MiscPAL1[j]);
	}

	fprintf(pFile, "        { ");		/* tv2OverTable.MiscPAL2[] */
	for (j = 0; j < VIA_BIOS_NUM_TV_SPECIAL_REG; j++) {
	    if (j == (VIA_BIOS_NUM_TV_SPECIAL_REG - 1))
		fprintf(pFile, "%#X },\n",	/* tv2OverTable.MiscPAL2[] */
			pViaModeTable->tv2OverTable[i].MiscPAL2[j]);
	    else
		fprintf(pFile, "%#X, ",
			pViaModeTable->tv2OverTable[i].MiscPAL2[j]);
	}

	fprintf(pFile, "        { ");		/* tv2OverTable.CRTCPAL2_8BPP[] */
	for (j = 0; j < VIA_BIOS_MAX_NUM_TV_CRTC; j++) {
	    if (j == (VIA_BIOS_MAX_NUM_TV_CRTC - 1))
		fprintf(pFile, "%#X },\n",	/* tv2OverTable.CRTCPAL2_8BPP[] */
			pViaModeTable->tv2OverTable[i].CRTCPAL2_8BPP[j]);
	    else
		fprintf(pFile, "%#X, ",
			pViaModeTable->tv2OverTable[i].CRTCPAL2_8BPP[j]);
	}

	fprintf(pFile, "        { ");		/* tv2OverTable.CRTCPAL2_16BPP[] */
	for (j = 0; j < VIA_BIOS_MAX_NUM_TV_CRTC; j++) {
	    if (j == (VIA_BIOS_MAX_NUM_TV_CRTC - 1))
		fprintf(pFile, "%#X },\n",	/* tv2OverTable.CRTCPAL2_16BPP[] */
			pViaModeTable->tv2OverTable[i].CRTCPAL2_16BPP[j]);
	    else
		fprintf(pFile, "%#X, ",
			pViaModeTable->tv2OverTable[i].CRTCPAL2_16BPP[j]);
	}

	fprintf(pFile, "        { ");		/* tv2OverTable.CRTCPAL2_32BPP[] */
	for (j = 0; j < VIA_BIOS_MAX_NUM_TV_CRTC; j++) {
	    if (j == (VIA_BIOS_MAX_NUM_TV_CRTC - 1))
		fprintf(pFile, "%#X },\n",	/* tv2OverTable.CRTCPAL2_32BPP[] */
                        pViaModeTable->tv2OverTable[i].CRTCPAL2_32BPP[j]);
	    else
		fprintf(pFile, "%#X, ",
			pViaModeTable->tv2OverTable[i].CRTCPAL2_32BPP[j]);
	}

	fprintf(pFile, "        { ");		/* tv2Table.PatchPAL2[] */
	for (j = 0; j < VIA_BIOS_MAX_NUM_TV_PATCH; j++) {
	    if (j == (VIA_BIOS_MAX_NUM_TV_PATCH - 1))
		fprintf(pFile, "%#X }\n",	/* tv2Table.PatchPAL2[] */
			pViaModeTable->tv2OverTable[i].PatchPAL2[j]);
	    else
		fprintf(pFile, "%#X, ",
			pViaModeTable->tv2OverTable[i].PatchPAL2[j]);
	}

	if (i == (VIA_BIOS_NUM_TV2 - 1))
	    fprintf(pFile, "    }\n};\n");
	else
	    fprintf(pFile, "    }, \n");
    }

    fprintf(pFile, "\n");
    fprintf(pFile, "#endif\n");

    return (TRUE);
}

void
VIAGetTV2Mask(VIAModeTablePtr pViaModeTable,
	      unsigned char *pBIOS,
	      unsigned char *pTable)
{
    unsigned char *pRom;
    int i, j, k, m;
    CARD16 tmp;

    DEBUG(xf86Msg(X_INFO, "VIAGetTV2Mask\n"));
    /* Get start of TV Mask Table */
    pRom = pTable + VIA_BIOS_TVMASKTAB_POS;
    pRom = pBIOS + *((CARD16 *)pRom);

    for (i = 0, j = 0, k = 0; i < 9; i++) {
	tmp = *((CARD16 *)pRom);
	for (m = 0; m < 16; m++, j++) {
	    if ((tmp >> m) & 0x01) {
		pViaModeTable->tv2MaskTable.TV[j] = 0xFF;
		k++;
            }
	    else {
		pViaModeTable->tv2MaskTable.TV[j] = 0;
	    }
	}
	pRom += 2;
    }
    pViaModeTable->tv2MaskTable.numTV = k;

    for (i = 0, j = 0, k = 0; i < 2; i++) {
	tmp = *((CARD16 *)pRom);
	for (m = 0; m < 16; m++, j++) {
	    if ((tmp >> m) & 0x01) {
		pViaModeTable->tv2MaskTable.CRTC1[j] = 0xFF;
		k++;
	    }
	    else {
		pViaModeTable->tv2MaskTable.CRTC1[j] = 0;
	    }
	}
	pRom += 2;
    }
    pViaModeTable->tv2MaskTable.numCRTC1 = k;

    pViaModeTable->tv2MaskTable.misc1 = *pRom++;

    for (i = 0, j = 0, k = 0; i < 2; i++) {
	tmp = *((CARD16 *)pRom);
	for (m = 0; m < 16; m++, j++) {
	    if ((tmp >> m) & 0x01) {
		pViaModeTable->tv2MaskTable.CRTC2[j] = 0xFF;
		k++;
	    }
	    else {
		pViaModeTable->tv2MaskTable.CRTC2[j] = 0;
	    }
	}
	pRom += 2;
    }
    pViaModeTable->tv2MaskTable.numCRTC2 = k;

    pViaModeTable->tv2MaskTable.misc2 = *pRom++;
}

void
VIAGetTV2NTSC(VIAModeTablePtr pViaModeTable,
	      unsigned char *pBIOS,
	      unsigned char *pTable)
{
    VIABIOSTV2TablePtr pTV2Tab;
    unsigned char *pRom, *pNTSC, *pTVTable, numCRTC, numPatch, numReg;
    int vScan, offset, i, j, k, n;

    pTV2Tab = NULL;
    pTVTable = NULL;

    DEBUG(xf86Msg(X_INFO, "VIAGetTV2NTSC\n"));
    for (vScan = 0; vScan < 2; vScan++) {
	switch (vScan) {
	    case VIA_TVNORMAL:
		/* Get start of TV Table */
		pRom = pTable + VIA_BIOS_TVMODETAB_POS;
		pTVTable = pBIOS + *((CARD16 *)pRom);

		pTV2Tab = pViaModeTable->tv2Table;
		break;
	    case VIA_TVFIT:
		/* Get start of TV Table */
		pRom = pTable + VIA_BIOS_MODEOVERTAB_POS;
		pTVTable = pBIOS + *((CARD16 *)pRom);

		pTV2Tab = pViaModeTable->tv2OverTable;
		break;
	}

	for (offset = 16, n = 0; n < VIA_BIOS_NUM_TV2; offset += 5, n++) {
	    pRom = pTVTable + offset;
	    pNTSC = pBIOS + *((CARD16 *)pRom);

	    pRom = pNTSC + 1;
	    pRom = pBIOS + *((CARD16 *)pRom);

	    for (i = 0, j = 0; i < pViaModeTable->tv2MaskTable.numTV; j++) {
		if (pViaModeTable->tv2MaskTable.TV[j] == 0xFF) {
		    pTV2Tab[n].TVNTSCC[j] = *pRom++;
		    i++;
		}
	    }

	    for (i = 0, j = 0; i < pViaModeTable->tv2MaskTable.numTV; j++) {
		if (pViaModeTable->tv2MaskTable.TV[j] == 0xFF) {
		    if (j >= 0x53) {
			pTV2Tab[n].TVNTSCS[j] = *pRom++;
			i++;
		    }
		    else {
			pTV2Tab[n].TVNTSCS[j] =
			pTV2Tab[n].TVNTSCC[j];
			i++;
		    }
		}
	    }

	    numCRTC = *pRom++;

	    for (i = 0, j = 0, k = 0; i < numCRTC; j++) {
		if (pViaModeTable->tv2MaskTable.CRTC1[j] == 0xFF) {
		    if (i >= pViaModeTable->tv2MaskTable.numCRTC1) {
			pTV2Tab[n].MiscNTSC1[k++] = *pRom++;
			i++;
		    }
		    else {
			pTV2Tab[n].CRTCNTSC1[j] = *pRom++;
			i++;
		    }
		}
	    }

	    for (i = 0, j = 0, k = 0; i < pViaModeTable->tv2MaskTable.numCRTC2; j++) {
		if (pViaModeTable->tv2MaskTable.CRTC2[j] == 0xFF) {
		    /* CRTC 65-57 8bpp, 16bpp, 32bpp */
		    if (j == 0x15) {
			pTV2Tab[n].CRTCNTSC2_8BPP[j] = *pRom++;
			pTV2Tab[n].CRTCNTSC2_8BPP[j + 1] = *pRom++;
			pTV2Tab[n].CRTCNTSC2_8BPP[j + 2] = *pRom++;
			pTV2Tab[n].CRTCNTSC2_16BPP[j] = *pRom++;
			pTV2Tab[n].CRTCNTSC2_16BPP[j + 1] = *pRom++;
			pTV2Tab[n].CRTCNTSC2_16BPP[j + 2] = *pRom++;
			pTV2Tab[n].CRTCNTSC2_32BPP[j++] = *pRom++;
			pTV2Tab[n].CRTCNTSC2_32BPP[j++] = *pRom++;
			pTV2Tab[n].CRTCNTSC2_32BPP[j] = *pRom++;
			i += 3;
		    }
		    else {
			/* For CRTC 6A-6C */
			if (i >= (pViaModeTable->tv2MaskTable.numCRTC2 - 3)) {
			    pTV2Tab[n].CRTCNTSC2_8BPP[j] = *pRom;
			    pTV2Tab[n].CRTCNTSC2_16BPP[j] = *pRom;
			    pTV2Tab[n].CRTCNTSC2_32BPP[j] = *pRom++;
			    i++;
			}
			else {
			    pTV2Tab[n].CRTCNTSC2_8BPP[j] = *pRom;
			    pTV2Tab[n].CRTCNTSC2_16BPP[j] = *pRom;
			    pTV2Tab[n].CRTCNTSC2_32BPP[j] = *pRom++;
			    i++;
			}
		    }
		}
	    }

	    k = 3;
	    /* LCK 3c5.44-45 */
	    if (pViaModeTable->tv2MaskTable.misc2 & 0x18) {
		pTV2Tab[n].MiscNTSC2[k++] = *pRom++;
		pTV2Tab[n].MiscNTSC2[k++] = *pRom++;
	    }

	    /* Patch as setting 2nd path */
	    numPatch = (int)(pViaModeTable->tv2MaskTable.misc2 >> 5);
	    for (i = 0; i < numPatch; i++) {
		pTV2Tab[n].PatchNTSC2[i] = *((CARD16 *)pRom);
		pRom += 2;
	    }

	    /* Get DotCrawl Table */
	    pRom = pNTSC + 4;
	    pRom = pBIOS + *((CARD16 *)pRom);

	    numReg = *pRom++;
	    pTV2Tab[n].DotCrawlNTSC[0] = numReg;
	    for (i = 1; i < (numReg + 1); i++) {
		pTV2Tab[n].DotCrawlNTSC[i] = *((CARD16 *)pRom);
		pRom += 2;
	    }
	}
    }
}

void
VIAGetTV2PAL(VIAModeTablePtr pViaModeTable,
	     unsigned char *pBIOS,
	     unsigned char *pTable)
{
    VIABIOSTV2TablePtr pTV2Tab;
    unsigned char *pRom, *pPAL, *pTVTable, numCRTC;
    int vScan, offset, numPatch, i, j, k, n;

    pTV2Tab = NULL;
    pTVTable = NULL;

    DEBUG(xf86Msg(X_INFO, "VIAGetTV2PAL\n"));
    for (vScan = 0; vScan < 2; vScan++) {
	switch (vScan) {
	    case VIA_TVNORMAL:
		/* Get start of TV Table */
		pRom = pTable + VIA_BIOS_TVMODETAB_POS;
		pTVTable = pBIOS + *((CARD16 *)pRom);

		pTV2Tab = pViaModeTable->tv2Table;
		break;
	    case VIA_TVFIT:
		/* Get start of TV Table */
		pRom = pTable + VIA_BIOS_MODEOVERTAB_POS;
		pTVTable = pBIOS + *((CARD16 *)pRom);

		pTV2Tab = pViaModeTable->tv2OverTable;
		break;
	}

	for (offset = 18, n = 0; n < VIA_BIOS_NUM_TV2; offset += 5, n++) {
	    pRom = pTVTable + offset;
	    pPAL = pBIOS + *((CARD16 *)pRom);

	    pRom = pPAL + 1;
	    pRom = pBIOS + *((CARD16 *)pRom);

	    for (i = 0, j = 0; i < pViaModeTable->tv2MaskTable.numTV; j++) {
		if (pViaModeTable->tv2MaskTable.TV[j] == 0xFF) {
		    pTV2Tab[n].TVPALC[j] = *pRom++;
		    i++;
		}
	    }

	    for (i = 0, j = 0; i < pViaModeTable->tv2MaskTable.numTV; j++) {
		if (pViaModeTable->tv2MaskTable.TV[j] == 0xFF) {
		    if (j >= 0x53) {
			pTV2Tab[n].TVPALS[j] = *pRom++;
			i++;
		    }
		    else {
			pTV2Tab[n].TVPALS[j] =
			pTV2Tab[n].TVPALC[j];
			i++;
		    }
		}
	    }

	    numCRTC = *pRom++;

	    for (i = 0, j = 0, k = 0; i < numCRTC; j++) {
		if (pViaModeTable->tv2MaskTable.CRTC1[j] == 0xFF) {
		    if (i >= pViaModeTable->tv2MaskTable.numCRTC1) {
			pTV2Tab[n].MiscPAL1[k++] = *pRom++;
                        	i++;
		    }
		    else {
			pTV2Tab[n].CRTCPAL1[j] = *pRom++;
			i++;
		    }
		}
	    }

	    for (i = 0, j = 0, k = 0;
		 i < pViaModeTable->tv2MaskTable.numCRTC2; j++) {
		if (pViaModeTable->tv2MaskTable.CRTC2[j] == 0xFF) {
		    /* CRTC 65-57 8bpp, 16bpp, 32bpp */
		    if (j == 0x15) {
			pTV2Tab[n].CRTCPAL2_8BPP[j] = *pRom++;
			pTV2Tab[n].CRTCPAL2_8BPP[j + 1] = *pRom++;
			pTV2Tab[n].CRTCPAL2_8BPP[j + 2] = *pRom++;
			pTV2Tab[n].CRTCPAL2_16BPP[j] = *pRom++;
			pTV2Tab[n].CRTCPAL2_16BPP[j + 1] = *pRom++;
			pTV2Tab[n].CRTCPAL2_16BPP[j + 2] = *pRom++;
			pTV2Tab[n].CRTCPAL2_32BPP[j++] = *pRom++;
			pTV2Tab[n].CRTCPAL2_32BPP[j++] = *pRom++;
			pTV2Tab[n].CRTCPAL2_32BPP[j] = *pRom++;
			i += 3;
		    }
		    else {
			/* For CRTC 6A-6C */
			if (i >= (pViaModeTable->tv2MaskTable.numCRTC2 - 3)) {
			    pTV2Tab[n].CRTCPAL2_8BPP[j] = *pRom;
			    pTV2Tab[n].CRTCPAL2_16BPP[j] = *pRom;
			    pTV2Tab[n].CRTCPAL2_32BPP[j] = *pRom++;
			    i++;
			}
			else {
			    pTV2Tab[n].CRTCPAL2_8BPP[j] = *pRom;
			    pTV2Tab[n].CRTCPAL2_16BPP[j] = *pRom;
			    pTV2Tab[n].CRTCPAL2_32BPP[j] = *pRom++;
			    i++;
			}
		    }
		}
	    }

	    k = 3;
	    /* LCK 3c5.44-45 */
	    if (pViaModeTable->tv2MaskTable.misc2 & 0x18) {
		pTV2Tab[n].MiscPAL2[k++] = *pRom++;
		pTV2Tab[n].MiscPAL2[k++] = *pRom++;
	    }

	    /* Patch as setting 2nd path */
	    numPatch = (int)(pViaModeTable->tv2MaskTable.misc2 >> 5);
	    for (i = 0; i < numPatch; i++) {
		pTV2Tab[n].PatchPAL2[i] = *((CARD16 *)pRom);
		pRom += 2;
	    }
	}
    }
}
#endif /* CREATE_TV2_HEADERFILE */

