/*
 * Copyright 1998-2007 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2007 S3 Graphics, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
#include "via_driver.h"
#include "via_tbl1622.h"
#include "via_hw.h"
#include "via_tv.h"

/* Extern Functions Prototypes Declaration */
extern void load_crtc_timing(VIABIOSInfoPtr pBIOSInfo, struct display_timing device_timing, int set_iga);

void
load_vt1622_regs(VIABIOSInfoPtr pBIOSInfo)
{
    int i, index, TVModeIndex = pBIOSInfo->TVSettingInfo.resTVMode;
    vt1622_func_table *vt1622_func;
    struct display_timing tv_crtc_reg;
    CARD16 *tv_reg = NULL, *tv_func_reg = NULL;
    I2CDevPtr dev;
    unsigned char R_Buffer[1], W_Buffer[2];

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO,"load_vt1622_regs()!!\n"));

    for (i = 0; i < NUM_TOTAL_VT1622_TABLE; i++) {
	index = i;

	if (vt1622_tbl[i].tv_index == TVModeIndex)
	    break;
    }

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO,"TV Mode Index: %d !\n" , index));

    switch (pBIOSInfo->TVSettingInfo.TVType) {
	case TVTYPE_NTSC:
	    vt1622_func = vt1622_tbl[index].tv_mode_ntsc;
	    break;
	case TVTYPE_PAL:
	    vt1622_func = vt1622_tbl[index].tv_mode_pal;
	    break;
	default:
	    vt1622_func = vt1622_tbl[index].tv_mode_ntsc;
	    break;
    }

    if (pBIOSInfo->TVSettingInfo.TVVScan == VIA_TVNORMAL)
	tv_reg = vt1622_func[0].tv_normalscan;
    if (pBIOSInfo->TVSettingInfo.TVVScan == VIA_TVFIT)
	tv_reg = vt1622_func[0].tv_fitscan;
    if (pBIOSInfo->TVSettingInfo.TVVScan == VIA_TVOVER)
	tv_reg = vt1622_func[0].tv_overscan;

    /* unlock CRTC Register
    unlock_crt(); */

    /* update starting address,modified by cliff to avoid setting IGA1 start address while TV on IGA2, for mergedfb */
    if(pBIOSInfo->TVSettingInfo.IGAPath==IGA1) {
	write_reg(CR0C, VIACR, 0x00);
	write_reg(CR0D, VIACR, 0x00);
	/* write_reg(CR34, VIACR, 0x00); */
	write_reg_mask(CR48, VIACR, 0x00, BIT0+BIT1);
    }
    /*Marked By Jason 
    if(pBIOSInfo->s3utility==FALSE) {
        write_reg(CR62, VIACR, 0x00);
        write_reg(CR63, VIACR, 0x00);
        write_reg(CR64, VIACR, 0x00);
    }
    */

    /* Load TV Encoder Regsiters */
    dev = xf86CreateI2CDevRec();
    dev->DevName = "VT1622";
    dev->SlaveAddr = 0x40;
    dev->pI2CBus = pBIOSInfo->I2C_Port2;

    xf86I2CDevInit(dev);

    for (i= 0 ; i < NUM_TOTAL_VT1622_REG; i++) {
	W_Buffer[0] = GET_LOW_BYTE(tv_reg[i]);
	W_Buffer[1] = GET_HIGH_BYTE(tv_reg[i]);
	xf86I2CWriteRead(dev, W_Buffer, 2, NULL, 0);
	DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, 
			 "Write TV Reg Offset %#x value %#x!\n",
			 GET_LOW_BYTE(tv_reg[i]), GET_HIGH_BYTE(tv_reg[i])));
    }

    xf86DestroyI2CDevRec(dev, TRUE);

    /* Load GFX CRTC timing when TV work on IGA1 */
    for (i = 0; i< NUM_CRTC_TIMING; i++) {
	switch (i) {
	    case H_TOTAL_INDEX:
		tv_crtc_reg.hor_total = tv_reg[i + NUM_TOTAL_VT1622_REG];
		DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO,
				 "H_Total: %d!\n", tv_crtc_reg.hor_total));
                break;
	    case H_ADDR_INDEX:
		tv_crtc_reg.hor_addr = tv_reg[i + NUM_TOTAL_VT1622_REG];
		DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO,
				 "H_Addr: %d!\n", tv_crtc_reg.hor_addr));
		break;
	    case H_BLANK_START_INDEX:
		tv_crtc_reg.hor_blank_start = tv_reg[i + NUM_TOTAL_VT1622_REG];
		DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO,
				 "H_Blank_Start: %d!\n",
				 tv_crtc_reg.hor_blank_start));
		break;
	    case H_BLANK_END_INDEX:
		tv_crtc_reg.hor_blank_end = tv_reg[i + NUM_TOTAL_VT1622_REG];
		DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO,
				 "H_Blank_End: %d!\n",
				 tv_crtc_reg.hor_blank_end));
                break;
	    case H_SYNC_START_INDEX:
		tv_crtc_reg.hor_sync_start = tv_reg[i + NUM_TOTAL_VT1622_REG];
		DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO,
				 "H_Sync_Start: %d!\n",
				 tv_crtc_reg.hor_sync_start));
		break;
	    case H_SYNC_END_INDEX:
		tv_crtc_reg.hor_sync_end = tv_reg[i + NUM_TOTAL_VT1622_REG];
		DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO,
				 "H_Sync_End: %d!\n",
				 tv_crtc_reg.hor_sync_end));
		break;
	    case V_TOTAL_INDEX:
		tv_crtc_reg.ver_total = tv_reg[i + NUM_TOTAL_VT1622_REG];
		break;
	    case V_ADDR_INDEX:
		tv_crtc_reg.ver_addr = tv_reg[i + NUM_TOTAL_VT1622_REG];
		break;
	    case V_BLANK_START_INDEX:
		tv_crtc_reg.ver_blank_start = tv_reg[i + NUM_TOTAL_VT1622_REG];
		break;
	    case V_BLANK_END_INDEX:
		tv_crtc_reg.ver_blank_end = tv_reg[i + NUM_TOTAL_VT1622_REG];
		break;
	    case V_SYNC_START_INDEX:
		tv_crtc_reg.ver_sync_start = tv_reg[i + NUM_TOTAL_VT1622_REG];
		break;
	    case V_SYNC_END_INDEX:
		tv_crtc_reg.ver_sync_end = tv_reg[i + NUM_TOTAL_VT1622_REG];
		break;
	}
    }

    SetTVClockSource(pBIOSInfo, &(pBIOSInfo->TVSettingInfo));

    if (pBIOSInfo->TVSettingInfo.IGAPath == IGA2) {
	load_crtc_timing(pBIOSInfo, tv_crtc_reg,
			 pBIOSInfo->TVSettingInfo.IGAPath);
	/*Marked By Jason         
        write_reg(CR65, VIACR, 0x80);
        write_reg(CR66, VIACR, 0x00);
        write_reg(CR67, VIACR, 0x41);
	*/        
	/* load_offset_reg(h_addr, bpp_byte, IGA2); */
	/* load_fetch_count_reg(h_addr, bpp_byte, IGA2); */

	/* RainLin [2005.11.22]: Modify for sequence issue. */
	EnableSecondDisplayChannel();

	/* VansonWu [2003.8.27] add select data source according using digital interface */
	if (pBIOSInfo->Chipset != VIA_CLE266) {
	    if (pBIOSInfo->TVDIPort & VIA_DI_DVP0) {
		/* Set DVP0 clock skew CR96[2:0] and data source CR96[4] */
		write_reg_mask(CR96, VIACR, BIT4, BIT4);
	    }
	    if (pBIOSInfo->TVDIPort & VIA_DI_DVP1) {
		/* Set DVP1 clock skew CR9B[2:0] and data source CR9B[4] */
		if (pBIOSInfo->Chipset == VIA_KM400 &&
		    pBIOSInfo->ChipRev < 5) {
		    /* VT3205 Ax */
		    write_reg(CR9B, VIACR, 0x1F);
		}
                else {
		    write_reg(CR9B, VIACR, 0x15);
		}
	    }
	    if (pBIOSInfo->TVDIPort & VIA_DI_DFPHIGH) {
		write_reg_mask(CR96, VIACR, BIT4, BIT4);
		write_reg_mask(CR97, VIACR, BIT4, BIT4);
	    }
	    if (pBIOSInfo->TVDIPort & VIA_DI_DFPLOW) {
		write_reg_mask(CR9B, VIACR, BIT4, BIT4);
		write_reg_mask(CR99, VIACR, BIT4, BIT4);
	    }
	    /*=* VansonWu [2004.5.10] Fixed Set DVI+TV/DVT+CRT/CRT+TV SAMM mode,
		 the secondary monitor without display *=*/
	    write_reg_mask(CR91, VIACR, 0, BIT5);
	}

	/* Disable LCD Scaling */
	write_reg(CR79, VIACR, 0);
    }
    else {
	load_crtc_timing(pBIOSInfo, tv_crtc_reg,
			 pBIOSInfo->TVSettingInfo.IGAPath);
	/* clear first */
	write_reg_mask(CR47, VIACR, 0x00, BIT7+BIT6+BIT3);

	write_reg_mask(CR6A, VIACR, BIT6, BIT6);

	if (pBIOSInfo->Chipset != VIA_CLE266) {
	    if (pBIOSInfo->TVDIPort & VIA_DI_DVP0) {
		/* Set DVP0 clock skew CR96[2:0] and data source CR96[4] */
		write_reg_mask(CR96, VIACR, 0, BIT4);
	    }
	    if (pBIOSInfo->TVDIPort & VIA_DI_DVP1) {
		/* Set DVP1 clock skew CR9B[2:0] and data source CR9B[4] */
		if (pBIOSInfo->Chipset == VIA_KM400 &&
		    pBIOSInfo->ChipRev < 5) {
		    /*=* VT3205 Ax *=*/
		    write_reg(CR9B, VIACR, 0x0F);
		}
		else {
		    write_reg(CR9B, VIACR, 0x05);
		}
	    }
	    if (pBIOSInfo->TVDIPort & VIA_DI_DFPHIGH) {
		write_reg_mask(CR96, VIACR, 0, BIT4);
		write_reg_mask(CR97, VIACR, 0, BIT4);
	    }
	    if (pBIOSInfo->TVDIPort & VIA_DI_DFPLOW) {
		write_reg_mask(CR9B, VIACR, 0, BIT4);
		write_reg_mask(CR99, VIACR, 0, BIT4);
	    }
	}
    }
    /* DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "\n tv_out_signal = %2d \n", tv_setting_info.out_signal));
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "\nVT1622_MAP_TV = %2d \n", vt1622_map_tv_func(tv_setting_info.out_signal)));*/

    index = vt1622_map_tv_func(pBIOSInfo->TVSettingInfo.TVOutput);
    if (index != 0) {
	/* DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "\nVT1622_MAP_TV = %2d\n", VT1622));*/
	if (pBIOSInfo->TVSettingInfo.TVVScan == VIA_TVNORMAL)
	    tv_func_reg = vt1622_func[index].tv_normalscan;
	if (pBIOSInfo->TVSettingInfo.TVVScan == VIA_TVFIT)
	    tv_func_reg = vt1622_func[index].tv_fitscan;
	if (pBIOSInfo->TVSettingInfo.TVVScan == VIA_TVOVER)
	    tv_func_reg = vt1622_func[index].tv_overscan;
	load_tv_output_signal(pBIOSInfo, tv_func_reg, &(pBIOSInfo->TVSettingInfo));
    }

    if ((pBIOSInfo->TVSettingInfo.TVDotCrawl) &&
	(pBIOSInfo->TVSettingInfo.TVOutput == TV_OUTPUT_COMPOSITE) &&
	(pBIOSInfo->TVSettingInfo.TVType == TVTYPE_NTSC)) {
	index =  vt1622_map_tv_func(TV_OUTPUT_DEDOTCRAWL);
	if (index != 0) {
	    if (pBIOSInfo->TVSettingInfo.TVVScan == VIA_TVNORMAL)
		tv_func_reg = vt1622_func[index].tv_normalscan;
	    if (pBIOSInfo->TVSettingInfo.TVVScan == VIA_TVFIT)
		tv_func_reg = vt1622_func[index].tv_fitscan;
	    if (pBIOSInfo->TVSettingInfo.TVVScan == VIA_TVOVER)
		tv_func_reg = vt1622_func[index].tv_overscan;

	    dev = xf86CreateI2CDevRec();
	    dev->DevName = "VT1622";
	    dev->SlaveAddr = 0x40;
	    dev->pI2CBus = pBIOSInfo->I2C_Port2;

	    xf86I2CDevInit(dev);

	    W_Buffer[0] = VT1622_DEDOTCRAWL_REG;
	    xf86I2CWriteRead(dev, W_Buffer, 1, R_Buffer, 1);

	    W_Buffer[1] = R_Buffer[0] | BIT3;
	    xf86I2CWriteRead(dev, W_Buffer, 2, NULL, 0);

	    xf86DestroyI2CDevRec(dev, TRUE);

	    load_tv_output_signal(pBIOSInfo, tv_func_reg, &(pBIOSInfo->TVSettingInfo));
	}
    }

    /* Patch for SDTV */
    if (pBIOSInfo->TVSettingInfo.TVType == TVTYPE_480P ||
	pBIOSInfo->TVSettingInfo.TVType == TVTYPE_576P ||
	pBIOSInfo->TVSettingInfo.TVProgressive) {
	if (pBIOSInfo->TVSettingInfo.TVOutput == TV_OUTPUT_RGB) {
	    if (pBIOSInfo->TVSettingInfo.TVVScan == VIA_TVNORMAL)
		tv_func_reg = vt1622_func[TV_SDTV_RGB_FUNC].tv_normalscan;
	    if (pBIOSInfo->TVSettingInfo.TVVScan == VIA_TVFIT)
		tv_func_reg = vt1622_func[TV_SDTV_RGB_FUNC].tv_fitscan;
	    if (pBIOSInfo->TVSettingInfo.TVVScan == VIA_TVOVER)
		tv_func_reg = vt1622_func[TV_SDTV_RGB_FUNC].tv_overscan;
	}
	else if (pBIOSInfo->TVSettingInfo.TVOutput == TV_OUTPUT_YPBPR) {
	    if (pBIOSInfo->TVSettingInfo.TVVScan == VIA_TVNORMAL)
		tv_func_reg = vt1622_func[TV_SDTV_YPBPR_FUNC].tv_normalscan;
	    if (pBIOSInfo->TVSettingInfo.TVVScan == VIA_TVFIT)
		tv_func_reg = vt1622_func[TV_SDTV_YPBPR_FUNC].tv_fitscan;
	    if (pBIOSInfo->TVSettingInfo.TVVScan == VIA_TVOVER)
		tv_func_reg = vt1622_func[TV_SDTV_YPBPR_FUNC].tv_overscan;
	}

	load_tv_output_signal(pBIOSInfo, tv_func_reg, &(pBIOSInfo->TVSettingInfo));
    }
    /* lock_crt(); */
}

int vt1622_map_tv_func(int out_signal)
{
    int tv_func_index;

    switch(out_signal)
    {
        case TV_OUTPUT_COMPOSITE:
        case TV_OUTPUT_SVIDEO:
        case TV_OUTPUT_COMPOSITE_SVIDEO:
            tv_func_index = 0;
            break;
        case TV_OUTPUT_RGB:
            tv_func_index = TV_RGB_FUNC;
            break;
        case TV_OUTPUT_YPBPR:
            tv_func_index = TV_YPBPR_FUNC;
            break;
        case TV_OUTPUT_SDTV_PROGRESSIVE_RGB:
            tv_func_index = TV_SDTV_RGB_FUNC;
            break;
        case TV_OUTPUT_SDTV_PROGRESSIVE_YPBPR:
            tv_func_index = TV_SDTV_YPBPR_FUNC;
            break;
        case TV_OUTPUT_DEDOTCRAWL:
            tv_func_index = TV_DeDotCrawl_FUNC;
            break;
        case TV_OUTPUT_HDTV_720P_RGB:
        case TV_OUTPUT_HDTV_720P_YPBPR:
        case TV_OUTPUT_HDTV_1080I_RGB:
        case TV_OUTPUT_HDTV_1080I_YPBPR:
        case TV_OUTPUT_HDTV_1080P_RGB:
        case TV_OUTPUT_HDTV_1080P_YPBPR:
            tv_func_index = 0;
            break;
        default:
            tv_func_index = 0;
            break;
    }
    return(tv_func_index);
}


