/*
 * Copyright 1998-2007 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2007 S3 Graphics, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
#include "via_driver.h"
#include "via_tbl1625.h"
#include "via_vt1625.h"
#include "via_tv.h"

static CARD16 *VIAGetTVFuncRegTable(int TVVScan,
				    vt1625_func_table *pVT1625FuncTable,
				    int index);

/* Extern Functions Prototypes Declaration */
extern void load_crtc_timing(VIABIOSInfoPtr pBIOSInfo, struct display_timing device_timing, int set_iga);

/*for support dual CRT*/
void load_1625_CRT_regs(VIABIOSInfoPtr pBIOSInfo)
{
    GPIOI2CWrite_TV(pBIOSInfo, 0x00, 0x03);
    GPIOI2CWrite_TV(pBIOSInfo, 0x02, 0xA0);
    GPIOI2CWrite_TV(pBIOSInfo, 0x0E, 0x38);
    GPIOI2CWrite_TV(pBIOSInfo, 0x1C, 0x20);
    GPIOI2CWrite_TV(pBIOSInfo, 0x1E, 0x40);
}


static CARD16 *
VIAGetTVFuncRegTable(int TVVScan, vt1625_func_table *pVT1625FuncTable, int index)
{
    CARD16  *pTVFuncRegTable = NULL;
    
    if (TVVScan == VIA_TVNORMAL)
    {
        pTVFuncRegTable = pVT1625FuncTable[index].tv_normalscan;
    }
    else if (TVVScan == VIA_TVFIT)
    {
        pTVFuncRegTable = pVT1625FuncTable[index].tv_fitscan;
    }
    else if (TVVScan == VIA_TVOVER)
    {
        pTVFuncRegTable = pVT1625FuncTable[index].tv_overscan;
    }

    return pTVFuncRegTable;
}


void
load_vt1625_regs(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo)
{
    int i, index, TVModeIndex = pTVSettingInfo->resTVMode;
    CARD8  tmp;
    vt1625_func_table      *vt1625_func;
    struct display_timing   tv_crtc_reg;
    CARD16                  *tv_reg=NULL;
    CARD16                  *tv_func_reg=NULL;

    for(i=0;i<NUM_TOTAL_VT1625_TABLE;i++)
    {
        index = i;

        if (vt1625_tbl[i].tv_index == TVModeIndex)
            break;
    }

    /* tv_crtc_reg.hor_addr' may be used uninitialized in this function */
    tv_crtc_reg.hor_addr = tv_crtc_reg.ver_addr = 0;
    
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO,"TV Mode Index: %d !\n" , index));

    switch (pTVSettingInfo->TVType)
    {
        case TVTYPE_NTSC:
        case TVTYPE_480P:
            vt1625_func = vt1625_tbl[index].tv_mode_ntsc;
            break;
        case TVTYPE_PAL:
        case TVTYPE_576P:
            vt1625_func = vt1625_tbl[index].tv_mode_pal;
            break;
        case TVTYPE_720P:
            vt1625_func = vt1625_tbl[index].tv_mode_720p;
            break;
        case TVTYPE_1080I:
            vt1625_func = vt1625_tbl[index].tv_mode_1080i;
            break;
        default:
            vt1625_func = vt1625_tbl[index].tv_mode_ntsc;
            break;
    }

    if (pTVSettingInfo->TVVScan == VIA_TVNORMAL)
    {
        tv_reg = vt1625_func[0].tv_normalscan;
    }

    if (pTVSettingInfo->TVVScan == VIA_TVFIT)
    {
        tv_reg = vt1625_func[0].tv_fitscan;
    }

    if (pTVSettingInfo->TVVScan == VIA_TVOVER)
    {
        tv_reg = vt1625_func[0].tv_overscan;
    }

    /* update starting address,modified by cliff to avoid setting IGA1 start address while TV on IGA2, for mergedfb */
    if(pTVSettingInfo->IGAPath == IGA1)
    {
        
        write_reg(CR0C, VIACR, 0x00);
        write_reg(CR0D, VIACR, 0x00);
        /* write_reg(CR34, VIACR, 0x00); */
        write_reg_mask(CR48, VIACR, 0x00, BIT0+BIT1);
    }
    
    /* Load TV Encoder Regsiters */
    for (i=0;i<NUM_TOTAL_VT1625_REG;i++)
    {
        GPIOI2CWrite_TV(pBIOSInfo, GET_LOW_BYTE(tv_reg[i]), GET_HIGH_BYTE(tv_reg[i]));
        DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "Write TV Reg Offset %#x value %#x!\n" , GET_LOW_BYTE(tv_reg[i]), GET_HIGH_BYTE(tv_reg[i])));
    }
    
    if(pBIOSInfo->TwoCRT)
    {
        load_1625_CRT_regs(pBIOSInfo);
    }
    
    /* Load GFX CRTC timing when TV work on IGA1 */
    for (i=0; i< NUM_CRTC_TIMING; i++)
    {
        switch(i)
        {
            case H_TOTAL_INDEX:
                tv_crtc_reg.hor_total = tv_reg[i+NUM_TOTAL_VT1625_REG];
                DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "H_Total: %d!\n" , tv_crtc_reg.hor_total));
                break;
            case H_ADDR_INDEX:
                tv_crtc_reg.hor_addr = tv_reg[i+NUM_TOTAL_VT1625_REG];
                DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "H_Addr: %d!\n" , tv_crtc_reg.hor_addr));
                break;
            case H_BLANK_START_INDEX:
                tv_crtc_reg.hor_blank_start = tv_reg[i+NUM_TOTAL_VT1625_REG];
                DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "H_Blank_Start: %d!\n" , tv_crtc_reg.hor_blank_start));
                break;
            case H_BLANK_END_INDEX:
                tv_crtc_reg.hor_blank_end = tv_reg[i+NUM_TOTAL_VT1625_REG];
                DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "H_Blank_End: %d!\n" , tv_crtc_reg.hor_blank_end));
                break;
            case H_SYNC_START_INDEX:
                tv_crtc_reg.hor_sync_start = tv_reg[i+NUM_TOTAL_VT1625_REG];
                DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "H_Sync_Start: %d!\n" , tv_crtc_reg.hor_sync_start));
                break;
            case H_SYNC_END_INDEX:
                tv_crtc_reg.hor_sync_end = tv_reg[i+NUM_TOTAL_VT1625_REG];
                DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "H_Sync_End: %d!\n" , tv_crtc_reg.hor_sync_end));
                break;
            case V_TOTAL_INDEX:
                tv_crtc_reg.ver_total = tv_reg[i+NUM_TOTAL_VT1625_REG];
                DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "ver_total: %d!\n" , tv_crtc_reg.ver_total));                
                break;
            case V_ADDR_INDEX:
                tv_crtc_reg.ver_addr = tv_reg[i+NUM_TOTAL_VT1625_REG];
                DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "ver_addr: %d!\n" , tv_crtc_reg.ver_addr));                                
                break;
            case V_BLANK_START_INDEX:
                tv_crtc_reg.ver_blank_start = tv_reg[i+NUM_TOTAL_VT1625_REG];
                DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "ver_blank_start: %d!\n" , tv_crtc_reg.ver_blank_start));                
                break;
            case V_BLANK_END_INDEX:
                tv_crtc_reg.ver_blank_end = tv_reg[i+NUM_TOTAL_VT1625_REG];
                DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "ver_blank_end: %d!\n" , tv_crtc_reg.ver_blank_end));                
                break;
            case V_SYNC_START_INDEX:
                tv_crtc_reg.ver_sync_start = tv_reg[i+NUM_TOTAL_VT1625_REG];
                DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "ver_sync_start: %d!\n" , tv_crtc_reg.ver_sync_start));                
                break;
            case V_SYNC_END_INDEX:
                tv_crtc_reg.ver_sync_end = tv_reg[i+NUM_TOTAL_VT1625_REG];
                DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "ver_sync_end: %d!\n" , tv_crtc_reg.ver_sync_end));                
                break;
        }
    }

    if (!(pBIOSInfo->Is3DScaling))
    {
        /* RainLin [2006.06.14]: If the view size is smaller than the timing we set, */
        /* don't show the part out of real display range. */
        if (pBIOSInfo->ActualDesktopSizeX < pTVSettingInfo->HActive)
        {
            tv_crtc_reg.hor_addr = pBIOSInfo->ActualDesktopSizeX;        
            DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "hor_addr: %d!\n" , tv_crtc_reg.hor_addr));                
        }

        if (pBIOSInfo->ActualDesktopSizeY < pTVSettingInfo->VActive)
        {
            tv_crtc_reg.ver_addr = pBIOSInfo->ActualDesktopSizeY;        
            DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "ver_addr: %d!\n" , tv_crtc_reg.ver_addr));                
        }
    }
    
    if(!pBIOSInfo->TwoCRT)
    {
        load_crtc_timing(pBIOSInfo, tv_crtc_reg, pTVSettingInfo->IGAPath);
    }

    SetTVClockSource(pBIOSInfo, pTVSettingInfo);
    
    if (pTVSettingInfo->IGAPath == IGA2)
    {
        /* RainLin [2005.11.22]: Modify for sequence issue. */
        EnableSecondDisplayChannel();

        /* VansonWu [2003.8.27] add select data source according using digital interface */
        if (pBIOSInfo->Chipset != VIA_CLE266)
        {
            if (pBIOSInfo->TVDIPort & VIA_DI_DVP0)
            {
                /* Set DVP0 clock skew CR96[2:0] and data source CR96[4] */
                write_reg_mask(CR96, VIACR, BIT4, BIT4);
            }
            
            if (pBIOSInfo->TVDIPort & VIA_DI_DVP1)
            {
                /* Set DVP1 clock skew CR9B[2:0] and data source CR9B[4] */
                if (pBIOSInfo->Chipset == VIA_KM400 &&
                    pBIOSInfo->ChipRev < 5)
                {
                    /*=* VT3205 Ax *=*/
                    write_reg(CR9B, VIACR, 0x1F);
                }
                else
                {
                    write_reg(CR9B, VIACR, 0x15);
                }
            }
            
            if (pBIOSInfo->TVDIPort & VIA_DI_DFPHIGH)
            {
                write_reg_mask(CR96, VIACR, BIT4, BIT4);
                write_reg_mask(CR97, VIACR, BIT4, BIT4);
            }
            
            if (pBIOSInfo->TVDIPort & VIA_DI_DFPLOW)
            {
                write_reg_mask(CR9B, VIACR, BIT4, BIT4);
                write_reg_mask(CR99, VIACR, BIT4, BIT4);
            }
            
            /*=* VansonWu [2004.5.10] Fixed Set DVI+TV/DVT+CRT/CRT+TV SAMM mode,
                 the secondary monitor without display *=*/
            write_reg_mask(CR91, VIACR, 0, BIT5);
        }

        /* Disable LCD Scaling */
        write_reg(CR79, VIACR, 0);
    }
    else
    {
        /* patch TV IGA1 horizontal total can't be divided by 8 */
        if ((pBIOSInfo->Chipset != VIA_CLE266) && 
            (pBIOSInfo->Chipset != VIA_KM400) &&
            (pBIOSInfo->Chipset != VIA_K8M800))
        {
            PlusVCKToIGA1Timing(tv_crtc_reg.hor_total);
        }

        write_reg_mask(CR6A, VIACR, BIT6, BIT6);

        if (pBIOSInfo->Chipset != VIA_CLE266)
        {
            if (pBIOSInfo->TVDIPort & VIA_DI_DVP0)
            {
                /* Set DVP0 clock skew CR96[2:0] and data source CR96[4] */
                write_reg_mask(CR96, VIACR, 0, BIT4);
            }
            
            if (pBIOSInfo->TVDIPort & VIA_DI_DVP1)
            {
                /* Set DVP1 clock skew CR9B[2:0] and data source CR9B[4] */
                if (pBIOSInfo->Chipset == VIA_KM400 &&
                    pBIOSInfo->ChipRev < 5)
                {
                    /*=* VT3205 Ax *=*/
                    write_reg(CR9B, VIACR, 0x0F);
                }
                else
                {
                    write_reg(CR9B, VIACR, 0x05);
                }
            }
            
            if (pBIOSInfo->TVDIPort & VIA_DI_DFPHIGH)
            {
                write_reg_mask(CR96, VIACR, 0, BIT4);
                write_reg_mask(CR97, VIACR, 0, BIT4);
            }
            
            if (pBIOSInfo->TVDIPort & VIA_DI_DFPLOW)
            {
                write_reg_mask(CR9B, VIACR, 0, BIT4);
                write_reg_mask(CR99, VIACR, 0, BIT4);
            }
        }
    }

    /*DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "\n tv_out_signal = %2d \n", tv_setting_info.out_signal));
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "\nVT1625_MAP_TV = %2d \n", vt1625_map_tv_func(tv_setting_info.out_signal)));*/

    index =  vt1625_map_tv_func(pTVSettingInfo->TVOutput);
    if (index != 0)
    {
        /*DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "\nVT1625_MAP_TV = %2d\n", VT1625));*/
        tv_func_reg = VIAGetTVFuncRegTable(pTVSettingInfo->TVVScan, vt1625_func, index);        
        load_tv_output_signal(pBIOSInfo, tv_func_reg, pTVSettingInfo);
    }

    if ((pTVSettingInfo->TVDotCrawl) &&
        (pTVSettingInfo->TVOutput == TV_OUTPUT_COMPOSITE) &&
        (pTVSettingInfo->TVType == TVTYPE_NTSC))
    {
        index =  vt1625_map_tv_func(TV_OUTPUT_DEDOTCRAWL);
        
        if (index != 0)
        {
            tv_func_reg = VIAGetTVFuncRegTable(pTVSettingInfo->TVVScan, vt1625_func, index);
            
            tmp = GPIOI2CRead_TV(pBIOSInfo, VT1625_DEDOTCRAWL_REG);
            GPIOI2CWrite_TV(pBIOSInfo, VT1625_DEDOTCRAWL_REG, tmp | BIT3);
            load_tv_output_signal(pBIOSInfo, tv_func_reg, pTVSettingInfo);
        }
    }

    /* Patch for SDTV */
    if (pTVSettingInfo->TVType == TVTYPE_480P ||
        pTVSettingInfo->TVType == TVTYPE_576P ||
        pTVSettingInfo->TVProgressive)
    {
        if (pTVSettingInfo->TVOutput == TV_OUTPUT_RGB)
        {
            tv_func_reg = VIAGetTVFuncRegTable(pTVSettingInfo->TVVScan, vt1625_func, TV_SDTV_RGB_FUNC);
        }
        else if (pTVSettingInfo->TVOutput == TV_OUTPUT_YPBPR)
        {
            tv_func_reg = VIAGetTVFuncRegTable(pTVSettingInfo->TVVScan, vt1625_func, TV_SDTV_YPBPR_FUNC);
        }

        load_tv_output_signal(pBIOSInfo, tv_func_reg, pTVSettingInfo);
    } 

    /*Load CGMS registers*/
    for (i=0;i<NUM_TOTAL_VT1625_CGMS_TBL;i++)
    {
        GPIOI2CWrite_TV(pBIOSInfo, GET_LOW_BYTE(VT1625_CGMS_TBL[i]),GET_HIGH_BYTE(VT1625_CGMS_TBL[i]));
    }
}

int vt1625_map_tv_func(int out_signal)
{
    int tv_func_index;

    switch(out_signal)
    {
        case TV_OUTPUT_COMPOSITE:
        case TV_OUTPUT_SVIDEO:
        case TV_OUTPUT_COMPOSITE_SVIDEO:
            tv_func_index = 0;
            break;
        case TV_OUTPUT_RGB:
        case TV_OUTPUT_RGB_COMPOSITE_SVIDEO:
            tv_func_index = TV_RGB_FUNC;
            break;
        case TV_OUTPUT_YPBPR:
        case TV_OUTPUT_YPBPR_COMPOSITE_SVIDEO:
            tv_func_index = TV_YPBPR_FUNC;
            break;
        case TV_OUTPUT_SDTV_PROGRESSIVE_RGB:
            tv_func_index = TV_SDTV_RGB_FUNC;
            break;
        case TV_OUTPUT_SDTV_PROGRESSIVE_YPBPR:
            tv_func_index = TV_SDTV_YPBPR_FUNC;
            break;
        case TV_OUTPUT_DEDOTCRAWL:
            tv_func_index = TV_DeDotCrawl_FUNC;
            break;
        case TV_OUTPUT_HDTV_720P_RGB:
        case TV_OUTPUT_HDTV_720P_YPBPR:
        case TV_OUTPUT_HDTV_1080I_RGB:
        case TV_OUTPUT_HDTV_1080I_YPBPR:
        case TV_OUTPUT_HDTV_1080P_RGB:
        case TV_OUTPUT_HDTV_1080P_YPBPR:
            tv_func_index = 0;
        break;
        default:
            tv_func_index = 0;

    }
    return(tv_func_index);
}

/*=*
 *
 * unsigned char VIASensorVT1625(VIABIOSInfoPtr pBIOSInfo)
 *
 *     - Sense VT1625 Encoder
 *
 * Return Type:    unsigned char
 *
 * The Definition of Input Value:
 *
 *                 VIABIOSInfoPtr
 *
 * The Definition of Return Value:
 *
 *   Bit[5] Bit[4] Bit[3] Bit[2] Bit[1] Bit[0]
 *   DAC-A  DAC-B  DAC-C  DAC-D  DAC-E  DAC-F
 *   1      1      1      1      1      1      - None
 *   1      1      0      1      1      1      - Composite
 *   0      0      1      1      1      1      - S-Video
 *   0      0      0      1      1      1      - Composite + S-Video
 *   1      1      1      0      0      0      - Component or RGB
 *   0      0      0      0      0      0      - Component or RGB + Composite + S-Video
 *=*/
unsigned char VIASensorVT1625(VIABIOSInfoPtr pBIOSInfo)
{
    unsigned char   tv0e, tv1c, tmp = 0;
    unsigned char   W_Buffer[2];
    int             i;

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIASensorVT1625\n"));

    /* save value of TVReg.0E Power Management register */
    W_Buffer[0] = 0x0E;
    tv0e = GPIOI2CRead_TV(pBIOSInfo, W_Buffer[0]);

    W_Buffer[1] = 0;
    GPIOI2CWrite_TV(pBIOSInfo, W_Buffer[0], W_Buffer[1]);

    /* save value of TVReg.1C */
    W_Buffer[0] = 0x1C;
    tv1c = GPIOI2CRead_TV(pBIOSInfo, W_Buffer[0]);

    for(i=0; i<10; i++)
    {
        W_Buffer[0] = 0x1C;    
        W_Buffer[1] = 0x80;
        GPIOI2CWrite_TV(pBIOSInfo, W_Buffer[0], W_Buffer[1]);
    }

    W_Buffer[1] = tv1c;
    GPIOI2CWrite_TV(pBIOSInfo, W_Buffer[0], W_Buffer[1]);

    for (i=0; i<10; i++)    /* Read TV Reg.0F 10 times in case it didn't refresh soon */
    {
        W_Buffer[0] = 0x0F;
        tmp = GPIOI2CRead_TV(pBIOSInfo, W_Buffer[0]);
        tmp = tmp & 0x3F;
    }

    W_Buffer[0] = 0x0E;
    W_Buffer[1] = tv0e;
    GPIOI2CWrite_TV(pBIOSInfo, W_Buffer[0], W_Buffer[1]);

    return tmp;
}

Bool TVEncoderIdentify_VT1625(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo)
{
    CARD8 data;
    
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED, "TVEncoderIdentify_VT1625\n"));

    pBIOSInfo->TVI2CAdd = TV_I2C_ADDR_VT1625;        
    data = GPIOI2CRead_TV(pBIOSInfo, 0x1B);

    if (data == 0x50)
    {
        DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_PROBED, "Found TVEncoder VT1625!!\n"));
        pTVSettingInfo->TVEncoder = VIA_VT1625;
        
        return TRUE;
    }

    return FALSE;
}

void
EnableTV_VT1625(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo)
{
    CARD8  W_Buffer[2];

    /* Set TV output: */
    W_Buffer[0] = 0x0E;

    switch (pTVSettingInfo->TVOutput) {
	case TV_OUTPUT_NONE:
	    W_Buffer[1] = 0x3F;
	    break;
	case TV_OUTPUT_COMPOSITE:
	    W_Buffer[1] = 0x37;
	    break;
	case TV_OUTPUT_SVIDEO:
	    W_Buffer[1] = 0x0F;
	    break;
	case TV_OUTPUT_COMPOSITE_SVIDEO:
	    W_Buffer[1] = 0x07;
	    break;
	case TV_OUTPUT_YPBPR:
	case TV_OUTPUT_RGB:
	    W_Buffer[1] = 0x38;
	    break;
	case TV_OUTPUT_YPBPR_COMPOSITE_SVIDEO:
	case TV_OUTPUT_RGB_COMPOSITE_SVIDEO:
	    W_Buffer[1] = 0x0;
	    break;
	default:
	    W_Buffer[1] = 0;
	    break;
    }

    GPIOI2CWrite_TV(pBIOSInfo, W_Buffer[0], W_Buffer[1]);
}

void DisableTV_VT1625(VIABIOSInfoPtr pBIOSInfo)
{
    GPIOI2CWrite_TV(pBIOSInfo, 0x0E, 0x3F);
}

void GetTVInfo_VT1625(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo)
{
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "GetVT1625TVInfo\n"));
    CARD8       data;
    TVInfoPtr   pTVInfo = &(pTVSettingInfo->TVInfo);
    
    pTVInfo->DefaultScalH = 0;
    pTVInfo->DefaultScalV = VIA_TVNORMAL;
    pTVInfo->ScalHLevel = 0;
    pTVInfo->ScalVLevel = 3;

    pTVInfo->DefaultPositionV = 6;
    pTVInfo->PositionVLevel = 11;
    pTVInfo->DefaultPositionH = 6;
    pTVInfo->PositionHLevel = 11;

    data = GPIOI2CRead_TV(pBIOSInfo, 0x03);
    pTVInfo->DefaultFFilter = data & 0x03;
    pTVInfo->FFilterLevel = 3;

    data = GPIOI2CRead_TV(pBIOSInfo, 0x62);
    pTVInfo->DefaultAFFilter = (data >> 2);
    pTVInfo->AFFilterLevel = 64;

    if (pTVInfo->DefaultAFFilter && !pTVInfo->DefaultFFilter)
    {
        pTVInfo->AdaptiveFFilterOn = TRUE;
    }
    else
    {
        pTVInfo->AdaptiveFFilterOn = FALSE;
    }

    data = GPIOI2CRead_TV(pBIOSInfo, 0x0B);
    pTVInfo->DefaultBrightness = data;
    pTVInfo->BrightnessLevel = 255;

    if (pTVSettingInfo->TVOutput == TV_OUTPUT_RGB || pTVSettingInfo->TVOutput == TV_OUTPUT_RGB_COMPOSITE_SVIDEO)
    {
        data = GPIOI2CRead_TV(pBIOSInfo, 0x65);
        pTVInfo->DefaultContrast = data;
        pTVInfo->ContrastLevel = 250;

        pTVInfo->DefaultSaturation = 0;
        pTVInfo->SaturationLevel = 0;

        pTVInfo->DefaultTINT = 0;
        pTVInfo->TINTLevel = 0;
    }
    else if (pTVSettingInfo->TVOutput == TV_OUTPUT_YPBPR || pTVSettingInfo->TVOutput == TV_OUTPUT_YPBPR_COMPOSITE_SVIDEO)
    {
        data = GPIOI2CRead_TV(pBIOSInfo, 0x65);
        pTVInfo->DefaultContrast = data;
        pTVInfo->ContrastLevel = 120;
        pTVInfo->BrightnessLevel = 60;                       

        pTVInfo->DefaultSaturation = 0;
        pTVInfo->SaturationLevel = 0;

        pTVInfo->DefaultTINT = 0;
        pTVInfo->TINTLevel = 0;
    }
    else
    {
        if (pTVSettingInfo->TVOutput == TV_OUTPUT_SVIDEO)
        {
            data = GPIOI2CRead_TV(pBIOSInfo, 0x65);
        }
        else
        {
            data = GPIOI2CRead_TV(pBIOSInfo, 0x0C);
        }
        
        pTVInfo->DefaultContrast = data;
        pTVInfo->ContrastLevel = 255;

        data = GPIOI2CRead_TV(pBIOSInfo, 0x0D);
        pTVInfo->DefaultSaturation = data << 8;
        
        data = GPIOI2CRead_TV(pBIOSInfo, 0x0A);
        pTVInfo->DefaultSaturation += data;
        pTVInfo->SaturationLevel = 65535;

        data = GPIOI2CRead_TV(pBIOSInfo, 0x10);
        pTVInfo->DefaultTINT = data;

        data = GPIOI2CRead_TV(pBIOSInfo, 0x11);
        pTVInfo->DefaultTINT += (data & 0x07) << 8;
        pTVInfo->TINTLevel = 2047;
    }
}

void RestoreUserTVSettings_VT1625(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo)
{
    CARD8       data, tint;
    CARD32      diff; 
    TVInfoPtr   pTVInfo = &(pTVSettingInfo->TVInfo);
    int         HPos, VPos, VPosHigh;

    if (!(pBIOSInfo->Is3DScaling))
    {
    /* Position: */
        if ((pTVInfo->CurrentPositionH != 6) || (pTVInfo->CurrentPositionV != 6))
        {
            HPos = GPIOI2CRead_TV(pBIOSInfo, 0x08);

            VPos = GPIOI2CRead_TV(pBIOSInfo, 0x09);        
            VPosHigh = GPIOI2CRead_TV(pBIOSInfo, 0x1C);
            VPos |= (VPosHigh & 0x0E) << 7;

            HPos -= pTVInfo->CurrentPositionH - 6;
            if(HPos < 0)
                HPos = 0;
            VPos += pTVInfo->CurrentPositionV - 6;

            data = (unsigned char)(HPos & 0xFF);
            GPIOI2CWrite_TV(pBIOSInfo, 0x08, data);

            data = (unsigned char)(VPos & 0xFF);
            GPIOI2CWrite_TV(pBIOSInfo, 0x09, data);

            data = VPosHigh & ~0x0E;             /* 1CH[3:1] VP[10:8]*/
            data |= (unsigned char)((VPos >> 7) & 0x0E);
            GPIOI2CWrite_TV(pBIOSInfo, 0x1C, data);
        }
    }
    /* Flick Filter: */
    data = GPIOI2CRead_TV(pBIOSInfo, 0x03);
    
    if (pTVInfo->AdaptiveFFilterOn)
    {
        data = data & ~0x03;
    }
    else
    {
        data = data & ~0x03;
        data |= (unsigned char)(pTVInfo->CurrentFFilter);
    }
    
    GPIOI2CWrite_TV(pBIOSInfo, 0x03, data);

    /* Adaptive Flick Filter: */
    data = (unsigned char)(pTVInfo->CurrentAFFilter << 2);
    GPIOI2CWrite_TV(pBIOSInfo, 0x62, data);

    /* Brightness: */
    data = (unsigned char)pTVInfo->CurrentBrightness;
    GPIOI2CWrite_TV(pBIOSInfo, 0x0B, data);
    
    if (pTVSettingInfo->TVOutput == TV_OUTPUT_RGB || pTVSettingInfo->TVOutput == TV_OUTPUT_RGB_COMPOSITE_SVIDEO)
    {
        /* Contrast: */
        data = GPIOI2CRead_TV(pBIOSInfo, 0x65);
        diff = pTVInfo->CurrentContrast - data;
        data = data + diff;
        GPIOI2CWrite_TV(pBIOSInfo, 0x65, data);
        
        data = GPIOI2CRead_TV(pBIOSInfo, 0x66);
        data = data + diff;
        GPIOI2CWrite_TV(pBIOSInfo, 0x66 , data);
        
        data = GPIOI2CRead_TV(pBIOSInfo, 0x67);
        data = data + diff;
        GPIOI2CWrite_TV(pBIOSInfo, 0x67 , data);
    }
    else if (pTVSettingInfo->TVOutput == TV_OUTPUT_YPBPR || pTVSettingInfo->TVOutput == TV_OUTPUT_YPBPR_COMPOSITE_SVIDEO)
    {
        /* Contrast: */
        data = (unsigned char)pTVInfo->CurrentContrast;
        GPIOI2CWrite_TV(pBIOSInfo, 0x65, data);
    }
    else
    {
        /* Contrast: */
        if (pTVSettingInfo->TVOutput == TV_OUTPUT_SVIDEO)
        {
            data = (unsigned char)pTVInfo->CurrentContrast;
            GPIOI2CWrite_TV(pBIOSInfo, 0x65, data);
        }
        else
        {
            data = (unsigned char)pTVInfo->CurrentContrast;
            GPIOI2CWrite_TV(pBIOSInfo, 0x0C, data);
        }
        
        /* Saturation: */
        data = (unsigned char)(pTVInfo->CurrentSaturation >> 8);
        GPIOI2CWrite_TV(pBIOSInfo, 0x0D, data);
        data = (unsigned char)pTVInfo->CurrentSaturation & 0xFF;
        GPIOI2CWrite_TV(pBIOSInfo, 0x0A, data);

        /* Tint: */
        tint = GPIOI2CRead_TV(pBIOSInfo, 0x11);

        data = tint & ~0x07;
        data |= (unsigned char)(pTVInfo->CurrentTINT >> 8);
        GPIOI2CWrite_TV(pBIOSInfo, 0x11, data);

        data = (unsigned char)(pTVInfo->CurrentTINT & 0xFF);
        GPIOI2CWrite_TV(pBIOSInfo, 0x10, data);
    }
}

void SaveTVRegs_VT1625(VIABIOSInfoPtr pBIOSInfo, VIARegPtr pSaveRegs)
{
    int i;
    
    for (i = 0; i < 0x6C; i++)
    {
        pSaveRegs->TVRegs[i] = GPIOI2CRead_TV(pBIOSInfo, i);
    }
}

void RestoreTVRegs_VT1625(VIABIOSInfoPtr pBIOSInfo, VIARegPtr pRestoreRegs)
{
    int i;
    
    for (i = 0; i < 0x6C; i++)
    {
        GPIOI2CWrite_TV(pBIOSInfo, i, pRestoreRegs->TVRegs[i]);
    }
}

void AdjustTVPos_VT1625(VIABIOSInfoPtr pBIOSInfo, 
                              TVSETTINGINFOPTR pTVSettingInfo, 
                              CARD32 desired_hpos, 
                              CARD32 desired_vpos)
{
    CARD8       data;
    int      dst_hpos, dst_vpos, dst_vpos_high,dx_hpos;
    TVInfoPtr   pTVInfo = &(pTVSettingInfo->TVInfo);
    
    dst_hpos = GPIOI2CRead_TV(pBIOSInfo, 0x08);
    dst_vpos = GPIOI2CRead_TV(pBIOSInfo, 0x09);

    dst_vpos_high = GPIOI2CRead_TV(pBIOSInfo, 0x1C);
    dst_vpos |= (dst_vpos_high & 0x0E) << 7;        /* [3:1]1CH: VP[10:8] */

    dx_hpos = desired_hpos - pTVInfo->CurrentPositionH;
    if(dst_hpos >= dx_hpos)
        dst_hpos -= dx_hpos;
    else
        dst_hpos = 0;
    dst_vpos += desired_vpos - pTVInfo->CurrentPositionV;


    data = dst_hpos & 0xFF;
    GPIOI2CWrite_TV(pBIOSInfo, 0x08, data);

    data = dst_vpos & 0xFF;
    GPIOI2CWrite_TV(pBIOSInfo, 0x09, data);

    data = dst_vpos_high & ~0x0E;              /* [3:1]1CH: VP[10:8] */
    data |= (dst_vpos >> 7) & 0x0E;
    GPIOI2CWrite_TV(pBIOSInfo, 0x1C, data);

    pTVInfo->CurrentPositionH = desired_hpos;
    pTVInfo->CurrentPositionV = desired_vpos;
}

void AdjustTVBrightness_VT1625(VIABIOSInfoPtr pBIOSInfo, CARD32 desired_value)
{
    CARD8   data;
    
    data = (unsigned char)desired_value;
    GPIOI2CWrite_TV(pBIOSInfo, 0x0B, data);
}

void AdjustTVContrast_VT1625(VIABIOSInfoPtr pBIOSInfo,
                                   TVSETTINGINFOPTR pTVSettingInfo,
                                   CARD32 desired_value)
{
    TVInfoPtr   pTVInfo = &(pTVSettingInfo->TVInfo);
    CARD32      diff;
    CARD8       data;

    diff = (desired_value - pTVInfo->CurrentContrast);
    
    if (pTVSettingInfo->TVOutput == TV_OUTPUT_RGB || 
        pTVSettingInfo->TVOutput == TV_OUTPUT_RGB_COMPOSITE_SVIDEO)
    {
        /* PY Amplitude Factor */
        data = GPIOI2CRead_TV(pBIOSInfo, 0x65);
        data = data + diff;
        GPIOI2CWrite_TV(pBIOSInfo, 0x65, data);

        /* PB Amplitude Factor */
        data = GPIOI2CRead_TV(pBIOSInfo, 0x66);
        data = data + diff;
        GPIOI2CWrite_TV(pBIOSInfo, 0x66, data);
        
        /* PR Amplitude Factor */
        data = GPIOI2CRead_TV(pBIOSInfo, 0x67);
        data = data + diff;
        GPIOI2CWrite_TV(pBIOSInfo, 0x67, data);
    }
    else if (pTVSettingInfo->TVOutput == TV_OUTPUT_YPBPR ||
             pTVSettingInfo->TVOutput == TV_OUTPUT_YPBPR_COMPOSITE_SVIDEO)
    {
        /* PY Amplitude Factor */
        data = GPIOI2CRead_TV(pBIOSInfo, 0x65);
        data = data + diff;
        GPIOI2CWrite_TV(pBIOSInfo, 0x65, data);
    }
    else
    {        
        /* Y Amplitude Factor */
        data = GPIOI2CRead_TV(pBIOSInfo, 0x0C);
        data = data + diff;
        GPIOI2CWrite_TV(pBIOSInfo, 0x0C, data);

        if (pTVSettingInfo->TVOutput == TV_OUTPUT_SVIDEO)
        {
            /* PY Amplitude Factor */
            data = GPIOI2CRead_TV(pBIOSInfo, 0x65);            
            data = data + diff;            
            GPIOI2CWrite_TV(pBIOSInfo, 0x65, data);            
        }
    }

    pTVInfo->CurrentContrast = desired_value;
}

void AdjustTVSaturation_VT1625(VIABIOSInfoPtr pBIOSInfo,
                                     TVSETTINGINFOPTR pTVSettingInfo,
                                     CARD32 desired_value)
{
    CARD8       data;
    TVInfoPtr   pTVInfo = &(pTVSettingInfo->TVInfo);
    
    data = (unsigned char)((desired_value >> 8) & 0xFF);
    GPIOI2CWrite_TV(pBIOSInfo, 0x0D, data);

    data = (unsigned char)(desired_value & 0xFF);
    GPIOI2CWrite_TV(pBIOSInfo, 0x0A, data);

    pTVInfo->CurrentSaturation = desired_value;
}

void AdjustTVHue_VT1625(VIABIOSInfoPtr pBIOSInfo,
                              TVSETTINGINFOPTR pTVSettingInfo,
                              CARD32 desired_value)
{
    CARD8       data;
    TVInfoPtr   pTVInfo = &(pTVSettingInfo->TVInfo);
    
    data = GPIOI2CRead_TV(pBIOSInfo, 0x11);
    data = data & ~0x07;
    data |= (unsigned char)(desired_value >> 8) & 0xFF;
    GPIOI2CWrite_TV(pBIOSInfo, 0x11, data);
    
    data = (unsigned char)(desired_value & 0xFF);
    GPIOI2CWrite_TV(pBIOSInfo, 0x10, data);
    
    pTVInfo->CurrentTINT = desired_value;
}

void EnableTVFFilter_VT1625(VIABIOSInfoPtr pBIOSInfo,
                                 TVSETTINGINFOPTR pTVSettingInfo,
                                 int filter_type)
{
    CARD8       data;
    TVInfoPtr   pTVInfo = &(pTVSettingInfo->TVInfo);

    if (filter_type == VT1625_FILTER_TYPE_NORMAL)
    {
        /* Turn on FFilter: */
        data = GPIOI2CRead_TV(pBIOSInfo, 0x03);
        data = data & ~0x03;
        data |= (unsigned char)(pTVInfo->CurrentFFilter);
        GPIOI2CWrite_TV(pBIOSInfo, 0x03, data);

        /* Turn off AFFilter: */
        GPIOI2CWrite_TV(pBIOSInfo, 0x62, 0);
    }
    else if(filter_type == VT1625_FILTER_TYPE_ADAPTIVE)
    {
        /* Turn off FFilter: */
        data = GPIOI2CRead_TV(pBIOSInfo, 0x03);
        data = data & ~0x03;
        GPIOI2CWrite_TV(pBIOSInfo, 0x03, data);

        /* Turn on AFFilter: */
        data = (unsigned char)(pTVInfo->CurrentAFFilter << 2) | 0x01;
        GPIOI2CWrite_TV(pBIOSInfo, 0x62, data);
    }
    else
    {
        /* Turn off FFilter: */
        data = GPIOI2CRead_TV(pBIOSInfo, 0x03);
        data = data & ~0x03;
        GPIOI2CWrite_TV(pBIOSInfo, 0x03, data);
        
        /* Turn off AFFilter: */
        GPIOI2CWrite_TV(pBIOSInfo, 0x62, 0);
    }
}

void AdjustTVFFilter_VT1625(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo, int filter_type, CARD32 desired_value)
{
    CARD8       data;
    TVInfoPtr   pTVInfo = &(pTVSettingInfo->TVInfo);

    if (filter_type == VT1625_FILTER_TYPE_NORMAL)
    {
        /* Set default FFilter */
        if (desired_value == 0xFFFF)
        {
            desired_value = pTVInfo->DefaultFFilter;
            
            if(pTVInfo->DefaultFFilter == 0)
            {
                pTVInfo->AdaptiveFFilterOn = TRUE;
            }
        }

        data = GPIOI2CRead_TV(pBIOSInfo, 0x03);        
        data = data & ~0x03;
        data |= desired_value;
        GPIOI2CWrite_TV(pBIOSInfo, 0x03, data);

        pTVInfo->CurrentFFilter = desired_value;
    }
    else if(filter_type == VT1625_FILTER_TYPE_ADAPTIVE)
    {
        /* Set default AFFilter */
        if (desired_value == 0xFFFF)
        {
            desired_value = pTVInfo->DefaultAFFilter;
        }

        /* 62H[7:2] Deflicker Threshold */
        data = (unsigned char)(desired_value << 2) | 0x01;        
        GPIOI2CWrite_TV(pBIOSInfo, 0x62, data);

        pTVInfo->CurrentAFFilter = desired_value;
    }
}

void InitTV_VT1625(VIABIOSInfoPtr pBIOSInfo)
{
    /* TV 5 init value */
    GPIOI2CWrite_TV(pBIOSInfo, 0x4A, 0xC5);   /* DAC A/B/C/D select */
    GPIOI2CWrite_TV(pBIOSInfo, 0x4B, 0x0F);   /* DAC E/F select */
    GPIOI2CWrite_TV(pBIOSInfo, 0x69, 0x90);   /* init sense data */
    GPIOI2CWrite_TV(pBIOSInfo, 0x6A, 0x02);   /* init sense data */
    GPIOI2CWrite_TV(pBIOSInfo, 0x1C, 0);
}

void DoTVSWReset_VT1625(VIABIOSInfoPtr pBIOSInfo)
{
    CARD8 tv1d;
    
    tv1d = GPIOI2CRead_TV(pBIOSInfo, 0x1D);
    GPIOI2CWrite_TV(pBIOSInfo, 0x1D, tv1d & 0x7F);
    GPIOI2CWrite_TV(pBIOSInfo, 0x1D, tv1d | 0x80);
}

void VIASetDPA_VT1625(VIABIOSInfoPtr pBIOSInfo, CARD8 DPAValue)
{
    CARD8   data;
    
    data = GPIOI2CRead_TV(pBIOSInfo, 0x05);
    data = (data & 0xE0) | DPAValue;
    GPIOI2CWrite_TV(pBIOSInfo, 0x05, data);
}
