#ifdef XFree86Server
# include "xf86.h"
# include "xf86_OSproc.h"
# define _DRM_MALLOC xalloc
# define _DRM_FREE   xfree
# ifndef XFree86LOADER
#  include <sys/mman.h>
# endif
#else
# include <stdio.h>
# include <stdlib.h>
# include <unistd.h>
# include <string.h>
# include <ctype.h>
# include <fcntl.h>
# include <errno.h>
# include <signal.h>
# include <sys/types.h>
# include <sys/ioctl.h>
# include <sys/mman.h>
# include <sys/time.h>
# ifdef DRM_USE_MALLOC
#  define _DRM_MALLOC malloc
#  define _DRM_FREE   free
extern int xf86InstallSIGIOHandler(int fd, void (*f)(int, void *), void *);
extern int xf86RemoveSIGIOHandler(int fd);
# else
#  include <X11/Xlibint.h>
#  define _DRM_MALLOC Xmalloc
#  define _DRM_FREE   Xfree
# endif
#endif

/* Not all systems have MAP_FAILED defined */
#ifndef MAP_FAILED
#define MAP_FAILED ((void *)-1)
#endif

#ifdef __linux__
#include <sys/sysmacros.h>	/* for makedev() */
#endif
#include "xf86drm.h"
#include "xf86drmVIA.h"
#include "drm.h"

#include "close/via_drm.h"
#include "via_dri.h"

int drmVIAAgpInit(int fd, int offset, int size)
{
    drm_via_agp_t agp;
    agp.offset = offset;
    agp.size = size;

    if (ioctl(fd, DRM_IOCTL_VIA_AGP_INIT, &agp) < 0) {
         return -errno;
    }
    else {
        return 0;
    }
}

int drmVIAFBInit(int fd, int offset, int size)
{
    drm_via_fb_t fb;
    fb.offset = offset;
    fb.size = size;

    if (ioctl(fd, DRM_IOCTL_VIA_FB_INIT, &fb) < 0) {
	return -errno;
    }
    else
	return 0;
}

int drmVIAInitMAP(int fd, drmVIAInit *info)
{
    drm_via_init_t init;

    memset(&init, 0, sizeof(drm_via_init_t));

    init.func = VIA_INIT_MAP;

    init.sarea_priv_offset = info->sarea_priv_offset;
    init.fb_offset = info->fb_offset;
    init.mmio_offset = info->mmio_offset;
    init.agpAddr = info->agpAddr;
/*homer agp flush drm*/
init.agp_offset=info->agp_offset;
/*homer end*/
    if (ioctl(fd, DRM_IOCTL_VIA_MAP_INIT, &init ) < 0) {
        return -errno;
    }
    else
        return 0;
}

int
drmVIAAllocateDMA(int fd, drmVIADMABuf *buf)
{
    if (drmAddMap(fd, 0, buf->size,
		  DRM_SHM, 0,
		  (drm_handle_t *)(&buf->index)) < 0) {
	return -errno;
    }

    if (drmMap(fd,
	       (drmHandle)buf->index,
		buf->size,
	       (drmAddressPtr)(&buf->address)) < 0) {
	return -errno;
    }

    memset(buf->address, 0, buf->size);

    return 0;
}

int drmVIAReleaseDMA(int fd, drmVIADMABuf *buf)
{
    if (drmUnmap((drmAddress)(buf->address), buf->size) < 0)
	return -errno;

    return 0;
}
