/*
 * Copyright 1998-2007 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2007 S3 Graphics, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

/*************************************************************************
 *
 *  File:       via_driver.c
 *  Content:    XFree86 4.0 for VIA/S3G UniChrome
 *
 ************************************************************************/
#include <stdlib.h>
#include "via_driver.h"
#include "xf86RAC.h"
#include "shadowfb.h"

#include "globals.h"
#define DPMS_SERVER
#include "X11/extensions/dpms.h"


#include "via_video.h"
#include "videodev.h"
#include "close/VidInit.h"
#include "via_IntegratedTV.h"
#include "via_hw.h"
#include "via_vt1636.h"

#include "open/debug.h"      /* for DBG_DD */
#include "close/ddmpeg.h"
#include "close/capture.h"
#include "close/via.h"
#ifdef XF86DRI
#include "dri.h"
#endif
#include "via_mergedfb.h" /*added by cliff,for mergedfb*/

#include "via_rotate.h" /* add by STChen, for rotate feature.*/

#include "via_lcd.h"
#include "via_dvi.h"
#include "via_utility.h"


/* hyunzhang 03/02/2007 sync with vexp/vemp */
#include <signal.h>

#include <fcntl.h>
#include <sys/ipc.h>
#include <sys/shm.h>

CARD8 *MMIOMapBase;

static int exityes = 0;
/* sync with vexp/vemp end */
extern Bool    VIAnoPanoramiXExtension;

extern void VIAXVUtilityProc(ScrnInfoPtr pScrn, unsigned char* buf);
extern void UTGetTVInfo(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo);
extern void VIARestoreUserTVSetting(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo);

extern VIDEOMEMORY VidMemory;
static Bool IsVidDataAllocated = FALSE; /* To prevent VidData allocated twice. */
/*
 * prototypes
 */

static void VIAIdentify(int flags);
static Bool VIAProbe(DriverPtr drv, int flags);
static Bool VIAPreInit(ScrnInfoPtr pScrn, int flags);
static Bool VIAEnterVT(int scrnIndex, int flags);
static void VIALeaveVT(int scrnIndex, int flags);
static void VIASave(ScrnInfoPtr pScrn);
static void VIAWriteMode(ScrnInfoPtr pScrn, vgaRegPtr, VIARegPtr);
static Bool VIAModeInit(ScrnInfoPtr pScrn, DisplayModePtr mode);
static Bool VIACloseScreen(int scrnIndex, ScreenPtr pScreen);
static Bool VIASaveScreen(ScreenPtr pScreen, int mode);
static void VIALoadPalette(ScrnInfoPtr pScrn, int numColors, int *indicies,
                           LOCO *colors, VisualPtr pVisual);
static Bool VIAScreenInit(int scrnIndex, ScreenPtr pScreen, int argc,
                          char **argv);
static int VIAInternalScreenInit(int scrnIndex, ScreenPtr pScreen);
static void VIAFreeScreen(int scrnIndex, int flags);
static ModeStatus VIAValidMode(int index, DisplayModePtr mode,
                               Bool verbose, int flags);
static void VIADPMS(ScrnInfoPtr pScrn, int mode, int flags);
static const OptionInfoRec * VIAAvailableOptions(int chipid, int busid);
static void VIAProbeChipset(ScrnInfoPtr pScrn, EntityInfoPtr pEnt);
static Bool VIAFilterMode(ScrnInfoPtr pScrn);
static Bool VIAProbeMonitorLimitation(ScrnInfoPtr pScrn);
static Bool VIAProbeBIOSInfo(ScrnInfoPtr pScrn);

static void VIAEnableMMIO(ScrnInfoPtr pScrn);
#if 0
static void VIADisableMMIO(ScrnInfoPtr pScrn);
#endif
static Bool VIAMapMMIO(ScrnInfoPtr pScrn);
static Bool VIAMapFB(ScrnInfoPtr pScrn);
static void VIAUnmapMem(ScrnInfoPtr pScrn);
#if 0
static int VIAGetMemSize(void);
#endif
Bool VIADeviceSelection(ScrnInfoPtr pScrn);
Bool VIASetDisplayPath_SAMM(ScrnInfoPtr pScrn);
void VIASetDisplayPath(ScrnInfoPtr pScrn);
static void VIABuildModeList(ScrnInfoPtr pScrn);

/* S.T.Chen [2005.11.02]: query NB to get frame buffer size */
static int VIAGetFBSize(ScrnInfoPtr pScrn);

#ifdef _XORG_VER_6_8_LATER_
#ifdef _XORG_VER_6_9_LATER_
static Bool VIADriverFunc(ScrnInfoPtr pScrnInfo, xorgDriverFuncOp op, pointer data);
#else
static Bool VIADriverFunc(ScrnInfoPtr pScrn, xorgRRFuncFlags op, pointer data);
#endif
#endif

#ifdef XF86DRI
void VIAInitialize3DEngine(ScrnInfoPtr pScrn);
Bool b3DRegsInitialized = 0;
#endif

/* E X T E R N   F U N C T I O N S ------------------------------------------*/
/*=* John Sheng [2003.7.04] fix video can't get enough space from
                            Xserver fb manager                    *=*/
extern Bool VIADRIFBInit(ScreenPtr pScreen, VIAPtr pVia);
extern void VIAEnableLCD(VIABIOSInfoPtr pBIOSInfo, LVDSSETTINGINFOPTR pLVDSSettingInfo);
extern Bool VIAPostLCDNotDVO(VIABIOSInfoPtr pBIOSInfo);
extern Bool VIAPostLCDDVO(VIABIOSInfoPtr pBIOSInfo);
extern void VIADisableLCD(VIABIOSInfoPtr pBIOSInfo, LVDSSETTINGINFOPTR pLVDSSettingInfo);
extern Bool VIATMDSIdentify(VIABIOSInfoPtr pBIOSInfo);
extern unsigned char VIAGetPanelInfo(VIABIOSInfoPtr pBIOSInfo);
extern void VIAEnableDFP(VIABIOSInfoPtr pBIOSInfo);
extern void VIADisableDFP(VIABIOSInfoPtr pBIOSInfo);
extern int VIACheckTVExist(VIABIOSInfoPtr pBIOSInfo);
extern void VIAEnableTV(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo);
extern void VIADisableTV(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo);
extern void VIASaveTV(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo, VIARegPtr save);
extern void VIARestoreTV(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo, VIARegPtr restore);


static void VIALoadVideoOptionDefault(VIAPtr pVia,LPVIAGRAPHICINFO lpVIAGraphicInfo);
#if 0
static int GetIGAPath(VIABIOSInfoPtr pBIOSInfo);
#endif
static void FillGraphicInfo(ScrnInfoPtr pScrn);
static void VIACheckIfUseDuoView(ScrnInfoPtr pScrn);
static int TranslateDIPortNameToID(char *pStr);
static void capinfo_init(VIACAPINFO *pCapInfo);
static Bool VIADeviceDispatchDuoView(ScrnInfoPtr pScrn);
static void wait_ap_exit(int signum,siginfo_t *info,void *myact);
static void VIACorrectModeList(ScrnInfoPtr pScrn);
static void VIAModeInitRunOnce(ScrnInfoPtr pScrn);
static int VIAGetIGAInUse(VIABIOSInfoPtr pBIOSInfo);
static void VIASetStartingAddress(ScrnInfoPtr pScrn, int x, int y);
static Bool VIACheckROMName(void);

static void VIAvgaHWddc1SetSpeed(ScrnInfoPtr pScrn, xf86ddcSpeed speed);
/*---------------------------------------------------------------------------*/

DriverRec VIA =
{
    VIA_VERSION,
    DRIVER_NAME,
    VIAIdentify,
    VIAProbe,
    VIAAvailableOptions,
    NULL,
    0
};

/* Supported chipsets */

static SymTabRec VIAChipsets[] = {
    {VIA_CLE266,   "CLE266"},
    {VIA_KM400,    "KM400/KN400"},
    {VIA_K8M800,   "K8M800/K8N800"},
    {VIA_PM800,    "PM800/PM880/CN400"},
    {VIA_P4M800PRO,    "P4M800PRO"},
    {VIA_CX700,    "CX700"},
    {VIA_K8M890,   "K8M890"},
    {VIA_P4M890,   "P4M890"},
    {VIA_CN750,    "CN750"}, 
    {VIA_P4M900,   "P4M900"}, 
    {-1,            NULL }
};


/* This table maps a PCI device ID to a chipset family identifier. */

static PciChipsets VIAPciChipsets[] = {
    {VIA_CLE266,   PCI_CHIP_CLE3122,   RES_SHARED_VGA},
    {VIA_CLE266,   PCI_CHIP_CLE3022,   RES_SHARED_VGA},
    {VIA_KM400,    PCI_CHIP_VT7205,    RES_SHARED_VGA},
    {VIA_KM400,    PCI_CHIP_VT3205,    RES_SHARED_VGA},
    {VIA_K8M800,   PCI_CHIP_VT3204,    RES_SHARED_VGA},
    {VIA_PM800,    PCI_CHIP_VT3259,    RES_SHARED_VGA},
    {VIA_P4M800PRO, PCI_CHIP_VT3314,    RES_SHARED_VGA}, 
    {VIA_CX700,    PCI_CHIP_VT3324,    RES_SHARED_VGA},
    {VIA_K8M890,   PCI_CHIP_VT3336,    RES_SHARED_VGA},
    {VIA_P4M890,   PCI_CHIP_VT3327,    RES_SHARED_VGA},
    {VIA_CN750,    PCI_CHIP_VT3293,    RES_SHARED_VGA},
    {VIA_P4M900,   PCI_CHIP_VT3364,    RES_SHARED_VGA},
    {-1,            -1,                RES_UNDEFINED}
};

int gVIAEntityIndex = -1;

typedef enum {
    OPTION_PCI_BURST,
    OPTION_PCI_RETRY,
    OPTION_NOACCEL,
    OPTION_SWCURSOR,
    OPTION_HWCURSOR,
    OPTION_SHADOW_FB,
    OPTION_ROTATE,
    OPTION_ROTATE_TYPE,
    OPTION_INVERSE,    
    OPTION_USEBIOS,
    OPTION_VIDEORAM,
    OPTION_VIDEORAM1,
    OPTION_TWO_LCD,
    OPTION_TWO_CRT,
    OPTION_CRT2_SUPPORTED,    
    OPTION_ACTIVEDEVICE,
    OPTION_DISPLAY_HARDWARE_LAYOUT,
    OPTION_DVI_PORT,
    OPTION_DVI_DUAL_EDGE,
    OPTION_DVI_BUS_WIDTH,
    OPTION_SIMULTANEOUS,
    OPTION_LCD_PORT,
    OPTION_LCD_BUS_WIDTH,
    OPTION_CENTER,
    OPTION_TV_PORT,
    OPTION_TVDOTCRAWL,
    OPTION_TV2DOTCRAWL,
    OPTION_TVPROGRESSIVE,
    OPTION_TV2PROGRESSIVE,
    OPTION_TVTYPE,
    OPTION_TV2TYPE,
    OPTION_TVOUTPUT,
    OPTION_TV2OUTPUT,
    OPTION_TVOUTPUT2,
    OPTION_TVVSCAN,
    OPTION_TVHSCALE,
    OPTION_TVVSCALE,
    OPTION_TV2HSCALE,
    OPTION_TV2VSCALE,
    OPTION_TVENCODER,
    OPTION_TVFLICKERFILTER,
    OPTION_TV2FLICKERFILTER,
    OPTION_REFRESH,
    OPTION_DISABLEVQ,
    OPTION_NODDCVALUE,
    OPTION_DEVICE_SWITCH_HOTKEY,
    OPTION_CAP0_DEINTERLACE,
    OPTION_CAP1_DEINTERLACE,
    OPTION_CAP0_FIELDSWAP,
    OPTION_NOCAP0_HFILTER,
    OPTION_NOCAP1_HFILTER,
    OPTION_CAPTURE_OVERSCANOFF,
    OPTION_HQVFILTER_MANUALSELECT,
    OPTION_MPEG_HQV_VFILTER_MODE,
    OPTION_MPEG_HQV_HFILTER_MODE,
    OPTION_CAP_HQV_VFILTER_MODE,
    OPTION_CAP_HQV_HFILTER_MODE,
    OPTION_SETMPEG_FBNUMBER,
    OPTION_CAP1_USEIRQ,
    OPTION_CAP0_SWITCHTOV3,
    OPTION_HQV_MANUALSWITCH,
    OPTION_DRIXINERAMA,
    OPTION_CapInterVersion,
    OPTION_Cap0NoAutoDetect,
    OPTION_Cap0DataWidth,
    OPTION_Cap0VDODecoderDev,
    OPTION_Cap0VDODecoderI2C,
    OPTION_Cap0CompositeMode,
    OPTION_Cap0SVideoMode,
    OPTION_Cap0TunerMode,
    OPTION_Cap0Scaler,
    OPTION_Cap0Brightness,
    OPTION_Cap0Contrast,
    OPTION_Cap0Hue,
    OPTION_Cap0Saturation,
    OPTION_Cap0Standard,
    OPTION_Cap0TunerDev,
    OPTION_Cap0TunerI2C,
    OPTION_Cap0SourceWidth,
    OPTION_Cap0SourceHeight,
    OPTION_Cap1NoAutoDetect,
    OPTION_Cap1DataWidth,
    OPTION_Cap1VDODecoderDev,
    OPTION_Cap1VDODecoderI2C,
    OPTION_Cap1CompositeMode,
    OPTION_Cap1SVideoMode,
    OPTION_Cap1TunerMode,
    OPTION_Cap1Scaler,
    OPTION_Cap1Brightness,
    OPTION_Cap1Contrast,
    OPTION_Cap1Hue,
    OPTION_Cap1Saturation,
    OPTION_Cap1Standard,
    OPTION_Cap1TunerDev,
    OPTION_Cap1TunerI2C,
    OPTION_Cap1SourceWidth,
    OPTION_Cap1SourceHeight,
    OPTION_NOMPEGAddOneLineOnBottom,
    OPTION_ReDrawColorkey,
    OPTION_Cap0CLKinverse,
    OPTION_Cap1CLKinverse,
    OPTION_DeBlockingEnable,
    OPTION_DeBlockingMinimumWidth,
    OPTION_DeBlockingMinimumHeight,
    OPTION_Use2DBitBltSignature,
    OPTION_MERGEDFB, /*add by cliff to enable mergedfb,switch to virtualscreen*/
    OPTION_NOVIAXINERAMA,/*and set screen layout*/
    OPTION_SCRN2POS,
    OPTION_EPIA_DVI,/* add by Jason for TMDS on EPIA*/
    OPTION_CCIR601,/* add by Jason for CCIR601 support*/
    OPTION_PatchMPEGDecodeSliceErrorBitstreamDisable,
    OPTION_VideoOnDevice, /*set video to work on which device for 3314,3204 etc.in duo view mode */
    OPTION_FORCE_LCD, /* add by S.T.Chen, Force to enable LCD without mobile version BIOS. */
    OPTION_LCDPANELID, /* add by S.T.Chen, to simply LCD panel option. */
    OPTION_DRIVER_VERSION,
    OPTION_DPA_SETTING_DVP0,
    OPTION_DPA_SETTING_DVP1,
    OPTION_DPA_SETTING_DFPHIGH,
    OPTION_DPA_SETTING_DFPLOW,
    OPTION_DPA_SETTING_VT1636,
    OPTION_DPA_SETTING_VT1625,
    OPTION_DPA_SETTING_VT1623,        
    OPTION_DISP3DScaling,
    OPTION_VIDEO_COLOR_KEY
} VIAOpts;


static OptionInfoRec VIAOptions[] =
{
    {OPTION_NOACCEL,    "NoAccel",      OPTV_BOOLEAN, {0}, FALSE},
    {OPTION_HWCURSOR,   "HWCursor",     OPTV_BOOLEAN, {0}, FALSE},
    {OPTION_SWCURSOR,   "SWCursor",     OPTV_BOOLEAN, {0}, FALSE},
    {OPTION_SHADOW_FB,  "ShadowFB",     OPTV_BOOLEAN, {0}, FALSE},
    {OPTION_ROTATE,     "Rotate",     OPTV_ANYSTR,  {0}, FALSE},
    {OPTION_ROTATE_TYPE,"RotateType",     OPTV_ANYSTR,  {0}, FALSE},
    {OPTION_INVERSE,    "Inverse",      OPTV_BOOLEAN, {0}, FALSE},    
    {OPTION_USEBIOS,    "UseBIOS",      OPTV_BOOLEAN, {0}, FALSE},
    {OPTION_VIDEORAM,   "VideoRAM",     OPTV_INTEGER, {0}, FALSE},
    {OPTION_VIDEORAM1,  "VideoRAM1",    OPTV_INTEGER, {0}, FALSE},
    {OPTION_TWO_LCD,    "TwoLCD",       OPTV_BOOLEAN, {0}, FALSE},
    {OPTION_TWO_CRT,    "TwoCRT",       OPTV_BOOLEAN, {0}, FALSE},
    {OPTION_CRT2_SUPPORTED, "CRT2Supported",OPTV_BOOLEAN, {0}, FALSE},    
    {OPTION_ACTIVEDEVICE,   "ActiveDevice", OPTV_ANYSTR,  {0}, FALSE},
    {OPTION_DISPLAY_HARDWARE_LAYOUT, "DisplayHardwareLayout",   OPTV_ANYSTR,  {0}, FALSE},
    {OPTION_DVI_PORT,       "DVIPort",      OPTV_ANYSTR,  {0}, FALSE},
    {OPTION_DVI_DUAL_EDGE,  "DVIDualEdge",  OPTV_ANYSTR,  {0}, FALSE},
    {OPTION_DVI_BUS_WIDTH,  "DVIBusWidth",  OPTV_ANYSTR, {0}, FALSE},
    {OPTION_SIMULTANEOUS,   "Simultaneous", OPTV_BOOLEAN, {0}, FALSE},
    {OPTION_LCD_PORT,       "LCDPort",      OPTV_ANYSTR, {0}, FALSE},
    {OPTION_LCD_BUS_WIDTH,  "LCDBusWidth",  OPTV_ANYSTR, {0}, FALSE},
    {OPTION_CENTER,         "Center",       OPTV_BOOLEAN, {0}, FALSE},
    {OPTION_LCDPANELID,     "PanelID",      OPTV_INTEGER, {0}, FALSE},
    {OPTION_TV_PORT,        "TVPort",       OPTV_ANYSTR, {0}, FALSE},
    {OPTION_TVDOTCRAWL,     "TVDotCrawl",   OPTV_BOOLEAN, {0}, FALSE},
    {OPTION_TV2DOTCRAWL,    "TV2DotCrawl",   OPTV_BOOLEAN, {0}, FALSE},
    {OPTION_TVPROGRESSIVE,  "TVProgressive", OPTV_BOOLEAN, {0}, FALSE},
    {OPTION_TV2PROGRESSIVE, "TV2Progressive", OPTV_BOOLEAN, {0}, FALSE},
    {OPTION_TVTYPE,     "TVType",       OPTV_ANYSTR,  {0}, FALSE},
    {OPTION_TV2TYPE,    "TV2Type",      OPTV_ANYSTR,  {0}, FALSE},
    {OPTION_TVOUTPUT,   "TVOutput",     OPTV_ANYSTR,  {0}, FALSE},
    {OPTION_TV2OUTPUT,  "TV2Output",    OPTV_ANYSTR,  {0}, FALSE},
    {OPTION_TVOUTPUT2,  "TVOutput2",    OPTV_ANYSTR,  {0}, FALSE},
    {OPTION_TVVSCAN,    "TVVScan",      OPTV_ANYSTR,  {0}, FALSE},
    {OPTION_TVHSCALE,   "TVHScale",     OPTV_INTEGER, {0}, FALSE},
    {OPTION_TVVSCALE,   "TVVScale",     OPTV_INTEGER, {0}, FALSE},
    {OPTION_TV2HSCALE,  "TV2HScale",    OPTV_INTEGER, {0}, FALSE},
    {OPTION_TV2VSCALE,  "TV2VScale",    OPTV_INTEGER, {0}, FALSE},
    {OPTION_TVENCODER,  "TVEncoder",    OPTV_ANYSTR,  {0}, FALSE},
    {OPTION_TVFLICKERFILTER,    "TVFlickerFilter",  OPTV_INTEGER,   {0},    FALSE},
    {OPTION_TV2FLICKERFILTER,   "TV2FlickerFilter", OPTV_INTEGER,   {0},    FALSE},
    {OPTION_REFRESH,    "Refresh",      OPTV_INTEGER, {0}, FALSE},
    {OPTION_DISABLEVQ,  "DisableVQ",    OPTV_BOOLEAN, {0}, FALSE},
    {OPTION_NODDCVALUE, "NoDDCValue",   OPTV_BOOLEAN, {0}, FALSE},
    {OPTION_DEVICE_SWITCH_HOTKEY, "DeviceSwitchHotkey", OPTV_BOOLEAN, {0}, FALSE},
    {OPTION_CAP0_DEINTERLACE, "Cap0Deinterlace",OPTV_ANYSTR,  {0}, FALSE},
    {OPTION_CAP1_DEINTERLACE, "Cap1Deinterlace",OPTV_ANYSTR,  {0}, FALSE},
    {OPTION_CAP0_FIELDSWAP, "Cap0FieldSwap",    OPTV_BOOLEAN,  {0}, FALSE},
    {OPTION_NOCAP0_HFILTER, "NoCap0HFilter",    OPTV_BOOLEAN,  {0}, FALSE},
    {OPTION_NOCAP1_HFILTER, "NoCap1HFilter",    OPTV_BOOLEAN,  {0}, FALSE},
    {OPTION_CAPTURE_OVERSCANOFF, "CaptureOverScanOff",    OPTV_BOOLEAN,  {0}, FALSE},
    {OPTION_HQVFILTER_MANUALSELECT, "HQVFilterManualSelect",    OPTV_BOOLEAN,  {0}, FALSE},
    {OPTION_MPEG_HQV_VFILTER_MODE,   "MpegHQVVFilterMode", OPTV_INTEGER, {0}, FALSE},
    {OPTION_MPEG_HQV_HFILTER_MODE,   "MpegHQVHFilterMode", OPTV_INTEGER, {0}, FALSE},
    {OPTION_CAP_HQV_VFILTER_MODE,   "CapHQVVFilterMode", OPTV_INTEGER, {0}, FALSE},
    {OPTION_CAP_HQV_HFILTER_MODE,   "CapHQVHFilterMode", OPTV_INTEGER, {0}, FALSE},
    {OPTION_SETMPEG_FBNUMBER, "SetMpegFBNumber",    OPTV_BOOLEAN,  {0}, FALSE},
    {OPTION_CAP1_USEIRQ, "Cap1UseIRQ",    OPTV_BOOLEAN,  {0}, FALSE},
    {OPTION_CAP0_SWITCHTOV3, "Cap0SwitchToV3",    OPTV_BOOLEAN,  {0}, FALSE},
    {OPTION_HQV_MANUALSWITCH, "HQVManualSwitch",    OPTV_BOOLEAN,  {0}, FALSE},
    {OPTION_DRIXINERAMA,  "DRIXINERAMA",    OPTV_BOOLEAN, {0}, FALSE},
    {OPTION_CapInterVersion,"CapInterVersion",  OPTV_INTEGER,  {0}, FALSE},
    {OPTION_Cap0NoAutoDetect,"Cap0NoAutoDetect",  OPTV_BOOLEAN,  {0}, FALSE},
    {OPTION_Cap0DataWidth,  "Cap0DataWidth",    OPTV_INTEGER,  {0}, FALSE},
    {OPTION_Cap0VDODecoderDev,  "Cap0VDODecoderDev",   OPTV_ANYSTR, {0}, FALSE},
    {OPTION_Cap0VDODecoderI2C,  "Cap0VDODecoderI2C",    OPTV_INTEGER, {0}, FALSE},
    {OPTION_Cap0CompositeMode,   "Cap0CompositeMode", OPTV_INTEGER, {0}, FALSE},
    {OPTION_Cap0SVideoMode,   "Cap0SVideoMode", OPTV_INTEGER, {0}, FALSE},
    {OPTION_Cap0TunerMode,   "Cap0TunerMode", OPTV_INTEGER, {0}, FALSE},
    {OPTION_Cap0Scaler,   "Cap0Scaler", OPTV_INTEGER, {0}, FALSE},
    {OPTION_Cap0Brightness,   "Cap0Brightness", OPTV_INTEGER, {0}, FALSE},
    {OPTION_Cap0Contrast,   "Cap0Contrast", OPTV_INTEGER, {0}, FALSE},
    {OPTION_Cap0Hue,   "Cap0Hue", OPTV_INTEGER, {0}, FALSE},
    {OPTION_Cap0Saturation,   "Cap0Saturation", OPTV_INTEGER, {0}, FALSE},
    {OPTION_Cap0Standard,   "Cap0Standard", OPTV_ANYSTR, {0}, FALSE},
    {OPTION_Cap0TunerDev,   "Cap0TunerDev", OPTV_ANYSTR, {0}, FALSE},
    {OPTION_Cap0TunerI2C,   "Cap0TunerI2C", OPTV_ANYSTR, {0}, FALSE},
    {OPTION_Cap0SourceWidth,  "Cap0SourceWidth",    OPTV_INTEGER, {0}, FALSE},
    {OPTION_Cap0SourceHeight,  "Cap0SourceHeight",    OPTV_INTEGER, {0}, FALSE},
    {OPTION_Cap1NoAutoDetect,"Cap1NoAutoDetect",  OPTV_BOOLEAN,  {0}, FALSE},
    {OPTION_Cap1DataWidth,  "Cap1DataWidth",    OPTV_INTEGER,  {0}, FALSE},
    {OPTION_Cap1VDODecoderDev,  "Cap1VDODecoderDev",   OPTV_ANYSTR, {0}, FALSE},
    {OPTION_Cap1VDODecoderI2C,  "Cap1VDODecoderI2C",    OPTV_INTEGER, {0}, FALSE},
    {OPTION_Cap1CompositeMode,   "Cap1CompositeMode", OPTV_INTEGER, {0}, FALSE},
    {OPTION_Cap1SVideoMode,   "Cap1SVideoMode", OPTV_INTEGER, {0}, FALSE},
    {OPTION_Cap1TunerMode,   "Cap1TunerMode", OPTV_INTEGER, {0}, FALSE},
    {OPTION_Cap1Scaler,   "Cap1Scaler", OPTV_INTEGER, {0}, FALSE},
    {OPTION_Cap1Brightness,   "Cap1Brightness", OPTV_INTEGER, {0}, FALSE},
    {OPTION_Cap1Contrast,   "Cap1Contrast", OPTV_INTEGER, {0}, FALSE},
    {OPTION_Cap1Hue,   "Cap1Hue", OPTV_INTEGER, {0}, FALSE},
    {OPTION_Cap1Saturation,   "Cap1Saturation", OPTV_INTEGER, {0}, FALSE},
    {OPTION_Cap1Standard,   "Cap1Standard", OPTV_ANYSTR, {0}, FALSE},
    {OPTION_Cap1TunerDev,   "Cap1TunerDev", OPTV_ANYSTR, {0}, FALSE},
    {OPTION_Cap1TunerI2C,   "Cap1TunerI2C", OPTV_ANYSTR, {0}, FALSE},
    {OPTION_Cap1SourceWidth,  "Cap1SourceWidth",    OPTV_INTEGER, {0}, FALSE},
    {OPTION_Cap1SourceHeight,  "Cap1SourceHeight",    OPTV_INTEGER, {0}, FALSE},
    {OPTION_NOMPEGAddOneLineOnBottom,   "NOMPEGAddOneLineOnBottom", OPTV_BOOLEAN, {0}, FALSE},
    {OPTION_ReDrawColorkey,   "ReDrawColorkey", OPTV_BOOLEAN, {0}, FALSE},
    {OPTION_Cap0CLKinverse,   "Cap0CLKinverse", OPTV_BOOLEAN, {0}, FALSE},
    {OPTION_Cap1CLKinverse,   "Cap1CLKinverse", OPTV_BOOLEAN, {0}, FALSE},
    {OPTION_DeBlockingEnable,   "DeBlockingEnable", OPTV_BOOLEAN, {0}, FALSE},
    {OPTION_DeBlockingMinimumWidth,  "DeBlockingMinimumWidth",    OPTV_INTEGER, {0}, FALSE},
    {OPTION_DeBlockingMinimumHeight,  "DeBlockingMinimumHeight",    OPTV_INTEGER, {0}, FALSE},
    {OPTION_PatchMPEGDecodeSliceErrorBitstreamDisable,   "PatchMPEGDecodeSliceErrorBitstreamDisable", OPTV_BOOLEAN, {0}, FALSE},
    {OPTION_Use2DBitBltSignature, "Use2DBitBltSignature",    OPTV_BOOLEAN,  {0}, FALSE},
    {OPTION_MERGEDFB, "MergedFB",    OPTV_BOOLEAN,  {0}, FALSE},
    {OPTION_NOVIAXINERAMA, "NoVIAXinerama",    OPTV_BOOLEAN,  {0}, FALSE},
    {OPTION_SCRN2POS,   "Scrn2Pos", OPTV_ANYSTR, {0}, FALSE},   
    {OPTION_EPIA_DVI, "EPIA_DVI",    OPTV_BOOLEAN,  {0}, FALSE},
    {OPTION_VideoOnDevice,  "VideoOnDevice",       OPTV_ANYSTR,  {0}, FALSE},
    {OPTION_CCIR601, "CCIR601",    OPTV_BOOLEAN,  {0}, FALSE},
    {OPTION_FORCE_LCD, "ForceLCD", OPTV_BOOLEAN, {0}, FALSE},
    {OPTION_SCRN2POS,   "Scrn2Pos", OPTV_ANYSTR, {0}, FALSE},
    {OPTION_DPA_SETTING_DVP0,       "DPASetting_DVP0",      OPTV_ANYSTR, {0}, FALSE},
    {OPTION_DPA_SETTING_DVP1,       "DPASetting_DVP1",      OPTV_ANYSTR, {0}, FALSE},
    {OPTION_DPA_SETTING_DFPHIGH,    "DPASetting_DFPHIGH",   OPTV_ANYSTR, {0}, FALSE},
    {OPTION_DPA_SETTING_DFPLOW,     "DPASetting_DFPLOW",    OPTV_ANYSTR, {0}, FALSE},
    {OPTION_DPA_SETTING_VT1636,     "DPASetting_VT1636",    OPTV_ANYSTR, {0}, FALSE},
    {OPTION_DPA_SETTING_VT1625,     "DPASetting_VT1625",    OPTV_ANYSTR, {0}, FALSE},
    {OPTION_DPA_SETTING_VT1623,     "DPASetting_VT1623",    OPTV_ANYSTR, {0}, FALSE},
    {OPTION_DISP3DScaling,     "DISPLAY_3DScaling",    OPTV_BOOLEAN, {0}, FALSE},
    {OPTION_VIDEO_COLOR_KEY,    "VideoColorKey",  OPTV_INTEGER, {0}, FALSE},
    {-1,                NULL,           OPTV_NONE,    {0}, FALSE}
};


static const char *vgaHWSymbols[] = {
    "vgaHWGetHWRec",
    "vgaHWSetMmioFuncs",
    "vgaHWSetStdFuncs",
    "vgaHWGetIOBase",
    "vgaHWSave",
    "vgaHWProtect",
    "vgaHWRestore",
    "vgaHWMapMem",
    "vgaHWUnmapMem",
    "vgaHWInit",
    "vgaHWSaveScreen",
    "vgaHWLock",
    "vgaHWUnlock",
    "vgaHWFreeHWRec",
    NULL
};


static const char *ramdacSymbols[] = {
    "xf86InitCursor",
    "xf86CreateCursorInfoRec",
    "xf86DestroyCursorInfoRec",
    NULL
};

static const char *vbeSymbols[] = {
    "VBEInit",
    "VBEGetVBEMode",
    "VBESetVBEMode",
    "vbeDoEDID",
    "vbeFree",
    NULL
};

static const char *vbeSymbols429900[] = {
    "VBEExtendedInit",
    "VBEGetVBEMode",
    "VBESetVBEMode",
    "vbeDoEDID",
    "vbeFree",
    NULL
};

static const char *ddcSymbols[] = {
    "xf86PrintEDID",
    "xf86DoEDID_DDC1",
    "xf86DoEDID_DDC2",
    "xf86SetDDCproperties",
    NULL
};


static const char *i2cSymbols[] = {
    "xf86CreateI2CBusRec",
    "xf86I2CBusInit",
    "xf86CreateI2CDevRec",
    "xf86I2CDevInit",
    "xf86I2CWriteRead",
    "xf86I2CProbeAddress",
    "xf86DestroyI2CDevRec",
    NULL
};

static const char *xaaSymbols[] = {
    "XAAGetCopyROP",
    "XAAGetCopyROP_PM",
    "XAAGetPatternROP",
    "XAACreateInfoRec",
    "XAADestroyInfoRec",
    "XAAInit",
    "XAAFillSolidRects",
    NULL
};

static const char *int10Symbols[] = {
    "xf86FreeInt10",
    "xf86InitInt10",
    NULL
};

static const char *shadowSymbols[] = {
    "ShadowFBInit",
    NULL
};

#ifdef USE_FB
static const char *fbSymbols[] = {
    "fbScreenInit",
    "fbPictureInit",
    NULL
};
#else
static const char *cfbSymbols[] = {
    "cfbScreenInit",
    "cfb16ScreenInit",
    "cfb24ScreenInit",
    "cfb24_32ScreenInit",
    "cfb32ScreenInit",
    "cfb16BresS",
    "cfb24BresS",
    NULL
};
#endif

/* for Via DVD and Video support */
/*
 * No use now, for old Xserver module code use
 */
#if 0
static const char *mpegSymbols[] = {
     "VDEC_Set_PAL",
     "VDEC_Set_NTSC",
     "VDEC_Set_Hue",
     "VDEC_SET_TV",
     "VDEC_SET_AV",
     "VDEC_SET_SVIDEO",
     "VIAPutVideo",
     "VIAStopVideo",
     "VIAVideoControl",
     "VDEC_Set_Luminance",
     "VDEC_Set_Saturation",
     "VDEC_Set_Brightness",
     "VDEC_Set_Contrast",
     "VDEC_Init_Reg",
     "VDEC_Detect",
     "TUNER_DetectTuner",
     "TUNER_Init",
     "TUNER_SetTVBand_Cable",
     "TUNER_SetTVBand_Air",
     "TUNER_SetVideoStandard_PAL",
     "TUNER_SetVideoStandard_NTSC",
     "TUNER_SetChannel",
     "SUBPicture",
     "DisplayControl",
     "SliceReceiveData",
     "BeginPicture",
     "EndPicture",
     "DriverProc",
     "gVIAGraphicInfo",
     NULL
};
#endif

#ifdef XFree86LOADER
#ifdef XF86DRI
static const char *drmSymbols[] = {
    "drmOpen",
    "drmClose",
    "drmAddMap",
    "drmAgpAcquire",
    "drmAgpAlloc",
    "drmAgpBase",
    "drmAgpBind",
    "drmAgpEnable",
    "drmAgpFree",
    "drmAgpGetMode",
    "drmAgpRelease",
    "drmCtlInstHandler",
    "drmGetInterruptFromBusID",
    "drmMap",
    "drmUnmap",
    NULL
};

static const char *driSymbols[] = {
    "DRICloseScreen",
    "DRICreateInfoRec",
    "DRIDestroyInfoRec",
    "DRIFinishScreenInit",
    "DRIGetSAREAPrivate",
    "DRILock",
    "DRIQueryVersion",
    "DRIScreenInit",
    "DRIUnlock",
    "GlxSetVisualConfigs",
    NULL
};
#endif

static MODULESETUPPROTO(VIASetup);

static XF86ModuleVersionInfo VIAVersRec = {
    "via",
    MODULEVENDORSTRING,
    MODINFOSTRING1,
    MODINFOSTRING2,
    XF86_VERSION_CURRENT,
    VERSION_MAJOR, VERSION_MINOR, PATCHLEVEL,
    ABI_CLASS_NONE,
    ABI_VIDEODRV_VERSION,
    MOD_CLASS_VIDEODRV,
    {0, 0, 0, 0}
};

XF86ModuleData viaarenaModuleData = {&VIAVersRec, VIASetup, NULL};

unsigned int via_xf86_version_current;

static void VIALoadVideoOptionDefault(VIAPtr pVia,LPVIAGRAPHICINFO lpVIAGraphicInfo) 
{
     switch ( pVia->ChipId )
         {
         case VIA_DEVICE_CLE1:
         case VIA_DEVICE_CLE2:
             pVia->CapInfo[0].dwDeinterlaceMode = CAP_BOB;
             pVia->CapInfo[1].dwDeinterlaceMode = CAP_BOB;
             break;
             
         case VIA_DEVICE_VT3259:
             pVia->CapInfo[0].dwDeinterlaceMode = CAP_MEDIAN;
             pVia->CapInfo[1].dwDeinterlaceMode = CAP_MEDIAN;
             break;
             
         default:
             pVia->CapInfo[0].dwDeinterlaceMode = CAP_MEDIAN;
             pVia->CapInfo[1].dwDeinterlaceMode = CAP_MEDIAN;
             break;
         }
     pVia->CapInfo[0].FieldSwap = TRUE;
     pVia->CapInfo[0].No_HFilter = FALSE;
     pVia->CapInfo[0].Capture_OverScanOff = FALSE;

     lpVIAGraphicInfo->HQVFilter_ManualSelect = FALSE;
     lpVIAGraphicInfo->MpegHQV_VFilterMode = 0;
     lpVIAGraphicInfo->MpegHQV_HFilterMode = 0;
     lpVIAGraphicInfo->CapHQV_VFilterMode = 0;
     lpVIAGraphicInfo->CapHQV_HFilterMode = 0;
     lpVIAGraphicInfo->SetMpeg_FBNumber = FALSE;
     lpVIAGraphicInfo->Cap1_UseIRQ = FALSE;
     lpVIAGraphicInfo->Cap0_SwitchToV3 = FALSE;
     lpVIAGraphicInfo->HQV_ManualSwitch = FALSE;

     pVia->CapInfo[0].CapInterVersion = 0;
     pVia->CapInfo[0].NoAutoDetect = FALSE;
     pVia->CapInfo[0].DataWidth = 8;
     pVia->CapInfo[0].VideoDecoder = CAP_NONE;
     pVia->CapInfo[0].Vdec_Slave_Write = 0;
     pVia->CapInfo[0].CompositeMode = 2;
     pVia->CapInfo[0].SVideoMode = 7;
     pVia->CapInfo[0].TunerMode = 0;
     pVia->CapInfo[0].Scaler = 0;
     pVia->CapInfo[0].Brightness = 128;
     pVia->CapInfo[0].Contrast = 71;
     pVia->CapInfo[0].Hue = 0;
     pVia->CapInfo[0].Saturation = 64;
     pVia->CapInfo[0].Standard = VIDEO_MODE_NTSC;
     pVia->CapInfo[0].Tuner = FI1236MK2;
     pVia->CapInfo[0].Tuner_Slave_Write = 198;
     pVia->CapInfo[0].SourceWidth = 0;
     pVia->CapInfo[0].SourceHeight = 0;

     pVia->CapInfo[1].FieldSwap = TRUE;
     pVia->CapInfo[1].No_HFilter = FALSE;
     pVia->CapInfo[1].Capture_OverScanOff = FALSE;
     pVia->CapInfo[1].CapInterVersion = pVia->CapInfo[0].CapInterVersion;
     pVia->CapInfo[1].NoAutoDetect = FALSE;     
     pVia->CapInfo[1].DataWidth = 8;
     pVia->CapInfo[1].VideoDecoder = CAP_NONE;
     pVia->CapInfo[1].CompositeMode = 2;
     pVia->CapInfo[1].SVideoMode = 7;
     pVia->CapInfo[1].TunerMode = 0;
     pVia->CapInfo[1].Scaler = 0;
     pVia->CapInfo[1].Brightness = 128;
     pVia->CapInfo[1].Contrast = 71;
     pVia->CapInfo[1].Hue = 0;
     pVia->CapInfo[1].Saturation = 64;
     pVia->CapInfo[1].Standard = VIDEO_MODE_NTSC;
     pVia->CapInfo[1].Tuner = FI1236MK2;
     pVia->CapInfo[1].Tuner_Slave_Write = 192;
     pVia->CapInfo[1].SourceWidth = 0;
     pVia->CapInfo[1].SourceHeight = 0;

     lpVIAGraphicInfo->NoMPEGAddOneLineOnBottom = FALSE;
     lpVIAGraphicInfo->ReDrawColorkey = FALSE;
     lpVIAGraphicInfo->DeBlockingEnable = FALSE;
     lpVIAGraphicInfo->dwDeBlockingMinimumWidth = 320;     
     lpVIAGraphicInfo->dwDeBlockingMinimumHeight = 240;
     lpVIAGraphicInfo->dw3to2PullDownDisable = FALSE;
     /*lpVIAGraphicInfo->bPatchMPEGDecodeSliceErrorBitstreamDisable = FALSE;*/
     lpVIAGraphicInfo->bUse2DBitBltSignature = TRUE;
     return;
}

/* Load video option value from .VIAVIDEORC */
Bool VIALoadVideoOption(VIAPtr pVia,LPVIAGRAPHICINFO lpVIAGraphicInfo)
{
    FILE	*VIAVIDEORC = NULL;
    char 	filename[40], *home = NULL;
    Bool	ret = FALSE;
    char line[1024];
    char *value;

    int iValue;


   xf86Msg(X_INFO, "VIALoadVideoOption\n");

    home="/etc/X11";
    if (home)
    {
        filename[0] = '\0';
        strcpy(filename, home);
        strcat(filename, "/.VIAVIDEORC");
        if ((VIAVIDEORC = fopen(filename, "r")) == NULL)
        {
            xf86Msg(X_WARNING,
                    "No Video option file .VIAVIDEORC exist. \n");
            ret = FALSE;
        }
        else
        {
            /*
             *  Default video setting,
             *  Can be replaced by user-specified value later.
             */
            switch ( pVia->ChipId )
            {
            case VIA_DEVICE_CLE1:
            case VIA_DEVICE_CLE2:
                pVia->CapInfo[0].dwDeinterlaceMode = CAP_BOB;
                pVia->CapInfo[1].dwDeinterlaceMode = CAP_BOB;
                break;

            case VIA_DEVICE_VT3259:
                pVia->CapInfo[0].dwDeinterlaceMode = CAP_MEDIAN;
                pVia->CapInfo[1].dwDeinterlaceMode = CAP_MEDIAN;
                break;

            default:
                pVia->CapInfo[0].dwDeinterlaceMode = CAP_MEDIAN;
                pVia->CapInfo[1].dwDeinterlaceMode = CAP_MEDIAN;
                break;
            }

            while (fgets (line, 1023, VIAVIDEORC))
            {
                 line[strlen(line)-1]= (char) 0;

                 if (line[0] == '#') continue;

                 if ((value = strchr (line, ':')))
                 {
                 	*value = (char) 0;
	     			value++;

                   	if (strcmp(line,"CAP0_DEINTERLACE")==0)
       	     		{
     	               	if (atoi(value)==CAP_BOB)
     	               	{
     	                   	pVia->CapInfo[0].dwDeinterlaceMode = CAP_BOB;
     	                	DBG_DD(("Capture 0 de-interlace mode is Bob\n"));
     	               	}
     	               	else if (atoi(value)==CAP_WEAVE)
             	        {
     	                   	pVia->CapInfo[0].dwDeinterlaceMode = CAP_WEAVE;
 	         	   			DBG_DD(("Capture 0 de-interlace mode is Weave\n"));
     	            	}
                        else if (atoi(value)==CAP_MEDIAN)
                        {
                            pVia->CapInfo[0].dwDeinterlaceMode = CAP_MEDIAN;
                            DBG_DD(("Capture 0 de-interlace mode is Median\n"));  
                        }
       	     		}
                    else  if (strcmp(line,"CAP1_DEINTERLACE")==0)
     	     		{
     	               	if (atoi(value)==CAP_BOB)
     	               	{
     	                    pVia->CapInfo[1].dwDeinterlaceMode = CAP_BOB;
     	                	DBG_DD(("Capture 1 de-interlace mode is Bob\n"));
     	               	}
     	               	else if (atoi(value)==CAP_WEAVE)
             	        {
     	                	pVia->CapInfo[1].dwDeinterlaceMode = CAP_WEAVE;
 	                    	DBG_DD(("Capture 1 de-interlace mode is Weave\n"));
     	            	}
                        else if (atoi(value)==CAP_MEDIAN)
                        {
                            pVia->CapInfo[1].dwDeinterlaceMode = CAP_MEDIAN;
                            DBG_DD(("Capture 1 de-interlace mode is Median\n"));  
                        }
       	     		}
                    else  if (strcmp(line,"CAP0_FIELDSWAP")==0)
     	     		{
     	              	if (atoi(value)==TRUE)
     	              	{
     	                   	pVia->CapInfo[0].FieldSwap = TRUE;
   	               	   		DBG_DD(("Option: Cap0_FieldSwap Enabled\n"));
     	              	}	
     	              	else
     	              	{
     	                   	pVia->CapInfo[0].FieldSwap = FALSE;
      	                	DBG_DD(("Option: Cap0_FieldSwap Disabled\n"));
     	            	}
       	     		}
                    else  if (strcmp(line,"NOCAP0_HFILTER")==0)
     	     		{
     	              	if (atoi(value)==TRUE)
     	              	{
     	                   	pVia->CapInfo[0].No_HFilter = TRUE;
   	               	   		DBG_DD(("Option: Cap0_HFilter Disabled\n"));
     	              	}
     	              	else
     	              	{
     	    	   			pVia->CapInfo[0].No_HFilter = FALSE;
      	    	   			DBG_DD(("Option: Cap0_HFilter Enabled\n"));
     	            	}
       	     		}
                    else  if (strcmp(line,"NOCAP1_HFILTER")==0)
     	     		{
     	              	if (atoi(value)==TRUE)
     	              	{
     	                   	pVia->CapInfo[1].No_HFilter = TRUE;
   	                   		DBG_DD(("Option: Cap1_HFilter Disabled\n"));
                        }
     	              	else
     	              	{
     	    	   			pVia->CapInfo[1].No_HFilter = FALSE;
      	    	   			DBG_DD(("Option: Cap1_HFilter Enabled\n"));
     	            	}
       	     		}
                    else  if (strcmp(line,"CAPTURE_OVERSCANOFF")==0)
                    {
     	              	if (atoi(value)==TRUE)
     	              	{
     	                   	pVia->CapInfo[0].Capture_OverScanOff = TRUE;
                            pVia->CapInfo[1].Capture_OverScanOff = TRUE;
   	                   		DBG_DD(("Option: Capture Over Scan OFF\n"));
     	              	}
     	              	else
     	              	{
     	                    pVia->CapInfo[0].Capture_OverScanOff = FALSE;
                            pVia->CapInfo[1].Capture_OverScanOff = FALSE;
      	    	   			DBG_DD(("Option: Capture Over Scan ON\n"));
     	            	}
       	     		}
                    else  if (strcmp(line,"HQVFILTER_MANUALSELECT")==0)
     	     		{
     	              	if (atoi(value)==TRUE)
     	             	{
     	                 	lpVIAGraphicInfo->HQVFilter_ManualSelect = TRUE;
   	               	 		DBG_DD(("Option: HQV Filter Manual Select Enabled\n"));
     	             	}
     	             	else
     	             	{
     	                  	lpVIAGraphicInfo->HQVFilter_ManualSelect = FALSE;
      	                	DBG_DD(("Option: HQV Filter Manual Select Disabled\n"));
     	            	}
       	     		}
                    else  if (strcmp(line,"MPEG_HQV_VFILTER_MODE")==0)
     	     		{
     	               	lpVIAGraphicInfo->MpegHQV_VFilterMode = atoi(value);
     	            	DBG_DD(("MPEG HQV vertical filter mode: %d\n",lpVIAGraphicInfo->MpegHQV_VFilterMode));
       	     		}
                    else  if (strcmp(line,"MPEG_HQV_HFILTER_MODE")==0)
     	     		{
     	               	lpVIAGraphicInfo->MpegHQV_HFilterMode = atoi(value);
     	            	DBG_DD(("MPEG HQV horizontal filter mode: %d\n",lpVIAGraphicInfo->MpegHQV_HFilterMode));
       	     		}
                    else  if (strcmp(line,"CAP_HQV_VFILTER_MODE")==0)
     	     		{
     	               	lpVIAGraphicInfo->CapHQV_VFilterMode = atoi(value);
     	            	DBG_DD(("Capture HQV vertical filter mode: %d\n",lpVIAGraphicInfo->CapHQV_VFilterMode));
       	     		}
                    else  if (strcmp(line,"CAP_HQV_HFILTER_MODE")==0)
     	     		{
     	               	lpVIAGraphicInfo->CapHQV_HFilterMode = atoi(value);
     	            	DBG_DD(("Capture HQV Hertical filter mode: %d\n",lpVIAGraphicInfo->CapHQV_HFilterMode));
       	     		}
                    else  if (strcmp(line,"SETMPEG_FBNUMBER")==0)
     	     		{
     	               	if (atoi(value)==TRUE)
     	               	{
     	                   	lpVIAGraphicInfo->SetMpeg_FBNumber = TRUE;
   	                   		DBG_DD(("Option: Set MPEG decode frame buffer number Enable\n"));
     	              	}
     	              	else
     	              	{	
     	                   	lpVIAGraphicInfo->SetMpeg_FBNumber = FALSE;
      	                	DBG_DD(("Option: Set MPEG decode frame buffer number Disable\n"));
     	            	}
       	     		}
                    else  if (strcmp(line,"CAP1_USEIRQ")==0)
     	     		{
     	              	if (atoi(value)==TRUE)
     	              	{
     	                   	lpVIAGraphicInfo->Cap1_UseIRQ = TRUE;
   	               	   		DBG_DD(("Option: Capture 1 use S/W flip\n"));
	     	        	}
     	              	else
     	              	{
     	                   	lpVIAGraphicInfo->Cap1_UseIRQ = FALSE;
      	                	DBG_DD(("Option: Capture 1 use H/W auto flip\n"));
     	            	}
       	     		}
                    else  if (strcmp(line,"CAP0_SWITCHTOV3")==0)
     	     		{
     	              	if (atoi(value)==TRUE)
     	              	{
     	                   	lpVIAGraphicInfo->Cap0_SwitchToV3 = TRUE;
   	                   		DBG_DD(("Option: Capture 0 use V3 engine : for SAMSUNG\n"));
     	              	}
     	              	else
     	              	{
     	                   	lpVIAGraphicInfo->Cap0_SwitchToV3 = FALSE;
      	                	DBG_DD(("Option: Capture 0 use V1 engine : Default path\n"));
     	            	}
       	     		}
                    else  if (strcmp(line,"HQV_MANUALSWITCH")==0)
     	     		{
     	              	if (atoi(value)==TRUE)
     	              	{
     	                   	lpVIAGraphicInfo->HQV_ManualSwitch = TRUE;
   	               	   		DBG_DD(("Option: HQV Manual Switch Enabled\n"));
  	              		}
     	              	else
     	              	{
     	    	   			lpVIAGraphicInfo->HQV_ManualSwitch = FALSE;
      	    	   			DBG_DD(("Option: HQV Manual Switch Disabled\n"));
     	            	}
       	     		}
                    else  if (strcmp(line,"CapInterVersion")==0)
     	     		{
        	          pVia->CapInfo[0].CapInterVersion = atoi(value);
     	          		pVia->CapInfo[1].CapInterVersion = pVia->CapInfo[0].CapInterVersion;
     	          		DBG_DD(("Capture internal version is %d\n", pVia->CapInfo[0].CapInterVersion));
       	     		}
                    else  if (strcmp(line,"Cap0NoAutoDetect")==0)
     	     		{
     	             	if (atoi(value)==TRUE)
     	             	{
     	                   	pVia->CapInfo[0].NoAutoDetect = TRUE;
                            DBG_DD(("Option: Cap0 no auto detect= %x\n", pVia->CapInfo[0].NoAutoDetect));
     	             	}
     	             	else
     	             	{
     	                	pVia->CapInfo[0].NoAutoDetect = FALSE;
                            DBG_DD(("Option: Cap0 no auto detect= %x\n", pVia->CapInfo[0].NoAutoDetect));
     	            	}
       	     		}
                    else  if (strcmp(line,"Cap0DataWidth")==0)
     	     		{
     	               	pVia->CapInfo[0].DataWidth = atoi(value);
      	            	DBG_DD(("Option: Capture 0 data width is %d\n", pVia->CapInfo[0].DataWidth));
       	     		}
                    else  if (strcmp(line,"Cap0VDODecoderDev")==0)
     	     		{
 	                    	pVia->CapInfo[0].VideoDecoder = CAP_NONE;
   	              	    	DBG_DD(("No Cap0 VideoDecoder\n"));
       	     		}
                    else  if (strcmp(line,"Cap0VDODecoderI2C")==0)
     	     		{
     	            	pVia->CapInfo[0].Vdec_Slave_Write = atoi(value);
     	            	DBG_DD(("Cap0 VideoDecoder slave address: %d\n", pVia->CapInfo[0].Vdec_Slave_Write));
       	     		}
                    else  if (strcmp(line,"Cap0CompositeMode")==0)
     	     		{
     	            	pVia->CapInfo[0].CompositeMode = atoi(value);
     	            	DBG_DD(("Cap0 VideoDecoder Composite mode: %d\n", pVia->CapInfo[0].CompositeMode));
       	     		}
                    else  if (strcmp(line,"Cap0SVideoMode")==0)
     	     		{
     	            	pVia->CapInfo[0].SVideoMode = atoi(value);
     	            	DBG_DD(("Cap0 VideoDecoder SVideo mode: %d\n", pVia->CapInfo[0].SVideoMode));
       	     		}
                    else  if (strcmp(line,"Cap0TunerMode")==0)
     	     		{
     	            	pVia->CapInfo[0].TunerMode = atoi(value);
     	            	DBG_DD(("Cap0 VideoDecoder tuner mode: %d\n", pVia->CapInfo[0].TunerMode));
       	     		}
                    else  if (strcmp(line,"Cap0Scaler")==0)
     	     		{
     	            	pVia->CapInfo[0].Scaler = atoi(value);
     	            	DBG_DD(("Cap0 VideoDecoder Scaler: %d\n", pVia->CapInfo[0].Scaler));
       	     		}
                    else  if (strcmp(line,"Cap0Brightness")==0)
     	     		{
     	            	pVia->CapInfo[0].Brightness = atoi(value);
     	            	DBG_DD(("Cap0 VideoDecoder Brightness: %d\n", pVia->CapInfo[0].Brightness));
       	     		}
                    else  if (strcmp(line,"Cap0Contrast")==0)
     	     		{
     	            	pVia->CapInfo[0].Contrast = atoi(value);
     	            	DBG_DD(("Cap0 VideoDecoder Contrast: %d\n", pVia->CapInfo[0].Contrast));
       	     		}
                    else  if (strcmp(line,"Cap0Hue")==0)
     	     		{
     	            	pVia->CapInfo[0].Hue = atoi(value);
     	            	DBG_DD(("Cap0 VideoDecoder Hue: %d\n", pVia->CapInfo[0].Hue));
					}
                    else  if (strcmp(line,"Cap0Saturation")==0)
     	     		{
     	            	pVia->CapInfo[0].Saturation = atoi(value);
     	            	DBG_DD(("Cap0 VideoDecoder Saturation: %d\n", pVia->CapInfo[0].Saturation));
       	     		}
                    else  if (strcmp(line,"Cap0Standard")==0)
     	     		{
     	            	if (atoi(value) == VIDEO_MODE_PAL)
     	            	{
     	                   	pVia->CapInfo[0].Standard = VIDEO_MODE_PAL;
   	               	   		DBG_DD(("Cap0 VideoDecoder Standard: PAL\n"));
                        }
     	            	else if (atoi(value) == VIDEO_MODE_NTSC)
     	            	{
     	                   	pVia->CapInfo[0].Standard = VIDEO_MODE_NTSC;
   	               	   		DBG_DD(("Cap0 VideoDecoder Standard: NTSC\n"));
     	            	}
     	            	else if (atoi(value) == VIDEO_MODE_SECAM)
     	            	{
      	                   	pVia->CapInfo[0].Standard = VIDEO_MODE_SECAM;
   	                   		DBG_DD(("Cap0 VideoDecoder Standard: SECAM\n"));
     	            	}
       	     		}
                    else  if (strcmp(line,"Cap0TunerDev")==0)
     	     		{
     	              	if (atoi(value) == FI1236MK2)
     	              	{
     	                   	pVia->CapInfo[0].Tuner = FI1236MK2;
   	               	  		DBG_DD(("Cap0 Tuner: FI1236MK2\n"));
     	              	}
       	     		}
                    else  if (strcmp(line,"Cap0TunerI2C")==0)
     	     		{
     	               	pVia->CapInfo[0].Tuner_Slave_Write = atoi(value);
     	            	DBG_DD(("Cap0 tuner slave address: %d\n", pVia->CapInfo[0].Tuner_Slave_Write));
       	     		}
                    else  if (strcmp(line,"Cap0SourceWidth")==0)
     	     		{
     	               	pVia->CapInfo[0].SourceWidth = atoi(value);
     	            	DBG_DD(("Cap0 Source Width: %d\n", pVia->CapInfo[0].SourceWidth));
       	     		}
                    else  if (strcmp(line,"Cap0SourceHeight")==0)
     	     		{
     	               	pVia->CapInfo[0].SourceHeight = atoi(value);
     	            	DBG_DD(("Cap0 Source Height: %d\n", pVia->CapInfo[0].SourceHeight));
       	     		}
                    else  if (strcmp(line,"Cap1NoAutoDetect")==0)
     	     		{
     	               if (atoi(value)==TRUE)
     	               {
     	                   pVia->CapInfo[1].NoAutoDetect = TRUE;
   	               	   DBG_DD(("Option: Cap1 auto detect Disabled\n"));
     	               }
     	               else
     	               {
     	                   pVia->CapInfo[1].NoAutoDetect = FALSE;
      	             	   DBG_DD(("Option: Cap1 auto detect Enabled\n"));
     	               }
       	     		}
                    else  if (strcmp(line,"Cap1DataWidth")==0)
     				{
    	 	            pVia->CapInfo[1].DataWidth = atoi(value);
	      	            DBG_DD(("Option: Capture 1 data width is %d\n", pVia->CapInfo[1].DataWidth));
       	     		}
                    else  if (strcmp(line,"Cap1VDODecoderDev")==0)
     	     		{
 	                 		pVia->CapInfo[1].VideoDecoder = CAP_NONE;
   	               	  		DBG_DD(("No Cap1 VideoDecoder\n"));
       	     		}
                    else  if (strcmp(line,"Cap1VDODecoderI2C")==0)
     	     		{
     	            	pVia->CapInfo[1].Vdec_Slave_Write = atoi(value);
     	            	DBG_DD(("Cap1 VideoDecoder slave address: %d\n", pVia->CapInfo[1].Vdec_Slave_Write));
       	     		}
                    else  if (strcmp(line,"Cap1CompositeMode")==0)
     	     		{
     	            	pVia->CapInfo[1].CompositeMode = atoi(value);
     	            	DBG_DD(("Cap1 VideoDecoder Composite mode: %d\n", pVia->CapInfo[1].CompositeMode));
       	     		}
                    else  if (strcmp(line,"Cap1SVideoMode")==0)
     	     		{
     	            	pVia->CapInfo[1].SVideoMode = atoi(value);
     	            	DBG_DD(("Cap1 VideoDecoder SVideo mode: %d\n", pVia->CapInfo[1].SVideoMode));
       	     		}
                    else  if (strcmp(line,"Cap1TunerMode")==0)
     	     		{
     	            	pVia->CapInfo[1].TunerMode = atoi(value);	
     	            	DBG_DD(("Cap1 VideoDecoder tuner mode: %d\n", pVia->CapInfo[1].TunerMode));
       	     		}
                    else  if (strcmp(line,"Cap1Scaler")==0)
     	     		{
     	            	pVia->CapInfo[1].Scaler = atoi(value);
     	            	DBG_DD(("Cap1 VideoDecoder Scaler: %d\n", pVia->CapInfo[1].Scaler));
       	     		}
                    else  if (strcmp(line,"Cap1Brightness")==0)
     	     		{
     	            	pVia->CapInfo[1].Brightness = atoi(value);
     	            	DBG_DD(("Cap1 VideoDecoder Brightness: %d\n", pVia->CapInfo[1].Brightness));
       	     		}
                    else  if (strcmp(line,"Cap1Contrast")==0)
     	     		{
     	            	pVia->CapInfo[1].Contrast = atoi(value);
     	            	DBG_DD(("Cap1 VideoDecoder Contrast: %d\n", pVia->CapInfo[1].Contrast));
       	     		}
                    else  if (strcmp(line,"Cap1Hue")==0)
     	     		{
     	            	pVia->CapInfo[1].Hue = atoi(value);
     	            	DBG_DD(("Cap1 VideoDecoder Hue: %d\n", pVia->CapInfo[1].Hue));
       	     		}
                    else  if (strcmp(line,"Cap1Saturation")==0)
     	     		{
     	            	pVia->CapInfo[1].Saturation = atoi(value);
     	            	DBG_DD(("Cap1 VideoDecoder Saturation: %d\n", pVia->CapInfo[1].Saturation));
					}
                    else  if (strcmp(line,"Cap1Standard")==0)
     	     		{
     	            	if (atoi(value) == VIDEO_MODE_PAL)
     	            	{
     	                	pVia->CapInfo[1].Standard = VIDEO_MODE_PAL;
   	               	  		DBG_DD(("Cap1 VideoDecoder Standard: PAL\n"));
     	            	}
     	            	else if (atoi(value) == VIDEO_MODE_NTSC)
     	            	{
     	                	pVia->CapInfo[1].Standard = VIDEO_MODE_NTSC;
   	               	  		DBG_DD(("Cap1 VideoDecoder Standard: NTSC\n"));
     	            	}
     	            	else if (atoi(value) == VIDEO_MODE_SECAM)
     	            	{
      	                  	pVia->CapInfo[1].Standard = VIDEO_MODE_SECAM;
   	               	  		DBG_DD(("Cap1 VideoDecoder Standard: SECAM\n"));	
     	            	}
       	     		}
                    else  if (strcmp(line,"Cap1TunerDev")==0)
     	     		{
	     	            if (atoi(value) == FI1236MK2)
    	 	            {
     	                  pVia->CapInfo[1].Tuner = FI1236MK2;
   	    	          	  DBG_DD(("Cap1 Tuner: FI1236MK2\n"));
     	    	        }
       	     		}
                    else  if (strcmp(line,"Cap1TunerI2C")==0)
     	     		{
     	            	pVia->CapInfo[1].Tuner_Slave_Write = atoi(value);
     	            	DBG_DD(("Cap1 tuner slave address: %d\n", pVia->CapInfo[1].Tuner_Slave_Write));
       	     		}
                    else  if (strcmp(line,"Cap1SourceWidth")==0)
     	     		{
     	            	pVia->CapInfo[1].SourceWidth = atoi(value);
     	            	DBG_DD(("Cap1 Source Width: %d\n", pVia->CapInfo[1].SourceWidth));
       	     		}
                    else  if (strcmp(line,"Cap1SourceHeight")==0)
     	     		{
     	            	pVia->CapInfo[1].SourceHeight = atoi(value);
     	            	DBG_DD(("Cap1 Source Height: %d\n", pVia->CapInfo[1].SourceHeight));
       	     		}
                    else  if (strcmp(line,"NOMPEGAddOneLineOnBottom")==0)
     	     		{
     	              if (atoi(value)==TRUE)
     	              {
     	                   lpVIAGraphicInfo->NoMPEGAddOneLineOnBottom = TRUE;
   	               	   DBG_DD(("Option: No mpeg add one line on bottom\n"));
     	              }
     	              else
     	              {
     	                   lpVIAGraphicInfo->NoMPEGAddOneLineOnBottom = FALSE;
      	            	   DBG_DD(("Option: mpeg add one line on bottom\n"));
     	              }
       	     		}
                    else  if (strcmp(line,"ReDrawColorkey")==0)
     	     		{
     	              if (atoi(value)==TRUE)
     	              {
     	                   lpVIAGraphicInfo->ReDrawColorkey = TRUE;
   	               	   DBG_DD(("Option: ReDrawColorkey Enable!\n"));
     	              }
     	              else
     	              {
     	                   lpVIAGraphicInfo->ReDrawColorkey = FALSE;
      	            	   DBG_DD(("ReDrawColorkey Disabled!\n"));
     	              }
       	     		}
                    else  if (strcmp(line,"DeBlocking")==0) 
                    {
                        if (atoi(value)==TRUE)
                        {
							lpVIAGraphicInfo->DeBlockingEnable = TRUE;
                            DBG_DD(("Option: DeBlocking Enable!!\n"));                                                         
                        } 
                        else
                        {
							lpVIAGraphicInfo->DeBlockingEnable = FALSE;
                            DBG_DD(("Option: DeBlocking Disable!!\n")); 
                        }    
		      		}
                    else  if (strcmp(line,"DeBlockingMinimumWidth")==0)
     	     	    {
     	     	      		iValue= atoi(value);
     	     	      		if (iValue > 0)
	    	 			lpVIAGraphicInfo->dwDeBlockingMinimumWidth = (unsigned long)iValue;
     	     	      		else 
				    	lpVIAGraphicInfo->dwDeBlockingMinimumWidth = DEBLOCKING_MINIMUM_WIDTH_DEFAULT;
		     	       DBG_DD(("DeBlocking Minimum Width Default : %d\n", lpVIAGraphicInfo->dwDeBlockingMinimumWidth));

           	      	}
                    else  if (strcmp(line,"DeBlockingMinimumHeight")==0)
     	     	    {
     	     	      		iValue= atoi(value);
     	     	      		if (iValue > 0)
	    	 			lpVIAGraphicInfo->dwDeBlockingMinimumHeight = (unsigned long)iValue;
     	     	      		else 
				    	lpVIAGraphicInfo->dwDeBlockingMinimumHeight = DEBLOCKING_MINIMUM_HEIGHT_DEFAULT;
		     	       DBG_DD(("DeBlocking Minimum Height Default : %d\n", lpVIAGraphicInfo->dwDeBlockingMinimumHeight));
       	      		}
                    else  if (strcmp(line,"3to2PullDownDisable")==0)
     	     	      {
                        if (atoi(value)==TRUE)
                        {
				lpVIAGraphicInfo->dw3to2PullDownDisable = TRUE;
                            DBG_DD(("Option: 3:2PullDown Disable!!\n"));
                        } 
                        else
                        {
				lpVIAGraphicInfo->dw3to2PullDownDisable = FALSE;
                            DBG_DD(("Option: 3:2PullDown Enable!!\n")); 
                        }    
                    }
                    else  if (strcmp(line,"PatchMPEGDecodeSliceErrorBitstreamDisable")==0)
     	     	      {
                        if (atoi(value)==TRUE)
                        {
				lpVIAGraphicInfo->bPatchMPEGDecodeSliceErrorBitstreamDisable = TRUE;
/*                            DBG_DD(("Option: Patch MPEG Decode Slice Error Bistream Disable!!\n"));
*/
                        } 
                        else
                        {
				lpVIAGraphicInfo->bPatchMPEGDecodeSliceErrorBitstreamDisable = FALSE;
/*                            DBG_DD(("Option: Patch MPEG Decode Slice Error Bistream Enable\n")); 
*/
                        }    
                    }
                    else  if (strcmp(line,"Use2DBitBltSignature")==0)                      
                    {
                        if (atoi(value)==TRUE)
                        {
                            lpVIAGraphicInfo->bUse2DBitBltSignature = TRUE;                                                                         
                            DBG_DD(("Option: Use 2D BitBlt method to write event-tag into VQ and make sure MPEG decode END Enable\n"));                                                     
                        } 
                        else
                        {
                            lpVIAGraphicInfo->bUse2DBitBltSignature = FALSE;
                            DBG_DD(("Option: Use 2D BitBlt method to write event-tag into VQ and make sure MPEG decode END Disable\n"));                                                     
                        }    
                    }
                    else  if (strcmp(line,"VideoRotation")==0)                      
                    {
                        lpVIAGraphicInfo->rotate= atoi(value);
                        DBG_DD(("Option:rotate: %d\n", lpVIAGraphicInfo->rotate));                         
                    }					
                 }
            }

        ret = TRUE;
        fclose(VIAVIDEORC);
        }
    }
    return ret;

}

#if 0
static int
GetIGAPath(VIABIOSInfoPtr pBIOSInfo)
{
    int result;

    switch (pBIOSInfo->ActiveDevice) {
	case VIA_DEVICE_LCD:
	    result = pBIOSInfo->LVDSSettingInfo.IGAPath;
	    break;
	case VIA_DEVICE_CRT1:
	case VIA_DEVICE_CRT2:
	    result = pBIOSInfo->CRTSettingInfo.IGAPath;
	    break;
	case VIA_DEVICE_DFP:
	    result = pBIOSInfo->TMDSSettingInfo.IGAPath;
	    break;
	case VIA_DEVICE_TV:
	    result = pBIOSInfo->TVSettingInfo.IGAPath;
	    break;
	default:
	    result = IGA1;
	    break;
    }

    return (result);
}
#endif

static void
FillGraphicInfo(ScrnInfoPtr pScrn)
{
    VIAPtr pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr pBIOSInfo = pVia->pBIOSInfo;
    PVIDDATA pVidData = pVia->pVidData;
    LPVIAGRAPHICINFO lpVIAGraphicInfo = &pVidData->VIAGraphicInfo;
    FILE*    pFilePointer;        /* for SAMM mode passing screen info */
    char     szBuffer[256];
    char*    pBuffer;
    int      icount = 0;
    int      i;
    /*[2004.10.27 - seanzhang]Added for SAMM Mode*/
    unsigned char* FBBase;
    ScrnInfoPtr pScrn0 = xf86Screens[0];
    VIAPtr pVia0 = VIAPTR(pScrn0);
    DBG_DD(("  FillGraphicInfo!!\n"));

    lpVIAGraphicInfo->scrnIndex = pScrn->scrnIndex;
    lpVIAGraphicInfo->Screen[pScrn->scrnIndex].TotalVRAM = pVia->videoRambytes;
    lpVIAGraphicInfo->Screen[pScrn->scrnIndex].VideoHeapBase = (unsigned long) pVia->FBFreeStart;
    lpVIAGraphicInfo->Screen[pScrn->scrnIndex].VideoHeapEnd = (unsigned long) (pVia->FBFreeEnd - 1);
    lpVIAGraphicInfo->Screen[pScrn->scrnIndex].dwWidth = pBIOSInfo->CrtcHDisplay;
    lpVIAGraphicInfo->Screen[pScrn->scrnIndex].dwHeight = pBIOSInfo->CrtcVDisplay;
    lpVIAGraphicInfo->Screen[pScrn->scrnIndex].dwSaveWidth = pBIOSInfo->SaveHDisplay;
    lpVIAGraphicInfo->Screen[pScrn->scrnIndex].dwSaveHeight = pBIOSInfo->SaveVDisplay;
    lpVIAGraphicInfo->Screen[pScrn->scrnIndex].dwBPP = pScrn->bitsPerPixel;
    lpVIAGraphicInfo->Screen[pScrn->scrnIndex].dwPitch = (((pScrn->virtualX) + 15) & ~15) * (pScrn->bitsPerPixel) / 8;
    lpVIAGraphicInfo->Screen[pScrn->scrnIndex].dwRefreshRate = (unsigned long)pBIOSInfo->FoundRefresh;
    lpVIAGraphicInfo->Screen[pScrn->scrnIndex].dwDVIOn = pBIOSInfo->DVIAttach;
    lpVIAGraphicInfo->Screen[pScrn->scrnIndex].dwLCDOn = pBIOSInfo->LCDAttach;
    /*if set viatual screen ,virtual width is virtualX ,height is virtualY ,enable  panning mode */
    if (pBIOSInfo->SaveHDisplay > pBIOSInfo->CrtcHDisplay ||pScrn->virtualX> 0 || pScrn->virtualY >0)
    {
        lpVIAGraphicInfo->Screen[pScrn->scrnIndex].dwPanning = 1;
    }
    else
    {
        lpVIAGraphicInfo->Screen[pScrn->scrnIndex].dwPanning = 0;
    }
    lpVIAGraphicInfo->Screen[pScrn->scrnIndex].dwExpand = pBIOSInfo->scaleY;
    lpVIAGraphicInfo->Screen[pScrn->scrnIndex].dwPanelWidth = pBIOSInfo->LVDSSettingInfo.PanelSizeX;
    lpVIAGraphicInfo->Screen[pScrn->scrnIndex].dwPanelHeight = pBIOSInfo->LVDSSettingInfo.PanelSizeY;
    lpVIAGraphicInfo->Screen[pScrn->scrnIndex].dwActiveDevice = pBIOSInfo->ActiveDevice;

    lpVIAGraphicInfo->dwRevisionID = pVia->ChipRev;
    lpVIAGraphicInfo->dwDeviceID = pVia->ChipId;
    /* VansonWu [2004.4.21] Simplify Device ID + Revision ID combination */
    lpVIAGraphicInfo->dwProjectIDRevision = pBIOSInfo->dwProjectIDRevision;
    lpVIAGraphicInfo->SAMM = pBIOSInfo->SAMM;
    lpVIAGraphicInfo->dwActiveDevice = (pBIOSInfo->SAMM ? pVia->ActiveDevice : pBIOSInfo->ActiveDevice);
    lpVIAGraphicInfo->ScreenPhysAddr = pVia->FrameBufferBase;
    lpVIAGraphicInfo->TVEncoder = pBIOSInfo->TVSettingInfo.TVEncoder;
    lpVIAGraphicInfo->HasTwoTVEncoder = pBIOSInfo->HasTwoTVEncoder;
    lpVIAGraphicInfo->TVSystem = pBIOSInfo->TVSettingInfo.TVType;
    lpVIAGraphicInfo->TVOutput = pBIOSInfo->TVSettingInfo.TVOutput;
    lpVIAGraphicInfo->TV1I2CAddr = pBIOSInfo->TVI2CAdd;
    lpVIAGraphicInfo->TV2I2CAddr = pBIOSInfo->TVI2CAdd_2;
    lpVIAGraphicInfo->TVDIPort = pBIOSInfo->TVDIPort;
    lpVIAGraphicInfo->VideoUseSecondDisplay = pBIOSInfo->UseIGA2;
    lpVIAGraphicInfo->dwXServerEnabled = TRUE;
    xf86DrvMsg(pScrn->scrnIndex, X_INFO, "lobo lpVIAGraphicInfo->dwXServerEnabled is %ld\n",lpVIAGraphicInfo->dwXServerEnabled);
    lpVIAGraphicInfo->TMDSChipID = pBIOSInfo->TMDSSettingInfo.ChipID;
    
    pVidData->Vhwinfo.pjVideo       = pVia->MapBase; /* save register MMIO pointer to VidData */

    /* RainLin [2007.03.15]: Fix the problem: To enable MAMM (NVIDIA GeForce 4 + VIA) with Individual */
    /* Desktops (Clone), when VIA's graphics is used as screen 1, it will fail to play video.*/
    if (pVia->IsSecondary)
    {
        /*[2004.10.27 - seanzhang ] in samm mode, we should keep the screen[0]'s FBBase*/
        /* Fix bug: CR56427 - crt+lcd samm: play xine-0.9.23 (xv) caused the video window is no image*/
        FBBase = pVia0->FBBase;
    }
    else
    {
        FBBase = pVia->FBBase;
    }

    pVidData->Vhwinfo.pjScreen       = FBBase; /* save linear screen address to VidData */
   /* [2005.06.24 seanzhang ] */
 
    pVidData->Vhwinfo.xpjVideo= pVia->MapBase; 
    pVidData->Vhwinfo.xpjScreen       = FBBase; 

    lpVIAGraphicInfo->IGAPanningStatus=(unsigned long)(((pBIOSInfo->IGA1SettingInfo.IsPanning)&0x01) | (((pBIOSInfo->IGA2SettingInfo.IsPanning)&0x01)<<1));
    if(pScrn->scrnIndex==SCREEN_INDEX_0){
        /* Graf [2006-09-25]: Note Screen0 is first called, then screen1 */
        for(i=0;i<3;i++) {
            lpVIAGraphicInfo->Screen_IGA_Map[i] = -1;
            lpVIAGraphicInfo->IGA_Video_Map[i] = -1;
        }
    }
    if(lpVIAGraphicInfo->SAMM) {
        lpVIAGraphicInfo->duoview=0;
        if(pScrn->scrnIndex==SCREEN_INDEX_1) {
            if(pBIOSInfo->IGA1SettingInfo.IsActive) {
                lpVIAGraphicInfo->Screen_IGA_Map[SCREEN_INDEX_1] = IGA1;
                lpVIAGraphicInfo->Screen[SCREEN_INDEX_1].dwWidth=pBIOSInfo->IGA1SettingInfo.HActive;
                lpVIAGraphicInfo->Screen[SCREEN_INDEX_1].dwHeight=pBIOSInfo->IGA1SettingInfo.VActive;
                lpVIAGraphicInfo->Screen[SCREEN_INDEX_1].dwPanning=pBIOSInfo->IGA1SettingInfo.IsPanning;
            }
            if(pBIOSInfo->IGA2SettingInfo.IsActive) {
                lpVIAGraphicInfo->Screen_IGA_Map[SCREEN_INDEX_1] = IGA2;   
                lpVIAGraphicInfo->Screen[SCREEN_INDEX_1].dwWidth=pBIOSInfo->IGA2SettingInfo.HActive;
                lpVIAGraphicInfo->Screen[SCREEN_INDEX_1].dwHeight=pBIOSInfo->IGA2SettingInfo.VActive;
                lpVIAGraphicInfo->Screen[SCREEN_INDEX_1].dwPanning=pBIOSInfo->IGA2SettingInfo.IsPanning;
            }
        }
        else if(pScrn->scrnIndex==SCREEN_INDEX_0) {
            if(pBIOSInfo->IGA1SettingInfo.IsActive) {
                lpVIAGraphicInfo->Screen_IGA_Map[SCREEN_INDEX_0] = IGA1;
                lpVIAGraphicInfo->Screen[SCREEN_INDEX_0].dwWidth=pBIOSInfo->IGA1SettingInfo.HActive;
                lpVIAGraphicInfo->Screen[SCREEN_INDEX_0].dwHeight=pBIOSInfo->IGA1SettingInfo.VActive;
                lpVIAGraphicInfo->Screen[SCREEN_INDEX_0].dwPanning=pBIOSInfo->IGA1SettingInfo.IsPanning;
            }
            if(pBIOSInfo->IGA2SettingInfo.IsActive) {
                lpVIAGraphicInfo->Screen_IGA_Map[SCREEN_INDEX_0] = IGA2;   
                lpVIAGraphicInfo->Screen[SCREEN_INDEX_0].dwWidth=pBIOSInfo->IGA2SettingInfo.HActive;
                lpVIAGraphicInfo->Screen[SCREEN_INDEX_0].dwHeight=pBIOSInfo->IGA2SettingInfo.VActive;
                lpVIAGraphicInfo->Screen[SCREEN_INDEX_0].dwPanning=pBIOSInfo->IGA2SettingInfo.IsPanning;
            }
        }
    } 
    else if(lpVIAGraphicInfo->duoview&DUOVIEW_ENABLED)
    {
      /* If DuoView, Screen_IGA_Map[] are fixed to {IGA1,IGA2} */
      lpVIAGraphicInfo->Screen_IGA_Map[ SCREEN_INDEX_0 ]=IGA1; 
      lpVIAGraphicInfo->Screen_IGA_Map[ SCREEN_INDEX_1 ]=IGA2; 
      /* Screen's size on IGA1,2 may different. Both Screen[0:1] are used */
      lpVIAGraphicInfo->Screen[SCREEN_INDEX_1]=lpVIAGraphicInfo->Screen[SCREEN_INDEX_0]; 
      lpVIAGraphicInfo->Screen[SCREEN_INDEX_0].dwWidth=pBIOSInfo->IGA1SettingInfo.HActive;
      lpVIAGraphicInfo->Screen[SCREEN_INDEX_0].dwHeight=pBIOSInfo->IGA1SettingInfo.VActive;
      lpVIAGraphicInfo->Screen[SCREEN_INDEX_0].dwPanning=pBIOSInfo->IGA1SettingInfo.IsPanning;
      lpVIAGraphicInfo->Screen[SCREEN_INDEX_1].dwWidth=pBIOSInfo->IGA2SettingInfo.HActive;
      lpVIAGraphicInfo->Screen[SCREEN_INDEX_1].dwHeight=pBIOSInfo->IGA2SettingInfo.VActive;
      lpVIAGraphicInfo->Screen[SCREEN_INDEX_1].dwPanning=pBIOSInfo->IGA2SettingInfo.IsPanning;
    }
    else 
    { /* single or simultanious */
      if(pBIOSInfo->IGA1SettingInfo.IsActive) 
      {
        lpVIAGraphicInfo->Screen_IGA_Map[ SCREEN_INDEX_0 ]=IGA1;
        lpVIAGraphicInfo->Screen_IGA_Map[ SCREEN_INDEX_1 ]=-1;
        lpVIAGraphicInfo->Screen[SCREEN_INDEX_0].dwWidth=pBIOSInfo->IGA1SettingInfo.HActive;
        lpVIAGraphicInfo->Screen[SCREEN_INDEX_0].dwHeight=pBIOSInfo->IGA1SettingInfo.VActive;
        lpVIAGraphicInfo->Screen[SCREEN_INDEX_0].dwPanning=pBIOSInfo->IGA1SettingInfo.IsPanning;
      } 
      else if(pBIOSInfo->IGA2SettingInfo.IsActive)
      {
        lpVIAGraphicInfo->Screen_IGA_Map[ SCREEN_INDEX_0 ]=IGA2;
        lpVIAGraphicInfo->Screen_IGA_Map[ SCREEN_INDEX_1 ]=-1;
        lpVIAGraphicInfo->Screen[SCREEN_INDEX_0].dwWidth=pBIOSInfo->IGA2SettingInfo.HActive;
        lpVIAGraphicInfo->Screen[SCREEN_INDEX_0].dwHeight=pBIOSInfo->IGA2SettingInfo.VActive;
        lpVIAGraphicInfo->Screen[SCREEN_INDEX_0].dwPanning=pBIOSInfo->IGA2SettingInfo.IsPanning;
      }  
    }
    
    pVidData->gfx3DScal_info.gfx3DScalingEnable = FALSE;    
    if (pBIOSInfo->Is3DScaling)
    {
    
    xf86DrvMsg(pScrn->scrnIndex, X_INFO, "FillGraphicInfo:pBIOSInfo->Is3DScaling is TRUE\n");       
        pVidData->gfx3DScal_info.gfx3DScalingEnable = TRUE;
        pVidData->gfx3DScal_info.OrigHActive = pVia->DISP3DSCALCTRLInfo.OrigHActive;
        pVidData->gfx3DScal_info.OrigVActive = pVia->DISP3DSCALCTRLInfo.OrigVActive;
        pVidData->gfx3DScal_info.RealHActive = pVia->DISP3DSCALCTRLInfo.RealHActive;
        pVidData->gfx3DScal_info.RealVActive = pVia->DISP3DSCALCTRLInfo.RealVActive;
        pVidData->gfx3DScal_info.DISP3DScalIGAPath = pVia->DISP3DSCALCTRLInfo.DISP3DScalIGAPath;
  xf86DrvMsg(pScrn->scrnIndex, X_INFO, "DISP3DScalIGAPath is %d\n",pVia->DISP3DSCALCTRLInfo.DISP3DScalIGAPath);         
        pVidData->gfx3DScal_info.XSIZEOffset = pBIOSInfo->TVSettingInfo.TV3DScalInfo.XSIZEOffset; 
        pVidData->gfx3DScal_info.YSIZEOffset = pBIOSInfo->TVSettingInfo.TV3DScalInfo.YSIZEOffset;
        pVidData->gfx3DScal_info.XPOSITIONOffset = pBIOSInfo->TVSettingInfo.TV3DScalInfo.XPOSITIONOffset;
        pVidData->gfx3DScal_info.YPOSITIONOffset = pBIOSInfo->TVSettingInfo.TV3DScalInfo.YPOSITIONOffset;
        switch (pVidData->gfx3DScal_info.DISP3DScalIGAPath)
        {
            case IGA1:
                if (IGA1 == lpVIAGraphicInfo->Screen_IGA_Map[ SCREEN_INDEX_0 ])
                {
                    lpVIAGraphicInfo->Screen[SCREEN_INDEX_0].dwWidth=pBIOSInfo->CrtcHDisplay;
                    lpVIAGraphicInfo->Screen[SCREEN_INDEX_0].dwHeight=pBIOSInfo->CrtcVDisplay;
                }
                else if (IGA1 == lpVIAGraphicInfo->Screen_IGA_Map[ SCREEN_INDEX_1 ])
                {
                    lpVIAGraphicInfo->Screen[SCREEN_INDEX_1].dwWidth=pBIOSInfo->CrtcHDisplay;
                    lpVIAGraphicInfo->Screen[SCREEN_INDEX_1].dwHeight=pBIOSInfo->CrtcVDisplay;                
                }
                break;
            case IGA2:
                if (IGA2 == lpVIAGraphicInfo->Screen_IGA_Map[ SCREEN_INDEX_0 ])
                {
                    lpVIAGraphicInfo->Screen[SCREEN_INDEX_0].dwWidth=pBIOSInfo->CrtcHDisplay;
                    lpVIAGraphicInfo->Screen[SCREEN_INDEX_0].dwHeight=pBIOSInfo->CrtcVDisplay;                
                }
                else if (IGA2 == lpVIAGraphicInfo->Screen_IGA_Map[ SCREEN_INDEX_1 ])
                {
                    lpVIAGraphicInfo->Screen[SCREEN_INDEX_1].dwWidth=pBIOSInfo->CrtcHDisplay;
                    lpVIAGraphicInfo->Screen[SCREEN_INDEX_1].dwHeight=pBIOSInfo->CrtcVDisplay;                
                }
                break;
             default:
                break;

        }

    }
    else
    {
        pVidData->gfx3DScal_info.gfx3DScalingEnable = FALSE;    
    }

    strcpy(pVidData->video0Path, pVia->video0Path);
    strcpy(pVidData->video1Path, pVia->video1Path);
    strcpy(pVidData->video2Path, pVia->video2Path);	

    if((pFilePointer =fopen("/etc/X11/XF86Config-4", "r")) == NULL)
    {
        if((pFilePointer =fopen("/etc/X11/XF86Config", "r")) == NULL)
        {
            if((pFilePointer =fopen("/etc/X11/xorg.conf", "r")) == NULL)
            {
                DBG_DD(("Error: Can NOT open X config file !!!!!!!!!!!!\n"));
            }
        }
    }

    if( pFilePointer != NULL )
    {
        while( !feof(pFilePointer) )
        {
            fgets(szBuffer, sizeof(szBuffer), pFilePointer);
            pBuffer = szBuffer;
            for ( icount=0 ; icount<256 ; icount++ )
            {
                if ( strncmp(pBuffer, "RightOf", 7) == 0 )
                {
                    DBG_DD(("Screen 1 is Right side !!!!!!!!!!!!\n"));
                    break;
                }
                if ( strncmp(pBuffer, "LeftOf", 6) == 0 )
                {
                    lpVIAGraphicInfo->Screen1IsLeft = 1;
                    DBG_DD(("Screen 1 is Left side !!!!!!!!!!!!\n"));
                    break;
                }
                if ( strncmp(pBuffer, "Below", 5) == 0 )
                {
                    DBG_DD(("Screen 1 is Below !!!!!!!!!!!!\n"));
                    break;
                }
                if ( strncmp(pBuffer, "Above", 5) == 0 )
                {
                    lpVIAGraphicInfo->Screen1IsAbove = 1;
                    DBG_DD(("Screen 1 is Above !!!!!!!!!!!!\n"));
                    break;
                }
                pBuffer++;
            }
        }

        /* if TV only and TV ecoder support YUV ByPass enable flag */  
        /* TV YUV By Pass only can be used when TV is in IGA1. */
        if ((lpVIAGraphicInfo->Screen[pScrn->scrnIndex].dwActiveDevice == VIA_DEVICE_TV) &&
            (pBIOSInfo->TVSettingInfo.IGAPath == IGA1) && (!pBIOSInfo->SAMM))   
        {
            switch (lpVIAGraphicInfo->TVEncoder)
            {
          
                case VIA_VT1625:
                    lpVIAGraphicInfo->bSupportTVYUVByPass = TRUE;
                    lpVIAGraphicInfo->bTVYUVByPassFromADD = TRUE;
                    break;

                default:
                    lpVIAGraphicInfo->bSupportTVYUVByPass = FALSE;
                    lpVIAGraphicInfo->bTVYUVByPassFromADD = FALSE;
                    break;
            }
        }
        else
        {
            lpVIAGraphicInfo->bSupportTVYUVByPass = FALSE;
            lpVIAGraphicInfo->bTVYUVByPassFromADD = FALSE;
        }

        fclose( pFilePointer );
    }

    /* Added to pass DRM info to V4L */
#ifdef XF86DRI
    lpVIAGraphicInfo->DRMEnabled = pVia->directRenderingEnabled;
#else
    lpVIAGraphicInfo->DRMEnabled = 0;
#endif
}

static pointer VIASetup(pointer module, pointer opts, int *errmaj, int *errmin)
{
    static Bool setupDone = FALSE;

    if (!setupDone) {
        setupDone = TRUE;
        xf86AddDriver(&VIA, module, 0);
        LoaderRefSymLists(vgaHWSymbols,
#ifdef USE_FB
                          fbSymbols,
#else
                          cfbSymbols,
#endif
                          ramdacSymbols,
                          xaaSymbols,
                          shadowSymbols,
                          vbeSymbols,
                          i2cSymbols,
                          ddcSymbols,
                          /*
                          mpegSymbols,
                          */
#ifdef XF86DRI
                                  drmSymbols,
                                  driSymbols,
#endif
                          NULL);

        return (pointer) 1;
    }
    else {
        if (errmaj)
            *errmaj = LDR_ONCEONLY;

        return NULL;
    }
} /* VIASetup */

#endif /* XFree86LOADER */


static int
WaitIdleCLE266(VIAPtr pVia)
{
    int loop = 0;

    mem_barrier();

    while (!(VIAGETREG(VIA_REG_STATUS) & VIA_VR_QUEUE_BUSY) && (loop++ < MAXLOOP))
        ;

    while ((VIAGETREG(VIA_REG_STATUS) &
          (VIA_CMD_RGTR_BUSY | VIA_2D_ENG_BUSY | VIA_3D_ENG_BUSY)) &&
          (loop++ < MAXLOOP))
        ;

    return loop >= MAXLOOP;
}

static int
WaitIdleK8M890(VIAPtr pVia)
{
    int loop = 0;

    mem_barrier();

    while ((VIAGETREG(VIA_REG_STATUS) &
          (VIA_CMD_RGTR_BUSY | VIA_2D_ENG_BUSY|VIA_3D_ENG_BUSY)) &&
          (loop++ < MAXLOOP))
        ;

    return loop >= MAXLOOP;
}

static Bool VIAGetRec(ScrnInfoPtr pScrn)
{
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIAGetRec\n"));
    if (pScrn->driverPrivate)
        return TRUE;

    pScrn->driverPrivate = xnfcalloc(sizeof(VIARec), 1);
    ((VIARec *)(pScrn->driverPrivate))->pBIOSInfo =
        xnfcalloc(sizeof(VIABIOSInfoRec), 1);
    ((VIARec *)(pScrn->driverPrivate))->pBIOSInfo->pModeTable =
        xnfcalloc(sizeof(VIAModeTableRec), 1);
    /* Marked by STChen[2006.9.4]
    ((VIARec *)(pScrn->driverPrivate))->pBIOSInfo->pTVInfo =
        xnfcalloc(sizeof(TVInfoRec), 1);
    */
    ((VIARec *)(pScrn->driverPrivate))->pBIOSVer =
        xnfcalloc(sizeof(VGABIOSVER),1);

    /* initial value in VIARec */
    ((VIARec *)(pScrn->driverPrivate))->SavedReg.mode = 0xFF;
    ((VIARec *)(pScrn->driverPrivate))->ModeReg.mode = 0xFF;

    ((VIARec *)(pScrn->driverPrivate))->pBIOSInfo->FirstInit = TRUE;
    ((VIARec *)(pScrn->driverPrivate))->CursorImage = NULL;

    return TRUE;

} /* VIAGetRec */


static void VIAFreeRec(ScrnInfoPtr pScrn)
{
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIAFreeRec\n"));
    if (!pScrn->driverPrivate)
        return;

    xfree(((VIARec *)(pScrn->driverPrivate))->pBIOSVer);
    xfree(((VIARec *)(pScrn->driverPrivate))->pBIOSInfo->pModeTable);
    /* Marked by STChen[2006.9.4]
    xfree(((VIARec *)(pScrn->driverPrivate))->pBIOSInfo->pTVInfo);
    */
    xfree(((VIARec *)(pScrn->driverPrivate))->pBIOSInfo);
    xfree(pScrn->driverPrivate);
    pScrn->driverPrivate = NULL;

    VIAUnmapMem(pScrn);

} /* VIAFreeRec */


static const OptionInfoRec * VIAAvailableOptions(int chipid, int busid)
{

    return VIAOptions;

} /* VIAAvailableOptions */


static void VIAIdentify(int flags)
{
    xf86PrintChipsets(DRIVER_NAME,
                      "driver for VIA chipsets",
                      VIAChipsets);
} /* VIAIdentify */


static Bool VIAProbe(DriverPtr drv, int flags)
{
    GDevPtr *devSections;
    int     *usedChips;
    int     numDevSections;
    int     numUsed;
    Bool    foundScreen = FALSE;
    int     i;

    /* sanity checks */
    if ((numDevSections = xf86MatchDevice("viaarena", &devSections)) <= 0)
        return FALSE;

    if (xf86GetPciVideoInfo() == NULL)
        return FALSE;

    numUsed = xf86MatchPciInstances(DRIVER_NAME,
                                    PCI_VIA_VENDOR_ID,
                                    VIAChipsets,
                                    VIAPciChipsets,
                                    devSections,
                                    numDevSections,
                                    drv,
                                    &usedChips);
    xfree(devSections);

    if (numUsed <= 0)
        return FALSE;

    if (flags & PROBE_DETECT) {
        foundScreen = TRUE;
    }
    else {
        for (i = 0; i < numUsed; i++) {
            ScrnInfoPtr pScrn = xf86AllocateScreen(drv, 0);
            EntityInfoPtr pEnt;
            static int instance = 0;
            DevUnion* pPriv;

            if ((pScrn = xf86ConfigPciEntity(pScrn, 0, usedChips[i],
                 VIAPciChipsets, 0, 0, 0, 0, 0)))
            {
                pScrn->driverVersion = VIA_VERSION;
                pScrn->driverName = DRIVER_NAME;
                pScrn->name = "VIA";
                pScrn->Probe = VIAProbe;
                pScrn->PreInit = VIAPreInit;
                pScrn->ScreenInit = VIAScreenInit;
                pScrn->SwitchMode = VIASwitchMode;
                pScrn->AdjustFrame = VIAAdjustFrame;
                pScrn->EnterVT = VIAEnterVT;
                pScrn->LeaveVT = VIALeaveVT;
                pScrn->FreeScreen = VIAFreeScreen;
                pScrn->ValidMode = VIAValidMode;
                foundScreen = TRUE;
            }
            /*
            xf86ConfigActivePciEntity(pScrn,
                                      usedChips[i],
                                      VIAPciChipsets,
                                      NULL,
                                      NULL,
                                      NULL,
                                      NULL,
                                      NULL);
            */

            pEnt = xf86GetEntityInfo(usedChips[i]);

                xf86SetEntitySharable(usedChips[i]);
                xf86SetEntityInstanceForScreen(pScrn,
                    pScrn->entityList[0], instance);

                if(gVIAEntityIndex < 0)
                {
                    gVIAEntityIndex = xf86AllocateEntityPrivateIndex();
                    pPriv = xf86GetEntityPrivate(pScrn->entityList[0],
                            gVIAEntityIndex);

                    if (!pPriv->ptr)
                    {
                        VIAEntPtr pVIAEnt;
                        pPriv->ptr = xnfcalloc(sizeof(VIAEntRec), 1);
                        pVIAEnt = pPriv->ptr;
                        pVIAEnt->IsDRIEnabled = FALSE;
                        pVIAEnt->BypassSecondary = FALSE;
                        pVIAEnt->HasSecondary = FALSE;
                        pVIAEnt->IsSecondaryRestored = FALSE;
                    }
                }
                instance++;

            xfree(pEnt);
        }
    }

    xfree(usedChips);

    return foundScreen;

} /* VIAProbe */


static int LookupChipID(PciChipsets* pset, int ChipID)
{
    /* Is there a function to do this for me? */
    while (pset->numChipset >= 0)
    {
        if (pset->PCIid == ChipID)
            return pset->numChipset;

        pset++;
    }

    return -1;

} /* LookupChipID */


static unsigned int
VIAddc1Read(ScrnInfoPtr pScrn)
{
    register vgaHWPtr hwp = VGAHWPTR(pScrn);
    VIAPtr pVia = VIAPTR(pScrn);
    register CARD8 tmp;

    while (hwp->readST01(hwp)&0x8) {};
    while (!(hwp->readST01(hwp)&0x8)) {};

    VGAOUT8(0x3c4, 0x26);
    tmp = VGAIN8(0x3c5);
    return ((unsigned int) ((tmp & 0x08) >> 3));
}

static void
VIAvgaHWddc1SetSpeed(ScrnInfoPtr pScrn, xf86ddcSpeed speed)
{
    vgaHWPtr hwp = VGAHWPTR(pScrn);
    unsigned char tmp;
    struct _vgaDdcSave* save;
    switch (speed) {
    case DDC_FAST:
        
        if (hwp->ddc != NULL) break;
        hwp->ddc = xnfcalloc(sizeof(struct _vgaDdcSave),1);
	save = (struct _vgaDdcSave *)hwp->ddc;
      /* Lightpen register disable - allow access to cr10 & 11; just in case */
	save->cr03 = hwp->readCrtc(hwp, 0x03); 
	hwp->writeCrtc(hwp,0x03,(save->cr03 |0x80));
	save->cr12 = hwp->readCrtc(hwp, 0x12); 
	hwp->writeCrtc(hwp,0x12,DISPLAY_END);
	save->cr15 = hwp->readCrtc(hwp, 0x15); 
	hwp->writeCrtc(hwp,0x15,BLANK_START);
	save->cr10 = hwp->readCrtc(hwp, 0x10); 
	hwp->writeCrtc(hwp,0x10,SYNC_START);
	save->cr11 = hwp->readCrtc(hwp, 0x11); 
	/* unprotect group 1 registers; just in case ...*/
	hwp->writeCrtc(hwp,0x11,((save->cr11 & 0x70) | SYNC_END));
	save->cr16 = hwp->readCrtc(hwp, 0x16); 
	hwp->writeCrtc(hwp,0x16,BLANK_END);
	save->cr06 = hwp->readCrtc(hwp, 0x06); 
	hwp->writeCrtc(hwp,0x06,V_TOTAL);
	/* all values have less than 8 bit - mask out 9th and 10th bits */
	save->cr09 = hwp->readCrtc(hwp, 0x09); 
	hwp->writeCrtc(hwp,0x09,(save->cr09 &0xDF));
	save->cr07 = hwp->readCrtc(hwp, 0x07); 
	hwp->writeCrtc(hwp,0x07,(save->cr07 &0x10));
	/* vsync polarity negativ & ensure a 25MHz clock */
	save->msr = hwp->readMiscOut(hwp); 
	hwp->writeMiscOut(hwp,((save->msr & 0xF3) | 0x80));
	break;
    case DDC_SLOW:
        if (hwp->ddc == NULL) break;
	save = (struct _vgaDdcSave *)hwp->ddc;
	hwp->writeMiscOut(hwp,save->msr);
	hwp->writeCrtc(hwp,0x07,save->cr07);
	tmp = hwp->readCrtc(hwp, 0x09);
	hwp->writeCrtc(hwp,0x09,((save->cr09 & 0x20) | (tmp & 0xDF)));
	hwp->writeCrtc(hwp,0x06,save->cr06);
	hwp->writeCrtc(hwp,0x16,save->cr16);
	hwp->writeCrtc(hwp,0x11,save->cr11);
	hwp->writeCrtc(hwp,0x10,save->cr10);
	hwp->writeCrtc(hwp,0x15,save->cr15);
	hwp->writeCrtc(hwp,0x12,save->cr12);
	hwp->writeCrtc(hwp,0x03,save->cr03);
	xfree(save);
	hwp->ddc = NULL;
	break;
    default:
	break;
    }
}

static Bool
VIAddc1(int scrnIndex)
{
    ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
    VIAPtr pVia = VIAPTR(pScrn);
    xf86MonPtr pMon;
    CARD8 tmp;
    Bool success = FALSE;

    /* initialize chipset */
    VGAOUT8(0x3c4, 0x26);
    tmp = VGAIN8(0x3c5);
    VGAOUT8(0x3c4, 0x26);
    VGAOUT8(0x3c5, (tmp | 0x11));

    if ((pMon = xf86PrintEDID(
        xf86DoEDID_DDC1(scrnIndex,VIAvgaHWddc1SetSpeed,VIAddc1Read))) != NULL)
        success = TRUE;
    xf86SetDDCproperties(pScrn,pMon);

    /* undo initialization */
    VGAOUT8(0x3c4, 0x26);
    VGAOUT8(0x3c5, tmp);
    return success;
}

static Bool
VIAddc2(int scrnIndex)
{
    ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
    VIAPtr pVia = VIAPTR(pScrn);
    xf86MonPtr pMon;
    CARD8 tmp;
    Bool success = FALSE;

    VGAOUT8(0x3c4, 0x26);
    tmp = VGAIN8(0x3c5);
    pMon = xf86DoEDID_DDC2(pScrn->scrnIndex, pVia->I2C_Port1);
    if (pMon)
        success = TRUE;
    pVia->DDC1 =  pMon;
    xf86PrintEDID(pMon);
    xf86SetDDCproperties(pScrn, pMon);
    VGAOUT8(0x3c4, 0x26);
    VGAOUT8(0x3c5, tmp);

    return success;
}

static void
VIAProbeDDC(ScrnInfoPtr pScrn, int index)
{
    vbeInfoPtr pVbe;

    if (xf86LoadSubModule(pScrn, "vbe"))
    {
        if (via_xf86_version_current < XF86_VERSION_NUMERIC(4,2,99,0,0))
            pVbe = VBEInit(NULL,index);
        else
            pVbe = VBEExtendedInit(NULL,index,0);

        ConfiguredMonitor = vbeDoEDID(pVbe, NULL);
    }
}

static void
VIACheckIfUseDuoView(ScrnInfoPtr pScrn)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;
    LPVIAGRAPHICINFO lpVIAGraphicInfo = &pVia->pVidData->VIAGraphicInfo;
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIACheckIfUseDuoView\n"));
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "ActiveDev: %d\n", pBIOSInfo->ActiveDevice));

    pBIOSInfo->DuoView = TRUE;

    /* RainLin [2005.10.07]: Add option "Simultaneous" and remove option "DuoView" */
    /* since DuoView has became the default path. */
    if (xf86ReturnOptValBool(VIAOptions, OPTION_SIMULTANEOUS, FALSE))
    {
        pBIOSInfo->DuoView = FALSE;
        DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Option: Simultaneous support!!\n"));
    }
    
    /* Check device number, we only support duo view when enable 2 device. */
    /* LCD + LCD2 case only work on IGA2, so disable "Douview" flag */
    if ((pBIOSInfo->ActiveDevice == VIA_DEVICE_CRT1) ||
        (pBIOSInfo->ActiveDevice == VIA_DEVICE_LCD)  ||
        (pBIOSInfo->ActiveDevice == (VIA_DEVICE_LCD+VIA_DEVICE_LCD2)) ||
        (pBIOSInfo->ActiveDevice == VIA_DEVICE_DFP)  ||
        (pBIOSInfo->ActiveDevice == VIA_DEVICE_TV)   ||
        (pBIOSInfo->ActiveDevice == VIA_DEVICE_CRT2))
    {
        pBIOSInfo->DuoView = FALSE;
        DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Not support DuoView because only enable one device.\n"));
    }

    /* The two combinations only support DuoView and SAMM, and we use DuoView
       as defuault. */
    if ((pBIOSInfo->ActiveDevice == (VIA_DEVICE_TV | VIA_DEVICE_LCD)) ||
        (pBIOSInfo->ActiveDevice == (VIA_DEVICE_TV | VIA_DEVICE_DFP)) ||
        (pBIOSInfo->ActiveDevice == (VIA_DEVICE_TV | VIA_DEVICE_TV2)))
    {
        pBIOSInfo->DuoView = TRUE;
    }
                    
    if (pVia->HasSecondary)
    {
        pBIOSInfo->DuoView = FALSE;
        DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Not support DuoView because SAMM is on.\n"));
    }

    if (pBIOSInfo->DuoView)
    {
        DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "DuoView support!!\n"));
        lpVIAGraphicInfo->duoview |= DUOVIEW_ENABLED;
    }
    else
    {
        lpVIAGraphicInfo->duoview &= ~DUOVIEW_ENABLED;
    }        
}

void
VIACheckCursorTypeToUse(ScrnInfoPtr pScrn)
{
    VIAPtr pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr pBIOSInfo = pVia->pBIOSInfo;
    LPVIAGRAPHICINFO lpVIAGraphicInfo = &pVia->pVidData->VIAGraphicInfo;

    if (
        (pVia->shadowFB)        ||
       /* (pVia->IsSecondary)     ||*/
        (pVia->ForceSWCursor)   ||
        ( pBIOSInfo->ConnectedDevice&VIA_DEVICE_LCD)   ||
        (pBIOSInfo->TwoCRT)     ||
        (pBIOSInfo->TwoLCD)     ||
        (pBIOSInfo->MergedFB)   ||
        (pVia->IsHWRotateEnabled))
    {
        /* Use SW Cursor */
        pVia->hwcursor = FALSE;
    }
    else
    {
        /* Use HW Cursor */
        pVia->hwcursor = TRUE;
    }

    if(pBIOSInfo->Is3DScaling)
    {
        if(pBIOSInfo->SAMM)
        {
            pVia->hwcursor = FALSE;        
        }
        else
        {
            if (VIA_DEVICE_TV == pBIOSInfo->ActiveDevice)
            {
                pVia->hwcursor = TRUE;
            }
            else
            {
                pVia->hwcursor = FALSE;
            }
        }
    }

    lpVIAGraphicInfo->IsHWCursorEnabled = pVia->hwcursor;
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Using %s cursor\n", pVia->hwcursor ? "HW" : "SW"));
}

static int VIATranslateTVOutputNameToID(char *pName)
{
    if (!xf86NameCmp(pName, "S-Video"))
    {
        return TV_OUTPUT_SVIDEO;
    }
    else if(!xf86NameCmp(pName, "Composite"))
    {
        return TV_OUTPUT_COMPOSITE;
    }
    else if(!xf86NameCmp(pName, "SC"))
    {
        return TV_OUTPUT_COMPOSITE_SVIDEO;
    }
    else if(!xf86NameCmp(pName, "RGB"))
    {
        return TV_OUTPUT_RGB;
    }
    else if(!xf86NameCmp(pName, "YCbCr") || !xf86NameCmp(pName, "YPbPr"))
    {
        return TV_OUTPUT_YPBPR;
    }
    else if(!xf86NameCmp(pName, "RGBSC"))
    {
        return TV_OUTPUT_RGB_COMPOSITE_SVIDEO;
    }
    else if(!xf86NameCmp(pName, "YCbCrSC") || !xf86NameCmp(pName, "YPbPrSC"))
    {
        return TV_OUTPUT_YPBPR_COMPOSITE_SVIDEO;
    }
    else
    {
        ErrorF("Invalid TV Output.\n");
        return TV_OUTPUT_NONE;
    }
}

static void VIAGetOptions_MergedFB(ScrnInfoPtr pScrn)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    char        *strptr;
    VIABIOSInfoPtr pBIOSInfo = pVia->pBIOSInfo;
	LPVIAGRAPHICINFO lpVIAGraphicInfo = &pVia->pVidData->VIAGraphicInfo;
   
    /* collect MergedFB options */
    pBIOSInfo->MergedFB = FALSE;
    pBIOSInfo->UseVIAXinerama = TRUE;
    pBIOSInfo->Scrn2Position = viaRightOf;
    
    if (xf86ReturnOptValBool(VIAOptions, OPTION_MERGEDFB, FALSE))
    {  
        pBIOSInfo->MergedFB = TRUE;
        pBIOSInfo->DuoView = TRUE;
        lpVIAGraphicInfo->duoview = 1;
        xf86DrvMsg(pScrn->scrnIndex, X_INFO, "MergedFB mode forced on.\n");
    }
    else
    {
        pBIOSInfo->MergedFB = FALSE;
        xf86DrvMsg(pScrn->scrnIndex, X_INFO, "MergedFB mode forced off.\n");
    }
       
    /* Do some MergedFB mode initialisation */
   
    if(pBIOSInfo->MergedFB)
    {
        strptr = (char *)xf86GetOptValString(VIAOptions, OPTION_SCRN2POS);
        if(strptr)
        {
            if((!strcmp(strptr,"LeftOf")) || (!strcmp(strptr,"leftof")))
            {
                pBIOSInfo->Scrn2Position = viaLeftOf;
            }
            else if((!strcmp(strptr,"RightOf")) || (!strcmp(strptr,"rightof")))
            {
                pBIOSInfo->Scrn2Position = viaRightOf;
            }
            else if((!strcmp(strptr,"Above")) || (!strcmp(strptr,"above")))
            {
                pBIOSInfo->Scrn2Position = viaAbove;
            }
            else if((!strcmp(strptr,"Below")) || (!strcmp(strptr,"below")))
            {
                pBIOSInfo->Scrn2Position = viaBelow;
            }
            else
            {
	            xf86DrvMsg(pScrn->scrnIndex, X_WARNING,
	                "\"%s\" is not a valid parameter for Option \"Scrn2Position\"\n", strptr);
	    	    xf86DrvMsg(pScrn->scrnIndex, X_INFO,
	                "Valid parameters are \"RightOf\", \"LeftOf\", \"Above\", or\"Below\"\n");
            }
        }

        /*don't use pseudo xinerama,but virtual screen*/
        if (xf86ReturnOptValBool(VIAOptions, OPTION_NOVIAXINERAMA, FALSE))
        {  
            pBIOSInfo->UseVIAXinerama = FALSE;
        }
    }
}

static void VIAGetOptions_Cursor(ScrnInfoPtr pScrn)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr pBIOSInfo = pVia->pBIOSInfo;
    /*
     * The SWCursor setting takes priority over HWCursor.  The default
     * if neither is specified is HW.
     */
    if (xf86GetOptValBool(VIAOptions, OPTION_HWCURSOR, &pVia->hwcursor))
    {
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Option HWCursor enabled.\n");
    }

    pVia->ForceSWCursor = FALSE;
    
    if (xf86ReturnOptValBool(VIAOptions, OPTION_SWCURSOR, FALSE))
    {
        pVia->ForceSWCursor = TRUE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Option SWCursor enabled.\n");
    }
    /*cliff add for hardware icon color cursor*/
    pBIOSInfo->ARGB_cursor = FALSE;
    /* cle266 only use SW cursor
    */
    if (pBIOSInfo->Chipset==VIA_CLE266)
    {
         pVia->ForceSWCursor = TRUE;
    }
}

static void VIAGetOptions_ActiveDevice(ScrnInfoPtr pScrn)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr pBIOSInfo = pVia->pBIOSInfo;
    char    *pStr = NULL;
    
    pBIOSInfo->TwoLCD = FALSE;
    pBIOSInfo->TwoCRT = FALSE;    

    /*=* For Advantech Project, support two LCD MHS *=*/
    if (xf86ReturnOptValBool(VIAOptions, OPTION_TWO_LCD, FALSE))
    {
        pBIOSInfo->TwoLCD = TRUE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Option: Two LCD support!!\n");
    }
    /*support Dual CRT with TV card vt1625*/
    else if (xf86ReturnOptValBool(VIAOptions, OPTION_TWO_CRT, FALSE))
    {
        pBIOSInfo->TwoCRT = TRUE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Option: Two CRT support!!\n");
    }    
    else if (xf86ReturnOptValBool(VIAOptions, OPTION_CRT2_SUPPORTED, FALSE))
    {
        pBIOSInfo->CRT2_SUPPORTED = TRUE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Option: CRT2 SUPPORTED!!\n");
    } 

    if(pBIOSInfo->TwoLCD)
    {
        /*=* For Advantech Project, support two LCD MHS.
             Always set ActiveDevice to CRT1+LCD. *=*/
        pBIOSInfo->ActiveDevice = VIA_DEVICE_CRT1 | VIA_DEVICE_LCD;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Active Device is LCD1 and LCD2.\n");
    }
    else if(pBIOSInfo->TwoCRT)
    {
        pBIOSInfo->ActiveDevice = VIA_DEVICE_CRT1|VIA_DEVICE_CRT2;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Active Device is Dual CRT.\n");
    }
    else
    {
        /* ActiveDevice Option for device selection */
        pBIOSInfo->ActiveDevice = 0x00;
        
        if ((pStr = xf86GetOptValString(VIAOptions, OPTION_ACTIVEDEVICE)))
        {
            xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Active Device is %s.\n", pStr);
            
            if (!xf86NameCmp(pStr, "CRT,TV"))
            {
                pBIOSInfo->ActiveDevice = VIA_DEVICE_CRT1 | VIA_DEVICE_TV;
                pBIOSInfo->PrimaryDevice = VIA_DEVICE_CRT1;
            }
            else if(!xf86NameCmp(pStr, "TV,CRT"))   
            {
                pBIOSInfo->ActiveDevice = VIA_DEVICE_CRT1 | VIA_DEVICE_TV;
                pBIOSInfo->PrimaryDevice = VIA_DEVICE_TV;
            }
            else if(!xf86NameCmp(pStr, "CRT,CRT2")) 
            {
                if(!(pBIOSInfo->MobileFamily || pVia->Chipset == VIA_CLE266))
                {
                    pBIOSInfo->ActiveDevice = VIA_DEVICE_CRT1 | VIA_DEVICE_CRT2;
                    pBIOSInfo->PrimaryDevice = VIA_DEVICE_CRT1;
                    pBIOSInfo->TwoCRT = TRUE;
                }
                else
                {
                    xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,"Active Device CRT1 and CRT2 is NOT ENABLED,it is for CN400 and DESKTOP BIOS ONLY.\n");
                }
            }
            else if(!xf86NameCmp(pStr, "CRT2,CRT")) 
            {
                if(!(pBIOSInfo->MobileFamily || pVia->Chipset == VIA_CLE266))
                {
                    pBIOSInfo->ActiveDevice = VIA_DEVICE_CRT1 | VIA_DEVICE_CRT2;
                    pBIOSInfo->PrimaryDevice = VIA_DEVICE_CRT2;
                    pBIOSInfo->TwoCRT = TRUE;
                }
                else
                {
                    xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,"Active Device CRT2 and CRT1 is NOT ENABLED,it is for CN400 and DESKTOP BIOS ONLY.\n");
                }               
            }           
            else if(!xf86NameCmp(pStr, "CRT,LCD"))
            {
                pBIOSInfo->ActiveDevice = VIA_DEVICE_CRT1 | VIA_DEVICE_LCD;
                pBIOSInfo->PrimaryDevice = VIA_DEVICE_CRT1;
            }
            else if(!xf86NameCmp(pStr, "LCD,CRT"))
            {
                pBIOSInfo->ActiveDevice = VIA_DEVICE_CRT1 | VIA_DEVICE_LCD;
                pBIOSInfo->PrimaryDevice = VIA_DEVICE_LCD;
            }
            else if(!xf86NameCmp(pStr, "LCD,LCD2"))
            {
                if (pVia->Chipset == VIA_CX700)
                {
                    pBIOSInfo->ActiveDevice = VIA_DEVICE_LCD | VIA_DEVICE_LCD2;
                    pBIOSInfo->PrimaryDevice = VIA_DEVICE_LCD;
                }
                else
                {
                    pBIOSInfo->ActiveDevice = VIA_DEVICE_CRT1 | VIA_DEVICE_LCD;
                    pBIOSInfo->PrimaryDevice = VIA_DEVICE_CRT1;
                    pBIOSInfo->TwoLCD = TRUE;
                }
            }
            else if(!xf86NameCmp(pStr, "LCD2,LCD"))
            {
                if (pVia->Chipset == VIA_CX700)
                {
                    pBIOSInfo->ActiveDevice = VIA_DEVICE_LCD | VIA_DEVICE_LCD2;
                    pBIOSInfo->PrimaryDevice = VIA_DEVICE_LCD2;
                }
                else
                {
                    pBIOSInfo->ActiveDevice = VIA_DEVICE_CRT1 | VIA_DEVICE_LCD;
                    pBIOSInfo->PrimaryDevice = VIA_DEVICE_LCD;
                    pBIOSInfo->TwoLCD = TRUE;
                    pBIOSInfo->UseIGA2 = TRUE;
                }
            }
            else if(!xf86NameCmp(pStr, "TV,LCD"))
            {
                pBIOSInfo->ActiveDevice = VIA_DEVICE_TV | VIA_DEVICE_LCD;
                pBIOSInfo->PrimaryDevice = VIA_DEVICE_TV;
            }
            else if(!xf86NameCmp(pStr, "LCD,TV"))
            {
                pBIOSInfo->ActiveDevice = VIA_DEVICE_TV | VIA_DEVICE_LCD;
                pBIOSInfo->PrimaryDevice = VIA_DEVICE_LCD;
            }
            else if(!xf86NameCmp(pStr, "TV,TV2"))   
            {
                if (pVia->Chipset == VIA_CX700)
                {
                    pBIOSInfo->ActiveDevice = VIA_DEVICE_TV | VIA_DEVICE_TV2;
                    pBIOSInfo->PrimaryDevice = VIA_DEVICE_TV;
                }
                else
                {
                    pBIOSInfo->ActiveDevice = VIA_DEVICE_TV;
                }
            }
            else if(!xf86NameCmp(pStr, "TV2,TV"))   
            {
                if (pVia->Chipset == VIA_CX700)
                {
                    pBIOSInfo->ActiveDevice = VIA_DEVICE_TV | VIA_DEVICE_TV2;
                    pBIOSInfo->PrimaryDevice = VIA_DEVICE_TV2;
                }
                else
                {
                    pBIOSInfo->ActiveDevice = VIA_DEVICE_TV;
                }
            }
            else if(!xf86NameCmp(pStr, "CRT,DFP") ||!xf86NameCmp(pStr, "CRT,DVI"))
            {
                pBIOSInfo->ActiveDevice = VIA_DEVICE_CRT1 | VIA_DEVICE_DFP;
                pBIOSInfo->PrimaryDevice = VIA_DEVICE_CRT1;
            }
            else if(!xf86NameCmp(pStr, "DFP,CRT") || !xf86NameCmp(pStr, "DVI,CRT"))
            {
                pBIOSInfo->ActiveDevice = VIA_DEVICE_CRT1 | VIA_DEVICE_DFP;
                pBIOSInfo->PrimaryDevice = VIA_DEVICE_DFP;
            }
            else if(!xf86NameCmp(pStr, "DFP,TV")|| !xf86NameCmp(pStr, "DVI,TV"))
            {
                pBIOSInfo->ActiveDevice = VIA_DEVICE_TV | VIA_DEVICE_DFP;
                pBIOSInfo->PrimaryDevice = VIA_DEVICE_DFP;
            }
            else if(!xf86NameCmp(pStr, "TV,DFP") ||!xf86NameCmp(pStr, "TV,DVI"))
            {
                pBIOSInfo->ActiveDevice = VIA_DEVICE_TV | VIA_DEVICE_DFP;
                pBIOSInfo->PrimaryDevice = VIA_DEVICE_TV;
            }
            else if(!xf86NameCmp(pStr, "DFP,LCD") || !xf86NameCmp(pStr, "DVI,LCD"))
            {
                pBIOSInfo->ActiveDevice = VIA_DEVICE_DFP | VIA_DEVICE_LCD;
                pBIOSInfo->PrimaryDevice = VIA_DEVICE_DFP;
            }
            else if(!xf86NameCmp(pStr, "LCD,DFP") || !xf86NameCmp(pStr, "LCD,DVI"))
            {
                pBIOSInfo->ActiveDevice = VIA_DEVICE_DFP | VIA_DEVICE_LCD;
                pBIOSInfo->PrimaryDevice = VIA_DEVICE_LCD;
            }
            else if(!xf86NameCmp(pStr, "CRT") || !xf86NameCmp(pStr, "CRT ONLY"))
            {
                pBIOSInfo->ActiveDevice = VIA_DEVICE_CRT1;
            }
            else if(!xf86NameCmp(pStr, "LCD") || !xf86NameCmp(pStr, "LCD ONLY"))
            {
                pBIOSInfo->ActiveDevice = VIA_DEVICE_LCD;
            }
            else if(!xf86NameCmp(pStr, "TV") || !xf86NameCmp(pStr, "TV ONLY"))
            {
                pBIOSInfo->ActiveDevice = VIA_DEVICE_TV;
            }
            else if(!xf86NameCmp(pStr, "DFP") || !xf86NameCmp(pStr, "DFP ONLY") ||
                    !xf86NameCmp(pStr, "DVI") || !xf86NameCmp(pStr, "DVI ONLY"))
            {
                pBIOSInfo->ActiveDevice = VIA_DEVICE_DFP;
            }
            else 
            {
                xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,
                    "Option \"%s\" can't recognize!, Active Device by default.\n", pStr);
            }
        }
    }
}

static void VIAGetOptions_DisplayHardwareLayout(ScrnInfoPtr pScrn)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr pBIOSInfo = pVia->pBIOSInfo;
    char    *pStr = NULL;
    
    /* RainLin [2006.03.07]: Because we can't get any information about hardware layout */
    /* from BIOS, so add option DisplayHardwareLayout to determine it. */
    pBIOSInfo->DisplayHardwareLayout = HW_LAYOUT_LCD_DVI;
    
    if ((pStr = xf86GetOptValString(VIAOptions, OPTION_DISPLAY_HARDWARE_LAYOUT)))
    {
        if (!xf86NameCmp(pStr, "LCD") || !xf86NameCmp(pStr, "LCD ONLY"))
        {
            pBIOSInfo->DisplayHardwareLayout = HW_LAYOUT_LCD_ONLY;
        }
        else if (!xf86NameCmp(pStr, "DVI") || !xf86NameCmp(pStr, "DVI ONLY"))
        {
            pBIOSInfo->DisplayHardwareLayout = HW_LAYOUT_DVI_ONLY;
        }
        else if (!xf86NameCmp(pStr, "LCD+LCD2") || !xf86NameCmp(pStr, "LCD2+LCD"))
        {
            pBIOSInfo->DisplayHardwareLayout = HW_LAYOUT_LCD_LCD2;
        }
        else if (!xf86NameCmp(pStr, "DVI+LCD") || !xf86NameCmp(pStr, "LCD+DVI"))
        {
            pBIOSInfo->DisplayHardwareLayout = HW_LAYOUT_LCD_DVI;
        }
        else if (!xf86NameCmp(pStr, "LCD+ExternalLCD2") || !xf86NameCmp(pStr, "ExternalLCD2+LCD"))
        {
            pBIOSInfo->DisplayHardwareLayout = HW_LAYOUT_LCD_EXTERNAL_LCD2;
        }
        else if (!xf86NameCmp(pStr, "TV+ExternalTV2") || !xf86NameCmp(pStr, "ExternalTV2+TV"))
        {
            pBIOSInfo->DisplayHardwareLayout = HW_LAYOUT_TV_EXTERNAL_TV2;
        }

        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Display Hardware Layout is %s\n", pStr);
    }
}


static int
TranslateDIPortNameToID(char *pStr)
{
    if (!xf86NameCmp(pStr, "DFP_HIGH"))
    {
        return VIA_DI_DFPHIGH;
    }
    else if (!xf86NameCmp(pStr, "DFP_LOW"))
    {
        return VIA_DI_DFPLOW;
    }
    else if (!xf86NameCmp(pStr, "DFP_HIGHLOW"))
    {
        return VIA_DI_DFPHIGHLOW;
    }
    else if (!xf86NameCmp(pStr, "DVP0"))
    {
        return VIA_DI_DVP0;
    }
    else if (!xf86NameCmp(pStr, "DVP1"))
    {
        return VIA_DI_DVP1;
    }
    else
    {
        /* Invalid port. */
        return VIA_DI_NONE;
    }
}


static void VIAGetOptions_LCDInfo(ScrnInfoPtr pScrn)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr pBIOSInfo = pVia->pBIOSInfo;    
    int option_panel_id;
    char    *pStr = NULL;
    
    /* Initialize LCD panel size: */
    pBIOSInfo->LVDSSettingInfo.PanelSizeID = VIA_PANEL_INVALID;
    
    if (xf86ReturnOptValBool(VIAOptions, OPTION_FORCE_LCD, FALSE))
    {
        pBIOSInfo->MobileFamily = TRUE;        
        pBIOSInfo->LVDSSettingInfo.ChipID = VIA_Hardwired;
        
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "(OPTION) Force enable LCD. \n");
    }    

    
    /* LCD Center/Expand Option */
    if (xf86ReturnOptValBool(VIAOptions, OPTION_CENTER, FALSE))
    {
        pBIOSInfo->LVDSSettingInfo.Center = TRUE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "(OPTION) LCD Center is On\n");
    }
    else
    {
        pBIOSInfo->LVDSSettingInfo.Center = FALSE;
    }

    /* S.T.Chen[2006.6.19]: Set LCD Panel Info. */
    if (xf86GetOptValInteger(VIAOptions, OPTION_LCDPANELID, &option_panel_id))
    {
        VIASetLCDPanelInfo(pBIOSInfo, option_panel_id);
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "(OPTION) LCD Panel ID is %d.\n", option_panel_id);
    }

    /* Bus Width Option */
    pBIOSInfo->LVDSSettingInfo.BusWidth = VIA_DI_12BIT;

    if ((pStr = xf86GetOptValString(VIAOptions, OPTION_LCD_BUS_WIDTH)))
    {
        if (!xf86NameCmp(pStr, "12BIT"))
        {
            pBIOSInfo->LVDSSettingInfo.BusWidth = VIA_DI_12BIT;
            xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "LCD Bus Width is 12BIT\n");
        }
        else if (!xf86NameCmp(pStr, "24BIT"))
        {
            pBIOSInfo->LVDSSettingInfo.BusWidth = VIA_DI_24BIT;
            xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "LCD Bus Width is 24BIT\n");
        }
    }

    /* Support LCD port selection */
    pBIOSInfo->LVDSSettingInfo.DIPort = VIA_DI_NONE;
    if ((pStr = xf86GetOptValString(VIAOptions, OPTION_LCD_PORT)))
    {
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "LCD port set to %s.\n", pStr);
        pBIOSInfo->LVDSSettingInfo.DIPort = TranslateDIPortNameToID(pStr);        
    }
}

static void VIAGetOptions_TVInfo(ScrnInfoPtr pScrn)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr pBIOSInfo = pVia->pBIOSInfo;
    char    *pStr = NULL;
    
    /* TV DotCrawl Enable Option */
    if (xf86ReturnOptValBool(VIAOptions, OPTION_TVDOTCRAWL, FALSE))
    {
        pBIOSInfo->TVSettingInfo.TVDotCrawl = TRUE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "DotCrawl is Enable\n");
    }
    else
    {
        pBIOSInfo->TVSettingInfo.TVDotCrawl = FALSE;
    }

    /* TV Type: */
    pBIOSInfo->TVSettingInfo.TVType = TVTYPE_NONE;
    pBIOSInfo->TVSettingInfo.TVTypeIsNTSCJ = FALSE;
    if ((pStr = xf86GetOptValString(VIAOptions, OPTION_TVTYPE)))
    {
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "TV Type is %s.\n", pStr);
        
        if (!xf86NameCmp(pStr, "NTSC"))
        {
            pBIOSInfo->TVSettingInfo.TVType = TVTYPE_NTSC;
        }
        if (!xf86NameCmp(pStr, "NTSC_J"))
        {
            pBIOSInfo->TVSettingInfo.TVType = TVTYPE_NTSC;
            pBIOSInfo->TVSettingInfo.TVTypeIsNTSCJ = TRUE;
        }
        else if(!xf86NameCmp(pStr, "PAL"))
        {
            pBIOSInfo->TVSettingInfo.TVType = TVTYPE_PAL;
        }
        else if(!xf86NameCmp(pStr, "480P"))
        {
            pBIOSInfo->TVSettingInfo.TVType = TVTYPE_480P;
        }
        else if(!xf86NameCmp(pStr, "576P"))
        {
            pBIOSInfo->TVSettingInfo.TVType = TVTYPE_576P;
        }
        else if(!xf86NameCmp(pStr, "720P"))
        {
            if ((pBIOSInfo->Chipset == VIA_CLE266) ||
                (pBIOSInfo->Chipset == VIA_KM400) ||
                (pBIOSInfo->Chipset == VIA_K8M800))
            {
                ErrorF("HDTV 720P couldn't be supported\n");
            }
            else
            {
                pBIOSInfo->TVSettingInfo.TVType = TVTYPE_720P;                
            }
        }
        else if(!xf86NameCmp(pStr, "1080I"))
        {
            if ((pBIOSInfo->Chipset == VIA_CLE266) ||
                (pBIOSInfo->Chipset == VIA_KM400) ||
                (pBIOSInfo->Chipset == VIA_K8M800))
            {
                ErrorF("HDTV 1080I couldn't be supported\n");
            }
            else
            {
                pBIOSInfo->TVSettingInfo.TVType = TVTYPE_1080I;                
            }
        }
    }

    /* TV Progressive Mode Enable Option */
    if (xf86ReturnOptValBool(VIAOptions, OPTION_TVPROGRESSIVE, FALSE))
    {
        pBIOSInfo->TVSettingInfo.TVProgressive = TRUE;
        if (pBIOSInfo->TVSettingInfo.TVType == TVTYPE_NTSC)
        {
            pBIOSInfo->TVSettingInfo.TVType = TVTYPE_480P;
        }
        else if (pBIOSInfo->TVSettingInfo.TVType == TVTYPE_PAL)
        {
            pBIOSInfo->TVSettingInfo.TVType = TVTYPE_576P;
        }

        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "TV Progressive Mode is Enable\n");
    }
    else
    {
        pBIOSInfo->TVSettingInfo.TVProgressive = FALSE;
    }

    /* TV out put signal Option */
    pBIOSInfo->TVSettingInfo.TVOutput = TV_OUTPUT_NONE;
    if ((pStr = xf86GetOptValString(VIAOptions, OPTION_TVOUTPUT)))
    {
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "TV Output Signal is %s\n", pStr);
        pBIOSInfo->TVSettingInfo.TVOutput = VIATranslateTVOutputNameToID(pStr);
    }

    /* 2nd TV out put signal Option */
    pBIOSInfo->TVOutput2 = TV_OUTPUT_YPBPR;
    if ((pStr = xf86GetOptValString(VIAOptions, OPTION_TVOUTPUT2)))
    {
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "2nd TV Output Signal is %s\n", pStr);
        pBIOSInfo->TVOutput2 = VIATranslateTVOutputNameToID(pStr);
    }

    /* TV Standard Option */
    pBIOSInfo->TVSettingInfo.TVVScan = VIA_TVNORMAL;
    if ((pStr = xf86GetOptValString(VIAOptions, OPTION_TVVSCAN)))
    {
        if (!xf86NameCmp(pStr, "under") || !xf86NameCmp(pStr, "normal"))
        {
            pBIOSInfo->TVSettingInfo.TVVScan = VIA_TVNORMAL;
        }
        else if (!xf86NameCmp(pStr, "fit"))
        {
            pBIOSInfo->TVSettingInfo.TVVScan = VIA_TVFIT;
        }
        else if (!xf86NameCmp(pStr, "over"))
        {
            pBIOSInfo->TVSettingInfo.TVVScan = VIA_TVOVER;
        }
    }
    if(pVia->DISP3DScaling)
    {
        if ((TVTYPE_1080I==pBIOSInfo->TVSettingInfo.TVType )|(TVTYPE_720P==pBIOSInfo->TVSettingInfo.TVType))
        {
            pBIOSInfo->TVSettingInfo.TVVScan = VIA_TVNORMAL;
        }
        else
        {
            pBIOSInfo->TVSettingInfo.TVVScan = VIA_TVOVER;
        }
    }

    /* For H-Scaling adjustment: */
    /* We can use this option to specify the number of pixels to adjust each time. */
    pBIOSInfo->TVSettingInfo.TVHScale = VIA_TVSCALE5;
    if (xf86GetOptValInteger(VIAOptions, OPTION_TVHSCALE, &(pBIOSInfo->TVSettingInfo.TVHScale)))
    {
        if ((pBIOSInfo->TVSettingInfo.TVHScale < VIA_TVSCALE1) ||
            (pBIOSInfo->TVSettingInfo.TVHScale > VIA_TVSCALE5))
        {
            pBIOSInfo->TVSettingInfo.TVHScale = VIA_TVSCALE5;
        }
    }

    /* For V-Scaling adjustment: */
    /* We can use this option to specify the number of lines to adjust each time. */
    pBIOSInfo->TVSettingInfo.TVVScale = VIA_TVSCALE5;
    if (xf86GetOptValInteger(VIAOptions, OPTION_TVVSCALE, &(pBIOSInfo->TVSettingInfo.TVVScale)))
    {
        if ((pBIOSInfo->TVSettingInfo.TVVScale < VIA_TVSCALE1) ||
            (pBIOSInfo->TVSettingInfo.TVVScale > VIA_TVSCALE5))
        {
            pBIOSInfo->TVSettingInfo.TVVScale = VIA_TVSCALE5;
        }
    }

    /* TV Encoder Type Option */
    pBIOSInfo->TVSettingInfo.TVEncoder = VIA_NONETV;
    if ((pStr = xf86GetOptValString(VIAOptions, OPTION_TVENCODER)))
    {
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "TV Encoder is %s!\n", pStr);
        
        if (!xf86NameCmp(pStr, "VT1621"))
        {
            pBIOSInfo->TVSettingInfo.TVEncoder = VIA_VT1621;
            pBIOSInfo->TVI2CAdd = 0x40;
        }
        else if(!xf86NameCmp(pStr, "VT1622"))
        {
            pBIOSInfo->TVSettingInfo.TVEncoder = VIA_VT1622;
            pBIOSInfo->TVI2CAdd = 0x40;
        }
        else if(!xf86NameCmp(pStr, "VT1622A"))
        {
            pBIOSInfo->TVSettingInfo.TVEncoder = VIA_VT1622A;
            pBIOSInfo->TVI2CAdd = 0x40;
        }
        else if(!xf86NameCmp(pStr, "VT1623"))
        {
            pBIOSInfo->TVSettingInfo.TVEncoder = VIA_VT1623;
            pBIOSInfo->TVI2CAdd = 0x40;
        }
        else if(!xf86NameCmp(pStr, "VT1625"))
        {
            pBIOSInfo->TVSettingInfo.TVEncoder = VIA_VT1625;
            pBIOSInfo->TVI2CAdd = 0x40;
        }
        if ((pBIOSInfo->TVSettingInfo.TVEncoder != VIA_VT1625) && 
            (pBIOSInfo->TVSettingInfo.TVOutput == TV_OUTPUT_RGB_COMPOSITE_SVIDEO || 
             pBIOSInfo->TVSettingInfo.TVOutput == TV_OUTPUT_YPBPR_COMPOSITE_SVIDEO))
            pBIOSInfo->TVSettingInfo.TVOutput = TV_OUTPUT_NONE;
    }



    /* Flicker Filter: */
    pBIOSInfo->TVSettingInfo.TVFlickerFilter = 0xFFFF;
    if (xf86GetOptValInteger(VIAOptions, OPTION_TVFLICKERFILTER, &(pBIOSInfo->TVSettingInfo.TVFlickerFilter)))
    {
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,
                   "TV Flicker Fliter is %d\n", pBIOSInfo->TVSettingInfo.TVFlickerFilter);
    }

    pBIOSInfo->TVDIPort = VIA_DI_NONE;
    if ((pStr = xf86GetOptValString(VIAOptions, OPTION_TV_PORT)))
    {
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "TV port set to %s.\n", pStr);
        pBIOSInfo->TVDIPort = TranslateDIPortNameToID(pStr);        
    }
}



static void VIAGetOptions_TV2Info(ScrnInfoPtr pScrn)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr pBIOSInfo = pVia->pBIOSInfo;
    char    *pStr = NULL;    


    /* TV2 DotCrawl Enable Option */
    if (xf86ReturnOptValBool(VIAOptions, OPTION_TV2DOTCRAWL, FALSE))
    {
        pBIOSInfo->TVSettingInfo2.TVDotCrawl = TRUE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Option: TV2 DotCrawl Enable\n");
    }
    else
    {
        pBIOSInfo->TVSettingInfo2.TVDotCrawl = FALSE;
    }

    /* TV2 Type: */
    pBIOSInfo->TVSettingInfo2.TVType = TVTYPE_NONE;
    if ((pStr = xf86GetOptValString(VIAOptions, OPTION_TV2TYPE)))
    {
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Option: TV2 Type is %s.\n", pStr);
        
        if (!xf86NameCmp(pStr, "NTSC"))
        {
            pBIOSInfo->TVSettingInfo2.TVType = TVTYPE_NTSC;
        }
        else if(!xf86NameCmp(pStr, "PAL"))
        {
            pBIOSInfo->TVSettingInfo2.TVType = TVTYPE_PAL;
        }
        else if(!xf86NameCmp(pStr, "480P"))
        {
            pBIOSInfo->TVSettingInfo2.TVType = TVTYPE_480P;
        }
        else if(!xf86NameCmp(pStr, "576P"))
        {
            pBIOSInfo->TVSettingInfo2.TVType = TVTYPE_576P;
        }
        else if(!xf86NameCmp(pStr, "720P"))
        {
            pBIOSInfo->TVSettingInfo2.TVType = TVTYPE_720P;                            
        }
        else if(!xf86NameCmp(pStr, "1080I"))
        {            
            pBIOSInfo->TVSettingInfo2.TVType = TVTYPE_1080I;                            
        }
    }


    /* TV2 Progressive Mode Enable Option */
    if (xf86ReturnOptValBool(VIAOptions, OPTION_TV2PROGRESSIVE, FALSE))
    {
        pBIOSInfo->TVSettingInfo2.TVProgressive = TRUE;
        if (pBIOSInfo->TVSettingInfo2.TVType == TVTYPE_NTSC)
        {
            pBIOSInfo->TVSettingInfo2.TVType = TVTYPE_480P;
        }
        else if (pBIOSInfo->TVSettingInfo2.TVType == TVTYPE_PAL)
        {
            pBIOSInfo->TVSettingInfo2.TVType = TVTYPE_576P;
        }

        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Option: TV2 Progressive Mode Enable\n");
    }
    else
    {
        pBIOSInfo->TVSettingInfo2.TVProgressive = FALSE;
    }

    
    /* TV2 out put signal Option */
    pBIOSInfo->TVSettingInfo2.TVOutput = TV_OUTPUT_NONE;
    if ((pStr = xf86GetOptValString(VIAOptions, OPTION_TV2OUTPUT)))
    {
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Option: TV2 Output Signal is %s\n", pStr);
        pBIOSInfo->TVSettingInfo2.TVOutput = VIATranslateTVOutputNameToID(pStr);
    }

    /* TV2 Standard Option: TV2 only could be internal TV, so it only use normal scan. */
    pBIOSInfo->TVSettingInfo2.TVVScan = VIA_TVNORMAL;


    /* For TV2 H-Scaling adjustment: */
    /* We can use this option to specify the number of pixels to adjust each time. */
    pBIOSInfo->TVSettingInfo2.TVHScale = VIA_TVSCALE5;
    if (xf86GetOptValInteger(VIAOptions, OPTION_TV2HSCALE, &(pBIOSInfo->TVSettingInfo2.TVHScale)))
    {
        if ((pBIOSInfo->TVSettingInfo2.TVHScale < VIA_TVSCALE1) ||
            (pBIOSInfo->TVSettingInfo2.TVHScale > VIA_TVSCALE5))
        {
            pBIOSInfo->TVSettingInfo2.TVHScale = VIA_TVSCALE5;
        }
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,
                   "Option: TV2 HScale is %d\n", pBIOSInfo->TVSettingInfo2.TVHScale);
    }
    
    /* For TV2 V-Scaling adjustment: */
    /* We can use this option to specify the number of lines to adjust each time. */
    pBIOSInfo->TVSettingInfo2.TVVScale = VIA_TVSCALE5;
    if (xf86GetOptValInteger(VIAOptions, OPTION_TV2VSCALE, &(pBIOSInfo->TVSettingInfo2.TVVScale)))
    {
        if ((pBIOSInfo->TVSettingInfo2.TVVScale < VIA_TVSCALE1) ||
            (pBIOSInfo->TVSettingInfo2.TVVScale > VIA_TVSCALE5))
        {
            pBIOSInfo->TVSettingInfo2.TVVScale = VIA_TVSCALE5;
        }
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,
                   "Option: TV2 VScale is %d\n", pBIOSInfo->TVSettingInfo2.TVVScale);
    }

    /* Flicker Filter: */
    pBIOSInfo->TVSettingInfo2.TVFlickerFilter = 0xFFFF;
    if (xf86GetOptValInteger(VIAOptions, OPTION_TV2FLICKERFILTER, &(pBIOSInfo->TVSettingInfo2.TVFlickerFilter)))
    {
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,
                   "Option: TV2 Flicker Fliter is %d\n", pBIOSInfo->TVSettingInfo2.TVFlickerFilter);
    }
    
}



static void VIAGetOptions_DVIInfo(ScrnInfoPtr pScrn)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr pBIOSInfo = pVia->pBIOSInfo;
    char    *pStr = NULL;

    /* Initialize DVI panel size: */
    pBIOSInfo->TMDSSettingInfo.DFPSize = M_INVALID;

    /* DVI on EPIA: */
    pBIOSInfo->EPIA_DVI = FALSE;
    if (xf86ReturnOptValBool(VIAOptions, OPTION_EPIA_DVI, FALSE))
    {
         pBIOSInfo->EPIA_DVI = TRUE;
         xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Option: support DVI on EPIA!!\n");
    }

    /* Support DVI port selection */
    pBIOSInfo->IsDemoBoard = TRUE;
    pBIOSInfo->DFPDIPort = VIA_DI_NONE;
    if ((pStr = xf86GetOptValString(VIAOptions, OPTION_DVI_PORT)))
    {
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "DVI port set to %s.\n", pStr);
        
        if (!xf86NameCmp(pStr, "DFP_HIGH"))
        {
            pBIOSInfo->IsDemoBoard = FALSE;
            pBIOSInfo->DFPDIPort = VIA_DI_DFPHIGH;
        }
        else if (!xf86NameCmp(pStr, "DFP_LOW"))
        {
            pBIOSInfo->DFPDIPort = VIA_DI_DFPLOW;
        }
        else if (!xf86NameCmp(pStr, "DVP0"))
        {
            pBIOSInfo->DFPDIPort = VIA_DI_DVP0;
        }
        else if (!xf86NameCmp(pStr, "DVP1"))
        {
            pBIOSInfo->DFPDIPort = VIA_DI_DVP1;
        }
    }

    /* DVI Dual Edge: */
    pBIOSInfo->TMDSSettingInfo.IsDualEdge = FALSE;

    if (xf86ReturnOptValBool(VIAOptions, OPTION_DVI_DUAL_EDGE, FALSE))
    {
        pBIOSInfo->TMDSSettingInfo.IsDualEdge = TRUE;
    }

    /* Bus Width Option */
    pBIOSInfo->TMDSSettingInfo.BusWidth = VIA_DI_12BIT;

    if ((pStr = xf86GetOptValString(VIAOptions, OPTION_DVI_BUS_WIDTH)))
    {
        if (!xf86NameCmp(pStr, "12BIT"))
        {
            pBIOSInfo->TMDSSettingInfo.BusWidth = VIA_DI_12BIT;
            xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "DVI Bus Width is 12BIT\n");
        }
        else if (!xf86NameCmp(pStr, "24BIT"))
        {
            pBIOSInfo->TMDSSettingInfo.BusWidth = VIA_DI_24BIT;
            xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "DVI Bus Width is 24BIT\n");
        }
    }
}


static void VIAParseDPAValues(char *pStr, int *pReturnedBuffer, int BufSize)
{
    char *pToken;
    int i = 0;

    pToken = strtok(pStr, ",");
    
    if (pToken)
    {
        *pReturnedBuffer = atoi(pToken);
        pReturnedBuffer++;
        i++;
        
    }

    for (; i < BufSize; i++)
    {
        pToken = strtok('\0', ",");

        if (pToken)
        {
            *pReturnedBuffer = atoi(pToken);
            pReturnedBuffer++;
        }
    }
}

static void VIAGetOptions_DPASettingInfo(ScrnInfoPtr pScrn)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr pBIOSInfo = pVia->pBIOSInfo;
    char    *pStr = NULL;
    int     TmpBuffer[3];

    pBIOSInfo->UserDPASettingInfo.HasUserSetting_Gfx = FALSE;

    if ((pStr = xf86GetOptValString(VIAOptions, OPTION_DPA_SETTING_DVP0)))
    {
        /* The data we get should be "v1,v2,v3", they are for */
        /* "Clock Polarity/Adjust,Data Driving,Clock Driving". */
        VIAParseDPAValues(pStr, TmpBuffer, 3);
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "DVP0 DPA=%d, DataDriving=%d, ClockDriving=%d.\n", TmpBuffer[0], TmpBuffer[1], TmpBuffer[2]);
        
        pBIOSInfo->UserDPASettingInfo.HasUserSetting_Gfx = TRUE;
        pBIOSInfo->UserDPASettingInfo.GfxDPA.DVP0 = TmpBuffer[0] & 0x0F;
        pBIOSInfo->UserDPASettingInfo.GfxDPA.DVP0DataDri_S = (TmpBuffer[1] & BIT0) << 1;     /* for SR1B[1] */
        pBIOSInfo->UserDPASettingInfo.GfxDPA.DVP0DataDri_S1 = (TmpBuffer[1] & BIT1) << 4;    /* for SR2A[5] */
        pBIOSInfo->UserDPASettingInfo.GfxDPA.DVP0ClockDri_S = (TmpBuffer[2] & BIT0) << 2;    /* for SR1E[2] */
        pBIOSInfo->UserDPASettingInfo.GfxDPA.DVP0ClockDri_S1 = (TmpBuffer[2] & BIT1) << 3;   /* for SR2A[4] */
    }

    if ((pStr = xf86GetOptValString(VIAOptions, OPTION_DPA_SETTING_DVP1)))
    {
        /* The data we get should be "v1,v2,v3", they are for */
        /* "Clock Polarity/Adjust,Data Driving,Clock Driving". */
        VIAParseDPAValues(pStr, TmpBuffer, 3);
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "DVP1 DPA=%d, DataDriving=%d, ClockDriving=%d.\n", TmpBuffer[0], TmpBuffer[1], TmpBuffer[2]);

        pBIOSInfo->UserDPASettingInfo.HasUserSetting_Gfx = TRUE;
        pBIOSInfo->UserDPASettingInfo.GfxDPA.DVP1 = TmpBuffer[0] & 0x0F;
        pBIOSInfo->UserDPASettingInfo.GfxDPA.DVP1Driving = ((TmpBuffer[2] & 0x03) << 2) | (TmpBuffer[1] & 0x03);
    }

    if ((pStr = xf86GetOptValString(VIAOptions, OPTION_DPA_SETTING_DFPHIGH)))
    {
        /* The data we get is for "Clock Polarity/Adjust" */
        VIAParseDPAValues(pStr, TmpBuffer, 1);

        pBIOSInfo->UserDPASettingInfo.HasUserSetting_Gfx = TRUE;
        pBIOSInfo->UserDPASettingInfo.GfxDPA.DFPHigh = TmpBuffer[0] & 0x0F;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "DFPHIGH DPA = %d.\n", pBIOSInfo->UserDPASettingInfo.GfxDPA.DFPHigh);
    }

    if ((pStr = xf86GetOptValString(VIAOptions, OPTION_DPA_SETTING_DFPLOW)))
    {
        /* The data we get is for "Clock Polarity/Adjust" */
        VIAParseDPAValues(pStr, TmpBuffer, 1);

        pBIOSInfo->UserDPASettingInfo.HasUserSetting_Gfx = TRUE;
        pBIOSInfo->UserDPASettingInfo.GfxDPA.DFPLow = TmpBuffer[0] & 0x0F;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "DFPLOW DPA = %d.\n", pBIOSInfo->UserDPASettingInfo.GfxDPA.DFPLow);
    }

    if ((pStr = xf86GetOptValString(VIAOptions, OPTION_DPA_SETTING_VT1636)))
    {
        /* The data we get should be "v1,v2", they are for "CLK_SEL_ST1, CLK_SEL_ST2" */
        VIAParseDPAValues(pStr, TmpBuffer, 2);

        pBIOSInfo->UserDPASettingInfo.HasUserSetting_VT1636 = TRUE;        
        pBIOSInfo->UserDPASettingInfo.VT1636DPA.CLK_SEL_ST1 = TmpBuffer[0] & 0x1F;
        pBIOSInfo->UserDPASettingInfo.VT1636DPA.CLK_SEL_ST2 = TmpBuffer[1] & 0x0F;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "VT1636 DPA: CLK_SEL_ST1=%d, CLK_SEL_ST2=%d.\n", 
            pBIOSInfo->UserDPASettingInfo.VT1636DPA.CLK_SEL_ST1,
            pBIOSInfo->UserDPASettingInfo.VT1636DPA.CLK_SEL_ST2);
    }
    else
    {
        pBIOSInfo->UserDPASettingInfo.HasUserSetting_VT1636 = FALSE;
    }

    if ((pStr = xf86GetOptValString(VIAOptions, OPTION_DPA_SETTING_VT1625)))
    {
        /* The data we get is for VT1625 DPA. */
        VIAParseDPAValues(pStr, TmpBuffer, 1);
        
        pBIOSInfo->UserDPASettingInfo.HasUserSetting_VT1625 = TRUE;        
        pBIOSInfo->UserDPASettingInfo.VT1625DPA = TmpBuffer[0] & 0x1F;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "VT1625 DPA = %d.\n", pBIOSInfo->UserDPASettingInfo.VT1625DPA);

    }
    else
    {
        pBIOSInfo->UserDPASettingInfo.HasUserSetting_VT1625 = FALSE;
    }

    if ((pStr = xf86GetOptValString(VIAOptions, OPTION_DPA_SETTING_VT1623)))
    {
        /* The data we get is for VT1623 DPA. */
        VIAParseDPAValues(pStr, TmpBuffer, 1);

        pBIOSInfo->UserDPASettingInfo.HasUserSetting_VT1623 = TRUE;        
        pBIOSInfo->UserDPASettingInfo.VT1623DPA = TmpBuffer[0] & 0x3F;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "VT1623 DPA = %d.\n", pBIOSInfo->UserDPASettingInfo.VT1623DPA);
    }
    else
    {
        pBIOSInfo->UserDPASettingInfo.HasUserSetting_VT1623 = FALSE;
    }
}


static void VIAGetOptions_RotateInfo(ScrnInfoPtr pScrn)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr pBIOSInfo = pVia->pBIOSInfo;
    LPVIAGRAPHICINFO lpVIAGraphicInfo;        
    CARD32 RotateDegree = VIA_ROTATE_DEGREE_0;
    CARD32 RotateType = VIA_ROTATE_TYPE_SW;
    char* s = NULL;
    
    lpVIAGraphicInfo = &pVia->pVidData->VIAGraphicInfo;
    lpVIAGraphicInfo->duoview = lpVIAGraphicInfo->rotate = 0; /*initilize Xserver rotation status */
    lpVIAGraphicInfo->IsRotationEnabled = FALSE;
    
    /* Initilize */    
    pVia->SWRotate = VIA_ROTATE_DEGREE_0;
    pBIOSInfo->HWRotate = VIA_ROTATE_DEGREE_0;
    pVia->RandRRotation = FALSE;
    pVia->IsHWRotateEnabled = FALSE;
    
    if ((s = xf86GetOptValString(VIAOptions, OPTION_ROTATE)))
    {                    
        if (!xf86NameCmp(s, "CW"))
        {                        
            RotateDegree = VIA_ROTATE_DEGREE_90;                        
            xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Rotating screen clockwise.\n");
        }
        else if(!xf86NameCmp(s, "CCW"))
        {            
            RotateDegree = VIA_ROTATE_DEGREE_270;                        
            xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,  "Rotating screen counter-clockwise. \n");
        }
        else if(!xf86NameCmp(s, "UD"))
        {            
            RotateDegree = VIA_ROTATE_DEGREE_180;                                    
            xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,  "Rotating screen upside-down. \n");
        }        
        else
        {
            xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "\"%s\" is not a valid"
                       "value for Option \"Rotate\"\n", s);
            xf86DrvMsg(pScrn->scrnIndex, X_INFO,
                       "Valid options are \"CW\" or \"CCW\" or \"UD\" \n");
        }        
    }    
        
        
    if ((s = xf86GetOptValString(VIAOptions, OPTION_ROTATE_TYPE)))
    {        
        if (!xf86NameCmp(s, "SW"))
        {
            RotateType = VIA_ROTATE_TYPE_SW;
            xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Use Software Rotation - Acceleration disabled\n");
        }
        else if(!xf86NameCmp(s, "HW"))
        {            
            RotateType = VIA_ROTATE_TYPE_HW;          
            xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Use Hardware Rotation.\n");
        }
        else if(!xf86NameCmp(s, "RandR"))
        {
            RotateType = VIA_ROTATE_TYPE_RANDR;
            xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,  "Rotating screen"
                       "Enable RandR - acceleration disabled\n");
        }
        else
        {
            xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "\"%s\" is not a valid"
                       "value for Option \"RotateType\"\n", s);
            xf86DrvMsg(pScrn->scrnIndex, X_INFO,
                       "Valid options are \"SW\" or \"HW\"  \n");
        }
    }


    if ((RotateDegree != VIA_ROTATE_DEGREE_0) || (RotateType == VIA_ROTATE_TYPE_RANDR))
    {
        switch (RotateType)
        {
            /* accel is disabled below for shadowFB */
            case VIA_ROTATE_TYPE_SW:            
                pVia->shadowFB = TRUE;
                pVia->NoXVideo = TRUE;
                pVia->SWRotate = RotateDegree; 
                lpVIAGraphicInfo->rotate = RotateDegree;            
                break;
                
/*            case VIA_ROTATE_TYPE_HW:
                pVia->IsHWRotateEnabled = TRUE;
                pBIOSInfo->HWRotate = RotateDegree;
                lpVIAGraphicInfo->rotate = RotateDegree;
                break;
  */              
            case VIA_ROTATE_TYPE_RANDR:
                pVia->shadowFB = TRUE;
                pVia->NoXVideo = TRUE;
                pVia->NoAccel = TRUE;
                pVia->hwcursor = FALSE;
                pVia->RandRRotation = TRUE;
                break;
        }

        lpVIAGraphicInfo->IsRotationEnabled = TRUE;
    }

    /* Horace add for video rotation */
    /*[Jack Liu,2006.07.28], OPTION_INVERSE for video use should be instead by option "Rotate".*/
    if (xf86ReturnOptValBool(VIAOptions, OPTION_INVERSE, FALSE))
    {
        lpVIAGraphicInfo->rotate = VIA_ROTATE_DEGREE_180;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Option: Inverse video\n");
    }
}

static void
VIAGetOptions_Video(ScrnInfoPtr pScrn)
{
    int colorKey;
    VIAPtr pVia = VIAPTR(pScrn);
    LPVIAGRAPHICINFO lpVIAGraphicInfo = &pVia->pVidData->VIAGraphicInfo;

    if (xf86GetOptValInteger(VIAOptions, OPTION_VIDEO_COLOR_KEY, &colorKey)) {
	lpVIAGraphicInfo->VideoColorKey = colorKey;
	xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,
		   "video color key set to 0x%x\n", colorKey);
    }
    else {
	lpVIAGraphicInfo->VideoColorKey =
	    (1 << pScrn->offset.red) |
	    (1 << pScrn->offset.green) |
	    (((pScrn->mask.blue >> pScrn->offset.blue) - 1) <<
	     pScrn->offset.blue);
    }
}

static void VIAProbeChipset(ScrnInfoPtr pScrn, EntityInfoPtr pEnt)
{
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIAProbeChipSet!\n"));
    
    VIAPtr pVia;
    VIABIOSInfoPtr pBIOSInfo;
    MessageType     from = X_DEFAULT;

    pVia = VIAPTR(pScrn);
    pBIOSInfo = pVia->pBIOSInfo;
    
    pVia->PciInfo = xf86GetPciInfoForEntity(pEnt->index);
    xf86RegisterResources(pEnt->index, NULL, ResNone);
    /*
    xf86SetOperatingState(RES_SHARED_VGA, pEnt->index, ResUnusedOpr);
    xf86SetOperatingState(resVgaMemShared, pEnt->index, ResDisableOpr);
    */

    if (pEnt->device->chipset && *pEnt->device->chipset)
    {
        pScrn->chipset = pEnt->device->chipset;
        pVia->ChipId = pEnt->device->chipID;
        pVia->Chipset = xf86StringToToken(VIAChipsets, pScrn->chipset);
        from = X_CONFIG;
    }
    else if (pEnt->device->chipID >= 0)
    {
        pVia->ChipId = pEnt->device->chipID;
        pVia->Chipset = LookupChipID(VIAPciChipsets, pVia->ChipId);
        pScrn->chipset = (char *)xf86TokenToString(VIAChipsets,
                                                   pVia->Chipset);
        from = X_CONFIG;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "ChipID override: 0x%04X\n",
                   pEnt->device->chipID);
    }
    else
    {
        from = X_PROBED;
        pVia->ChipId = pVia->PciInfo->chipType;
        pVia->Chipset = LookupChipID(VIAPciChipsets, pVia->ChipId);
        pScrn->chipset = (char *)xf86TokenToString(VIAChipsets,
                                                   pVia->Chipset);
    }

    if (pEnt->device->chipRev >= 0)
    {
        pVia->ChipRev = pEnt->device->chipRev;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "ChipRev override: %d\n",
                   pVia->ChipRev);
    }
    else
    {
        /*pVia->ChipRev = pVia->PciInfo->chipRev;*/
        /* Read PCI bus 0, dev 0, function 0, index 0xF6 to get chip rev. */
        pVia->ChipRev = pciReadByte(pciTag(0, 0, 0), 0xF6);
    }

    pBIOSInfo->Chipset = pVia->Chipset;
    pBIOSInfo->ChipRev = pVia->ChipRev;

    
    xf86DrvMsg(pScrn->scrnIndex, from, "Chipset: \"%s\"\n", pScrn->chipset);

    pVia->PciTag = pciTag(pVia->PciInfo->bus, pVia->PciInfo->device,
                          pVia->PciInfo->func);
    xf86DrvMsg(pScrn->scrnIndex, from, "Chipset Rev.: %d\n", pVia->ChipRev);

    /* VansonWu [2004.4.21] for CLE AX/CX differential behavior.    *
     * Define UniChrome Family Project ID/Revision, for simplify    *
     * Device ID + Revision combination.                            *
     * TODO: Use uni-variable capability flag to replace this.      */
     CARD8  BondingOption;;
    switch (pVia->Chipset)
    {
        case VIA_CLE266:
            if (pVia->ChipRev < 15)
            {
                /*=* CLE266A2 need some patch for TV clock Polarity *=*/
                if (pVia->ChipRev == 2)
                {
                    pBIOSInfo->CLE266A2 = TRUE;
                }
                pBIOSInfo->dwProjectIDRevision = UNICHROME_VT3123AX;
            }
            else
            {
                pBIOSInfo->dwProjectIDRevision = UNICHROME_VT3123CX;
            }
            break;
        case VIA_KM400:
            pBIOSInfo->dwProjectIDRevision = UNICHROME_VT3205;
            break;
        case VIA_K8M800:
            pBIOSInfo->dwProjectIDRevision = UNICHROME_VT3204;
            break;
        case VIA_P4M800PRO:
            pBIOSInfo->dwProjectIDRevision = UNICHROME_VT3314;
            break;
        case VIA_PM800:
            /*=* Check VT3259 lite version, lite version does not support TV and Capture *=*/
            if (pciReadByte(pciTag(0, 0, 2), 0x57) & 0x20)
            {
                pBIOSInfo->dwProjectIDRevision = UNICHROME_VT3259;
            }
            else
            {
                DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_PROBED, "PCI:0,0,2 : 0x57 = 0x%2X !\n", pciReadByte(pciTag(0, 0, 2), 0x57)));
                DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_PROBED, "VT3259 Lite Version Found!!\n"));
                pBIOSInfo->dwProjectIDRevision = UNICHROME_VT3259Lite;
            }
            break;
        case VIA_CX700:
            /*MarkTu [2006.08.15]:add for get 3324A3 revision*/
            outb(VIASR, SR43);
            BondingOption = inb(VIASR + 1);
            if(BondingOption & 0x02)
            {
                pBIOSInfo->dwProjectIDRevision = UNICHROME_VT3324M2;
            }
            else if(BondingOption & 0x40)
            {
                pBIOSInfo->dwProjectIDRevision = UNICHROME_VT3324M;
            }
            else
            {
                pBIOSInfo->dwProjectIDRevision = UNICHROME_VT3324;
            }    
            break;
        case VIA_K8M890:
            pBIOSInfo->dwProjectIDRevision = UNICHROME_VT3336;
            break;
        case VIA_P4M890:
            pBIOSInfo->dwProjectIDRevision = UNICHROME_VT3327;            
            break;
        case VIA_CN750:
            pBIOSInfo->dwProjectIDRevision = UNICHROME_VT3293;
            break;
        case VIA_P4M900:
            pBIOSInfo->dwProjectIDRevision = UNICHROME_VT3364;
            break;
        default:
            xf86DrvMsg(pScrn->scrnIndex, X_ERROR, "VIABIOSInit: Unknown Device ID\n");
            break;
    }
}

static Bool VIAFilterMode(ScrnInfoPtr pScrn)
{
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIAFilterMode!\n"));
    
    ClockRangePtr   clockRanges;
    VIAPtr          pVia;
    int result;
    pVia = VIAPTR(pScrn);

    /* STChen[2007.4.16] Use this function to replace the modeline in the config file. */
    ViaModesAttachHelper(pScrn, pScrn->monitor, ViaSupportModes);
    
     /*
     * Setup the ClockRanges, which describe what clock ranges are available,
     * and what sort of modes they can be used for.
     */

    clockRanges = xnfcalloc(sizeof(*clockRanges), 1);
    clockRanges->next = NULL;
    clockRanges->minClock = 20000;
    clockRanges->maxClock = 270000;

    clockRanges->clockIndex = -1;
    clockRanges->interlaceAllowed = TRUE;
    clockRanges->doubleScanAllowed = FALSE;
    
    /*
     * xf86ValidateModes will check that the mode HTotal and VTotal values
     * don't exceed the chipset's limit if pScrn->maxHValue and
     * pScrn->maxVValue are set.  Since our VIAValidMode() already takes
     * care of this, we don't worry about setting them here.
     */

    pScrn->xInc = 2;
    
    /* Select valid modes from those available */
    result = xf86ValidateModes(pScrn,
                          pScrn->monitor->Modes,    /* List of modes available for the monitor */
                          pScrn->display->modes,    /* List of mode names that the screen is requesting. */
                          clockRanges,              /* list of clock ranges */
                          NULL,                     /* list of line pitches */
                          256,                      /* mini line pitch */
                          3840,                     /* max line pitch edit by cliff for virtual screen*/
                          16 * pScrn->bitsPerPixel, /* pitch inc (bits) */
                          128,                      /* min height */
                          2880,                     /* max height edit by cliff for virtual screen*/
                          pScrn->display->virtualX, /* virtual width */
                          pScrn->display->virtualY, /* virutal height */
                          pVia->videoRambytes,      /* size of video memory */
                          LOOKUP_BEST_REFRESH);     /* lookup mode flags */    

    if (result == -1)
    {
        xf86DrvMsg(pScrn->scrnIndex, X_ERROR, "xf86ValidateModes failure\n");
        VIAFreeRec(pScrn);
        return FALSE;
    }
    
    /* This function deletes modes in the modes field of the ScrnInfoRec that have been marked as invalid. */
    xf86PruneDriverModes(pScrn);

    if (result == 0 || pScrn->modes == NULL)
    {
        xf86DrvMsg(pScrn->scrnIndex, X_ERROR, "No valid modes found\n");
        VIAFreeRec(pScrn);
        return FALSE;
    }

    /* This function fills in the Crtc fields for all the modes in the modes field of the ScrnInfoRec. */
    xf86SetCrtcForModes(pScrn, INTERLACE_HALVE_V);

    /* Set the current mode to the first in the list */
    pScrn->currentMode = pScrn->modes;
    
    /* Print the list of modes being used */    
    xf86PrintModes(pScrn);

    /* Set display resolution */
    xf86SetDpi(pScrn, 0, 0);
           
    return TRUE;
}


static Bool VIAProbeMonitorLimitation(ScrnInfoPtr pScrn)
{
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIAProbeMonitorLimitation!\n"));
    VIAPtr          pVia;
    VIABIOSInfoPtr pBIOSInfo;
    int max_h = 0, max_v;
    int i;
    
    pVia = VIAPTR(pScrn);
    pBIOSInfo = pVia->pBIOSInfo;

    /*[Jack Liu,2006.08.14]: If we set Option "NoDDCValue", then CRT's  MonitorSizeH/MonitorSizeV
        which is gotten from CRT's EDID/DDC is meaningless and should not be used to judge CRT panning
        or not. set these two value bigger than max resolution that we support is safe. now we max support
        1920x1440.*/
    if (pBIOSInfo->NoDDCValue)
    {               
        pBIOSInfo->CRTSettingInfo.MonitorSizeH = 4096;
        pBIOSInfo->CRTSettingInfo.MonitorSizeV = 2048;

        /* Assign a default value to indicate the limitation of monitor.*/
        pScrn->monitor->nHsync = 1;         /* number of hsync. */
        pScrn->monitor->hsync[0].hi = 113;  /* upper bound */
        pScrn->monitor->hsync[0].lo = 30;   /* lower bound */
        
        return TRUE;
    }
    
    if (!xf86LoadSubModule(pScrn, "ddc"))
    {
        VIAFreeRec(pScrn);
        return FALSE;
    }
    else
    {
        xf86MonPtr pMon = NULL;

        xf86LoaderReqSymLists(ddcSymbols, NULL);
        if ((pVia->pVbe) &&
            ((pMon = xf86PrintEDID(vbeDoEDID(pVia->pVbe, NULL))) != NULL))
        {
            pVia->DDC1 = pMon;
            xf86SetDDCproperties(pScrn,pMon);
        }
        else if (!VIAddc2(pScrn->scrnIndex))
        {
            VIAddc1(pScrn->scrnIndex);
        }

        /* Try to get monitor's max size. */
        if (pVia->DDC1)
        {
            for (i = 0; i < 8; i++)
            {
                if (pVia->DDC1->timings2[i].hsize > max_h)
                {
                    max_h = pVia->DDC1->timings2[i].hsize;
                    max_v = pVia->DDC1->timings2[i].vsize;
                }                
            }
            
            /* Correct monitor size to avoid invalid mode. */
            VIACorrectMonitorSize(&max_h, &max_v);
            xf86DrvMsg(pScrn->scrnIndex, X_PROBED, "Max Monitor H =  %d\n",
               max_h);
            xf86DrvMsg(pScrn->scrnIndex, X_PROBED, "Max Monitor V =  %d\n",
               max_v);
            
            pBIOSInfo->CRTSettingInfo.MonitorSizeH = max_h;
            pBIOSInfo->CRTSettingInfo.MonitorSizeV = max_v;
        }
        else
        {
            /* If DDC fail, set a default value as monitor size. */
            pBIOSInfo->CRTSettingInfo.MonitorSizeH = 1024;
            pBIOSInfo->CRTSettingInfo.MonitorSizeV = 768;
        }
    }     
    
    /* Reset HorizSync & VertRefresh Rang Using DDC Value */
    if (pVia->DDC1 && !pBIOSInfo->NoDDCValue)
    {
        int i;
        int h = 0;
        int v = 0;

        for (i = 0; i < DET_TIMINGS; i++)
        {
            if (pVia->DDC1->det_mon[i].type == DS_RANGES)
            {
                pScrn->monitor->hsync[h].lo
                    = pVia->DDC1->det_mon[i].section.ranges.min_h;
                pScrn->monitor->hsync[h++].hi
                    = pVia->DDC1->det_mon[i].section.ranges.max_h;
                pScrn->monitor->vrefresh[v].lo
                    = pVia->DDC1->det_mon[i].section.ranges.min_v;
                pScrn->monitor->vrefresh[v++].hi
                    = pVia->DDC1->det_mon[i].section.ranges.max_v;
                break;
            }
        }
         /*[JackLiu,2006.08.15]:Fix 4.1.0.71 bug: DVI only, 1600*1200*16, VT5859A
            The screen is not full, there is a black border in the below of the screen.
            Reason: Till now, we do not know which device we will light, but we always use
            CRT's EDID/DDC to dertermine whether we can support user's mode set in xorg.conf
            by call xf86ValidateModes later. so by this  logic,  if CRT can not support 1600x1200, 
            DVI only can not support too even DVI has a big 1600x1200 pannel.
            Solution: we give a max hsync value 112.5Khz of 1920x1440 Mode here and make all
            Current mode can pass xf86ValidateModes. when we set mode actually to CRT/DVI/TV/
            LCD later, we will judge max mode each device can support there.
            Note: If larger mode than 1920x1440x75 appears, 113(112.5 should be modified)
        */
        if(h > 0 )
        {
             pScrn->monitor->hsync[h-1].hi
                    = 113; 
         }
        
        pScrn->monitor->nHsync = h;
        pScrn->monitor->nVrefresh = v;
    }
    else
    {
        /* STChen[2006.8.25]: Sometimes we will fail to get CRT EDID,
           just like CRT is not connected. We can't know the limitation of monitor, and
           the screen will be set to an unexpect mode. So we should assign a default value
           to indicate the limitation of monitor.
        */
        pScrn->monitor->nHsync = 1;         /* number of hsync. */
        pScrn->monitor->hsync[0].hi = 113;  /* upper bound */
        pScrn->monitor->hsync[0].lo = 30;   /* lower bound */
    }
    
    return TRUE;
}


static Bool VIAProbeBIOSInfo(ScrnInfoPtr pScrn)
{
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIAProbeBIOSInfo!\n"));
    VIAPtr          pVia;
    VIABIOSInfoPtr pBIOSInfo;
    
    pVia = VIAPTR(pScrn);
    pBIOSInfo = pVia->pBIOSInfo;
    int tmp;

    if (!pVia->IsMAMMEnable)
    {
        /* Get BIOS ver. From BIOS ROM or Function Call */
        if (VIAGetBIOSInfoFromROM(pVia, VIAGETBIOSVER))
        {
            pBIOSInfo->BIOSMajorVersion = (int)pVia->pBIOSVer->MajorVer;
            pBIOSInfo->BIOSMinorVersion = (int)pVia->pBIOSVer->MinorVer;
            
            /* S.T.Chen[2006.6.9]: If we don't assign FORCELCD option, check BIOS type. */
            if(!pBIOSInfo->MobileFamily)
            {
                if (pVia->pBIOSVer->CoreBase & 0x08)        
                {
                    pBIOSInfo->MobileFamily = FALSE;
                    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Desktop BIOS Found!!\n"));
                }
                else
                {
                    pBIOSInfo->MobileFamily = TRUE;
                    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Mobile BIOS Found!!\n"));
                }        
            }
        }
        else
        {
            /* Get BIOS version through function call. */
            tmp = BIOS_GetBIOSVersion(pScrn);
            pBIOSInfo->BIOSMajorVersion = tmp >> 8;
            pBIOSInfo->BIOSMinorVersion = tmp & 0xFF;
        }
        
        DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "BIOS Version is = %x.%x\n",
                        pBIOSInfo->BIOSMajorVersion, pBIOSInfo->BIOSMinorVersion));


        /* Get BIOS date through function call. */
        BIOS_GetBIOSDate(pScrn);

        /* get default TV standard from scratch pad */
        if (pBIOSInfo->TVSettingInfo.TVType == TVTYPE_NONE)
        {
            if (pVia->Chipset == VIA_CLE266 && pVia->ChipRev < 15)
            {
                VGAOUT8(0x3D4, 0x3C);
                tmp = VGAIN8(0x3D5) & 0x0F;
            }
            else
            {
                VGAOUT8(0x3D4, 0x4E);
                tmp = VGAIN8(0x3D5) & 0x0F;
            }
            if (tmp)
            {
                pBIOSInfo->TVSettingInfo.TVType = TVTYPE_PAL;
            }
            else
            {
                pBIOSInfo->TVSettingInfo.TVType = TVTYPE_NTSC;
            }
        }

        if (pBIOSInfo->Hotkey)
        {
            VIABuildModeList(pScrn);
        }
    }
    
    if (!VIAGetBIOSTable(pBIOSInfo))
    {
        VIAFreeRec(pScrn);
        return FALSE;
    }
    return TRUE;
    
}

static void
capinfo_init(VIACAPINFO *pCapInfo) 
{
    memset(pCapInfo,0,sizeof(VIACAPINFO));
    /* Graf [2006.12.06]: It's safe to let below value be Zero */
    /*
    pCapInfo->CapInterVersion=0;
    pCapInfo->NoAutoDetect=FALSE;
    pCapInfo->dwCapPort=0;
    pCapInfo->dwDeinterlaceMode=CAP_MEDIAN;
    pCapInfo->FieldSwap=TRUE;
    pCapInfo->No_HFilter=FALSE;
    pCapInfo->Capture_OverScanOff=TRUE;
    pCapInfo->Tuner=0;
    pCapInfo->TVEncoder=VIA_NONETV;
    pCapInfo->SourceWidth=720;
    pCapInfo->SourceHeight=576;
    pCapInfo->Vdec_Slave_Write=0x40;
    pCapInfo->Tuner_Slave_Write=192;
    pCapInfo->TVEncoder_Slave_Write=0;
    pCapInfo->DataWidth=8;
    pCapInfo->CompositeMode=2;
    pCapInfo->SVideoMode=9;
    pCapInfo->TunerMode=0;
    pCapInfo->Scaler=0; */
    pCapInfo->Brightness=128;
    pCapInfo->Contrast=71;
    pCapInfo->Hue=0;
    pCapInfo->Saturation=64;
    /* pCapInfo->Standard=VIDEO_MODE_NTSC; */
    return;
}


/*
*   STChen[2007.3.13]: If use VIA's ROM, return true.
*/
static Bool
VIACheckROMName(void)
{    
    unsigned char   *pBIOS = NULL, *pRom;
    int             romSize, sum, i;
    
    /* Allocate shadow memory */
    if (!(pBIOS=xcalloc(1, 0x10000)))
    {
        ErrorF("Allocate memory fail !!\n");
        return FALSE;
    }
    /* Read BIOS ROM and check! */
    if (xf86ReadBIOS(0xC0000, 0, pBIOS, 0x10000) != 0x10000)
    {
        xfree(pBIOS);
        ErrorF("Read VGA BIOS image fail !!\n");
    }
    else
    {
        if (*((CARD16 *) pBIOS) != 0xAA55)
        {
            xfree(pBIOS);
            ErrorF("VGA BIOS image is wrong!!\n");
            return FALSE;
        }
        else
        {
            romSize = *((CARD8 *) (pBIOS + VIA_BIOS_SIZE_POS)) * 512;
            pRom = pBIOS;
            sum = 0;

            for (i = 0; i < romSize; i++)
            {
                sum += *pRom++;
            }

            if (((CARD8) sum) != 0)
            {
                xfree(pBIOS);
                ErrorF("VGA BIOS image is wrong!! CheckSum = %x\n", sum);
                return FALSE;
            }
        }
    }
    
    pBIOS += 0x61;  

    if(!memcmp(pBIOS, "VIA", 3))
    {
        return TRUE;
    }
    else
    {
        return FALSE;
    }
}

    
static Bool VIAPreInit(ScrnInfoPtr pScrn, int flags)
{
    EntityInfoPtr   pEnt;
    VIAPtr          pVia;
    VIABIOSInfoPtr  pBIOSInfo;
    MessageType     from = X_DEFAULT;    
    char            *s = NULL;
#ifndef USE_FB
    char            *mod = NULL;
    const char      *reqSym = NULL;
#endif
    LPVIAGRAPHICINFO lpVIAGraphicInfo;

    int iValue;
    
    
    
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIAPreInit\n"));

    
    via_xf86_version_current = xf86GetVersion();
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "XF86_VER=%d\n", via_xf86_version_current));
    

    if (pScrn->numEntities > 1)
        return FALSE;

    if (flags & PROBE_DETECT)
        return FALSE;

    if (!xf86LoadSubModule(pScrn, "vgahw"))
        return FALSE;

    xf86LoaderReqSymLists(vgaHWSymbols, NULL);
    if (!vgaHWGetHWRec(pScrn))
        return FALSE;

#if 0
    /* Here we can alter the number of registers saved and restored by the
     * standard vgaHWSave and Restore routines.
     */
    vgaHWSetRegCounts(pScrn, VGA_NUM_CRTC, VGA_NUM_SEQ, VGA_NUM_GFX, VGA_NUM_ATTR);
#endif

    if (!VIAGetRec(pScrn)) {
        return FALSE;
    }

    pVia = VIAPTR(pScrn);
    pBIOSInfo = pVia->pBIOSInfo;
    pBIOSInfo->s3utility=FALSE;
    pBIOSInfo->FirstTimeToSenseIntegratedTV = TRUE;
    pVia->IsSuspending = FALSE;
    pVia->IsVIAModeInitRunOnceDone = FALSE;
    /*clear the setting of the DISP3DSCALINGCTRLInfo setting*/
    memset(&(pVia->DISP3DSCALCTRLInfo),0,sizeof(VIADISP3DSCALCTRL));    
    
    /*
     *  Add VidData struct allocation & initial codes
     */
    if (!IsVidDataAllocated)
    {
#if 1
        /* Allocate shared memory for VidData */
        pVia->shmid = shmget((key_t)0x1234, sizeof(VIDDATA), 0666 | IPC_CREAT);

        if (pVia->shmid == -1)
        {
            /*There may be undeleted share memory, we should delete it first*/
            /*Or if our share memory is bigger , allocation will fail*/
            pVia->shmid = shmget((key_t)0x1234, 4, 0666 | IPC_CREAT);
            shmctl(pVia->shmid, IPC_RMID, 0);
            pVia->shmid = shmget((key_t)0x1234, sizeof(VIDDATA), 0666 | IPC_CREAT);
            if (pVia->shmid == -1)
            {
                DBG_DD(("shmget failed\n"));
            }
        }

        pVia->pVidData = (PVIDDATA) shmat(pVia->shmid, NULL, 0);

        if (pVia->pVidData == NULL)
        {
            DBG_DD(("shmat failed\n"));
        }

        DBG_DD(("VidData memory attached at %p\n", pVia->pVidData));

        /* pVia->pVidData = (PVIDDATA) xalloc(sizeof(VIDDATA) ); */
        memset(pVia->pVidData, 0, sizeof(VIDDATA));
        IsVidDataAllocated = TRUE;
#endif
        /*pVia->pVidData = (PVIDDATA) xalloc(sizeof(VIDDATA) );*/
        /*memset(pVia->pVidData, 0, sizeof(VIDDATA));*/

    }
    else
    {
        /* for SAMM case - Second display */
        if ( pScrn->scrnIndex == 1 ) 
        {
            ScrnInfoPtr pScrn0 = xf86Screens[0];
            VIAPtr pVia0 = VIAPTR(pScrn0);
            pVia->pVidData = pVia0->pVidData;
        }
    }

    lpVIAGraphicInfo = &pVia->pVidData->VIAGraphicInfo;
    lpVIAGraphicInfo->duoview = lpVIAGraphicInfo->rotate = 0;	/*initilize Xserver rotation status */
    lpVIAGraphicInfo->IsSupportedColorSpaceChange = FALSE;
    pVia->IsSecondary = FALSE;
    pEnt = xf86GetEntityInfo(pScrn->entityList[0]);
    if (pEnt->resources) {
        xfree(pEnt);
        VIAFreeRec(pScrn);
        return FALSE;
    }

    pVia->EntityIndex = pEnt->index;

    if(xf86IsEntityShared(pScrn->entityList[0]))
    {
        if(xf86IsPrimInitDone(pScrn->entityList[0]))
        {
            DevUnion* pPriv;
            VIAEntPtr pVIAEnt;
            VIAPtr pVia1;

            pVia->IsSecondary = TRUE;
            pVia->HasSecondary = TRUE;
            pPriv = xf86GetEntityPrivate(pScrn->entityList[0],
                    gVIAEntityIndex);
            pVIAEnt = pPriv->ptr;
            if (pVIAEnt->BypassSecondary)
            {
		        xfree(pEnt);
		        VIAFreeRec(pScrn);
		        return FALSE;
	        }
            pVIAEnt->pSecondaryScrn = pScrn;
            pVIAEnt->HasSecondary = TRUE;
            pVia1 = VIAPTR(pVIAEnt->pPrimaryScrn);
            pVia1->HasSecondary = TRUE;
            /*[JackLiu,20060725]:  In current duoview flag seting logic(VIACheckIfUseDuoView), 
            SAMM case screen 0 's  pBIOSInfo->DuoView will be set to TRUE, so  set screen0's Duoview 
            to False when  found SAMM here*/
            pVia1->pBIOSInfo->DuoView=FALSE;
            /* VansonWu [2003.9.2] If SAMM, Hot key zoom locked
             * pScrn->zoomLocked = TRUE;*/
            pVIAEnt->pSecondaryScrn->zoomLocked = TRUE;
            pVIAEnt->pPrimaryScrn->zoomLocked = TRUE;
        }
        else
        {
            DevUnion* pPriv;
            VIAEntPtr pVIAEnt;

            xf86SetPrimInitDone(pScrn->entityList[0]);
            pPriv = xf86GetEntityPrivate(pScrn->entityList[0],
                    gVIAEntityIndex);
            pVIAEnt = pPriv->ptr;
            pVIAEnt->pPrimaryScrn = pScrn;
            pVIAEnt->IsDRIEnabled = FALSE;
            pVIAEnt->BypassSecondary = FALSE;
            pVIAEnt->HasSecondary = FALSE;
            pVIAEnt->RestorePrimary = FALSE;
            pVIAEnt->IsSecondaryRestored = FALSE;
        }
    }

    if (flags & PROBE_DETECT) {
        VIAProbeDDC(pScrn, pVia->EntityIndex);
        return TRUE;
    }

    pScrn->monitor = pScrn->confScreen->monitor;

    /*
     * We support depths of 8, 16 and 24.
     * We support bpp of 8, 16, and 32.
     */

    if (!xf86SetDepthBpp(pScrn, 0, 0, 0, Support32bppFb))
    {
        xfree(pEnt);
        VIAFreeRec(pScrn);
        return FALSE;
    }
    else
    {
        switch (pScrn->depth)
        {
        case 8:
        case 16:
        case 24:
        case 32:
            /* OK */
            break;
        default:
            xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
                       "Given depth (%d) is not supported by this driver\n",
                       pScrn->depth);
            xfree(pEnt);
            VIAFreeRec(pScrn);
            return FALSE;
        }
    }

    xf86PrintDepthBpp(pScrn);

    if (pScrn->depth == 32) {
        pScrn->depth = 24;
    }

    if (pScrn->depth > 8)
    {
        rgb zeros = {0, 0, 0};

        if (!xf86SetWeight(pScrn, zeros, zeros))
        {
            xfree(pEnt);
            VIAFreeRec(pScrn);
            return FALSE;
        } 
        else
        {
            /* TODO check weight returned is supported */
            ;
        }
    }

    if (!xf86SetDefaultVisual(pScrn, -1)) {
        return FALSE;
    }
    else
    {
        /* We don't currently support DirectColor at > 8bpp */
        if (pScrn->depth > 8 && pScrn->defaultVisual != TrueColor)
        {
            xf86DrvMsg(pScrn->scrnIndex, X_ERROR, "Given default visual"
                       " (%s) is not supported at depth %d\n",
                       xf86GetVisualName(pScrn->defaultVisual), pScrn->depth);
            xfree(pEnt);
            VIAFreeRec(pScrn);
            return FALSE;
        }
    }

    /* We use a programmable clock */
    pScrn->progClock = TRUE;

    /* Check if VIA's ROM. If not, it imply that we are under MAMM. */
    if(VIACheckROMName())
    {        
        pVia->IsMAMMEnable = FALSE;
        pBIOSInfo->IsMAMMEnable = FALSE;
    }
    else
    {        
        pVia->IsMAMMEnable = TRUE;   
        pBIOSInfo->IsMAMMEnable = TRUE;
    }

    xf86CollectOptions(pScrn, NULL);

    /* Set the bits per RGB for 8bpp mode */
    if (pScrn->depth == 8)
        pScrn->rgbBits = 8;

    xf86ProcessOptions(pScrn->scrnIndex, pScrn->options, VIAOptions);    

#ifdef XF86DRI
    pVia->drixinerama = FALSE;
    if (xf86IsOptionSet(VIAOptions, OPTION_DRIXINERAMA))
        pVia->drixinerama = TRUE;
#else
    if (xf86IsOptionSet(VIAOptions, OPTION_DRIXINERAMA))
    	xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,
    		"Option: drixinerama ignored, no DRI support compiled into driver.\n");
#endif

    if (xf86ReturnOptValBool(VIAOptions, OPTION_PCI_BURST, FALSE)) {
        pVia->pci_burst = TRUE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,
                   "Option: pci_burst - PCI burst read enabled\n");
    }
    else {
        pVia->pci_burst = FALSE;
    }

    pVia->NoPCIRetry = 1;       /* default */
    if (xf86ReturnOptValBool(VIAOptions, OPTION_PCI_RETRY, FALSE)) {
        if (xf86ReturnOptValBool(VIAOptions, OPTION_PCI_BURST, FALSE)) {
            pVia->NoPCIRetry = 0;
            xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Option: pci_retry\n");
        }
        else {
            xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,
                       "\"pci_retry\" option requires \"pci_burst\"\n");
        }
    }

    if (xf86IsOptionSet(VIAOptions, OPTION_SHADOW_FB))
    {
        pVia->shadowFB = TRUE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Option: ShadowFB %s.\n",
                   pVia->shadowFB ? "enabled" : "disabled");
    }
    else
    {
        pVia->shadowFB = FALSE;
    }

    /* STChen[2007.1.3]: Get rotate option. */
    VIAGetOptions_RotateInfo(pScrn);   

    if (xf86ReturnOptValBool(VIAOptions, OPTION_NOACCEL, FALSE))
    {
        pVia->NoAccel = TRUE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,
                   "Option: NoAccel -Acceleration Disabled\n");
    }
    else
    {
        pVia->NoAccel = FALSE;
    }

    if (pVia->shadowFB && !pVia->NoAccel)
    {
        xf86DrvMsg(pScrn->scrnIndex, X_WARNING,
                   "HW acceleration not supported with \"shadowFB\".\n");
        pVia->NoAccel = TRUE;
    }    

/* Initialize the DISPLAY 3D scaling CAP */
    pVia->DISP3DScaling = FALSE;
    if (xf86IsOptionSet(VIAOptions, OPTION_DISP3DScaling))
    {
        pVia->DISP3DScaling = TRUE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Option: Display SW scaling %s.\n",
                   pVia->DISP3DScaling ? "enabled" : "disabled");
    }
    else
    {
        pVia->DISP3DScaling = FALSE;
    }
//    pVia->DISP3DScaling = TRUE;

    
    /*cliff add to get mergedfb options*/
    if (!pVia->IsSecondary)
        VIAGetOptions_MergedFB(pScrn);

    from = X_DEFAULT;
    if (xf86ReturnOptValBool(VIAOptions, OPTION_USEBIOS, FALSE)) {
        from = X_CONFIG;
        pBIOSInfo->UseBIOS = TRUE;
    }
    else {
        pBIOSInfo->UseBIOS = FALSE;
    }
    xf86DrvMsg(pScrn->scrnIndex, from, "%ssing video BIOS to set modes\n",
               pBIOSInfo->UseBIOS ? "U" : "Not u" );

    pScrn->videoRam = 0;
    if(xf86GetOptValInteger(VIAOptions, OPTION_VIDEORAM, &pScrn->videoRam)) {
        xf86DrvMsg( pScrn->scrnIndex, X_CONFIG,
                    "Option: VideoRAM %dkB\n", pScrn->videoRam );
    }

    if (xf86ReturnOptValBool(VIAOptions, OPTION_DISABLEVQ, FALSE)) {
        pVia->VQEnable = FALSE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,
                   "Option: DisableVQ -VQ Disabled\n");
    }
    else {
        pVia->VQEnable = TRUE;
    }

    if (!pVia->IsMAMMEnable)
    {
        if (xf86LoadSubModule(pScrn, "int10"))
        {
            xf86LoaderReqSymLists(int10Symbols, NULL);
            pVia->pInt10 = xf86InitInt10(pEnt->index);
        }
    }
 
    if (!pVia->IsSecondary)
    {
        VIARegPtr       save;

        pVia->PciInfo = xf86GetPciInfoForEntity(pEnt->index);
        pVia->MmioBase = pVia->PciInfo->memBase[1];
        pVia->MapBase = xf86MapPciMem(pScrn->scrnIndex, VIDMEM_MMIO, pVia->PciTag,
                                  pVia->MmioBase, VIA_MMIO_REGSIZE);
        xf86DrvMsg(pScrn->scrnIndex, X_INFO, "MapBase = 0x%p, MmioBase = 0x%lx\n",
		   pVia->MapBase, pVia->MmioBase);

	save = &pVia->SavedReg; 
        /* Unlock Extended Regs */
       VGAOUT8(0x3c4, 0x10);
       VGAOUT8(0x3c5, VGAIN8(0x3c5) | 0x01);
       /* Unlock CRTC register protect */
       VGAOUT8(0x3d4, 0x47);
       VGAOUT8(0x3d5, VGAIN8(0x3d5) & ~0x01);
       VGAOUT8(0x3d4, CR08);
       save->CRTCRegs[0x08] = VGAIN8(0x3d5);
       VGAOUT8(0x3d4, CR09);
       save->CRTCRegs[0x09] = VGAIN8(0x3d5);
       xf86DrvMsg(pScrn->scrnIndex, X_INFO, "RegCR08 = %x, RegCR09=%x\n",
            save->CRTCRegs[0x08], save->CRTCRegs[0x09]);

        /* RainLin [2007.01.30]: To fix the problem: In SAMM, the system will hang when suspends. (On VT3364) */
        /* In VIAEnableMMIO(), it will enable software reset of SR1A, so the value of SR1A we saved */
        /* is not the original value set by BIOS, and we found that it will cause system hang when */
        /* restore registers in VIALeaveVT(). In order to keep the original value set by BIOS, we should */
        /* save SR1A before doing VIAEnableMMIO() to avoid this problem. So, we move the saving code from */
        /* VIASave() to here.*/
        VGAOUT8(0x3C4, SR1A);
        save->SRRegs[0x1A] = VGAIN8(0x3C5);
        
       xf86UnMapVidMem(pScrn->scrnIndex, (pointer)pVia->MapBase,
                        VIA_MMIO_REGSIZE);	
    }

    if (!pVia->IsMAMMEnable) /* STChen[2007.3.8]: For MAMM */
    {
        if (pVia->pInt10 && xf86LoadSubModule(pScrn, "vbe"))
        {
            
            if (via_xf86_version_current < XF86_VERSION_NUMERIC(4,2,99,0,0))
            {
                xf86LoaderReqSymLists(vbeSymbols, NULL);
                pVia->pVbe = VBEInit(pVia->pInt10, pVia->EntityIndex);
            }    
            else
            {
                xf86LoaderReqSymLists(vbeSymbols429900, NULL);
                pVia->pVbe = VBEExtendedInit(pVia->pInt10, pVia->EntityIndex,0);
            }   
            
        }
    }
    
    if (pEnt->device->videoRam != 0)
    {
        if (!pScrn->videoRam)
        {
            pScrn->videoRam = pEnt->device->videoRam;
        }
        else
        {
            xf86DrvMsg(pScrn->scrnIndex, X_WARNING,
                      "Video Memory Size in Option is %d KB, Detect is %d KB!",
            pScrn->videoRam, pEnt->device->videoRam);
        }
    }

    /* Specified a number to replace the original driver version patch level. */
    pBIOSInfo->DriverVersionPatchLevel = 0;
    if(xf86GetOptValInteger(VIAOptions, OPTION_DRIVER_VERSION, &pBIOSInfo->DriverVersionPatchLevel))
    {
        xf86DrvMsg( pScrn->scrnIndex, X_CONFIG, "Option: Driver Version = %d\n", pBIOSInfo->DriverVersionPatchLevel);
    }
    
    /* STChen[2006.6.27]: Detect which chipset we used. */
    VIAProbeChipset(pScrn, pEnt);

    /* STChen[2007.1.4]: Probe the capabilities about rotation. */
    ProbeRotateCaps(pScrn);

    VIAGetOptions_ActiveDevice(pScrn);
    VIAGetOptions_DisplayHardwareLayout(pScrn);
    VIAGetOptions_LCDInfo(pScrn);
    VIAGetOptions_TVInfo(pScrn);    
    VIAGetOptions_DVIInfo(pScrn);
    VIAGetOptions_DPASettingInfo(pScrn);

    /* For CX700 InTV + ExtTV */
    if (pBIOSInfo->DisplayHardwareLayout == HW_LAYOUT_TV_EXTERNAL_TV2)
    {
        VIAGetOptions_TV2Info(pScrn);
    }
    
    /*cliff :here add this function because  chipset id is identified just above*/ 
    VIAGetOptions_Cursor(pScrn);

    VIAGetOptions_Video(pScrn);

    /* NoDDCValue Option */
    if (xf86ReturnOptValBool(VIAOptions, OPTION_NODDCVALUE, FALSE))
    {        
        pBIOSInfo->NoDDCValue = TRUE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,
        "Option: Not using DDC probed value to set HorizSync & VertRefresh\n");
    }
    else
    {       
        pBIOSInfo->NoDDCValue = FALSE;
    }

    if(pBIOSInfo->TwoLCD)
    {
        /*=* For Advantech Project, support two LCD MHS.
             Always set Refresh Rate to 60. *=*/
        pBIOSInfo->OptRefresh = 60;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Refresh is 60.\n");
    }
    else
    {
        pBIOSInfo->OptRefresh = 0;
        if (xf86GetOptValInteger(VIAOptions, OPTION_REFRESH,
                                 &(pBIOSInfo->OptRefresh)))
        {
            xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Refresh is %d\n",
                       pBIOSInfo->OptRefresh);
        }
    }

    /* Device Switch Hotkey Option */
    if (xf86ReturnOptValBool(VIAOptions, OPTION_DEVICE_SWITCH_HOTKEY, FALSE))
    {
        pBIOSInfo->Hotkey = TRUE;
        pScrn->zoomLocked = TRUE;           /* Locked Zoom Hotkey */
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,
                   "Option: Using Device Switch Hotkey\n");
    }
    else
    {
        pBIOSInfo->Hotkey = FALSE;
    }

    xfree(pEnt);    

    if (!VIAMapMMIO(pScrn))
    {
        vbeFree(pVia->pVbe);
        return FALSE;
    }

    /* RainLin [2006.01.23]: Move the initialization from VIAScreenInit() to here. And before */
    /* initialize, we'd better save the original value of every integrated TV register first */
    /* or it may effects the settings in console mode. */
    if (((pVia->Chipset == VIA_CX700)||(pVia->Chipset == VIA_CN750)) && (!pVia->IsSecondary))
    {        
        SaveTVRegs_IntegratedTV(pBIOSInfo, &pVia->SavedReg);
        InitIntegratedTVRegs(pScrn);
    }

    /* Get some useful information(BIOS version, date, mobile flag)
    *  through BIOS.(ROM or Function call)     
    */
    if (!VIAProbeBIOSInfo(pScrn))
    {
        return FALSE;
    }

    /* Initialize the colormap and this step must be done before calling xf86HandleColormaps. */
    Gamma zeros = {0.0, 0.0, 0.0};
    if (!xf86SetGamma(pScrn, zeros))
    {
        VIAFreeRec(pScrn);
        vbeFree(pVia->pVbe);
        return FALSE;
    }
    

    /* S.T.Chen [2005.11.03]: Read PCI configuration space to get frame buffer size. */
    pScrn->videoRam = VIAGetFBSize(pScrn);

    /* Split FB for SAMM */
    /* FIXME: For now, split FB into two equal sections. This should
     *        be able to be adjusted by user with a config option. */
    if (pVia->IsSecondary)
    {
        DevUnion* pPriv;
        VIAEntPtr pVIAEnt;
        VIAPtr    pVia1;
        int	videoram1=0;

        pPriv = xf86GetEntityPrivate(pScrn->entityList[0],gVIAEntityIndex);
        pVIAEnt = pPriv->ptr;

        if(xf86GetOptValInteger(VIAOptions, OPTION_VIDEORAM1, &videoram1))
        {
            xf86DrvMsg( pScrn->scrnIndex, X_CONFIG,
                        "Option: VideoRAM1 %dkB\n", videoram1 );
        }
        
        if(videoram1 > 0)
            videoram1 = (videoram1 >> 10) << 10;
  
        if(videoram1 >= pScrn->videoRam - 16384)
            videoram1 = pScrn->videoRam - 16384;

        if((videoram1 <= 16384) && (videoram1 > 0))
            videoram1 = 16384;

        if(videoram1)
            pScrn->videoRam = pScrn->videoRam - videoram1;	
        else
            pScrn->videoRam = pScrn->videoRam >> 1;
        
        pVIAEnt->pPrimaryScrn->videoRam = pScrn->videoRam;
        pVia1 = VIAPTR(pVIAEnt->pPrimaryScrn);
        pVia1->videoRambytes = pScrn->videoRam << 10;
        pVia->FrameBufferBase += (pScrn->videoRam << 10);
        
        if(videoram1)	  
            pScrn->videoRam = videoram1;
    }

    pVia->videoRambytes = pScrn->videoRam << 10;
    xf86DrvMsg(pScrn->scrnIndex, X_PROBED, "videoram =  %dk\n",
               pScrn->videoRam);

    /* Set status word positions based on chip type. */

    switch (pVia->Chipset)
    {
        case VIA_P4M890:    
        case VIA_K8M890:
        case VIA_CN750:
        case VIA_P4M900:
            pVia->myWaitIdle = WaitIdleK8M890;
            break;

        default:
            pVia->myWaitIdle = WaitIdleCLE266;
            break;
    }

    /* Before using DDC, we must initialize the I2C. */
    if (!xf86LoadSubModule(pScrn, "i2c"))
    {
        VIAFreeRec(pScrn);
        return FALSE;
    }
    else
    {
        xf86LoaderReqSymLists(i2cSymbols,NULL);
        VIAI2CInit(pScrn);
    }

    /* Use DDC to detect the capability limitation of CRT monitor. */
    if (!VIAProbeMonitorLimitation(pScrn))
    {
        return FALSE;
    }
    
    /* Set up screen parameters. */
    pVia->Bpp = pScrn->bitsPerPixel >> 3;
    pVia->Bpl = pScrn->displayWidth * pVia->Bpp;

    if (!VIAFilterMode(pScrn))
    {
        return FALSE;
    }

    /* After calling xf86ValidateModes(), pScrn->virtualX(Y) will get the final value of virtual size. */
    pBIOSInfo->VirtualX = pScrn->virtualX;
    pBIOSInfo->VirtualY = pScrn->virtualY;

    if (pBIOSInfo->MergedFB)
    {
        /*add by cliff to set virtul X,Y value while mergedfb*/
        if((pBIOSInfo->Scrn2Position==viaRightOf)||(pBIOSInfo->Scrn2Position==viaLeftOf))
        {
            pScrn->virtualX += pScrn->virtualX;
        }

        if((pBIOSInfo->Scrn2Position==viaAbove)||(pBIOSInfo->Scrn2Position==viaBelow))
        {
            pScrn->virtualY += pScrn->virtualY;

        }
        pScrn->displayWidth = pScrn->virtualX; 
    }

    /* Before we detect TV encoder, LVDS and TMDS chip, we have to assign I2C port and DDC.*/
    pBIOSInfo->I2C_Port1 = pVia->I2C_Port1;
    pBIOSInfo->I2C_Port2 = pVia->I2C_Port2;
    pBIOSInfo->DDC1 = pVia->DDC1;
    pBIOSInfo->DDC2 = pVia->DDC2;

    /* Detect and determine TV Encoder */
    if (!pBIOSInfo->TVSettingInfo.TVEncoder)
    {
        /* RainLin [2005.12.22]: For HW strapping issue. */
        InitTVChipInfo(pBIOSInfo);
    }

    /* Detect TMDS Transmiter */
    if (VIATMDSIdentify(pBIOSInfo))
    {
        if (pBIOSInfo->NoDDCValue)
        {            
            pBIOSInfo->TMDSSettingInfo.DFPSize = VIAGetModeIndex(pScrn->virtualX, pScrn->virtualY);
            VIAGetModeSizeByIndex(pBIOSInfo->TMDSSettingInfo.DFPSize, &pBIOSInfo->TMDSSettingInfo.PanelSizeH, &pBIOSInfo->TMDSSettingInfo.PanelSizeV);                        
        }
        else
        {
            VIAGetPanelInfo(pBIOSInfo);
        }
    }

    VIAPreInit_LVDS(pBIOSInfo);

    /* Detect LVDS Transmiter */
    if (pVia->Chipset == VIA_CLE266)
    {
        /* For CLE266 device work on AMR slot */
        VIAPostLCDNotDVO(pBIOSInfo);
    }
    else
    {
        /* For KM400 or later device work on AGP slot */
        VIAPostLCDDVO(pBIOSInfo);
    }    

    pBIOSInfo->ConnectedDevice = VIAGetDeviceDetect(pBIOSInfo);
    
    VIADeviceSelection(pScrn);

    VIASetDisplayPath(pScrn);
    
    /* RainLin [2006.03.23]: Check what type of cursor (SW/HW) to use. */
    VIACheckCursorTypeToUse(pScrn);    

#ifdef USE_FB
    if (xf86LoadSubModule(pScrn, "fb") == NULL)
    {
        xf86DrvMsg(pScrn->scrnIndex, X_ERROR, "Failed to load fb module");
        VIAFreeRec(pScrn);
        return FALSE;
    }

    xf86LoaderReqSymLists(fbSymbols, NULL);

#else
    /* load bpp-specific modules */
    switch (pScrn->bitsPerPixel)
    {
        case 8:
            mod = "cfb";
            reqSym = "cfbScreenInit";
            break;
        case 16:
            mod = "cfb16";
            reqSym = "cfb16ScreenInit";
            break;
        case 32:
            mod = "cfb32";
            reqSym = "cfb32ScreenInit";
            break;
    }

    if (mod && xf86LoadSubModule(pScrn, mod) == NULL)
    {
        VIAFreeRec(pScrn);
        return FALSE;
    }

    xf86LoaderReqSymbols(reqSym, NULL);
#endif

    if (!pVia->NoAccel)
    {
        if(!xf86LoadSubModule(pScrn, "xaa"))
        {
            VIAFreeRec(pScrn);
            return FALSE;
        }
        xf86LoaderReqSymLists(xaaSymbols, NULL);
    }

    if (!pVia->ForceSWCursor)
    {
        if (!xf86LoadSubModule(pScrn, "ramdac"))
        {
            VIAFreeRec(pScrn);
            return FALSE;
        }
        xf86LoaderReqSymLists(ramdacSymbols, NULL);
    }

    if (pVia->shadowFB)
    {
        if (!xf86LoadSubModule(pScrn, "shadowfb"))
        {
            VIAFreeRec(pScrn);
            return FALSE;
        }
        xf86LoaderReqSymLists(shadowSymbols, NULL);
    }

    /* Capture option parameter */
    /* initial CapInfo */
    capinfo_init(&(pVia->CapInfo[0]));
    capinfo_init(&(pVia->CapInfo[1]));
    /* Set capture source width and height */
    if (xf86GetOptValInteger(VIAOptions, OPTION_Cap0SourceWidth, &(pVia->CapInfo[0].SourceWidth))) {
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Cap0 Source Width: %d\n", pVia->CapInfo[0].SourceWidth);
    }
    if (xf86GetOptValInteger(VIAOptions, OPTION_Cap0SourceHeight, &(pVia->CapInfo[0].SourceHeight))) {
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Cap0 Source Height: %d\n", pVia->CapInfo[0].SourceHeight);
    }

    if (xf86GetOptValInteger(VIAOptions, OPTION_Cap1SourceWidth, &(pVia->CapInfo[1].SourceWidth))) {
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Cap1 Source Width: %d\n", pVia->CapInfo[1].SourceWidth);
    }
    if (xf86GetOptValInteger(VIAOptions, OPTION_Cap1SourceHeight, &(pVia->CapInfo[1].SourceHeight))) {
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Cap1 Source Height: %d\n", pVia->CapInfo[1].SourceHeight);
    }

    /*Set Capture using CCIR601/CCIR656 format*/
    if (xf86ReturnOptValBool(VIAOptions, OPTION_CCIR601, FALSE)) {
        lpVIAGraphicInfo->bCCIR601 = TRUE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,"Capture supports CCIR601 Format!!");
    }

    /* Get capture auto detection value */
    if (xf86ReturnOptValBool(VIAOptions, OPTION_Cap0NoAutoDetect, FALSE)) {
        pVia->CapInfo[0].NoAutoDetect = TRUE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,
                   "Option: Cap0 auto detect= %x\n", pVia->CapInfo[0].NoAutoDetect);
    }
    else {
        pVia->CapInfo[0].NoAutoDetect = FALSE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,
                   "Option: Cap0 auto detect= %x\n", pVia->CapInfo[0].NoAutoDetect);
    }

    if (xf86ReturnOptValBool(VIAOptions, OPTION_Cap1NoAutoDetect, FALSE)) {
        pVia->CapInfo[1].NoAutoDetect = TRUE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,
                   "Option: Cap1 auto detect= %x\n", pVia->CapInfo[1].NoAutoDetect);
    }
    else {
        pVia->CapInfo[1].NoAutoDetect = FALSE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,
                   "Option: Cap1 auto detect= %x\n", pVia->CapInfo[1].NoAutoDetect);
    }

    /* Get capture internal version */
    if (xf86GetOptValInteger(VIAOptions, OPTION_CapInterVersion, &(pVia->CapInfo[0].CapInterVersion))) {
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Capture internal version is %d\n", pVia->CapInfo[0].CapInterVersion);
        pVia->CapInfo[1].CapInterVersion = pVia->CapInfo[0].CapInterVersion;
    }

    /* Get capture output data width */
    if (xf86GetOptValInteger(VIAOptions, OPTION_Cap0DataWidth, &(pVia->CapInfo[0].DataWidth))) {
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Capture 0 data width is %d\n", pVia->CapInfo[0].DataWidth);
    }

    if (xf86GetOptValInteger(VIAOptions, OPTION_Cap1DataWidth, &(pVia->CapInfo[1].DataWidth))) {
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Capture 1 data width is %d\n", pVia->CapInfo[1].DataWidth);
    }

    /* Get capture video decoder device */
    if ((s = xf86GetOptValString(VIAOptions, OPTION_Cap0VDODecoderDev))) {
            pVia->CapInfo[0].VideoDecoder = CAP_NONE;
            xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "No Cap0 VideoDecoder\n");
    }

    if ((s = xf86GetOptValString(VIAOptions, OPTION_Cap1VDODecoderDev))) {
            pVia->CapInfo[1].VideoDecoder = CAP_NONE;
            xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "No Cap1 VideoDecoder\n");
    }

    /* Get capture video decoder device i2c slave write address */
    if (xf86GetOptValInteger(VIAOptions, OPTION_Cap0VDODecoderI2C, &(pVia->CapInfo[0].Vdec_Slave_Write))) {
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Cap0 VideoDecoder slave address: %d\n", pVia->CapInfo[0].Vdec_Slave_Write);
    }

    if (xf86GetOptValInteger(VIAOptions, OPTION_Cap1VDODecoderI2C, &(pVia->CapInfo[1].Vdec_Slave_Write))) {
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Cap1 VideoDecoder slave address: %d\n", pVia->CapInfo[1].Vdec_Slave_Write);
    }

    /* Get capture video decoder composite mode, ref the video decoder spec */
    if (xf86GetOptValInteger(VIAOptions, OPTION_Cap0CompositeMode, &(pVia->CapInfo[0].CompositeMode))) {
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Cap0 VideoDecoder Composite mode: %d\n", pVia->CapInfo[0].CompositeMode);
    }

    if (xf86GetOptValInteger(VIAOptions, OPTION_Cap1CompositeMode, &(pVia->CapInfo[1].CompositeMode))) {
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Cap1 VideoDecoder Composite mode: %d\n", pVia->CapInfo[1].CompositeMode);
    }

    /* Get capture video decoder SVideo mode, ref the video decoder spec */
    if (xf86GetOptValInteger(VIAOptions, OPTION_Cap0SVideoMode, &(pVia->CapInfo[0].SVideoMode))) {
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Cap0 VideoDecoder SVideo mode: %d\n", pVia->CapInfo[0].SVideoMode);
    }

    if (xf86GetOptValInteger(VIAOptions, OPTION_Cap1SVideoMode, &(pVia->CapInfo[1].SVideoMode))) {
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Cap1 VideoDecoder SVideo mode: %d\n", pVia->CapInfo[1].SVideoMode);
    }

    /* Get capture video decoder tuner mode, ref the video decoder spec */
    if (xf86GetOptValInteger(VIAOptions, OPTION_Cap0TunerMode, &(pVia->CapInfo[0].TunerMode))) {
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Cap0 VideoDecoder tuner mode: %d\n", pVia->CapInfo[0].TunerMode);
    }

    if (xf86GetOptValInteger(VIAOptions, OPTION_Cap1TunerMode, &(pVia->CapInfo[1].TunerMode))) {
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Cap1 VideoDecoder tuner mode: %d\n", pVia->CapInfo[1].TunerMode);
    }

    /* Get capture video decoder Scaler mode: 1 = use scaler, 0 = not use scaler
     * Not all of the video decoder has scaler function
     */
    if (xf86GetOptValInteger(VIAOptions, OPTION_Cap0Scaler, &(pVia->CapInfo[0].Scaler))) {
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Cap0 VideoDecoder Scaler: %d\n", pVia->CapInfo[0].Scaler);
    }

    if (xf86GetOptValInteger(VIAOptions, OPTION_Cap1Scaler, &(pVia->CapInfo[1].Scaler))) {
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Cap1 VideoDecoder Scaler: %d\n", pVia->CapInfo[1].Scaler);
    }

    /* Get capture video decoder Brightness */
    if (xf86GetOptValInteger(VIAOptions, OPTION_Cap0Brightness, &(pVia->CapInfo[0].Brightness))) {
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Cap0 VideoDecoder Brightness: %d\n", pVia->CapInfo[0].Brightness);
    }

    if (xf86GetOptValInteger(VIAOptions, OPTION_Cap1Brightness, &(pVia->CapInfo[1].Brightness))) {
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Cap1 VideoDecoder Brightness: %d\n", pVia->CapInfo[1].Brightness);
    }

    /* Get capture video decoder Contrast */
    if (xf86GetOptValInteger(VIAOptions, OPTION_Cap0Contrast, &(pVia->CapInfo[0].Contrast))) {
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Cap0 VideoDecoder Contrast: %d\n", pVia->CapInfo[0].Contrast);
    }

    if (xf86GetOptValInteger(VIAOptions, OPTION_Cap1Contrast, &(pVia->CapInfo[1].Contrast))) {
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Cap1 VideoDecoder Contrast: %d\n", pVia->CapInfo[1].Contrast);
    }

    /* Get capture video decoder Hue */
    if (xf86GetOptValInteger(VIAOptions, OPTION_Cap0Hue, &(pVia->CapInfo[0].Hue))) {
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Cap0 VideoDecoder Hue: %d\n", pVia->CapInfo[0].Hue);
    }

    if (xf86GetOptValInteger(VIAOptions, OPTION_Cap1Hue, &(pVia->CapInfo[1].Hue))) {
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Cap1 VideoDecoder Hue: %d\n", pVia->CapInfo[1].Hue);
    }

    /* Get capture video decoder Saturation */
    if (xf86GetOptValInteger(VIAOptions, OPTION_Cap0Saturation, &(pVia->CapInfo[0].Saturation))) {
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Cap0 VideoDecoder Saturation: %d\n", pVia->CapInfo[0].Saturation);
    }

    if (xf86GetOptValInteger(VIAOptions, OPTION_Cap0Saturation, &(pVia->CapInfo[1].Saturation))) {
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Cap1 VideoDecoder Saturation: %d\n", pVia->CapInfo[1].Saturation);
    }

    /* Get capture video decoder Standard: NTSC/PAL/SECAM */
    if ((s = xf86GetOptValString(VIAOptions, OPTION_Cap0Standard))) {
        if (!xf86NameCmp(s, "PAL")) {
            pVia->CapInfo[0].Standard = VIDEO_MODE_PAL;
            xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Cap0 VideoDecoder Standard: PAL\n");
        }
        else if(!xf86NameCmp(s, "NTSC")) {
            pVia->CapInfo[0].Standard = VIDEO_MODE_NTSC;
            xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Cap0 VideoDecoder Standard: NTSC\n");
        }
        else if(!xf86NameCmp(s, "SECAM")) {
            pVia->CapInfo[0].Standard = VIDEO_MODE_SECAM;
            xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Cap0 VideoDecoder Standard: SECAM\n");
        }
    }

    if ((s = xf86GetOptValString(VIAOptions, OPTION_Cap1Standard))) {
        if (!xf86NameCmp(s, "PAL")) {
            pVia->CapInfo[1].Standard = VIDEO_MODE_PAL;
            xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Cap1 VideoDecoder Standard: PAL\n");
        }
        else if(!xf86NameCmp(s, "NTSC")) {
            pVia->CapInfo[1].Standard = VIDEO_MODE_NTSC;
            xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Cap1 VideoDecoder Standard: NTSC\n");
        }
        else if(!xf86NameCmp(s, "SECAM")) {
            pVia->CapInfo[1].Standard = VIDEO_MODE_SECAM;
            xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Cap1 VideoDecoder Standard: SECAM\n");
        }
    }

    /* Get capture tuner device */
    if ((s = xf86GetOptValString(VIAOptions, OPTION_Cap0TunerDev))) {
        if (!xf86NameCmp(s, "FI1236MK2")) {
            pVia->CapInfo[0].Tuner = FI1236MK2;
            xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Cap0 Tuner: FI1236MK2\n");
        }
    }

    if ((s = xf86GetOptValString(VIAOptions, OPTION_Cap1TunerDev))) {
        if (!xf86NameCmp(s, "FI1236MK2")) {
            pVia->CapInfo[1].Tuner = FI1236MK2;
            xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Cap1 Tuner: FI1236MK2\n");
        }
    }

    /* Get capture tuner device i2c slave write address */
    if (xf86GetOptValInteger(VIAOptions, OPTION_Cap0TunerI2C, &(pVia->CapInfo[0].Tuner_Slave_Write))) {
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Cap0 tuner slave address: %d\n", pVia->CapInfo[0].Tuner_Slave_Write);
    }

    if (xf86GetOptValInteger(VIAOptions, OPTION_Cap1TunerI2C, &(pVia->CapInfo[1].Tuner_Slave_Write))) {
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Cap1 tuner slave address: %d\n", pVia->CapInfo[1].Tuner_Slave_Write);
    }

    /* Get capture deinterlace mode */
    switch ( pVia->ChipId )
    {
    case VIA_DEVICE_CLE1:
    case VIA_DEVICE_CLE2:
    pVia->CapInfo[0].dwDeinterlaceMode = CAP_BOB;
        pVia->CapInfo[1].dwDeinterlaceMode = CAP_BOB;
        break;

    case VIA_DEVICE_VT3259:
        pVia->CapInfo[0].dwDeinterlaceMode = CAP_MEDIAN;
        pVia->CapInfo[1].dwDeinterlaceMode = CAP_MEDIAN;
        break;

    default:
        pVia->CapInfo[0].dwDeinterlaceMode = CAP_MEDIAN;
        pVia->CapInfo[1].dwDeinterlaceMode = CAP_MEDIAN;
        break;
    }
    
    if ((s = xf86GetOptValString(VIAOptions, OPTION_CAP0_DEINTERLACE))) {
        if (!xf86NameCmp(s, "Bob")) {
            pVia->CapInfo[0].dwDeinterlaceMode = CAP_BOB;
            xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Capture 0 de-interlace mode is Bob\n");
        }
        else if(!xf86NameCmp(s, "Weave")) {
            pVia->CapInfo[0].dwDeinterlaceMode = CAP_WEAVE;
            xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Capture 0 de-interlace mode is Weave\n");
        }
        else if(!xf86NameCmp(s, "Median")) {
            pVia->CapInfo[0].dwDeinterlaceMode = CAP_MEDIAN;
            xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Capture 0 de-interlace mode is Median\n");
        }
    }

    if ((s = xf86GetOptValString(VIAOptions, OPTION_CAP1_DEINTERLACE))) {
        if (!xf86NameCmp(s, "Bob")) {
            pVia->CapInfo[1].dwDeinterlaceMode = CAP_BOB;
            xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Capture 1 de-interlace mode is Bob\n");
        }
        else if(!xf86NameCmp(s, "Weave")) {
            pVia->CapInfo[1].dwDeinterlaceMode = CAP_WEAVE;
            xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Capture 1 de-interlace mode is Weave\n");
        }
        else if(!xf86NameCmp(s, "Median")) {
            pVia->CapInfo[1].dwDeinterlaceMode = CAP_MEDIAN;
            xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Capture 1 de-interlace mode is Median\n");
        }
    }

    if (xf86ReturnOptValBool(VIAOptions, OPTION_CAP0_FIELDSWAP, FALSE)) {
        pVia->CapInfo[0].FieldSwap = TRUE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,
                   "Option: Cap0_FieldSwap Enabled\n");
    }
    else {
        pVia->CapInfo[0].FieldSwap = FALSE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,
                   "Option: Cap0_FieldSwap Disabled\n");
    }

    /* Get capture horizontal filter option */
    if (xf86ReturnOptValBool(VIAOptions, OPTION_NOCAP0_HFILTER, FALSE)) {
        pVia->CapInfo[0].No_HFilter = TRUE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,
                   "Option: Cap0_HFilter Disabled\n");
    }
    else {
        pVia->CapInfo[0].No_HFilter = FALSE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,
                   "Option: Cap0_HFilter Enabled\n");
    }

    if (xf86ReturnOptValBool(VIAOptions, OPTION_NOCAP1_HFILTER, FALSE)) {
        pVia->CapInfo[1].No_HFilter = TRUE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,
                   "Option: Cap1_HFilter Disabled\n");
    }
    else {
        pVia->CapInfo[1].No_HFilter = FALSE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,
                   "Option: Cap1_HFilter Enabled\n");
    }

    /* Get capture over-scan option */
    if (xf86ReturnOptValBool(VIAOptions, OPTION_CAPTURE_OVERSCANOFF, FALSE)) {
        pVia->CapInfo[0].Capture_OverScanOff = TRUE;
        pVia->CapInfo[1].Capture_OverScanOff = TRUE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,
                   "Option: Capture Over Scan OFF\n");
    }
    else {
        pVia->CapInfo[0].Capture_OverScanOff = FALSE;
        pVia->CapInfo[1].Capture_OverScanOff = FALSE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,
                   "Option: Capture Over Scan ON\n");
    }


    /* Set HQV filter mode */
    if (xf86ReturnOptValBool(VIAOptions, OPTION_HQVFILTER_MANUALSELECT, FALSE)) {
        lpVIAGraphicInfo->HQVFilter_ManualSelect = TRUE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,
                   "Option: HQV Filter Manual Select Enabled\n");
    }
    else {
        lpVIAGraphicInfo->HQVFilter_ManualSelect = FALSE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,
                   "Option: HQV Filter Manual Select Disabled\n");
    }

    if (xf86GetOptValInteger(VIAOptions, OPTION_MPEG_HQV_VFILTER_MODE, &(lpVIAGraphicInfo->MpegHQV_VFilterMode))) {
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "MPEG HQV vertical filter mode: %d\n", lpVIAGraphicInfo->MpegHQV_VFilterMode);
    }

    if (xf86GetOptValInteger(VIAOptions, OPTION_MPEG_HQV_HFILTER_MODE, &(lpVIAGraphicInfo->MpegHQV_HFilterMode))) {
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "MPEG HQV horizontal filter mode: %d\n", lpVIAGraphicInfo->MpegHQV_HFilterMode);
    }

    if (xf86GetOptValInteger(VIAOptions, OPTION_CAP_HQV_VFILTER_MODE, &(lpVIAGraphicInfo->CapHQV_VFilterMode))) {
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Capture HQV vertical filter mode: %d\n", lpVIAGraphicInfo->CapHQV_VFilterMode);
    }

    if (xf86GetOptValInteger(VIAOptions, OPTION_CAP_HQV_HFILTER_MODE, &(lpVIAGraphicInfo->CapHQV_HFilterMode))) {
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Capture HQV horizontal filter mode: %d\n", lpVIAGraphicInfo->CapHQV_HFilterMode);
    }

    /* Mpeg decode frame buffer number */
    if (xf86ReturnOptValBool(VIAOptions, OPTION_SETMPEG_FBNUMBER, FALSE)) {
        lpVIAGraphicInfo->SetMpeg_FBNumber = TRUE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,
                   "Option: Set MPEG decode frame buffer number Enable\n");
    }
    else {
        lpVIAGraphicInfo->SetMpeg_FBNumber = FALSE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,
                   "Option: Set MPEG decode frame buffer number Disable\n");
    }

    /* Capture 1 use IRQ, for Cap1 S/W Flip */
    if (xf86ReturnOptValBool(VIAOptions, OPTION_CAP1_USEIRQ, FALSE)) {
        lpVIAGraphicInfo->Cap1_UseIRQ = TRUE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,
                   "Option: Capture 1 use S/W flip\n");
    }
    else {
        lpVIAGraphicInfo->Cap1_UseIRQ = FALSE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,
                   "Option: Capture 1 use H/W auto flip\n");
    }

    /* Capture 0 use V3 engine, for SAMSUNG project */
    if (xf86ReturnOptValBool(VIAOptions, OPTION_CAP0_SWITCHTOV3, FALSE)) {
        lpVIAGraphicInfo->Cap0_SwitchToV3 = TRUE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,
                   "Option: Capture 0 use V3 engine : for SAMSUNG\n");
    }
    else {
        lpVIAGraphicInfo->Cap0_SwitchToV3 = FALSE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,
                   "Option: Capture 0 use V1 engine : Default path\n");
    }

    if (xf86ReturnOptValBool(VIAOptions, OPTION_HQV_MANUALSWITCH, FALSE)) {
        lpVIAGraphicInfo->HQV_ManualSwitch = TRUE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,
                   "Option: HQV Manual Switch Enabled\n");
    }
    else {
        lpVIAGraphicInfo->HQV_ManualSwitch = FALSE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,
                   "Option: HQV Manual Switch Disabled\n");
    }

    if (xf86ReturnOptValBool(VIAOptions, OPTION_NOMPEGAddOneLineOnBottom, FALSE)) {
        lpVIAGraphicInfo->NoMPEGAddOneLineOnBottom = TRUE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,
                   "Option: No mpeg add one line on bottom = %lx\n", lpVIAGraphicInfo->NoMPEGAddOneLineOnBottom);
    }
    else {
        lpVIAGraphicInfo->NoMPEGAddOneLineOnBottom = FALSE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,
                   "Option: No mpeg add one line on bottom = %lx\n", lpVIAGraphicInfo->NoMPEGAddOneLineOnBottom);
    }

    if (xf86ReturnOptValBool(VIAOptions, OPTION_ReDrawColorkey, FALSE))
    {
        lpVIAGraphicInfo->ReDrawColorkey = TRUE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,"Option: ReDrawColorkey Enable!!\n");
    }
    else
    {
        lpVIAGraphicInfo->ReDrawColorkey = FALSE;
    }

    if (xf86ReturnOptValBool(VIAOptions, OPTION_Cap0CLKinverse, FALSE))
    {
        lpVIAGraphicInfo->Cap0CLKinverse = TRUE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,"Option: Cap0CLKinverse Enable!!\n");
    }
    else
    {
        lpVIAGraphicInfo->Cap0CLKinverse = FALSE;
    }

    if (xf86ReturnOptValBool(VIAOptions, OPTION_Cap1CLKinverse, FALSE))
    {
        lpVIAGraphicInfo->Cap1CLKinverse = TRUE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,"Option: Cap1CLKinverse Enable!!\n");
    }
    else
    {
        lpVIAGraphicInfo->Cap1CLKinverse = FALSE;
    }

    /* Joe [2004,08,09] for DeBlocking */
    if (xf86ReturnOptValBool(VIAOptions, OPTION_DeBlockingEnable, FALSE))
    {
        lpVIAGraphicInfo->DeBlockingEnable = TRUE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,"Option: DeBlocking Enable!!\n");
    }
    else
    {
        lpVIAGraphicInfo->DeBlockingEnable = FALSE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,"Option: DeBlocking Disable!!\n");
    }


    if (xf86GetOptValInteger(VIAOptions, OPTION_DeBlockingMinimumWidth, &iValue) > 0) {
    	 lpVIAGraphicInfo->dwDeBlockingMinimumWidth = (unsigned long)iValue;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, 
        		"DeBlocking Minimum Width: %ld\n",
        		lpVIAGraphicInfo->dwDeBlockingMinimumWidth);
    }
    else {
	 /* use default if not define */
    	 lpVIAGraphicInfo->dwDeBlockingMinimumWidth = DEBLOCKING_MINIMUM_WIDTH_DEFAULT;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, 
        		"DeBlocking Minimum Width Default : %ld\n",
        		lpVIAGraphicInfo->dwDeBlockingMinimumWidth);
    }

    if (xf86GetOptValInteger(VIAOptions, OPTION_DeBlockingMinimumHeight, &iValue)  > 0) {
	 lpVIAGraphicInfo->dwDeBlockingMinimumHeight = (unsigned long)iValue;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, 
        		"DeBlocking Minimum Height: %ld\n", 
        		lpVIAGraphicInfo->dwDeBlockingMinimumHeight);
    }
    else {
	 /* use default if not define */
	 lpVIAGraphicInfo->dwDeBlockingMinimumHeight = DEBLOCKING_MINIMUM_HEIGHT_DEFAULT;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, 
        		"DeBlocking Minimum Height Default: %ld\n", 
        		lpVIAGraphicInfo->dwDeBlockingMinimumHeight);

    }



    if (xf86ReturnOptValBool(VIAOptions, OPTION_PatchMPEGDecodeSliceErrorBitstreamDisable, FALSE))
    {
        lpVIAGraphicInfo->bPatchMPEGDecodeSliceErrorBitstreamDisable = TRUE;
/*      xf86DrvMsg("Option: Patch MPEG Decode Slice Error Bistream Disable!!\n");
*/
    }
    else
    {
        lpVIAGraphicInfo->bPatchMPEGDecodeSliceErrorBitstreamDisable = FALSE;
    }


    if (xf86ReturnOptValBool(VIAOptions, OPTION_Use2DBitBltSignature, FALSE)) {
        lpVIAGraphicInfo->bUse2DBitBltSignature = TRUE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,
                   "Option: Use 2D BitBlt method to write event-tag into VQ and make sure MPEG decode END Enable\n");
    }
    else {
        lpVIAGraphicInfo->bUse2DBitBltSignature = FALSE;
        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,
                   "Option: Use 2D BitBlt method to write event-tag into VQ and make sure MPEG decode END Disable\n");
    }

    /* The function of ddmpeg module is moved to V4L */
    /*
    if (!xf86LoadSubModule(pScrn, "ddmpeg")) {
        VIAFreeRec(pScrn);
        return FALSE;
    }
    */

    /* xf86LoaderReqSymLists(mpegSymbols, NULL); */
    if ((s = xf86GetOptValString(VIAOptions, OPTION_VideoOnDevice)))
    {/*[2005.10.24] For Option VideoOnDevice in duo view mode by Al Zhang*/
        if (!xf86NameCmp(s, "TV")) {
            lpVIAGraphicInfo->duoview |= DUOVIEW_VIDEOOPTION;
            lpVIAGraphicInfo->duoview |= DUOVIEW_VIDEOONTV;
            xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Video has been set to TV\n");
        }
        else if(!xf86NameCmp(s, "DVI")) {
            lpVIAGraphicInfo->duoview |= DUOVIEW_VIDEOOPTION;
            lpVIAGraphicInfo->duoview |= DUOVIEW_VIDEOONDVI;
            xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Video has been set to DVI\n");
        }
        else if(!xf86NameCmp(s, "LCD")) {
            lpVIAGraphicInfo->duoview |= DUOVIEW_VIDEOOPTION;
            lpVIAGraphicInfo->duoview |= DUOVIEW_VIDEOONLCD;
            xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Video has been set to LCD\n");
        }
        else if(!xf86NameCmp(s, "CRT")) {
            lpVIAGraphicInfo->duoview |= DUOVIEW_VIDEOOPTION;
            lpVIAGraphicInfo->duoview |= DUOVIEW_VIDEOONCRT;
            xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Video has been set to CRT\n");
        }
    }
    if(!VIALoadVideoOption(pVia, lpVIAGraphicInfo)) 
        VIALoadVideoOptionDefault(pVia, lpVIAGraphicInfo);
    
    if (pBIOSInfo->DuoView)
    {
        VIADeviceDispatchDuoView(pScrn);
    }    

    VIAUnmapMem(pScrn);

    return TRUE;
}

static int stop_from_suspend = 0;
static int ap_pid = 0;
static Bool VIAEnterVT(int scrnIndex, int flags)
{
    ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
    VIAPtr      pVia = VIAPTR(pScrn);
    vgaHWPtr hwp = VGAHWPTR(pScrn);
    vgaRegPtr   vgaSavePtr = &hwp->SavedReg;
    VIARegPtr   viaSavePtr = &pVia->SavedReg;
    Bool        ret;

    /* hyunzhang 03/02/2007 sync with vexp/vemp */
    exityes = 0;
    /* clean pq file */
    remove("/var/pq");
    if(stop_from_suspend && ap_pid)
    {
        pid_t pid = ap_pid;
        int signum = 62;
        union sigval mysigval;
        mysigval.sival_int=getpid(); //send X's PID to vexp/vemp
        sigqueue(pid,signum,mysigval);
        stop_from_suspend = 0;
        ap_pid = 0;
    }
  
    /* FIXME: Rebind AGP memory here */
    /* FIXME: Unlock DRI here */
    DEBUG(xf86DrvMsg(scrnIndex, X_INFO, "VIAEnterVT\n"));

    vgaHWUnlock(hwp);

    /* RainLin [2007.02.01]: Restore first or some of BIOS settings will lose. */
    if (!pVia->IsSecondary)
    {
        VIAWriteMode(pScrn, vgaSavePtr, viaSavePtr);
    }
    
    ret = VIAModeInit(pScrn, pScrn->currentMode);

    /* Patch for APM suspend resume, HWCursor has garbage */
    if (pVia->hwcursor && pVia->CursorImage)
    {
        DEBUG(xf86DrvMsg(scrnIndex, X_INFO, "Restore Cursor Image!\n"));
        memcpy(pVia->FBBase + pVia->CursorStart, pVia->CursorImage, 0x1000);
        VIASETREG(VIA_REG_CURSOR_FG, pVia->CursorFG);
        VIASETREG(VIA_REG_CURSOR_BG, pVia->CursorBG);
        VIASETREG(VIA_REG_CURSOR_MODE, pVia->CursorMC);
        xfree(pVia->CursorImage);
        pVia->CursorImage = NULL;
        /*VIALoadCursorImage(pScrn, *pVia->CursorImage);*/
    }

    /* RainLin [2007.01.02]: We need to initialize related registers about hardware icon again */
    /* when system resumes, or the cursor will become garbage. */
    VIAARGBCursorInit(pScrn);
    
    /* retore video status */
    if (!pVia->IsSecondary)
        viaRestoreVideo(pScrn);

    VIAAdjustFrame(scrnIndex, pScrn->frameX0, pScrn->frameY0, 0);

    pVia->IsSuspending = FALSE;    /* System has resumed. */
    
    return ret;
}



/* hyunzhang 03/02/2007 sync with vexp/vemp */
/* wait for vexp/vemp clean over */	
static void
wait_ap_exit(int signum,siginfo_t *info,void *myact)
{
    exityes = 1;
}
/* sync with vexp/vemp end */

static void VIALeaveVT(int scrnIndex, int flags)
{
    ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
    vgaHWPtr    hwp = VGAHWPTR(pScrn);
    VIAPtr      pVia = VIAPTR(pScrn);
    vgaRegPtr   vgaSavePtr = &hwp->SavedReg;
    VIARegPtr   viaSavePtr = &pVia->SavedReg;
    xf86DrvMsg(scrnIndex, X_INFO, "VIALeaveVT\n");

    /* hyunzhang 03/02/2007 sync with vexp/vemp */
    /* get vexp/vemp entity's PID */
    int vexppid = 0;
    FILE* fd;
    fd = fopen("/var/pq","r");
    if(fd)
    {
        fscanf(fd,"%d", &vexppid); 
        fclose(fd);
    }
    /* tell vexp/vemp system want to suspend */
    if(vexppid)
    {
        stop_from_suspend = 1;
        pid_t pid = ap_pid = vexppid;
        int signum = 60;
        union sigval mysigval;
        mysigval.sival_int=getpid(); //send X's PID to vexp/vemp
        sigqueue(pid,signum,mysigval);
        /* install feedback signal from vexp/vemp */
        struct sigaction act2;
        int sig2 = 61;
        /* if there is vexp */
        sigemptyset(&act2.sa_mask);
        act2.sa_flags=SA_SIGINFO;
        act2.sa_sigaction=wait_ap_exit;
        sigaction(sig2,&act2,NULL);
        /* don't stop until vexp/vemp send feedback */
        int sum = 0;
        /* time out is 5 seconds */
        while(sum++ < 5)
        {
            sleep(1);
            if(exityes)
                goto ok;
        }
        ok: 
        exityes = 0;
    }
    /* sync with vexp/vemp end */
	
    /* FIXME: take the DRI lock here to avoid accidents */
    /* FIXME: unbind the AGP memory ? */

    DEBUG(xf86DrvMsg(scrnIndex, X_INFO, "VIALeaveVT\n"));
    /* Wait Hardware Engine idle to exit graphicd mode */
    WaitIdle();

    if (pVia->VQEnable)
    {
     /* if we use VQ, disable it before we exit */
      switch( pVia->Chipset )	
      { 
       case VIA_P4M890: 
       case VIA_K8M890:
            VIASETREG(0x41c, 0x00100000);
            VIASETREG(0x420, 0x74301000);
         break;
       default:	 
            VIASETREG(0x43c, 0x00fe0000);
            VIASETREG(0x440, 0x00000004);
            VIASETREG(0x440, 0x40008c0f);
            VIASETREG(0x440, 0x44000000);
            VIASETREG(0x440, 0x45080c04);
            VIASETREG(0x440, 0x46800408);
         break;
      }  
    }

    /* Save video status and turn off all video activities */
    if (!pVia->IsSecondary)
        viaSaveVideo(pScrn);

    if (pVia->hwcursor)
    {
        pVia->CursorImage = (unsigned char *)xcalloc(1, 0x1000);
        memcpy(pVia->CursorImage, pVia->FBBase + pVia->CursorStart, 0x1000);
        pVia->CursorFG = (CARD32)VIAGETREG(VIA_REG_CURSOR_FG);
        pVia->CursorBG = (CARD32)VIAGETREG(VIA_REG_CURSOR_BG);
        pVia->CursorMC = (CARD32)VIAGETREG(VIA_REG_CURSOR_MODE);
    }

    VIAWriteMode(pScrn, vgaSavePtr, viaSavePtr);

    pVia->IsSuspending = TRUE;     /* System will suspend. */
    pVia->IsVIAModeInitRunOnceDone = FALSE;
    
    vgaHWLock(hwp);
}


static void VIASave(ScrnInfoPtr pScrn)
{
    vgaHWPtr        hwp = VGAHWPTR(pScrn);
    vgaRegPtr       vgaSavePtr = &hwp->SavedReg;
    VIAPtr          pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;
    VIARegPtr       save = &pVia->SavedReg;
    int             vgaCRIndex, vgaCRReg, vgaIOBase;
    int             i;
    /*unsigned char   TVRegs[0xFF];*/

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIASave\n"));

    if(pVia->IsSecondary)
    {
        DevUnion* pPriv;
        VIAEntPtr pVIAEnt;
        VIAPtr   pVia1;
        vgaHWPtr hwp1;
        pPriv = xf86GetEntityPrivate(pScrn->entityList[0],
              gVIAEntityIndex);
        pVIAEnt = pPriv->ptr;
        hwp1 = VGAHWPTR(pVIAEnt->pPrimaryScrn);
        pVia1 = VIAPTR(pVIAEnt->pPrimaryScrn);
        hwp->SavedReg = hwp1->SavedReg;
        pVia->SavedReg = pVia1->SavedReg;
    }
    else
    {
        vgaHWProtect(pScrn, TRUE);

        if (xf86IsPrimaryPci(pVia->PciInfo))
        {
            vgaHWSave(pScrn, vgaSavePtr, VGA_SR_ALL);
            DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO,
                             "Primary Adapter! saving VGA_SR_ALL !!\n"));
        }
        else
        {
            vgaHWSave(pScrn, vgaSavePtr, VGA_SR_MODE);
            DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO,
                             "Non-Primary Adapter! saving VGA_SR_MODE only !!\n"));
        }

        if (pBIOSInfo->Hotkey)
        {
    	    (void)VBEGetVBEMode(pVia->pVbe, &pVia->stateMode);
    	}
    	else
    	{
            vgaIOBase = hwp->IOBase;
            vgaCRReg = vgaIOBase + 5;
            vgaCRIndex = vgaIOBase + 4;

            /* Unlock Extended Regs */
            VGAOUT8(0x3c4, 0x10);
            VGAOUT8(0x3c5, VGAIN8(0x3c5) | 0x01);
            /* Unlock CRTC register protect */
            VGAOUT8(0x3d4, 0x47);
            VGAOUT8(0x3d5, VGAIN8(0x3d5) & ~0x01);

            /* Save Sequencer regs: from SR00 to SRFF */
            i = 0;
            
            while (1)
            {
                /* We have saved SR1A in VIAPreInit(). */
                if (i != 0x1A)
                {
                    VGAOUT8(0x3C4, i);
                    save->SRRegs[i] = VGAIN8(0x3C5);
                }

                if (i == 0xFF)
                {
                    break;
                }
                else
                {
                    i++;
                }
            }
            
            
            /* Save CRTC regs: from CR00 to CRFF */
            i = 0;
            
            while (1)
            {
                /* We have saved CR08 and CR09 in VIAPreInit(). */
                if ((i != 0x08) && (i != 0x09))
                {
                    VGAOUT8(vgaCRIndex, i);
                    save->CRTCRegs[i] = VGAIN8(vgaCRReg);
                }

                if (i == 0xFF)
                {
                    break;
                }
                else
                {
                    i++;
                }
            }

            /* Saving TV register status before set mode */
            if (pBIOSInfo->TVSettingInfo.TVEncoder)
            {
                VIASaveTV(pBIOSInfo, &(pBIOSInfo->TVSettingInfo), save);
            }
            
            if (pBIOSInfo->TVSettingInfo2.TVEncoder)
            {
                VIASaveTV(pBIOSInfo, &(pBIOSInfo->TVSettingInfo2), save);
            }
            

            /* Save for VT1636 LVDS: */
            if (pBIOSInfo->LVDSSettingInfo.ChipID == VIA_VT1636)
            {
                VIASaveLVDS_VT1636(pBIOSInfo, &(pBIOSInfo->LVDSSettingInfo), save);
            }
            else if(pBIOSInfo->LVDSSettingInfo2.ChipID == VIA_VT1636)
            {
                VIASaveLVDS_VT1636(pBIOSInfo, &(pBIOSInfo->LVDSSettingInfo2), save);
            }

            if (!pVia->ModeStructInit)
            {
                vgaHWCopyReg(&hwp->ModeReg, vgaSavePtr);
                memcpy(&pVia->ModeReg, save, sizeof(VIARegRec));
                pVia->ModeStructInit = TRUE;
            }
        } /* non-using device switch hotkey */
        
        /* Saving HW Cursor state */
        save->dwCursorMode = VIAGETREG(VIA_REG_CURSOR_MODE);
        save->dwCursorPOS = VIAGETREG(VIA_REG_CURSOR_POS);
        save->dwCursorORG = VIAGETREG(VIA_REG_CURSOR_ORG);
        save->dwCursorBG = VIAGETREG(VIA_REG_CURSOR_BG);
        save->dwCursorFG = VIAGETREG(VIA_REG_CURSOR_FG);
        
        vgaHWProtect(pScrn, FALSE);
    }
    return;
}


static void VIAWriteMode(ScrnInfoPtr pScrn, vgaRegPtr vgaSavePtr,
                         VIARegPtr restore)
{
    vgaHWPtr        hwp = VGAHWPTR(pScrn);
    VIAPtr          pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;
    int             vgaCRIndex, vgaCRReg, vgaIOBase;
    int             i, mode;
    
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIAWriteMode\n"));
    vgaIOBase = hwp->IOBase;
    vgaCRIndex = vgaIOBase + 4;
    vgaCRReg = vgaIOBase + 5;

    /* Disable output device avoid set mode garbage */
    if (!pVia->IsSecondary && !pBIOSInfo->Hotkey)
    {
        VIADisableLCD(pBIOSInfo, &(pBIOSInfo->LVDSSettingInfo));

        if (pBIOSInfo->LVDSSettingInfo2.ChipID)
        {
            VIADisableLCD(pBIOSInfo, &(pBIOSInfo->LVDSSettingInfo2));
        }
        
        VIADisableDFP(pBIOSInfo);
    }

    /*
     * If we figured out a VESA mode number for this timing, just use
     * the VGA BIOS to do the switching, with a few additional tweaks.
     */
    if (restore->mode != 0xFF)
    {
        /* Set up the mode.  Don't clear video RAM. */
        if (pVia->IsSecondary)
            VIASetModeForMHS(pBIOSInfo);
        else if (pBIOSInfo->Hotkey)
            VIAPrimarySetMode(pScrn);
        else
/*            VIASetModeUseBIOSTable(pBIOSInfo);*/
            VIASetMode(pBIOSInfo);
        xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIASetMode exit:TVtype=%d, TVScan=%d\n",pBIOSInfo->TVSettingInfo.TVType, pBIOSInfo->TVSettingInfo.TVVScan);       

        if((pBIOSInfo->s3utility==TRUE)&&(pBIOSInfo->SAMM))
        {
            if(pVia->ActiveDevice & VIA_DEVICE_CRT1)
                VIACRTEnable();
                        
            if(pVia->ActiveDevice & VIA_DEVICE_DFP)
                VIAEnableDFP(pBIOSInfo);

            if(pVia->ActiveDevice & VIA_DEVICE_LCD)
                VIAEnableLCD(pBIOSInfo, &(pBIOSInfo->LVDSSettingInfo));
        }

        pBIOSInfo->s3utility=FALSE;
        /* Restore the DAC.*/
        if (pBIOSInfo->FirstInit ) {
            vgaHWRestore(pScrn, vgaSavePtr, VGA_SR_CMAP);
        }
        /*else {
            vgaHWRestore(pScrn, vgaSavePtr, VGA_SR_MODE);
        }
        vgaHWRestore(pScrn, vgaSavePtr, VGA_SR_CMAP);*/
        pBIOSInfo->FirstInit = FALSE;
        /* Enable the graphics engine. */
        if (!pVia->NoAccel)
            VIAInitialize2DEngine(pScrn);

#ifdef XF86DRI
        VIAInitialize3DEngine(pScrn);
#endif        
        return;
    }

    /* How can we know the mode is graphic mode or not ? */
    /* graphicsMode = (restore->mode == 0xFF) ? FALSE : TRUE; */

    vgaHWProtect(pScrn, TRUE);

    /* Unlock Extended Regs */
    VGAOUT8(0x3c4, 0x10);
    VGAOUT8(0x3c5, VGAIN8(0x3c5) | 0x01);
    /* Unlock CRTC register protect */
    VGAOUT8(0x3d4, 0x47);
    VGAOUT8(0x3d5, VGAIN8(0x3d5) & ~0x01);    
       
    /*[06.07.2006] Carrie: Fix CRT showing garbage, when switch X to console on EPIA-EN
      Solution: turn off CRT*/
    VGAOUT8(0x3d4, 0x36);
    VGAOUT8(0x3d5, VGAIN8(0x3d5) | 0x30);

    /*=* VansonWu [2003.9.18] CR6A, CR6B, CR6C must be reset before restore
         standard vga regs, or system will be hang. *=*/
    /*=* VansonWu [2004.3.11] Reset IGA2 channel before disable IGA2 channel
         or it may cause some line garbage. *=*/
    /* RainLin [2005.11.22]: Modify for the sequence issue of enable and disable */
    /* second display channel. */
    VGAOUT8(0x3d4, 0x6A);
    VGAOUT8(0x3d5, VGAIN8(0x3d5) & 0xBF);   /* Set CR6A[6] to 0. */
    VGAOUT8(0x3d5, VGAIN8(0x3d5) & 0x7F);   /* Set CR6A[7] to 0. */
    VGAOUT8(0x3d5, VGAIN8(0x3d5) | 0x40);   /* Set CR6A[6] to 1. */
    VGAOUT8(0x3d5, VGAIN8(0x3d5) & 0xC0);   /* Set CR6A[5:0] to 0. */

    VGAOUT8(0x3d4, 0x6B);
    VGAOUT8(0x3d5, 0);
    VGAOUT8(0x3d4, 0x6C);
    VGAOUT8(0x3d5, 0);

    /* Gamma must disable before restore pallette */
    switch (pVia->Chipset)
    {
        case VIA_CLE266:
        case VIA_KM400:
            VGAOUT8(0x3c4, 0x16);
            VGAOUT8(0x3c5, VGAIN8(0x3c5) & 0x7F);
            break;

        /* For K8M800 or later chipsets: */
        default:
            VGAOUT8(0x3d4, 0x33);
            VGAOUT8(0x3d5, VGAIN8(0x3d5) & 0x7F);
            break;
    }

    /* restore the standard vga regs */
    if (xf86IsPrimaryPci(pVia->PciInfo))
        vgaHWRestore(pScrn, vgaSavePtr, VGA_SR_ALL);
    else
        vgaHWRestore(pScrn, vgaSavePtr, VGA_SR_MODE);

    VIADisabledExtendedFIFO(pBIOSInfo);    

    if (pBIOSInfo->Hotkey)
    {
        if (pBIOSInfo->ActiveDevice & VIA_DEVICE_LCD)
        {
            /* set LCD expansion by default */
            BIOS_SetFlatPanelState(pScrn, TRUE);
        }
        mode = pVia->stateMode | (1 << 15);
        VBESetVBEMode(pVia->pVbe, mode, NULL);
        BIOS_SetActiveDevice(pScrn);
    }
    else
    {
        /* restore TV regs */
        if (pBIOSInfo->TVSettingInfo.TVEncoder)
        {
            VIARestoreTV(pBIOSInfo, &(pBIOSInfo->TVSettingInfo), restore);
        }
        if (pBIOSInfo->TVSettingInfo2.TVEncoder)
        {
            VIARestoreTV(pBIOSInfo, &(pBIOSInfo->TVSettingInfo2), restore);
        }

        /* restore Sequencer regs: From SR14 to SR3F */
        i = 0x14;
        
        while (1)
        {
            VGAOUT8(0x3C4, i);
            VGAOUT8(0x3C5, restore->SRRegs[i]);

            if (i == 0x3F)
            {
                break;
            }
            else
            {
                i++;
            }
        }

        VGAOUT8(0x3C4, SR42);
        VGAOUT8(0x3C5, restore->SRRegs[0x42]);
        VGAOUT8(0x3C4, SR4F);
        VGAOUT8(0x3C5, restore->SRRegs[0x4F]);
        VGAOUT8(0x3C4, SR50);
        VGAOUT8(0x3C5, restore->SRRegs[0x50]);
        VGAOUT8(0x3C4, SR57);
        VGAOUT8(0x3C5, restore->SRRegs[0x57]);
        VGAOUT8(0x3C4, SR5D);
        VGAOUT8(0x3C5, restore->SRRegs[0x5D]);

        /* DVP1 Clock and Data Pads Driving: */
        VGAOUT8(0x3C4, SR65);
        VGAOUT8(0x3C5, restore->SRRegs[0x65]);
        
        /*=* restore VCK, LCDCK and ECK *=*/
        if (pVia->Chipset >= VIA_K8M800)
        {
            /* VCK: */
            VGAOUT8(0x3c4, SR44);
            VGAOUT8(0x3c5, restore->SRRegs[0x44]);
            VGAOUT8(0x3c4, SR45);
            VGAOUT8(0x3c5, restore->SRRegs[0x45]);
            VGAOUT8(0x3c4, SR46);
            VGAOUT8(0x3c5, restore->SRRegs[0x46]);
            /* Reset VCK PLL */
            VGAOUT8(0x3c4, SR40);
            VGAOUT8(0x3c5, VGAIN8(0x3c5) | 0x02);    /* Set SR40[1] to 1 */
            VGAOUT8(0x3c5, VGAIN8(0x3c5) & 0xFD);    /* Set SR40[1] to 0 */

            /* LCK: */
            VGAOUT8(0x3c4, SR4A);
            VGAOUT8(0x3c5, restore->SRRegs[0x4A]);
            VGAOUT8(0x3c4, SR4B);
            VGAOUT8(0x3c5, restore->SRRegs[0x4B]);
            VGAOUT8(0x3c4, SR4C);
            VGAOUT8(0x3c5, restore->SRRegs[0x4C]);
            /* Reset LCK PLL */
            VGAOUT8(0x3c4, SR40);
            VGAOUT8(0x3c5, VGAIN8(0x3c5) | 0x04);    /* Set SR40[2] to 1 */
            VGAOUT8(0x3c5, VGAIN8(0x3c5) & 0xFB);    /* Set SR40[2] to 0 */

            /* ECK: */
            VGAOUT8(0x3c4, SR47);
            VGAOUT8(0x3c5, restore->SRRegs[0x47]);
            VGAOUT8(0x3c4, SR48);
            VGAOUT8(0x3c5, restore->SRRegs[0x48]);
            VGAOUT8(0x3c4, SR49);
            VGAOUT8(0x3c5, restore->SRRegs[0x49]);
            /* Reset ECK PLL */
            VGAOUT8(0x3c4, SR40);
            VGAOUT8(0x3c5, VGAIN8(0x3c5) | 0x01);    /* Set SR40[0] to 1 */
            VGAOUT8(0x3c5, VGAIN8(0x3c5) & 0xFE);    /* Set SR40[0] to 0 */
        }
        else
        {
            /* LCK: */
            VGAOUT8(0x3c4, SR44);
            VGAOUT8(0x3c5, restore->SRRegs[0x44]);
            VGAOUT8(0x3c4, SR45);
            VGAOUT8(0x3c5, restore->SRRegs[0x45]);
            /* Reset LCK PLL */
            VGAOUT8(0x3c4, SR40);
            VGAOUT8(0x3c5, VGAIN8(0x3c5) | 0x01);    /* Set SR40[0] to 1 */
            VGAOUT8(0x3c5, VGAIN8(0x3c5) & 0xFE);    /* Set SR40[0] to 0 */
            
            /* VCK: */
            VGAOUT8(0x3c4, SR46);
            VGAOUT8(0x3c5, restore->SRRegs[0x46]);
            VGAOUT8(0x3c4, SR47);
            VGAOUT8(0x3c5, restore->SRRegs[0x47]);
            /* Reset VCK PLL */
            VGAOUT8(0x3c4, SR40);
            VGAOUT8(0x3c5, VGAIN8(0x3c5) | 0x02);    /* Set SR40[1] to 1 */
            VGAOUT8(0x3c5, VGAIN8(0x3c5) & 0xFD);    /* Set SR40[1] to 0 */

            /* ECK: */
            VGAOUT8(0x3c4, SR48);
            VGAOUT8(0x3c5, restore->SRRegs[0x48]);
            VGAOUT8(0x3c4, SR49);
            VGAOUT8(0x3c5, restore->SRRegs[0x49]);
            /* Reset ECK PLL */
            VGAOUT8(0x3c4, SR40);
            VGAOUT8(0x3c5, VGAIN8(0x3c5) | 0x04);    /* Set SR40[2] to 1 */
            VGAOUT8(0x3c5, VGAIN8(0x3c5) & 0xFB);    /* Set SR40[2] to 0 */
        }


        /* Restore CRTC controller extended regs: */
        
        VGAOUT8(vgaCRIndex, CR08);
        VGAOUT8(vgaCRReg, restore->CRTCRegs[0x08]);
        VGAOUT8(vgaCRIndex, CR09);
        VGAOUT8(vgaCRReg, restore->CRTCRegs[0x09]);

        VGAOUT8(vgaCRIndex, CR13);
        VGAOUT8(vgaCRReg, restore->CRTCRegs[0x13]);
        VGAOUT8(vgaCRIndex, CR30);
        VGAOUT8(vgaCRReg, restore->CRTCRegs[0x30]);
        VGAOUT8(vgaCRIndex, CR31);
        VGAOUT8(vgaCRReg, restore->CRTCRegs[0x31]);
        VGAOUT8(vgaCRIndex, CR32);
        VGAOUT8(vgaCRReg, restore->CRTCRegs[0x32]);
        VGAOUT8(vgaCRIndex, CR33);
        VGAOUT8(vgaCRReg, restore->CRTCRegs[0x33]);

        /* From CR35 to CR47 */
        i = 0x35;
        
        while (1)
        {
            VGAOUT8(vgaCRIndex, i);
            VGAOUT8(vgaCRReg, restore->CRTCRegs[i]);

            if (i == 0x47)
            {
                break;
            }
            else
            {
                i++;
            }
        }

        /* Starting Address: */
        VGAOUT8(vgaCRIndex, CR0C);
        VGAOUT8(vgaCRReg, restore->CRTCRegs[0x0C]);
        VGAOUT8(vgaCRIndex, CR0D);
        VGAOUT8(vgaCRReg, restore->CRTCRegs[0x0D]);
        VGAOUT8(vgaCRIndex, CR48);
        VGAOUT8(vgaCRReg, restore->CRTCRegs[0x48]);
        VGAOUT8(vgaCRIndex, CR34);
        VGAOUT8(vgaCRReg, restore->CRTCRegs[0x34]);

        /* From CR49 to CRFF */
        i = 0x49;
        
        while (1)
        {
            VGAOUT8(vgaCRIndex, i);
            VGAOUT8(vgaCRReg, restore->CRTCRegs[i]);

            if (i == 0xFF)
            {
                break;
            }
            else
            {
                i++;
            }
        }
        

        /* Restore for VT1636 LVDS: */
        if (pBIOSInfo->LVDSSettingInfo.ChipID == VIA_VT1636)
        {
            VIARestoreLVDS_VT1636(pBIOSInfo, &(pBIOSInfo->LVDSSettingInfo), restore);
        }
        else if(pBIOSInfo->LVDSSettingInfo2.ChipID == VIA_VT1636)
        {
            VIARestoreLVDS_VT1636(pBIOSInfo, &(pBIOSInfo->LVDSSettingInfo2), restore);
        }

        if (pBIOSInfo->DefaultActiveDevice & VIA_DEVICE_LCD)
        {
            /* RainLin [2006.09.07]: To do this will affect the original CMOS settings for LVDS. */
            /* I think there is no need to do this for Integrated LVDS because it seems no sequence */
            /* issue for this transmitter. */
            if (pBIOSInfo->LVDSSettingInfo.ChipID != VIA_INTEGRATED_LVDS)
            {
                VIAEnableLCD(pBIOSInfo, &(pBIOSInfo->LVDSSettingInfo));
            }

            /* RainLin [2007.03.22]: Patch for the problem: When BIOS enable Spread Spectrum, */
            /* Spectrum IO selected bit (SR2C[0]) should be enabled, too, or the screen will scramble. */
            VGAOUT8(0x3c4, SR1E);

            if (VGAIN8(0x3c5) & BIT3)
            {
                VGAOUT8(0x3c4, SR2C);
                VGAOUT8(0x3c5, VGAIN8(0x3c5) | BIT0);
            }
        }

        /* Reset clock */
        VGAOUT8(0x3c2, VGAIN8(0x3cc));
    } /* non-using device switch hotkey */

    /* Restore HW Cursor state */
    VIASETREG(VIA_REG_CURSOR_MODE, restore->dwCursorMode);
    VIASETREG(VIA_REG_CURSOR_POS, restore->dwCursorPOS);
    VIASETREG(VIA_REG_CURSOR_ORG, restore->dwCursorORG);
    VIASETREG(VIA_REG_CURSOR_BG, restore->dwCursorBG);
    VIASETREG(VIA_REG_CURSOR_FG, restore->dwCursorFG);
    
    /*[06.7.2006] Carrie: Fix CRT showing garbage, when switch X to console on EPIA-EN
    Solution: turn on CRT*/
    if (pBIOSInfo->DefaultActiveDevice & VIA_DEVICE_CRT1)
    {
        VGAOUT8(0x3d4, 0x36);
        VGAOUT8(0x3d5, VGAIN8(0x3d5) & 0xCF);
    }

    vgaHWProtect(pScrn, FALSE);

    return;
}


static
Bool VIAMapMMIO(ScrnInfoPtr pScrn)
{
    VIAPtr pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr pBIOSInfo = pVia->pBIOSInfo;
    vgaHWPtr hwp;

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIAMapMMIO\n"));

    pVia->FrameBufferBase = pVia->PciInfo->memBase[0];
    pVia->MmioBase = pVia->PciInfo->memBase[1];

    xf86DrvMsg(pScrn->scrnIndex, X_PROBED,
	       "mapping MMIO @ 0x%lx with size 0x%x\n",
	       pVia->MmioBase, VIA_MMIO_REGSIZE);

    pVia->MapBase = xf86MapPciMem(pScrn->scrnIndex, VIDMEM_MMIO, pVia->PciTag,
				  pVia->MmioBase,
				  VIA_MMIO_REGSIZE);
    pBIOSInfo->MapBase = pVia->MapBase;
    MMIOMapBase = pVia->MapBase + 0x8000;

    xf86DrvMsg(pScrn->scrnIndex, X_PROBED,
	       "mapping BitBlt MMIO @ 0x%lx with size 0x%x\n",
	       pVia->MmioBase + VIA_MMIO_BLTBASE, VIA_MMIO_BLTSIZE);

    pVia->BltBase = xf86MapPciMem(pScrn->scrnIndex, VIDMEM_MMIO, pVia->PciTag,
				  pVia->MmioBase + VIA_MMIO_BLTBASE,
				  VIA_MMIO_BLTSIZE);

    if (!pVia->MapBase || !pVia->BltBase) {
	xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
		   "Internal error: cound not map registers\n");
	return (FALSE);
    }

    /* Memory mapped IO for Video Engine */
    pVia->VidMapBase = pVia->MapBase + 0x200;

    /* Memory mapped IO for Integrated TV. */
    if ((pVia->pBIOSInfo->Chipset == VIA_CX700) ||
	(pVia->pBIOSInfo->Chipset == VIA_CN750)) {
	xf86DrvMsg(pScrn->scrnIndex, X_PROBED,
		   "mapping Integrated TV MMIO @ 0x%lx with size 0x%x\n",
		   pVia->MmioBase + VIA_MMIO_INTEGRATED_TV_BASE,
		   VIA_MMIO_INTEGRATED_TV_SIZE);

	pVia->IntegratedTVMapBase =
	    xf86MapPciMem(pScrn->scrnIndex, VIDMEM_MMIO, pVia->PciTag,
			  pVia->MmioBase + VIA_MMIO_INTEGRATED_TV_BASE, 
			  VIA_MMIO_INTEGRATED_TV_SIZE);

	if (!pVia->IntegratedTVMapBase) {
	    xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
		       "Internal error: cound not map integrated TV registers\n");
	    return (FALSE);
	}

	pBIOSInfo->IntegratedTVMapBase = pVia->IntegratedTVMapBase;
    }

    VIAEnableMMIO(pScrn);
    hwp = VGAHWPTR(pScrn);
    vgaHWGetIOBase(hwp);
    if (!vgaHWMapMem(pScrn))
	return (FALSE);

    return (TRUE);
}


static Bool VIAMapFB(ScrnInfoPtr pScrn)
{
    DevUnion* pPriv;
    VIAEntPtr pVIAEnt;
    VIAPtr    pVia;
    pPriv =xf86GetEntityPrivate(pScrn->entityList[0],gVIAEntityIndex);
    pVIAEnt = pPriv->ptr;
    pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIAMapFB\n"));
    xf86DrvMsg(pScrn->scrnIndex, X_PROBED,
               "mapping framebuffer @ 0x%lx with size 0x%lx\n",
               pVia->FrameBufferBase, pVia->videoRambytes);

    if (pVia->videoRambytes) {
        pVia->FBBase = xf86MapPciMem(pScrn->scrnIndex, VIDMEM_FRAMEBUFFER,
                                     pVia->PciTag, pVia->FrameBufferBase,
                                     pVia->videoRambytes);
            pBIOSInfo->FBBase = pVia->FBBase;
            pBIOSInfo->videoRambytes = pVia->videoRambytes;

        if (!pVia->FBBase) {
            xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
                       "Internal error: could not map framebuffer\n");
            return FALSE;
        }

        pVia->FBStart = pVia->FBBase;
        pVia->FBFreeStart = (pScrn->displayWidth * pScrn->bitsPerPixel >> 3) *
                            pScrn->virtualY;
        pVia->FBFreeEnd = pVia->videoRambytes;

        xf86DrvMsg(pScrn->scrnIndex, X_PROBED,
                   "Frame buffer start: %p, free start: 0x%x end: 0x%x\n",
                   pVia->FBStart, pVia->FBFreeStart, pVia->FBFreeEnd);
    }

    pScrn->memPhysBase = pVia->PciInfo->memBase[0];
    pScrn->fbOffset = 0;
    if(pVia->IsSecondary) pScrn->fbOffset = pVIAEnt->pPrimaryScrn->videoRam << 10;

    return TRUE;
}


static void VIAUnmapMem(ScrnInfoPtr pScrn)
{
    VIAPtr pVia;

    pVia = VIAPTR(pScrn);

    /*=* VansonWu [2003.10.28] Fix 3205Ax exit XWindow screen crash
         when entry Linux using graphic console mode (ex:MDK 9.1) *=*/
#if 0
    VIADisableMMIO(pScrn);
#endif

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIAUnmapMem\n"));

    if (pVia->MapBase) {
        xf86UnMapVidMem(pScrn->scrnIndex, (pointer)pVia->MapBase,
                        VIA_MMIO_REGSIZE);
    }

    if (pVia->BltBase) {
        xf86UnMapVidMem(pScrn->scrnIndex, (pointer)pVia->BltBase,
                        VIA_MMIO_BLTSIZE);
    }

    if (pVia->FBBase) {
        xf86UnMapVidMem(pScrn->scrnIndex, (pointer)pVia->FBBase,
                        pVia->videoRambytes);
    }

    if ((pVia->pBIOSInfo->Chipset == VIA_CX700)
     || (pVia->pBIOSInfo->Chipset == VIA_CN750))
    {
        if (pVia->IntegratedTVMapBase)
        {
            xf86UnMapVidMem(pScrn->scrnIndex, (pointer)pVia->IntegratedTVMapBase, 
                            VIA_MMIO_INTEGRATED_TV_SIZE);
        }
    }
    
    return;
}


static Bool VIAScreenInit(int scrnIndex, ScreenPtr pScreen,
                          int argc, char **argv)
{
    ScrnInfoPtr pScrn = xf86Screens[pScreen->myNum];
    vgaHWPtr hwp = VGAHWPTR(pScrn);
    VIAPtr pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;
    PVIDDATA pVidData = pVia->pVidData;
    int ret, fd0, fd1, fd2;

    char devName0[20]="/dev/video";
    char devName1[20]="/dev/video";
    char devName2[20]="/dev/video";	
    const char *minorChar[31]={"0","1","2","3","4","5","6","7","8","9","10","11","12","13","14","15","16","17","18","19","20","21","22","23","24","25","26","27","28","29","30"}; 	
    unsigned char minorNum;
    
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIAScreenInit\n"));    

    /* STChen[2007.3.8]: For MAMM*/
    if (!pVia->IsMAMMEnable)
    { 
        vgaHWUnlock(hwp);
    }
    
    if (!VIAMapFB(pScrn))
        return FALSE;

    if (!VIAMapMMIO(pScrn))
        return FALSE;

    /* RainLin [2005.12.07]: We should arrange the base address of source and */
    /* destination buffers by frame buffer size, for integrated TV only. */
    /* Only need to do once, so secondary driver doesn't need to do. */
    if (((pVia->Chipset == VIA_CX700)||(pVia->Chipset == VIA_CN750)) && (!pVia->IsSecondary))
    {        
        SetIntegratedTVBufferAddress(pScrn);
    }

    if(pVia->DISP3DScaling)
    {
        Reserve3DScalingMemory(pScrn);
    }

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "- Mem Mapped\n"));

     /*fd0 = open("/dev/video0",O_RDWR);
    fd1 = open("/dev/video1",O_RDWR);
    fd2 = open("/dev/video2",O_RDWR);*/
    
    /* read minor number of video device from BIOS register 3C5.32 */
    /* this is for AP use dynamical method to open device */
    VGAOUT8(0x3c4, 0x32);
    minorNum = VGAIN8(0x3c5);	
    if(minorNum < 2)
        minorNum = 2;
    strcpy(pVia->video0Path, strcat(devName0,minorChar[minorNum-2]));
    strcpy(pVia->video1Path, strcat(devName1,minorChar[minorNum-1]));
    strcpy(pVia->video2Path, strcat(devName2,minorChar[minorNum]));	

    fd0 = open(pVia->video0Path ,O_RDWR);
    fd1 = open(pVia->video1Path ,O_RDWR);
    fd2 = open(pVia->video2Path ,O_RDWR);

    if( -1 == fd2)
    {
        xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
                   " Couldn't open \"/dev/video2\"\n");
        pVia->V4LEnabled = 0;
        xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIAScreenInit : V4L Disabled : fd2 = %d \n",fd2);
    }
    else
    {
        pVia->V4LEnabled = 1;
        xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIAScreenInit : V4L Enabled : fd2 = %d \n",fd2);
    }
    
    /*=* John Sheng [2003.7.02] for DRI overlap Xserver desktop map *=*/
    /*if (!pVia->NoAccel)
        VIAInitAccel(pScreen);*/
    
    /*=* John Sheng [2003.7.02] fix visual setting. *=*/
    /*=* John Sheng [2003.7.04] fix video can't get enough space from
                                Xserver fb manager                    *=*/
#ifdef XF86DRI
        pVia->directRenderingEnabled = VIADRIScreenInit(pScreen);
#endif


    VIASave(pScrn);

    vgaHWBlankScreen(pScrn, FALSE);

    if (!VIAModeInit(pScrn, pScrn->currentMode))
        return FALSE;

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "- State saved\n"));

    /* Darken the screen for aesthetic reasons and set the viewport */
    VIASaveScreen(pScreen, SCREEN_SAVER_ON);
    pScrn->AdjustFrame(scrnIndex, pScrn->frameX0, pScrn->frameY0, 0);

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "- Blanked\n"));

    miClearVisualTypes();

    if (pScrn->bitsPerPixel > 8 && !pVia->IsSecondary)
    {
        if (!miSetVisualTypes(pScrn->depth, TrueColorMask,
                              pScrn->rgbBits, pScrn->defaultVisual))
            return FALSE;
        if (!miSetPixmapDepths())
            return FALSE;
    }
    else
    {
        if (!miSetVisualTypes(pScrn->depth, miGetDefaultVisualMask(pScrn->depth),
                              pScrn->rgbBits, pScrn->defaultVisual))
            return FALSE;
        if (!miSetPixmapDepths())
            return FALSE;
    }

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "- Visuals set up\n"));

    ret = VIAInternalScreenInit(scrnIndex, pScreen);

    if (!ret)
        return FALSE;

    xf86SetBlackWhitePixels(pScreen);
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "- B & W\n"));

    if (pScrn->bitsPerPixel > 8) {
        VisualPtr visual;

        visual = pScreen->visuals + pScreen->numVisuals;
        while (--visual >= pScreen->visuals) {
            if ((visual->class | DynamicClass) == DirectColor) {
                visual->offsetRed = pScrn->offset.red;
                visual->offsetGreen = pScrn->offset.green;
                visual->offsetBlue = pScrn->offset.blue;
                visual->redMask = pScrn->mask.red;
                visual->greenMask = pScrn->mask.green;
                visual->blueMask = pScrn->mask.blue;
            }
        }
    }

#ifdef USE_FB
    /* must be after RGB ordering fixed */
    fbPictureInit(pScreen, 0, 0);
#endif
    /*=* John Sheng [2003.7.14] for 2D accel fail *=*/
    if (!pVia->NoAccel)
    {
        VIAInitAccel(pScreen);
    }
#ifdef X_USE_LINEARFB
    else
    {
	/*
	 * This is needed because xf86InitFBManagerLinear in VIAInitLinear
	 * needs xf86InitFBManager to have been initialized, and
	 * xf86InitFBManager needs at least one line of free memory to
	 * work. This is only for Xv in Noaccel part, and since Xv is in some
	 * sense accelerated, it might be a better idea to disable it
	 * altogether.
	 */
        BoxRec AvailFBArea;

        AvailFBArea.x1 = 0;
        AvailFBArea.y1 = 0;
        AvailFBArea.x2 = pScrn->displayWidth;
        AvailFBArea.y2 = pScrn->virtualY + 1;
	/*
	 * Update FBFreeStart also for other memory managers, since
	 * we steal one line to make xf86InitFBManager work.
	 */
    	pVia->FBFreeStart = (AvailFBArea.y2 + 1) * pVia->Bpl;
    	xf86InitFBManager(pScreen, &AvailFBArea);
    }
#endif /* X_USE_LINEARFB */
#ifdef XF86DRI
    /*=* John Sheng [2003.7.15] for VIDEO play memory overlap PIXEL_MAP_CACHE *=*/
    if (pVia->directRenderingEnabled) 
    {
	if (!(VIADRIFBInit(pScreen, pVia))) {
	    VIADRICloseScreen(pScreen);
	    xf86DrvMsg(pScrn->scrnIndex, X_ERROR, "[dri] frame buffer initialize fial .\n" );
	    pVia->directRenderingEnabled = FALSE;
	}
    }
#endif

    miInitializeBackingStore(pScreen);
    xf86SetBackingStore(pScreen);
    xf86SetSilkenMouse(pScreen);
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "- Backing store set up\n"));

    if(!pVia->shadowFB)         /* hardware cursor needs to wrap this layer */
        VIADGAInit(pScreen);

    miDCInitialize(pScreen, xf86GetPointerScreenFuncs());
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "- SW cursor set up\n"));

    if (!pVia->ForceSWCursor)
    {
        if (!VIAHWCursorInit(pScreen))
        {
            xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
                       "Hardware cursor initialization failed\n");
        }
    }

    if (pVia->shadowFB)
    {
        RefreshAreaFuncPtr refreshArea = VIARefreshArea;
        
        if(pVia->SWRotate || pVia->RandRRotation)
        {            
            pVia->PointerMoved = pScrn->PointerMoved;
            if(pVia->SWRotate)
            {
                pScrn->PointerMoved = VIAPointerMoved;
            }
           
            switch(pScrn->bitsPerPixel)
            {
                case 8:
                    refreshArea = VIARefreshArea8;
                    break;
                case 16:
                    refreshArea = VIARefreshArea16;
                    break;
                case 32:                
                    refreshArea = VIARefreshArea32;
                    break;
            }           
        }

        ShadowFBInit(pScreen, refreshArea);
    }    
    
    if (pVia->RandRRotation)
    {
        #ifdef _XORG_VER_6_8_LATER_
            #ifdef _XORG_VER_6_9_LATER_
            pScrn->DriverFunc = VIADriverFunc;
            #else
            pScrn->RRFunc = VIADriverFunc;
            #endif
        #endif
    }
    
    if (!miCreateDefColormap(pScreen))
        return FALSE;
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "- Def Color map set up\n"));

    /* only 3123Ax IGA2 use 6-Bit LUT, others chipset IGA2 use 8-Bit LUT */
    /* So, except 3123Ax, we use 8-Bit LUT as default. */
    if ((pBIOSInfo->Chipset == VIA_CLE266) && (pBIOSInfo->ChipRev < 15))
    {
        pScrn->rgbBits = 6;
        VGAOUT8(0x3C4, SR15);
        VGAOUT8(0x3C5, VGAIN8(0x3C5) & 0xEF);
        if (!xf86HandleColormaps(pScreen, 256, 6, VIALoadPalette, NULL,
                                 CMAP_RELOAD_ON_MODE_SWITCH | CMAP_PALETTED_TRUECOLOR))
        {
            return FALSE;
        }
    }
    else
    {
        pScrn->rgbBits = 8;
        VGAOUT8(0x3C4, SR15);
        VGAOUT8(0x3C5, VGAIN8(0x3C5) | 0x80);
        if (!xf86HandleColormaps(pScreen, 256, 8, VIALoadPalette, NULL,
                                 CMAP_RELOAD_ON_MODE_SWITCH | CMAP_PALETTED_TRUECOLOR))
        {
            return FALSE;
        }
    }

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "- Palette loaded\n"));

    /* Remove restart flag file for utility */
    VIAUTRemoveRestartFlag(pBIOSInfo);
    /* Load user utility Gamma setting */
    if (pScrn->bitsPerPixel != 8)
    {
        if (VIALoadGammaSetting(pBIOSInfo))
        {
            /* Restore Gamma at single view or SAMM secondary driver instance.
             * Due to IGA2 color map not initial yet, we can't restore Gamma
             * in SAMM primary driver instance */
            if(!pBIOSInfo->HasSecondary)
            {
                VIARestoreGamma(pScrn, pBIOSInfo->colors);
            }
        }
    }


    vgaHWBlankScreen(pScrn, TRUE);

    pVia->CloseScreen = pScreen->CloseScreen;
    pScreen->SaveScreen = VIASaveScreen;
    pScreen->CloseScreen = VIACloseScreen;

    xf86DPMSInit(pScreen, VIADPMS, 0);
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "- DPMS set up\n"));

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "- Color maps etc. set up\n"));

#ifdef XF86DRI
    /*=* John Sheng [2003.7.02] fix visual setting. *=*/
    /*pVia->directRenderingEnabled = VIADRIScreenInit(pScreen);*/

    if (pVia->directRenderingEnabled) 
    {
        pVia->directRenderingEnabled = VIADRIFinishScreenInit(pScreen);
    }
    if (pVia->directRenderingEnabled) 
    {
	
        xf86DrvMsg(pScrn->scrnIndex, X_INFO, "direct rendering enabled\n");
        VIASet3DSyncInfoParas(pScreen);
    }
    else 
    {
        xf86DrvMsg(pScrn->scrnIndex, X_INFO, "direct rendering disabled\n");
    }
#endif

    /* Initial VIA Video Driver */
    FillGraphicInfo(pScrn);

    /* Mark we have already use all memory */
    /* pVia->FBFreeStart = pVia->FBFreeEnd; */

    /* pass graphic info to via_v4l kernel module */

    if (pVia->V4LEnabled == 1)
    {     
        if ( -1 == ioctl(fd2, VIAMGR_INFO_XSERVERON, pVidData) )
        {
            xf86DrvMsg(pScrn->scrnIndex, X_ERROR, "IOCTL VIAMGR_INFO_XSERVERON fail.\n");
            return FALSE;
        }

        /*  Pass capture related info to V4L,
         *  and do 3rd party Device Init, ex: video decoder, tuner, audio device
         */
        if ( -1 == ioctl(fd0, VIA_VID_SET_CAP_INFO, &(pVia->CapInfo[0])) )
        {
            xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
                "IOCTL VIA_VID_SET_CAP_INFO to video0 fail.\n");
            return FALSE;
        }

        if ( -1 == ioctl(fd1, VIA_VID_SET_CAP_INFO, &(pVia->CapInfo[1])) )
        {
            xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
                "IOCTL VIA_VID_SET_CAP_INFO to video1 fail.\n");
            return FALSE;
        }
    }
    else
    {
        /* do initian stuff */
        /* If there is bandwidth issue, block the H/W overlay */
        pVidData->dwRejectOverlay = FALSE;

        /* For capture 1 use S/W flip */
        pVidData->Address0_Switch = TRUE;
        pVidData->Address1_Switch = TRUE;
        pVidData->V3Address0_Switch = TRUE;
        pVidData->V3Address1_Switch = TRUE;

        pVidData->AUDIO_MUTE=TRUE;

        DBG_DD(("libddmpeg: Heap Begin 0x%lx,End 0x%lx\n"
            , lpVIAGraphicInfo->Screen[ScrnIndex].VideoHeapBase, lpVIAGraphicInfo->Screen[ScrnIndex].VideoHeapEnd));
        DBG_DD(("libddmpeg: dwXserverEnabled=%d\n",lpVIAGraphicInfo->dwXServerEnabled));
        DBG_DD(("dwWidth: 0x%lx\n", lpVIAGraphicInfo->Screen[ScrnIndex].dwWidth));
        DBG_DD(("dwHeight: 0x%lx\n", lpVIAGraphicInfo->Screen[ScrnIndex].dwHeight));
        DBG_DD(("dwSaveWidth: 0x%lx\n", lpVIAGraphicInfo->Screen[ScrnIndex].dwSaveWidth));
        DBG_DD(("dwBPP: 0x%lx\n", lpVIAGraphicInfo->Screen[ScrnIndex].dwBPP));
        DBG_DD(("dwRefreshRate: 0x%lx\n", lpVIAGraphicInfo->Screen[ScrnIndex].dwRefreshRate));
        DBG_DD(("DRMEnabled: 0x%x\n", lpVIAGraphicInfo->DRMEnabled));
        DBG_DD(("dwExpand: 0x%lx\n", lpVIAGraphicInfo->Screen[ScrnIndex].dwExpand));

        /*  Init video structure, video flags & HW difference flags */		   
        VGAOUT8(0x16, 0x3C4); pVidData->Save_3C4_16 = VGAIN8(0x3C5);
        DBG_DD(("store 3c4.16 : %08x \n",inb(0x3C5)));
        VGAOUT8(0x17, 0x3C4); pVidData->Save_3C4_17 = VGAIN8(0x3C5);
        DBG_DD(("store 3c4.17 : %08x \n",inb(0x3C5)));
        VGAOUT8(0x18, 0x3C4); pVidData->Save_3C4_18 = VGAIN8(0x3C5);
        DBG_DD(("store 3c4.18 : %08x \n",inb(0x3C5)));
    }

    /*  Init video structure, video flags & HW difference flags */
#if 0/* XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX */
    vfInitVideoInfo(pVidData );
#endif
    
    viaInitVideo(pScreen);

    /* Horace Lan [2005-10-27]
       When rotate in FC3,viaInitMC will result in error.So disable it when rotate */
    /*if(!pVia->SWRotate && !pVia->RandRRotation)
        viaInitMC(pScreen);*/

    close(fd0);
    close(fd1);
    close(fd2);

    if(pBIOSInfo->MergedFB) {
      
       /* Psuedo xinerama */
       if(pBIOSInfo->UseVIAXinerama) {
          VIAnoPanoramiXExtension = FALSE;
          VIAXineramaExtensionInit(pScrn);
       }
    } 
    
    /* Create a extension handler to receive the request of s3utility. */
    /* STChen[2006.9.28]: XV path and Xext path is co-exists now.*/
    VIADisplayExtensionInit(pScrn);

    if (serverGeneration == 1)
        xf86ShowUnusedOptions(pScrn->scrnIndex, pScrn->options);

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "- Done\n"));
    return TRUE;
}


static int VIAInternalScreenInit(int scrnIndex, ScreenPtr pScreen)
{
    ScrnInfoPtr     pScrn;
    VIAPtr          pVia;
    VIABIOSInfoPtr  pBIOSInfo;
    int             width, height, displayWidth, shadowHeight;
    unsigned char   *FBStart;
    int             ret = TRUE;
        
    xf86DrvMsg(scrnIndex, X_INFO, "VIAInternalScreenInit\n");

    pScrn = xf86Screens[pScreen->myNum];
    pVia = VIAPTR(pScrn);
    pBIOSInfo = pVia->pBIOSInfo;

    displayWidth = pScrn->displayWidth;

    if ((pVia->SWRotate==VIA_ROTATE_DEGREE_90)||
        (pVia->SWRotate==VIA_ROTATE_DEGREE_270))
    {
        height = pScrn->virtualX;
        width = pScrn->virtualY;
    } else {
        width = pScrn->virtualX;
        height = pScrn->virtualY;
    }

    if(pVia->RandRRotation)
        shadowHeight = max(width, height);
    else
        shadowHeight = height;
    
    if (pVia->shadowFB) {
        pVia->ShadowPitch = BitmapBytePad(pScrn->bitsPerPixel * width);
        pVia->ShadowPtr = xalloc(pVia->ShadowPitch * shadowHeight);
        displayWidth = pVia->ShadowPitch / (pScrn->bitsPerPixel >> 3);
        FBStart = pVia->ShadowPtr;
    }
    else {
        pVia->ShadowPtr = NULL;
        FBStart = pVia->FBStart;
    }    
    
    /* 
    *   STChen[2006.12.20]: Use Direct Access Window Rotate. 
    *   When screen will be rotated, we must report a rotated size to OS
    *   to generate a rotated screen data. 
    */
/*    switch (pBIOSInfo->HWRotate)
    {
        case VIA_ROTATE_DEGREE_90:
        case VIA_ROTATE_DEGREE_270:
            height = pScrn->virtualX;
            width = pScrn->virtualY;
            displayWidth = width;
            break;
    }
  */  
#ifdef USE_FB    
    ret = fbScreenInit(pScreen, FBStart, width, height,
                       pScrn->xDpi, pScrn->yDpi, displayWidth,
                       pScrn->bitsPerPixel);
#else    
    switch (pScrn->bitsPerPixel) {
    case 8:
        ret = cfbScreenInit(pScreen, FBStart, width, height, pScrn->xDpi,
                            pScrn->yDpi, displayWidth);
        break;

    case 16:
        ret = cfb16ScreenInit(pScreen, FBStart, width, height, pScrn->xDpi,
                              pScrn->yDpi, displayWidth);
        break;

    case 32:
        ret = cfb32ScreenInit(pScreen, FBStart, width, height, pScrn->xDpi,
                              pScrn->yDpi, displayWidth);
        break;

    default:
        xf86DrvMsg(scrnIndex, X_ERROR,
                   "Internal error: invalid bpp (%d) in SavageScreenInit\n",
                   pScrn->bitsPerPixel);
        ret = FALSE;
        break;
    }
#endif

    return ret;
}


static ModeStatus VIAValidMode(int scrnIndex, DisplayModePtr mode,
                               Bool verbose, int flags)
{
    ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
    VIAPtr      pVia = VIAPTR(pScrn);

    DEBUG(xf86DrvMsg(scrnIndex, X_INFO, "VIAValidMode\n"));

    /* check modes */
    switch (mode->CrtcHDisplay)
    {
        case 640:
            if (mode->CrtcVDisplay == 480)
            {
                return MODE_OK;
            }
            else
            {
                return MODE_BAD_VVALUE;
            }
            break;
        case 720:
            if (mode->CrtcVDisplay == 480 || mode->CrtcVDisplay == 576)
            {
                return MODE_OK;
            }
            else
            {
                return MODE_BAD_VVALUE;
            }
            break;
        case 800:
            if (mode->CrtcVDisplay == 600 || mode->CrtcVDisplay == 480)
            {
                return MODE_OK;
            }
            else
            {
                return MODE_BAD_VVALUE;
            }
            break;
        case 848:
            if (mode->CrtcVDisplay == 480)
            {
                return MODE_OK;
            }
            else
            {
                return MODE_BAD_VVALUE;
            }
            break;
        case 856:
            if (mode->CrtcVDisplay == 480)
            {
                return MODE_OK;
            }
            else
            {
                return MODE_BAD_VVALUE;
            }
            break;
        case 960:
            if (mode->CrtcVDisplay == 600)
            {
                return MODE_OK;
            }
            else
            {
                return MODE_BAD_VVALUE;
            }
            break;
        case 1000:
            if (mode->CrtcVDisplay == 600)
            {
                return MODE_OK;
            }
            else
            {
                return MODE_BAD_VVALUE;
            }
            break;
        case 1088:
            if (mode->CrtcVDisplay == 612)
            {
                return MODE_OK;
            }
            else
            {
                return MODE_BAD_VVALUE;
            }
            break;
        case 1024:
            if (mode->CrtcVDisplay == 512 || mode->CrtcVDisplay == 576 ||
                mode->CrtcVDisplay == 768  || mode->CrtcVDisplay == 600)
            {
                return MODE_OK;
            }
            else
            {
                return MODE_BAD_VVALUE;
            }
            break;
        case 1152:
            if ((mode->CrtcVDisplay == 864) || (mode->CrtcVDisplay == 720))
            {
                return MODE_OK;
            }
            else
            {
                return MODE_BAD_VVALUE;
            }
            break;
        case 1200:
            if (mode->CrtcVDisplay == 720)
            {
                return MODE_OK;
            }
            else
            {
                return MODE_BAD_VVALUE;
            }
            break;
        case 1280:
            if (mode->CrtcVDisplay == 720 || mode->CrtcVDisplay == 768 || mode->CrtcVDisplay == 800 ||
                mode->CrtcVDisplay == 960 || mode->CrtcVDisplay == 1024 || mode->CrtcVDisplay == 600)
            {
                return MODE_OK;
            }
            else
            {
                return MODE_BAD_VVALUE;
            }
            break;
        case 1366:
            if (mode->CrtcVDisplay == 768)
            {
                return MODE_OK;
            }
            else
            {
                return MODE_BAD_VVALUE;
            }
            break;
        case 1360:
            if (mode->CrtcVDisplay == 768)
            {
                return MODE_OK;
            }
            else
            {
                return MODE_BAD_VVALUE;
            }
            break;
        case 1400:
            if (mode->CrtcVDisplay == 1050)
            {
                return MODE_OK;
            }
            else
            {
                return MODE_BAD_VVALUE;
            }
            break;
        case 1440:
            if ((mode->CrtcVDisplay == 1050) || (mode->CrtcVDisplay == 900))
            {
                return MODE_OK;
            }
            else
            {
                return MODE_BAD_VVALUE;
            }
            break;
        case 1600:
            if ((mode->CrtcVDisplay == 1200) || (mode->CrtcVDisplay == 900) || 
                (mode->CrtcVDisplay == 1024))
            {
                return MODE_OK;
            }
            else
            {
                return MODE_BAD_VVALUE;
            }
            break;
        case 1680:
            if ((mode->CrtcVDisplay == 1050) || (mode->CrtcVDisplay == 600))
            {
                return MODE_OK;
            }
            else
            {
                return MODE_BAD_VVALUE;
            }
            break;
        case 1792:
            if (mode->CrtcVDisplay == 1344)
            {
                return MODE_OK;
            }
            else
            {
                return MODE_BAD_VVALUE;
            }
            break;
        case 1856:
            if (mode->CrtcVDisplay == 1392)
            {
                return MODE_OK;
            }
            else
            {
                return MODE_BAD_VVALUE;
            }
            break;
        case 1920:
            if ((pVia->Chipset != VIA_CLE266) &&
                ((mode->CrtcVDisplay == 1440) || 
                 (mode->CrtcVDisplay == 1080) ||
                 (mode->CrtcVDisplay == 1200)))
            {
                return MODE_OK;
            }
            else
            {
                return MODE_BAD_VVALUE;
            }
            break;
        case 2048:
            if (mode->CrtcVDisplay == 1536)
            {
                return MODE_OK;
            }
            else
            {
                return MODE_BAD_VVALUE;
            }
            break;
    }

    return MODE_BAD_HVALUE;
}


static void VIABIOSInit(VIAPtr pVia, ScrnInfoPtr pScrn, DisplayModePtr pMode)
{
    int             i;
    VIABIOSInfoPtr pBIOSInfo = pVia->pBIOSInfo;
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIABIOSInit begin\n"));

    pBIOSInfo->bitsPerPixel = pScrn->bitsPerPixel;
    pBIOSInfo->displayWidth = pScrn->displayWidth;
    pBIOSInfo->frameX1 = pScrn->frameX1;
    pBIOSInfo->frameY1 = pScrn->frameY1;
    pBIOSInfo->VirtualX = pScrn->virtualX;
    pBIOSInfo->VirtualY = pScrn->virtualY;
    pBIOSInfo->scrnIndex = pScrn->scrnIndex;
    pBIOSInfo->IGA1SettingInfo.IsActive = FALSE;
    pBIOSInfo->IGA2SettingInfo.IsActive = FALSE;

    pBIOSInfo->Clock = pMode->Clock;
    pBIOSInfo->HTotal = pMode->HTotal;
    pBIOSInfo->VTotal = pMode->VTotal;
    pBIOSInfo->HDisplay = pMode->HDisplay;
    pBIOSInfo->VDisplay = pMode->VDisplay;
    pBIOSInfo->CrtcHDisplay = pMode->CrtcHDisplay;
    pBIOSInfo->CrtcVDisplay = pMode->CrtcVDisplay;

    if (pBIOSInfo->FirstInit)
    {
        pBIOSInfo->SaveframeX1 = pScrn->frameX1;
        pBIOSInfo->SaveframeY1 = pScrn->frameY1;
        pBIOSInfo->SaveHDisplay = pMode->HDisplay;
        pBIOSInfo->SaveVDisplay = pMode->VDisplay;
        pBIOSInfo->SaveCrtcHDisplay = pMode->CrtcHDisplay;
        pBIOSInfo->SaveCrtcVDisplay = pMode->CrtcVDisplay;
        pBIOSInfo->SaveVirtualX = pScrn->virtualX;
        pBIOSInfo->SaveVirtualY = pScrn->virtualY;
    }

    pBIOSInfo->IsSecondary = pVia->IsSecondary;
    pBIOSInfo->HasSecondary = pVia->HasSecondary;

    if (pBIOSInfo->Hotkey)
    {
        pBIOSInfo->VIAModeList = pVia->VIAModeList;
    }
    else
    {
        for (i = 0; i < 0xFF; i++)
        {
            pBIOSInfo->TVRegs[i] = pVia->SavedReg.TVRegs[i];
        }
    }
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "pBIOSInfo:bitsPerPixel=%d,displayWidth=%d,frameX1=%d,frameY1=%d\n",pBIOSInfo->bitsPerPixel,pBIOSInfo->displayWidth,pBIOSInfo->frameX1,pBIOSInfo->frameY1));     
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "pBIOSInfo:,VirtualX=%d,VirtualY=%d\n",pBIOSInfo->VirtualX,pBIOSInfo->VirtualY)); 
    
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "pMode:HTotal=%d,VTotal=%d,HDisplay=%d,VDisplay=%d\n",pMode->HTotal,pMode->VTotal,pMode->HDisplay,pMode->VDisplay));     
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "pMode:,CrtcHDisplay=%d,CrtcVDisplay=%d\n",pMode->CrtcHDisplay,pMode->CrtcVDisplay)); 
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIABIOSInit exit\n"));    
}


static void VIAPostFindMode(VIAPtr pVia, ScrnInfoPtr pScrn, DisplayModePtr pMode)
{
    VIABIOSInfoPtr pBIOSInfo = pVia->pBIOSInfo;
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIAPostFindMode begin\n"));

    pVia->ModeReg.mode = pBIOSInfo->mode;
    pVia->ModeReg.resMode = pBIOSInfo->resMode;
    pVia->ModeReg.refresh = pBIOSInfo->Refresh;
    pVia->ModeReg.offsetWidthByQWord = pBIOSInfo->offsetWidthByQWord;
    pVia->ModeReg.countWidthByQWord = pBIOSInfo->countWidthByQWord;

    pScrn->frameX0 = 0;
    pScrn->frameY0 = 0;
    pScrn->frameX1 = pBIOSInfo->frameX1;
    pScrn->frameY1 = pBIOSInfo->frameY1;


    if (pBIOSInfo->ActualDesktopSizeX > pBIOSInfo->HDisplay)
    {
        pMode->HDisplay = pBIOSInfo->HDisplay;
    }
    else
    {
        pMode->HDisplay = pBIOSInfo->ActualDesktopSizeX;
    }

    if (pBIOSInfo->ActualDesktopSizeY > pBIOSInfo->VDisplay)
    {
        pMode->VDisplay = pBIOSInfo->VDisplay;
    }
    else
    {
        pMode->VDisplay = pBIOSInfo->ActualDesktopSizeY;
    }
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "pBIOSInfo:mode=%d,resMode=%d,offsetWidthByQWord=%d,countWidthByQWord=%d\n",pBIOSInfo->mode,pBIOSInfo->resMode,pBIOSInfo->offsetWidthByQWord,pBIOSInfo->countWidthByQWord));     
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "pBIOSInfo:,frameX1=%d,frameY1=%d,HDisplay=%d,VDisplay=%d\n",pBIOSInfo->frameX1,pBIOSInfo->frameY1,pBIOSInfo->HDisplay,pBIOSInfo->VDisplay)); 
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "pBIOSInfo:,ActualDesktopSizeX=%d,ActualDesktopSizeY=%d\n",pBIOSInfo->ActualDesktopSizeX,pBIOSInfo->ActualDesktopSizeY)); 
    
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIAPostFindMode exit\n"));    
}


static void
VIACorrectModeList(ScrnInfoPtr pScrn)
{
    DisplayModePtr mode;

    for (mode = pScrn->modes; ; mode = mode->next)
    {
        VIAGetModeSizeByName(mode->name, &mode->HDisplay, &mode->VDisplay);

        if (mode->next == pScrn->modes)
        {
            break;  /* The end of mode list. */
        }
    }
}


static void
VIAModeInitRunOnce(ScrnInfoPtr pScrn)
{
    VIAPtr          pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;

    /* RainLin [2007.01.16]: All the things can be added here if they just need to initialize once */
    /* until the system suspends or logout. */
    VIATMDSRegInit(pBIOSInfo);
}


static Bool VIAModeInit(ScrnInfoPtr pScrn, DisplayModePtr mode)
{
    vgaHWPtr        hwp = VGAHWPTR(pScrn);
    VIAPtr          pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;
    VIARegPtr       new = &pVia->ModeReg;
    vgaRegPtr       vganew = &hwp->ModeReg;
    int             fd;
    PVIDDATA        pVidData = pVia->pVidData;

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIAModeInit\n"));

    /* RainLin [2006.08.01]: Because we may update HDisplay/VDisplay of some mode for */
    /* panning issue in the previous mode setting, so we'd better restore them to fit */
    /* the real mode size first, or it may cause problems when that mode is not in use, */
    /* especially when the mode setting is called by RandR. */
    /* For example, CRT+DVI (panel size is 1280x1024), when we set to 1920x1080, this mode */
    /* will be dropped from RandR's mode list until we re-login X. */
    VIACorrectModeList(pScrn);
    
    /*Al Zhang[2006.04.24]: If MV enabled, block mode setting*/
    if(CheckMacrovisionEnabled(pBIOSInfo) && 
       (pBIOSInfo->ActiveDevice& VIA_DEVICE_TV))
    {
        return FALSE;
    }
    /* fd = open("/dev/video2",O_RDWR);*/
    fd = open(pVia->video2Path ,O_RDWR);
    /* memset(&UpdateOverlay,0,sizeof(UpdateOverlay)); */

    if (!vgaHWInit(pScrn, mode))
    {
        vgaHWBlankScreen(pScrn, TRUE);
        return FALSE;
    }

    pScrn->vtSema = TRUE;

    if (!pVia->IsVIAModeInitRunOnceDone)
    {
        VIAModeInitRunOnce(pScrn);

        /* Just run once. */
        pVia->IsVIAModeInitRunOnceDone = TRUE;
    }

    VIABIOSInit(pVia, pScrn, mode);
    
    /* load utility user active device setting */
    /* RainLin [2005.12.05]: We also need to load user settings in SAMM case, */
    /* so I move this call to here. */
    VIALoadUserSetting(pBIOSInfo);
    
    if( pVia->DISP3DScaling)
    {
        pBIOSInfo->Is3DScaling = FALSE;
        if (0!=(pBIOSInfo->ActiveDevice & (VIA_DEVICE_TV+VIA_DEVICE_TV2)))
        {
            pBIOSInfo->Is3DScaling = TRUE;
        }
        else
        {
            pBIOSInfo->Is3DScalingEnable = FALSE;
        }
    }
    
    if(!pBIOSInfo->SAMM)
    {        
        VIASetDisplayPath(pScrn);

        /* RainLin [2007.01.12]: There is no need to reinitialize the display path of video */
        /* when the system resumes from suspends, just keep the user settings. */
        if (pBIOSInfo->DuoView && (!pVia->IsSuspending))
        {
            VIADeviceDispatchDuoView(pScrn);
        }

        if (pBIOSInfo->ActiveDevice & VIA_DEVICE_CRT2)/*ian*/
        { 
          pBIOSInfo->TwoCRT = TRUE;
        } 
        else
          pBIOSInfo->TwoCRT = FALSE;

        /* RainLin [2006.03.23]: Check what type of cursor (SW/HW) to use. */
        VIACheckCursorTypeToUse(pScrn);
    }

    /* load utility user TV setting */
    /* change by keatexi [20005.10.12} for SAMM mode can support save TV setting*/
    if (pBIOSInfo->ActiveDevice & VIA_DEVICE_TV)
    {
        VIALoadUserTVSetting(pBIOSInfo, &(pBIOSInfo->TVSettingInfo), "/.VIATVRC");
        xf86DrvMsg(pScrn->scrnIndex, X_INFO,
		   "VIALoadUserTVSetting:TVtype=%d, TVScan=%d\n",
		   pBIOSInfo->TVSettingInfo.TVType,
		   pBIOSInfo->TVSettingInfo.TVVScan);       

        if(pVia->DISP3DScaling)
        {
            if ((TVTYPE_1080I==pBIOSInfo->TVSettingInfo.TVType )|(TVTYPE_720P==pBIOSInfo->TVSettingInfo.TVType))
            {
                pBIOSInfo->TVSettingInfo.TVVScan = VIA_TVNORMAL;
            }
            else
            {
                pBIOSInfo->TVSettingInfo.TVVScan = VIA_TVOVER;
            }

        }
    }
    
    if (pBIOSInfo->ActiveDevice & VIA_DEVICE_TV2)
    {
        VIALoadUserTVSetting(pBIOSInfo, &(pBIOSInfo->TVSettingInfo2), "/.VIATV2RC"); 
    }
    
    /* Set DuoView Video Output Device of S3Utility setting*/
    VIALoadUserDuoViewVideoOutputDeviceSetting(pScrn);
    xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIALoadUserDuoViewVideoOutputDeviceSetting exit:TVtype=%d, TVScan=%d\n",pBIOSInfo->TVSettingInfo.TVType, pBIOSInfo->TVSettingInfo.TVVScan);       

    if (pBIOSInfo->Hotkey)
    {
        if (!VIAFindModeUseModeList(pBIOSInfo))
        {
            vgaHWBlankScreen(pScrn, TRUE);
            VIAFreeRec(pScrn);
            return FALSE;
        }
    }
    else
    {
        if (!VIAFindModeUseBIOSTable(pBIOSInfo))
        {
            vgaHWBlankScreen(pScrn, TRUE);
            VIAFreeRec(pScrn);
            return FALSE;
        }
    }
    xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIAFindModeUseBIOSTable exit:TVtype=%d, TVScan=%d\n",pBIOSInfo->TVSettingInfo.TVType, pBIOSInfo->TVSettingInfo.TVVScan);       

    VIAPostFindMode(pVia, pScrn, mode);

    pBIOSInfo->IsStableStatus=FALSE;
    if( pVia->DISP3DScaling)
    {
        if (pBIOSInfo->Is3DScaling)
        {
            VIADISP3DScalingParasSetting(pScrn); 
        }
    }        
    xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIADISP3DScalingParasSetting exit:TVtype=%d, TVScan=%d\n",pBIOSInfo->TVSettingInfo.TVType, pBIOSInfo->TVSettingInfo.TVVScan);       

    /* Enable Direct Access Window Rotate. */
/*    if (pVia->IsHWRotateEnabled) {
        EnableDirectAccessWindow(pScrn);
    }*/

    /* FIXME - need DRI lock for this bit - see i810 */
    /* do it! */
    VIAWriteMode(pScrn, vganew, new);
    xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIAWriteMode exit:TVtype=%d, TVScan=%d\n",pBIOSInfo->TVSettingInfo.TVType, pBIOSInfo->TVSettingInfo.TVVScan);       
    pBIOSInfo->IsStableStatus=TRUE;

    /* pass graphic info to via_v4l kernel module */
    /* Coz mode changes, some member in VIAGraphicInfo need to modify */
    FillGraphicInfo(pScrn);

    if( -1 == fd)
    {
        xf86DrvMsg(pScrn->scrnIndex, X_ERROR, " Couldn't open \"/dev/video2\"\n");
    }

    if (pVia->V4LEnabled == 1)
    { 
        if ( -1 == ioctl(fd, VIA_VID_SET_2D_INFO, pVidData) )
        {
            xf86DrvMsg(pScrn->scrnIndex, X_ERROR, "Can't get through IOCTL to V4L\n");
        }
    }
    else
    {
        DBG_DD(("SWOV:  VIAVidSet2DInfo\n"));

        /* Save MCLK value */
        VGAOUT8(0x3C4, 0x16); pVidData->Save_3C4_16 = VGAIN8(0x3C5);
        DBG_DD(("        3c4.16 : %08x \n",VGAIN8(0x3C5)));
        VGAOUT8(0x3C4, 0x17); pVidData->Save_3C4_17 = VGAIN8(0x3C5);
        DBG_DD(("        3c4.17 : %08x \n",VGAIN8(0x3C5)));
        VGAOUT8(0x3C4, 0x18); pVidData->Save_3C4_18 = VGAIN8(0x3C5);
        DBG_DD(("        3c4.18 : %08x \n",VGAIN8(0x3C5)));
    }

    close(fd);

    VIAAdjustFrame(pScrn->scrnIndex, pScrn->frameX0, pScrn->frameY0, 0);
    xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIAModeInit exit:TVtype=%d, TVScan=%d\n",pBIOSInfo->TVSettingInfo.TVType, pBIOSInfo->TVSettingInfo.TVVScan);       

    return TRUE;
}


static
Bool VIACloseScreen(int scrnIndex, ScreenPtr pScreen)
{
    ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
    vgaHWPtr    hwp = VGAHWPTR(pScrn);
    VIAPtr      pVia = VIAPTR(pScrn);
    vgaRegPtr   vgaSavePtr = &hwp->SavedReg;
    VIARegPtr   viaSavePtr = &pVia->SavedReg;
    CARD32      dwCursorMode;
    int         fd;
    PVIDDATA pVidData = pVia->pVidData;
    LPVIAGRAPHICINFO lpVIAGraphicInfo = &pVidData->VIAGraphicInfo;

    /*Homer, when log out X, if DRI is enabled, the DRI resource should be cleaned up*/
    if(pVia->directRenderingEnabled)
    {
        VIADRICloseScreen(pScreen);
    }

    DEBUG(xf86DrvMsg(scrnIndex, X_INFO, "VIACloseScreen\n"));
    xf86DrvMsg(scrnIndex, X_INFO, "VIACloseScreen\n");

    /* Clear direct access window. */
    /*DisableDirectAccessWindow(pScrn);*/
    
    /* For utility api */
    VIADisplayExtUnregister(pScrn);

    /*Turn off color cursor here*/
    VIASETREG(PRIM_HI_CTRL, 0);
    VIASETREG(HI_CONTROL, 0);
    /*=* Carl Liu [2004.10.26]
         When you open two X-Window and use "kill PID" to kill secondary X-Window process.
         Then system will hang after few seconds. So patch for above situation.
         If not really exit X-Window, do this patch *=*/ 
    if (pScrn->vtSema)
    {
    	/* JasonM Chang move exit video here if exitvideo after restore SR1A (PCI reset)
    	   the V3 command fire may stick and caused system hang when playing video */
        /* Turn off all video activities */
        viaExitVideo(pScrn);
        if(!H5S1_UMA_CHIPID)/*H5 needn't reset PCI*/
        {
            /*=* John Sheng [2003.10.17] Patch for normal log out and restart X, 3D application will hang *=*/
            VGAOUT8(0x3c4, SR1A);
            VGAOUT8(0x3c5, viaSavePtr->SRRegs[0x1A] | 0x40);
        }
    }
 
    if (!pVia->IsSecondary)
    {
        /* pass graphic info to via_v4l kernel module */
        /* fd = open("/dev/video2",O_RDWR);*/
        fd = open(pVia->video2Path ,O_RDWR);

        if( -1 == fd)
        {
            xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
                       " Couldn't open \"/dev/video2\"\n");
        }

        if (pVia->V4LEnabled == 1)
        {
    
            if ( -1 == ioctl(fd, VIAMGR_INFO_XSERVEROFF, lpVIAGraphicInfo) )
            {
                xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
                           "Can't get through IOCTL to V4L\n");
            }
	    close(fd);
        } else {
          /* clean viddata structure*/
            int i;

            /* Reset panning mode parameter */
            /*Modified for SAMM*/
            for (i=0;i<2;i++)
            {
                pVidData->panning_x[i] = 0;
                pVidData->panning_y[i] = 0;
                pVidData->panning_old_x[i] = 0;
                pVidData->panning_old_y[i] = 0;
            }

            lpVIAGraphicInfo->dwXServerEnabled = FALSE;
            xf86DrvMsg(scrnIndex, X_INFO, "VIACloseScreen, lpVIAGraphicInfo->dwXServerEnabled = FALSE\n");            
            lpVIAGraphicInfo->DRMEnabled = FALSE;
	}        

        /* Diable Hardware Cursor */
        dwCursorMode = VIAGETREG(VIA_REG_CURSOR_MODE);
        VIASETREG(VIA_REG_CURSOR_MODE, dwCursorMode & 0xFFFFFFFE);
    }

    /* Wait Hardware Engine idle to exit graphic mode */
    WaitIdle();

    /*=* Carl Liu [2004.10.26]
         When you open two X-Window and use "kill PID" to kill secondary X-Window process.
         Then system will hang after few seconds. So patch for above situation.
         If not really exit X-Window, do this patch *=*/ 
    if ((pVia->VQEnable)&&(pScrn->vtSema))
    {
        /* if we use VQ, disable it before we exit */
/*        if ((pVia->Chipset != VIA_K8M890) && 
            (pVia->Chipset != VIA_P4M890) && 
            (pVia->Chipset != VIA_CN750) && 
            (pVia->Chipset != VIA_P4M900))  
        {
            VIASETREG(0x43c, 0x00fe0000);
            VIASETREG(0x440, 0x00000004);
            VIASETREG(0x440, 0x40008c0f);
            VIASETREG(0x440, 0x44000000);
            VIASETREG(0x440, 0x45080c04);
            VIASETREG(0x440, 0x46800408);
        } 
*/
        /* if we use VQ, disable it before we exit */
        xf86DrvMsg(scrnIndex, X_INFO, "VIACloseScreen, VQ, disable \n");            
        
         switch( pVia->Chipset )   
         { 
         /*=* Lobo Kong [2007.4.10]When disalbe VQ 336/364 platform will hang   *=*/
          case VIA_P4M900: 
          case VIA_K8M890:
          case VIA_CN750:
          /*
               VIASETREG(0x41c, 0x00100000);
               VIASETREG(0x420, 0x74301000);
          */               
            break;
          default:  
               VIASETREG(0x43c, 0x00fe0000);
               VIASETREG(0x440, 0x00000004);
               VIASETREG(0x440, 0x40008c0f);
               VIASETREG(0x440, 0x44000000);
               VIASETREG(0x440, 0x45080c04);
               VIASETREG(0x440, 0x46800408);
            break;
         } 

    }



    if (pVia->CursorImage)
    {
        xfree(pVia->CursorImage);
        pVia->CursorImage = NULL;
    }
    if (pVia->AccelInfoRec)
    {
        XAADestroyInfoRec(pVia->AccelInfoRec);
        pVia->AccelInfoRec = NULL;
    }
    if (pVia->CursorInfoRec)
    {
        xf86DestroyCursorInfoRec(pVia->CursorInfoRec);
        pVia->CursorInfoRec = NULL;
    }
    if (pVia->ShadowPtr)
    {
        xfree(pVia->ShadowPtr);
        pVia->ShadowPtr = NULL;
    }
    if (pVia->DGAModes)
    {
        xfree(pVia->DGAModes);
        pVia->DGAModes = NULL;
    }
    if (pScrn->vtSema)
    {
    
    xf86DrvMsg(scrnIndex, X_INFO, "VIACloseScreen, pScrn->vtSema is true \n");            
        VIAWriteMode(pScrn, vgaSavePtr, viaSavePtr);
        vgaHWLock(hwp);
        VIAUnmapMem(pScrn);
        vgaHWUnmapMem(pScrn);
    }

    /* STChen[2007.3.8]: For MAMM*/
    if (!pVia->IsMAMMEnable)
    {
        if(pVia->pInt10)
        {
            xf86FreeInt10(pVia->pInt10);
            pVia->pInt10 = NULL;
        }
    }
    pVia->IsVIAModeInitRunOnceDone = FALSE;
    pScrn->vtSema = FALSE;
    pScreen->CloseScreen = pVia->CloseScreen;

    return (*pScreen->CloseScreen)(scrnIndex, pScreen);
}

/*
 * This only gets called when a screen is being deleted.  It does not
 * get called routinely at the end of a server generation.
 */
static void VIAFreeScreen(int scrnIndex, int flags)
{
    ScrnInfoPtr pScrn = xf86Screens[scrnIndex];    
    VIAPtr      pVia = VIAPTR(pScrn);
    
    DEBUG(xf86DrvMsg(scrnIndex, X_INFO, "VIAFreeScreen\n"));
    xf86DrvMsg(scrnIndex, X_INFO, "VIAFreeScreen\n");
    /* delete the shared memory of VidData */
    if (shmdt((void *)pVia->pVidData) == -1)
    {
        DBG_DD(("shmdt failed\n"));
    }

    if (shmctl(pVia->shmid, IPC_RMID, 0) == -1)
    {
        DBG_DD(("shmctl(IPC_RMID) failed\n"));
    }
    
    if (xf86LoaderCheckSymbol("vgaHWFreeHWRec"))
    vgaHWFreeHWRec(xf86Screens[scrnIndex]);
    VIAFreeRec(xf86Screens[scrnIndex]);
}

static Bool VIASaveScreen(ScreenPtr pScreen, int mode)
{
    return vgaHWSaveScreen(pScreen, mode);
}

static int
VIAGetIGAInUse(VIABIOSInfoPtr pBIOSInfo)
{
    switch (pBIOSInfo->ActiveDevice)
    {
        case VIA_DEVICE_CRT1:
            return pBIOSInfo->CRTSettingInfo.IGAPath;

        case VIA_DEVICE_LCD:
            return pBIOSInfo->LVDSSettingInfo.IGAPath;

        case VIA_DEVICE_DFP:
            return pBIOSInfo->TMDSSettingInfo.IGAPath;

        case VIA_DEVICE_TV:
            return pBIOSInfo->TVSettingInfo.IGAPath;

        case VIA_DEVICE_TV2:
            return pBIOSInfo->TVSettingInfo2.IGAPath;

        case VIA_DEVICE_CRT2:
            return pBIOSInfo->CRT2_IGA;

        default:
            /* Wrong Device ID! */
            return 0;
    }
}

static void
VIASetStartingAddress(ScrnInfoPtr pScrn, int x, int y)
{
    vgaHWPtr        hwp = VGAHWPTR(pScrn);
    VIAPtr          pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;
    int             vgaCRIndex, vgaCRReg, vgaIOBase;
    CARD32          tmp, Base, tmpBase;
    Bool            bAdjustPrimaryStartAddr, bAdjustSecondaryStartAddr;
    unsigned long HalfScreen;
    int HalfLine;
    vgaIOBase = hwp->IOBase;
    vgaCRIndex = vgaIOBase + 4;
    vgaCRReg = vgaIOBase + 5;

    Base = (y * pScrn->displayWidth + x) * (pScrn->bitsPerPixel / 8);
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIASetStartingAddress begin\n"));
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIASetStartingAddress:pScrn->fbOffset=%d,Base=%d\n",pScrn->fbOffset,Base));

    /* 'bAdjustSecondaryStartAddr' may be used uninitialized in this function */
    bAdjustSecondaryStartAddr = FALSE;

    if(pBIOSInfo->UseIGA2 && pBIOSInfo->TwoLCD)/*for toshiba case support two lcd on cle266*/
    {
        if (!pVia->IsSecondary)
        {
            Base = Base >> 1;
            VGAOUT8(vgaCRIndex, CR62);
            tmp = VGAIN8(vgaCRReg) & 0x01;
            tmp |= (Base & 0x7F) << 1;
            VGAOUT8(vgaCRReg, tmp);
            VGAOUT8(vgaCRIndex, CR63);
            VGAOUT8(vgaCRReg, ((Base & 0x7F80) >> 7));
            VGAOUT8(vgaCRIndex, CR64);
            VGAOUT8(vgaCRReg, ((Base & 0x7F8000) >> 15));
            VGAOUT8(vgaCRIndex, CRA3);
            VGAOUT8(vgaCRReg, ((Base & 0x03800000) >> 23));
        }
        else
        {
            Base = (Base + pScrn->fbOffset) >> 1;
            VGAOUT16(vgaCRIndex, (Base & 0x00ff00) | CR0C);
            VGAOUT16(vgaCRIndex, ((Base & 0x00ff) << 8) | CR0D);
            VGAOUT16(vgaCRIndex, ((Base & 0xff0000) >> 8) | CR34);
            VGAOUT16(vgaCRIndex, ((Base & 0xff000000) >> 16) | CR48);
        }
    } 
    else
    {
        /* Check which starting should be updated: */
        if (pBIOSInfo->UseIGA2)
        {
            bAdjustPrimaryStartAddr = FALSE;
            bAdjustSecondaryStartAddr = TRUE;
        }
        else if (!pBIOSInfo->DuoView || pBIOSInfo->SAMM)
        {
            /* For single and SAMM case: */
            if (VIAGetIGAInUse(pBIOSInfo) == IGA1)
            {                
                bAdjustPrimaryStartAddr = TRUE;
                bAdjustSecondaryStartAddr = FALSE;
            }
            else
            {                
                bAdjustPrimaryStartAddr = FALSE;
                bAdjustSecondaryStartAddr = TRUE;
            }
        }
        else
        {
            /* We will set starting address when set mode or panning. */
            if (pBIOSInfo->IGA1SettingInfo.SetStartingAddrFirstTime)
            {
                bAdjustPrimaryStartAddr = TRUE;
                pBIOSInfo->IGA1SettingInfo.SetStartingAddrFirstTime = FALSE;
            }
            else
            {
                bAdjustPrimaryStartAddr = pBIOSInfo->IGA1SettingInfo.IsPanning;
            }

            if (pBIOSInfo->IGA2SettingInfo.SetStartingAddrFirstTime)
            {
                bAdjustPrimaryStartAddr = TRUE;
                pBIOSInfo->IGA2SettingInfo.SetStartingAddrFirstTime = FALSE;
            }
            else
            {
                bAdjustSecondaryStartAddr = pBIOSInfo->IGA2SettingInfo.IsPanning;
            }
        }

        /* Decide which image (primary or secondary) to show: */
        /* We will show primary image except the secondary device in SAMM case.  */
        if (pBIOSInfo->SAMM && (pBIOSInfo->ActiveDevice != pBIOSInfo->PrimaryDevice))
        {
            Base = Base + pScrn->fbOffset;
        }

        if (bAdjustPrimaryStartAddr)
        {
            tmpBase = Base >> 1;
            
            if(pBIOSInfo->MergedFB)
            {
                /* add by cliff set IGA1 start address while mergedfb*/
                if((pBIOSInfo->Scrn2Position == viaRightOf)||(pBIOSInfo->Scrn2Position == viaBelow))
                {
          
                    VGAOUT16(vgaCRIndex,  0 | CR0C);
                    VGAOUT16(vgaCRIndex,  0 | CR0D);
                    VGAOUT16(vgaCRIndex,  0 | CR34);
                }
                else if(pBIOSInfo->Scrn2Position == viaLeftOf)
                {
                    HalfLine = pBIOSInfo->HalfLine*2;  
                    VGAOUT16(vgaCRIndex, (HalfLine & 0x00FF00) | CR0C);
                    VGAOUT16(vgaCRIndex, ((HalfLine& 0x0000FF) << 8) | CR0D);
                    VGAOUT16(vgaCRIndex, ((HalfLine & 0xFF0000) >> 8) | CR34);
                }
                else if(pBIOSInfo->Scrn2Position == viaAbove)
                {
                    HalfScreen = pBIOSInfo->HalfScreen*2;  
                    VGAOUT16(vgaCRIndex, (HalfScreen & 0x00FF00) | CR0C);
                    VGAOUT16(vgaCRIndex, ((HalfScreen& 0x0000FF) << 8) | CR0D);
                    VGAOUT16(vgaCRIndex, ((HalfScreen & 0xFF0000) >> 8) | CR34);
                    VGAOUT16(vgaCRIndex, ((HalfScreen & 0x03000000) >> 16) | CR48);
                }
            }
            else
            {
                VGAOUT16(vgaCRIndex, (tmpBase & 0x00ff00) | CR0C);
                VGAOUT16(vgaCRIndex, ((tmpBase & 0x00ff) << 8) | CR0D);
                VGAOUT16(vgaCRIndex, ((tmpBase & 0xff0000) >> 8) | CR34);
                VGAOUT16(vgaCRIndex, ((tmpBase & 0x03000000) >> 16) | CR48);
            }
        }
        
        if (bAdjustSecondaryStartAddr)
        {
            tmpBase = Base >> 3;
            
            VGAOUT8(vgaCRIndex, CR62);

            /* VansonWu [2004.1.9] patch VT3259 unknow panning bug temporary */
            if((pBIOSInfo->Chipset == VIA_PM800)||(pBIOSInfo->Chipset == VIA_P4M800PRO)||(pBIOSInfo->Chipset == VIA_K8M890)||(pBIOSInfo->Chipset == VIA_P4M890))
            {
                tmp = VGAIN8(vgaCRReg) & 0x01;
                tmp |= (tmpBase & 0x7E) << 1;
            }
            else
            {
                tmp = VGAIN8(vgaCRReg) & 0x01;
                tmp |= (tmpBase & 0x7F) << 1;
            }
            
            if(pBIOSInfo->MergedFB)
            {
                /* add by cliff set IGA2 start address while mergedfb*/
                if(pBIOSInfo->Scrn2Position == viaRightOf)
                {
                    HalfLine =  pBIOSInfo->HalfLine;      
                    VGAOUT8(vgaCRReg, HalfLine & 0xFF);
                    VGAOUT8(vgaCRIndex, CR63);
                    VGAOUT8(vgaCRReg, ((HalfLine & 0xFF00) >>8));
                    VGAOUT8(vgaCRIndex, CR64);
                    VGAOUT8(vgaCRReg, ((HalfLine & 0xFF0000) >> 16));
                }
                else if(pBIOSInfo->Scrn2Position == viaBelow)
                {        
                    HalfScreen =  pBIOSInfo->HalfScreen;  
                    VGAOUT8(vgaCRReg, HalfScreen & 0xFF);
                    VGAOUT8(vgaCRIndex, CR63);
                    VGAOUT8(vgaCRReg, ((HalfScreen & 0xFF00) >>8));
                    VGAOUT8(vgaCRIndex, CR64);
                    VGAOUT8(vgaCRReg, ((HalfScreen & 0xFF0000) >> 16));
                    VGAOUT8(vgaCRIndex, CRA3);
                    VGAOUT8(vgaCRReg, ((HalfScreen & 0xFF000000) >> 24));
                }
                else if((pBIOSInfo->Scrn2Position == viaAbove)||(pBIOSInfo->Scrn2Position == viaLeftOf))
                {
                    VGAOUT8(vgaCRReg, 0);
                    VGAOUT8(vgaCRIndex, CR63);
                    VGAOUT8(vgaCRReg, 0);
                    VGAOUT8(vgaCRIndex, CR64);
                    VGAOUT8(vgaCRReg, 0);
                }
            }
            else
            {
                VGAOUT8(vgaCRReg, tmp);
                VGAOUT8(vgaCRIndex, CR63);
                VGAOUT8(vgaCRReg, ((tmpBase & 0x7F80) >> 7));
                VGAOUT8(vgaCRIndex, CR64);
                VGAOUT8(vgaCRReg, ((tmpBase & 0x7F8000) >> 15));
                VGAOUT8(vgaCRIndex, CRA3);
                VGAOUT8(vgaCRReg, ((tmpBase & 0x03800000) >> 23));
            }
        }        
    } 

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIASetStartingAddress:pScrn->fbOffset=%d,Base=%d\n",pScrn->fbOffset,Base));
    if (pBIOSInfo->Is3DScaling)
    {
        tmp = pVia->DISP3DSCALCTRLInfo.DISP3DScalingBufAddr[pVia->DISP3DSCALCTRLInfo.DISP3DScalingBufIndex];
        pVia->DISP3DSCALCTRLInfo.PanningX = x;
        pVia->DISP3DSCALCTRLInfo.PanningX = y;
        switch(pVia->DISP3DSCALCTRLInfo.DISP3DScalIGAPath)
        {
            case IGA1:
                VIASETREG(0x214, tmp>>1);
                break;
            case IGA2:
                VIASETREG(0x218, tmp);
                break;
        }
    }
}

void VIAAdjustFrame(int scrnIndex, int x, int y, int flags)
{
    ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
    VIAPtr      pVia = VIAPTR(pScrn);
    int         fd0 = -1, fd1 = -1, fd2 = -1;
    /* LPDDLOCK    lpddLock = &(pVidData->ddLock); */
    DDUPDATEOVERLAY UpdateOverlayRec;
    ADJUSTFRAME AdjustFrame;
    PVIDDATA pVidData = pVia->pVidData;
    unsigned long handle = pVidData->VMI_Info.handle;
    struct video_window      ov_win_cap1;
    struct video_window      ov_win_cap0;
    LPOVERLAYRECORD lpOverlayRecordTV0 = &pVidData->OverlayRecordSet.OverlayRecordTV0;
    LPOVERLAYRECORD lpOverlayRecordTV1 = &pVidData->OverlayRecordSet.OverlayRecordTV1;
    LPOVERLAYRECORD lpOverlayRecordMPEG = NULL;
    LPOVERLAYRECORD lpOverlayRecordSW = &pVidData->OverlayRecordSet.OverlayRecordSW[handle];
    LPVIAGRAPHICINFO lpVIAGraphicInfo = &pVidData->VIAGraphicInfo; 
    unsigned int isCapture1Enabled = 0; 

    DEBUG(xf86DrvMsg(scrnIndex, X_INFO, "VIAAdjustFrame\n"));

    /* now program the start address registers*/
    VIASetStartingAddress(pScrn, x, y);

    /* add for vt3324,3293 capture1*/
    if(( lpVIAGraphicInfo->dwProjectIDRevision == UNICHROME_VT3324) ||(lpVIAGraphicInfo->dwProjectIDRevision == UNICHROME_VT3324M)  ||
       (lpVIAGraphicInfo->dwProjectIDRevision == UNICHROME_VT3324M2)  || ( lpVIAGraphicInfo->dwProjectIDRevision == UNICHROME_VT3293))
    {
        isCapture1Enabled = VIAGETREG(CAPTURE0_CONTROL|REG_Capture1_INDEX) & 0x1;
    }
    else
    {
        isCapture1Enabled = VIAGETREG(CAPTURE1_CONTROL) & 0x1;
    }
    
    if (pVia->V4LEnabled == 1) {
    /* deal with panning case via via_v4l kernel module */
    /*fd0 = open("/dev/video0",O_RDWR);*/

    fd0 = open(pVia->video0Path ,O_RDWR);
    if( -1 == fd0) 
    {
       DBG_DD((" Couldn't open \"/dev/video0\"\n"));
    }

    /*fd1 = open("/dev/video1",O_RDWR);*/
    fd1 = open(pVia->video1Path ,O_RDWR);
    if( -1 == fd1) 
    {
       DBG_DD((" Couldn't open \"/dev/video1\"\n"));
    }

    /*fd2 = open("/dev/video2",O_RDWR);*/
    fd2 = open(pVia->video2Path ,O_RDWR);
    if( -1 == fd2) 
    {
       DBG_DD((" Couldn't open \"/dev/video2\"\n"));
    }

    /* Store Panning (x, y) info in VidData */
    pVidData->panning_x[pScrn->scrnIndex] = x;
    pVidData->panning_y[pScrn->scrnIndex] = y;

    /* Pass Panning (x, y) info to V4L */
    AdjustFrame.x = x;
    AdjustFrame.y = y;
    /*Added for SAMM*/
    if ( pScrn->scrnIndex == 1 )
    {
        AdjustFrame.dwFlags = DDOVER_ON_SND;
    }
    else
    {
        AdjustFrame.dwFlags = 0;
    }

    if ( -1 == ioctl(fd2, VIA_VID_ADJUSTFRAME, &AdjustFrame))
    {
       DBG_DD(("IOCTL VIA_VID_ADJUSTFRAME failed.\n"));
    }

    if ( -1 == ioctl(fd2, VIA_VID_GET_VIDDATA, pVidData))
    {
        DBG_DD(("IOCTL VIA_VID_GET_VIDDATA failed.\n"));
    }

    /* for(handle=0;handle<2;handle++) { */

    DBG_DD(("dwVideoFlagMPEG=%lx\n", lpVidStatus->dwVideoFlagMPEG[handle]));
    if(pVidData->VidStatus.dwVideoFlagMPEG[handle]) {
        UpdateOverlayRec = pVidData->UpdateOverlayMPEGBackup;
        lpOverlayRecordMPEG = &pVidData->OverlayRecordSet.OverlayRecordMPEG[handle];
        UpdateOverlayRec.dwColorSpaceLowValue = lpOverlayRecordMPEG->dwKeyLow;
        UpdateOverlayRec.rSrc = lpOverlayRecordMPEG->dwOriSrcRect;
        UpdateOverlayRec.rDest = lpOverlayRecordMPEG->dwOriDestRect;
        UpdateOverlayRec.dwFourcc = FOURCC_VIA;
        UpdateOverlayRec.dwFlags |= (DDOVER_SHOW | DDOVER_KEYDEST);
        memcpy(&pVidData->UpdateOverlayRec, &UpdateOverlayRec, sizeof(DDUPDATEOVERLAY));
        if ( -1 == ioctl(fd2, VIA_VID_UPDATEOVERLAY, pVidData))	
        {
           DBG_DD(("IOCTL VIA_VID_UPDATEOVERLAY failed.\n"));
        }
    }

    DBG_DD(("dwVideoFlagSW=%lx\n", lpVidStatus->dwVideoFlagSW[handle]));
    if(pVidData->VidStatus.dwVideoFlagSW[handle]) {
        UpdateOverlayRec = pVidData->UpdateOverlaySWBackup;
        lpOverlayRecordSW = &pVidData->OverlayRecordSet.OverlayRecordSW[handle];
        UpdateOverlayRec.dwColorSpaceLowValue = lpOverlayRecordSW->dwKeyLow;
        UpdateOverlayRec.rSrc = lpOverlayRecordSW->dwOriSrcRect;
        UpdateOverlayRec.rDest = lpOverlayRecordSW->dwOriDestRect;
        UpdateOverlayRec.dwFourcc = FOURCC_YUY2;
        UpdateOverlayRec.dwFlags |= (DDOVER_SHOW | DDOVER_KEYDEST);
        memcpy(&pVidData->UpdateOverlayRec, &UpdateOverlayRec, sizeof(DDUPDATEOVERLAY));
        if ( -1 == ioctl(fd2, VIA_VID_UPDATEOVERLAY, pVidData))
		
        {
           DBG_DD(("IOCTL VIA_VID_UPDATEOVERLAY failed.\n"));
        }
    }
    /* }  for handle=0,1 */

    if(pVidData->VidStatus.dwVideoFlagTV0) {
        DBG_DD((" Call TV0 UpdateOverlay at panning mode.\n"));
        lpOverlayRecordTV0 = &pVidData->OverlayRecordSet.OverlayRecordTV0;
        ov_win_cap0.width = lpOverlayRecordTV0->dwOriDestRect.right - lpOverlayRecordTV0->dwOriDestRect.left;
        ov_win_cap0.height = lpOverlayRecordTV0->dwOriDestRect.bottom - lpOverlayRecordTV0->dwOriDestRect.top;
        ov_win_cap0.x = lpOverlayRecordTV0->dwOriDestRect.left;
        ov_win_cap0.y = lpOverlayRecordTV0->dwOriDestRect.top;
        ov_win_cap0.flags      = 0;
        ov_win_cap0.chromakey  = 0x080408;
        ov_win_cap0.clipcount  = 0;

        if (-1 == ioctl(fd0, VIDIOCSWIN, &ov_win_cap0))
        {
           DBG_DD(("Can't upateoverlay TV0 via V4L\n"));
        }
    }  

       if(pVidData->VidStatus.dwVideoFlagTV1) {
        DBG_DD((" Call TV1 UpdateOverlay at panning mode.\n"));
        lpOverlayRecordTV1 = &pVidData->OverlayRecordSet.OverlayRecordTV1;
        ov_win_cap1.width = lpOverlayRecordTV1->dwOriDestRect.right - lpOverlayRecordTV1->dwOriDestRect.left;
        ov_win_cap1.height = lpOverlayRecordTV1->dwOriDestRect.bottom - lpOverlayRecordTV1->dwOriDestRect.top;
        ov_win_cap1.x = lpOverlayRecordTV1->dwOriDestRect.left;
        ov_win_cap1.y = lpOverlayRecordTV1->dwOriDestRect.top;
        ov_win_cap1.flags      = 0;
        ov_win_cap1.chromakey  = 0x080408;
        ov_win_cap1.clipcount  = 0;

        if (-1 == ioctl(fd1, VIDIOCSWIN, &ov_win_cap1))
        {
           DBG_DD(("Can't upateoverlay TV1 via V4L\n"));
        }
	
    }

    close(fd0);
    close(fd1);
    close(fd2);

  }  /* if (pVia->V4LEnabled == 1) */

  else 
  {  /* if (pVia->V4LEnabled != 1) */
    /* Pass Panning (x, y) info to V4L */
    AdjustFrame.x = x;
    AdjustFrame.y = y;

    /*Added for SAMM*/
    pVidData->panning_x[pScrn->scrnIndex] = AdjustFrame.x;
    pVidData->panning_y[pScrn->scrnIndex] = AdjustFrame.y;

    /* for(handle=0;handle<2;handle++) { */

    if(pVidData->VidStatus.dwVideoFlagMPEG[handle]) 
    {
        UpdateOverlayRec = pVidData->UpdateOverlayMPEGBackup;
        lpOverlayRecordMPEG = &pVidData->OverlayRecordSet.OverlayRecordMPEG[handle];
        UpdateOverlayRec.dwColorSpaceLowValue = lpOverlayRecordMPEG->dwKeyLow;
        UpdateOverlayRec.rSrc.left = 0;
        UpdateOverlayRec.rSrc.top = 0;
        UpdateOverlayRec.rSrc.right = lpOverlayRecordMPEG->dwOriSrcWidth;
        UpdateOverlayRec.rSrc.bottom = lpOverlayRecordMPEG->dwOriSrcHeight;
        UpdateOverlayRec.rDest = lpOverlayRecordMPEG->dwOriDestRect;
        UpdateOverlayRec.dwFourcc = FOURCC_VIA;
        UpdateOverlayRec.dwFlags |= (DDOVER_SHOW | DDOVER_KEYDEST);
        pVidData->UpdateOverlayRec = UpdateOverlayRec;
#ifdef WITHOUT_SOURCE
        UpdateOverlay(pVidData, &pVidData->UpdateOverlayRec);
#endif
    } /* MPEG */
    if(pVidData->VidStatus.dwVideoFlagSW[handle]) 
    {
        UpdateOverlayRec = pVidData->UpdateOverlaySWBackup;
        lpOverlayRecordSW = &pVidData->OverlayRecordSet.OverlayRecordSW[handle];
        UpdateOverlayRec.dwColorSpaceLowValue = lpOverlayRecordSW->dwKeyLow;
        UpdateOverlayRec.rSrc.left = 0;
        UpdateOverlayRec.rSrc.top = 0;
        UpdateOverlayRec.rSrc.right = lpOverlayRecordSW->dwOriSrcWidth;
        UpdateOverlayRec.rSrc.bottom = lpOverlayRecordSW->dwOriSrcHeight;
        UpdateOverlayRec.rDest = lpOverlayRecordSW->dwOriDestRect;
        UpdateOverlayRec.dwFourcc = FOURCC_YUY2;
        UpdateOverlayRec.dwFlags |= (DDOVER_SHOW | DDOVER_KEYDEST);
        pVidData->UpdateOverlayRec = UpdateOverlayRec;
#ifdef WITHOUT_SOURCE
        UpdateOverlay(pVidData, &pVidData->UpdateOverlayRec);
#endif
    } /* SW */
    /* }  for handle=0,1 */ 
  }  /* else (pVia->V4LEnabled == 1) */
  return;
}

Bool VIASwitchMode(int scrnIndex, DisplayModePtr mode, int flags)
{
    ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
    VIAPtr      pVia = VIAPTR(pScrn);

    DEBUG(xf86DrvMsg(scrnIndex, X_INFO, "VIASwitchMode\n"));

    /* Wait Hardware Engine idle to switch graphicd mode */
    WaitIdle();

    if (pVia->VQEnable) 
    { 	
       /* if we use VQ, disable it before we exit */
       switch (pVia->Chipset)
       {
         case VIA_P4M890:             
         case VIA_K8M890:
              VIASETREG(0x41c, 0x00100000);
              VIASETREG(0x420, 0x74301000);
           break;
         default:  
              VIASETREG(0x43c, 0x00fe0000);
              VIASETREG(0x440, 0x00000004);
              VIASETREG(0x440, 0x40008c0f);
              VIASETREG(0x440, 0x44000000);
              VIASETREG(0x440, 0x45080c04);
              VIASETREG(0x440, 0x46800408);
           break;
        }
    }

    return VIAModeInit(xf86Screens[scrnIndex], mode);
}


void VIAEnableMMIO(ScrnInfoPtr pScrn)
{
    vgaHWPtr hwp = VGAHWPTR(pScrn);
    VIAPtr pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr pBIOSInfo = pVia->pBIOSInfo;
    unsigned char val;

    if (xf86IsPrimaryPci(pVia->PciInfo))
    {
        /* If we are primary card, we still use std vga port. If we use
         * MMIO, system will hang in vgaHWSave when our card used in
         * PLE and KLE (integrated Trident MVP4)
         */
        vgaHWSetStdFuncs(hwp);
    }
    else
    {
        vgaHWSetMmioFuncs(hwp, pVia->MapBase, 0x8000);
    }

    val = VGAIN8(0x3c3);
    VGAOUT8(0x3c3, val | 0x01);
    val = VGAIN8(VGA_MISC_OUT_R);
    VGAOUT8(VGA_MISC_OUT_W, val | 0x01);

    /* Unlock Extended IO Space */
    VGAOUT8(0x3c4, 0x10);
    VGAOUT8(0x3c5, 0x01);
    /* Unlock CRTC register protect */
    VGAOUT8(0x3d4, 0x47);
    VGAOUT8(0x3d5, VGAIN8(0x3d5) & ~0x01);

    /* Enable MMIO */
    if (pVia->IsSecondary || pBIOSInfo->UseIGA2)
    {
        VGAOUT8(0x3c4, 0x1a);
        val = VGAIN8(0x3c5);
        DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "secondary val = %x\n", val));
        VGAOUT8(0x3c5, val | 0x38);
    }
    else
    {
        VGAOUT8(0x3c4, 0x1a);
        val = VGAIN8(0x3c5);
        DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "primary val = %x\n", val));
        VGAOUT8(0x3c5, val | 0x68);
    }

    return;
}

#if 0
void VIADisableMMIO(ScrnInfoPtr pScrn)
{
    VIAPtr pVia = VIAPTR(pScrn);
    unsigned char val;

    VGAOUT8(0x3c4, 0x1a);
    val = VGAIN8(0x3c5);
    VGAOUT8(0x3c5, val & 0x97);

    return;
}
#endif

void VIALoadPalette(ScrnInfoPtr pScrn, int numColors, int *indicies,
                    LOCO *colors, VisualPtr pVisual)
{
    VIAPtr pVia = VIAPTR(pScrn);
    int i, index;
    int sr1a, sr1b, cr67, cr6a;

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIALoadPalette\n"));    
    
    if (pScrn->bitsPerPixel != 8)
        return;
    
    VGAOUT8(0x3C4, SR1A);
    sr1a = VGAIN8(0x3C5);
    VGAOUT8(0x3C4, SR1B);
    sr1b = VGAIN8(0x3C5);
    VGAOUT8(0x3D4, CR67);
    cr67 = VGAIN8(0x3D5);
    VGAOUT8(0x3D4, CR6A);
    cr6a = VGAIN8(0x3D5);

    /* RainLin [2006.05.05]: We should initialize the values of both LUT, or sometimes */
    /* the display color on IGA2 may be incorrect. */

    /* Prepare for IGA2's LUT initialization: */
    VGAOUT8(0x3C4, SR1A);
    VGAOUT8(0x3C5, sr1a | 0x01);    /* Change Shadow to Secondary Display's LUT. */
    
    VGAOUT8(0x3C4, SR1B);    
    VGAOUT8(0x3C5, sr1b | 0x80);    /* Enable Secondary Display Engine. */
    
    VGAOUT8(0x3D4, CR67);
    VGAOUT8(0x3D5, cr67 & 0x3F);    /* Second Display Color Depth should be 8bpp. */

    /* S.T.Chen[2006.5.9]: If secondary display had been enable, we don't need to enable it again.
       Otherwise, the integrated TV's screen maybe incorrect which is caused by sync problem. */
    VGAOUT8(0x3D4, CR6A);           
    if(!(VGAIN8(0x3D5) & 0x80))
    {
        /* Enable Second Display Channel. Please keep the setting sequence. */
        VGAOUT8(0x3D5, VGAIN8(0x3D5) & 0xBF);   /* Set CR6A[6] to 0. */
        VGAOUT8(0x3D5, VGAIN8(0x3D5) | 0x80);   /* Set CR6A[7] to 1. */
        VGAOUT8(0x3D5, VGAIN8(0x3D5) | 0x40);   /* Set CR6A[6] to 1. */
    }

    /* Fill in IGA2's LUT. */
    for (i = 0; i < numColors; i++)
    {
        index = indicies[i];
        VGAOUT8(0x3c8, index);
        VGAOUT8(0x3c9, colors[index].red);
        VGAOUT8(0x3c9, colors[index].green);
        VGAOUT8(0x3c9, colors[index].blue);
    }
    
    /* Prepare for initialize IGA1's LUT: */
    VGAOUT8(0x3C4, SR1A);
    VGAOUT8(0x3C5, sr1a); /* Change to Primary Display's LUT. */
    VGAOUT8(0x3C4, SR1B);
    VGAOUT8(0x3C5, sr1b);
    VGAOUT8(0x3D4, CR67);
    VGAOUT8(0x3D5, cr67);
    VGAOUT8(0x3D4, CR6A);
    VGAOUT8(0x3D5, cr6a);

    /* Fill in IGA1's LUT. */
    for (i = 0; i < numColors; i++)
    {
        index = indicies[i];
        VGAOUT8(0x3c8, index);
        VGAOUT8(0x3c9, colors[index].red);
        VGAOUT8(0x3c9, colors[index].green);
        VGAOUT8(0x3c9, colors[index].blue);
    }
}


static void VIADPMS(ScrnInfoPtr pScrn, int mode, int flags)
{
    VIAPtr          pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;
    CARD8           cr36;

    /* Clear DPMS setting */
    VGAOUT8(0x3d4, 0x36);
    cr36 = VGAIN8(0x3d5) & ~0x30;

    switch (mode)
    {
    case DPMSModeOn:
        if (pBIOSInfo->ActiveDevice & VIA_DEVICE_CRT1)
        {
            VGAOUT(0x3d4, 0x36);
            VGAOUT(0x3d5, cr36);
        }
        if (pBIOSInfo->ActiveDevice & VIA_DEVICE_LCD)
        {
            /* Enable LCD */
            VIAEnableLCD(pBIOSInfo, &(pBIOSInfo->LVDSSettingInfo));
        }
        if (pBIOSInfo->ActiveDevice & VIA_DEVICE_TV)
        {
            /* Enable TV */
            VIAEnableTV(pBIOSInfo, &(pBIOSInfo->TVSettingInfo));
        }
        if (pBIOSInfo->ActiveDevice & VIA_DEVICE_DFP)
        {
            /* Enable DFP */
            VIAEnableDFP(pBIOSInfo);
        }
        break;
    case DPMSModeStandby:
    case DPMSModeSuspend:
    case DPMSModeOff:
        if (pBIOSInfo->ActiveDevice & VIA_DEVICE_CRT1)
        {
            VGAOUT(0x3d4, 0x36);
            VGAOUT(0x3d5, cr36 | 0x30);
        }
        if (pBIOSInfo->ActiveDevice & (VIA_DEVICE_DFP | VIA_DEVICE_LCD))
        {
            VIADisableLCD(pBIOSInfo, &(pBIOSInfo->LVDSSettingInfo));
        }
        if (pBIOSInfo->ActiveDevice & VIA_DEVICE_TV)
        {
            VIADisableTV(pBIOSInfo, &(pBIOSInfo->TVSettingInfo));
        }
        if (pBIOSInfo->ActiveDevice & VIA_DEVICE_DFP)
        {
            VIADisableDFP(pBIOSInfo);
        }
        break;
    default:
        xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
                   "Invalid DPMS mode %d\n", mode);
        break;
    }

    return;
}


#if 0
int VIAGetMemSize()
{
    /* TODO: Do memory sizing  */

    /* Default 16MB */
    return (16 << 10);
}
#endif


/* S.T.Chen [2005.11.02]: query NB to get frame buffer size */
int VIAGetFBSize(ScrnInfoPtr pScrn)
{
    unsigned long   configid,deviceid,FBSize;
    int   VideoMemSize;
    int   DeviceFound = FALSE;
    VIAPtr pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr pBIOSInfo = pVia->pBIOSInfo;
    
    for ( configid = 0x80000000; configid < 0x80010800; configid += 0x100 )
    {
        outl((unsigned long)0xCF8,configid);
        deviceid = (inl((unsigned long)0xCFC)>>16) & 0xffff;

        /* S.T.Chen [2005.12.09]: Add KM800 case*/
        switch (deviceid)
        {
            case CLE266:
            case KM400:
                outl((unsigned long)0xCF8,configid+0xE0);
                FBSize = inl((unsigned long)0xCFC);
                DeviceFound = TRUE; /* Found device id */
                break;
                
            case CN400_FUNCTION3:
            case CN900_FUNCTION3:
            case CX700_FUNCTION3:
            case KM800_FUNCTION3:
            case KM890_FUNCTION3:
            case P4M890_FUNCTION3:
            case P4M900_FUNCTION3:
            /*S.T.Chen [2006.4.19]: (Need to Fix)
                 The value of CN750_FUNCTION3 is duplicate with CN900_FUNCTION3 in VT3293_A0.
                 This value maybe change in the next version.
            */
            /*case CN750_FUNCTION3:*/
                outl((unsigned long)0xCF8,configid+0xA0);
                FBSize = inl((unsigned long)0xCFC);
                DeviceFound = TRUE; /* Found device id */
                break;
                
            /* Can't set default, otherwise FBSize will be modified incorrectly */    
            default:
		/* 'FBSize' may be used uninitialized in this function */
		FBSize = 0;
                break;
         }

         if (DeviceFound)
         {
            break; /* Found device id, so exit for loop */
         }
    }

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Device ID = %x\n", deviceid));
    
    FBSize = FBSize & 0x00007000;
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "FB Size = %x\n", FBSize));
    
    if (pBIOSInfo->Chipset < VIA_CX700)
    {
        switch(FBSize)
        {
            case 0x00004000:
                VideoMemSize = (16 << 10);  /*16M*/              
                break;

            case 0x00005000:
                VideoMemSize = (32 << 10);  /*32M*/                
                break;

            case 0x00006000:
                VideoMemSize = (64 << 10);  /*64M*/                
                break;

            default:
                VideoMemSize = (32 << 10);  /*32M*/
                break;     
        }
    }
    else
    {
        switch(FBSize)
        {
            case 0x00001000:
                VideoMemSize = (8 << 10);   /*8M*/                
                break;

            case 0x00002000:
                VideoMemSize = (16 << 10);  /*16M*/               
                break;

            case 0x00003000:
                VideoMemSize = (32 << 10);  /*32M*/                
                break;

            case 0x00004000:
                VideoMemSize = (64 << 10);  /*64M*/               
                break;

            case 0x00005000:
                VideoMemSize = (128 << 10); /*128M*/
                break;

            case 0x00006000:
                VideoMemSize = (256 << 10); /*256M*/
                break;

            default:
                VideoMemSize = (32 << 10);  /*32M*/                
                break;
        }
    }

    return VideoMemSize;
}




/* Active Device according connected status */
Bool VIADeviceSelection(ScrnInfoPtr pScrn)
{
    VIAPtr pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr pBIOSInfo = pVia->pBIOSInfo;
    /*unsigned int  i;
    unsigned char   numDevice;*/
    pBIOSInfo->SAMM = FALSE;

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIADeviceSelection\n"));
    
    /* STChen[2007.3.8]: For MAMM*/
    if (!pVia->IsMAMMEnable)
    {    
        pBIOSInfo->DefaultActiveDevice = BIOS_GetActiveDevice(pScrn);
    }
    else
    {
        pBIOSInfo->DefaultActiveDevice= VIA_DEVICE_CRT1;
    }
    
    /* if XF86Config Option "ActiveDevice" hasn't set, active device according CMOS */
    if (!pBIOSInfo->ActiveDevice) {
        pBIOSInfo->ActiveDevice = pBIOSInfo->DefaultActiveDevice;
    }
    /* if XF86Config-4 set SAMM, and only active one device, active device according CMOS */
    else if (pVia->IsSecondary &&
            (pBIOSInfo->ActiveDevice == VIA_DEVICE_CRT1 ||
             pBIOSInfo->ActiveDevice == VIA_DEVICE_LCD  ||
             pBIOSInfo->ActiveDevice == VIA_DEVICE_TV   ||
             pBIOSInfo->ActiveDevice == VIA_DEVICE_DFP  ||
             pBIOSInfo->ActiveDevice == VIA_DEVICE_CRT2))
    {
        pBIOSInfo->ActiveDevice = BIOS_GetActiveDevice(pScrn);
    }
    if (((pBIOSInfo->ActiveDevice & pBIOSInfo->ConnectedDevice) == pBIOSInfo->ActiveDevice)
         && (pVia->IsSecondary)) {
        pBIOSInfo->SAMM = TRUE;
    }
    if((pBIOSInfo->ActiveDevice & (VIA_DEVICE_CRT1 | VIA_DEVICE_CRT2)) && pVia->IsSecondary)
    {
      pBIOSInfo->SAMM = TRUE;
      pBIOSInfo->ConnectedDevice|=VIA_DEVICE_CRT2;
    }    
    pBIOSInfo->ActiveDevice &= pBIOSInfo->ConnectedDevice;
    pVia->ActiveDevice = pBIOSInfo->ActiveDevice;
    
    /* To keep the actual active device no matter it is in SAMM or Duoview. */
    pBIOSInfo->ActualActiveDevice = pVia->ActiveDevice;

    /*if (pBIOSInfo->ActiveDevice & VIA_DEVICE_LCD) {
        pBIOSInfo->DVIEncoder = VIA_VT3191;
        numDevice = 0x02;
        i = BIOS_GetDisplayDeviceInfo(pScrn, &numDevice);
        if (i != 0xFFFF) {
            if (pBIOSInfo->PanelSize == VIA_PANEL_INVALID)
                pBIOSInfo->PanelSize = numDevice;
    }*/

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Actual Connected Device is %d\n", pBIOSInfo->ConnectedDevice));
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Actual Active Device is %d\n", pBIOSInfo->ActiveDevice));

    return TRUE;
}

/* SAMM device dispatch */
Bool VIASetDisplayPath_SAMM(ScrnInfoPtr pScrn)
{
    DevUnion* pPriv = xf86GetEntityPrivate(pScrn->entityList[0], gVIAEntityIndex);
    VIAEntPtr pVIAEnt = pPriv->ptr;
    VIAPtr pVia1 = VIAPTR(pScrn);
    VIAPtr pVia0 = VIAPTR(pVIAEnt->pPrimaryScrn);
    VIABIOSInfoPtr pBIOSInfo1 = pVia1->pBIOSInfo;
    VIABIOSInfoPtr pBIOSInfo0 = pVia0->pBIOSInfo;

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIASetDisplayPath_SAMM\n"));
    pBIOSInfo0->SAMM = TRUE;
    pBIOSInfo1->PrimaryDevice = pBIOSInfo0->PrimaryDevice;    
    
    switch (pBIOSInfo1->ActiveDevice)
    {
        case (VIA_DEVICE_CRT1 | VIA_DEVICE_LCD):
            if (pBIOSInfo0->PrimaryDevice == VIA_DEVICE_LCD)
            {
                /* LCD+CRT */
                pBIOSInfo0->ActiveDevice = VIA_DEVICE_LCD;
                pBIOSInfo1->ActiveDevice = VIA_DEVICE_CRT1;                
                pBIOSInfo0->LVDSSettingInfo.IGAPath = IGA2;
                pBIOSInfo1->CRTSettingInfo.IGAPath = IGA1;
            }
            else
            {
                /* CRT+LCD */
                pBIOSInfo0->ActiveDevice = VIA_DEVICE_CRT1;
                pBIOSInfo1->ActiveDevice = VIA_DEVICE_LCD;
                pBIOSInfo0->CRTSettingInfo.IGAPath = IGA1;
                pBIOSInfo1->LVDSSettingInfo.IGAPath = IGA2;
            }
            
            break;
            
        case (VIA_DEVICE_CRT1 | VIA_DEVICE_TV):
            if (pBIOSInfo0->PrimaryDevice == VIA_DEVICE_TV)
            {
                /* TV+CRT */
                pBIOSInfo0->ActiveDevice = VIA_DEVICE_TV;
                pBIOSInfo1->ActiveDevice = VIA_DEVICE_CRT1;
                pBIOSInfo0->TVSettingInfo.IGAPath = IGA1;
                pBIOSInfo1->CRTSettingInfo.IGAPath = IGA2;                
            }
            else
            {
                /* CRT+TV */
                pBIOSInfo0->ActiveDevice = VIA_DEVICE_CRT1;
                pBIOSInfo1->ActiveDevice = VIA_DEVICE_TV;
                pBIOSInfo0->CRTSettingInfo.IGAPath = IGA1;
                pBIOSInfo1->TVSettingInfo.IGAPath = IGA2;
            }
            
            break;
            
        case (VIA_DEVICE_CRT1 | VIA_DEVICE_DFP):
            if (pBIOSInfo0->PrimaryDevice == VIA_DEVICE_DFP)
            {
                /* DVI+CRT */
                pBIOSInfo0->ActiveDevice = VIA_DEVICE_DFP;
                pBIOSInfo1->ActiveDevice = VIA_DEVICE_CRT1;
                pBIOSInfo0->TMDSSettingInfo.IGAPath = IGA1;
                pBIOSInfo1->CRTSettingInfo.IGAPath = IGA2;
            }
            else
            {
                /* CRT+DVI */
                pBIOSInfo0->ActiveDevice = VIA_DEVICE_CRT1;
                pBIOSInfo1->ActiveDevice = VIA_DEVICE_DFP;
                pBIOSInfo0->CRTSettingInfo.IGAPath = IGA1;
                pBIOSInfo1->TMDSSettingInfo.IGAPath = IGA2;
            }

            break;
            
        case (VIA_DEVICE_LCD | VIA_DEVICE_TV):
            if (pBIOSInfo0->PrimaryDevice == VIA_DEVICE_LCD)
            {
                /* LCD+TV */
                pBIOSInfo0->ActiveDevice = VIA_DEVICE_LCD;
                pBIOSInfo1->ActiveDevice = VIA_DEVICE_TV;
                pBIOSInfo0->LVDSSettingInfo.IGAPath = IGA2;
                pBIOSInfo1->TVSettingInfo.IGAPath = IGA1;
            }
            else
            {
                /* TV+LCD */
                pBIOSInfo0->ActiveDevice = VIA_DEVICE_TV;
                pBIOSInfo1->ActiveDevice = VIA_DEVICE_LCD;
                pBIOSInfo0->TVSettingInfo.IGAPath = IGA1;
                pBIOSInfo1->LVDSSettingInfo.IGAPath = IGA2;
            }
            
            break;
            
        case (VIA_DEVICE_LCD | VIA_DEVICE_DFP):
            if (pBIOSInfo0->PrimaryDevice == VIA_DEVICE_DFP)
            {
                /* DVI+LCD */
                pBIOSInfo0->ActiveDevice = VIA_DEVICE_DFP;
                pBIOSInfo1->ActiveDevice = VIA_DEVICE_LCD;
                pBIOSInfo0->TMDSSettingInfo.IGAPath = IGA1;
                pBIOSInfo1->LVDSSettingInfo.IGAPath = IGA2;
            }
            else
            {
                /* LCD+DVI */
                pBIOSInfo0->ActiveDevice = VIA_DEVICE_LCD;
                pBIOSInfo1->ActiveDevice = VIA_DEVICE_DFP;
                pBIOSInfo0->LVDSSettingInfo.IGAPath = IGA2;
                pBIOSInfo1->TMDSSettingInfo.IGAPath = IGA1;
            }
            
            break;

        case (VIA_DEVICE_LCD | VIA_DEVICE_LCD2):
            if (pBIOSInfo0->PrimaryDevice == VIA_DEVICE_LCD)
            {
                /* LCD+LCD2 */
                pBIOSInfo0->ActiveDevice = VIA_DEVICE_LCD;
                pBIOSInfo0->LVDSSettingInfo.IGAPath = IGA1;
                pBIOSInfo1->ActiveDevice = VIA_DEVICE_LCD2;
                pBIOSInfo1->LVDSSettingInfo2.IGAPath = IGA2;
            }
            else
            {
                /* LCD2+LCD */
                pBIOSInfo0->ActiveDevice = VIA_DEVICE_LCD2;                
                pBIOSInfo0->LVDSSettingInfo2.IGAPath = IGA1;
                pBIOSInfo1->ActiveDevice = VIA_DEVICE_LCD;
                pBIOSInfo1->LVDSSettingInfo.IGAPath = IGA2;
            }
            
            break;
            
        case (VIA_DEVICE_TV | VIA_DEVICE_DFP):
            if (pBIOSInfo0->PrimaryDevice == VIA_DEVICE_TV)
            {
                /* TV+DVI */
                pBIOSInfo0->ActiveDevice = VIA_DEVICE_TV;
                pBIOSInfo1->ActiveDevice = VIA_DEVICE_DFP;
                pBIOSInfo0->TVSettingInfo.IGAPath = IGA1;
                pBIOSInfo1->TMDSSettingInfo.IGAPath = IGA2;
            }
            else
            {
                /* DVI+TV */
                pBIOSInfo0->ActiveDevice = VIA_DEVICE_DFP;
                pBIOSInfo1->ActiveDevice = VIA_DEVICE_TV;
                pBIOSInfo0->TMDSSettingInfo.IGAPath = IGA1;
                pBIOSInfo1->TVSettingInfo.IGAPath = IGA2;
            }

            break;
            
        case (VIA_DEVICE_CRT1 | VIA_DEVICE_CRT2):
            if (pBIOSInfo0->PrimaryDevice == VIA_DEVICE_CRT2)
            {
                /* CRT2+CRT1 */
                pBIOSInfo0->ActiveDevice = VIA_DEVICE_CRT2;
                pBIOSInfo1->ActiveDevice = VIA_DEVICE_CRT1;
                pBIOSInfo0->CRT2_IGA = IGA1;
                pBIOSInfo1->CRTSettingInfo.IGAPath = IGA2;
            }
            else
            {
                /* CRT1+CRT2 */
                pBIOSInfo0->ActiveDevice = VIA_DEVICE_CRT1;
                pBIOSInfo1->ActiveDevice = VIA_DEVICE_CRT2;
                pBIOSInfo0->CRTSettingInfo.IGAPath = IGA1;
                pBIOSInfo1->CRT2_IGA = IGA2;
            }
            
            break;
            
        case (VIA_DEVICE_TV | VIA_DEVICE_TV2):
            if (pBIOSInfo0->PrimaryDevice == VIA_DEVICE_TV)
            {
                /* TV+TV2 */
                pBIOSInfo0->ActiveDevice = VIA_DEVICE_TV;
                pBIOSInfo0->TVSettingInfo.IGAPath = IGA1;
                pBIOSInfo1->ActiveDevice = VIA_DEVICE_TV2;
                pBIOSInfo1->TVSettingInfo2.IGAPath = IGA2;                
            }
            else
            {
                /* TV2+TV */
                pBIOSInfo0->ActiveDevice = VIA_DEVICE_TV2;                
                pBIOSInfo0->TVSettingInfo2.IGAPath = IGA1;
                pBIOSInfo1->ActiveDevice = VIA_DEVICE_TV;
                pBIOSInfo1->TVSettingInfo.IGAPath = IGA2;
            }
            
            break;    
            
        default:
            return FALSE;            
    }

    DEBUG(xf86DrvMsg(pVIAEnt->pPrimaryScrn->scrnIndex, X_INFO, "Scrn0: Active Device is 0x%x\n", pBIOSInfo0->ActiveDevice));
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Scrn1: Active Device is 0x%x\n", pBIOSInfo1->ActiveDevice));
    DEBUG(xf86DrvMsg(pVIAEnt->pPrimaryScrn->scrnIndex,  X_INFO, "Primary: CRT=%d, TV=%d, DFP=%d, LCD=%d\n", pBIOSInfo0->CRTSettingInfo.IGAPath, pBIOSInfo0->TVSettingInfo.IGAPath, pBIOSInfo0->TMDSSettingInfo.IGAPath, pBIOSInfo0->LVDSSettingInfo.IGAPath));
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Secondary: CRT=%d, TV=%d, DFP=%d, LCD=%d\n", pBIOSInfo1->CRTSettingInfo.IGAPath, pBIOSInfo1->TVSettingInfo.IGAPath, pBIOSInfo1->TMDSSettingInfo.IGAPath, pBIOSInfo1->LVDSSettingInfo.IGAPath));

    return TRUE;
}

void VIASetDisplayPath(ScrnInfoPtr pScrn)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;

    VIACheckIfUseDuoView(pScrn);    

    if (pVia->IsSecondary)
    {
        /* SAMM: */
        if (pBIOSInfo->SAMM)
        {
            xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIASetDisplayPath is SAMM\n");        
            VIASetDisplayPath_SAMM(pScrn);
        }
    }
    else if (pBIOSInfo->DuoView)
    {
    xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIASetDisplayPath is DuoView\n"); 
        /* DuoView: */
        switch (pBIOSInfo->ActiveDevice)
        {
            case (VIA_DEVICE_CRT1 | VIA_DEVICE_LCD):
                pBIOSInfo->CRTSettingInfo.IGAPath = IGA1;
                pBIOSInfo->LVDSSettingInfo.IGAPath = IGA2;
                break;
                
            case (VIA_DEVICE_CRT1 | VIA_DEVICE_TV):
                pBIOSInfo->CRTSettingInfo.IGAPath = IGA1;
                pBIOSInfo->TVSettingInfo.IGAPath = IGA2;
                break;
                
            case (VIA_DEVICE_CRT1 | VIA_DEVICE_DFP):
                pBIOSInfo->CRTSettingInfo.IGAPath = IGA1;
                pBIOSInfo->TMDSSettingInfo.IGAPath = IGA2;
                break;
                
            case (VIA_DEVICE_LCD | VIA_DEVICE_TV):
                pBIOSInfo->TVSettingInfo.IGAPath = IGA1;
                pBIOSInfo->LVDSSettingInfo.IGAPath = IGA2;
                break;
                
            case (VIA_DEVICE_LCD | VIA_DEVICE_DFP):
                pBIOSInfo->TMDSSettingInfo.IGAPath = IGA1;
                pBIOSInfo->LVDSSettingInfo.IGAPath = IGA2;
                break;

            case (VIA_DEVICE_LCD | VIA_DEVICE_LCD2):
                pBIOSInfo->LVDSSettingInfo.IGAPath = IGA2;
                pBIOSInfo->LVDSSettingInfo2.IGAPath = IGA2;
                break;
                
            case (VIA_DEVICE_TV | VIA_DEVICE_DFP):
                pBIOSInfo->TMDSSettingInfo.IGAPath = IGA1;
                pBIOSInfo->TVSettingInfo.IGAPath = IGA2;
                break;

            case (VIA_DEVICE_TV | VIA_DEVICE_TV2):
                pBIOSInfo->TVSettingInfo.IGAPath = IGA1;
                pBIOSInfo->TVSettingInfo2.IGAPath = IGA2;
                break;
                
            default :
                pBIOSInfo->CRTSettingInfo.IGAPath = IGA1;
                break;
        }
    }    
    else if (pBIOSInfo->ActualActiveDevice == (VIA_DEVICE_LCD & VIA_DEVICE_LCD2))
    {
        pBIOSInfo->LVDSSettingInfo.IGAPath = IGA2;
        pBIOSInfo->LVDSSettingInfo2.IGAPath = IGA2;
    }
    else
    {
        xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIASetDisplayPath is Single\n"); 
        /* Simultaneous: */
        pBIOSInfo->CRTSettingInfo.IGAPath = IGA1;
        pBIOSInfo->TVSettingInfo.IGAPath = IGA1;
        pBIOSInfo->TMDSSettingInfo.IGAPath = IGA1;
        pBIOSInfo->LVDSSettingInfo.IGAPath = IGA2;        
    }
    xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Active Device is 0x%x\n", pBIOSInfo->ActiveDevice);
    xf86DrvMsg(pScrn->scrnIndex, X_INFO, "IGA: CRT=%d, TV=%d, DFP=%d, LCD=%d\n", 
                                                pBIOSInfo->CRTSettingInfo.IGAPath,
                                                pBIOSInfo->TVSettingInfo.IGAPath,
                                                pBIOSInfo->TMDSSettingInfo.IGAPath,
                                                pBIOSInfo->LVDSSettingInfo.IGAPath);
                                                
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Active Device is 0x%x\n", pBIOSInfo->ActiveDevice));
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "IGA: CRT=%d, TV=%d, DFP=%d, LCD=%d\n", 
                                                pBIOSInfo->CRTSettingInfo.IGAPath,
                                                pBIOSInfo->TVSettingInfo.IGAPath,
                                                pBIOSInfo->TMDSSettingInfo.IGAPath,
                                                pBIOSInfo->LVDSSettingInfo.IGAPath));
}

/* DuoVoiew device dispatch */
static Bool
VIADeviceDispatchDuoView(ScrnInfoPtr pScrn)
{
    VIAPtr pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr pBIOSInfo = pVia->pBIOSInfo;
    LPVIAGRAPHICINFO lpVIAGinfo = &pVia->pVidData->VIAGraphicInfo;

    /* Horace 2005-11-04 add for duoview device switch */
    int only_one_video_engine = 0;
    switch (pVia->Chipset)
    {
          case VIA_KM400:
          case VIA_K8M800:
          case VIA_P4M800PRO:
          case VIA_K8M890:
          case VIA_P4M890:
		  	only_one_video_engine = 1;
			break;
	   default:
	   	break;	  	
    }
	
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIADeviceDispatchDuoView !!\n"));
    switch (pBIOSInfo->ActiveDevice) {
        case (VIA_DEVICE_CRT1 | VIA_DEVICE_LCD) :       /* If CRT + LCD, CRT is IGA1 */
	     if((only_one_video_engine)&& (pBIOSInfo->s3utility==FALSE))
	     {
	            if(lpVIAGinfo->duoview & DUOVIEW_VIDEOOPTION)
	            {/*[2005.10.24] For Option VideoOnDevice in duo view mode by Al Zhang*/
	                if(lpVIAGinfo->duoview & DUOVIEW_VIDEOONLCD)  
	                    lpVIAGinfo->duoview |= DUOVIEW_V3_ON_IGA2;
	                else
	                    lpVIAGinfo->duoview &= ~DUOVIEW_V3_ON_IGA2;
	            }
	            
	     }
            break;
        case (VIA_DEVICE_CRT1 | VIA_DEVICE_TV) :        /* If CRT + TV, CRT is IGA1 */
	     if((only_one_video_engine)&& (pBIOSInfo->s3utility==FALSE))
	     {			
	            if(lpVIAGinfo->duoview & DUOVIEW_VIDEOOPTION)
	            {
	                if(lpVIAGinfo->duoview & DUOVIEW_VIDEOONTV)  
	                    lpVIAGinfo->duoview |= DUOVIEW_V3_ON_IGA2;
	                else
	                    lpVIAGinfo->duoview &= ~DUOVIEW_V3_ON_IGA2;
	            }
	            else
	            {
	                lpVIAGinfo->duoview |= DUOVIEW_V3_ON_IGA2;
	            }
	     }
            break;
        case (VIA_DEVICE_CRT1 | VIA_DEVICE_DFP) :       /* If CRT + DFP, CRT is IGA1 */
	     if((only_one_video_engine)&& (pBIOSInfo->s3utility==FALSE))
	     {			
	            if(lpVIAGinfo->duoview & DUOVIEW_VIDEOOPTION)
	            {
	                if(lpVIAGinfo->duoview & DUOVIEW_VIDEOONDVI)  
	                    lpVIAGinfo->duoview |= DUOVIEW_V3_ON_IGA2;
	                else
	                    lpVIAGinfo->duoview &= ~DUOVIEW_V3_ON_IGA2;
	            }
	            else
	            {
	                lpVIAGinfo->duoview |= DUOVIEW_V3_ON_IGA2;
	            }
	     }
            break;
        case (VIA_DEVICE_LCD | VIA_DEVICE_TV) :         /* If LCD + TV, TV is IGA1 */
	     if((only_one_video_engine)&& (pBIOSInfo->s3utility==FALSE))
	     {			
	            if(lpVIAGinfo->duoview & DUOVIEW_VIDEOOPTION)
	            {
	                if(lpVIAGinfo->duoview & DUOVIEW_VIDEOONLCD)  
	                    lpVIAGinfo->duoview |= DUOVIEW_V3_ON_IGA2;
	                else
	                    lpVIAGinfo->duoview &= ~DUOVIEW_V3_ON_IGA2;
	            }
	     }
            break;
        case (VIA_DEVICE_LCD | VIA_DEVICE_DFP) :        /* If LCD + DFP, DFP is IGA1 */
	     if((only_one_video_engine)&& (pBIOSInfo->s3utility==FALSE))
	     {			
	            if(lpVIAGinfo->duoview & DUOVIEW_VIDEOOPTION)
	            {
	                if(lpVIAGinfo->duoview & DUOVIEW_VIDEOONLCD)  
	                    lpVIAGinfo->duoview |= DUOVIEW_V3_ON_IGA2;
	                else
	                    lpVIAGinfo->duoview &= ~DUOVIEW_V3_ON_IGA2;
	            }
	     }
            break;
        case (VIA_DEVICE_TV | VIA_DEVICE_DFP) :         /* If TV + DFP, DFP is IGA1 */
	        if((only_one_video_engine)&& (pBIOSInfo->s3utility==FALSE))
	        {			
	            if(lpVIAGinfo->duoview & DUOVIEW_VIDEOOPTION)
	            {
	                if(lpVIAGinfo->duoview & DUOVIEW_VIDEOONTV)  
	                    lpVIAGinfo->duoview |= DUOVIEW_V3_ON_IGA2;
	                else
	                    lpVIAGinfo->duoview &= ~DUOVIEW_V3_ON_IGA2;
	            }
	            else
	            {
	                lpVIAGinfo->duoview |= DUOVIEW_V3_ON_IGA2;
	            }
	        }
            break;
        default :
            break;
    }
    
    return TRUE;
}

/* Build a list of the VESA modes the BIOS reports as valid */
static void
VIABuildModeList(ScrnInfoPtr pScrn)
{
    VIAPtr pVia = VIAPTR(pScrn);
	VIAModeInfoPtr m;
	int serno1 = 0, ModeNo = 0, Xres = 0, Yres = 0, bpp = 0;
	int serno2 = 0, refIndex = 0;

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIABuildModeList\n"));
    while ((serno1 = BIOS_QueryModeList(pScrn, serno1, &ModeNo)) != 0)
    {
        BIOS_GetVESAModeInfo(pScrn, ModeNo, &Xres, &Yres, &bpp);
    	m = xnfcalloc(sizeof(VIAModeInfoRec),1);
    	m->width = Xres;
    	m->height = Yres;
    	m->bpp = bpp;
    	m->modeno = ModeNo;

        /* Get Support Refresh Rate by Mode No. */
        while ((serno2 = BIOS_QuerySupportRefreshRate(pScrn, ModeNo, serno2, &refIndex)) != 0)
        {
            switch (refIndex)
            {
                case 0:
                    m->SupRefMask |= VIA_REF_60HZ;
                    break;
                case 1:
                    m->SupRefMask |= VIA_REF_56HZ;
                    break;
                case 2:
                    m->SupRefMask |= VIA_REF_65HZ;
                    break;
                case 3:
                    m->SupRefMask |= VIA_REF_70HZ;
                    break;
                case 4:
                    m->SupRefMask |= VIA_REF_72HZ;
                    break;
                case 5:
                    m->SupRefMask |= VIA_REF_75HZ;
                    break;
                case 6:
                    m->SupRefMask |= VIA_REF_80HZ;
                    break;
                case 7:
                    m->SupRefMask |= VIA_REF_85HZ;
                    break;
                case 8:
                    m->SupRefMask |= VIA_REF_90HZ;
                    break;
                case 9:
                    m->SupRefMask |= VIA_REF_100HZ;
                    break;
                case 10:
                    m->SupRefMask |= VIA_REF_120HZ;
                    break;
                case 11:
                    m->SupRefMask |= VIA_REF_86HZ;
                    break;
                case 12:
                    m->SupRefMask |= VIA_REF_94HZ;
                    break;
            }
        }
    	m->next = pVia->VIAModeList;

    	xf86DrvMsg(pScrn->scrnIndex, X_PROBED,
    	      "Support VESA mode 0x%x: X:%d Y:%d bpp:%d Refresh:0x%x\n",
    	       m->modeno, m->width, m->height, m->bpp, m->SupRefMask);

    	pVia->VIAModeList = m;
    }
}

Bool VIACheckLCDExpand(ScrnInfoPtr pScrn)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr pBIOSInfo = pVia->pBIOSInfo;
    int countWidthByQWord, offsetWidthByQWord;
    unsigned char   cr13, cr35, sr1c, sr1d;

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIACheckLCDExpand\n"));
    VGAOUT8(0x3c4, 0x33);
    /* Hotkey event occur */
    if (VGAIN8(0x3c5) & 0x01)
    {
        xf86DrvMsg(pScrn->scrnIndex, X_INFO,
                   "Device Switch Hotkey has been press!!\n");
        pBIOSInfo->ActiveDevice = BIOS_GetActiveDevice(pScrn);
        if (pBIOSInfo->ActiveDevice & VIA_DEVICE_LCD)
        {
            VGAOUT8(0x3d4, 0x79);
            if (VGAIN8(0x3d5) & 0x1)
            {
                pBIOSInfo->scaleY = TRUE;
            }
            else
            {
                pBIOSInfo->scaleY = FALSE;
            }
        }
        else
        {
            pBIOSInfo->scaleY = FALSE;
            /* if CRT Only, no panning */
            if (pBIOSInfo->ActiveDevice == VIA_DEVICE_CRT1)
            {
                pScrn->frameX1 = pBIOSInfo->SaveframeX1;
                pScrn->frameY1 = pBIOSInfo->SaveframeY1;
                pScrn->currentMode->HDisplay = pBIOSInfo->SaveHDisplay;
                pScrn->currentMode->VDisplay = pBIOSInfo->SaveVDisplay;
                pScrn->currentMode->CrtcHDisplay = pBIOSInfo->SaveCrtcHDisplay;
                pScrn->currentMode->CrtcVDisplay = pBIOSInfo->SaveCrtcVDisplay;
                VIAAdjustFrame(pScrn->scrnIndex, pScrn->frameX0, pScrn->frameY0, 0);
            }
        }

        /* clean hotkey event */
        VGAOUT8(0x3c4, 0x33);
        VGAOUT8(0x3c5, VGAIN8(0x3c5) & ~0x01);

        /* Set Quadword offset and counter */
        /*=* Fix bandwidth problem when using virtual desktop *=*/
        countWidthByQWord = pBIOSInfo->countWidthByQWord;
        offsetWidthByQWord = pBIOSInfo->offsetWidthByQWord;

        VGAOUT8(0x3d4, 0x35);
        cr35 = VGAIN8(0x3d5) & 0x1f;
        cr35 |= (offsetWidthByQWord & 0x700) >> 3;        /* bit 7:5: offset 10:8 */
        cr13 = offsetWidthByQWord & 0xFF;
 
        /* Enable Refresh to avoid data lose when enter screen saver */
        /* Refresh disable & 128-bit alignment */
        VGAOUT8(0x3c4, 0x1d);
        sr1d = VGAIN8(0x3c5);     
        sr1d = (sr1d & 0xfc) | (countWidthByQWord >> 9);
        sr1c = ((countWidthByQWord >> 1) + 1) & 0xff;

        VGAOUT8(0x3d4, 0x13);
        VGAOUT8(0x3d5, cr13);
        VGAOUT8(0x3d4, 0x35);
        VGAOUT8(0x3d5, cr35);
        VGAOUT8(0x3c4, 0x1c);
        VGAOUT8(0x3c5, sr1c);
        VGAOUT8(0x3c4, 0x1d);
        VGAOUT8(0x3c5, sr1d);               
    }

    return pBIOSInfo->scaleY;
}

void VIADebugBreak()
{
    /* use int3 to set breakpoint */
    __asm __volatile("int $3");
}

#ifdef XF86DRI
void
VIAInitialize3DEngine(ScrnInfoPtr pScrn)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    int i;
    WaitIdle();

    if (!b3DRegsInitialized)
    {
        /*reset the 3d hardware first
        VGAOUT8(0x3c4, 0x1a);
        TmpVal = VGAIN8(0x3c5);
        VGAOUT8(0x3c5, TmpVal|0x40);
        VGAOUT8(0x3c5, TmpVal|0x40);        
        VGAOUT8(0x3c5, TmpVal); */       

        VIASETREG(0x43C, 0x00010000);

        for (i = 0; i <= 0x7D; i++)
        {
            VIASETREG(0x440, (CARD32) i << 24);
        }

        VIASETREG(0x43C, 0x00020000);

        for (i = 0; i <= 0x94; i++)
        {
            VIASETREG(0x440, (CARD32) i << 24);
        }

        VIASETREG(0x440, 0x82400000);

        VIASETREG(0x43C, 0x01020000);


        for (i = 0; i <= 0x94; i++)
        {
            VIASETREG(0x440, (CARD32) i << 24);
        }

        VIASETREG(0x440, 0x82400000);
        VIASETREG(0x43C, 0xfe020000);

        for (i = 0; i <= 0x03; i++)
        {
            VIASETREG(0x440, (CARD32) i << 24);
        }

        VIASETREG(0x43C, 0x00030000);

        for (i = 0; i <= 0xff; i++)
        {
            VIASETREG(0x440, 0);
        }
        VIASETREG(0x43C, 0x00100000);
        VIASETREG(0x440, 0x00333004);
        VIASETREG(0x440, 0x10000002);
        VIASETREG(0x440, 0x60000000);
        VIASETREG(0x440, 0x61000000);
        VIASETREG(0x440, 0x62000000);
        VIASETREG(0x440, 0x63000000);
        VIASETREG(0x440, 0x64000000);
        
        VIASETREG(0x43C, 0x00fe0000);

        if (pVia->ChipRev >= 3 )
            VIASETREG(0x440,0x40008c0f);
        else
            VIASETREG(0x440,0x4000800f);

        VIASETREG(0x440,0x44000000);
        VIASETREG(0x440,0x45080C04);
        VIASETREG(0x440,0x46800408);
        VIASETREG(0x440,0x50000000);
        VIASETREG(0x440,0x51000000);
        VIASETREG(0x440,0x52000000);
        VIASETREG(0x440,0x53000000);

        b3DRegsInitialized = 1;
        xf86DrvMsg(pScrn->scrnIndex, X_INFO,
            "3D Engine has been initilized.\n");
    }

    VIASETREG(0x43C,0x00fe0000);
    VIASETREG(0x440,0x08000001);
    VIASETREG(0x440,0x0A000183);
    VIASETREG(0x440,0x0B00019F);
    VIASETREG(0x440,0x0C00018B);
    VIASETREG(0x440,0x0D00019B);
    VIASETREG(0x440,0x0E000000);
    VIASETREG(0x440,0x0F000000);
    VIASETREG(0x440,0x10000000);
    VIASETREG(0x440,0x11000000);
    VIASETREG(0x440,0x20000000);
    
}
#endif


#ifdef _XORG_VER_6_8_LATER_
/* 
    STChen[2006.7.25]: Following functions are used to support RandR rotation,
    it can help to rotate screen on the fly.

    ps. some definitions of struct are different on FC4 and FC5.
*/
static Bool
VIARandRGetInfo(ScrnInfoPtr pScrn, Rotation *rotations)
{
    VIAPtr pVia = VIAPTR(pScrn);

    /* to report what ability we can support. */
    if(pVia->RandRRotation)
       *rotations = RR_Rotate_0 | RR_Rotate_90 |RR_Rotate_180 | RR_Rotate_270;
    else
       *rotations = RR_Rotate_0;

    return TRUE;
}

static Bool
VIARandRSetConfig(ScrnInfoPtr pScrn, xorgRRConfig *config)
{
    xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIARandRSetConfig!\n");
    VIAPtr pVia = VIAPTR(pScrn);
    PVIDDATA pVidData = pVia->pVidData;
    LPVIAGRAPHICINFO lpVIAGraphicInfo = &pVia->pVidData->VIAGraphicInfo;
    int fd;
    
    switch(config->rotation) {
        case RR_Rotate_0:            
            pVia->SWRotate = VIA_ROTATE_DEGREE_0;
            pScrn->PointerMoved = pVia->PointerMoved;
            break;

        case RR_Rotate_90:            
            pVia->SWRotate = VIA_ROTATE_DEGREE_270;            
            pScrn->PointerMoved = VIAPointerMoved;
            break;
            
        case RR_Rotate_180:            
            pVia->SWRotate = VIA_ROTATE_DEGREE_180;            
            pScrn->PointerMoved = VIAPointerMoved;
            break;

        case RR_Rotate_270:            
            pVia->SWRotate = VIA_ROTATE_DEGREE_90;
            pScrn->PointerMoved = VIAPointerMoved;
            break;

        default:
            xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
                    "Unexpected rotation in VIARandRSetConfig!\n");
            pVia->SWRotate = VIA_ROTATE_DEGREE_0;
            pScrn->PointerMoved = pVia->PointerMoved;
            return FALSE;
    }

    lpVIAGraphicInfo->rotate = pVia->SWRotate;

    if (pVia->V4LEnabled == 1)
    {
        /* Synchronize new settings to V4L: */
        
        fd = open(pVia->video2Path ,O_RDWR);

        if( -1 == fd)
        {
            xf86DrvMsg(pScrn->scrnIndex, X_ERROR, "VIARandRSetConfig(): Couldn't open \"/dev/video2\"\n");
        }        
        
        if ( -1 == ioctl(fd, VIA_VID_SET_2D_INFO, pVidData) )
        {
            xf86DrvMsg(pScrn->scrnIndex, X_ERROR, "VIARandRSetConfig(): Can't set 2D Info to V4L\n");
        }

        close(fd);
    }
    
    return TRUE;
}

#ifdef _XORG_VER_6_9_LATER_
static Bool
VIADriverFunc(ScrnInfoPtr pScrn, xorgDriverFuncOp op, pointer data)
#else
static Bool
VIADriverFunc(ScrnInfoPtr pScrn, xorgRRFuncFlags op, pointer data)
#endif
{
    switch(op) {
       case RR_GET_INFO:          
          return VIARandRGetInfo(pScrn, (Rotation*)data);
       case RR_SET_CONFIG:          
          return VIARandRSetConfig(pScrn, (xorgRRConfig*)data);
       default:
          return FALSE;
    }

    return FALSE;
}

#endif

