/*
 * Copyright 1998-2007 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2007 S3 Graphics, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

/*
 * I N C L U D E S
 */
#include "X11/Xproto.h"     /* For Xextension */
#include "extnsionst.h" /* For Xextension */
#include "dixstruct.h"  /* For Xextension */
#include "via_driver.h"
#include "via_utility.h"
#include "via_video.h"
#include "via_IntegratedTV.h"
#include "via_vt1625.h"
#include "close/via.h"
#include "via_rotate.h"
#include "via_hw.h"
#include "via_tv.h"

#include <fcntl.h>

/* Extern Functions Prototypes Declaration */
extern void load_vt1622_regs(VIABIOSInfoPtr pBIOSInfo);
extern void load_vt1623_regs(VIABIOSInfoPtr pBIOSInfo);
extern void VIAEnableLCD(VIABIOSInfoPtr pBIOSInfo, LVDSSETTINGINFOPTR pLVDSSettingInfo);
extern void VIADisableLCD(VIABIOSInfoPtr pBIOSInfo, LVDSSETTINGINFOPTR pLVDSSettingInfo);
extern void VIAEnableDFP(VIABIOSInfoPtr pBIOSInfo);
extern void VIADisableDFP(VIABIOSInfoPtr pBIOSInfo);
extern void VIAPreSetTV2Mode(VIABIOSInfoPtr pBIOSInfo);
extern void VIAPostSetTV2Mode(VIABIOSInfoPtr pBIOSInfo);
extern void VIAEnableTV(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo);
extern void VIADisableTV(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo);
extern void VIATVPatch(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo);


extern int gVIAEntityIndex;

VIABIOSInfoPtr pBIOSInfo_local;

/* Record screen number. */
int g_ScreenNumber;
/****************************************************************************
 ****************************                    ****************************
 **************************** INTERNAL FUNCTIONS ****************************
 ****************************                    ****************************
 ****************************************************************************/

CARD8 CrReadByte(CARD8 byIndex);
void CrWriteByte(CARD8 byIndex, CARD8 byValue);
CARD8 SrReadByte(CARD8 byIndex);
void SrWriteByte(CARD8 byIndex, CARD8 byValue);
CARD8 TV_Read_Data(CARD8 addr);
Bool TV_Write_Data(CARD8 addr, CARD8 data);
Bool TV_BIOSDLL_Init_Linux(VIABIOSInfoPtr pBIOSInfo);


static CARD32 MapDeviceStateForUT(CARD32 DeviceStateForDrv);
static CARD32 MapDeviceStateForDrv(CARD32 DeviceStateForUT);
static ScrnInfoPtr VIAScreenSelection(ScrnInfoPtr pScrn, CARD32 dwPrimaryID, CARD32 dwSecondaryID);
static ScrnInfoPtr VIADriverSelection(ScrnInfoPtr pScrn, unsigned char* buf);
static Bool VIASaveUserTVSetting(VIABIOSInfoPtr pBIOSInfo,
				 TVSETTINGINFOPTR pTVSettingInfo,
				 const char* pchFileName);

static void VIA_UT_XV_FUNC_BIOS_GetChipID
(ScrnInfoPtr pScrn, CARD32 *pdwChipID);
static void VIA_UT_XV_FUNC_BIOS_GetVersion
(VIABIOSInfoPtr pBIOSInfo, UTBIOSVERSION* pUTBIOSVERSION);
static void VIA_UT_XV_FUNC_BIOS_GetDate
(ScrnInfoPtr pScrn, UTBIOSDATE *pUTBIOSDATE);
static void VIA_UT_XV_FUNC_BIOS_GetVideoMemSizeMB
(ScrnInfoPtr pScrn, CARD32 *pdwVideoRam);
static void VIA_UT_XV_FUNC_BIOS_GetTVChipID
(VIABIOSInfoPtr pBIOSInfo, CARD32 *pdwTVChipID);
static Bool VIA_UT_XV_FUNC_DRIVER_GetFileName
(CARD32 dwDriverType, char *szDriverName);
static Bool VIA_UT_XV_FUNC_DRIVER_GetFileVersion
(ScrnInfoPtr pScrn, CARD32 dwDriverType,  UTDriverVersion *pUTDriverVersion);
static CARD32 VIA_UT_XV_FUNC_DEVICE_GetPanningState(VIABIOSInfoPtr pBIOSInfo);
static void VIA_UT_XV_FUNC_DISPLAY_EnumModes
(VIABIOSInfoPtr pBIOSInfo, CARD32 ModeNum, LPUTDEVMODE lpDevMode);
static void VIA_UT_XV_FUNC_DISPLAY_GetCurrentMode
(VIABIOSInfoPtr pBIOSInfo, LPUTDEVMODE lpDevMode);
static void VIA_UT_XV_FUNC_DISPLAY_GetRotationCaps
(ScrnInfoPtr pScrn, CARD32 RotateType, CARD32 *pRotationCaps);
static void VIA_UT_XV_FUNC_DISPLAY_GetCurrentRotateDegree
(ScrnInfoPtr pScrn, CARD32 *RotateType, CARD32 *RotateDegree);
static void VIA_UT_XV_FUNC_DEVICE_GetPanelInfo
(VIABIOSInfoPtr pBIOSInfo, CARD32 dwPanelType, UTPANELINFO *pPanelInfo);
static void VIA_UT_XV_FUNC_DEVICE_GetSupportExpand
(VIABIOSInfoPtr pBIOSInfo, CARD32 *pdwSupportState);
static void VIA_UT_XV_FUNC_DEVICE_GetExpandState
(VIABIOSInfoPtr pBIOSInfo, CARD32 *dwExpandState);
static Bool VIA_UT_XV_FUNC_DEVICE_SetExpandState
(VIABIOSInfoPtr pBIOSInfo, DisplayModePtr mode, UTPANELINFO UTPanelInfo);
static void VIA_UT_XV_FUNC_DEVICE_GetSupportState
(VIABIOSInfoPtr pBIOSInfo, CARD32 *pdwSupportState);
static void VIA_UT_XV_FUNC_DEVICE_GetConnectState
(VIABIOSInfoPtr pBIOSInfo, CARD32 *pdwConnectState);
static void VIA_UT_XV_FUNC_DEVICE_GetActiveState
(ScrnInfoPtr pScrn, CARD32 *pdwActiveDevice);
static void VIA_UT_XV_FUNC_DEVICE_SetActiveState
(VIABIOSInfoPtr pBIOSInfo, DisplayModePtr mode, CARD32 dwActiveDevice);
static void VIA_UT_XV_FUNC_DEVICE_GetSAMMState
(VIABIOSInfoPtr pBIOSInfo, CARD32 *pdwSAMMState);
static void VIA_UT_XV_FUNC_DEVICE_GetSupportSAMMState
(VIABIOSInfoPtr pBIOSInfo, CARD32 *pdwSAMMState);
static Bool VIA_UT_XV_FUNC_DEVICE_SetV3OnIGAStatus
(ScrnInfoPtr pScrn, CARD32 dwIGAPath);
static void VIA_UT_XV_FUNC_DEVICE_GetV3OnIGAStatus
(ScrnInfoPtr pScrn, CARD32 *pdwStatus);
static void VIA_UT_XV_FUNC_DEVICE_GetDuoViewStatus
(ScrnInfoPtr pScrn, CARD32 *pdwState);
static void VIA_UT_XV_FUNC_TV_GetSupportStandardState
(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo,
 CARD32 *pdwSupportState);
static void VIA_UT_XV_FUNC_TV_GetStandard
(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo, CARD32 *pdwStandard);
static Bool VIA_UT_XV_FUNC_TV_SetStandard
(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo, DisplayModePtr mode, CARD32 dwStandard);
static void VIA_UT_XV_FUNC_TV_GetSupportSignalTypeState
(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo,
 CARD32 *pdwSupportState);
static void VIA_UT_XV_FUNC_TV_GetSignalType
(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo,
 CARD32 *pdwSignalType);
static Bool VIA_UT_XV_FUNC_TV_SetSignalType
(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo,
 DisplayModePtr mode, CARD32 dwSignal);
static void VIA_UT_XV_FUNC_TV_GetMaxViewSizeValue
(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo,
 UTXYVALUE *pMaxViewSizeValue);
static void VIA_UT_XV_FUNC_TV_GetViewSizeValue
(VIABIOSInfoPtr pBIOSInfo,  TVSETTINGINFOPTR pTVSettingInfo,
 UTXYVALUE *pViewSizeValue);
static void VIA_UT_XV_FUNC_TV_SetViewSizeValue
(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo, DisplayModePtr mode,
 UTXYVALUE ViewSizeValue);
static void VIA_UT_XV_FUNC_TV_GetMaxViewPositionValue
(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo,
 UTXYVALUE *pMaxViewPosValue);
static void VIA_UT_XV_FUNC_TV_GetViewPositionValue
(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo,
 UTXYVALUE *pViewPosValue);
static Bool VIA_UT_XV_FUNC_TV_SetViewPositionValue
(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo,
 UTXYVALUE ViewPosValue);
static void VIA_UT_XV_FUNC_TV_GetSupportTuningState
(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo,
 CARD32 *pdwSupportState);
static Bool VIA_UT_XV_FUNC_TV_GetTuningItemMaxValue
(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo, CARD32 dwTuningItem,
 CARD32 *pdwMaxValue);
static Bool VIA_UT_XV_FUNC_TV_GetTuningItemValue
(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo, CARD32 dwTuningItem,
 CARD32 *pdwValue);
static Bool VIA_UT_XV_FUNC_TV_SetTuningItemValue
(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo,
 UTSETTVTUNINGINFO TuningInfo);
static void VIA_UT_XV_FUNC_TV_GetSupportSettingState
(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo,
 CARD32 *pdwSupportState);
static Bool VIA_UT_XV_FUNC_TV_GetSettingItemState
(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo,
 CARD32 dwItem, CARD32 *pdwState);
static Bool VIA_UT_XV_FUNC_TV_SetSettingItemState
(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo,
 UTSETTVITEMSTATE SettingInfo);
static void VIA_UT_XV_FUNC_HDTV_GetSupportStandardState
(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo,
 CARD32 *pdwSupportState);
static void VIA_UT_XV_FUNC_HDTV_GetStandard
(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo, CARD32 *pdwStandard);
static Bool VIA_UT_XV_FUNC_HDTV_SetStandard
(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo,
 DisplayModePtr mode, CARD32 dwStandard);
static void VIA_UT_XV_FUNC_HDTV_GetSupportSignalTypeState
(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo,
 CARD32 *pdwSupportState);
static void VIA_UT_XV_FUNC_HDTV_GetSignalType
(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo,
 CARD32 *pdwSignalType);
static Bool VIA_UT_XV_FUNC_HDTV_SetSignalType
(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo,
 DisplayModePtr mode, CARD32 dwSignalType);
static void VIA_UT_ESC_FUNC_GAMMA_GetDeviceSupportState
(ScrnInfoPtr pScrn, CARD32 *pdwSupportState);
static void VIA_UT_ESC_FUNC_GAMMA_GetLookUpTable
(VIABIOSInfoPtr pBIOSInfo, CARD32 *pGammaLUT);
static Bool VIA_UT_ESC_FUNC_GAMMA_SetLookUpTable
(ScrnInfoPtr pScrn, CARD32* pGammaLUT);
static void VIA_UT_ESC_FUNC_GAMMA_GetDefaultLookUpTable
(VIABIOSInfoPtr pBIOSInfo, CARD32* pGammaLUT);


CARD8 CrReadByte(CARD8 byIndex)
{
    MMIO_OUT8(pBIOSInfo_local->MapBase+0x8000, 0x3d4, byIndex);
    return MMIO_IN8(pBIOSInfo_local->MapBase+0x8000, 0x3d5);
}

void CrWriteByte(CARD8 byIndex, CARD8 byValue)
{
    MMIO_OUT8(pBIOSInfo_local->MapBase+0x8000, 0x3d4, byIndex);
    MMIO_OUT8(pBIOSInfo_local->MapBase+0x8000, 0x3d5, byValue);
}

CARD8 SrReadByte(CARD8 byIndex)
{
    MMIO_OUT8(pBIOSInfo_local->MapBase+0x8000, 0x3c4, byIndex);
    return MMIO_IN8(pBIOSInfo_local->MapBase+0x8000, 0x3c5);
}

void SrWriteByte(CARD8 byIndex, CARD8 byValue)
{
    MMIO_OUT8(pBIOSInfo_local->MapBase+0x8000, 0x3c4, byIndex);
    MMIO_OUT8(pBIOSInfo_local->MapBase+0x8000, 0x3c5, byValue);
}

CARD8 TV_Read_Data(CARD8 addr)
{
    I2CDevPtr   dev;
    CARD8       W_Buffer[1], R_Buffer[1];

    dev = xf86CreateI2CDevRec();
    dev->DevName = "TV";
    dev->SlaveAddr = pBIOSInfo_local->TVI2CAdd;
    dev->pI2CBus = pBIOSInfo_local->I2C_Port2;

    xf86I2CDevInit(dev);

    W_Buffer[0] = addr;
    xf86I2CWriteRead(dev, W_Buffer,1, R_Buffer, 1);

    xf86DestroyI2CDevRec(dev,TRUE);

    return R_Buffer[0];
}

Bool TV_Write_Data(CARD8 addr, CARD8 data)
{
    I2CDevPtr   dev;
    CARD8       W_Buffer[2];

    dev = xf86CreateI2CDevRec();
    dev->DevName = "TV";
    dev->SlaveAddr = pBIOSInfo_local->TVI2CAdd;
    dev->pI2CBus = pBIOSInfo_local->I2C_Port2;

    if(xf86I2CDevInit(dev))
    {
        W_Buffer[0] = addr;
        W_Buffer[1] = data;
        xf86I2CWriteRead(dev, W_Buffer, 2, NULL, 0);
        xf86DestroyI2CDevRec(dev,TRUE);
        return TRUE;
    }
    else
    {
        xf86DestroyI2CDevRec(dev,TRUE);
        return FALSE;
    }
}

Bool TV_BIOSDLL_Init_Linux(VIABIOSInfoPtr pBIOSInfo)
{
    pBIOSInfo_local = pBIOSInfo;

    return TRUE;
}
/*CheckMactovisionEnabled: to check if MV is enabled
*                          according to TV encoder's registers 0x0F and 0x30
*                          Currently only support VT1622 and VT1623
*                          AlZhang[2006.04.24]
*/
Bool
CheckMacrovisionEnabled(VIABIOSInfoPtr  pBIOSInfo)
{
    I2CDevPtr       dev;
    unsigned char   W_Buffer[3];
    unsigned char   R_Buffer[3];

    dev = xf86CreateI2CDevRec();
    dev->DevName = "TV";
    dev->SlaveAddr = pBIOSInfo->TVI2CAdd;
    dev->pI2CBus = pBIOSInfo->I2C_Port2;

    if (xf86I2CDevInit(dev))
    {
        switch (pBIOSInfo->TVSettingInfo.TVEncoder)
        {
        case VIA_VT1622:
        case VIA_VT1622A:

            W_Buffer[0] = 0x0F;
            xf86I2CWriteRead(dev, W_Buffer,1, R_Buffer,1);
            R_Buffer[1] = R_Buffer[0];
            W_Buffer[0] = 0x30;
            xf86I2CWriteRead(dev, W_Buffer,1, R_Buffer,1);
            R_Buffer[2] = R_Buffer[0];
            break;
        case VIA_VT1623:
        case VIA_VT1625:
            R_Buffer[1] = GPIOI2CRead_TV(pBIOSInfo, 0x0F);
            R_Buffer[2] = GPIOI2CRead_TV(pBIOSInfo, 0x30);
            break;
        default:
            xf86DestroyI2CDevRec(dev,TRUE);
            return FALSE;
        }
        xf86DestroyI2CDevRec(dev,TRUE);
        if((R_Buffer[1]&0x80) && (R_Buffer[2]&0x30))
            return TRUE;
        else
            return FALSE;
    }
    return FALSE;

}

/*
    STChen[2006.8.22]: Driver keep a device list but utility keep a different list,
    so we must do a mapping operation from driver to utility.
*/
static CARD32
MapDeviceStateForUT(CARD32 DeviceStateForDrv)
{
    CARD32 DeviceStateForUT = 0;

    if (DeviceStateForDrv & VIA_DEVICE_CRT1)
    {
        DeviceStateForUT |= UT_DEVICE_CRT1;
    }

    if (DeviceStateForDrv & VIA_DEVICE_LCD)
    {
        DeviceStateForUT |= UT_DEVICE_LCD;
    }

    if (DeviceStateForDrv & VIA_DEVICE_TV)
    {
        DeviceStateForUT |= UT_DEVICE_TV;
    }

    if (DeviceStateForDrv & VIA_DEVICE_DFP)
    {
        DeviceStateForUT |= UT_DEVICE_DFP;
    }

    if (DeviceStateForDrv & VIA_DEVICE_CRT2)
    {
        DeviceStateForUT |= UT_DEVICE_CRT2;
    }

    if (DeviceStateForDrv & VIA_DEVICE_LCD2)
    {
        DeviceStateForUT |= UT_DEVICE_LCD2;
    }

    if (DeviceStateForDrv & VIA_DEVICE_TV2)
    {
        DeviceStateForUT |= UT_DEVICE_TV2;
    }

    if (DeviceStateForDrv & VIA_DEVICE_DFP2)
    {
        DeviceStateForUT |= UT_DEVICE_DFP2;
    }

    return DeviceStateForUT;

}


static CARD32
MapDeviceStateForDrv(CARD32 DeviceStateForUT)
{
    CARD32 DeviceStateForDrv = 0;

    if (DeviceStateForUT & UT_DEVICE_CRT1)
    {
        DeviceStateForDrv |= VIA_DEVICE_CRT1;
    }

    if (DeviceStateForUT & UT_DEVICE_LCD)
    {
        DeviceStateForDrv |= VIA_DEVICE_LCD;
    }

    if (DeviceStateForUT & UT_DEVICE_TV)
    {
        DeviceStateForDrv |= VIA_DEVICE_TV;
    }

    if (DeviceStateForUT & UT_DEVICE_DFP)
    {
        DeviceStateForDrv |= VIA_DEVICE_DFP;
    }

    if (DeviceStateForUT & UT_DEVICE_CRT2)
    {
        DeviceStateForDrv |= VIA_DEVICE_CRT2;
    }

    if (DeviceStateForUT & UT_DEVICE_LCD2)
    {
        DeviceStateForDrv |= VIA_DEVICE_LCD2;
    }

    if (DeviceStateForUT & UT_DEVICE_TV2)
    {
        DeviceStateForDrv |= VIA_DEVICE_TV2;
    }

    if (DeviceStateForUT & UT_DEVICE_DFP2)
    {
        DeviceStateForDrv |= VIA_DEVICE_DFP2;
    }

    if (DeviceStateForUT & UT_DEVICE_HDTV)
    {
        DeviceStateForDrv |= VIA_DEVICE_TV;
    }

    if (DeviceStateForUT & UT_DEVICE_HDTV2)
    {
        DeviceStateForDrv |= VIA_DEVICE_TV2;
    }

    return DeviceStateForDrv;

}



static ScrnInfoPtr
VIAScreenSelection(ScrnInfoPtr pScrn,
		   CARD32 dwPrimaryID,
		   CARD32 dwSecondaryID)
{
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIAScreenSelection\n"));

    DevUnion* pPriv = xf86GetEntityPrivate(pScrn->entityList[0], gVIAEntityIndex);
    VIAEntPtr pVIAEnt = pPriv->ptr;
    VIAPtr pVia0 = VIAPTR(pVIAEnt->pPrimaryScrn);

    /* Both LCD and DVI use this kind of utility function. So when application call
       this kind of function. we must switch to the driver which device dependent.*/
    if (dwPrimaryID == UT_XV_FUNC_PANEL)
    {
        if (pVia0->ActiveDevice & VIA_DEVICE_LCD)
        {
            if (pVia0->ActiveDevice & VIA_DEVICE_LCD2)
            {
                /* If using dual LCD, we always return secondary screen because of IGA2.*/
                return pVIAEnt->pSecondaryScrn;
            }

            if (pVia0->pBIOSInfo->PrimaryDevice == VIA_DEVICE_LCD)
            {
                DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "LCD in primary.\n"));
                return pVIAEnt->pPrimaryScrn;
            }
            else
            {
                DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "LCD in secondary.\n"));
                return pVIAEnt->pSecondaryScrn;
            }
        }

        if (pVia0->ActiveDevice & VIA_DEVICE_DFP)
        {
            if (pVia0->pBIOSInfo->PrimaryDevice == VIA_DEVICE_DFP)
            {
                DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "DVI in primary.\n"));
                return pVIAEnt->pPrimaryScrn;
            }
            else
            {
                DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "DVI in secondary.\n"));
                return pVIAEnt->pSecondaryScrn;
            }
        }
    }
    /* Switch to the driver which TV dependent. */
    else if ((dwPrimaryID == UT_XV_FUNC_TV) || (dwPrimaryID == UT_XV_FUNC_HDTV))
    {
        if (pVia0->ActiveDevice & VIA_DEVICE_TV)
        {
            if (pVia0->pBIOSInfo->PrimaryDevice == VIA_DEVICE_TV)
            {
                DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "TV in primary.\n"));
                return pVIAEnt->pPrimaryScrn;
            }
            else
            {
                DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "TV in secondary.\n"));
                return pVIAEnt->pSecondaryScrn;
            }
        }
    }
    else if ((dwPrimaryID == UT_XV_FUNC_TV2) || (dwPrimaryID == UT_XV_FUNC_HDTV2))
    {
        if (pVia0->ActiveDevice & VIA_DEVICE_TV2)
        {
            if (pVia0->pBIOSInfo->PrimaryDevice == VIA_DEVICE_TV2)
            {
                DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "TV in primary.\n"));
                return pVIAEnt->pPrimaryScrn;
            }
            else
            {
                DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "TV in secondary.\n"));
                return pVIAEnt->pSecondaryScrn;
            }
        }
    }

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Not Change.\n"));
    return pScrn;
}


static ScrnInfoPtr
VIADriverSelection(ScrnInfoPtr pScrn, unsigned char* buf)
{
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIADriverSelection\n"));
    unsigned char* _InParam;
    CARD32 _dwFunc = 0, _dwAction = 0;

    _InParam = buf;
    _dwFunc =  *((CARD32 *)_InParam);
    _InParam += 4;

    _dwAction =*((CARD32 *)_InParam);
    _InParam += 4;

    return VIAScreenSelection(pScrn, _dwFunc, _dwAction);
}



static void
VIA_UT_XV_FUNC_BIOS_GetChipID(ScrnInfoPtr pScrn, CARD32* pdwChipID)
{
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIA_UT_XV_FUNC_BIOS_GetChipID\n"));
    VIAPtr pVia = VIAPTR(pScrn);
    *pdwChipID = pVia->ChipId;
}

static void
VIA_UT_XV_FUNC_BIOS_GetVersion(VIABIOSInfoPtr pBIOSInfo,
			       UTBIOSVERSION* pUTBIOSVERSION)
{
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIA_UT_XV_FUNC_BIOS_GetVersion\n"));
    pUTBIOSVERSION->dwVersion = pBIOSInfo->BIOSMajorVersion;
    pUTBIOSVERSION->dwRevision = pBIOSInfo->BIOSMinorVersion;
}

static void
VIA_UT_XV_FUNC_BIOS_GetDate(ScrnInfoPtr pScrn, UTBIOSDATE* pUTBIOSDATE)
{
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIA_UT_XV_FUNC_BIOS_GetDate\n"));
    VIAPtr pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;

    if (!pBIOSInfo->BIOSDateYear)
    {
        BIOS_GetBIOSDate(pScrn);
    }
    pUTBIOSDATE->dwYear = (CARD32)pBIOSInfo->BIOSDateYear + 2000;
    pUTBIOSDATE->dwMonth = (CARD32)pBIOSInfo->BIOSDateMonth;
    pUTBIOSDATE->dwDay = (CARD32)pBIOSInfo->BIOSDateDay;
}


static void
VIA_UT_XV_FUNC_BIOS_GetVideoMemSizeMB(ScrnInfoPtr pScrn,
				      CARD32* pdwVideoRam)
{
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIA_UT_XV_FUNC_BIOS_GetVideoMemSizeMB\n"));
    VIAPtr pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;

    *pdwVideoRam = pScrn->videoRam >> 10;

    if (pBIOSInfo->SAMM)
    {
        *pdwVideoRam <<= 1;
    }
}


static void
VIA_UT_XV_FUNC_BIOS_GetTVChipID(VIABIOSInfoPtr pBIOSInfo, CARD32* pdwTVChipID)
{
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIA_UT_XV_FUNC_BIOS_GetTVChipID\n"));
    *pdwTVChipID = pBIOSInfo->TVSettingInfo.TVEncoder;
}

static Bool
VIA_UT_XV_FUNC_DRIVER_GetFileName(CARD32 dwDriverType, char* szDriverName)
{
    Bool result				= TRUE;
    char DISPLAY_DRIVER_NAME[]		= "via_drv.o     \0";
    char VDO_CAPTURE_DRIVER_NAME[]	= "via_v4l_drv\0";
    char HWOVERLAY_DRIVER_NAME[]	= "libddmpeg.a\0";
    char HWMPEG2_DECODER_NAME[]		= "libddmpeg.so\0";

    DEBUG(ErrorF("VIA_UT_XV_FUNC_DRIVER_GetFileName\n"));

    switch (dwDriverType) {
	case DISPLAY_DRIVER:
	    memcpy((void *)szDriverName, (void *)DISPLAY_DRIVER_NAME, sizeof(DISPLAY_DRIVER_NAME));
	    break;
	case VIDEO_CAPTURE_DRIVER:
	    memcpy((void *)szDriverName, (void *)VDO_CAPTURE_DRIVER_NAME, sizeof(VDO_CAPTURE_DRIVER_NAME));
	    break;
	case HWOVERLAY_DRIVER:
	    memcpy((void *)szDriverName, (void *)HWOVERLAY_DRIVER_NAME, sizeof(HWOVERLAY_DRIVER_NAME));
	    break;
	case HWMPEG2_DECODER:
	    memcpy((void *)szDriverName, (void *)HWMPEG2_DECODER_NAME, sizeof(HWMPEG2_DECODER_NAME));
            break;
	default:
	    result = FALSE;
	    break;
    }

    return result;
}


static Bool
VIA_UT_XV_FUNC_DRIVER_GetFileVersion(ScrnInfoPtr pScrn,
				     CARD32 dwDriverType,
				     UTDriverVersion* pUTDriverVersion)
{
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIA_UT_XV_FUNC_DRIVER_GetFileVersion\n"));
    VIAPtr pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;
    int ret = 0;

    switch(dwDriverType)
    {
        case DISPLAY_DRIVER :
            pUTDriverVersion->dwMajorNum = VERSION_MAJOR;
            pUTDriverVersion->dwMinorNum = VERSION_MINOR;
            pUTDriverVersion->dwOSNum = VERSION_OS;

            if (pBIOSInfo->DriverVersionPatchLevel)
            {
                pUTDriverVersion->dwReversionNum = pBIOSInfo->DriverVersionPatchLevel;
            }
            else
            {
                pUTDriverVersion->dwReversionNum = PATCHLEVEL;
            }
            ret = TRUE;
            break;

        case VIDEO_CAPTURE_DRIVER :
        {
            int fd2;
            unsigned int driver_version;

            fd2 = open(pVia->video2Path ,O_RDWR);

            if( -1 == fd2)
            {
                driver_version=0xffffffff;
            }
            else  if ( -1 == ioctl(fd2, VIA_VID_GET_VERSION , &driver_version) )
            {
                driver_version=0xfffffffe;
            }

            if(  fd2 != -1)
            {
                close(fd2);
            }

            pUTDriverVersion->dwMajorNum = (driver_version & 0xff000000)>>24;
            pUTDriverVersion->dwMinorNum = (driver_version & 0x00ff0000)>>16;
            pUTDriverVersion->dwOSNum = (driver_version & 0x0000ff00)>>8;
            pUTDriverVersion->dwReversionNum = (driver_version & 0x000000ff);
            ret = TRUE;
            break;
        }

        case HWOVERLAY_DRIVER :
            pUTDriverVersion->dwMajorNum = VERSION_MAJOR;
            pUTDriverVersion->dwMinorNum = VERSION_MINOR;
            pUTDriverVersion->dwOSNum = VERSION_OS;
            pUTDriverVersion->dwReversionNum = PATCHLEVEL;
            ret = TRUE;
            break;

        case HWMPEG2_DECODER :
            pUTDriverVersion->dwMajorNum = VERSION_MAJOR;
            pUTDriverVersion->dwMinorNum = VERSION_MINOR;
            pUTDriverVersion->dwOSNum = VERSION_OS;
            pUTDriverVersion->dwReversionNum = PATCHLEVEL;
            ret = TRUE;
            break;
        default:
            ret = FALSE;
            break;
    }

    return ret;
}


static CARD32
VIA_UT_XV_FUNC_DEVICE_GetPanningState(VIABIOSInfoPtr pBIOSInfo)
{
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIA_UT_XV_FUNC_DEVICE_GetPanningState\n"));

    CARD32 PanningState = 0;

    if ((pBIOSInfo->ActiveDevice & VIA_DEVICE_CRT1) && (pBIOSInfo->CRTSettingInfo.IsPanning))
    {
        PanningState |= UT_DEVICE_CRT1;
    }

    if ((pBIOSInfo->ActiveDevice & VIA_DEVICE_LCD) && (pBIOSInfo->LVDSSettingInfo.IsPanning))
    {
        PanningState |= UT_DEVICE_LCD;
    }

    if ((pBIOSInfo->ActiveDevice & VIA_DEVICE_LCD2) && (pBIOSInfo->LVDSSettingInfo2.IsPanning))
    {
        PanningState |= UT_DEVICE_LCD2;
    }

    if ((pBIOSInfo->ActiveDevice & VIA_DEVICE_DFP) && (pBIOSInfo->TMDSSettingInfo.IsPanning))
    {
        PanningState |= UT_DEVICE_DFP;
    }

    if ((pBIOSInfo->ActiveDevice & VIA_DEVICE_TV) && (pBIOSInfo->TVSettingInfo.IsPanning))
    {
        if ((pBIOSInfo->TVSettingInfo.TVType == TVTYPE_NTSC) || (pBIOSInfo->TVSettingInfo.TVType == TVTYPE_PAL))
        {
            PanningState |= UT_DEVICE_TV;
        }
        else
        {
            PanningState |= UT_DEVICE_HDTV;
        }
    }

    if ((pBIOSInfo->ActiveDevice & VIA_DEVICE_TV2) && (pBIOSInfo->TVSettingInfo2.IsPanning))
    {
        if ((pBIOSInfo->TVSettingInfo2.TVType == TVTYPE_NTSC) ||
            (pBIOSInfo->TVSettingInfo2.TVType == TVTYPE_PAL))
        {
            PanningState |= UT_DEVICE_TV2;
        }
        else
        {
            PanningState |= UT_DEVICE_HDTV2;
        }
    }

    return PanningState;
}


static void
VIA_UT_XV_FUNC_DISPLAY_EnumModes(VIABIOSInfoPtr pBIOSInfo, CARD32 ModeNum,
				 LPUTDEVMODE lpDevMode)
{
    VIASUPPORTMODE SupportMode;
    int index = 0, count = -1, RefreshRateNum;
    Bool IsSupported;

    while (VIAEnumModes(pBIOSInfo, index, &SupportMode, &IsSupported))
    {
        if (SupportMode.ModeIndex == -1)
        {
            lpDevMode->ModeIndex = M_INVALID;
            lpDevMode->PelsWidth = 0;
            lpDevMode->PelsHeight = 0;
            break;
        }

        if (IsSupported)
        {
            count++;

            if (count == ModeNum)
            {
                lpDevMode->ModeIndex = SupportMode.ModeIndex;
                lpDevMode->PelsWidth = SupportMode.HActive;
                lpDevMode->PelsHeight = SupportMode.VActive;

                /* Enumerate supported refresh rates: */
                if ((pBIOSInfo->ActiveDevice & VIA_DEVICE_DFP) ||
                    (pBIOSInfo->ActiveDevice & VIA_DEVICE_LCD) ||
                    (pBIOSInfo->ActiveDevice & VIA_DEVICE_TV))
                {
                    /* Currently, we only support 60Hz for DVI, LCD and TV. */
                    lpDevMode->RefreshRate[0] = 60;
                    lpDevMode->RefreshRateNum = 1;
                }
                else
                {
                    RefreshRateNum = 0;

                    while (1)
                    {
                        VIAEnumRefreshRate(SupportMode.ModeIndex, RefreshRateNum, &(lpDevMode->RefreshRate[RefreshRateNum]));

                        if (lpDevMode->RefreshRate[RefreshRateNum] == 0)
                        {
                            break;
                        }

                        RefreshRateNum++;
                    }

                    lpDevMode->RefreshRateNum = RefreshRateNum;
                }

                break;
            }
        }

        index++;
    }
}


static void
VIA_UT_XV_FUNC_DISPLAY_GetCurrentMode(VIABIOSInfoPtr pBIOSInfo,
				      LPUTDEVMODE lpDevMode)
{
    lpDevMode->ModeIndex = VIAGetModeIndex(pBIOSInfo->CrtcHDisplay, pBIOSInfo->CrtcVDisplay);
    lpDevMode->PelsWidth = pBIOSInfo->CrtcHDisplay;
    lpDevMode->PelsHeight = pBIOSInfo->CrtcVDisplay;
    lpDevMode->RefreshRateNum = 1;
    lpDevMode->RefreshRate[0] =  pBIOSInfo->FoundRefresh;
}

static void
VIA_UT_XV_FUNC_DISPLAY_GetRotationCaps(ScrnInfoPtr pScrn, CARD32 RotateType,
				       CARD32* pRotationCaps)
{
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIA_UT_XV_FUNC_DISPLAY_GetRotationCaps\n"));
    VIAPtr pVia = VIAPTR(pScrn);

    *pRotationCaps = UT_ROTATE_NONE;

    switch (RotateType)
    {
/*        case UT_ROTATE_TYPE_HW:
            if (IsSupportDirectAccessWindow(pBIOSInfo->Chipset))
            {
                *pRotationCaps = UT_ROTATE_90_DEG | UT_ROTATE_180_DEG | UT_ROTATE_270_DEG;
            }
            break;
  */
        case UT_ROTATE_TYPE_SW:
            *pRotationCaps = UT_ROTATE_90_DEG | UT_ROTATE_180_DEG | UT_ROTATE_270_DEG;
            break;

        /* STChen[2007.1.9]: Reserve this opion. We have no plan to open this feature currently. */
        case UT_ROTATE_TYPE_RANDR:
            if(pVia->RandRRotation)
            {
                /* This value should reference to VIARandRGetInfo(). */
                *pRotationCaps = UT_ROTATE_90_DEG | UT_ROTATE_180_DEG | UT_ROTATE_270_DEG;
            }
            break;
    }

}


static void
VIA_UT_XV_FUNC_DISPLAY_GetCurrentRotateDegree(ScrnInfoPtr pScrn, CARD32* RotateType,
					      CARD32* RotateDegree)
{
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIA_UT_XV_FUNC_DISPLAY_GetCurrentMode\n"));
    VIAPtr pVia = VIAPTR(pScrn);

    if (pVia->SWRotate && pVia->shadowFB && pVia->NoAccel)
    {
        *RotateType = UT_ROTATE_TYPE_SW;

        switch (pVia->SWRotate)
        {
            case VIA_ROTATE_DEGREE_90:
                *RotateDegree = UT_ROTATE_90_DEG;
                break;

            case VIA_ROTATE_DEGREE_180:
                *RotateDegree = UT_ROTATE_180_DEG;
                break;

            case VIA_ROTATE_DEGREE_270:
                *RotateDegree = UT_ROTATE_270_DEG;
                break;
        }
    }
    else
    {
        *RotateType = UT_ROTATE_TYPE_NONE;
        *RotateDegree = UT_ROTATE_NONE;
    }
}


static void
VIA_UT_XV_FUNC_DEVICE_GetPanelInfo(VIABIOSInfoPtr pBIOSInfo, CARD32 dwPanelType,
				   UTPANELINFO* pPanelInfo)
{
    int dwPanelSize;

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIA_UT_XV_FUNC_DEVICE_GetPanelInfo\n"));

    pPanelInfo->dwType = UT_PANEL_TYPE_TFT;

    /* VansonWu [2004.5.28] utility should add parameter for
       distinguish get LCD or DFP info. Before utility change
       query panel info. parameter. we patch this according
       desktop or mobile version. */
    /* RainLin [2005.09.06]: Add parameter to distinguish
       getting LCD or DFP information. */
    if (dwPanelType == VIA_DEVICE_LCD) {
	dwPanelSize = pBIOSInfo->LVDSSettingInfo.PanelSizeID;
    }
    else {
	VIAGetModeSizeByIndex(pBIOSInfo->TMDSSettingInfo.DFPSize,
			      (int*)(&pPanelInfo->dwResX),
			      (int*)(&pPanelInfo->dwResY));
	return;
    }

    switch (dwPanelSize) {
	case VIA_PANEL6X4:
	    pPanelInfo->dwResX = 640;
	    pPanelInfo->dwResY = 480;
	    break;
	case VIA_PANEL8X6:
	    pPanelInfo->dwResX = 800;
	    pPanelInfo->dwResY = 600;
	    break;
	case VIA_PANEL10X7:
	    pPanelInfo->dwResX = 1024;
	    pPanelInfo->dwResY = 768;
	    break;
	case VIA_PANEL1280X720:
	    pPanelInfo->dwResX = 1280;
	    pPanelInfo->dwResY = 720;
	    break;
	case VIA_PANEL12X7:
	    pPanelInfo->dwResX = 1280;
	    pPanelInfo->dwResY = 768;
	    break;
	case VIA_PANEL12X10:
	    pPanelInfo->dwResX = 1280;
	    pPanelInfo->dwResY = 1024;
	    break;
	case VIA_PANEL14X10:
	    pPanelInfo->dwResX = 1400;
	    pPanelInfo->dwResY = 1050;
	    break;
	case VIA_PANEL14X9:
	    pPanelInfo->dwResX = 1440;
	    pPanelInfo->dwResY = 900;
	    break;
	case VIA_PANEL16X12:
	    pPanelInfo->dwResX = 1600;
	    pPanelInfo->dwResY = 1200;
	    break;
	case VIA_PANEL12X8:
	    pPanelInfo->dwResX = 1280;
	    pPanelInfo->dwResY = 800;
	    break;
	case VIA_PANEL1366X7:
	    pPanelInfo->dwResX = 1366;
	    pPanelInfo->dwResY = 768;
	    break;
	case VIA_PANEL1360X7:
	    pPanelInfo->dwResX = 1360;
	    pPanelInfo->dwResY = 768;
	    break;
	case VIA_PANEL1920x1080:
	    pPanelInfo->dwResX = 1920;
	    pPanelInfo->dwResY = 1080;
	    break;
	case VIA_PANEL1920x1200:
	    pPanelInfo->dwResX = 1920;
	    pPanelInfo->dwResY = 1200;
	    break;
	default:
	    pPanelInfo->dwResX = 0;
	    pPanelInfo->dwResY = 0;
	    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "Unknown Panel Size!\n"));
	    break;
    }
}


static void
VIA_UT_XV_FUNC_DEVICE_GetSupportExpand(VIABIOSInfoPtr pBIOSInfo,
				       CARD32* pdwSupportState)
{
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIA_UT_XV_FUNC_DEVICE_GetSupportExpand\n"));
    /* RainLin [2005.12.14]: Don't support expansion if LCD uses IGA1. */
    if ((pBIOSInfo->LVDSSettingInfo.ChipID) &&
        ((pBIOSInfo->LVDSSettingInfo.IGAPath != IGA1) || (pBIOSInfo->LVDSSettingInfo2.ChipID)))
    {
        switch (pBIOSInfo->LVDSSettingInfo.PanelSizeID)
        {
            case VIA_PANEL6X4:
                if (pBIOSInfo->SaveHDisplay < 640 &&
                    pBIOSInfo->SaveVDisplay < 480)
                {
                    *pdwSupportState = UT_PANEL_SUPPORT_EXPAND_ON;
                }
                else
                {
                    *pdwSupportState = UT_PANEL_SUPPORT_EXPAND_OFF;
                }
                break;
            case VIA_PANEL8X6:
                if (pBIOSInfo->SaveHDisplay < 800 &&
                    pBIOSInfo->SaveVDisplay < 600)
                {
                    *pdwSupportState = UT_PANEL_SUPPORT_EXPAND_ON;
                }
                else
                {
                    *pdwSupportState = UT_PANEL_SUPPORT_EXPAND_OFF;
                }
                break;
            case VIA_PANEL10X7:
                if (pBIOSInfo->SaveHDisplay < 1024 &&
                    pBIOSInfo->SaveVDisplay < 768)
                {
                    *pdwSupportState = UT_PANEL_SUPPORT_EXPAND_ON;
                }
                else
                {
                    *pdwSupportState = UT_PANEL_SUPPORT_EXPAND_OFF;
                }
                break;
            case VIA_PANEL12X7:
                if (pBIOSInfo->SaveHDisplay < 1280 &&
                    pBIOSInfo->SaveVDisplay < 768)
                {
                    *pdwSupportState = UT_PANEL_SUPPORT_EXPAND_ON;
                }
                else
                {
                    *pdwSupportState = UT_PANEL_SUPPORT_EXPAND_OFF;
                }
                break;
            case VIA_PANEL12X10:
                if (pBIOSInfo->SaveHDisplay < 1280 &&
                    pBIOSInfo->SaveVDisplay < 1024)
                {
                    *pdwSupportState = UT_PANEL_SUPPORT_EXPAND_ON;
                }
                else
                {
                    *pdwSupportState = UT_PANEL_SUPPORT_EXPAND_OFF;
                }
                break;
            case VIA_PANEL14X10:
                if (pBIOSInfo->SaveHDisplay < 1400 &&
                    pBIOSInfo->SaveVDisplay < 1050)
                {
                    *pdwSupportState = UT_PANEL_SUPPORT_EXPAND_ON;
                }
                else
                {
                    *pdwSupportState = UT_PANEL_SUPPORT_EXPAND_OFF;
                }
                break;
            case VIA_PANEL16X12:
                if (pBIOSInfo->SaveHDisplay < 1600 &&
                    pBIOSInfo->SaveVDisplay < 1200)
                {
                    *pdwSupportState = UT_PANEL_SUPPORT_EXPAND_ON;
                }
                else
                {
                    *pdwSupportState = UT_PANEL_SUPPORT_EXPAND_OFF;
                }
                break;
            case VIA_PANEL1366X7:
                if (pBIOSInfo->SaveHDisplay < 1366 &&
                    pBIOSInfo->SaveVDisplay < 768)
                {
                    *pdwSupportState = UT_PANEL_SUPPORT_EXPAND_OFF;
                }
                else
                {
                    *pdwSupportState = UT_PANEL_SUPPORT_EXPAND_OFF;
                }
                break;
            case VIA_PANEL1360X7:
                if (pBIOSInfo->SaveHDisplay < 1360 &&
                    pBIOSInfo->SaveVDisplay < 768)
                {
                    *pdwSupportState = UT_PANEL_SUPPORT_EXPAND_OFF;
                }
                else
                {
                    *pdwSupportState = UT_PANEL_SUPPORT_EXPAND_OFF;
                }
                break;
            default:
                *pdwSupportState = UT_PANEL_SUPPORT_EXPAND_OFF;
                break;
        }
    }
    else
    {
        *pdwSupportState = UT_PANEL_SUPPORT_EXPAND_OFF;
    }

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIA_UT_XV_FUNC_DEVICE_GetSupportExpand\n"));

}


static void
VIA_UT_XV_FUNC_DEVICE_GetExpandState(VIABIOSInfoPtr pBIOSInfo,
				     CARD32* dwExpandState)
{
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIA_UT_XV_FUNC_DEVICE_GetExpandState\n"));
    if (pBIOSInfo->SAMM)
    {
        if (pBIOSInfo->ActiveDevice == VIA_DEVICE_LCD)
        {
            if (pBIOSInfo->LVDSSettingInfo.Center)
            {
                *dwExpandState = UT_STATE_EXPAND_OFF;
            }
            else
            {
                *dwExpandState = UT_STATE_EXPAND_ON;
            }
            DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "LCD:VIA_UT_XV_FUNC_DEVICE_GetExpandState = %d\n",*dwExpandState));
        }
        else if (pBIOSInfo->ActiveDevice == VIA_DEVICE_LCD2)
        {
            if (pBIOSInfo->LVDSSettingInfo2.Center)
            {
                *dwExpandState = UT_STATE_EXPAND_OFF;
            }
            else
            {
                *dwExpandState = UT_STATE_EXPAND_ON;
            }
            DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "LCD2:VIA_UT_XV_FUNC_DEVICE_GetExpandState = %d\n",*dwExpandState));
        }
    }
    else /* Single or Duoview */
    {
        if (pBIOSInfo->ActiveDevice == (VIA_DEVICE_LCD | VIA_DEVICE_LCD2))
        {
            if (pBIOSInfo->LVDSSettingInfo2.Center)
            {
                *dwExpandState = UT_STATE_EXPAND_OFF;
            }
            else
            {
                *dwExpandState = UT_STATE_EXPAND_ON;
            }
            DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "LCD2:VIA_UT_XV_FUNC_DEVICE_GetExpandState = %d\n",*dwExpandState));
        }
        else
        {
            if (pBIOSInfo->LVDSSettingInfo.Center)
            {
                *dwExpandState = UT_STATE_EXPAND_OFF;
            }
            else
            {
                *dwExpandState = UT_STATE_EXPAND_ON;
            }
            DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "LCD:VIA_UT_XV_FUNC_DEVICE_GetExpandState = %d\n",*dwExpandState));
        }
    }
}


static Bool
VIA_UT_XV_FUNC_DEVICE_SetExpandState(VIABIOSInfoPtr pBIOSInfo,
				     DisplayModePtr mode,
				     UTPANELINFO UTPanelInfo)
{
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIA_UT_XV_FUNC_DEVICE_SetExpandState\n"));

    if (UTPanelInfo.dwExpandState == UT_STATE_EXPAND_OFF)
    {
        if (pBIOSInfo->ActiveDevice & VIA_DEVICE_LCD)
        {
            pBIOSInfo->LVDSSettingInfo.Center = TRUE;
            ErrorF("LVDS1 Center = true\n");
        }

        if (pBIOSInfo->ActiveDevice & VIA_DEVICE_LCD2)
        {
            pBIOSInfo->LVDSSettingInfo2.Center = TRUE;
            ErrorF("LVDS2 Center = true\n");
        }

        pBIOSInfo->s3utility=TRUE;
    }
    else if (UTPanelInfo.dwExpandState == UT_STATE_EXPAND_ON)
    {
        if (pBIOSInfo->ActiveDevice & VIA_DEVICE_LCD)
        {
            pBIOSInfo->LVDSSettingInfo.Center = FALSE;
            ErrorF("LVDS1 Center = false\n");
        }

        if (pBIOSInfo->ActiveDevice & VIA_DEVICE_LCD2)
        {
            pBIOSInfo->LVDSSettingInfo2.Center = FALSE;
            ErrorF("LVDS2 Center = false\n");
        }

        pBIOSInfo->s3utility=TRUE;
    }
    else
    {
        return FALSE;
    }

    VIASaveUserSetting(pBIOSInfo);
    VIASwitchMode(pBIOSInfo->scrnIndex, mode, 0);

    return TRUE;
}


static void
VIA_UT_XV_FUNC_DEVICE_GetSupportState(VIABIOSInfoPtr pBIOSInfo,
				      CARD32*  pdwSupportState)
{
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIA_UT_XV_FUNC_DEVICE_GetSupportState\n"));

    unsigned int dwSupportState = UT_DEVICE_CRT1;

    if (pBIOSInfo->TVSettingInfo.TVEncoder)
    {
        /*[William 20070404] remove TV support if set below resolution*/
        if( (pBIOSInfo->HDisplay == 1280 && pBIOSInfo->VDisplay == 720) ||
            (pBIOSInfo->HDisplay == 1920 && pBIOSInfo->VDisplay == 1080))
                dwSupportState &= ~UT_DEVICE_TV;
        else
                dwSupportState |= UT_DEVICE_TV;

        if (pBIOSInfo->TVSettingInfo.TVEncoder == VIA_VT1622A ||
            pBIOSInfo->TVSettingInfo.TVEncoder == VIA_VT1623 ||
            pBIOSInfo->TVSettingInfo.TVEncoder == VIA_VT1625)
        {
            dwSupportState |= UT_DEVICE_HDTV;
        }

        if( pBIOSInfo->TVSettingInfo.TVEncoder == VIA_INTEGRATED_TV)
        {
            if(pBIOSInfo->dwProjectIDRevision != UNICHROME_VT3324)
            {
                dwSupportState |= UT_DEVICE_HDTV;
            }
        }

        if (pBIOSInfo->TVSettingInfo.TVEncoder == VIA_VT1625 && pBIOSInfo->CRT2_SUPPORTED)/*ian*/
        {
          dwSupportState |= UT_DEVICE_CRT2;
        }
    }

    if (pBIOSInfo->TVSettingInfo2.TVEncoder && (pBIOSInfo->Chipset == VIA_CX700))
    {
        dwSupportState |= UT_DEVICE_TV2 | UT_DEVICE_HDTV2;
    }


    if (pBIOSInfo->TMDSSettingInfo.ChipID)
    {
        dwSupportState |= UT_DEVICE_DFP;
    }

    if (pBIOSInfo->LVDSSettingInfo.ChipID)
    {
        dwSupportState |= UT_DEVICE_LCD;
    }

    if (pBIOSInfo->LVDSSettingInfo2.ChipID)
    {
        dwSupportState |= UT_DEVICE_LCD2;
    }

    *pdwSupportState = dwSupportState;

}


static void
VIA_UT_XV_FUNC_DEVICE_GetConnectState(VIABIOSInfoPtr pBIOSInfo,
				      CARD32* pdwConnectState)
{
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIA_UT_XV_FUNC_DEVICE_GetConnectState\n"));

    unsigned int dwConnectState = VIAGetDeviceDetect(pBIOSInfo);

    /* Map the device state for utility. */
    dwConnectState = MapDeviceStateForUT(dwConnectState);

    if (pBIOSInfo->TVSettingInfo.TVEncoder == VIA_VT1622A ||
        pBIOSInfo->TVSettingInfo.TVEncoder == VIA_VT1623 ||
        pBIOSInfo->TVSettingInfo.TVEncoder == VIA_VT1625 ||
        pBIOSInfo->TVSettingInfo.TVEncoder == VIA_INTEGRATED_TV)
    {
        if (dwConnectState & UT_DEVICE_TV)
        {
            dwConnectState |= UT_DEVICE_HDTV;
        }

        if (dwConnectState & UT_DEVICE_TV2)
        {
            dwConnectState |= UT_DEVICE_HDTV2;
        }

        if (pBIOSInfo->TVSettingInfo.TVEncoder == VIA_VT1625)/*ian*/
            dwConnectState |= UT_DEVICE_CRT2;
    }

    *pdwConnectState = dwConnectState;

}


static void
VIA_UT_XV_FUNC_DEVICE_GetActiveState(ScrnInfoPtr pScrn, CARD32* pdwActiveDevice)
{
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIA_UT_XV_FUNC_DEVICE_GetActiveState\n"));
    VIAPtr pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;
    unsigned int dwActiveState = 0;

    if (pBIOSInfo->SAMM)
    {
        DevUnion* pPriv = xf86GetEntityPrivate(pScrn->entityList[0], gVIAEntityIndex);
        VIAEntPtr pVIAEnt = pPriv->ptr;
        VIAPtr pVia1 = VIAPTR(pVIAEnt->pSecondaryScrn);
        VIAPtr pVia0 = VIAPTR(pVIAEnt->pPrimaryScrn);

        dwActiveState = (CARD32)pVia->ActiveDevice;

        /* Map the device state for utility. */
        dwActiveState = MapDeviceStateForUT(dwActiveState);

        if ((pVia0->pBIOSInfo->ActiveDevice & VIA_DEVICE_TV) &&
            (pVia0->pBIOSInfo->TVSettingInfo.TVType == TVTYPE_480P ||
             pVia0->pBIOSInfo->TVSettingInfo.TVType == TVTYPE_576P ||
             pVia0->pBIOSInfo->TVSettingInfo.TVType == TVTYPE_720P ||
             pVia0->pBIOSInfo->TVSettingInfo.TVType == TVTYPE_1080I))
        {
            dwActiveState &= ~UT_DEVICE_TV;
            dwActiveState |= UT_DEVICE_HDTV;
        }

        if ((pVia0->pBIOSInfo->ActiveDevice & VIA_DEVICE_TV2) &&
            (pVia0->pBIOSInfo->TVSettingInfo2.TVType == TVTYPE_480P ||
             pVia0->pBIOSInfo->TVSettingInfo2.TVType == TVTYPE_576P ||
             pVia0->pBIOSInfo->TVSettingInfo2.TVType == TVTYPE_720P ||
             pVia0->pBIOSInfo->TVSettingInfo2.TVType == TVTYPE_1080I))
        {
            dwActiveState &= ~UT_DEVICE_TV2;
            dwActiveState |= UT_DEVICE_HDTV2;
        }

        if ((pVia1->pBIOSInfo->ActiveDevice & VIA_DEVICE_TV) &&
            (pVia1->pBIOSInfo->TVSettingInfo.TVType == TVTYPE_480P ||
             pVia1->pBIOSInfo->TVSettingInfo.TVType == TVTYPE_576P ||
             pVia1->pBIOSInfo->TVSettingInfo.TVType == TVTYPE_720P ||
             pVia1->pBIOSInfo->TVSettingInfo.TVType == TVTYPE_1080I))
        {
            dwActiveState &= ~UT_DEVICE_TV;
            dwActiveState |= UT_DEVICE_HDTV;
        }

        if ((pVia1->pBIOSInfo->ActiveDevice & VIA_DEVICE_TV2) &&
            (pVia1->pBIOSInfo->TVSettingInfo2.TVType == TVTYPE_480P ||
             pVia1->pBIOSInfo->TVSettingInfo2.TVType == TVTYPE_576P ||
             pVia1->pBIOSInfo->TVSettingInfo2.TVType == TVTYPE_720P ||
             pVia1->pBIOSInfo->TVSettingInfo2.TVType == TVTYPE_1080I))
        {
            dwActiveState &= ~UT_DEVICE_TV2;
            dwActiveState |= UT_DEVICE_HDTV2;
        }
    }
    else
    {
        dwActiveState = (CARD32)pBIOSInfo->ActiveDevice;

        /* Map the device state for utility. */
        dwActiveState = MapDeviceStateForUT(dwActiveState);

        /* correct device ID for utility. */
        if ((dwActiveState & UT_DEVICE_TV) &&
            (pBIOSInfo->TVSettingInfo.TVType == TVTYPE_480P ||
             pBIOSInfo->TVSettingInfo.TVType == TVTYPE_576P ||
             pBIOSInfo->TVSettingInfo.TVType == TVTYPE_720P ||
             pBIOSInfo->TVSettingInfo.TVType == TVTYPE_1080I))
        {
            dwActiveState &= ~UT_DEVICE_TV;
            dwActiveState |= UT_DEVICE_HDTV;
        }

        if ((dwActiveState & UT_DEVICE_TV2) &&
            (pBIOSInfo->TVSettingInfo2.TVType == TVTYPE_480P ||
             pBIOSInfo->TVSettingInfo2.TVType == TVTYPE_576P ||
             pBIOSInfo->TVSettingInfo2.TVType == TVTYPE_720P ||
             pBIOSInfo->TVSettingInfo2.TVType == TVTYPE_1080I))
        {
            dwActiveState &= ~UT_DEVICE_TV2;
            dwActiveState |= UT_DEVICE_HDTV2;
        }
    }

    *pdwActiveDevice = dwActiveState;
}


static void
VIA_UT_XV_FUNC_DEVICE_SetActiveState(VIABIOSInfoPtr pBIOSInfo,
				     DisplayModePtr mode,
				     CARD32 dwActiveDevice)
{
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIA_UT_XV_FUNC_DEVICE_SetActiveState = 0x%x\n", dwActiveDevice));
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "TVType = 0x%x\n", pBIOSInfo->TVSettingInfo.TVType));
    if (!(pBIOSInfo->Is3DScaling))
    {
        if (dwActiveDevice & UT_DEVICE_TV)
        {
            if((pBIOSInfo->TVSettingInfo.TVType == TVTYPE_480P)||
               (pBIOSInfo->TVSettingInfo.TVType == TVTYPE_576P)||
               (pBIOSInfo->TVSettingInfo.TVType == TVTYPE_720P)||
               (pBIOSInfo->TVSettingInfo.TVType == TVTYPE_1080I))
            {
                pBIOSInfo->TVSettingInfo.TVType = TVTYPE_NTSC;
            }
        }

        if (dwActiveDevice & UT_DEVICE_TV2)
        {
            if((pBIOSInfo->TVSettingInfo2.TVType == TVTYPE_480P)||
               (pBIOSInfo->TVSettingInfo2.TVType == TVTYPE_576P)||
               (pBIOSInfo->TVSettingInfo2.TVType == TVTYPE_720P)||
               (pBIOSInfo->TVSettingInfo2.TVType == TVTYPE_1080I))
            {
                pBIOSInfo->TVSettingInfo2.TVType = TVTYPE_NTSC;
            }
        }

        if (dwActiveDevice & UT_DEVICE_HDTV)
        {
            if((pBIOSInfo->TVSettingInfo.TVType != TVTYPE_480P)&&
               (pBIOSInfo->TVSettingInfo.TVType != TVTYPE_576P)&&
               (pBIOSInfo->TVSettingInfo.TVType != TVTYPE_720P)&&
               (pBIOSInfo->TVSettingInfo.TVType != TVTYPE_1080I))
            {
                if (pBIOSInfo->HDisplay == 720 && pBIOSInfo->VDisplay == 480)
                {
                    pBIOSInfo->TVSettingInfo.TVType = TVTYPE_480P;
                }
                else if (pBIOSInfo->HDisplay == 720 && pBIOSInfo->VDisplay == 576)
                {
                    pBIOSInfo->TVSettingInfo.TVType = TVTYPE_576P;
                }
                else if (pBIOSInfo->HDisplay == 1280 && pBIOSInfo->VDisplay == 720)
                {
                    pBIOSInfo->TVSettingInfo.TVType = TVTYPE_720P;
                }
                else if (pBIOSInfo->HDisplay == 1920 && pBIOSInfo->VDisplay == 1080)
                {
                    pBIOSInfo->TVSettingInfo.TVType = TVTYPE_1080I;
                }
                else
                {
                    pBIOSInfo->TVSettingInfo.TVType = TVTYPE_480P;
                }
            }

            /* HDTV only using YPbPr or RGB */
            if (pBIOSInfo->TVSettingInfo.TVOutput != TV_OUTPUT_RGB &&
                pBIOSInfo->TVSettingInfo.TVOutput != TV_OUTPUT_YPBPR)
            {
                pBIOSInfo->TVSettingInfo.TVOutput = TV_OUTPUT_YPBPR;
            }
        }

        if (dwActiveDevice & UT_DEVICE_HDTV2)
        {
            if((pBIOSInfo->TVSettingInfo2.TVType != TVTYPE_480P)&&
               (pBIOSInfo->TVSettingInfo2.TVType != TVTYPE_576P)&&
               (pBIOSInfo->TVSettingInfo2.TVType != TVTYPE_720P)&&
               (pBIOSInfo->TVSettingInfo2.TVType != TVTYPE_1080I))
            {
                if (pBIOSInfo->HDisplay == 720 && pBIOSInfo->VDisplay == 480)
                {
                    pBIOSInfo->TVSettingInfo2.TVType = TVTYPE_480P;
                }
                else if (pBIOSInfo->HDisplay == 720 && pBIOSInfo->VDisplay == 576)
                {
                    pBIOSInfo->TVSettingInfo2.TVType = TVTYPE_576P;
                }
                else if (pBIOSInfo->HDisplay == 1280 && pBIOSInfo->VDisplay == 720)
                {
                    pBIOSInfo->TVSettingInfo2.TVType = TVTYPE_720P;
                }
                else if (pBIOSInfo->HDisplay == 1920 && pBIOSInfo->VDisplay == 1080)
                {
                    pBIOSInfo->TVSettingInfo2.TVType = TVTYPE_1080I;
                }
                else
                {
                    pBIOSInfo->TVSettingInfo2.TVType = TVTYPE_480P;
                }
            }

            /* HDTV only using YPbPr or RGB */
            if (pBIOSInfo->TVSettingInfo2.TVOutput != TV_OUTPUT_RGB &&
                pBIOSInfo->TVSettingInfo2.TVOutput != TV_OUTPUT_YPBPR)
            {
                pBIOSInfo->TVSettingInfo2.TVOutput = TV_OUTPUT_YPBPR;
            }
        }
    }
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "TVType = 0x%x\n", pBIOSInfo->TVSettingInfo.TVType));


    if (dwActiveDevice & UT_DEVICE_CRT2) /*ian*/
    {
        pBIOSInfo->TwoCRT = TRUE;
    }
    else
    {
        pBIOSInfo->TwoCRT = FALSE;
    }

    /* Convert utility's device ID into driver's device ID. */
    pBIOSInfo->ActiveDevice = MapDeviceStateForDrv(dwActiveDevice);

    /*update the information for .VIARC*/
    pBIOSInfo->ActualActiveDevice = pBIOSInfo->ActiveDevice;

    /* Avoid device switch in panning mode or expansion */
    pBIOSInfo->scaleY = FALSE;

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "1SetActiveState = 0x%x\n", pBIOSInfo->ActiveDevice));

    VIASwitchMode(pBIOSInfo->scrnIndex, mode, 0);
    VIASaveUserSetting(pBIOSInfo);
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "2SetActiveState = 0x%x\n", pBIOSInfo->ActiveDevice));
}


static void
VIA_UT_XV_FUNC_DEVICE_GetSAMMState(VIABIOSInfoPtr pBIOSInfo,
				   CARD32* pdwSAMMState)
{
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIA_UT_XV_FUNC_DEVICE_GetSAMMState\n"));
    if (pBIOSInfo->SAMM)
        *pdwSAMMState = UT_STATE_SAMM_ON;
    else
        *pdwSAMMState = UT_STATE_SAMM_OFF;
}


static void
VIA_UT_XV_FUNC_DEVICE_GetSupportSAMMState(VIABIOSInfoPtr pBIOSInfo,
					  CARD32* pdwSAMMState)
{
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIA_UT_XV_FUNC_DEVICE_GetSupportSAMMState\n"));
    if (!pBIOSInfo->Hotkey)
    {
        *pdwSAMMState = UT_DEVICE_SUPPORT_SAMM_STATE_ON;
    }
    else
    {
        *pdwSAMMState = UT_DEVICE_SUPPORT_SAMM_STATE_OFF;
    }
}


static Bool
VIA_UT_XV_FUNC_DEVICE_SetV3OnIGAStatus(ScrnInfoPtr pScrn, CARD32 dwIGAPath)
{
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIA_UT_XV_FUNC_DEVICE_SetV3OnIGAStatus\n"));
    VIAPtr pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;
    PVIDDATA        pVidData = pVia->pVidData;

    viaPortPrivRec      tmp_priv;
    LPDDUPDATEOVERLAY   lpUpdateOverlay = &pVidData->UpdateOverlayRec;
    LPVIAGRAPHICINFO    lpVIAGraphicInfo = &pVidData->VIAGraphicInfo;
    int ret = 0;
    int fd2;

    if (dwIGAPath == UT_STATE_DuoView_IGA1)
    {
        if(pVia->pVidData->VIAGraphicInfo.duoview &&
          ((pBIOSInfo->Chipset ==VIA_KM400) ||
          (pBIOSInfo->Chipset ==VIA_K8M800) ||
          (pBIOSInfo->Chipset ==VIA_K8M890) ||
          (pBIOSInfo->Chipset ==VIA_P4M890) ||
          (pBIOSInfo->Chipset == VIA_P4M800PRO) ||
          (pBIOSInfo->Chipset == VIA_P4M900)))
            lpVIAGraphicInfo->duoview &=~DUOVIEW_V3_ON_IGA2;

        ret = TRUE;
    }
    else if (dwIGAPath == UT_STATE_DuoView_IGA2)
    {
        if(pVia->pVidData->VIAGraphicInfo.duoview &&
          ((pBIOSInfo->Chipset ==VIA_KM400) ||
          (pBIOSInfo->Chipset ==VIA_K8M800) ||
          (pBIOSInfo->Chipset ==VIA_K8M890) ||
          (pBIOSInfo->Chipset ==VIA_P4M890) ||
          (pBIOSInfo->Chipset == VIA_P4M800PRO) ||
          (pBIOSInfo->Chipset == VIA_P4M900)))
            lpVIAGraphicInfo->duoview |=DUOVIEW_V3_ON_IGA2;

        ret = TRUE;
    }
    else
    {
        ret = FALSE;
    }

    switch( pBIOSInfo->Chipset )
    {
        case VIA_KM400:
        case VIA_CLE266:
        case VIA_K8M800:
        case VIA_P4M800PRO:
           tmp_priv.SurfaceDesc.dwFourCC = FOURCC_YV12;
             break;
        case VIA_PM800:
        case VIA_CX700:
        case VIA_CN750:
        case VIA_P4M890:
        case VIA_K8M890:
        case VIA_P4M900:
        default:
            tmp_priv.SurfaceDesc.dwFourCC = FOURCC_NV12;/*FOURCC_YUY2;*/
            break;
    }

    if (pVia->V4LEnabled == 1)
    {
        fd2 = open(pVia->video2Path ,O_RDWR);
        if ( -1 == ioctl(fd2, VIA_VID_SET_2D_INFO, pVia->pVidData) )
        {/*2005.10.3 Fix VMI can't switch in duoview mode (Al Zhang)*/
            xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
                       "IOCTL VIAMGR_INFO_XSERVERON fail.\n");
            return FALSE;
        }
        if(pVidData->VidStatus.dwVideoFlagMPEG[pVidData->VMI_Info.handle])
        {
             *lpUpdateOverlay = pVidData->UpdateOverlayMPEGBackup;
        }
        if(pVidData->VidStatus.dwVideoFlagSW[pVidData->VMI_Info.handle])
        {
            *lpUpdateOverlay = pVidData->UpdateOverlaySWBackup;
        }
        tmp_priv.fd = fd2;
        VIAOVUpdateOverlay(pScrn,&tmp_priv,lpUpdateOverlay,0,0,0,0,0,0);
        close(fd2);
    }
    else /*V4L not enabled, VMI Only*/
    {
        if(pVidData->VidStatus.dwVideoFlagMPEG[pVidData->VMI_Info.handle])
        {
             *lpUpdateOverlay = pVidData->UpdateOverlayMPEGBackup;
        }
        if(pVidData->VidStatus.dwVideoFlagSW[pVidData->VMI_Info.handle])
        {
            *lpUpdateOverlay = pVidData->UpdateOverlaySWBackup;
        }
       VIAOVUpdateOverlay(pScrn,&tmp_priv,lpUpdateOverlay,0,0,0,0,0,0);
    }

    return ret;
}


static void
VIA_UT_XV_FUNC_DEVICE_GetV3OnIGAStatus(ScrnInfoPtr pScrn, CARD32* pdwStatus)
{
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIA_UT_XV_FUNC_DEVICE_SetV3OnIGAStatus\n"));
    VIAPtr pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;
    if(pVia->pVidData->VIAGraphicInfo.duoview &&
       ((pBIOSInfo->Chipset == VIA_KM400)  ||
       (pBIOSInfo->Chipset == VIA_K8M800)  ||
       (pBIOSInfo->Chipset == VIA_K8M890)  ||
       (pBIOSInfo->Chipset == VIA_P4M890)  ||
       (pBIOSInfo->Chipset == VIA_P4M800PRO) ||
       (pBIOSInfo->Chipset == VIA_P4M900)))
    {
        *pdwStatus = (pVia->pVidData->VIAGraphicInfo.duoview >> 1) & 0x01;
    }
}

static void
VIA_UT_XV_FUNC_DEVICE_GetDuoViewStatus(ScrnInfoPtr pScrn, CARD32* pdwState)
{
    VIAPtr pVia = VIAPTR(pScrn);

    if (pVia->pVidData->VIAGraphicInfo.duoview&DUOVIEW_ENABLED)
        *pdwState = UT_STATE_DuoView_ON;
    else
        *pdwState = UT_STATE_DuoView_OFF;
}

static void
VIA_UT_XV_FUNC_TV_GetSupportStandardState(VIABIOSInfoPtr pBIOSInfo,
					  TVSETTINGINFOPTR pTVSettingInfo,
					  CARD32* pdwSupportState)
{
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIA_UT_XV_FUNC_TV_GetSupportStandardState\n"));

    if (pBIOSInfo->HDisplay == 720)
    {
        if (pBIOSInfo->VDisplay == 480)
        {
            *pdwSupportState = UT_TV_STD_NTSC;
        }
        else if (pBIOSInfo->VDisplay == 576)
        {
            *pdwSupportState = UT_TV_STD_PAL;
        }
        else
        {
            *pdwSupportState = UT_TV_STD_NTSC | UT_TV_STD_PAL;
        }
    }
    else
    {
        *pdwSupportState = UT_TV_STD_NTSC | UT_TV_STD_PAL;
    }
}


static void
VIA_UT_XV_FUNC_TV_GetStandard(VIABIOSInfoPtr pBIOSInfo,
			      TVSETTINGINFOPTR pTVSettingInfo,
			      CARD32* pdwStandard)
{
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIA_UT_XV_FUNC_TV_GetStandard\n"));

    switch (pTVSettingInfo->TVType)
    {
        case TVTYPE_NTSC:
            *pdwStandard = UT_TV_STD_NTSC;
            break;
        case TVTYPE_PAL:
            *pdwStandard = UT_TV_STD_PAL;
            break;
        case TVTYPE_NONE:
        default:
            *pdwStandard = 0;
            break;
    }
   xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIA_UT_XV_FUNC_TV_GetStandard:pTVSettingInfo->TVType=%d \n",pTVSettingInfo->TVType);
}


static Bool
VIA_UT_XV_FUNC_TV_SetStandard(VIABIOSInfoPtr pBIOSInfo,
			      TVSETTINGINFOPTR pTVSettingInfo,
			      DisplayModePtr mode, CARD32 dwStandard)
{
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIA_UT_XV_FUNC_TV_SetStandard\n"));

    int ret;
    switch (dwStandard)
    {
        case UT_TV_STD_NTSC :
            pTVSettingInfo->TVType = TVTYPE_NTSC;
            ret = TRUE;
            break;

        case UT_TV_STD_PAL :
            pTVSettingInfo->TVType = TVTYPE_PAL;
            ret = TRUE;
            break;

        default :
            ret = FALSE;
            break;
    }
    if(ret)
    {
        if (pBIOSInfo->ActiveDevice & (VIA_DEVICE_TV | VIA_DEVICE_TV2))
        {
            pBIOSInfo->s3utility=TRUE;
            VIASwitchMode(pBIOSInfo->scrnIndex, mode, 0);
            VIASaveUserSetting(pBIOSInfo);
        }
    }
    xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIA_UT_XV_FUNC_TV_SetStandard: pTVSettingInfo->TVType=%d\n",pTVSettingInfo->TVType);

    return ret;

}

static void
VIA_UT_XV_FUNC_TV_GetSupportSignalTypeState(VIABIOSInfoPtr pBIOSInfo,
					    TVSETTINGINFOPTR pTVSettingInfo,
					    CARD32* pdwSupportState)
{
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIA_UT_XV_FUNC_TV_GetSupportSignalTypeState\n"));

    VIAGetActiveDisplay(pBIOSInfo);
    switch (pTVSettingInfo->TVEncoder)
    {
        case VIA_VT1621:
            *pdwSupportState = UT_TV_SGNL_COMPOSITE | UT_TV_SGNL_S_VIDEO;
            break;
        case VIA_VT1622:
        case VIA_VT1622A:
        case VIA_VT1623:
        case VIA_VT1625:
        case VIA_INTEGRATED_TV:
            *pdwSupportState = UT_TV_SGNL_COMPOSITE | UT_TV_SGNL_S_VIDEO |
                          UT_TV_SGNL_SCART | UT_TV_SGNL_COMP_OUTPUT;
            break;
        default:
            *pdwSupportState = 0;
            break;
    }
}

static void
VIA_UT_XV_FUNC_TV_GetSignalType(VIABIOSInfoPtr pBIOSInfo,
				TVSETTINGINFOPTR pTVSettingInfo,
				CARD32* pdwSignalType)
{
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIA_UT_XV_FUNC_TV_GetSignalType\n"));

    switch (pTVSettingInfo->TVOutput)
    {
        case TV_OUTPUT_COMPOSITE:
        case TV_OUTPUT_COMPOSITE_SVIDEO:
            *pdwSignalType = UT_TV_SGNL_COMPOSITE;
            break;
        case TV_OUTPUT_SVIDEO:
            *pdwSignalType = UT_TV_SGNL_S_VIDEO;
            break;
        case TV_OUTPUT_RGB:
            *pdwSignalType = UT_TV_SGNL_SCART;
            break;
        case TV_OUTPUT_YPBPR:
            *pdwSignalType = UT_TV_SGNL_COMP_OUTPUT;
            break;
        case TV_OUTPUT_RGB_COMPOSITE_SVIDEO:
            *pdwSignalType = UT_TV_SGNL_RGB_COMPOSITE_SVIDEO;
            break;
        case TV_OUTPUT_YPBPR_COMPOSITE_SVIDEO:
            *pdwSignalType = UT_TV_SGNL_COMPONENT_COMPOSITE_SVIDEO;
            break;
        default:
            *pdwSignalType = 0;
            break;
    }
}


static Bool
VIA_UT_XV_FUNC_TV_SetSignalType(VIABIOSInfoPtr pBIOSInfo,
				TVSETTINGINFOPTR pTVSettingInfo,
				DisplayModePtr mode,
				CARD32 dwSignal)
{
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIA_UT_XV_FUNC_TV_SetSignalType\n"));

    int ret;
    switch (dwSignal)
    {
        case UT_TV_SGNL_COMPOSITE :
            pTVSettingInfo->TVOutput = TV_OUTPUT_COMPOSITE;
            ret = TRUE;
            break;
        case UT_TV_SGNL_S_VIDEO :
            pTVSettingInfo->TVOutput = TV_OUTPUT_SVIDEO;
            ret = TRUE;
            break;
        case UT_TV_SGNL_SCART :
            pTVSettingInfo->TVOutput = TV_OUTPUT_RGB;
            ret = TRUE;
            break;
        case UT_TV_SGNL_COMP_OUTPUT :
            pTVSettingInfo->TVOutput = TV_OUTPUT_YPBPR;
            ret = TRUE;
            break;
        case UT_TV_SGNL_COMPONENT_COMPOSITE_SVIDEO :
            pTVSettingInfo->TVOutput = TV_OUTPUT_YPBPR_COMPOSITE_SVIDEO;
            ret = TRUE;
            break;
        case UT_TV_SGNL_RGB_COMPOSITE_SVIDEO:
            pTVSettingInfo->TVOutput = TV_OUTPUT_RGB_COMPOSITE_SVIDEO;
            ret = TRUE;
            break;
        default :
            ret = FALSE;
            break;
    }
    if(ret)
    {
        if (pBIOSInfo->ActiveDevice & (VIA_DEVICE_TV | VIA_DEVICE_TV2))
        {
            pBIOSInfo->s3utility=TRUE;
            VIASwitchMode(pBIOSInfo->scrnIndex, mode, 0);
            VIASaveUserSetting(pBIOSInfo);
        }
    }

    return ret;
}

static void
VIA_UT_XV_FUNC_TV_GetMaxViewSizeValue(VIABIOSInfoPtr pBIOSInfo,
				      TVSETTINGINFOPTR pTVSettingInfo,
				      UTXYVALUE* pMaxViewSizeValue)
{
    xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO,
	       "VIA_UT_XV_FUNC_TV_GetMaxViewSizeValue\n");

    if (pBIOSInfo->Is3DScaling) {
        VIA3DSclSCNParasPtr pTV3DScalInfo = &(pTVSettingInfo->TV3DScalInfo);

	pMaxViewSizeValue->dwX = pTV3DScalInfo->XScaleMaximum;
	pMaxViewSizeValue->dwY = pTV3DScalInfo->YScaleMaximum;
	xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO,
		   "GetViewMaxSizeValue: MaxX=%d,MaxY=%d\n",
		   (int)pMaxViewSizeValue->dwX,
		   (int)pMaxViewSizeValue->dwY);

    }
    else {
	TVInfoPtr  pTVInfo = &(pTVSettingInfo->TVInfo);

	pMaxViewSizeValue->dwX = pTVInfo->ScalHLevel;
	pMaxViewSizeValue->dwY = pTVInfo->ScalVLevel;
    }
}

static void
VIA_UT_XV_FUNC_TV_GetViewSizeValue(VIABIOSInfoPtr pBIOSInfo,
				   TVSETTINGINFOPTR pTVSettingInfo,
				   UTXYVALUE* pViewSizeValue)
{
    xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO,
	       "VIA_UT_XV_FUNC_TV_GetViewSizeValue\n");

    if (pBIOSInfo->Is3DScaling) {
	VIA3DSclSCNParasPtr pTV3DScalInfo = &(pTVSettingInfo->TV3DScalInfo);

	pViewSizeValue->dwX = pTV3DScalInfo->XScaleLevel;
	pViewSizeValue->dwY = pTV3DScalInfo->YScaleLevel;
	xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO,
		   "GetViewSizeValue: X=%d,Y=%d\n",
		   (int)pViewSizeValue->dwX, (int)pViewSizeValue->dwY);
    }
    else {
	TVInfoPtr  pTVInfo = &(pTVSettingInfo->TVInfo);

	pViewSizeValue->dwX = pTVInfo->CurrentScalH + 1;
	pViewSizeValue->dwY = pTVInfo->CurrentScalV + 1;
    }
}

/*adjust TV/HDTV or other device size, position, we dont need to call switch mode */
/*change TV/HDTV or other device, or change TV/HDTV Signal standard, , we need to call switch mode */
static void
VIA_UT_XV_FUNC_TV_SetViewSizeValue(VIABIOSInfoPtr pBIOSInfo,
				   TVSETTINGINFOPTR pTVSettingInfo,
				   DisplayModePtr mode,
				   UTXYVALUE ViewSizeValue)
{
    VIA3DSclSCNParasPtr     pTV3DScalInfo = &(pTVSettingInfo->TV3DScalInfo);
    ScrnInfoPtr pScrn = xf86Screens[pBIOSInfo->scrnIndex];
    VIAPtr      pVia = VIAPTR(pScrn);

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIA_UT_XV_FUNC_TV_SetViewSizeValue\n"));
    if (pBIOSInfo->Is3DScaling)
    {
        if (ViewSizeValue.dwY == 0xFFFF ||
            ViewSizeValue.dwX == 0xFFFF)
        {
            ViewSizeValue.dwX = pTV3DScalInfo->XScaleDefault;
            ViewSizeValue.dwY = pTV3DScalInfo->YScaleDefault;
            pTV3DScalInfo->XScaleLevel = pTV3DScalInfo->XScaleDefault;
            pTV3DScalInfo->YScaleLevel = pTV3DScalInfo->YScaleDefault;
            pTVSettingInfo->TVVScan = VIA_TVOVER;
        }
        pTV3DScalInfo->XScaleLevel = ViewSizeValue.dwX;
        pTV3DScalInfo->YScaleLevel = ViewSizeValue.dwY;
        pTV3DScalInfo->XSIZEOffset = (pTV3DScalInfo->XScaleDefault - pTV3DScalInfo->XScaleLevel)*pTV3DScalInfo->XScaleStep;
        pTV3DScalInfo->YSIZEOffset = (pTV3DScalInfo->YScaleDefault - pTV3DScalInfo->YScaleLevel)*pTV3DScalInfo->YScaleStep;

        pTV3DScalInfo->XSIZENow = pTV3DScalInfo->XSIZEMax - (pTV3DScalInfo->XScaleLevel*pTV3DScalInfo->XScaleStep);
        pTV3DScalInfo->YSIZENow = pTV3DScalInfo->YSIZEMax - (pTV3DScalInfo->YScaleLevel*pTV3DScalInfo->YScaleStep);

        pTV3DScalInfo->bNeedBlack3dScalingBorders =TRUE;

        hw3d_CursorPositionUpdate(pBIOSInfo);

        pVia->pVidData->gfx3DScal_info.XSIZEOffset = pTV3DScalInfo->XSIZEOffset;
        pVia->pVidData->gfx3DScal_info.YSIZEOffset = pTV3DScalInfo->YSIZEOffset;
        VIAAdjustFrame(pScrn->scrnIndex, pScrn->frameX0, pScrn->frameY0, 0);

        VIASaveUserSetting(pBIOSInfo);
    }
    else
    {
        TVInfoPtr       pTVInfo = &(pTVSettingInfo->TVInfo);

        if (pTVSettingInfo->TVEncoder == VIA_INTEGRATED_TV)
        {
            AdjustTVSize_IntegratedTV(pBIOSInfo, pTVSettingInfo, ViewSizeValue.dwX, ViewSizeValue.dwY);
            VIASaveUserSetting(pBIOSInfo);
        }
        else
        {
            if (ViewSizeValue.dwY == 0xFFFF ||
                ViewSizeValue.dwX == 0xFFFF)
            {
                ViewSizeValue.dwX = pTVInfo->DefaultScalH + 1;
                ViewSizeValue.dwY = pTVInfo->DefaultScalV + 1;
                pTVInfo->CurrentScalH = pTVInfo->DefaultScalH;
                pTVInfo->CurrentScalV = pTVInfo->DefaultScalV;
                pTVSettingInfo->TVVScan = VIA_TVOVER;
            }

            pTVInfo->CurrentScalH = ViewSizeValue.dwX - 1;
            pTVInfo->CurrentScalV = ViewSizeValue.dwY - 1;

            {
                pTVSettingInfo->TVVScan = (int)ViewSizeValue.dwY - 1;

                pBIOSInfo->s3utility=TRUE;
                VIASaveUserSetting(pBIOSInfo);
                VIASwitchMode(pBIOSInfo->scrnIndex, mode, 0);
            }
        }
    }
}

static void
VIA_UT_XV_FUNC_TV_GetMaxViewPositionValue(VIABIOSInfoPtr pBIOSInfo,
					  TVSETTINGINFOPTR pTVSettingInfo,
					  UTXYVALUE* pMaxViewPosValue)
{
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO,
		     "VIA_UT_XV_FUNC_TV_GetMaxViewPositionValue\n"));

    if (pBIOSInfo->Is3DScaling) {
	VIA3DSclSCNParasPtr pTV3DScalInfo = &(pTVSettingInfo->TV3DScalInfo);

	pMaxViewPosValue->dwX = pTV3DScalInfo->XPOSITIONMaximum;
	pMaxViewPosValue->dwY = pTV3DScalInfo->YPOSITIONMaximum;
	xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO,
		   "GetMaxViewPositionValue:MaxX=%d,MaxY=%d\n",
		   (int)pMaxViewPosValue->dwX, (int)pMaxViewPosValue->dwY);

    }
    else {
	TVInfoPtr pTVInfo = &(pTVSettingInfo->TVInfo);

	pMaxViewPosValue->dwX = pTVInfo->PositionHLevel;
	pMaxViewPosValue->dwY = pTVInfo->PositionVLevel;
    }
}

static void
VIA_UT_XV_FUNC_TV_GetViewPositionValue(VIABIOSInfoPtr pBIOSInfo,
				       TVSETTINGINFOPTR pTVSettingInfo,
				       UTXYVALUE* pViewPosValue)
{
    xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO,
	       "VIA_UT_XV_FUNC_TV_GetViewPositionValue\n");

    if (pBIOSInfo->Is3DScaling) {
	VIA3DSclSCNParasPtr pTV3DScalInfo = &(pTVSettingInfo->TV3DScalInfo);

	if (pViewPosValue->dwX == 0xFFFF && pViewPosValue->dwY == 0xFFFF) {
	    pViewPosValue->dwX = pTV3DScalInfo->XPOSITIONDefault;
	    pViewPosValue->dwY = pTV3DScalInfo->YPOSITIONDefault;
	}
	else {
	    pViewPosValue->dwX = pTV3DScalInfo->XPOSITIONLevel;
	    pViewPosValue->dwY = pTV3DScalInfo->YPOSITIONLevel;
	}

	xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO,
		   "GetViewPositionValue:X=%d,Y=%d\n",
		   (int)pViewPosValue->dwX, (int)pViewPosValue->dwY);
    }
    else {
	TVInfoPtr pTVInfo = &(pTVSettingInfo->TVInfo);

	if (pViewPosValue->dwX == 0xFFFF && pViewPosValue->dwY == 0xFFFF) {
	    pViewPosValue->dwX = pTVInfo->DefaultPositionH;
	    pViewPosValue->dwY = pTVInfo->DefaultPositionV;
	}
	else {
	    pViewPosValue->dwX = pTVInfo->CurrentPositionH;
	    pViewPosValue->dwY = pTVInfo->CurrentPositionV;
	}
	xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO,
		   "PositionValue_X = %d\n", (int)pViewPosValue->dwX);
	xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO,
		   "PositionValue_Y = %d\n", (int)pViewPosValue->dwY);
    }
}

static Bool
VIA_UT_XV_FUNC_TV_SetViewPositionValue(VIABIOSInfoPtr pBIOSInfo,
				       TVSETTINGINFOPTR pTVSettingInfo,
				       UTXYVALUE ViewPosValue)
{
    unsigned char W_Buffer[4], R_Buffer[4];	/* For VT1622, VT1623*/
    I2CDevPtr dev = NULL;			/* For I2C use. */
    VIA3DSclSCNParasPtr pTV3DScalInfo = &(pTVSettingInfo->TV3DScalInfo);
    ScrnInfoPtr pScrn = xf86Screens[pBIOSInfo->scrnIndex];
    VIAPtr pVia = VIAPTR(pScrn);
    TVInfoPtr pTVInfo = &(pTVSettingInfo->TVInfo);
    int HPos = 0, VPos = 0;
    Bool result = TRUE;

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO,
		     "VIA_UT_XV_FUNC_TV_SetViewPositionValue\n"));

    if (pBIOSInfo->Is3DScaling) {
	if (ViewPosValue.dwY == 0xFFFF || ViewPosValue.dwX == 0xFFFF) {
	    ViewPosValue.dwX = pTV3DScalInfo->XPOSITIONDefault;
	    ViewPosValue.dwY = pTV3DScalInfo->YPOSITIONDefault;
	    pTV3DScalInfo->XPOSITIONLevel = pTV3DScalInfo->XPOSITIONDefault;
	    pTV3DScalInfo->YPOSITIONLevel = pTV3DScalInfo->YPOSITIONDefault;
	    pTVSettingInfo->TVVScan = VIA_TVOVER;
	    xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO,
		       "Default setting SetViewPositionValue:TVVScan= %d\n",
		       pTVSettingInfo->TVVScan);
	}
	pTV3DScalInfo->XPOSITIONLevel = ViewPosValue.dwX;
	pTV3DScalInfo->YPOSITIONLevel = ViewPosValue.dwY;
	pTV3DScalInfo->XPOSITIONOffset = (pTV3DScalInfo->XPOSITIONLevel -
					  pTV3DScalInfo->XPOSITIONDefault) *
	    pTV3DScalInfo->XPOSITIONStep;
	pTV3DScalInfo->YPOSITIONOffset = (pTV3DScalInfo->YPOSITIONLevel -
					  pTV3DScalInfo->YPOSITIONDefault) *
	    pTV3DScalInfo->YPOSITIONStep;

	pTV3DScalInfo->XPOSITIONNow = (pTV3DScalInfo->XPOSITIONLevel -
				       pTV3DScalInfo->XPOSITIONDefault) *
	    pTV3DScalInfo->XPOSITIONStep;
	pTV3DScalInfo->YPOSITIONNow = (pTV3DScalInfo->YPOSITIONLevel -
				       pTV3DScalInfo->YPOSITIONDefault) *
	    pTV3DScalInfo->YPOSITIONStep;

	pTV3DScalInfo->bNeedBlack3dScalingBorders = TRUE;

	hw3d_CursorPositionUpdate(pBIOSInfo);

	pVia->pVidData->gfx3DScal_info.XPOSITIONOffset =
	    pTV3DScalInfo->XPOSITIONOffset;
	pVia->pVidData->gfx3DScal_info.YPOSITIONOffset =
	    pTV3DScalInfo->YPOSITIONOffset;
	VIAAdjustFrame(pScrn->scrnIndex, pScrn->frameX0, pScrn->frameY0, 0);

	VIASaveUserSetting(pBIOSInfo);
    }
    else {
	/* Set default Position */
	if (ViewPosValue.dwX == 0xFFFF && ViewPosValue.dwX == 0xFFFF) {
	    ViewPosValue.dwX = pTVInfo->DefaultPositionH;
	    ViewPosValue.dwY = pTVInfo->DefaultPositionV;
	}

	dev = xf86CreateI2CDevRec();
	dev->DevName = "TV";
	dev->SlaveAddr = pBIOSInfo->TVI2CAdd;
	dev->pI2CBus = pBIOSInfo->I2C_Port2;

	if (xf86I2CDevInit(dev)) {
	    switch (pTVSettingInfo->TVEncoder) {
		case VIA_VT1621:
		    pTVInfo->CurrentPositionH = 0;
		    pTVInfo->CurrentPositionV = 0;
		    break;
		case VIA_VT1622:
		case VIA_VT1622A:
		    W_Buffer[0] = 0x08;
		    xf86I2CWriteRead(dev, W_Buffer,1, R_Buffer,2);
		    HPos = R_Buffer[0];
		    VPos = R_Buffer[1];
		    W_Buffer[0] = 0x1C;
		    xf86I2CWriteRead(dev, W_Buffer,1, R_Buffer,1);
		    HPos |= (R_Buffer[0] & 0x04) << 6;
		    VPos |= (R_Buffer[0] & 0x02) << 7;

		    HPos += ViewPosValue.dwX - pTVInfo->CurrentPositionH;
		    VPos += ViewPosValue.dwY - pTVInfo->CurrentPositionV;

		    W_Buffer[0] = 0x08;
		    W_Buffer[1] = HPos & 0xFF;
		    W_Buffer[2] = VPos & 0xFF;
		    xf86I2CWriteRead(dev, W_Buffer,3, NULL,0);

		    W_Buffer[0] = 0x1C;
		    W_Buffer[1] = R_Buffer[0] & ~0x06;	/* [2]1CH, [1]1CH */
		    W_Buffer[1] |= (HPos >> 6) & 0x04;
		    W_Buffer[1] |= (VPos >> 7) & 0x02;
		    xf86I2CWriteRead(dev, W_Buffer,2, NULL,0);

		    pTVInfo->CurrentPositionH = ViewPosValue.dwX;
		    pTVInfo->CurrentPositionV = ViewPosValue.dwY;
		    break;
		case VIA_VT1623:
		    GPIOI2C_Initial(pBIOSInfo, 0x40);
		    W_Buffer[0] = 0x08;
		    GPIOI2C_Read(pBIOSInfo, W_Buffer[0], R_Buffer, 2);
		    HPos = R_Buffer[0];
		    VPos = R_Buffer[1];
		    W_Buffer[0] = 0x1C;
		    GPIOI2C_Read(pBIOSInfo, W_Buffer[0], R_Buffer, 1);
		    HPos |= (R_Buffer[0] & 0x04) << 6;
		    VPos |= (R_Buffer[0] & 0x02) << 7;

		    HPos += ViewPosValue.dwX - pTVInfo->CurrentPositionH;
		    VPos += ViewPosValue.dwY - pTVInfo->CurrentPositionV;

		    W_Buffer[0] = 0x08;
		    W_Buffer[1] = HPos & 0xFF;
		    W_Buffer[2] = VPos & 0xFF;
		    GPIOI2C_Write(pBIOSInfo, W_Buffer[0], W_Buffer[1]);
		    W_Buffer[0] = 0x09;
		    GPIOI2C_Write(pBIOSInfo, W_Buffer[0], W_Buffer[2]);

		    W_Buffer[0] = 0x1C;
		    W_Buffer[1] = R_Buffer[0] & ~0x06;	/* [2]1CH, [1]1CH */
		    W_Buffer[1] |= (HPos >> 6) & 0x04;
		    W_Buffer[1] |= (VPos >> 7) & 0x02;
		    GPIOI2C_Write(pBIOSInfo, W_Buffer[0], W_Buffer[1]);

		    pTVInfo->CurrentPositionH = ViewPosValue.dwX;
		    pTVInfo->CurrentPositionV = ViewPosValue.dwY;
		    break;
		case VIA_VT1625:
		    AdjustTVPos_VT1625(pBIOSInfo, pTVSettingInfo,
				       ViewPosValue.dwX, ViewPosValue.dwY);
		    break;
		case VIA_INTEGRATED_TV:
		    AdjustTVPos_IntegratedTV(pBIOSInfo, pTVSettingInfo,
					     ViewPosValue.dwX, ViewPosValue.dwY);
		    break;
		default:
		    pTVInfo->CurrentPositionH = 0;
		    pTVInfo->CurrentPositionV = 0;
		    break;
	    }
	    xf86DestroyI2CDevRec(dev, TRUE);
            VIASaveUserSetting(pBIOSInfo);
	}
        else {
	    xf86DestroyI2CDevRec(dev, TRUE);
	    result = FALSE;
	}
    }

    return (result);
}

static void
VIA_UT_XV_FUNC_TV_GetSupportTuningState(VIABIOSInfoPtr pBIOSInfo,
					TVSETTINGINFOPTR pTVSettingInfo,
					CARD32* pdwSupportState)
{
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIA_UT_XV_FUNC_TV_GetSupportTuningState\n"));

    switch (pTVSettingInfo->TVEncoder)
    {
        case VIA_VT1621:
            *pdwSupportState = UT_TV_TUNING_FFILTER |
                               UT_TV_TUNING_BRIGHTNESS |
                               UT_TV_TUNING_CONTRAST |
                               UT_TV_TUNING_SATURATION |
                               UT_TV_TUNING_TINT;
            break;
        case VIA_VT1622:
        case VIA_VT1622A:
        case VIA_VT1623:
        case VIA_VT1625:
            *pdwSupportState = UT_TV_TUNING_FFILTER |
                               UT_TV_TUNING_ADAPTIVE_FFILTER |
                               UT_TV_TUNING_BRIGHTNESS |
                               UT_TV_TUNING_CONTRAST;
            /* RGB/YCbCr don't support SATURATION & TINT */
            if (pTVSettingInfo->TVOutput == TV_OUTPUT_COMPOSITE ||
                pTVSettingInfo->TVOutput == TV_OUTPUT_SVIDEO    ||
                pTVSettingInfo->TVOutput == TV_OUTPUT_COMPOSITE_SVIDEO)
            {
                *pdwSupportState |= UT_TV_TUNING_SATURATION | UT_TV_TUNING_TINT;
            }
            break;
        case VIA_INTEGRATED_TV:
            *pdwSupportState = UT_TV_TUNING_FFILTER |
                               UT_TV_TUNING_ADAPTIVE_FFILTER |
                               UT_TV_TUNING_BRIGHTNESS;

            /* RGB/YCbCr don't support SATURATION & TINT */
            if (pTVSettingInfo->TVOutput == TV_OUTPUT_COMPOSITE ||
                pTVSettingInfo->TVOutput == TV_OUTPUT_SVIDEO    ||
                pTVSettingInfo->TVOutput == TV_OUTPUT_COMPOSITE_SVIDEO)
            {
                *pdwSupportState |= UT_TV_TUNING_TINT;
            }

            break;
        default:
            *pdwSupportState = 0;
            break;
    }
}


static Bool
VIA_UT_XV_FUNC_TV_GetTuningItemMaxValue(VIABIOSInfoPtr pBIOSInfo,
					TVSETTINGINFOPTR pTVSettingInfo,
					CARD32 dwTuningItem,
					CARD32* pdwMaxValue)
{
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIA_UT_XV_FUNC_TV_GetTuningItemMaxValue\n"));

    TVInfoPtr       pTVInfo = &(pTVSettingInfo->TVInfo);
    int ret;

    switch (dwTuningItem)
    {
        case UT_TV_TUNING_FFILTER :
            ret = TRUE;
            *pdwMaxValue = pTVInfo->FFilterLevel;
            break;
        case UT_TV_TUNING_ADAPTIVE_FFILTER :
            ret = TRUE;
            *pdwMaxValue = pTVInfo->AFFilterLevel;
            break;
        case UT_TV_TUNING_BRIGHTNESS :
            ret = TRUE;
            *pdwMaxValue = pTVInfo->BrightnessLevel;
            break;
        case UT_TV_TUNING_CONTRAST :
            ret = TRUE;
            *pdwMaxValue = pTVInfo->ContrastLevel;
            break;
        case UT_TV_TUNING_SATURATION :
            ret = TRUE;
            *pdwMaxValue = pTVInfo->SaturationLevel;
            break;
        case UT_TV_TUNING_TINT :
            ret = TRUE;
            *pdwMaxValue = pTVInfo->TINTLevel;
            break;
        default :
            ret = FALSE;
            *pdwMaxValue = 0;
            break;
    }

    return ret;
}


static Bool
VIA_UT_XV_FUNC_TV_GetTuningItemValue(VIABIOSInfoPtr pBIOSInfo,
				     TVSETTINGINFOPTR pTVSettingInfo,
				     CARD32 dwTuningItem,
				     CARD32* pdwValue)
{
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIA_UT_XV_FUNC_TV_GetTuningItemValue\n"));

    TVInfoPtr       pTVInfo = &(pTVSettingInfo->TVInfo);
    int ret;

    switch (dwTuningItem)
    {
        case UT_TV_TUNING_FFILTER :
            *pdwValue = pTVInfo->CurrentFFilter;
            ret = TRUE;
            break;
        case UT_TV_TUNING_ADAPTIVE_FFILTER :
            *pdwValue = pTVInfo->CurrentAFFilter;
            ret = TRUE;
            break;
        case UT_TV_TUNING_BRIGHTNESS :
            *pdwValue = pTVInfo->CurrentBrightness;
            ret = TRUE;
            break;
        case UT_TV_TUNING_CONTRAST :
            *pdwValue = pTVInfo->CurrentContrast;
            ret = TRUE;
            break;
        case UT_TV_TUNING_SATURATION :
            *pdwValue = pTVInfo->CurrentSaturation;
            ret = TRUE;
            break;
        case UT_TV_TUNING_TINT :
            *pdwValue = pTVInfo->CurrentTINT;
            ret = TRUE;
            break;
        default :
            *pdwValue = 0;
            ret = FALSE;
            break;
    }
    return ret;
}


static Bool
VIA_UT_XV_FUNC_TV_SetTuningItemValue(VIABIOSInfoPtr pBIOSInfo,
				     TVSETTINGINFOPTR pTVSettingInfo,
				     UTSETTVTUNINGINFO TuningInfo)
{
    unsigned char   W_Buffer[4], R_Buffer[4];
    I2CDevPtr       dev=NULL;
    long            value;
    TVInfoPtr       pTVInfo = &(pTVSettingInfo->TVInfo);
    int ret;

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO,
		     "VIA_UT_XV_FUNC_TV_SetTuningItemValue\n"));

    switch (TuningInfo.dwItemID)
    {
        case UT_TV_TUNING_FFILTER :

            dev = xf86CreateI2CDevRec();
            dev->DevName = "TV";
            dev->SlaveAddr = pBIOSInfo->TVI2CAdd;
            dev->pI2CBus = pBIOSInfo->I2C_Port2;
            if (xf86I2CDevInit(dev))
            {
                switch (pTVSettingInfo->TVEncoder)
                {
                    case VIA_VT1621:
                        pTVInfo->CurrentFFilter = 0;
                        break;
                    case VIA_VT1622:
                    case VIA_VT1622A:
                        /* Set default FFilter */
                        if (TuningInfo.dwValue == 0xFFFF)
                        {
                            TuningInfo.dwValue = pTVInfo->DefaultFFilter;
                            if(pTVInfo->DefaultFFilter == 0)
                            {
                                pTVInfo->AdaptiveFFilterOn = TRUE;
                            }
                        }

                        W_Buffer[0] = 0x03;
                        xf86I2CWriteRead(dev, W_Buffer,1, R_Buffer,1);
                        W_Buffer[1] = R_Buffer[0] & ~0x03;
                        W_Buffer[1] |= TuningInfo.dwValue;
                        pTVInfo->CurrentFFilter = TuningInfo.dwValue;
                        xf86I2CWriteRead(dev, W_Buffer,2, NULL,0);
                        break;
                    case VIA_VT1623:
                        /* Set default FFilter */
                        if (TuningInfo.dwValue == 0xFFFF)
                        {
                            TuningInfo.dwValue = pTVInfo->DefaultFFilter;
                            if(pTVInfo->DefaultFFilter == 0)
                            {
                                pTVInfo->AdaptiveFFilterOn = TRUE;
                            }
                        }

                        GPIOI2C_Initial(pBIOSInfo, 0x40);
                        W_Buffer[0] = 0x03;
                        GPIOI2C_Read(pBIOSInfo, W_Buffer[0], R_Buffer, 1);
                        W_Buffer[1] = R_Buffer[0] & ~0x03;
                        W_Buffer[1] |= TuningInfo.dwValue;
                        pTVInfo->CurrentFFilter = TuningInfo.dwValue;
                        GPIOI2C_Write(pBIOSInfo, W_Buffer[0], W_Buffer[1]);
                        break;

                    case VIA_VT1625:
                        AdjustTVFFilter_VT1625(pBIOSInfo, pTVSettingInfo, VT1625_FILTER_TYPE_NORMAL, (CARD32)TuningInfo.dwValue);
                        break;

                    case VIA_INTEGRATED_TV:
                        AdjustTVFFilter_IntegratedTV(pBIOSInfo, pTVSettingInfo, ITV_FILTER_TYPE_NORMAL, (CARD32)TuningInfo.dwValue);
                        break;
                    default:
                        pTVInfo->CurrentFFilter = 0;
                        break;

                }

                VIASaveUserSetting(pBIOSInfo);
                xf86DestroyI2CDevRec(dev,TRUE);
                ret = TRUE;
            }
            else
            {
                xf86DestroyI2CDevRec(dev,TRUE);
                ret = FALSE;
            }
            break;
        case UT_TV_TUNING_ADAPTIVE_FFILTER :
            dev = xf86CreateI2CDevRec();
            dev->DevName = "TV";
            dev->SlaveAddr = pBIOSInfo->TVI2CAdd;
            dev->pI2CBus = pBIOSInfo->I2C_Port2;

            if (xf86I2CDevInit(dev))
            {
                switch (pTVSettingInfo->TVEncoder)
                {
                    case VIA_VT1621:
                        pTVInfo->CurrentAFFilter = 0;
                        break;
                    case VIA_VT1622:
                    case VIA_VT1622A:
                        /* Set default AFFilter */
                        if (TuningInfo.dwValue == 0xFFFF)
                        {
                            TuningInfo.dwValue = pTVInfo->DefaultAFFilter;
                        }

                        W_Buffer[0] = 0x61;
                        W_Buffer[1] = (unsigned char)TuningInfo.dwValue;
                        pTVInfo->CurrentAFFilter = TuningInfo.dwValue;
                        xf86I2CWriteRead(dev, W_Buffer,2, NULL,0);
                        break;
                    case VIA_VT1623:
                        /* Set default AFFilter */
                        if (TuningInfo.dwValue == 0xFFFF)
                        {
                            TuningInfo.dwValue = pTVInfo->DefaultAFFilter;
                        }

                        GPIOI2C_Initial(pBIOSInfo, 0x40);
                        W_Buffer[0] = 0x61;
                        W_Buffer[1] = (unsigned char)TuningInfo.dwValue;
                        pTVInfo->CurrentAFFilter = TuningInfo.dwValue;
                        GPIOI2C_Write(pBIOSInfo, W_Buffer[0], W_Buffer[1]);
                        break;

                    case VIA_VT1625:
                        AdjustTVFFilter_VT1625(pBIOSInfo, pTVSettingInfo, VT1625_FILTER_TYPE_ADAPTIVE, (CARD32)TuningInfo.dwValue);
                        break;

                    case VIA_INTEGRATED_TV:
                        AdjustTVFFilter_IntegratedTV(pBIOSInfo, pTVSettingInfo, ITV_FILTER_TYPE_ADAPTIVE, (CARD32)TuningInfo.dwValue);
                        break;
                    default:
                        pTVInfo->CurrentAFFilter = 0;
                        break;
                }
                VIASaveUserSetting(pBIOSInfo);
                xf86DestroyI2CDevRec(dev,TRUE);
                ret = TRUE;
            }
            else
            {
                xf86DestroyI2CDevRec(dev,TRUE);
                ret = FALSE;
            }
            break;
        case UT_TV_TUNING_BRIGHTNESS :
            /* Set default Brightness */
            if (TuningInfo.dwValue == 0xFFFF)
            {
                TuningInfo.dwValue = pTVInfo->DefaultBrightness;
            }
            pTVInfo->CurrentBrightness = TuningInfo.dwValue;

            dev = xf86CreateI2CDevRec();
            dev->DevName = "TV";
            dev->SlaveAddr = pBIOSInfo->TVI2CAdd;
            dev->pI2CBus = pBIOSInfo->I2C_Port2;

            if (xf86I2CDevInit(dev))
            {
                switch (pTVSettingInfo->TVEncoder)
                {
                    case VIA_VT1621:
                        pTVInfo->CurrentBrightness = 0;
                        break;
                    case VIA_VT1622:
                    case VIA_VT1622A:
                        W_Buffer[0] = 0x0B;
                        W_Buffer[1] = (unsigned char)TuningInfo.dwValue;
                        xf86I2CWriteRead(dev, W_Buffer,2, NULL,0);
                        break;
                    case VIA_VT1623:
                        GPIOI2C_Initial(pBIOSInfo, 0x40);
                        W_Buffer[0] = 0x0B;
                        W_Buffer[1] = (unsigned char)TuningInfo.dwValue;
                        GPIOI2C_Write(pBIOSInfo, W_Buffer[0], W_Buffer[1]);
                        break;

                    case VIA_VT1625:
                        AdjustTVBrightness_VT1625(pBIOSInfo, (CARD32)TuningInfo.dwValue);
                        break;

                    case VIA_INTEGRATED_TV:
                        AdjustTVBrightness_IntegratedTV(pBIOSInfo, (CARD32)TuningInfo.dwValue);
                        break;
                    default:
                        pTVInfo->CurrentBrightness = 0;
                        break;
                }
                VIASaveUserSetting(pBIOSInfo);
                xf86DestroyI2CDevRec(dev,TRUE);
                ret = TRUE;
            }
            else
            {
                xf86DestroyI2CDevRec(dev,TRUE);
                ret = FALSE;
            }
            break;
        case UT_TV_TUNING_CONTRAST :
            /* Set default Contrast */
            if (TuningInfo.dwValue == 0xFFFF)
            {
                TuningInfo.dwValue = pTVInfo->DefaultContrast;
            }

            dev = xf86CreateI2CDevRec();
            dev->DevName = "TV";
            dev->SlaveAddr = pBIOSInfo->TVI2CAdd;
            dev->pI2CBus = pBIOSInfo->I2C_Port2;

            if (xf86I2CDevInit(dev))
            {
                switch (pTVSettingInfo->TVEncoder)
                {
                    case VIA_VT1621:
                        pTVInfo->CurrentContrast = 0;
                        break;
                    case VIA_VT1622:
                    case VIA_VT1622A:
                        value = (TuningInfo.dwValue - pTVInfo->CurrentContrast);
                        if (pTVSettingInfo->TVOutput == TV_OUTPUT_RGB)
                        {
                            W_Buffer[0] = 0x65;
                            xf86I2CWriteRead(dev, W_Buffer, 1, R_Buffer, 3);
                            W_Buffer[1] = R_Buffer[0] + value;
                            W_Buffer[2] = R_Buffer[1] + value;
                            W_Buffer[3] = R_Buffer[2] + value;
                            xf86I2CWriteRead(dev, W_Buffer, 4, NULL, 0);
                        }
                        else if (pTVSettingInfo->TVOutput == TV_OUTPUT_YPBPR)
                        {
                            W_Buffer[0] = 0x65;
                            xf86I2CWriteRead(dev, W_Buffer, 1, R_Buffer, 1);
                            W_Buffer[1] = R_Buffer[0] + value;
                            xf86I2CWriteRead(dev, W_Buffer, 2, NULL, 0);
                        }
                        else
                        {
                            W_Buffer[0] = 0x0C;
                            xf86I2CWriteRead(dev, W_Buffer, 1, R_Buffer, 1);
                            W_Buffer[1] = R_Buffer[0] + value;
                            xf86I2CWriteRead(dev, W_Buffer, 2, NULL, 0);
                        }

                        pTVInfo->CurrentContrast = TuningInfo.dwValue;
                        break;
                    case VIA_VT1623:
                        value = (TuningInfo.dwValue - pTVInfo->CurrentContrast);
                        GPIOI2C_Initial(pBIOSInfo, 0x40);
                        if (pTVSettingInfo->TVOutput == TV_OUTPUT_RGB)
                        {
                            W_Buffer[0] = 0x65;
                            GPIOI2C_Read(pBIOSInfo, W_Buffer[0], R_Buffer, 3);
                            W_Buffer[1] = R_Buffer[0] + value;
                            W_Buffer[2] = R_Buffer[1] + value;
                            W_Buffer[3] = R_Buffer[2] + value;
                            GPIOI2C_Write(pBIOSInfo, W_Buffer[0], W_Buffer[1]);
                            GPIOI2C_Write(pBIOSInfo, W_Buffer[0] + 1 , W_Buffer[2]);
                            GPIOI2C_Write(pBIOSInfo, W_Buffer[0] + 2 , W_Buffer[3]);
                        }
                        else if (pTVSettingInfo->TVOutput == TV_OUTPUT_YPBPR)
                        {
                            W_Buffer[0] = 0x65;
                            GPIOI2C_Read(pBIOSInfo, W_Buffer[0], R_Buffer, 1);
                            W_Buffer[1] = R_Buffer[0] + value;
                            GPIOI2C_Write(pBIOSInfo, W_Buffer[0], W_Buffer[1]);
                        }
                        else
                        {
                           /*Jack liu, [2005.07.26], fix contrast not action bug. */
                           /* W_Buffer[0] = 0x0C;
                            GPIOI2C_Read(pBIOSInfo, W_Buffer[0], R_Buffer, 1);
                            W_Buffer[1] = R_Buffer[0] + value;
                            GPIOI2C_Write(pBIOSInfo, W_Buffer[0], W_Buffer[1]);*/


                            W_Buffer[0] = 0x0A;
                            GPIOI2C_Read(pBIOSInfo, W_Buffer[0], R_Buffer, 4);
                            W_Buffer[1] = R_Buffer[0] + value;
                            GPIOI2C_Write(pBIOSInfo, W_Buffer[0], W_Buffer[1]);
                            W_Buffer[0] = 0x0C;

                            W_Buffer[1] = R_Buffer[2] + value;
                            W_Buffer[2] = R_Buffer[3] + value;
                            GPIOI2C_Write(pBIOSInfo, W_Buffer[0], W_Buffer[1]);
                            GPIOI2C_Write(pBIOSInfo, W_Buffer[0]+1, W_Buffer[2]);

                        }

                        pTVInfo->CurrentContrast = TuningInfo.dwValue;
                        break;

                    case VIA_VT1625:
                        AdjustTVContrast_VT1625(pBIOSInfo, pTVSettingInfo, (CARD32)TuningInfo.dwValue);
                        break;
                    default:
                        pTVInfo->CurrentContrast = 0;
                        break;
                }

                VIASaveUserSetting(pBIOSInfo);
                xf86DestroyI2CDevRec(dev,TRUE);
                ret = TRUE;
            }
            else
            {
                xf86DestroyI2CDevRec(dev,TRUE);
                ret = FALSE;
            }
            break;
        case UT_TV_TUNING_SATURATION :
            /* Set default Saturation */
            if (TuningInfo.dwValue == 0xFFFF)
            {
                TuningInfo.dwValue = pTVInfo->DefaultSaturation;
            }
            pTVInfo->CurrentSaturation = TuningInfo.dwValue;
            dev = xf86CreateI2CDevRec();
            dev->DevName = "TV";
            dev->SlaveAddr = pBIOSInfo->TVI2CAdd;
            dev->pI2CBus = pBIOSInfo->I2C_Port2;

            if (xf86I2CDevInit(dev))
            {
                switch (pTVSettingInfo->TVEncoder)
                {
                    case VIA_VT1621:
                        pTVInfo->CurrentSaturation = 0;
                        break;
                    case VIA_VT1622:
                    case VIA_VT1622A:
                        if (pTVSettingInfo->TVOutput == TV_OUTPUT_RGB ||
                            pTVSettingInfo->TVOutput == TV_OUTPUT_YPBPR)
                        {
                            break;
                        }
                        W_Buffer[0] = 0x0D;
                        W_Buffer[1] = (unsigned char)((TuningInfo.dwValue >> 8) & 0xFF);
                        xf86I2CWriteRead(dev, W_Buffer,2, NULL,0);
                        W_Buffer[0] = 0x0A;
                        W_Buffer[1] = (unsigned char)(TuningInfo.dwValue & 0xFF);
                        pTVInfo->CurrentSaturation = TuningInfo.dwValue;
                        xf86I2CWriteRead(dev, W_Buffer,2, NULL,0);
                        break;
                    case VIA_VT1623:
                        if (pTVSettingInfo->TVOutput == TV_OUTPUT_RGB ||
                            pTVSettingInfo->TVOutput == TV_OUTPUT_YPBPR)
                        {
                            break;
                        }
                        GPIOI2C_Initial(pBIOSInfo, 0x40);
                        W_Buffer[0] = 0x0D;
                        W_Buffer[1] = (unsigned char)((TuningInfo.dwValue >> 8) & 0xFF);
                        GPIOI2C_Write(pBIOSInfo, W_Buffer[0], W_Buffer[1]);
                        W_Buffer[0] = 0x0A;
                        W_Buffer[1] = (unsigned char)(TuningInfo.dwValue & 0xFF);
                        pTVInfo->CurrentSaturation = TuningInfo.dwValue;
                        GPIOI2C_Write(pBIOSInfo, W_Buffer[0], W_Buffer[1]);
                        break;

                    case VIA_VT1625:
                        if (pTVSettingInfo->TVOutput == TV_OUTPUT_RGB ||
                            pTVSettingInfo->TVOutput == TV_OUTPUT_YPBPR ||
                            pTVSettingInfo->TVOutput == TV_OUTPUT_YPBPR_COMPOSITE_SVIDEO ||
                            pTVSettingInfo->TVOutput == TV_OUTPUT_RGB_COMPOSITE_SVIDEO )
                        {
                            break;
                        }

                        AdjustTVSaturation_VT1625(pBIOSInfo, pTVSettingInfo, (CARD32)TuningInfo.dwValue);
                        break;
                    default:
                        pTVInfo->CurrentSaturation = 0;
                        break;
                }

                VIASaveUserSetting(pBIOSInfo);
                xf86DestroyI2CDevRec(dev,TRUE);
                ret = TRUE;
            }
            else
            {
                xf86DestroyI2CDevRec(dev,TRUE);
                ret = FALSE;
            }
            break;
        case UT_TV_TUNING_TINT :
            /* Set default TINT */
            if (TuningInfo.dwValue == 0xFFFF)
            {
                TuningInfo.dwValue = pTVInfo->DefaultTINT;
            }
            pTVInfo->CurrentTINT = TuningInfo.dwValue;
            dev = xf86CreateI2CDevRec();
            dev->DevName = "TV";
            dev->SlaveAddr = pBIOSInfo->TVI2CAdd;
            dev->pI2CBus = pBIOSInfo->I2C_Port2;

            if (xf86I2CDevInit(dev))
            {
                switch (pTVSettingInfo->TVEncoder)
                {
                    case VIA_VT1621:
                        W_Buffer[0] = 0x11;
                        xf86I2CWriteRead(dev, W_Buffer,1, R_Buffer,1);
                        W_Buffer[2] = R_Buffer[0] & ~0xE0;
                        W_Buffer[2] |= (unsigned char)((TuningInfo.dwValue >> 3) & 0xFF);
                        W_Buffer[0] = 0x10;
                        W_Buffer[1] = (unsigned char)(TuningInfo.dwValue & 0xFF);
                        pTVInfo->CurrentTINT = TuningInfo.dwValue;
                        xf86I2CWriteRead(dev, W_Buffer,3, NULL,0);
                        break;
                    case VIA_VT1622:
                    case VIA_VT1622A:
                        if (pTVSettingInfo->TVOutput == TV_OUTPUT_RGB ||
                            pTVSettingInfo->TVOutput == TV_OUTPUT_YPBPR)
                        {
                            break;
                        }
                        W_Buffer[0] = 0x11;
                        xf86I2CWriteRead(dev, W_Buffer,1, R_Buffer,1);
                        W_Buffer[2] = R_Buffer[0] & ~0x07;
                        W_Buffer[2] |= (unsigned char)(TuningInfo.dwValue >> 8) & 0xFF;
                        W_Buffer[0] = 0x10;
                        W_Buffer[1] = (unsigned char)(TuningInfo.dwValue & 0xFF);
                        pTVInfo->CurrentTINT = TuningInfo.dwValue;
                        xf86I2CWriteRead(dev, W_Buffer,3, NULL,0);
                        break;
                    case VIA_VT1623:
                        if (pTVSettingInfo->TVOutput == TV_OUTPUT_RGB ||
                            pTVSettingInfo->TVOutput == TV_OUTPUT_YPBPR)
                        {
                            break;
                        }
                        GPIOI2C_Initial(pBIOSInfo, 0x40);
                        W_Buffer[0] = 0x11;
                        GPIOI2C_Read(pBIOSInfo, W_Buffer[0], R_Buffer, 1);
                        W_Buffer[2] = R_Buffer[0] & ~0x07;
                        W_Buffer[2] |= (unsigned char)(TuningInfo.dwValue >> 8) & 0xFF;
                        W_Buffer[0] = 0x10;
                        W_Buffer[1] = (unsigned char)(TuningInfo.dwValue & 0xFF);
                        GPIOI2C_Write(pBIOSInfo, W_Buffer[0], W_Buffer[1]);
                        W_Buffer[0] = 0x11;
                        GPIOI2C_Write(pBIOSInfo, W_Buffer[0], W_Buffer[2]);
                        pTVInfo->CurrentTINT = TuningInfo.dwValue;
                        break;

                    case VIA_VT1625:
                        if (pTVSettingInfo->TVOutput == TV_OUTPUT_RGB ||
                            pTVSettingInfo->TVOutput == TV_OUTPUT_YPBPR ||
                            pTVSettingInfo->TVOutput == TV_OUTPUT_YPBPR_COMPOSITE_SVIDEO ||
                            pTVSettingInfo->TVOutput == TV_OUTPUT_RGB_COMPOSITE_SVIDEO)
                        {
                            break;
                        }

                        AdjustTVHue_VT1625(pBIOSInfo, pTVSettingInfo, (CARD32)TuningInfo.dwValue);
                        break;

                    case VIA_INTEGRATED_TV:
                        if (pTVSettingInfo->TVOutput == TV_OUTPUT_RGB ||
                            pTVSettingInfo->TVOutput == TV_OUTPUT_YPBPR)
                        {
                            break;
                        }

                        AdjustTVHue_IntegratedTV(pBIOSInfo, pTVSettingInfo, (CARD32)TuningInfo.dwValue);
                        break;
                    default:
                        pTVInfo->CurrentTINT = 0;
                        break;
                }

                VIASaveUserSetting(pBIOSInfo);
                xf86DestroyI2CDevRec(dev,TRUE);
                ret = TRUE;
            }
            else
            {
                xf86DestroyI2CDevRec(dev,TRUE);
                ret = FALSE;
            }
            break;
        default :
            ret = FALSE;
            break;
    }

    return ret;
}

static void
VIA_UT_XV_FUNC_TV_GetSupportSettingState(VIABIOSInfoPtr pBIOSInfo,
					 TVSETTINGINFOPTR pTVSettingInfo,
					 CARD32* pdwSupportState)
{
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIA_UT_XV_FUNC_TV_GetSupportSettingState\n"));

    *pdwSupportState |= UT_TV_SETTING_FFILTER;
    switch (pTVSettingInfo->TVEncoder)
    {
        case VIA_VT1622:
        case VIA_VT1622A:
        case VIA_VT1623:
        case VIA_VT1625:
            *pdwSupportState |= UT_TV_SETTING_ADAPTIVE_FFILTER;
            if (pTVSettingInfo->TVType == TVTYPE_NTSC &&
                pTVSettingInfo->TVOutput == TV_OUTPUT_COMPOSITE)
            {
                *pdwSupportState |= UT_TV_SETTING_DOT_CRAWL;
            }
            break;
        case VIA_INTEGRATED_TV:
            *pdwSupportState |= UT_TV_SETTING_ADAPTIVE_FFILTER;
            break;
        default :
            break;
    }

}

static Bool
VIA_UT_XV_FUNC_TV_GetSettingItemState(VIABIOSInfoPtr pBIOSInfo,
				      TVSETTINGINFOPTR pTVSettingInfo,
				      CARD32 dwItem,
				      CARD32* pdwState)
{
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIA_UT_XV_FUNC_TV_GetSettingItemState\n"));

    TVInfoPtr       pTVInfo = &(pTVSettingInfo->TVInfo);

    int ret;

    switch (dwItem)
    {
        case UT_TV_SETTING_FFILTER :
            if (!pTVInfo->AdaptiveFFilterOn)
                *pdwState = UT_STATE_ON;
            else
                *pdwState = UT_STATE_OFF;
            ret = TRUE;
            break;
        case UT_TV_SETTING_ADAPTIVE_FFILTER :
            if (pTVInfo->AdaptiveFFilterOn)
                *pdwState = UT_STATE_ON;
            else
                *pdwState = UT_STATE_OFF;
            ret = TRUE;
            break;
        case UT_TV_SETTING_DOT_CRAWL :
            if(pTVSettingInfo->TVDotCrawl)
                *pdwState = UT_STATE_ON;
            else
                *pdwState = UT_STATE_OFF;
            ret = TRUE;
            break;
        case UT_TV_SETTING_LOCK_ASPECT_RATIO :
            ret = FALSE;
            *pdwState = UT_STATE_OFF;
            break;
        default :
            ret = FALSE;
            *pdwState = UT_STATE_OFF;
            break;
    }

    return ret;
}


static Bool
VIA_UT_XV_FUNC_TV_SetSettingItemState(VIABIOSInfoPtr pBIOSInfo,
				      TVSETTINGINFOPTR pTVSettingInfo,
				      UTSETTVITEMSTATE SettingInfo)
{
    unsigned char W_Buffer[4], R_Buffer[4];
    I2CDevPtr dev = NULL;
    TVInfoPtr pTVInfo = &(pTVSettingInfo->TVInfo);
    Bool result = TRUE;

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO,
		     "VIA_UT_XV_FUNC_TV_SetSettingItemState\n"));

    switch (SettingInfo.dwItemID) {
        case UT_TV_SETTING_FFILTER:
            if (SettingInfo.dwState == UT_STATE_OFF) {
		dev = xf86CreateI2CDevRec();
		dev->DevName = "TV";
		dev->SlaveAddr = pBIOSInfo->TVI2CAdd;
                dev->pI2CBus = pBIOSInfo->I2C_Port2;

		if (xf86I2CDevInit(dev)) {
		    switch (pTVSettingInfo->TVEncoder) {
			case VIA_VT1621:
			case VIA_VT1622:
			case VIA_VT1622A:
			    W_Buffer[0] = 0x03;
			    xf86I2CWriteRead(dev, W_Buffer,1, R_Buffer,1);
			    W_Buffer[1] = R_Buffer[0] & ~0x03;
			    xf86I2CWriteRead(dev, W_Buffer,2, NULL,0);
			    break;
			case VIA_VT1623:
			    GPIOI2C_Initial(pBIOSInfo, 0x40);
			    W_Buffer[0] = 0x03;
			    GPIOI2C_Read(pBIOSInfo, W_Buffer[0], R_Buffer, 1);
			    W_Buffer[1] = R_Buffer[0] & ~0x03;
			    GPIOI2C_Write(pBIOSInfo, W_Buffer[0], W_Buffer[1]);
			    break;
			case VIA_VT1625:
			    /* FFliter off, AFFliter on */
			    EnableTVFFilter_VT1625(pBIOSInfo, pTVSettingInfo,
						   VT1625_FILTER_TYPE_ADAPTIVE);
                            break;
			case VIA_INTEGRATED_TV:
			    /* FFliter off, AFFliter on */
			    EnableTVFFilter_IntegratedTV(pBIOSInfo,
							 ITV_FILTER_TYPE_ADAPTIVE);
			    break;
		    }
		    xf86DestroyI2CDevRec(dev, TRUE);
		    pTVInfo->AdaptiveFFilterOn = TRUE;
		    VIASaveUserSetting(pBIOSInfo);
                }
                else {
		    xf86DestroyI2CDevRec(dev, TRUE);
                    result = FALSE;
                }
            }
	    else if (SettingInfo.dwState == UT_STATE_ON) {
		dev = xf86CreateI2CDevRec();
		dev->DevName = "TV";
		dev->SlaveAddr = pBIOSInfo->TVI2CAdd;
		dev->pI2CBus = pBIOSInfo->I2C_Port2;

		if (xf86I2CDevInit(dev)) {
                    switch (pTVSettingInfo->TVEncoder) {
			case VIA_VT1621:
			case VIA_VT1622:
			case VIA_VT1622A:
			    W_Buffer[0] = 0x03;
			    xf86I2CWriteRead(dev, W_Buffer,1, R_Buffer,1);
			    W_Buffer[1] = R_Buffer[0] & ~0x03;
			    W_Buffer[1] |= (unsigned char)(pTVInfo->CurrentFFilter - 1);
                            xf86I2CWriteRead(dev, W_Buffer,2, NULL,0);
			    break;
			case VIA_VT1623:
			    GPIOI2C_Initial(pBIOSInfo, 0x40);
			    W_Buffer[0] = 0x03;
			    GPIOI2C_Read(pBIOSInfo, W_Buffer[0], R_Buffer, 1);
			    W_Buffer[1] = R_Buffer[0] & ~0x03;
			    W_Buffer[1] |= (unsigned char)(pTVInfo->CurrentFFilter - 1);
			    GPIOI2C_Write(pBIOSInfo, W_Buffer[0], W_Buffer[1]);
			    break;
			case VIA_VT1625:
			    /* FFliter on, AFFliter off */
			    EnableTVFFilter_VT1625(pBIOSInfo, pTVSettingInfo, VT1625_FILTER_TYPE_NORMAL);
                            break;
			case VIA_INTEGRATED_TV:
			    /* FFliter on, AFFliter off */
			    EnableTVFFilter_IntegratedTV(pBIOSInfo, ITV_FILTER_TYPE_NORMAL);
			    break;
		    }
		    xf86DestroyI2CDevRec(dev,TRUE);
		    pTVInfo->AdaptiveFFilterOn = FALSE;
		}
		else {
                    xf86DestroyI2CDevRec(dev,TRUE);
                    result = FALSE;
                }
            }
            else {
                result = FALSE;
            }
	    break;
	case UT_TV_SETTING_ADAPTIVE_FFILTER :
	    if (SettingInfo.dwState == UT_STATE_OFF) {
		dev = xf86CreateI2CDevRec();
		dev->DevName = "TV";
		dev->SlaveAddr = pBIOSInfo->TVI2CAdd;
                dev->pI2CBus = pBIOSInfo->I2C_Port2;

		if (xf86I2CDevInit(dev)) {
                    switch (pTVSettingInfo->TVEncoder) {
			case VIA_VT1621:
			    break;
			case VIA_VT1622:
			case VIA_VT1622A:
			    W_Buffer[0] = 0x61;
			    W_Buffer[1] = 0x00;
			    xf86I2CWriteRead(dev, W_Buffer,2, NULL,0);
			    break;
			case VIA_VT1623:
			    GPIOI2C_Initial(pBIOSInfo, 0x40);
			    W_Buffer[0] = 0x61;
			    W_Buffer[1] = 0x00;
			    GPIOI2C_Write(pBIOSInfo, W_Buffer[0], W_Buffer[1]);
			    break;
			case VIA_VT1625:
			    /* FFliter on, AFFliter off */
			    EnableTVFFilter_VT1625(pBIOSInfo, pTVSettingInfo, VT1625_FILTER_TYPE_NORMAL);
			    break;
			case VIA_INTEGRATED_TV:
			    /* FFliter on, AFFliter off */
			    EnableTVFFilter_IntegratedTV(pBIOSInfo, ITV_FILTER_TYPE_NORMAL);
			    break;
			default:
			    pTVInfo->CurrentAFFilter = 0;
			    break;
		    }

		    pTVInfo->AdaptiveFFilterOn = FALSE;
		    xf86DestroyI2CDevRec(dev,TRUE);
		    VIASaveUserSetting(pBIOSInfo);
		}
		else {
                    xf86DestroyI2CDevRec(dev,TRUE);
                    result = FALSE;
		}
	    }
	    else if (SettingInfo.dwState == UT_STATE_ON) {
		dev = xf86CreateI2CDevRec();
		dev->DevName = "TV";
		dev->SlaveAddr = pBIOSInfo->TVI2CAdd;
		dev->pI2CBus = pBIOSInfo->I2C_Port2;

		if (xf86I2CDevInit(dev)) {
                    switch (pTVSettingInfo->TVEncoder) {
			case VIA_VT1621:
			    pTVInfo->AdaptiveFFilterOn = FALSE;
			    pTVInfo->CurrentAFFilter = 0;
			    break;
			case VIA_VT1622:
			case VIA_VT1622A:
			    W_Buffer[0] = 0x03;
			    xf86I2CWriteRead(dev, W_Buffer,1, R_Buffer,1);
			    W_Buffer[1] = R_Buffer[0] & ~0x03;
			    xf86I2CWriteRead(dev, W_Buffer,2, NULL,0);
			    W_Buffer[0] = 0x61;
			    W_Buffer[1] = (unsigned char)pTVInfo->CurrentAFFilter;
			    xf86I2CWriteRead(dev, W_Buffer,2, NULL,0);
			    pTVInfo->AdaptiveFFilterOn = TRUE;
			    break;
			case VIA_VT1623:
			    GPIOI2C_Initial(pBIOSInfo, 0x40);
			    W_Buffer[0] = 0x03;
			    GPIOI2C_Read(pBIOSInfo, W_Buffer[0], R_Buffer, 1);
			    W_Buffer[1] = R_Buffer[0] & ~0x03;
			    GPIOI2C_Write(pBIOSInfo, W_Buffer[0], W_Buffer[1]);
			    W_Buffer[0] = 0x61;
			    W_Buffer[1] = (unsigned char)pTVInfo->CurrentAFFilter;
			    GPIOI2C_Write(pBIOSInfo, W_Buffer[0], W_Buffer[1]);
			    pTVInfo->AdaptiveFFilterOn = TRUE;
			    break;
			case VIA_VT1625:
			    EnableTVFFilter_VT1625(pBIOSInfo, pTVSettingInfo, VT1625_FILTER_TYPE_ADAPTIVE);
			    pTVInfo->AdaptiveFFilterOn = TRUE;
			    break;
			case VIA_INTEGRATED_TV:
			    /* FFliter off, AFFliter on */
			    EnableTVFFilter_IntegratedTV(pBIOSInfo, ITV_FILTER_TYPE_ADAPTIVE);
			    pTVInfo->AdaptiveFFilterOn = TRUE;
			    break;
			default:
			    pTVInfo->AdaptiveFFilterOn = FALSE;
			    pTVInfo->CurrentAFFilter = 0;
			    break;
		    }
		    xf86DestroyI2CDevRec(dev, TRUE);
		    VIASaveUserSetting(pBIOSInfo);
		}
                else {
                    xf86DestroyI2CDevRec(dev, TRUE);
                    result = FALSE;
                }
            }
            else {
                result = FALSE;
	    }
	    break;
	case UT_TV_SETTING_DOT_CRAWL :
	    if (SettingInfo.dwState == UT_STATE_OFF ||
		SettingInfo.dwState == UT_STATE_DEFAULT) {
		pTVSettingInfo->TVDotCrawl = FALSE;
		switch (pTVSettingInfo->TVEncoder) {
		    case VIA_VT1621:
			VIAPreSetTV2Mode(pBIOSInfo);
			VIAPostSetTV2Mode(pBIOSInfo);
			break;
		    case VIA_VT1622:
			load_vt1622_regs(pBIOSInfo);
			break;
		    case VIA_VT1622A:
		    case VIA_VT1623:
			load_vt1623_regs(pBIOSInfo);
			break;
		    case VIA_VT1625:
			load_vt1625_regs(pBIOSInfo, pTVSettingInfo);
			break;
			break;
		}
		VIATVPatch(pBIOSInfo, pTVSettingInfo);
		VIAEnableTV(pBIOSInfo, pTVSettingInfo);
		VIASaveUserSetting(pBIOSInfo);
		VIARestoreUserTVSetting(pBIOSInfo, pTVSettingInfo);
	    }
	    else if (SettingInfo.dwState == UT_STATE_ON) {
		pTVSettingInfo->TVDotCrawl = TRUE;
		switch (pTVSettingInfo->TVEncoder) {
		    case VIA_VT1621:
			VIAPreSetTV2Mode(pBIOSInfo);
			VIAPostSetTV2Mode(pBIOSInfo);
			break;
		    case VIA_VT1622:
			load_vt1622_regs(pBIOSInfo);
			break;
		    case VIA_VT1622A:
		    case VIA_VT1623:
			load_vt1623_regs(pBIOSInfo);
			break;
		    case VIA_VT1625:
			load_vt1625_regs(pBIOSInfo, pTVSettingInfo);
			break;
		}
		VIATVPatch(pBIOSInfo, pTVSettingInfo);
		VIAEnableTV(pBIOSInfo, pTVSettingInfo);
		VIASaveUserSetting(pBIOSInfo);
		VIARestoreUserTVSetting(pBIOSInfo, pTVSettingInfo);
	    }
	    else {
		result = FALSE;
	    }
	    break;
	case UT_TV_SETTING_LOCK_ASPECT_RATIO:
	    result = FALSE;
	    break;
	default:
	    result = FALSE;
	    break;
    }

    return (result);
}


static void
VIA_UT_XV_FUNC_HDTV_GetSupportStandardState(VIABIOSInfoPtr pBIOSInfo,
					    TVSETTINGINFOPTR pTVSettingInfo,
					    CARD32* pdwSupportState)
{
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIA_UT_XV_FUNC_HDTV_GetSupportStandardState\n"));

    switch (pTVSettingInfo->TVEncoder)
    {
        case VIA_VT1622A:
        case VIA_VT1623:
            *pdwSupportState = UT_TV_STD_480P | UT_TV_STD_576P;
            break;
        case VIA_VT1625:
            if((pBIOSInfo->Chipset == VIA_CLE266) ||
               (pBIOSInfo->Chipset == VIA_KM400) ||
               (pBIOSInfo->Chipset == VIA_K8M800))
            {
                *pdwSupportState = UT_TV_STD_480P | UT_TV_STD_576P;
            }
            else
            {
                *pdwSupportState = UT_TV_STD_480P | UT_TV_STD_576P |
                                 UT_TV_STD_720P | UT_TV_STD_1080I;
            }
            break;

        case VIA_INTEGRATED_TV:
            *pdwSupportState = UT_TV_STD_480P | UT_TV_STD_576P |
                             UT_TV_STD_720P | UT_TV_STD_1080I;
            break;

        default:
            *pdwSupportState = 0;
            break;
    }

    if (pBIOSInfo->HDisplay == 720 && pBIOSInfo->VDisplay == 480)
    {
        *pdwSupportState &= ~UT_TV_STD_576P;
    }
    else if (pBIOSInfo->HDisplay == 720 && pBIOSInfo->VDisplay == 576)
    {
        *pdwSupportState &= ~UT_TV_STD_480P;
    }
    else if (pBIOSInfo->HDisplay == 1280 && pBIOSInfo->VDisplay == 720)
    {
        *pdwSupportState &= ~(UT_TV_STD_480P | UT_TV_STD_576P | UT_TV_STD_1080I);
    }
    else if (pBIOSInfo->HDisplay == 1920 && pBIOSInfo->VDisplay == 1080)
    {
        *pdwSupportState &= ~(UT_TV_STD_480P | UT_TV_STD_576P | UT_TV_STD_720P);
    }
}


static void
VIA_UT_XV_FUNC_HDTV_GetStandard(VIABIOSInfoPtr pBIOSInfo,
				TVSETTINGINFOPTR pTVSettingInfo,
				CARD32* pdwStandard)
{
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIA_UT_XV_FUNC_HDTV_GetStandard\n"));

    switch (pTVSettingInfo->TVType)
    {
        case TVTYPE_480P:
            *pdwStandard = UT_TV_STD_480P;
            break;
        case TVTYPE_576P:
            *pdwStandard = UT_TV_STD_576P;
            break;
        case TVTYPE_720P:
            *pdwStandard = UT_TV_STD_720P;
            break;
        case TVTYPE_1080I:
            *pdwStandard = UT_TV_STD_1080I;
            break;
        default:
            *pdwStandard = 0;
            break;
    }
}

static Bool
VIA_UT_XV_FUNC_HDTV_SetStandard(VIABIOSInfoPtr pBIOSInfo,
				TVSETTINGINFOPTR pTVSettingInfo,
				DisplayModePtr mode,
				CARD32 dwStandard)
{
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIA_UT_XV_FUNC_HDTV_SetStandard\n"));
    pTVSettingInfo->TVVScan = VIA_TVNORMAL;
    int ret;
    switch (dwStandard)
    {
        case UT_TV_STD_480P :
            pTVSettingInfo->TVType = TVTYPE_480P;
            ret = TRUE;
            break;
        case UT_TV_STD_576P :
            pTVSettingInfo->TVType = TVTYPE_576P;
            ret = TRUE;
            break;
        case UT_TV_STD_720P :
            pTVSettingInfo->TVType = TVTYPE_720P;
            ret = TRUE;
            break;
        case UT_TV_STD_1080I :
            pTVSettingInfo->TVType = TVTYPE_1080I;
            ret = TRUE;
            break;
        default :
            ret = FALSE;
            break;
    }
    if(ret)
    {
        if (pBIOSInfo->ActiveDevice & (VIA_DEVICE_TV | VIA_DEVICE_TV2))
        {
            pBIOSInfo->s3utility=TRUE;
            VIASwitchMode(pBIOSInfo->scrnIndex, mode, 0);
            VIASaveUserSetting(pBIOSInfo);
        }
    }

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIA_UT_XV_FUNC_HDTV_SetStandard: TVType = 0x%x\n", pBIOSInfo->TVSettingInfo.TVType));

    return ret;
}


static void
VIA_UT_XV_FUNC_HDTV_GetSupportSignalTypeState(VIABIOSInfoPtr pBIOSInfo,
					      TVSETTINGINFOPTR pTVSettingInfo,
					      CARD32* pdwSupportState)
{
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIA_UT_XV_FUNC_HDTV_GetSupportSignalTypeState\n"));

    VIAGetActiveDisplay(pBIOSInfo);
    switch (pTVSettingInfo->TVEncoder)
    {
        case VIA_VT1622A:
        case VIA_VT1623:
        case VIA_VT1625:
        case VIA_INTEGRATED_TV:
            *pdwSupportState = UT_TV_SGNL_COMPOSITE | UT_TV_SGNL_S_VIDEO |
                          UT_TV_SGNL_SCART | UT_TV_SGNL_COMP_OUTPUT;
            break;
        default:
            *pdwSupportState = 0;
            break;
    }
}


static void
VIA_UT_XV_FUNC_HDTV_GetSignalType(VIABIOSInfoPtr pBIOSInfo,
				  TVSETTINGINFOPTR pTVSettingInfo,
				  CARD32* pdwSignalType)
{
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIA_UT_XV_FUNC_HDTV_GetSignalType\n"));

    switch (pTVSettingInfo->TVOutput)
    {
        case TV_OUTPUT_COMPOSITE:
        case TV_OUTPUT_COMPOSITE_SVIDEO:
            *pdwSignalType = UT_TV_SGNL_COMPOSITE;
            break;
        case TV_OUTPUT_SVIDEO:
            *pdwSignalType = UT_TV_SGNL_S_VIDEO;
            break;
        case TV_OUTPUT_RGB:
            *pdwSignalType = UT_TV_SGNL_SCART;
            break;
        case TV_OUTPUT_YPBPR:
            *pdwSignalType = UT_TV_SGNL_COMP_OUTPUT;
            break;
        case TV_OUTPUT_YPBPR_COMPOSITE_SVIDEO:
            *pdwSignalType = UT_TV_SGNL_COMPONENT_COMPOSITE_SVIDEO;
            break;
        case TV_OUTPUT_RGB_COMPOSITE_SVIDEO:
            *pdwSignalType = UT_TV_SGNL_RGB_COMPOSITE_SVIDEO;
            break;
        default:
            *pdwSignalType = 0;
            break;
    }
}


static Bool
VIA_UT_XV_FUNC_HDTV_SetSignalType(VIABIOSInfoPtr pBIOSInfo,
				  TVSETTINGINFOPTR pTVSettingInfo,
				  DisplayModePtr mode,
				  CARD32 dwSignalType)
{
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIA_UT_XV_FUNC_HDTV_SetSignalType\n"));

    int ret;
/* here we setting the HDTV tvvscan to Normal, because we have just on level timing now*/
    pTVSettingInfo->TVVScan = VIA_TVNORMAL;
    switch (dwSignalType)
    {
        case UT_TV_SGNL_COMPOSITE :
            pTVSettingInfo->TVOutput = TV_OUTPUT_COMPOSITE;
            ret = TRUE;
            break;
        case UT_TV_SGNL_S_VIDEO :
            pTVSettingInfo->TVOutput = TV_OUTPUT_SVIDEO;
            ret = TRUE;
            break;
        case UT_TV_SGNL_SCART :
            pTVSettingInfo->TVOutput = TV_OUTPUT_RGB;
            ret = TRUE;
            break;
        case UT_TV_SGNL_COMP_OUTPUT :
            pTVSettingInfo->TVOutput = TV_OUTPUT_YPBPR;
            ret = TRUE;
            break;
        default :
            ret = FALSE;
            break;
    }
    if(ret)
    {
        if (pBIOSInfo->ActiveDevice & (VIA_DEVICE_TV | VIA_DEVICE_TV2))
        {
            pBIOSInfo->s3utility=TRUE;
            VIASwitchMode(pBIOSInfo->scrnIndex, mode, 0);
            VIASaveUserSetting(pBIOSInfo);
        }
    }

     DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIA_UT_XV_FUNC_HDTV_SetSignalType: TVType = 0x%x\n", pBIOSInfo->TVSettingInfo.TVType));

    return ret;
}


static void
VIA_UT_ESC_FUNC_GAMMA_GetDeviceSupportState(ScrnInfoPtr pScrn,
					    CARD32* pdwSupportState)
{
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIA_UT_ESC_FUNC_GAMMA_GetDeviceSupportState\n"));
    if (pScrn->bitsPerPixel == 8)
        *pdwSupportState = UT_DEVICE_NONE;
    else
        *pdwSupportState = UT_DEVICE_CRT1 | UT_DEVICE_TV | UT_DEVICE_DFP | UT_DEVICE_LCD;
}


static void
VIA_UT_ESC_FUNC_GAMMA_GetLookUpTable(VIABIOSInfoPtr pBIOSInfo, CARD32* pGammaLUT)
{
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIA_UT_ESC_FUNC_GAMMA_GetLookUpTable\n"));
    int i =0;

    for (i=0; i<256; i++)
    {
        pGammaLUT[i] = ((CARD32)pBIOSInfo->colors[i].red) << 16;
        pGammaLUT[i] |= ((CARD32)pBIOSInfo->colors[i].green) << 8;
        pGammaLUT[i] |= (CARD32)pBIOSInfo->colors[i].blue;
    }
}


static Bool
VIA_UT_ESC_FUNC_GAMMA_SetLookUpTable(ScrnInfoPtr pScrn, CARD32* pGammaLUT)
{
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIA_UT_ESC_FUNC_GAMMA_SetLookUpTable\n"));
    VIAPtr pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;

    int i = 0;
    int ret = 0;

    if(pGammaLUT[0] == 0xFFFFFFFF)   /* Set default gamma value */
    {
        for (i=0; i<256; i++)
        {
            pBIOSInfo->colors[i].red = i;
            pBIOSInfo->colors[i].green = i;
            pBIOSInfo->colors[i].blue = i;
        }
    }
    else
    {
        for (i=0; i<256; i++)
        {
            pBIOSInfo->colors[i].red = (unsigned short)(pGammaLUT[i] >> 16);
            pBIOSInfo->colors[i].green = (unsigned short)(pGammaLUT[i] >> 8) & 0x00FF;
            pBIOSInfo->colors[i].blue = (unsigned short)(pGammaLUT[i] & 0x000000FF);
        }
    }
    if (VIARestoreGamma(pScrn, pBIOSInfo->colors))
    {
        ret = TRUE;
    }
    else
    {
        ret = FALSE;
    }

    return ret;
}


static void
VIA_UT_ESC_FUNC_GAMMA_GetDefaultLookUpTable(VIABIOSInfoPtr pBIOSInfo,
					    CARD32* pGammaLUT)
{
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIA_UT_ESC_FUNC_GAMMA_GetDefaultLookUpTable\n"));
    int i = 0;

    for (i=0; i<256; i++)
    {
        pGammaLUT[i] = (CARD32)i << 16;
        pGammaLUT[i] |= (CARD32)i << 8;
        pGammaLUT[i] |= (CARD32)i;
    }
}


void
VIAXVUtilityProc(ScrnInfoPtr pScrn, unsigned char* buf)
{
    VIAPtr pVia;
    VIABIOSInfoPtr pBIOSInfo;
    CARD32 dwFunc, dwAction = 0, dwInData = 0;
    UTBIOSVERSION UTBIOSVersion;
    UTBIOSDATE UTBIOSDate;
    UTDriverVersion pUTDriverVersion;
    UTPANELINFO UTPanelInfo;
    UTXYVALUE MaxViewSizeValue, ViewSizeValue, MaxViewPosValue, ViewPosValue;
    UTSETTVTUNINGINFO TVTuningInfo;
    UTSETTVITEMSTATE TVSettingItemState;
    UTGAMMAINFO pUTGAMMAINFO;
    CARD32 dwVideoRam, dwSupportState = 0, dwConnectState = 0, dwActiveState = 0;
    CARD32 PanningState = 0;
    CARD32 dwSAMMState, dwRotateState = 0, dwExpandState, dwStandard = 0;
    CARD32 dwSignalType, dwMaxValue, dwValue, dwState;
    char szDriverName[32];
    long dwUTRetOK = 1, dwUTRetFail = 0, dwUTRetNoFunc = -1;
    unsigned char *InParam;
    UTDEVMODE DevMode;
    TVSETTINGINFOPTR pTVSettingInfo;
    CARD32 dwChipID = 0, dwTVChipID = 0;


    pVia = VIAPTR(pScrn);
    /*
        STChen[2006.7.11]: In SAMM case, a driver controls a screen.
        In the past, we have to move application's window to switch to correct driver.
        So I add following step to detect which driver we should use to avoid this problem.
    */
    if (pVia->pBIOSInfo->SAMM) {
        pScrn = VIADriverSelection(pScrn, buf);
	pVia = VIAPTR(pScrn);
    }

    pBIOSInfo = pVia->pBIOSInfo;

    InParam = buf;
    dwFunc =  *((CARD32*)InParam);
    InParam += 4;

    dwAction =*((CARD32*)InParam);
    InParam += 4;

    switch(dwFunc) {
	case UT_XV_FUNC_BIOS:
	    switch(dwAction) {
		case UT_XV_FUNC_BIOS_GetChipID:
		    memcpy((void*)InParam, &dwUTRetOK, sizeof(CARD32));
		    InParam += 4;
		    VIA_UT_XV_FUNC_BIOS_GetChipID(pScrn, &dwChipID);
		    memcpy((void*)InParam, &dwChipID, sizeof(CARD32));
		    InParam += 4;
		    break;
		case UT_XV_FUNC_BIOS_GetVersion:
                    VIA_UT_XV_FUNC_BIOS_GetVersion(pBIOSInfo, &UTBIOSVersion);
		    memcpy((void*)InParam, &dwUTRetOK, sizeof(long));
		    InParam += 4;
                    memcpy((void*)InParam, &UTBIOSVersion,
			   sizeof(UTBIOSVERSION));
		    break;
		case UT_XV_FUNC_BIOS_GetDate:
		    VIA_UT_XV_FUNC_BIOS_GetDate(pScrn, &UTBIOSDate);
		    memcpy((void*)InParam, &dwUTRetOK, sizeof(long));
		    InParam += 4;
                    memcpy((void*)InParam, &UTBIOSDate, sizeof(UTBIOSDATE));
		    break;
                case UT_XV_FUNC_BIOS_GetVideoMemSizeMB:
		    VIA_UT_XV_FUNC_BIOS_GetVideoMemSizeMB(pScrn, &dwVideoRam);
		    memcpy((void*)InParam, &dwUTRetOK, sizeof(CARD32));
		    InParam += 4;
		    memcpy((void*)InParam, &dwVideoRam, sizeof(CARD32));
		    break;
		case UT_XV_FUNC_BIOS_GetTVChipID :
		    memcpy((void*)InParam, &dwUTRetOK, sizeof(CARD32));
		    InParam += 4;
		    VIA_UT_XV_FUNC_BIOS_GetTVChipID(pBIOSInfo, &dwTVChipID);
		    memcpy((void*)InParam, &dwTVChipID, sizeof(CARD32));
		    InParam += 4;
		    break;
		default:
		    memcpy((void*)InParam, &dwUTRetFail, sizeof(CARD32));
		    InParam += 4;
		    ErrorF(" via_utility.c : dwAction not supported\n");
		    break;
	    }
	    break;
	case UT_XV_FUNC_DRIVER:
	    switch (dwAction) {
		case UT_XV_FUNC_DRIVER_GetFileName:
		    InParam += 4;
		    dwInData = *((CARD32*)InParam);
		    if (VIA_UT_XV_FUNC_DRIVER_GetFileName(dwInData,
							  szDriverName)) {
			InParam = buf + 8;
			memcpy((void*)InParam, &dwUTRetOK, sizeof(CARD32));
			InParam += 4;
                        memcpy((void*)InParam, &szDriverName,
			       sizeof(szDriverName));
		    }
		    else {
			InParam = buf + 8;
			memcpy((void*)InParam, &dwUTRetFail, sizeof(CARD32));
		    }
		    break;
		case UT_XV_FUNC_DRIVER_GetFileVersion:
		    InParam += 4;
		    dwInData = *((CARD32 *)InParam);
		    if (VIA_UT_XV_FUNC_DRIVER_GetFileVersion(pScrn,
							     dwInData,
							     &pUTDriverVersion)) {
			InParam = buf + 8;
			memcpy((void*)InParam, &dwUTRetOK, sizeof(CARD32));
			InParam += 4;
                        memcpy((void*)InParam, &pUTDriverVersion, sizeof(UTDriverVersion));
		    }
                    else {
			InParam = buf + 8;
			memcpy((void*)InParam, &dwUTRetFail, sizeof(CARD32));
		    }
		    break;
		default:
		    memcpy((void*)InParam, &dwUTRetFail, sizeof(CARD32));
		    ErrorF(" via_utility.c : dwAction not supported\n");
		    break;
	    }
	    break;
	case UT_XV_FUNC_DEVICE:
            switch(dwAction)  {
		case UT_XV_FUNC_DEVICE_GetSupportState:
		    VIA_UT_XV_FUNC_DEVICE_GetSupportState(pBIOSInfo,
							  &dwSupportState);
		    memcpy((void*)InParam, &dwUTRetOK, sizeof(CARD32));
		    InParam += 4;
                    memcpy((void*)InParam, &dwSupportState, sizeof(CARD32));
		    break;
		case UT_XV_FUNC_DEVICE_GetConnectState:
			VIA_UT_XV_FUNC_DEVICE_GetConnectState(pBIOSInfo, &dwConnectState);
		    memcpy((void *)InParam, &dwUTRetOK, sizeof(CARD32));
		    InParam += 4;
		    memcpy((void *)InParam, &dwConnectState, sizeof(CARD32));
		    break;
		case UT_XV_FUNC_DEVICE_GetActiveState:
		    VIA_UT_XV_FUNC_DEVICE_GetActiveState(pScrn, &dwActiveState);
		    memcpy((void*)InParam, &dwUTRetOK, sizeof(CARD32));
		    InParam += 4;
                    memcpy((void*)InParam, &dwActiveState, sizeof(CARD32));
		    break;
		case UT_XV_FUNC_DEVICE_SetActiveState:
		    InParam += 4;
		    dwInData = *((CARD32 *)InParam);
		    /*Al Zhang[2006.04.24]: If MV enabled, block TV and HDTV status changing*/
		    if(CheckMacrovisionEnabled(pBIOSInfo) &&
		       (((dwInData & UT_DEVICE_HDTV) !=
			 (pBIOSInfo->ActiveDevice & UT_DEVICE_HDTV)) ||
			((dwInData & UT_DEVICE_TV) !=
			 (pBIOSInfo->ActiveDevice & UT_DEVICE_TV)))) {
			InParam = buf + 8;
			memcpy((void*)InParam, &dwUTRetOK, sizeof(CARD32));
			break;
		    }
                    VIA_UT_XV_FUNC_DEVICE_SetActiveState(pBIOSInfo,
							 pScrn->currentMode,
							 dwInData);
		    InParam = buf + 8;
		    memcpy((void*)InParam, &dwUTRetOK, sizeof(CARD32));
		    break;
		case UT_XV_FUNC_DEVICE_GetSAMMState:
		    memcpy((void*)InParam, &dwUTRetOK, sizeof(CARD32));
		    InParam += 4;
		    VIA_UT_XV_FUNC_DEVICE_GetSAMMState(pBIOSInfo, &dwSAMMState);
		    memcpy((void*)InParam, &dwSAMMState, sizeof(CARD32));
		    break;
		case UT_XV_FUNC_DEVICE_GetRoateState:
		    memcpy((void*)InParam, &dwUTRetOK, sizeof(CARD32));
		    InParam += 4;
		    dwRotateState = UT_ROTATE_NONE;
                    memcpy((void*)InParam, &dwRotateState, sizeof(CARD32));
		    break;
		case UT_XV_FUNC_DEVICE_SetRoateState:
		    memcpy((void*)InParam, &dwUTRetNoFunc, sizeof(CARD32));
		    break;
		case UT_XV_FUNC_DEVICE_GetSupportSAMMState:
		    memcpy((void*)InParam, &dwUTRetOK, sizeof(CARD32));
		    InParam += 4;
		    VIA_UT_XV_FUNC_DEVICE_GetSupportSAMMState(pBIOSInfo,
							      &dwSAMMState);
		    memcpy((void*)InParam, &dwSAMMState, sizeof(CARD32));
		    break;
		case UT_XV_FUNC_DEVICE_SetV3OnIGAStatus:
		    InParam += 4;
		    dwInData = *((CARD32*)InParam);
		    if (VIA_UT_XV_FUNC_DEVICE_SetV3OnIGAStatus(pScrn,
							       dwInData)) {
			InParam = buf + 8;
			memcpy((void*)InParam, &dwUTRetOK, sizeof(CARD32));
		    }
                    else {
			InParam = buf + 8;
			memcpy((void*)InParam, &dwUTRetFail, sizeof(CARD32));
		    }
		    break;
                case UT_XV_FUNC_DEVICE_GetV3OnIGAStatus: {
			CARD32 tmp = 0;

			memcpy((void*)InParam, &dwUTRetOK, sizeof(CARD32));
			InParam += 4;
			VIA_UT_XV_FUNC_DEVICE_GetV3OnIGAStatus(pScrn, &tmp);
                        memcpy((void*)InParam, &tmp, sizeof(CARD32));
                    } break;
		case UT_XV_FUNC_DEVICE_GetDuoViewStatus:
		    memcpy((void*)InParam, &dwUTRetOK, sizeof(CARD32));
		    InParam += 4;
		    VIA_UT_XV_FUNC_DEVICE_GetDuoViewStatus(pScrn, &dwSAMMState);
		    memcpy((void*)InParam, &dwSAMMState, sizeof(CARD32));
                    break;
		case UT_XV_FUNC_DEVICE_GetPanningState:
		    PanningState = VIA_UT_XV_FUNC_DEVICE_GetPanningState(pBIOSInfo);
		    InParam = buf + 8;
		    memcpy((void*)InParam, &dwUTRetOK, sizeof(CARD32));
                    InParam += 4;
                    memcpy((void*)InParam, &PanningState, sizeof(CARD32));
		    break;
                default:
		    memcpy((void *)InParam, &dwUTRetFail, sizeof(CARD32));
		    ErrorF(" via_utility.c : dwAction not supported\n");
		    break;
	    }
	    break;
	case UT_XV_FUNC_PANEL:
            switch (dwAction) {
		case UT_XV_FUNC_DEVICE_SetTargetPanel:
		    memcpy((void*)InParam, &dwUTRetNoFunc, sizeof(CARD32));
		    break;
		case UT_XV_FUNC_DEVICE_GetPanelInfo:
		    InParam += 4;
		    dwInData = *((CARD32*)InParam);
		    VIA_UT_XV_FUNC_DEVICE_GetPanelInfo(pBIOSInfo, dwInData,
						       &UTPanelInfo);
		    InParam = buf + 8;
		    memcpy((void*)InParam, &dwUTRetOK, sizeof(CARD32));
		    InParam += 4;
                    memcpy((void*)InParam, &UTPanelInfo, sizeof(UTPANELINFO));
		    break;
		case UT_XV_FUNC_DEVICE_GetExpandState:
		    VIA_UT_XV_FUNC_DEVICE_GetExpandState(pBIOSInfo,
							 &dwExpandState);
		    memcpy((void*)InParam, &dwUTRetOK, sizeof(CARD32));
		    InParam += 4;
                    memcpy((void*)InParam, &dwExpandState, sizeof(CARD32));
		    break;
		case UT_XV_FUNC_DEVICE_SetExpandState:
		    InParam += 4;
		    memcpy(&UTPanelInfo, (void *)InParam, sizeof(UTPANELINFO));
		    if(VIA_UT_XV_FUNC_DEVICE_SetExpandState(pBIOSInfo,
							    pScrn->currentMode,
							    UTPanelInfo)) {
			InParam = buf + 8;
			memcpy((void*)InParam, &dwUTRetOK, sizeof(CARD32));
		    }
                    else {
			InParam = buf + 8;
			memcpy((void*)InParam, &dwUTRetFail, sizeof(CARD32));
		    }
		    break;
                case UT_XV_FUNC_DEVICE_GetSupportExpand:
		    InParam += 4;
		    dwInData = *((CARD32*)InParam);
                    VIA_UT_XV_FUNC_DEVICE_GetSupportExpand(pBIOSInfo,
							   &dwSupportState);
		    InParam = buf + 8;
		    memcpy((void*)InParam, &dwUTRetOK, sizeof(CARD32));
		    InParam += 4;
		    memcpy((void*)InParam, &dwSupportState, sizeof(CARD32));
                    break;
		default:
		    memcpy((void*)InParam, &dwUTRetFail, sizeof(CARD32));
		    InParam += 4;
		    ErrorF(" via_utility.c : dwAction not supported\n");
		    break;
	    }
	    break;
	case UT_XV_FUNC_TV:
	case UT_XV_FUNC_TV2:
            if (dwFunc == UT_XV_FUNC_TV2) {
		DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO,
				 "Use TV2 setting. \n"));
		pTVSettingInfo = &(pBIOSInfo->TVSettingInfo2);
	    }
            else {
		DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO,
				 "Use TV setting. \n"));
		pTVSettingInfo = &(pBIOSInfo->TVSettingInfo);
	    }
	    switch (dwAction) {
		case UT_XV_FUNC_TV_GetSupportStandardState:
		    VIA_UT_XV_FUNC_TV_GetSupportStandardState(pBIOSInfo,
							      pTVSettingInfo,
							      &dwSupportState);
		    memcpy((void*)InParam, &dwUTRetOK, sizeof(CARD32));
		    InParam += 4;
		    memcpy((void*)InParam, &dwSupportState, sizeof(CARD32));
		    break;
		case UT_XV_FUNC_TV_GetStandard:
		    VIA_UT_XV_FUNC_TV_GetStandard(pBIOSInfo, pTVSettingInfo,
						  &dwStandard);
		    memcpy((void*)InParam, &dwUTRetOK, sizeof(CARD32));
		    InParam += 4;
                    memcpy((void*)InParam, &dwStandard, sizeof(CARD32));
		    break;
		case UT_XV_FUNC_TV_SetStandard:
		    InParam += 4;
		    dwInData = *((CARD32*)InParam);
		    /*Al Zhang[2006.04.24]: If MV enabled, block TV and HDTV status changing*/
		    if (CheckMacrovisionEnabled(pBIOSInfo)) {
			InParam = buf + 8;
			memcpy((void*)InParam, &dwUTRetOK, sizeof(CARD32));
			break;
		    }
		    if (VIA_UT_XV_FUNC_TV_SetStandard(pBIOSInfo, pTVSettingInfo,
						      pScrn->currentMode,
						      dwInData)) {
			InParam = buf + 8;
			memcpy((void*)InParam, &dwUTRetOK, sizeof(CARD32));
		    }
		    else {
			InParam = buf + 8;
			memcpy((void*)InParam, &dwUTRetFail, sizeof(CARD32));
		    }
		    break;
		case UT_XV_FUNC_TV_GetSupportSignalTypeState:
		    VIA_UT_XV_FUNC_TV_GetSupportSignalTypeState(pBIOSInfo,
								pTVSettingInfo,
								&dwSupportState);
		    memcpy((void*)InParam, &dwUTRetOK, sizeof(CARD32));
		    InParam += 4;
		    memcpy((void*)InParam, &dwSupportState, sizeof(CARD32));
		    break;
		case UT_XV_FUNC_TV_GetSignalType:
		    VIA_UT_XV_FUNC_TV_GetSignalType(pBIOSInfo, pTVSettingInfo,
						    &dwSignalType);
		    memcpy((void*)InParam, &dwUTRetOK, sizeof(CARD32));
		    InParam += 4;
                    memcpy((void*)InParam, &dwSignalType, sizeof(CARD32));
		    break;
		case UT_XV_FUNC_TV_SetSignalType:
		    InParam += 4;
		    dwInData = *((CARD32*)InParam);
		    /*Al Zhang[2006.04.24]: If MV enabled, block TV and HDTV status changing*/
		    if (CheckMacrovisionEnabled(pBIOSInfo)) {
			InParam = buf + 8;
			memcpy((void*)InParam, &dwUTRetOK, sizeof(CARD32));
			break;
		    }
                    if (VIA_UT_XV_FUNC_TV_SetSignalType(pBIOSInfo,
							pTVSettingInfo,
							pScrn->currentMode,
							dwInData)) {
			InParam = buf + 8;
			memcpy((void*)InParam, &dwUTRetOK, sizeof(CARD32));
		    }
		    else {
			InParam = buf + 8;
			memcpy((void*)InParam, &dwUTRetFail, sizeof(CARD32));
		    }
                    break;
		case UT_XV_FUNC_TV_GetMaxViewSizeValue:
		    VIA_UT_XV_FUNC_TV_GetMaxViewSizeValue(pBIOSInfo,
							  pTVSettingInfo,
							  &MaxViewSizeValue);
		    memcpy((void*)InParam, &dwUTRetOK, sizeof(CARD32));
		    InParam += 4;
		    memcpy((void*)InParam, &MaxViewSizeValue, sizeof(UTXYVALUE));
		    break;
		case UT_XV_FUNC_TV_GetViewSizeValue:
		    VIA_UT_XV_FUNC_TV_GetViewSizeValue(pBIOSInfo,
						       pTVSettingInfo,
						       &ViewSizeValue);
		    memcpy((void*)InParam, &dwUTRetOK, sizeof(CARD32));
		    InParam += 4;
		    memcpy((void*)InParam, &ViewSizeValue, sizeof(UTXYVALUE));
		    break;
		case UT_XV_FUNC_TV_SetViewSizeValue:
		    InParam = buf + 12;
		    memcpy(&ViewSizeValue, (void*)InParam, sizeof(UTXYVALUE));
		    VIA_UT_XV_FUNC_TV_SetViewSizeValue(pBIOSInfo,
						       pTVSettingInfo,
						       pScrn->currentMode,
						       ViewSizeValue);
		    InParam = buf + 8;
		    memcpy((void*)InParam, &dwUTRetOK, sizeof(CARD32));
		    break;
		case UT_XV_FUNC_TV_GetMaxViewPositionValue:
		    VIA_UT_XV_FUNC_TV_GetMaxViewPositionValue(pBIOSInfo,
							      pTVSettingInfo,
							      &MaxViewPosValue);
		    memcpy((void*)InParam, &dwUTRetOK, sizeof(CARD32));
		    InParam += 4;
		    memcpy((void*)InParam, &MaxViewPosValue, sizeof(UTXYVALUE));
		    break;
		case UT_XV_FUNC_TV_GetViewPositionValue:
		    InParam = buf + 12;
		    memcpy(&ViewPosValue, (void*)InParam, sizeof(UTXYVALUE));
		    VIA_UT_XV_FUNC_TV_GetViewPositionValue(pBIOSInfo,
							   pTVSettingInfo,
							   &ViewPosValue);
		    InParam = buf + 8;
		    memcpy((void*)InParam, &dwUTRetOK, sizeof(CARD32));
		    InParam += 4;
		    memcpy((void*)InParam, &ViewPosValue, sizeof(UTXYVALUE));
		    break;
		case UT_XV_FUNC_TV_SetViewPositionValue:
		    InParam = buf + 12;
		    memcpy(&ViewPosValue, (void*)InParam, sizeof(UTXYVALUE));
		    if (VIA_UT_XV_FUNC_TV_SetViewPositionValue(pBIOSInfo,
							       pTVSettingInfo,
							       ViewPosValue)) {
			InParam = buf + 8;
			memcpy((void*)InParam, &dwUTRetOK, sizeof(CARD32));
		    }
		    else {
			InParam = buf + 8;
			memcpy((void*)InParam, &dwUTRetFail, sizeof(CARD32));
		    }
		    break;
		case UT_XV_FUNC_TV_GetSupportTuningState:
		    VIA_UT_XV_FUNC_TV_GetSupportTuningState(pBIOSInfo,
							    pTVSettingInfo,
							    &dwSupportState);
		    memcpy((void*)InParam, &dwUTRetOK, sizeof(CARD32));
		    InParam += 4;
		    memcpy((void*)InParam, &dwSupportState, sizeof(UTXYVALUE));
		    break;
		case UT_XV_FUNC_TV_GetTuningItemMaxValue:	/* 0 BASE */
		    InParam += 4;
		    dwInData = *((CARD32*)InParam);
		    if (VIA_UT_XV_FUNC_TV_GetTuningItemMaxValue(pBIOSInfo,
								pTVSettingInfo,
								dwInData,
								&dwMaxValue)) {
			InParam = buf + 8;
			memcpy((void*)InParam, &dwUTRetOK, sizeof(CARD32));
		    }
		    else {
			InParam = buf + 8;
			memcpy((void*)InParam, &dwUTRetFail, sizeof(CARD32));
		    }
		    InParam += 4;
		    memcpy((void*)InParam, &dwMaxValue, sizeof(CARD32));
		    break;
		case UT_XV_FUNC_TV_GetTuningItemValue:
		    InParam += 4;
		    dwInData = *((CARD32*)InParam);
		    if (VIA_UT_XV_FUNC_TV_GetTuningItemValue(pBIOSInfo,
							     pTVSettingInfo,
							     dwInData,
							     &dwValue)) {
			InParam = buf + 8;
			memcpy((void*)InParam, &dwUTRetOK, sizeof(CARD32));
		    }
                    else {
			InParam = buf + 8;
			memcpy((void*)InParam, &dwUTRetFail, sizeof(CARD32));
		    }
		    InParam += 4;
		    memcpy((void*)InParam, &dwValue, sizeof(CARD32));
		    break;
		case UT_XV_FUNC_TV_SetTuningItemValue:
		    InParam = buf + 12;
		    memcpy(&TVTuningInfo, (void*)InParam,
			   sizeof(UTSETTVTUNINGINFO));
		    if (VIA_UT_XV_FUNC_TV_SetTuningItemValue(pBIOSInfo,
							     pTVSettingInfo,
							     TVTuningInfo)) {
			InParam = buf + 8;
			memcpy((void*)InParam, &dwUTRetOK, sizeof(CARD32));
		    }
                    else {
			InParam = buf + 8;
			memcpy((void*)InParam, &dwUTRetFail, sizeof(CARD32));
		    }
		    break;
		case UT_XV_FUNC_TV_GetSupportSettingState:
		    VIA_UT_XV_FUNC_TV_GetSupportSettingState(pBIOSInfo,
							     pTVSettingInfo,
							     &dwSupportState);
		    memcpy((void*)InParam, &dwUTRetOK, sizeof(CARD32));
		    InParam += 4;
		    memcpy((void*)InParam, &dwSupportState, sizeof(CARD32));
		    break;
		case UT_XV_FUNC_TV_GetSettingItemState:
		    InParam += 4;
		    dwInData = *((CARD32*)InParam);
		    if (VIA_UT_XV_FUNC_TV_GetSettingItemState(pBIOSInfo,
							      pTVSettingInfo,
							      dwInData,
							      &dwState)) {
			InParam = buf + 8;
			memcpy((void*)InParam, &dwUTRetOK, sizeof(CARD32));
		    }
                    else {
			if (dwInData == UT_TV_SETTING_LOCK_ASPECT_RATIO) {
			    InParam = buf + 8;
			    memcpy((void*)InParam, &dwUTRetNoFunc,
				   sizeof(CARD32));
			}
			else {
			    InParam = buf + 8;
			    memcpy((void*)InParam, &dwUTRetFail,
				   sizeof(CARD32));
			}
		    }
		    InParam += 4;
		    memcpy((void*)InParam, &dwState, sizeof(CARD32));
		    break;
		case UT_XV_FUNC_TV_SetSettingItemState:
		    InParam = buf + 12;
		    memcpy(&TVSettingItemState, (void*)InParam,
			   sizeof(UTSETTVITEMSTATE));
		    if (VIA_UT_XV_FUNC_TV_SetSettingItemState(pBIOSInfo,
							      pTVSettingInfo,
							      TVSettingItemState)) {
			InParam = buf + 8;
			memcpy((void*)InParam, &dwUTRetOK, sizeof(CARD32));
		    }
		    else {
			if (TVSettingItemState.dwItemID ==
			    UT_TV_SETTING_LOCK_ASPECT_RATIO) {
			    InParam = buf + 8;
			    memcpy((void*)InParam, &dwUTRetNoFunc,
				   sizeof(CARD32));
			}
			else {
			    InParam = buf + 8;
			    memcpy((void*)InParam, &dwUTRetFail,
				   sizeof(CARD32));
			}
		    }
		    break;
		default:
		    memcpy((void*)InParam, &dwUTRetFail, sizeof(CARD32));
		    InParam += 4;
		    ErrorF(" via_utility.c : dwAction not supported\n");
		    break;
	    }
	    break;
	case UT_XV_FUNC_GAMMA:
	    switch (dwAction) {
		case UT_ESC_FUNC_GAMMA_GetDeviceSupportState:
		    VIA_UT_ESC_FUNC_GAMMA_GetDeviceSupportState(pScrn,
								&dwSupportState);
		    memcpy((void*)InParam, &dwUTRetOK, sizeof(CARD32));
		    InParam += 4;
		    memcpy((void*)InParam, &dwSupportState, sizeof(CARD32));
		    break;
		case UT_ESC_FUNC_GAMMA_GetLookUpTable:
		    VIA_UT_ESC_FUNC_GAMMA_GetLookUpTable(pBIOSInfo,
							 pUTGAMMAINFO.LookUpTable);
		    memcpy((void*)InParam, &dwUTRetOK, sizeof(CARD32));
		    InParam += 4;
                    memcpy((void*)InParam, pUTGAMMAINFO.LookUpTable,
			   sizeof(pUTGAMMAINFO.LookUpTable));
		    break;
		case UT_ESC_FUNC_GAMMA_SetLookUpTable:
		    InParam = buf + 12;
		    memcpy(&pUTGAMMAINFO, (void*)InParam, sizeof(UTGAMMAINFO));
		    if (VIA_UT_ESC_FUNC_GAMMA_SetLookUpTable(pScrn,
							     pUTGAMMAINFO.LookUpTable)) {
			InParam = buf + 8;
			memcpy((void*)InParam, &dwUTRetOK, sizeof(CARD32));
		    }
                    else {
			InParam = buf + 8;
			memcpy((void*)InParam, &dwUTRetFail, sizeof(CARD32));
                    }
		    break;
		case UT_ESC_FUNC_GAMMA_GetDefaultLookUpTable:
		    VIA_UT_ESC_FUNC_GAMMA_GetDefaultLookUpTable(pBIOSInfo,
								pUTGAMMAINFO.LookUpTable);
		    memcpy((void*)InParam, &dwUTRetOK, sizeof(CARD32));
		    InParam += 4;
		    memcpy((void*)InParam, pUTGAMMAINFO.LookUpTable,
			   sizeof(pUTGAMMAINFO.LookUpTable));
		    break;
		default:
		    memcpy((void*)InParam, &dwUTRetNoFunc, sizeof(CARD32));
		    InParam += 4;
		    ErrorF(" via_utility.c : UT_XV_FUNC_GAMMA: dwAction not supported\n");
		    break;
	    }
	    break;
	case UT_XV_FUNC_HDTV:
	case UT_XV_FUNC_HDTV2:
	    if (dwFunc == UT_XV_FUNC_HDTV2) {
		DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO,
				 "Use HDTV2 setting. \n"));
		pTVSettingInfo = &(pBIOSInfo->TVSettingInfo2);
	    }
            else {
		DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO,
				 "Use HDTV setting. \n"));
		pTVSettingInfo = &(pBIOSInfo->TVSettingInfo);
	    }
	    switch (dwAction) {
		case UT_XV_FUNC_TV_GetSupportStandardState:
		    VIA_UT_XV_FUNC_HDTV_GetSupportStandardState(pBIOSInfo,
								pTVSettingInfo,
								&dwSupportState);
		    memcpy((void*)InParam, &dwUTRetOK, sizeof(CARD32));
		    InParam += 4;
		    memcpy((void*)InParam, &dwSupportState, sizeof(CARD32));
		    break;
		case UT_XV_FUNC_TV_GetStandard:
		    VIA_UT_XV_FUNC_HDTV_GetStandard(pBIOSInfo, pTVSettingInfo,
						    &dwStandard);
		    memcpy((void*)InParam, &dwUTRetOK, sizeof(CARD32));
		    InParam += 4;
		    memcpy((void*)InParam, &dwStandard, sizeof(CARD32));
		    break;
		case UT_XV_FUNC_TV_SetStandard:
		    InParam += 4;
		    dwInData = *((CARD32*)InParam);
		    if (VIA_UT_XV_FUNC_HDTV_SetStandard(pBIOSInfo,
							pTVSettingInfo,
							pScrn->currentMode,
							dwInData)) {
			InParam = buf + 8;
			memcpy((void*)InParam, &dwUTRetOK, sizeof(CARD32));
		    }
		    else {
			InParam = buf + 8;
			memcpy((void*)InParam, &dwUTRetFail, sizeof(CARD32));
		    }
		    break;
		case UT_XV_FUNC_TV_GetSupportSignalTypeState:
		    VIA_UT_XV_FUNC_HDTV_GetSupportSignalTypeState(pBIOSInfo,
								  pTVSettingInfo,
								  &dwSupportState);
		    memcpy((void*)InParam, &dwUTRetOK, sizeof(CARD32));
		    InParam += 4;
		    memcpy((void*)InParam, &dwSupportState, sizeof(CARD32));
		    break;
		case UT_XV_FUNC_TV_GetSignalType:
		    VIA_UT_XV_FUNC_HDTV_GetSignalType(pBIOSInfo, pTVSettingInfo,
						      &dwSignalType);
		    memcpy((void*)InParam, &dwUTRetOK, sizeof(CARD32));
		    InParam += 4;
		    memcpy((void*)InParam, &dwSignalType, sizeof(CARD32));
		    break;
		case UT_XV_FUNC_TV_SetSignalType:
		    InParam += 4;
		    dwInData = *((CARD32*)InParam);
		    if (VIA_UT_XV_FUNC_HDTV_SetSignalType(pBIOSInfo,
							  pTVSettingInfo,
							  pScrn->currentMode,
							  dwInData)) {
			InParam = buf + 8;
			memcpy((void*)InParam, &dwUTRetOK, sizeof(CARD32));
		    }
		    else {
			InParam = buf + 8;
			memcpy((void*)InParam, &dwUTRetFail, sizeof(CARD32));
		    }
		    break;
		default:
		    memcpy((void*)InParam, &dwUTRetNoFunc, sizeof(CARD32));
		    InParam += 4;
		    ErrorF(" via_utility.c : UT_XV_FUNC_HDTV: dwAction not supported\n");
		    break;
	    }
            break;
	case UT_XV_FUNC_DISPLAY:
	    switch (dwAction) {
		case UT_XV_FUNC_DISPLAY_EnumModes:
		    InParam += 4;
		    dwInData = *((CARD32 *)InParam);
		    InParam = buf + 8;
		    memcpy(&DevMode, (LPUTDEVMODE)InParam, sizeof(UTDEVMODE));
		    VIA_UT_XV_FUNC_DISPLAY_EnumModes(pBIOSInfo, dwInData,
						     &DevMode);
		    InParam = buf + 8;
		    memcpy((void*)InParam, &dwUTRetOK, sizeof(long));
		    InParam += 4;
		    memcpy((void*)InParam, &DevMode, sizeof(UTDEVMODE));
		    break;
		case UT_XV_FUNC_DISPLAY_GetCurrentMode:
		    VIA_UT_XV_FUNC_DISPLAY_GetCurrentMode(pBIOSInfo, &DevMode);
		    InParam = buf + 8;
		    memcpy((void*)InParam, &dwUTRetOK, sizeof(long));
		    InParam += 4;
		    memcpy((void*)InParam, &DevMode, sizeof(UTDEVMODE));
		    break;
		default:
		    memcpy((void *)InParam, &dwUTRetNoFunc, sizeof(CARD32));
		    InParam += 4;
		    ErrorF(" via_utility.c : UT_XV_FUNC_DISPLAY: sub function not supported.\n");
		    break;
	    }
	    break;
	default:
	    ErrorF(" via_utility.c : dwFunc not supported\n");
	    memcpy((void*)InParam, &dwUTRetFail, sizeof(CARD32));
            InParam += 4;
	    break;
    } /* end of switch */
}

void
VIAUTGetTVInfo(VIABIOSInfoPtr pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo)
{
    I2CDevPtr       dev;
    unsigned char   W_Buffer[3];
    unsigned char   R_Buffer[3];
    TVInfoPtr       pTVInfo = &(pTVSettingInfo->TVInfo);

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "UTGetTVInfo\n"));
    {
    dev = xf86CreateI2CDevRec();
    dev->DevName = "TV";
    dev->SlaveAddr = pBIOSInfo->TVI2CAdd;
    dev->pI2CBus = pBIOSInfo->I2C_Port2;

    if (xf86I2CDevInit(dev))
    {
        switch (pTVSettingInfo->TVEncoder)
        {
            case VIA_VT1621:
                pTVInfo->DefaultScalH = 0;
                pTVInfo->DefaultScalV = VIA_TVNORMAL;
                pTVInfo->ScalHLevel = 0;
                pTVInfo->ScalVLevel = 2;

                pTVInfo->DefaultPositionV = 6;
                pTVInfo->PositionVLevel = 11;
                pTVInfo->DefaultPositionH = 6;
                pTVInfo->PositionHLevel = 11;

                W_Buffer[0] = 0x03;
                xf86I2CWriteRead(dev, W_Buffer,1, R_Buffer,1);
                pTVInfo->DefaultFFilter = R_Buffer[0] & 0x03;
                pTVInfo->FFilterLevel = 2;

                pTVInfo->DefaultAFFilter = 0;
                pTVInfo->AFFilterLevel = 0;

                pTVInfo->AdaptiveFFilterOn = FALSE;

                pTVInfo->DefaultBrightness = 0;
                pTVInfo->BrightnessLevel = 0;

                pTVInfo->DefaultContrast = 0;
                pTVInfo->ContrastLevel = 0;

                pTVInfo->DefaultSaturation = 0;
                pTVInfo->SaturationLevel = 0;

                W_Buffer[0] = 0x10;
                xf86I2CWriteRead(dev, W_Buffer,1, R_Buffer,1);
                pTVInfo->DefaultTINT = R_Buffer[0];
                W_Buffer[0] = 0x11;
                xf86I2CWriteRead(dev, W_Buffer,1, R_Buffer,1);
                pTVInfo->DefaultTINT += (R_Buffer[0] & 0xE0) << 8;
                pTVInfo->TINTLevel = 2047;
            case VIA_VT1622:
            case VIA_VT1622A:
                pTVInfo->DefaultScalH = 0;
                pTVInfo->DefaultScalV = VIA_TVNORMAL;
                pTVInfo->ScalHLevel = 0;
                pTVInfo->ScalVLevel = 3;

                pTVInfo->DefaultPositionV = 6;
                pTVInfo->PositionVLevel = 11;
                pTVInfo->DefaultPositionH = 6;
                pTVInfo->PositionHLevel = 11;

                W_Buffer[0] = 0x03;
                xf86I2CWriteRead(dev, W_Buffer,1, R_Buffer,1);
                pTVInfo->DefaultFFilter = R_Buffer[0] & 0x03;
                pTVInfo->FFilterLevel = 2;

                W_Buffer[0] = 0x61;
                xf86I2CWriteRead(dev, W_Buffer,1, R_Buffer,1);
                pTVInfo->DefaultAFFilter = R_Buffer[0];
                pTVInfo->AFFilterLevel = 255;

                if (pTVInfo->DefaultAFFilter && !pTVInfo->DefaultFFilter)
                    pTVInfo->AdaptiveFFilterOn = TRUE;
                else
                    pTVInfo->AdaptiveFFilterOn = FALSE;

                W_Buffer[0] = 0x0B;
                xf86I2CWriteRead(dev, W_Buffer,1, R_Buffer,1);
                pTVInfo->DefaultBrightness = R_Buffer[0];
                pTVInfo->BrightnessLevel = 255;

                if (pTVSettingInfo->TVOutput == TV_OUTPUT_RGB)
                {
                    W_Buffer[0] = 0x65;
                    xf86I2CWriteRead(dev, W_Buffer,1, R_Buffer,3);
                    pTVInfo->DefaultContrast = R_Buffer[0];
                    pTVInfo->ContrastLevel = 250;

                    pTVInfo->DefaultSaturation = 0;
                    pTVInfo->SaturationLevel = 0;

                    pTVInfo->DefaultTINT = 0;
                    pTVInfo->TINTLevel = 0;
                }
                else if (pTVSettingInfo->TVOutput == TV_OUTPUT_YPBPR)
                {
                    W_Buffer[0] = 0x65;
                    xf86I2CWriteRead(dev, W_Buffer,1, R_Buffer,1);
                    pTVInfo->DefaultContrast = R_Buffer[0];
                    pTVInfo->ContrastLevel = 255;

                    pTVInfo->DefaultSaturation = 0;
                    pTVInfo->SaturationLevel = 0;

                    pTVInfo->DefaultTINT = 0;
                    pTVInfo->TINTLevel = 0;
                }
                else
                {
                    W_Buffer[0] = 0x0C;
                    xf86I2CWriteRead(dev, W_Buffer,1, R_Buffer,1);
                    pTVInfo->DefaultContrast = R_Buffer[0];
                    pTVInfo->ContrastLevel = 255;

                    W_Buffer[0] = 0x0D;
                    xf86I2CWriteRead(dev, W_Buffer,1, R_Buffer,1);
                    pTVInfo->DefaultSaturation = R_Buffer[0] << 8;
                    W_Buffer[0] = 0x0A;
                    xf86I2CWriteRead(dev, W_Buffer,1, R_Buffer,1);
                    pTVInfo->DefaultSaturation += R_Buffer[0];
                    pTVInfo->SaturationLevel = 65535;

                    W_Buffer[0] = 0x10;
                    xf86I2CWriteRead(dev, W_Buffer,1, R_Buffer,1);
                    pTVInfo->DefaultTINT = R_Buffer[0];
                    W_Buffer[0] = 0x11;
                    xf86I2CWriteRead(dev, W_Buffer,1, R_Buffer,1);
                    pTVInfo->DefaultTINT += (R_Buffer[0] & 0x07) << 8;
                    pTVInfo->TINTLevel = 2047;
                }
                break;
            case VIA_VT1623:
                GPIOI2C_Initial(pBIOSInfo, 0x40);
                pTVInfo->DefaultScalH = 0;
                pTVInfo->DefaultScalV = VIA_TVNORMAL;
                pTVInfo->ScalHLevel = 0;
                pTVInfo->ScalVLevel = 3;

                pTVInfo->DefaultPositionV = 6;
                pTVInfo->PositionVLevel = 11;
                pTVInfo->DefaultPositionH = 6;
                pTVInfo->PositionHLevel = 11;

                GPIOI2C_Read(pBIOSInfo, 0x03, R_Buffer, 1);
                pTVInfo->DefaultFFilter = R_Buffer[0] & 0x03;
                pTVInfo->FFilterLevel = 2;

                W_Buffer[0] = 0x61;
                GPIOI2C_Read(pBIOSInfo, W_Buffer[0], R_Buffer, 1);
                pTVInfo->DefaultAFFilter = R_Buffer[0];
                pTVInfo->AFFilterLevel = 255;

                if (pTVInfo->DefaultAFFilter && !pTVInfo->DefaultFFilter)
                    pTVInfo->AdaptiveFFilterOn = TRUE;
                else
                    pTVInfo->AdaptiveFFilterOn = FALSE;

                GPIOI2C_Read(pBIOSInfo, 0x0B, R_Buffer, 1);
                pTVInfo->DefaultBrightness = R_Buffer[0];
                pTVInfo->BrightnessLevel = 255;

                if (pTVSettingInfo->TVOutput == TV_OUTPUT_RGB)
                {
                    GPIOI2C_Read(pBIOSInfo, 0x65, R_Buffer, 3);
                    pTVInfo->DefaultContrast = R_Buffer[0];
                    pTVInfo->ContrastLevel = 250;

                    pTVInfo->DefaultSaturation = 0;
                    pTVInfo->SaturationLevel = 0;

                    pTVInfo->DefaultTINT = 0;
                    pTVInfo->TINTLevel = 0;
                }
                else if (pTVSettingInfo->TVOutput == TV_OUTPUT_YPBPR)
                {
                    GPIOI2C_Read(pBIOSInfo, 0x65, R_Buffer, 1);
                    pTVInfo->DefaultContrast = R_Buffer[0];
                    pTVInfo->ContrastLevel = 255;

                    pTVInfo->DefaultSaturation = 0;
                    pTVInfo->SaturationLevel = 0;

                    pTVInfo->DefaultTINT = 0;
                    pTVInfo->TINTLevel = 0;
                }
                else
                {
                    GPIOI2C_Read(pBIOSInfo, 0x0C, R_Buffer, 1);
                    pTVInfo->DefaultContrast = R_Buffer[0];
                    pTVInfo->ContrastLevel = 255;

                    W_Buffer[0] = 0x0D;
                    GPIOI2C_Read(pBIOSInfo, W_Buffer[0], R_Buffer, 1);
                    pTVInfo->DefaultSaturation = R_Buffer[0] << 8;
                    W_Buffer[0] = 0x0A;
                    GPIOI2C_Read(pBIOSInfo, W_Buffer[0], R_Buffer, 1);
                    pTVInfo->DefaultSaturation += R_Buffer[0];
                    pTVInfo->SaturationLevel = 65535;

                    W_Buffer[0] = 0x10;
                    GPIOI2C_Read(pBIOSInfo, W_Buffer[0], R_Buffer, 1);
                    pTVInfo->DefaultTINT = R_Buffer[0];
                    W_Buffer[0] = 0x11;
                    GPIOI2C_Read(pBIOSInfo, W_Buffer[0], R_Buffer, 1);
                    pTVInfo->DefaultTINT += (R_Buffer[0] & 0x07) << 8;
                    pTVInfo->TINTLevel = 2047;
                }
                break;
            case VIA_VT1625:
                GetTVInfo_VT1625(pBIOSInfo, pTVSettingInfo);
                break;
            case VIA_INTEGRATED_TV:
                GetTVInfo_IntegratedTV(pBIOSInfo, pTVSettingInfo);
                break;
            default:
                pTVInfo->DefaultScalH = 0;
                pTVInfo->DefaultScalV = VIA_TVNORMAL;
                pTVInfo->ScalHLevel = 0;
                pTVInfo->ScalVLevel = 2;
                pTVInfo->DefaultPositionV = 0;
                pTVInfo->PositionVLevel = 0;
                pTVInfo->DefaultPositionH = 0;
                pTVInfo->PositionHLevel = 0;
                pTVInfo->AdaptiveFFilterOn = FALSE;
                pTVInfo->DefaultFFilter = 0;
                pTVInfo->FFilterLevel = 0;
                pTVInfo->DefaultAFFilter = 0;
                pTVInfo->AFFilterLevel = 0;
                pTVInfo->DefaultBrightness = 0;
                pTVInfo->BrightnessLevel = 0;
                pTVInfo->DefaultContrast = 0;
                pTVInfo->ContrastLevel = 0;
                pTVInfo->DefaultSaturation = 0;
                pTVInfo->SaturationLevel = 0;
                pTVInfo->DefaultTINT = 0;
                pTVInfo->TINTLevel = 0;
                break;
        }

        xf86DestroyI2CDevRec(dev,TRUE);
    }
    else
    {
        xf86DestroyI2CDevRec(dev,TRUE);
        DEBUG(xf86Msg(X_DEFAULT, "DevInit fail!\n"));
    }
    }   /* not FS453/FS454 */

    if (!pTVInfo->HasUserSetting)
    {
        pTVInfo->CurrentScalH = pTVInfo->DefaultScalH;
        pTVInfo->CurrentScalV = pTVInfo->DefaultScalV;
        pTVInfo->CurrentPositionV = pTVInfo->DefaultPositionV;
        pTVInfo->CurrentPositionH = pTVInfo->DefaultPositionH;
        pTVInfo->CurrentFFilter = pTVInfo->DefaultFFilter;
        pTVInfo->CurrentAFFilter = pTVInfo->DefaultAFFilter;
        pTVInfo->CurrentBrightness = pTVInfo->DefaultBrightness;
        pTVInfo->CurrentContrast = pTVInfo->DefaultContrast;
        pTVInfo->CurrentSaturation = pTVInfo->DefaultSaturation;
        pTVInfo->CurrentTINT = pTVInfo->DefaultTINT;
        DEBUG(xf86Msg(X_INFO, "No user TV setting exist! Current setting equal default setting!\n"));
    }
}

Bool VIALoadGammaSetting(VIABIOSInfoPtr pBIOSInfo)
{
    FILE	*VIARC = NULL;
    char 	filename[40], *home = NULL;
    Bool	ret = FALSE;
    unsigned int i;

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIALoadGammaSetting\n"));
    home="/etc/X11";
    if (home)
    {
        filename[0] = '\0';
        strcpy(filename, home);
        strcat(filename, "/.VIAGAMMARC");
        if ((VIARC = fopen(filename, "r")) == NULL)
        {
            xf86Msg(X_WARNING,
                    "No Gamma setting file exist, Create new ones\n");
            if ((VIARC = fopen(filename, "a")) == NULL)
            {
                xf86Msg(X_ERROR, "Create .VIAGAMMARC failure!\n");
                return FALSE;
            }
            for (i=0; i<256; i++)
            {
                pBIOSInfo->colors[i].red = i;
                pBIOSInfo->colors[i].green = i;
                pBIOSInfo->colors[i].blue = i;
            }
        }
        else
        {
            /* S.T.Chen[2006.5.24]: Gamma record in driver is different with gamma record in utility.
               So I remove resolution save setting to solve asynchronous problem.
            */
            /* File maybe exist but it is empty. So we must load a default value first. */
            for (i=0; i<256; i++)
            {
                pBIOSInfo->colors[i].red = i;
                pBIOSInfo->colors[i].green = i;
                pBIOSInfo->colors[i].blue = i;
            }

            /* If file is not empty then load previous setting */
            while(!feof(VIARC))
            {
                for (i=0; i<256; i++)
                {
                    fscanf(VIARC, "%hu", &pBIOSInfo->colors[i].red);
                    fscanf(VIARC, "%hu", &pBIOSInfo->colors[i].green);
                    fscanf(VIARC, "%hu", &pBIOSInfo->colors[i].blue);
                }
            }
            ret = TRUE;
            fclose(VIARC);
        }
    }
    return ret;
}

void
VIARestoreUserTVSetting(VIABIOSInfoPtr  pBIOSInfo, TVSETTINGINFOPTR pTVSettingInfo)
{
    TVInfoPtr       pTVInfo = &(pTVSettingInfo->TVInfo);
    I2CDevPtr       dev;
    int             HPos, VPos, value;
    unsigned char   W_Buffer[4], R_Buffer[3];

    {
    /* if no user setting, we do not need to restore. (except FS453/454) */
    if (!pTVInfo->HasUserSetting)
        return;

    dev = xf86CreateI2CDevRec();
    dev->DevName = "TV";
    dev->SlaveAddr = pBIOSInfo->TVI2CAdd;
    dev->pI2CBus = pBIOSInfo->I2C_Port2;
    if (xf86I2CDevInit(dev))
    {
        switch (pTVSettingInfo->TVEncoder)
        {
            case VIA_VT1622:
            case VIA_VT1622A:
                if ((pTVInfo->CurrentPositionH != 6)
                    || (pTVInfo->CurrentPositionV != 6))
                {
                    W_Buffer[0] = 0x08;
                    xf86I2CWriteRead(dev, W_Buffer,1, R_Buffer,2);
                    HPos = R_Buffer[0];
                    VPos = R_Buffer[1];
                    W_Buffer[0] = 0x1C;
                    xf86I2CWriteRead(dev, W_Buffer,1, R_Buffer,1);
                    HPos |= (R_Buffer[0] & 0x04) << 6;
                    VPos |= (R_Buffer[0] & 0x02) << 7;

                    HPos += pTVInfo->CurrentPositionH - 6;
                    VPos += pTVInfo->CurrentPositionV - 6;

                    W_Buffer[0] = 0x08;
                    W_Buffer[1] = (unsigned char)(HPos & 0xFF);
                    W_Buffer[2] = (unsigned char)(VPos & 0xFF);
                    xf86I2CWriteRead(dev, W_Buffer,3, NULL,0);

                    W_Buffer[0] = 0x1C;
                    W_Buffer[1] = R_Buffer[0] & ~0x06;             /* [2]1CH, [1]1CH */
                    W_Buffer[1] |= (unsigned char)((HPos >> 6) & 0x04);
                    W_Buffer[1] |= (unsigned char)((VPos >> 7) & 0x02);
                    xf86I2CWriteRead(dev, W_Buffer,2, NULL,0);
                }

                W_Buffer[0] = 0x03;                 /* FFilter */
                xf86I2CWriteRead(dev, W_Buffer,1, R_Buffer,1);
                if (pTVInfo->AdaptiveFFilterOn)
                {
                    W_Buffer[1] = R_Buffer[0] & ~0x03;
                }
                else
                {
                    W_Buffer[1] = R_Buffer[0] & ~0x03;
                    W_Buffer[1] |= (unsigned char)(pTVInfo->CurrentFFilter);
                }
                xf86I2CWriteRead(dev, W_Buffer,2, R_Buffer,0);

                W_Buffer[0] = 0x61;                 /* Adaptive FFliter */
                W_Buffer[1] = (unsigned char)pTVInfo->CurrentAFFilter;
                xf86I2CWriteRead(dev, W_Buffer,2, R_Buffer,0);

                W_Buffer[0] = 0x0B;                 /* BRIGHTNESS */
                W_Buffer[1] = (unsigned char)pTVInfo->CurrentBrightness;
                xf86I2CWriteRead(dev, W_Buffer,2, R_Buffer,0);

                if (pTVSettingInfo->TVOutput == TV_OUTPUT_RGB)
                {
                    W_Buffer[0] = 0x65;             /* CONTRAST */
                    xf86I2CWriteRead(dev, W_Buffer,1, R_Buffer,3);
                    value = pTVInfo->CurrentContrast - R_Buffer[0];
                    W_Buffer[1] = R_Buffer[0] + value;
                    W_Buffer[2] = R_Buffer[1] + value;
                    W_Buffer[3] = R_Buffer[2] + value;
                    xf86I2CWriteRead(dev, W_Buffer,4, R_Buffer,0);
                }
                else if (pTVSettingInfo->TVOutput == TV_OUTPUT_YPBPR)
                {
                    W_Buffer[0] = 0x65;             /* CONTRAST */
                    W_Buffer[1] = (unsigned char)pTVInfo->CurrentContrast;
                    xf86I2CWriteRead(dev, W_Buffer,2, R_Buffer,0);
                }
                else
                {
                    W_Buffer[0] = 0x0C;             /* CONTRAST */
                    W_Buffer[1] = (unsigned char)pTVInfo->CurrentContrast;
                    xf86I2CWriteRead(dev, W_Buffer,2, R_Buffer,0);

                    W_Buffer[0] = 0x0D;             /* SATURATION highbyte */
                    W_Buffer[1] = (unsigned char)(pTVInfo->CurrentSaturation >> 8);
                    xf86I2CWriteRead(dev, W_Buffer,2, R_Buffer,0);
                    W_Buffer[0] = 0x0A;             /* SATURATION lowbyte */
                    W_Buffer[1] = (unsigned char)(pTVInfo->CurrentSaturation & 0xFF);
                    xf86I2CWriteRead(dev, W_Buffer,2, R_Buffer,0);

                    W_Buffer[0] = 0x11;             /* TINT highbyte */
                    xf86I2CWriteRead(dev, W_Buffer,1, R_Buffer,1);
                    W_Buffer[2] = R_Buffer[0] & ~0x07;
                    W_Buffer[2] |= (unsigned char)(pTVInfo->CurrentTINT >> 8);
                    W_Buffer[0] = 0x10;             /* TINT lowbyte */
                    W_Buffer[1] = (unsigned char)(pTVInfo->CurrentTINT & 0xFF);
                    xf86I2CWriteRead(dev, W_Buffer,3, R_Buffer,0);
                }
                break;
            case VIA_VT1623:
                if ((pTVInfo->CurrentPositionH != 6)
                    || (pTVInfo->CurrentPositionV != 6))
                {
                    GPIOI2C_Initial(pBIOSInfo, 0x40);
                    W_Buffer[0] = 0x08;
                    GPIOI2C_Read(pBIOSInfo, W_Buffer[0], R_Buffer, 2);
                    HPos = R_Buffer[0];
                    VPos = R_Buffer[1];
                    W_Buffer[0] = 0x1C;
                    GPIOI2C_Read(pBIOSInfo, W_Buffer[0], R_Buffer, 1);
                    HPos |= (R_Buffer[0] & 0x04) << 6;
                    VPos |= (R_Buffer[0] & 0x02) << 7;

                    HPos += pTVInfo->CurrentPositionH - 6;
                    VPos += pTVInfo->CurrentPositionV - 6;

                    W_Buffer[0] = 0x08;
                    W_Buffer[1] = (unsigned char)(HPos & 0xFF);
                    W_Buffer[2] = (unsigned char)(VPos & 0xFF);
                    GPIOI2C_Write(pBIOSInfo, W_Buffer[0], W_Buffer[1]);
                    W_Buffer[0] = 0x09;
                    GPIOI2C_Write(pBIOSInfo, W_Buffer[0], W_Buffer[2]);

                    W_Buffer[0] = 0x1C;
                    W_Buffer[1] = R_Buffer[0] & ~0x06;             /* [2]1CH, [1]1CH */
                    W_Buffer[1] |= (unsigned char)((HPos >> 6) & 0x04);
                    W_Buffer[1] |= (unsigned char)((VPos >> 7) & 0x02);
                    GPIOI2C_Write(pBIOSInfo, W_Buffer[0], W_Buffer[1]);
                }

                W_Buffer[0] = 0x03;                 /* Fflick */
                GPIOI2C_Read(pBIOSInfo, W_Buffer[0], R_Buffer, 1);
                if (pTVInfo->AdaptiveFFilterOn)
                {
                    W_Buffer[1] = R_Buffer[0] & ~0x03;
                }
                else
                {
                    W_Buffer[1] = R_Buffer[0] & ~0x03;
                    W_Buffer[1] |= (unsigned char)(pTVInfo->CurrentFFilter);
                }
                GPIOI2C_Write(pBIOSInfo, W_Buffer[0], W_Buffer[1]);

                W_Buffer[0] = 0x61;                 /* Adaptive Fflick */
                W_Buffer[1] = (unsigned char)pTVInfo->CurrentAFFilter;
                GPIOI2C_Write(pBIOSInfo, W_Buffer[0], W_Buffer[1]);

                W_Buffer[0] = 0x0B;                 /* BRIGHTNESS */
                W_Buffer[1] = (unsigned char)pTVInfo->CurrentBrightness;
                GPIOI2C_Write(pBIOSInfo, W_Buffer[0], W_Buffer[1]);

                if (pTVSettingInfo->TVOutput == TV_OUTPUT_RGB)
                {
                    W_Buffer[0] = 0x65;             /* CONTRAST */
                    GPIOI2C_Read(pBIOSInfo, W_Buffer[0], R_Buffer, 3);
                    value = pTVInfo->CurrentContrast - R_Buffer[0];
                    W_Buffer[1] = R_Buffer[0] + value;
                    W_Buffer[2] = R_Buffer[1] + value;
                    W_Buffer[3] = R_Buffer[2] + value;
                    GPIOI2C_Write(pBIOSInfo, W_Buffer[0], W_Buffer[1]);
                    GPIOI2C_Write(pBIOSInfo, W_Buffer[0] + 1 , W_Buffer[2]);
                    GPIOI2C_Write(pBIOSInfo, W_Buffer[0] + 2 , W_Buffer[3]);
                }
                else if (pTVSettingInfo->TVOutput == TV_OUTPUT_YPBPR)
                {
                    W_Buffer[0] = 0x65;             /* CONTRAST */
                    W_Buffer[1] = (unsigned char)pTVInfo->CurrentContrast;
                    GPIOI2C_Write(pBIOSInfo, W_Buffer[0], W_Buffer[1]);
                }
                else
                {
                    W_Buffer[0] = 0x0C;
                    W_Buffer[1] = (unsigned char)pTVInfo->CurrentContrast;
                    GPIOI2C_Write(pBIOSInfo, W_Buffer[0], W_Buffer[1]);

                    W_Buffer[0] = 0x0D;             /* SATURATION highbyte */
                    W_Buffer[1] = (unsigned char)(pTVInfo->CurrentSaturation >> 8);
                    GPIOI2C_Write(pBIOSInfo, W_Buffer[0], W_Buffer[1]);
                    W_Buffer[0] = 0x0A;             /* SATURATION lowbyte */
                    W_Buffer[1] = (unsigned char)pTVInfo->CurrentSaturation & 0xFF;
                    GPIOI2C_Write(pBIOSInfo, W_Buffer[0], W_Buffer[1]);

                    W_Buffer[0] = 0x11;             /* TINT highbyte */
                    GPIOI2C_Read(pBIOSInfo, W_Buffer[0], R_Buffer, 1);
                    W_Buffer[2] = R_Buffer[0] & ~0x07;
                    W_Buffer[2] |= (unsigned char)(pTVInfo->CurrentTINT >> 8);
                    W_Buffer[0] = 0x10;             /* TINT lowbyte */
                    W_Buffer[1] = (unsigned char)(pTVInfo->CurrentTINT & 0xFF);
                    GPIOI2C_Write(pBIOSInfo, W_Buffer[0], W_Buffer[1]);
                    W_Buffer[0] = 0x11;
                    GPIOI2C_Write(pBIOSInfo, W_Buffer[0], W_Buffer[2]);
                }
                break;
            case VIA_VT1625:
                RestoreUserTVSettings_VT1625(pBIOSInfo, pTVSettingInfo);
                break;
            case VIA_INTEGRATED_TV:
                RestoreUserTVSettings_IntegratedTV(pBIOSInfo, pTVSettingInfo);
                break;
            default:
                break;
        }

        xf86DestroyI2CDevRec(dev,TRUE);
    }
    else
    {
        xf86DestroyI2CDevRec(dev,TRUE);
    }
    }   /* not FS453/FS454 */
}


static Bool
VIASaveUserTVSetting(VIABIOSInfoPtr pBIOSInfo,
		     TVSETTINGINFOPTR pTVSettingInfo,
		     const char *pchFileName)
{
    char filename[40], tmpname[40], buffer[200], *home = NULL;
    Bool found = FALSE, ret = FALSE;
    FILE *VIATVRC = NULL, *VIAtmp = NULL;
    int  HDisplay, VDisplay, bitsPerPixel, tmp, TVStandard, TVOutput;
    TVInfoPtr pTVInfo = &(pTVSettingInfo->TVInfo);

    DEBUG(xf86Msg(X_ERROR, "VIASaveUserTVSetting!\n"));

    filename[0] = '\0';
    found = FALSE;

    home = "/etc/X11";
    strcpy(filename, home);
    strcat(filename, pchFileName);
    if ((VIATVRC = fopen(filename, "r")) == NULL) {
	xf86Msg(X_DEFAULT, "Open user TV setting file failed.\n");
	if ((VIATVRC = fopen(filename, "a")) == NULL) {
	    DEBUG(xf86Msg(X_ERROR, "Create .VIATVRC failure!\n"));
	    return FALSE;
	}
    }

    pTVInfo->HasUserSetting = TRUE;

    /* save user setting to temp file */
    strcpy(tmpname, home);
    strcat(tmpname, "/.VIAtmp");

    VIAtmp = fopen(tmpname, "w+");
    while (fgets(buffer, 200, VIATVRC) != NULL) {
	/* RainLin [2005.12.05]: TV output should be a condition of saving TV advanced settings because */
	/* not all of TV outputs support the same TV advanced setting items. */
        sscanf(buffer,"%d %d %d %d %d %d %d %d", &HDisplay, &VDisplay, &bitsPerPixel, &tmp, &tmp, &tmp, &TVStandard, &TVOutput);

	if (HDisplay == pBIOSInfo->SaveHDisplay &&
	    VDisplay == pBIOSInfo->SaveVDisplay &&
	    bitsPerPixel == pBIOSInfo->bitsPerPixel &&
	    TVStandard == pTVSettingInfo->TVType &&
	    TVOutput == pTVSettingInfo->TVOutput) {
	    fprintf(VIAtmp, "%ld %ld %ld %ld %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %d %lu %lu %lu %lu\n",
	            pBIOSInfo->SaveHDisplay,
	            pBIOSInfo->SaveVDisplay,
	            pBIOSInfo->bitsPerPixel,
	            pTVInfo->AdaptiveFFilterOn,
	            pTVInfo->CurrentScalH,
	            pTVInfo->CurrentScalV,
	            pTVSettingInfo->TVType,
	            pTVSettingInfo->TVOutput,
	            pTVInfo->CurrentPositionH,
	            pTVInfo->CurrentPositionV,
	            pTVInfo->CurrentFFilter,
	            pTVInfo->CurrentAFFilter,
	            pTVInfo->CurrentBrightness,
	            pTVInfo->CurrentContrast,
	            pTVInfo->CurrentSaturation,
	            pTVInfo->CurrentTINT,
	            pTVSettingInfo->TVDotCrawl,
	            pTVSettingInfo->TV3DScalInfo.XScaleLevel,
	            pTVSettingInfo->TV3DScalInfo.YScaleLevel,
	            pTVSettingInfo->TV3DScalInfo.XPOSITIONLevel,
	            pTVSettingInfo->TV3DScalInfo.YPOSITIONLevel);
	    found = TRUE;
	    xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO,
		       "After save TV info: XScaleLevel=%ld,YScaleLevel=%ld,XPOSITIONLevel=%ld,YPOSITIONLevel=%ld\n",
		       pTVSettingInfo->TV3DScalInfo.XScaleLevel,
		       pTVSettingInfo->TV3DScalInfo.YScaleLevel,
		       pTVSettingInfo->TV3DScalInfo.XPOSITIONLevel,
		       pTVSettingInfo->TV3DScalInfo.YPOSITIONLevel);
	}
	else {
	    fputs(buffer, VIAtmp);
	}
    }
    if (!found) {
	fprintf(VIAtmp, "%ld %ld %ld %ld %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %d %lu %lu %lu %lu\n",
		pBIOSInfo->SaveHDisplay,
		pBIOSInfo->SaveVDisplay,
		pBIOSInfo->bitsPerPixel,
		pTVInfo->AdaptiveFFilterOn,
		pTVInfo->CurrentScalH,
		pTVInfo->CurrentScalV,
		pTVSettingInfo->TVType,
		pTVSettingInfo->TVOutput,
		pTVInfo->CurrentPositionH,
		pTVInfo->CurrentPositionV,
		pTVInfo->CurrentFFilter,
		pTVInfo->CurrentAFFilter,
		pTVInfo->CurrentBrightness,
		pTVInfo->CurrentContrast,
		pTVInfo->CurrentSaturation,
		pTVInfo->CurrentTINT,
		pTVSettingInfo->TVDotCrawl,
		pTVSettingInfo->TV3DScalInfo.XScaleLevel,
		pTVSettingInfo->TV3DScalInfo.YScaleLevel,
		pTVSettingInfo->TV3DScalInfo.XPOSITIONLevel,
		pTVSettingInfo->TV3DScalInfo.YPOSITIONLevel);
    }
    fclose(VIATVRC);
    fclose(VIAtmp);
    if (remove(filename) != -1) {
	if (rename(tmpname, filename) == 0) {
	    ret = TRUE;
	}
    }

    return ret;
}


Bool VIASaveUserSetting(VIABIOSInfoPtr  pBIOSInfo)
{
    FILE *VIARC=NULL, *VIAtmp=NULL;
    char filename[40], tmpname[40], buffer[200], *home = NULL;
    int  HDisplay, VDisplay, bitsPerPixel, ActiveDevice;
    Bool found = FALSE, ret = FALSE;
    Bool IsCenter;
    int  OldCenter, OldTVStandard, OldTVOutput, OldTV2Standard, OldTV2Output;
    int  TVStandard, TVOutput, TV2Standard, TV2Output;

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIASaveUserSetting\n"));

    if (pBIOSInfo->ActiveDevice == VIA_DEVICE_LCD2)
    {
        IsCenter = pBIOSInfo->LVDSSettingInfo2.Center;
    }
    else
    {
        IsCenter = pBIOSInfo->LVDSSettingInfo.Center;
    }


    /* VansonWu [2004.2.16] Add HDTV_480P for FS454 */
    home="/etc/X11";
    if (home)
    {
        filename[0] = '\0';
        strcpy(filename, home);
        strcat(filename, "/.VIARC");
        if ((VIARC = fopen(filename, "r")) == NULL)
        {
            xf86Msg(X_DEFAULT, "Open user setting file failed.\n");
            if ((VIARC = fopen(filename, "a")) == NULL)
            {
                DEBUG(xf86Msg(X_ERROR, "Create .VIARC failure!\n"));
                return FALSE;
            }
        }

        /* save user setting to temp file */
        strcpy(tmpname, home);
        strcat(tmpname, "/.VIAtmp");
        VIAtmp = fopen(tmpname, "w+");
        /* looking for old setting by resolution & color depth index */
        while (fgets(buffer, 40, VIARC) != NULL)
        {
            sscanf(buffer,"%d %d %d %d %d %d %d %d %d",
                   &HDisplay,
                   &VDisplay,
                   &bitsPerPixel,
                   &ActiveDevice,
                   &OldCenter,
                   &OldTVStandard,
                   &OldTVOutput,
                   &OldTV2Standard,
                   &OldTV2Output);

            if(pBIOSInfo->SAMM)
            {
                /* dual TV's case */
                if (pBIOSInfo->ActualActiveDevice ==
		    (VIA_DEVICE_TV | VIA_DEVICE_TV2))
                {
                    if (pBIOSInfo->ActiveDevice == VIA_DEVICE_TV)
                    {
                        /* The driver which control the TV doesn't have the information
                           about TV2's setting. So we use the saved data.
                        */
                        TVStandard = pBIOSInfo->TVSettingInfo.TVType;
                        TVOutput = pBIOSInfo->TVSettingInfo.TVOutput;
                        TV2Standard = OldTV2Standard;
                        TV2Output = OldTV2Output;
                    }
                    else
                    {
                        TVStandard = OldTVStandard;
                        TVOutput = OldTVOutput;
                        TV2Standard = pBIOSInfo->TVSettingInfo2.TVType;
                        TV2Output = pBIOSInfo->TVSettingInfo2.TVOutput;
                    }
                }
                else /* non-dual TV's case*/
                {
                    TVStandard = pBIOSInfo->TVSettingInfo.TVType;
                    TVOutput = pBIOSInfo->TVSettingInfo.TVOutput;
                    TV2Standard = pBIOSInfo->TVSettingInfo2.TVType;
                    TV2Output = pBIOSInfo->TVSettingInfo2.TVOutput;
                }
            }
            else
            {
                TVStandard = pBIOSInfo->TVSettingInfo.TVType;
                TVOutput = pBIOSInfo->TVSettingInfo.TVOutput;
                TV2Standard = pBIOSInfo->TVSettingInfo2.TVType;
                TV2Output = pBIOSInfo->TVSettingInfo2.TVOutput;
            }



            if (HDisplay == pBIOSInfo->SaveHDisplay &&
                VDisplay == pBIOSInfo->SaveVDisplay &&
                bitsPerPixel == pBIOSInfo->bitsPerPixel)
            {
                /* RainLin [2005.12.05]: Also save TV output. */
                fprintf(VIAtmp, "%d %d %d %d %d %d %d %d %d \n",
                pBIOSInfo->SaveHDisplay,
                pBIOSInfo->SaveVDisplay,
                pBIOSInfo->bitsPerPixel,
                pBIOSInfo->ActualActiveDevice,
                IsCenter,
                TVStandard,
                TVOutput,
                TV2Standard,
                TV2Output);

                found = TRUE;
            }
            else
            {
                fputs(buffer, VIAtmp);
            }
        }

        if (!found)
        {
            fprintf(VIAtmp, "%d %d %d %d %d %d %d %d %d\n",
                    pBIOSInfo->SaveHDisplay,
                    pBIOSInfo->SaveVDisplay,
                    pBIOSInfo->bitsPerPixel,
                    pBIOSInfo->ActualActiveDevice,
                    IsCenter,
                    pBIOSInfo->TVSettingInfo.TVType,
                    pBIOSInfo->TVSettingInfo.TVOutput,
                    pBIOSInfo->TVSettingInfo2.TVType,
                    pBIOSInfo->TVSettingInfo2.TVOutput);
        }

        fclose(VIARC);
        fclose(VIAtmp);
        if(remove(filename) != -1)              /* remove old setting file */
        {
            if(rename(tmpname, filename) == 0)
            {
                ret = TRUE;
            }
        }

        /* save tv user setting to file (.VIATVRC) */
        if (pBIOSInfo->ActiveDevice & VIA_DEVICE_TV)
        {
            if (VIASaveUserTVSetting(pBIOSInfo, &(pBIOSInfo->TVSettingInfo), "/.VIATVRC"))
            {
                ret = TRUE;
            }
        }

        if (pBIOSInfo->ActiveDevice & VIA_DEVICE_TV2)
        {
            if (VIASaveUserTVSetting(pBIOSInfo, &(pBIOSInfo->TVSettingInfo2), "/.VIATV2RC"))
            {
                ret = TRUE;
            }
        }

    }
    return ret;
}


void VIALoadUserSetting(VIABIOSInfoPtr pBIOSInfo)
{
    FILE *VIARC=NULL;
    char filename[40], buffer[40], *home = NULL;
    int  HDisplay, VDisplay, bitsPerPixel, ActiveDevice, tmp, TVType, TVOutput, TV2Type, TV2Output;
    Bool Center;

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIALoadUserSetting\n"));
    DEBUG(xf86Msg(X_INFO, "pBIOSInfo->HDisplay = %d, pBIOSInfo->VDisplay = %d\n"
                    , pBIOSInfo->HDisplay, pBIOSInfo->VDisplay));
    home="/etc/X11";
    if (home)
    {
        filename[0] = '\0';
        strcpy(filename, home);
        strcat(filename, "/.VIARC");
        if ((VIARC = fopen(filename, "r")) == NULL)
        {
            DEBUG(xf86Msg(X_WARNING, "No user setting file exist, Create new ones\n"));
            if ((VIARC = fopen(filename, "a")) == NULL)
                DEBUG(xf86Msg(X_ERROR, "Create .VIARC failure!\n"));
        }
        else
        {
            while (fgets(buffer, 40, VIARC) != NULL)
            {
                sscanf(buffer,"%d %d %d", &HDisplay, &VDisplay, &bitsPerPixel);

                if (HDisplay == pBIOSInfo->SaveHDisplay &&
                    VDisplay == pBIOSInfo->SaveVDisplay &&
                    bitsPerPixel == pBIOSInfo->bitsPerPixel)
                {
                    /* RainLin [2005.12.05]: Also get TV output since we saved it. */
                    sscanf(buffer,"%d %d %d %d %d %d %d %d %d",
                            &tmp, &tmp, &tmp, &ActiveDevice, &Center, &TVType, &TVOutput, &TV2Type, &TV2Output);

                    /* RainLin [2005.12.05]: No need to get active device in SAMM case.  */
                    if (pBIOSInfo->FirstInit && !pBIOSInfo->SAMM)
                    {
                        pBIOSInfo->ActiveDevice = ActiveDevice & pBIOSInfo->ConnectedDevice;
                    }

                    if (pBIOSInfo->FirstInit)
                    {
                        pBIOSInfo->TVSettingInfo.IsPreSetting=FALSE;
                        if (pBIOSInfo->TVSettingInfo.TVType == TVType)
                        {
                            pBIOSInfo->TVSettingInfo.IsPreSetting=TRUE;
                        }
                        else
                        {
                            pBIOSInfo->TVSettingInfo.TVType = TVType;
                        }
                        pBIOSInfo->TVSettingInfo.TVOutput = TVOutput;

                        if ((pBIOSInfo->ActiveDevice & VIA_DEVICE_TV2) &&
                            (pBIOSInfo->DisplayHardwareLayout == HW_LAYOUT_TV_EXTERNAL_TV2))
                        {
                            pBIOSInfo->TVSettingInfo2.TVType = TV2Type;
                            pBIOSInfo->TVSettingInfo2.TVOutput = TV2Output;
                        }
                    }

                    pBIOSInfo->LVDSSettingInfo.Center = Center;
                    pBIOSInfo->LVDSSettingInfo2.Center = Center;

                    DEBUG(xf86Msg(X_INFO, "Found user active device setting!!\n"));
                    DEBUG(xf86Msg(X_INFO, "Utility: Active Device = 0x%x!!\n", pBIOSInfo->ActiveDevice));
                    break;
                }
            }
        }
        fclose(VIARC);
    }
}

void VIALoadUserTVSetting(VIABIOSInfoPtr pBIOSInfo,
                                TVSETTINGINFOPTR pTVSettingInfo,
                                const char* pchFileName)
{
    TVInfoPtr       pTVInfo = &(pTVSettingInfo->TVInfo);
    FILE *VIATVRC=NULL;
    char filename[40], buffer[200], *home = NULL;
    int  HDisplay, VDisplay, bitsPerPixel, tmp, TVStandard, TVOutput;
    Bool            UT_TV_DOTCRAWL, found=FALSE;
    CARD32          UT_TV_STD=0, UT_TV_SGNL=0;

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIALoadUserTVSetting\n"));
    home="/etc/X11";
    if (home)
    {
        if (pTVSettingInfo->TVEncoder)
        {
            filename[0] = '\0';
            strcpy(filename, home);
            strcat(filename, pchFileName);
            if ((VIATVRC = fopen(filename, "r")) == NULL)
            {
                DEBUG(xf86Msg(X_WARNING, "No user TV setting file exist, Create new ones\n"));
                if ((VIATVRC = fopen(filename, "a")) == NULL)
                    DEBUG(xf86Msg(X_ERROR, "Create .VIATVRC failure!\n"));
            }
            else
            {
                while (fgets(buffer, 200, VIATVRC) != NULL)
                {
                    sscanf(buffer,"%d %d %d %d %d %d %d %d", &HDisplay, &VDisplay, &bitsPerPixel, &tmp, &tmp, &tmp, &TVStandard, &TVOutput);

                    if (HDisplay == pBIOSInfo->SaveHDisplay &&
                        VDisplay == pBIOSInfo->SaveVDisplay &&
                        bitsPerPixel == pBIOSInfo->bitsPerPixel &&
                        TVStandard == pTVSettingInfo->TVType &&
                        TVOutput == pTVSettingInfo->TVOutput)
                    {
                        sscanf(buffer,"%d %d %d %d %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %d %lu %lu %lu %lu",
                                &tmp, &tmp, &tmp, &pTVInfo->AdaptiveFFilterOn,
                                &pTVInfo->CurrentScalH, &pTVInfo->CurrentScalV,
                                &UT_TV_STD,
                                &UT_TV_SGNL,
                                &pTVInfo->CurrentPositionH,
                                &pTVInfo->CurrentPositionV,
                                &pTVInfo->CurrentFFilter,
                                &pTVInfo->CurrentAFFilter,
                                &pTVInfo->CurrentBrightness,
                                &pTVInfo->CurrentContrast,
                                &pTVInfo->CurrentSaturation,
                                &pTVInfo->CurrentTINT,
                                &UT_TV_DOTCRAWL,
                                &pTVInfo->XScalingLevel,
                                &pTVInfo->YScalingLevel,
                                &pTVInfo->XScalPosLevel,
                                &pTVInfo->YScalPosLevel);
                        xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO,
				   "After load TV info: XScaleLevel=%ld,YScaleLevel=%ld,XPOSITIONLevel=%ld,YPOSITIONLevel=%ld\n",
				   pTVInfo->XScalingLevel,
				   pTVInfo->YScalingLevel,
				   pTVInfo->XScalPosLevel,
				   pTVInfo->YScalPosLevel);

                        pTVSettingInfo->TVVScan = pTVInfo->CurrentScalV;
                        pTVSettingInfo->TVType = UT_TV_STD;
                        pTVSettingInfo->TVOutput = UT_TV_SGNL;
                        pTVSettingInfo->TVDotCrawl = UT_TV_DOTCRAWL;
                        pTVInfo->HasUserSetting = TRUE;
                        found = TRUE;
                        DEBUG(xf86Msg(X_INFO, "Found user TV setting!!\n"));
                        break;
                    }
                }
            }

            if (!found)
            {
                pTVInfo->HasUserSetting = FALSE;
                pTVInfo->AdaptiveFFilterOn = FALSE;
                pTVInfo->CurrentScalH = pTVInfo->DefaultScalH;
                pTVInfo->CurrentScalV = pTVInfo->DefaultScalV;
                pTVInfo->CurrentPositionV = pTVInfo->DefaultPositionV;
                pTVInfo->CurrentPositionH = pTVInfo->DefaultPositionH;
                pTVInfo->CurrentFFilter = pTVInfo->DefaultFFilter;
                pTVInfo->CurrentAFFilter = pTVInfo->DefaultAFFilter;
                pTVInfo->CurrentBrightness = pTVInfo->DefaultBrightness;
                pTVInfo->CurrentContrast = pTVInfo->DefaultContrast;
                pTVInfo->CurrentSaturation = pTVInfo->DefaultSaturation;
                pTVInfo->CurrentTINT = pTVInfo->DefaultTINT;
                DEBUG(xf86Msg(X_INFO, "No user TV setting exist!!\n"));
            }

            fclose(VIATVRC);
        }
    }
}

void VIALoadUserDuoViewVideoOutputDeviceSetting(ScrnInfoPtr pScrn)
{
    VIAPtr pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;
    PVIDDATA        pVidData = pVia->pVidData;
    LPVIAGRAPHICINFO lpVIAGraphicInfo = &pVidData->VIAGraphicInfo;
    FILE *VIADuoView=NULL;
    char filename[40], buffer[2], *home = NULL;
    int	 nGetV3OnIGAState;

    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "VIALoadUserDuoViewVideoOutputDeviceSetting\n"));
    home="/etc/X11";
    if (home)
    {
        filename[0] = '\0';
        strcpy(filename, home);
        strcat(filename, "/.VIADuoView");
        if ((VIADuoView = fopen(filename, "r")) == NULL)
        {
            DEBUG(xf86Msg(X_WARNING, "No user DuoView setting file exist, Create new ones\n"));
            if ((VIADuoView = fopen(filename, "a")) == NULL)
                DEBUG(xf86Msg(X_ERROR, "Create .VIADuoView failure!\n"));
        }
        else
        {
            while (fgets(buffer, 2, VIADuoView) != NULL)
            {
                sscanf(buffer,"%d", &nGetV3OnIGAState);
                if (nGetV3OnIGAState == UT_STATE_DuoView_IGA1)
                {
                    if(pVia->pVidData->VIAGraphicInfo.duoview &&
                      ((pBIOSInfo->Chipset ==VIA_KM400) ||
                      (pBIOSInfo->Chipset ==VIA_K8M800) ||
                      (pBIOSInfo->Chipset ==VIA_K8M890) ||
                      (pBIOSInfo->Chipset ==VIA_P4M890) ||
                      (pBIOSInfo->Chipset == VIA_P4M800PRO) ||
                      (pBIOSInfo->Chipset == VIA_P4M900)))
                        lpVIAGraphicInfo->duoview &=~DUOVIEW_V3_ON_IGA2;
                }
                else if (nGetV3OnIGAState == UT_STATE_DuoView_IGA2)
                {
                    if(pVia->pVidData->VIAGraphicInfo.duoview &&
                      ((pBIOSInfo->Chipset ==VIA_KM400) ||
                      (pBIOSInfo->Chipset ==VIA_K8M800) ||
                      (pBIOSInfo->Chipset ==VIA_K8M890) ||
                      (pBIOSInfo->Chipset ==VIA_P4M890) ||
                      (pBIOSInfo->Chipset == VIA_P4M800PRO) ||
                      (pBIOSInfo->Chipset == VIA_P4M900)))
                        lpVIAGraphicInfo->duoview |=DUOVIEW_V3_ON_IGA2;
                }
            }
        }

        fclose(VIADuoView);
    }
}

/* Remove restart flag file for utility */
void
VIAUTRemoveRestartFlag(VIABIOSInfoPtr pBIOSInfo)
{
    char filename[40], *home = NULL;

    filename[0] = '\0';
    home="/etc/X11";
    if (home)
    {
        filename[0] = '\0';
        strcpy(filename, home);
        strcat(filename, "/.XWindowNeedtobeRestart");
        if (remove(filename) == -1)
            DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO,
                  "Restart flag file not exist!\n"));
        else
            DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO,
                  "Restart flag file has been remove!\n"));
    }
}

/*=* for [Hitachi Escher] turn off device to avoid suspend garbage. *=*/
void VIAXVDeviceControl(VIAPtr pVia, CARD32 value)
{
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;
    CARD8   cr36;

    if (value & XV_DC_ON)
    {
        if (value & VIA_DEVICE_CRT1)
        {
            /* turn on CRT */
            VGAOUT(0x3d4, 0x36);
            cr36 = VGAIN8(0x3d5);
            VGAOUT(0x3d5, cr36 & ~0x30);
        }
        if (value & VIA_DEVICE_LCD)
        {
            /* turn on LCD */
            VIAEnableLCD(pBIOSInfo, &(pBIOSInfo->LVDSSettingInfo));
        }
        if (value & VIA_DEVICE_TV)
        {
            /* turn on TV */
            VIAEnableTV(pBIOSInfo, &(pBIOSInfo->TVSettingInfo));
        }
        if (value & VIA_DEVICE_DFP)
        {
            /* turn on DFP */
            VIAEnableDFP(pBIOSInfo);
        }
    }
    else if (value & XV_DC_OFF)
    {
        if (value & VIA_DEVICE_CRT1)
        {
            /* turn off CRT */
            VGAOUT(0x3d4, 0x36);
            cr36 = VGAIN8(0x3d5);
            VGAOUT(0x3d5, cr36 | 0x30);
        }
        if (value & VIA_DEVICE_LCD)
        {
            /* turn off LCD */
            VIADisableLCD(pBIOSInfo, &(pBIOSInfo->LVDSSettingInfo));
        }
        if (value & VIA_DEVICE_TV)
        {
            /* turn off TV */
            VIADisableTV(pBIOSInfo, &(pBIOSInfo->TVSettingInfo));
        }
        if (value & VIA_DEVICE_DFP)
        {
            /* turn off DFP */
            VIADisableDFP(pBIOSInfo);
        }
    }
    else
    {
        ErrorF("VIAXVDeviceControl: unknow XV device control!! \n");
    }
}

void VIAXVSetTVStand(ScrnInfoPtr pScrn, CARD32 value)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;

    if (value & XV_SetStandard)
    {
        switch (value & 0xFFFF)
        {
            case UT_TV_STD_NTSC :
                pBIOSInfo->TVSettingInfo.TVType = TVTYPE_NTSC;
                if (pBIOSInfo->TVSettingInfo.TVProgressive)
                {
                    pBIOSInfo->TVSettingInfo.TVProgressive = FALSE;
                }
                if (pBIOSInfo->ActiveDevice & VIA_DEVICE_TV)
                {
                    VIASwitchMode(pScrn->scrnIndex, pScrn->currentMode, 0);
                }
                break;
            case UT_TV_STD_PAL :
                pBIOSInfo->TVSettingInfo.TVType = TVTYPE_PAL;
                if (pBIOSInfo->TVSettingInfo.TVProgressive)
                {
                    pBIOSInfo->TVSettingInfo.TVProgressive = FALSE;
                }
                if (pBIOSInfo->ActiveDevice & VIA_DEVICE_TV)
                {
                    VIASwitchMode(pScrn->scrnIndex, pScrn->currentMode, 0);
                }
                break;
            case UT_TV_STD_480P :
                pBIOSInfo->TVSettingInfo.TVType = TVTYPE_480P;
                pBIOSInfo->TVSettingInfo.TVProgressive = TRUE;
                if (pBIOSInfo->ActiveDevice & VIA_DEVICE_TV)
                {
                    VIASwitchMode(pScrn->scrnIndex, pScrn->currentMode, 0);
                }
                break;
            default :
                ErrorF("VIAXVSetTVStand: unknow XV Set TV Standard parameter %lu!! \n"
                       , value);
                break;
        }
    }
    else
    {
        ErrorF("VIAXVSetTVStand: unknow XV Set TV Standard parameter %lu!! \n"
               , value);
    }
}



/*
    Query the VIAGFX_API version.
*/
static int ProcVIAGFXQueryVersion (ClientPtr client)
{
    xVIAGFXQueryVersionReply rep;
    register int n;

    REQUEST_SIZE_MATCH(xVIAGFXQueryVersionReq);
    rep.type = X_Reply;
    rep.length = 0;
    rep.sequenceNumber = client->sequence;
    rep.majorVersion = 2;
    rep.minorVersion = 0;

    if(client->swapped)
    {
        swaps(&rep.sequenceNumber, n);
        swapl(&rep.length, n);
        swaps(&rep.majorVersion, n);
        swaps(&rep.minorVersion, n);
    }

    WriteToClient(client, sizeof(xVIAGFXQueryVersionReply), (char *)&rep);
    return (client->noClientException);
}


/*
*   Receive and process the request from AP.
*/
static int ProcVIAGFXCommand(ClientPtr client)
{
    REQUEST(xVIAGFXCommandReq);
    xVIAGFXCommandReply rep;
    ExtensionEntry   *myext;
    register int     n;
    int          i;

    REQUEST_SIZE_MATCH(xVIAGFXCommandReq);

    /* copy the request to reply. */
    memcpy(&rep, stuff, sizeof(xVIAGFXCommandReply));

    /* Get pointer to ExtensionEntry */
    if(!(myext = CheckExtension(VIAGFX_PROTOCOL_NAME))) return BadMatch;

    /* Process the request. */
    VIAGFXUtilityProc(&rep);

    rep.type = X_Reply;
    rep.length = (sizeof(xVIAGFXCommandReply) - sizeof(xGenericReply)) >> 2;
    rep.sequenceNumber = client->sequence;

    if(client->swapped)
    {
       swaps(&rep.sequenceNumber, n);
       swapl(&rep.length, n);
       swapl(&rep.command, n);
       swapl(&rep.primary_id, n);
       swapl(&rep.secondary_id, n);
       swapl(&rep.result_header, n);

       for(i = 0; i < VIAGFX_PARM_NUM; i++)
       {
          swapl(&rep.parm[i], n);
          swapl(&rep.result[i], n);
       }
    }

    WriteToClient(client, sizeof(xVIAGFXCommandReply), (char *)&rep);
    return client->noClientException;
}

static void VIAGFXResetProc(ExtensionEntry* extEntry)
{
    if(extEntry->extPrivate)
    {
       xfree(extEntry->extPrivate);
       extEntry->extPrivate = NULL;
    }
}

static int ProcVIAGFXDispatch(ClientPtr client)
{
    REQUEST(xReq);
    switch(stuff->data)
    {
        case X_VIAGFXQueryVersion:
            return ProcVIAGFXQueryVersion(client);
        case X_VIAGFXCommand:
            return ProcVIAGFXCommand(client);
    }

    return BadRequest;
}

void VIADisplayExtensionInit(ScrnInfoPtr pScrn)
{
    VIAPtr pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;

    ExtensionEntry *extEntry;
    xVIAGFXScreenTable *VIAGFXScrnTbl;

    g_ScreenNumber = pScrn->scrnIndex;

    if(!(extEntry = CheckExtension(VIAGFX_PROTOCOL_NAME)))
    {
        if(!(VIAGFXScrnTbl = xcalloc(sizeof(xVIAGFXScreenTable), 1)))
            return;

        if (!(extEntry = AddExtension(VIAGFX_PROTOCOL_NAME, 0, 0,
                                     ProcVIAGFXDispatch,
                                     ProcVIAGFXDispatch,
                                     VIAGFXResetProc,
                                     StandardMinorOpcode)))
        {
            xf86DrvMsg(pScrn->scrnIndex, X_ERROR, "Failed to add VIAGFX extension\n");
            return;
        }

        extEntry->extPrivate = (pointer)VIAGFXScrnTbl;
    }
    else
    {
        if(!(VIAGFXScrnTbl = (xVIAGFXScreenTable *)extEntry->extPrivate))
        {
            xf86DrvMsg(pScrn->scrnIndex, X_ERROR, "Internal error: Found VIAGFX extension with NULL-private!\n");
            return;
        }
    }

    VIAGFXScrnTbl->HandleVIAGFXCommand[pScrn->scrnIndex] = VIAGFXUtilityProc;
    pBIOSInfo->DisplayExtEntry = extEntry;
}


void
VIADisplayExtUnregister(ScrnInfoPtr pScrn)
{
    VIAPtr pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;
    ExtensionEntry  *VIAGFXExt;
    xVIAGFXScreenTable *VIAGFXScreenTbl;

    if(!pBIOSInfo->DisplayExtEntry) return;

    if((VIAGFXExt = CheckExtension(VIAGFX_PROTOCOL_NAME)))
    {
        if((VIAGFXScreenTbl = (xVIAGFXScreenTable *)VIAGFXExt->extPrivate)) {
         VIAGFXScreenTbl->HandleVIAGFXCommand[pScrn->scrnIndex] = NULL;
      }
    }
}

int VIAGFXUtilityProc(xVIAGFXCommandReply* rep)
{
    ScrnInfoPtr pScrn;
    VIAPtr pVia;
    VIABIOSInfoPtr pBIOSInfo;
    TVSETTINGINFOPTR pTVSettingInfo;
    /* variables for escape function. */
    CARD32 i;
    char szDriverName[32];
    CARD32 dwSupportState = 0, dwConnectState = 0, dwCurrentState = 0,
	dwMaxValue = 0, dwCurrentValue = 0, dwChipID = 0, dwVideoRam = 0,
	dwGammaLUT[256], dwRotateDegree = 0, dwRotateType = 0;
    UTXYVALUE MaxViewSizeValue, ViewSizeValue, MaxViewPosValue, ViewPosValue;
    UTSETTVTUNINGINFO TVTuningInfo;
    UTSETTVITEMSTATE TVSettingItemState;
    UTPANELINFO UTPanelInfo;
    UTBIOSVERSION UTBIOSVersion;
    UTBIOSDATE UTBIOSDate;
    UTDriverVersion UTDriverVer;
    UTDEVMODE DevMode;

    pScrn = xf86Screens[g_ScreenNumber];
    pVia = VIAPTR(pScrn);
    /*
        STChen[2006.7.11]: In SAMM case, a driver controls a screen.
        In the past, we have to move application's window to switch to correct driver.
        So I add following step to detect which driver we should use to avoid this problem.
    */
    if (pVia->pBIOSInfo->SAMM) {
	pScrn = VIAScreenSelection(pScrn, rep->primary_id, rep->secondary_id);
	pVia = VIAPTR(pScrn);
    }

    pBIOSInfo = pVia->pBIOSInfo;

    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "VIAGFXUtilityProc\n"));
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO,
		     "rep->primary_id = %x\n", rep->primary_id));
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO,
		     "rep->secondary_id = %x\n", rep->secondary_id));

    /* Chose the TV setting info. */
    if ((rep->primary_id == UT_XV_FUNC_TV2) ||
	(rep->primary_id == UT_XV_FUNC_HDTV2)) {
	pTVSettingInfo = &(pBIOSInfo->TVSettingInfo2);
    }
    else {
	pTVSettingInfo = &(pBIOSInfo->TVSettingInfo);
    }

    if (rep->primary_id == UT_XV_FUNC_BIOS) {
	switch (rep->secondary_id) {
	    case UT_XV_FUNC_BIOS_GetChipID:
		VIA_UT_XV_FUNC_BIOS_GetChipID(pScrn, &dwChipID);
		rep->result[0] = dwChipID;
		rep->result_header = TRUE;
		break;
	    case UT_XV_FUNC_BIOS_GetVersion:
		VIA_UT_XV_FUNC_BIOS_GetVersion(pBIOSInfo, &UTBIOSVersion);
		rep->result[0] = UTBIOSVersion.dwVersion;
		rep->result[1] = UTBIOSVersion.dwRevision;
		rep->result_header = TRUE;
                break;
	    case UT_XV_FUNC_BIOS_GetDate:
		VIA_UT_XV_FUNC_BIOS_GetDate(pScrn, &UTBIOSDate);
		rep->result[0] = UTBIOSDate.dwYear;
		rep->result[1] = UTBIOSDate.dwMonth;
		rep->result[2] = UTBIOSDate.dwDay;
		rep->result_header = TRUE;
		break;
	    case UT_XV_FUNC_BIOS_GetVideoMemSizeMB:
		VIA_UT_XV_FUNC_BIOS_GetVideoMemSizeMB(pScrn, &dwVideoRam);
		rep->result[0] = dwVideoRam;
		rep->result_header = TRUE;
		break;
	    case UT_XV_FUNC_BIOS_GetTVChipID:
		VIA_UT_XV_FUNC_BIOS_GetTVChipID(pBIOSInfo, &dwChipID);
		rep->result[0] = dwChipID;
		rep->result_header = TRUE;
		break;
	}
    }
    else if (rep->primary_id == UT_XV_FUNC_DRIVER) {
	switch (rep->secondary_id) {
	    case UT_XV_FUNC_DRIVER_GetFileName:
		if (VIA_UT_XV_FUNC_DRIVER_GetFileName(rep->parm[0],
						      szDriverName)) {
		    memcpy((void*)rep->result, (void*)szDriverName,
			   sizeof(szDriverName));
		    rep->result_header = TRUE;
		}
		else {
		    rep->result_header = FALSE;
		}
		break;
	    case UT_XV_FUNC_DRIVER_GetFileVersion:
		if (VIA_UT_XV_FUNC_DRIVER_GetFileVersion(pScrn, rep->parm[0],
							 &UTDriverVer)) {
		    rep->result[0] = UTDriverVer.dwMajorNum;
		    rep->result[1] = UTDriverVer.dwMinorNum;
		    rep->result[2] = UTDriverVer.dwOSNum;
		    rep->result[3] = UTDriverVer.dwReversionNum;
		    rep->result_header = TRUE;
		}
                else {
		    rep->result_header = FALSE;
		}
		break;
	    case UT_XV_FUNC_DRIVER_VIASync:
		if (pVia->pBIOSInfo->Is3DScalingEnable) {
		    if(pVia->pBIOSInfo->IsStableStatus) {
			VIADISP3DScalingImageProcessing(pVia);
		    }
		}
		rep->result_header = TRUE;
		break;
	    case UT_XV_FUNC_DRIVER_GetFuncState:
		if (pVia->DISP3DScaling) {
		    rep->result[0] = TV_SUPPORT_3DSCALING;
		}
		else {
		    rep->result[0] = NO_3DSCALING;
		}
		rep->result_header = TRUE;
		break;
	}
    }
    else if (rep->primary_id == UT_XV_FUNC_DEVICE) {
	switch (rep->secondary_id) {
	    case UT_XV_FUNC_DEVICE_GetSupportState:
		VIA_UT_XV_FUNC_DEVICE_GetSupportState(pBIOSInfo,
						      &dwSupportState);
		rep->result[0] = dwSupportState;
		rep->result_header = TRUE;
		break;
	    case UT_XV_FUNC_DEVICE_GetConnectState:
		VIA_UT_XV_FUNC_DEVICE_GetConnectState(pBIOSInfo,
						      &dwConnectState);
		rep->result[0] = dwConnectState;
		rep->result_header = TRUE;
		break;
	    case UT_XV_FUNC_DEVICE_GetActiveState:
		VIA_UT_XV_FUNC_DEVICE_GetActiveState(pScrn, &dwCurrentState);
		rep->result[0] = dwCurrentState;
		rep->result_header = TRUE;
		break;
	    case UT_XV_FUNC_DEVICE_SetActiveState:
		/*Al Zhang[2006.04.24]: If MV enabled, block TV and HDTV status changing*/
		if(CheckMacrovisionEnabled(pBIOSInfo) &&
		   (((rep->parm[0] & UT_DEVICE_HDTV) !=
		     (pBIOSInfo->ActiveDevice& UT_DEVICE_HDTV)) ||
		    ((rep->parm[0] & UT_DEVICE_TV) !=
		     (pBIOSInfo->ActiveDevice& UT_DEVICE_TV)))) {
		    rep->result_header = TRUE;
		    break;
		}
		VIA_UT_XV_FUNC_DEVICE_SetActiveState(pBIOSInfo,
						     pScrn->currentMode,
						     rep->parm[0]);
		rep->result_header = TRUE;
		break;
	    case UT_XV_FUNC_DEVICE_GetSAMMState:
		VIA_UT_XV_FUNC_DEVICE_GetSAMMState(pBIOSInfo, &dwCurrentState);
		rep->result[0] = dwCurrentState;
		rep->result_header = TRUE;
		break;
	    case UT_XV_FUNC_DEVICE_GetSupportSAMMState:
		VIA_UT_XV_FUNC_DEVICE_GetSupportSAMMState(pBIOSInfo,
							  &dwSupportState);
		rep->result[0] = dwSupportState;
		rep->result_header = TRUE;
		break;
	    case UT_XV_FUNC_DEVICE_SetV3OnIGAStatus:
		if (VIA_UT_XV_FUNC_DEVICE_SetV3OnIGAStatus(pScrn,
							   rep->parm[0])) {
		    rep->result_header = TRUE;
		}
		else {
		    rep->result_header = FALSE;
		}
		break;
	    case UT_XV_FUNC_DEVICE_GetV3OnIGAStatus:
		VIA_UT_XV_FUNC_DEVICE_GetV3OnIGAStatus(pScrn, &dwCurrentState);
		rep->result[0] = dwCurrentState;
		rep->result_header = TRUE;
		break;
	    case UT_XV_FUNC_DEVICE_GetDuoViewStatus:
		VIA_UT_XV_FUNC_DEVICE_GetDuoViewStatus(pScrn, &dwCurrentState);
		rep->result[0] = dwCurrentState;
		rep->result_header = TRUE;
		break;
	    case UT_XV_FUNC_DEVICE_GetPanningState:
		dwCurrentState = VIA_UT_XV_FUNC_DEVICE_GetPanningState(pBIOSInfo);
		rep->result[0] = dwCurrentState;
		rep->result_header = TRUE;
		break;
	}        
    }
    else if (rep->primary_id == UT_XV_FUNC_PANEL) {
	switch (rep->secondary_id) {
	    case UT_XV_FUNC_DEVICE_GetPanelInfo:
		VIA_UT_XV_FUNC_DEVICE_GetPanelInfo(pBIOSInfo, rep->parm[0],
						   &UTPanelInfo);
		rep->result[0] = UTPanelInfo.dwType;
		rep->result[1] = UTPanelInfo.dwResX;
		rep->result[2] = UTPanelInfo.dwResY;
		rep->result[3] = UTPanelInfo.dwDeviceID;
		rep->result[4] = UTPanelInfo.dwExpandState;
		rep->result_header = TRUE;
		break;
	    case UT_XV_FUNC_DEVICE_GetSupportExpand:
		VIA_UT_XV_FUNC_DEVICE_GetSupportExpand(pBIOSInfo,
						       &dwSupportState);
		rep->result[0] = dwSupportState;
		rep->result_header = TRUE;
		break;
	    case UT_XV_FUNC_DEVICE_GetExpandState:
		VIA_UT_XV_FUNC_DEVICE_GetExpandState(pBIOSInfo, &dwCurrentState);
		rep->result[0] = dwCurrentState;
		rep->result_header = TRUE;
		break;
	    case UT_XV_FUNC_DEVICE_SetExpandState:
		UTPanelInfo.dwType = rep->parm[0];
		UTPanelInfo.dwResX = rep->parm[1];
		UTPanelInfo.dwResY = rep->parm[2];
		UTPanelInfo.dwDeviceID = rep->parm[3];
		UTPanelInfo.dwExpandState = rep->parm[4];
		if(VIA_UT_XV_FUNC_DEVICE_SetExpandState(pBIOSInfo,
							pScrn->currentMode,
							UTPanelInfo)) {
		    rep->result_header = TRUE;
		}
		else {
		    rep->result_header = FALSE;
		}
		break;
	}
    }
    else if ((rep->primary_id == UT_XV_FUNC_TV)  ||
	     (rep->primary_id == UT_XV_FUNC_TV2) ||
	     (rep->primary_id == UT_XV_FUNC_HDTV)||
	     (rep->primary_id == UT_XV_FUNC_HDTV2)) {
	switch (rep->secondary_id) {
	    case UT_XV_FUNC_TV_GetSupportStandardState:
		if ((rep->primary_id == UT_XV_FUNC_TV) ||
		    (rep->primary_id == UT_XV_FUNC_TV2)) {
		    VIA_UT_XV_FUNC_TV_GetSupportStandardState(pBIOSInfo,
							      pTVSettingInfo, &dwSupportState);
		}
		else {
		    VIA_UT_XV_FUNC_HDTV_GetSupportStandardState(pBIOSInfo,
								pTVSettingInfo, &dwSupportState);
		}
		rep->result[0] = dwSupportState;
		rep->result_header = TRUE;
		break;
	    case UT_XV_FUNC_TV_GetStandard :
		if ((rep->primary_id == UT_XV_FUNC_TV) ||
		    (rep->primary_id == UT_XV_FUNC_TV2)) {
		    VIA_UT_XV_FUNC_TV_GetStandard(pBIOSInfo, pTVSettingInfo,
						  &dwCurrentState);
		}
		else {
		    VIA_UT_XV_FUNC_HDTV_GetStandard(pBIOSInfo, pTVSettingInfo,
						    &dwCurrentState);
		}
		rep->result[0] = dwCurrentState;
		rep->result_header = TRUE;
		break;
	    case UT_XV_FUNC_TV_SetStandard:
		if ((rep->primary_id == UT_XV_FUNC_TV) ||
		    (rep->primary_id == UT_XV_FUNC_TV2)) {
		    /*Al Zhang[2006.04.24]: If MV enabled, block TV and HDTV status changing*/
		    if (CheckMacrovisionEnabled(pBIOSInfo)) {
			rep->result_header = TRUE;
			break;
		    }
		    if (VIA_UT_XV_FUNC_TV_SetStandard(pBIOSInfo, pTVSettingInfo,
						      pScrn->currentMode,
						      rep->parm[0]))
			rep->result_header = TRUE;
		    else
			rep->result_header = FALSE;
		}
		else {
		    if (VIA_UT_XV_FUNC_HDTV_SetStandard(pBIOSInfo,
							pTVSettingInfo,
							pScrn->currentMode,
							rep->parm[0]))
			rep->result_header = TRUE;
		    else
			rep->result_header = FALSE;
		}
		break;
	    case UT_XV_FUNC_TV_GetSupportSignalTypeState:
		if ((rep->primary_id == UT_XV_FUNC_TV) ||
		    (rep->primary_id == UT_XV_FUNC_TV2)) {
		    VIA_UT_XV_FUNC_TV_GetSupportSignalTypeState(pBIOSInfo,
								pTVSettingInfo,
								&dwSupportState);
		}
		else {
		    VIA_UT_XV_FUNC_HDTV_GetSupportSignalTypeState(pBIOSInfo,
								  pTVSettingInfo,
								  &dwSupportState);
		}
		rep->result[0] = dwSupportState;
		rep->result_header = TRUE;
		break;
	    case UT_XV_FUNC_TV_GetSignalType:
		if ((rep->primary_id == UT_XV_FUNC_TV) ||
		    (rep->primary_id == UT_XV_FUNC_TV2)) {
		    VIA_UT_XV_FUNC_TV_GetSignalType(pBIOSInfo, pTVSettingInfo,
						    &dwCurrentState);
		}
		else {
		    VIA_UT_XV_FUNC_HDTV_GetSignalType(pBIOSInfo, pTVSettingInfo,
						      &dwCurrentState);
		}
		rep->result[0] = dwCurrentState;
		rep->result_header = TRUE;
                break;
	    case UT_XV_FUNC_TV_SetSignalType:
		if ((rep->primary_id == UT_XV_FUNC_TV) ||
		    (rep->primary_id == UT_XV_FUNC_TV2)) {
		    /*Al Zhang[2006.04.24]: If MV enabled, block TV and HDTV status changing*/
		    if (CheckMacrovisionEnabled(pBIOSInfo)) {
			rep->result_header = TRUE;
			break;
		    }
		    if (VIA_UT_XV_FUNC_TV_SetSignalType(pBIOSInfo,
							pTVSettingInfo,
							pScrn->currentMode,
							rep->parm[0]))
			rep->result_header = TRUE;
		    else
			rep->result_header = FALSE;
		}
		else {
		    if (VIA_UT_XV_FUNC_HDTV_SetSignalType(pBIOSInfo,
							  pTVSettingInfo,
							  pScrn->currentMode,
							  rep->parm[0]))
			rep->result_header = TRUE;
		    else
			rep->result_header = FALSE;
		}
		break;
	    case UT_XV_FUNC_TV_GetMaxViewSizeValue:
		VIA_UT_XV_FUNC_TV_GetMaxViewSizeValue(pBIOSInfo, pTVSettingInfo,
						      &MaxViewSizeValue);
		rep->result[0] = MaxViewSizeValue.dwX;
		rep->result[1] = MaxViewSizeValue.dwY;
		rep->result_header = TRUE;
		break;
	    case UT_XV_FUNC_TV_GetViewSizeValue:
		VIA_UT_XV_FUNC_TV_GetViewSizeValue(pBIOSInfo, pTVSettingInfo,
						   &ViewSizeValue);
		rep->result[0] = ViewSizeValue.dwX;
		rep->result[1] = ViewSizeValue.dwY;
		rep->result_header = TRUE;
		break;
	    case UT_XV_FUNC_TV_SetViewSizeValue:
		ViewSizeValue.dwX = rep->parm[0];
		ViewSizeValue.dwY = rep->parm[1];
		VIA_UT_XV_FUNC_TV_SetViewSizeValue(pBIOSInfo, pTVSettingInfo,
						   pScrn->currentMode,
						   ViewSizeValue);
		rep->result_header = TRUE;
		break;
	    case UT_XV_FUNC_TV_GetMaxViewPositionValue:
		VIA_UT_XV_FUNC_TV_GetMaxViewPositionValue(pBIOSInfo,
							  pTVSettingInfo,
							  &MaxViewPosValue);
		rep->result[0] = MaxViewPosValue.dwX;
		rep->result[1] = MaxViewPosValue.dwY;
		rep->result_header = TRUE;
		break;
	    case UT_XV_FUNC_TV_GetViewPositionValue:
		VIA_UT_XV_FUNC_TV_GetViewPositionValue(pBIOSInfo,
						       pTVSettingInfo,
						       &ViewPosValue);
		rep->result[0] = ViewPosValue.dwX;
		rep->result[1] = ViewPosValue.dwY;
		rep->result_header = TRUE;
		break;
	    case UT_XV_FUNC_TV_SetViewPositionValue:
		ViewPosValue.dwX = rep->parm[0];
		ViewPosValue.dwY = rep->parm[1];
		if (VIA_UT_XV_FUNC_TV_SetViewPositionValue(pBIOSInfo,
							   pTVSettingInfo,
							   ViewPosValue))
		    rep->result_header = TRUE;
		else
		    rep->result_header = FALSE;
		break;
	    case UT_XV_FUNC_TV_GetSupportTuningState:
		VIA_UT_XV_FUNC_TV_GetSupportTuningState(pBIOSInfo,
							pTVSettingInfo,
							&dwSupportState);
		rep->result[0] = dwSupportState;
		rep->result_header = TRUE;
		break;
	    case UT_XV_FUNC_TV_GetTuningItemMaxValue:	/* 0 BASE */
		if (VIA_UT_XV_FUNC_TV_GetTuningItemMaxValue(pBIOSInfo,
							    pTVSettingInfo,
							    rep->parm[0],
							    &dwMaxValue)) {
		    rep->result[0] = dwMaxValue;
		    rep->result_header = TRUE;
		}
		else {
		    rep->result_header = FALSE;
		}
		break;
	    case UT_XV_FUNC_TV_GetTuningItemValue:
		if (VIA_UT_XV_FUNC_TV_GetTuningItemValue(pBIOSInfo,
							 pTVSettingInfo,
							 rep->parm[0],
							 &dwCurrentValue)) {
		    rep->result[0] = dwCurrentValue;
		    rep->result_header = TRUE;
		}
		else {
		    rep->result_header = FALSE;
		}
		break;
	    case UT_XV_FUNC_TV_SetTuningItemValue:
		TVTuningInfo.dwItemID = rep->parm[0];
		TVTuningInfo.dwValue= rep->parm[1];
		if (VIA_UT_XV_FUNC_TV_SetTuningItemValue(pBIOSInfo,
							 pTVSettingInfo,
							 TVTuningInfo))
		    rep->result_header = TRUE;
		else
		    rep->result_header = FALSE;
		break;
	    case UT_XV_FUNC_TV_GetSupportSettingState:
		VIA_UT_XV_FUNC_TV_GetSupportSettingState(pBIOSInfo,
							 pTVSettingInfo,
							 &dwSupportState);
		rep->result[0] = dwSupportState;
		rep->result_header = TRUE;
		break;
	    case UT_XV_FUNC_TV_GetSettingItemState:
		if (VIA_UT_XV_FUNC_TV_GetSettingItemState(pBIOSInfo,
							  pTVSettingInfo,
							  rep->parm[0],
							  &dwCurrentState)) {
		    rep->result[0] = dwCurrentState;
		    rep->result_header = TRUE;
		}
		else {
		    rep->result_header = FALSE;
		}
		break;
	    case UT_XV_FUNC_TV_SetSettingItemState:
		TVSettingItemState.dwItemID = rep->parm[0];
		TVSettingItemState.dwState = rep->parm[1];
		if (VIA_UT_XV_FUNC_TV_SetSettingItemState(pBIOSInfo,
							  pTVSettingInfo,
							  TVSettingItemState))
		    rep->result_header = TRUE;
		else
		    rep->result_header = FALSE;
		break;
	    default:
		rep->result_header = FALSE;
		break;
	}
    }
    else if (rep->primary_id == UT_XV_FUNC_GAMMA) {
	switch (rep->secondary_id) {
	    case UT_ESC_FUNC_GAMMA_GetDeviceSupportState:
                VIA_UT_ESC_FUNC_GAMMA_GetDeviceSupportState(pScrn,
							    &dwSupportState);
		rep->result[0] = dwSupportState;
		rep->result_header = TRUE;
		break;
	    case UT_ESC_FUNC_GAMMA_GetLookUpTable:
		VIA_UT_ESC_FUNC_GAMMA_GetLookUpTable(pBIOSInfo, dwGammaLUT);
		for (i = 0; i < 256; i++) {
		    rep->result[i] = dwGammaLUT[i];
		}
		rep->result_header = TRUE;
		break;
	    case UT_ESC_FUNC_GAMMA_SetLookUpTable:
                for (i = 0; i < 256; i++) {
		    dwGammaLUT[i] = rep->parm[i];
		}
		if (VIA_UT_ESC_FUNC_GAMMA_SetLookUpTable(pScrn, dwGammaLUT)) {
		    rep->result_header = TRUE;
		}
		else {
		    rep->result_header = FALSE;
		}
		break;
	    case UT_ESC_FUNC_GAMMA_GetDefaultLookUpTable:
		VIA_UT_ESC_FUNC_GAMMA_GetDefaultLookUpTable(pBIOSInfo,
							    dwGammaLUT);
		for (i = 0; i < 256; i++) {
		    rep->result[i] = dwGammaLUT[i];
		}
		rep->result_header = TRUE;
		break;
	}
    }
    else if (rep->primary_id == UT_XV_FUNC_DISPLAY) {
	switch (rep->secondary_id) {
	    case UT_XV_FUNC_DISPLAY_EnumModes:
		VIA_UT_XV_FUNC_DISPLAY_EnumModes(pBIOSInfo, rep->parm[0],
						 &DevMode);
		rep->result[0] = DevMode.ModeIndex;
		rep->result[1] = DevMode.PelsWidth;
		rep->result[2] = DevMode.PelsHeight;
		rep->result[3] = DevMode.RefreshRateNum;
		for (i = 0; i < DevMode.RefreshRateNum; i++) {
		    rep->result[i + 4] = DevMode.RefreshRate[i];
		}
		rep->result_header = TRUE;
		break;
	    case UT_XV_FUNC_DISPLAY_GetCurrentMode:
		VIA_UT_XV_FUNC_DISPLAY_GetCurrentMode(pBIOSInfo, &DevMode);
		rep->result[0] = DevMode.ModeIndex;
		rep->result[1] = DevMode.PelsWidth;
		rep->result[2] = DevMode.PelsHeight;
		rep->result[3] = DevMode.RefreshRateNum;
		for (i = 0; i < DevMode.RefreshRateNum; i++) {
		    rep->result[i+4] = DevMode.RefreshRate[i];
		}
		rep->result_header = TRUE;
		break;
	    case UT_XV_FUNC_DISPLAY_GetCurrentRotateDegree:
		VIA_UT_XV_FUNC_DISPLAY_GetCurrentRotateDegree(pScrn,
							      &dwRotateType,
							      &dwRotateDegree);
		rep->result[0] = dwRotateType;
		rep->result[1] = dwRotateDegree;
		rep->result_header = TRUE;
		break;
	    case UT_XV_FUNC_DISPLAY_GetRotateCaps:
		VIA_UT_XV_FUNC_DISPLAY_GetRotationCaps(pScrn, rep->parm[0],
						       &dwSupportState);
		rep->result[0] = dwSupportState;
		rep->result_header = TRUE;
		break;
	}
    }

    return (Success);
}

